@@ -15,12 +15,12 @@ odoo.define('account.dashboard.setup.tour', function (require) {
         trigger: '.o_app[data-menu-xmlid="account.menu_finance"]',
         position: 'bottom',
     }, {
-        trigger: '.o_data_row:first',
+        trigger: '.o_data_row:first .o_data_cell',
         extra_trigger: '.breadcrumb',
     }, {
         trigger: '.o_control_panel button:contains("' + _t('Print') + '")',
     }, {
-        trigger: '.o_control_panel .o_dropdown_menu a:contains("' + _t('Invoices without Payment') + '")',
+        trigger: '.o_control_panel .o-dropdown--menu span:contains("' + _t('Invoices without Payment') + '")',
     }, {
         trigger: 'iframe .o_report_layout_standard h2',
         content: 'Primary color is correct',
