@@ -11,24 +11,38 @@ class TestAccountPayment(AccountTestInvoicingCommon):
     def setUpClass(cls, chart_template_ref=None):
         super().setUpClass(chart_template_ref=chart_template_ref)
 
-        cls.payment_debit_account_id = cls.copy_account(cls.company_data['default_journal_bank'].payment_debit_account_id)
-        cls.payment_credit_account_id = cls.copy_account(cls.company_data['default_journal_bank'].payment_credit_account_id)
+        company = cls.company_data['default_journal_bank'].company_id
 
-        cls.partner_bank_account = cls.env['res.partner.bank'].create({
-            'acc_number': 'BE32707171912447',
+        cls.payment_debit_account_id = cls.copy_account(company.account_journal_payment_debit_account_id)
+        cls.payment_credit_account_id = cls.copy_account(company.account_journal_payment_credit_account_id)
+
+        cls.bank_journal_1 = cls.company_data['default_journal_bank']
+        cls.bank_journal_2 = cls.company_data['default_journal_bank'].copy()
+
+        cls.partner_bank_account1 = cls.env['res.partner.bank'].create({
+            'acc_number': "0123456789",
             'partner_id': cls.partner_a.id,
             'acc_type': 'bank',
         })
-
-        cls.company_data['default_journal_bank'].write({
-            'payment_debit_account_id': cls.payment_debit_account_id.id,
-            'payment_credit_account_id': cls.payment_credit_account_id.id,
-            'inbound_payment_method_ids': [(6, 0, cls.env.ref('account.account_payment_method_manual_in').ids)],
-            'outbound_payment_method_ids': [(6, 0, cls.env.ref('account.account_payment_method_manual_out').ids)],
+        cls.partner_bank_account2 = cls.env['res.partner.bank'].create({
+            'acc_number': "9876543210",
+            'partner_id': cls.partner_a.id,
+            'acc_type': 'bank',
+        })
+        cls.comp_bank_account1 = cls.env['res.partner.bank'].create({
+            'acc_number': "985632147",
+            'partner_id': cls.env.company.partner_id.id,
+            'acc_type': 'bank',
+        })
+        cls.comp_bank_account2 = cls.env['res.partner.bank'].create({
+            'acc_number': "741258963",
+            'partner_id': cls.env.company.partner_id.id,
+            'acc_type': 'bank',
         })
 
-        cls.partner_a.write({
-            'bank_ids': [(6, 0, cls.partner_bank_account.ids)],
+        company.write({
+            'account_journal_payment_debit_account_id': cls.payment_debit_account_id.id,
+            'account_journal_payment_credit_account_id': cls.payment_credit_account_id.id
         })
 
     def test_payment_move_sync_create_write(self):
@@ -50,13 +64,11 @@ class TestAccountPayment(AccountTestInvoicingCommon):
             'currency_id': self.company_data['currency'].id,
             'partner_id': False,
             'destination_account_id': copy_receivable.id,
-            'payment_method_id': self.env.ref('account.account_payment_method_manual_in').id,
-            'partner_bank_id': False,
+            'payment_method_line_id': self.inbound_payment_method_line.id,
         }
         expected_move_values = {
             'currency_id': self.company_data['currency'].id,
             'partner_id': False,
-            'partner_bank_id': False,
         }
         expected_liquidity_line = {
             'debit': 50.0,
@@ -94,13 +106,11 @@ class TestAccountPayment(AccountTestInvoicingCommon):
             'destination_account_id': self.partner_a.property_account_payable_id.id,
             'currency_id': self.currency_data['currency'].id,
             'partner_id': self.partner_a.id,
-            'partner_bank_id': self.partner_bank_account.id,
         }])
         self.assertRecordValues(payment.move_id, [{
             **expected_move_values,
             'currency_id': self.currency_data['currency'].id,
             'partner_id': self.partner_a.id,
-            'partner_bank_id': self.partner_bank_account.id,
         }])
         self.assertRecordValues(payment.line_ids.sorted('balance'), [
             {
@@ -124,7 +134,6 @@ class TestAccountPayment(AccountTestInvoicingCommon):
 
         liquidity_lines, counterpart_lines, writeoff_lines = payment._seek_for_lines()
         payment.move_id.write({
-            'partner_bank_id': False,
             'line_ids': [
                 (1, counterpart_lines.id, {
                     'debit': 0.0,
@@ -189,14 +198,110 @@ class TestAccountPayment(AccountTestInvoicingCommon):
             },
         ])
 
-    def test_payment_move_sync_onchange(self):
-        copy_receivable = self.copy_account(self.company_data['default_account_receivable'])
+    def test_payment_move_sync_update_journal_custom_accounts(self):
+        """The objective is to edit the journal of a payment in order to check if the accounts are updated."""
 
+        company = self.company_data['company']
+        # Create two different inbound accounts
+        outstanding_payment_A = company.account_journal_payment_debit_account_id
+        outstanding_payment_B = company.account_journal_payment_debit_account_id.copy()
+        # Create two different journals with a different account
+        journal_A = self.company_data['default_journal_bank']
+        journal_A.inbound_payment_method_line_ids.payment_account_id = outstanding_payment_A
+        journal_B = self.company_data['default_journal_bank'].copy()
+        journal_B.inbound_payment_method_line_ids.payment_account_id = outstanding_payment_B
+
+        # Fill the form payment
         pay_form = Form(self.env['account.payment'].with_context(default_journal_id=self.company_data['default_journal_bank'].id))
         pay_form.amount = 50.0
         pay_form.payment_type = 'inbound'
-        pay_form.partner_type = 'customer'
-        pay_form.destination_account_id = copy_receivable
+        pay_form.partner_id = self.partner_a
+        pay_form.journal_id = journal_A
+        # Save the form (to create move and move line)
+        payment = pay_form.save()
+
+        # Check the payment
+        self.assertRecordValues(payment, [{
+            'amount': 50.0,
+            'payment_type': 'inbound',
+            'partner_type': 'customer',
+            'payment_reference': False,
+            'is_reconciled': False,
+            'currency_id': self.company_data['currency'].id,
+            'partner_id': self.partner_a.id,
+            'journal_id': journal_A.id
+        }])
+        self.assertRecordValues(payment.move_id, [{
+            'currency_id': self.company_data['currency'].id,
+            'partner_id': self.partner_a.id,
+            'journal_id': journal_A.id,
+        }])
+        self.assertRecordValues(payment.line_ids.sorted('balance'), [
+            {
+                'debit': 0.0,
+                'credit': 50.0,
+                'amount_currency': -50.0,
+                'currency_id': self.company_data['currency'].id,
+                'account_id': self.company_data['default_account_receivable'].id,
+            },
+            {
+                'debit': 50.0,
+                'credit': 0.0,
+                'amount_currency': 50.0,
+                'currency_id': self.company_data['currency'].id,
+                'account_id': outstanding_payment_A.id,
+            },
+        ])
+
+        # Change the journal on the form
+        pay_form.journal_id = journal_B
+        # Save the form (to write move and move line)
+        payment = pay_form.save()
+
+        # Check the payment
+        self.assertRecordValues(payment, [{
+            'amount': 50.0,
+            'payment_type': 'inbound',
+            'partner_type': 'customer',
+            'payment_reference': False,
+            'is_reconciled': False,
+            'currency_id': self.company_data['currency'].id,
+            'partner_id': self.partner_a.id,
+            'journal_id': journal_B.id
+        }])
+        self.assertRecordValues(payment.move_id, [{
+            'currency_id': self.company_data['currency'].id,
+            'partner_id': self.partner_a.id,
+            'journal_id': journal_B.id,
+        }])
+        self.assertRecordValues(payment.line_ids.sorted('balance'), [
+            {
+                'debit': 0.0,
+                'credit': 50.0,
+                'amount_currency': -50.0,
+                'currency_id': self.company_data['currency'].id,
+                'account_id': self.company_data['default_account_receivable'].id,
+            },
+            {
+                'debit': 50.0,
+                'credit': 0.0,
+                'amount_currency': 50.0,
+                'currency_id': self.company_data['currency'].id,
+                'account_id': outstanding_payment_B.id,
+            },
+        ])
+
+    def test_payment_move_sync_onchange(self):
+
+        pay_form = Form(self.env['account.payment'].with_context(
+            default_journal_id=self.company_data['default_journal_bank'].id,
+            # The `partner_type` is set through the window action context in the web client
+            # the field is otherwise invisible in the form.
+            default_partner_type='customer',
+        ))
+        pay_form.amount = 50.0
+        pay_form.payment_type = 'inbound'
+        pay_form.partner_id = self.partner_a
         payment = pay_form.save()
 
         expected_payment_values = {
@@ -206,13 +311,13 @@ class TestAccountPayment(AccountTestInvoicingCommon):
             'payment_reference': False,
             'is_reconciled': False,
             'currency_id': self.company_data['currency'].id,
-            'partner_id': False,
-            'destination_account_id': copy_receivable.id,
-            'payment_method_id': self.env.ref('account.account_payment_method_manual_in').id,
+            'partner_id': self.partner_a.id,
+            'destination_account_id': self.partner_a.property_account_receivable_id.id,
+            'payment_method_line_id': self.inbound_payment_method_line.id,
         }
         expected_move_values = {
             'currency_id': self.company_data['currency'].id,
-            'partner_id': False,
+            'partner_id': self.partner_a.id,
         }
         expected_liquidity_line = {
             'debit': 50.0,
@@ -226,7 +331,7 @@ class TestAccountPayment(AccountTestInvoicingCommon):
             'credit': 50.0,
             'amount_currency': -50.0,
             'currency_id': self.company_data['currency'].id,
-            'account_id': copy_receivable.id,
+            'account_id': self.company_data['default_account_receivable'].id,
         }
 
         self.assertRecordValues(payment, [expected_payment_values])
@@ -238,8 +343,12 @@ class TestAccountPayment(AccountTestInvoicingCommon):
 
         # ==== Check editing the account.payment ====
 
+        # `partner_type` on payment is always invisible. It's supposed to be set through a context `default_` key
+        # In this case the goal of the test is to take an existing customer payment and change it to a supplier payment,
+        # which is not supposed to be possible through the web interface.
+        # So, change the payment partner_type beforehand rather than in the form view.
+        payment.partner_type = 'supplier'
         pay_form = Form(payment)
-        pay_form.partner_type = 'supplier'
         pay_form.currency_id = self.currency_data['currency']
         pay_form.partner_id = self.partner_a
         payment = pay_form.save()
@@ -284,13 +393,13 @@ class TestAccountPayment(AccountTestInvoicingCommon):
         with move_form.line_ids.edit(1) as line_form:
             line_form.currency_id = self.company_data['currency']
             line_form.amount_currency = -75.0
-            line_form.account_id = copy_receivable
             line_form.partner_id = self.partner_b
+            line_form.account_id = self.company_data['default_account_receivable']
         with move_form.line_ids.new() as line_form:
             line_form.currency_id = self.company_data['currency']
             line_form.amount_currency = -25.0
-            line_form.account_id = self.company_data['default_account_revenue']
             line_form.partner_id = self.partner_b
+            line_form.account_id = self.company_data['default_account_revenue']
         move_form.save()
 
         self.assertRecordValues(payment, [{
@@ -333,7 +442,7 @@ class TestAccountPayment(AccountTestInvoicingCommon):
 
         # ==== Edit the account.move.line ====
 
-        liquidity_lines, counterpart_lines, writeoff_lines = payment._seek_for_lines()
+        liquidity_lines, counterpart_lines, dummy = payment._seek_for_lines()
         payment.move_id.write({
             'line_ids': [
                 (1, liquidity_lines.id, {'debit': 100.0}),
@@ -383,7 +492,7 @@ class TestAccountPayment(AccountTestInvoicingCommon):
 
         # ==== Edit the account.move.line ====
 
-        liquidity_lines, counterpart_lines, writeoff_lines = payment._seek_for_lines()
+        liquidity_lines, counterpart_lines, dummy = payment._seek_for_lines()
         payment.move_id.write({
             'line_ids': [
                 (1, liquidity_lines.id, {'debit': 100.0}),
@@ -433,7 +542,7 @@ class TestAccountPayment(AccountTestInvoicingCommon):
 
         # ==== Edit the account.move.line ====
 
-        liquidity_lines, counterpart_lines, writeoff_lines = payment._seek_for_lines()
+        liquidity_lines, counterpart_lines, dummy = payment._seek_for_lines()
         payment.move_id.write({
             'line_ids': [
                 (1, liquidity_lines.id, {'credit': 100.0}),
@@ -483,7 +592,7 @@ class TestAccountPayment(AccountTestInvoicingCommon):
 
         # ==== Edit the account.move.line ====
 
-        liquidity_lines, counterpart_lines, writeoff_lines = payment._seek_for_lines()
+        liquidity_lines, counterpart_lines, dummy = payment._seek_for_lines()
         payment.move_id.write({
             'line_ids': [
                 (1, liquidity_lines.id, {'credit': 100.0}),
@@ -530,6 +639,7 @@ class TestAccountPayment(AccountTestInvoicingCommon):
         payment = self.env['account.payment'].create({
             'amount': 50.0,
             'is_internal_transfer': True,
+            'destination_journal_id': self.company_data['default_journal_cash'].id,
         })
 
         expected_payment_values = {
@@ -614,6 +724,49 @@ class TestAccountPayment(AccountTestInvoicingCommon):
             expected_liquidity_line,
         ])
 
+        # ==== Check creation of paired internal transfer payment ====
+
+        payment = self.env['account.payment'].create({
+            'amount': 50.0,
+            'is_internal_transfer': True,
+            'payment_type': 'inbound',
+            'destination_journal_id': self.company_data['default_journal_cash'].id,
+        })
+        payment.action_post()
+        paired_payment = self.env['account.payment'].search([('payment_type', '=', 'outbound')])
+        expected_payment_values = {
+            'amount': 50.0,
+            'payment_type': 'outbound',
+            'currency_id': self.company_data['currency'].id,
+            'partner_id': self.company_data['company'].partner_id.id,
+            'destination_account_id': self.company_data['company'].transfer_account_id.id,
+        }
+        expected_move_values = {
+            'currency_id': self.company_data['currency'].id,
+            'partner_id': self.company_data['company'].partner_id.id,
+        }
+        expected_liquidity_line = {
+            'debit': 0.0,
+            'credit': 50.0,
+            'amount_currency': -50.0,
+            'currency_id': self.company_data['currency'].id,
+            'account_id': self.company_data['default_journal_cash'].company_id.account_journal_payment_credit_account_id.id,
+        }
+        expected_counterpart_line = {
+            'debit': 50.0,
+            'credit': 0.0,
+            'amount_currency': 50.0,
+            'currency_id': self.company_data['currency'].id,
+            'account_id': self.company_data['company'].transfer_account_id.id,
+        }
+
+        self.assertRecordValues(paired_payment, [expected_payment_values])
+        self.assertRecordValues(paired_payment.move_id, [expected_move_values])
+        self.assertRecordValues(paired_payment.line_ids.sorted('balance'), [
+            expected_liquidity_line,
+            expected_counterpart_line,
+        ])
+
     def test_compute_currency_id(self):
         ''' When creating a new account.payment without specifying a currency, the default currency should be the one
         set on the journal.
@@ -679,7 +832,7 @@ class TestAccountPayment(AccountTestInvoicingCommon):
         payment.action_post()
         invoice.action_post()
 
-        (counterpart_lines + invoice.line_ids.filtered(lambda line: line.account_internal_type == 'receivable'))\
+        (counterpart_lines + invoice.line_ids.filtered(lambda line: line.account_type == 'asset_receivable'))\
             .reconcile()
 
         self.assertRecordValues(payment, [{
@@ -687,21 +840,19 @@ class TestAccountPayment(AccountTestInvoicingCommon):
             'is_matched': False,
         }])
 
-        statement = self.env['account.bank.statement'].create({
-            'name': 'test_statement',
+        statement_line = self.env['account.bank.statement.line'].create({
+            'payment_ref': '50 to pay',
             'journal_id': self.company_data['default_journal_bank'].id,
-            'line_ids': [
-                (0, 0, {
-                    'payment_ref': '50 to pay',
-                    'partner_id': self.partner_a.id,
-                    'amount': 50.0,
-                }),
-            ],
+            'partner_id': self.partner_a.id,
+            'amount': 50.0,
         })
-        statement.button_post()
-        statement_line = statement.line_ids
 
-        statement_line.reconcile([{'id': liquidity_lines.id}])
+        # Reconcile without the bank reconciliation widget since the widget is in enterprise.
+        _st_liquidity_lines, st_suspense_lines, _st_other_lines = statement_line\
+            .with_context(skip_account_move_synchronization=True)\
+            ._seek_for_lines()
+        st_suspense_lines.account_id = liquidity_lines.account_id
+        (st_suspense_lines + liquidity_lines).reconcile()
 
         self.assertRecordValues(payment, [{
             'is_reconciled': True,
@@ -715,52 +866,139 @@ class TestAccountPayment(AccountTestInvoicingCommon):
         payment = AccountPayment.create({
             'journal_id': self.company_data['default_journal_bank'].id,
         })
-        self.assertRegex(payment.name, 'BNK1/\d{4}/\d{2}/0001')
+        self.assertRegex(payment.name, r'BNK1/\d{4}/00001')
 
         with Form(AccountPayment.with_context(default_move_journal_types=('bank', 'cash'))) as payment_form:
             self.assertEqual(payment_form._values['name'], '/')
             payment_form.journal_id = self.company_data['default_journal_cash']
-            self.assertRegex(payment_form._values['name'], 'CSH1/\d{4}/\d{2}/0001')
+            self.assertRegex(payment_form._values['name'], r'CSH1/\d{4}/00001')
             payment_form.journal_id = self.company_data['default_journal_bank']
         payment = payment_form.save()
         self.assertEqual(payment.name, '/')
         payment.action_post()
-        self.assertRegex(payment.name, 'BNK1/\d{4}/\d{2}/0002')
+        self.assertRegex(payment.name, r'BNK1/\d{4}/00002')
 
-    def test_payment_form_onchange_journal(self):
-        pay_form = Form(self.env['account.payment'])
-        pay_form.journal_id = self.company_data['default_journal_bank']
-        pay_form.partner_id = self.partner_a
-        pay_form.name = '/'
-        pay_form.amount = 123
-        pay = pay_form.save()
+    def test_payment_without_default_company_account(self):
+        """ The purpose of this test is to check the specific behavior when duplicating an inbound payment, then change
+        the copy to an outbound payment when we set the outstanding accounts (payments and receipts) on a journal but
+        not on the company level.
+        """
+        company = self.company_data['company']
+        bank_journal = self.company_data['default_journal_bank']
 
-        self.assertRecordValues(pay.line_ids.sorted('balance'), [
-            {
-                'debit': 0.0,
-                'credit': 123.0,
-                'account_id': self.company_data['default_account_receivable'].id,
-            },
-            {
-                'debit': 123.0,
-                'credit': 0.0,
-                'account_id': self.company_data['default_journal_bank'].payment_debit_account_id.id,
-            },
+        bank_journal.outbound_payment_method_line_ids.payment_account_id = company.account_journal_payment_credit_account_id
+        bank_journal.inbound_payment_method_line_ids.payment_account_id = company.account_journal_payment_debit_account_id
+        company.account_journal_payment_debit_account_id = False
+        company.account_journal_payment_credit_account_id = False
+
+        payment = self.env['account.payment'].create({
+            'amount': 5.0,
+            'payment_type': 'inbound',
+            'partner_type': 'customer',
+            'journal_id': bank_journal.id,
+        })
+        self.assertRecordValues(payment, [{
+            'amount': 5.0,
+            'payment_type': 'inbound',
+            'partner_type': 'customer',
+            'payment_reference': False,
+            'is_reconciled': False,
+            'currency_id': self.company_data['currency'].id,
+            'partner_id': False,
+            'payment_method_line_id': self.inbound_payment_method_line.id,
+        }])
+
+        payment.payment_type = 'outbound'
+        self.assertRecordValues(payment, [{
+            'amount': 5.0,
+            'payment_type': 'outbound',
+            'partner_type': 'customer',
+            'payment_reference': False,
+            'is_reconciled': False,
+            'currency_id': self.company_data['currency'].id,
+            'partner_id': False,
+            'payment_method_line_id': self.outbound_payment_method_line.id,
+        }])
+
+    def test_suggested_default_partner_bank(self):
+        """ Ensure the 'partner_bank_id' is well computed on payments. When the payment is inbound, the money must be
+        received by a bank account linked to the company. In case of outbound payment, the bank account must be found
+        on the partner.
+        """
+        payment = self.env['account.payment'].create({
+            'journal_id': self.bank_journal_1.id,
+            'amount': 50.0,
+            'payment_type': 'outbound',
+            'partner_type': 'supplier',
+            'partner_id': self.partner_a.id,
+        })
+        self.assertRecordValues(payment, [{
+            'available_partner_bank_ids': self.partner_a.bank_ids.ids,
+            'partner_bank_id': self.partner_bank_account1.id,
+        }])
+
+        payment.payment_type = 'inbound'
+        self.assertRecordValues(payment, [{
+            'available_partner_bank_ids': [],
+            'partner_bank_id': False,
+        }])
+
+        self.bank_journal_2.bank_account_id = self.comp_bank_account2
+        # A sequence is automatically added on the first move. We need to clean it before changing the journal.
+        payment.name = False
+        payment.journal_id = self.bank_journal_2
+        self.assertRecordValues(payment, [{
+            'available_partner_bank_ids': self.comp_bank_account2.ids,
+            'partner_bank_id': self.comp_bank_account2.id,
+        }])
+
+    def test_internal_transfer_right_accounts(self):
+        """The purpose of this test is to check that the right accounts are computed when making an internal bank transfer"""
+
+        company = self.env.company
+        transfer_account = company.transfer_account_id
+        bank = self.bank_journal_1
+        bank_2 = self.bank_journal_2
+
+        payment = self.env['account.payment'].create({
+            'is_internal_transfer': True,
+            'payment_type': 'outbound',
+            'amount': 100.0,
+            'journal_id': bank.id,
+            'destination_journal_id': bank_2.id,
+        })
+        payment.action_post()
+
+        self.assertRecordValues(payment.line_ids, [
+            {'account_id': company.account_journal_payment_credit_account_id.id},
+            {'account_id': transfer_account.id},
+        ])
+        self.assertRecordValues(payment.paired_internal_transfer_payment_id.line_ids, [
+            {'account_id': company.account_journal_payment_debit_account_id.id},
+            {'account_id': transfer_account.id},
         ])
 
-        with Form(pay) as pay_form:
-            pay_form.journal_id = self.company_data['default_journal_cash']
-            pay_form.name = '/'
+        # We check the behavior when setting specific receipts/payments account on bank journals
+        bank.inbound_payment_method_line_ids.payment_account_id = company.account_journal_payment_debit_account_id.copy({'code': 1001})
+        bank.outbound_payment_method_line_ids.payment_account_id = company.account_journal_payment_credit_account_id.copy({'code': 1002})
 
-        self.assertRecordValues(pay.line_ids.sorted('balance'), [
-            {
-                'debit': 0.0,
-                'credit': 123.0,
-                'account_id': self.company_data['default_account_receivable'].id,
-            },
-            {
-                'debit': 123.0,
-                'credit': 0.0,
-                'account_id': self.company_data['default_journal_cash'].payment_debit_account_id.id,
-            },
+        bank_2.inbound_payment_method_line_ids.payment_account_id = company.account_journal_payment_debit_account_id.copy({'code': 2001})
+        bank_2.outbound_payment_method_line_ids.payment_account_id = company.account_journal_payment_credit_account_id.copy({'code': 2002})
+
+        payment = self.env['account.payment'].create({
+            'is_internal_transfer': True,
+            'payment_type': 'outbound',
+            'amount': 100.0,
+            'journal_id': bank.id,
+            'destination_journal_id': bank_2.id,
+        })
+        payment.action_post()
+
+        self.assertRecordValues(payment.line_ids, [
+            {'account_id': bank.outbound_payment_method_line_ids.payment_account_id.id,},
+            {'account_id': transfer_account.id},
+        ])
+        self.assertRecordValues(payment.paired_internal_transfer_payment_id.line_ids, [
+            {'account_id': bank_2.inbound_payment_method_line_ids.payment_account_id.id},
+            {'account_id': transfer_account.id},
         ])
