@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
-from odoo import fields
+# pylint: disable=C0326
+from odoo import Command
 from odoo.addons.account.tests.common import AccountTestInvoicingCommon
 from odoo.tests import tagged, Form
 
@@ -52,27 +53,46 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             'sequence': 40,
         })
 
-        cls.tax_report = cls.env['account.tax.report'].create({
+        tax_report = cls.env['account.report'].create({
             'name': "Tax report",
             'country_id': cls.company_data['company'].country_id.id,
+            'column_ids': [
+                Command.create({
+                    'name': "Balance",
+                    'expression_label': 'balance',
+                }),
+            ],
         })
 
-        cls.tax_report_line = cls.env['account.tax.report.line'].create({
+        tax_report_line = cls.env['account.report.line'].create({
             'name': 'test_tax_report_line',
-            'tag_name': 'test_tax_report_line',
-            'report_id': cls.tax_report.id,
+            'report_id': tax_report.id,
             'sequence': 10,
+            'expression_ids': [
+                Command.create({
+                    'label': 'balance',
+                    'engine': 'tax_tags',
+                    'formula': 'test_tax_report_line',
+                }),
+            ],
         })
-        cls.tax_tag_pos = cls.tax_report_line.tag_ids.filtered(lambda x: not x.tax_negate)
-        cls.tax_tag_neg = cls.tax_report_line.tag_ids.filtered(lambda x: x.tax_negate)
-        cls.base_tax_report_line = cls.env['account.tax.report.line'].create({
+        tax_tags = tax_report_line.expression_ids._get_matching_tags()
+        cls.tax_tag_pos, cls.tax_tag_neg = tax_tags.sorted('tax_negate')
+
+        base_report_line = cls.env['account.report.line'].create({
             'name': 'base_test_tax_report_line',
-            'tag_name': 'base_test_tax_report_line',
-            'report_id': cls.tax_report.id,
+            'report_id': tax_report.id,
             'sequence': 10,
+            'expression_ids': [
+                Command.create({
+                    'label': 'balance',
+                    'engine': 'tax_tags',
+                    'formula': 'base_test_tax_report_line',
+                }),
+            ],
         })
-        cls.base_tag_pos = cls.base_tax_report_line.tag_ids.filtered(lambda x: not x.tax_negate)
-        cls.base_tag_neg = cls.base_tax_report_line.tag_ids.filtered(lambda x: x.tax_negate)
+        base_tags = base_report_line.expression_ids._get_matching_tags()
+        cls.base_tag_pos, cls.base_tag_neg = base_tags.sorted('tax_negate')
 
     def _create_invoice(self, taxes_per_line, inv_type='out_invoice', currency_id=False, invoice_payment_term_id=False):
         ''' Create an invoice on the fly.
@@ -95,6 +115,22 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             vals['invoice_payment_term_id'] = invoice_payment_term_id.id
         return self.env['account.move'].create(vals)
 
+    def test_setting_tax_separately(self):
+        ''' Test:
+        price_unit | Taxes
+        ------------------
+        100        | 21%
+        Expected:
+        Tax         | Taxes     | Base      | Amount
+        --------------------------------------------
+        21%         | /         | 100       | 21
+        '''
+        invoice = self._create_invoice([(100, self.env['account.tax'])])
+        invoice.invoice_line_ids[0].tax_ids = self.percent_tax_1
+        self.assertRecordValues(invoice.line_ids.filtered('tax_line_id'), [
+            {'name': self.percent_tax_1.name, 'tax_base_amount': 100, 'balance': -21, 'tax_ids': []},
+        ])
+
     def test_one_tax_per_line(self):
         ''' Test:
         price_unit | Taxes
@@ -116,10 +152,10 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             (100, self.percent_tax_2),
         ])
         invoice.action_post()
-        self.assertRecordValues(invoice.line_ids.filtered('tax_line_id'), [
-            {'name': self.percent_tax_1.name,       'tax_base_amount': 100, 'price_unit': 21, 'tax_ids': []},
-            {'name': self.percent_tax_1_incl.name,  'tax_base_amount': 100, 'price_unit': 21, 'tax_ids': []},
-            {'name': self.percent_tax_2.name,       'tax_base_amount': 100, 'price_unit': 12, 'tax_ids': []},
+        self.assertRecordValues(invoice.line_ids.filtered('tax_line_id').sorted(lambda x: x.name), [
+            {'name': self.percent_tax_2.name,       'tax_base_amount': 100, 'balance': -12, 'tax_ids': []},
+            {'name': self.percent_tax_1.name,       'tax_base_amount': 100, 'balance': -21, 'tax_ids': []},
+            {'name': self.percent_tax_1_incl.name,  'tax_base_amount': 100, 'balance': -21, 'tax_ids': []},
         ])
 
     def test_affecting_base_amount(self):
@@ -141,9 +177,9 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             (100, self.percent_tax_2),
         ])
         invoice.action_post()
-        self.assertRecordValues(invoice.line_ids.filtered('tax_line_id').sorted(lambda x: x.price_unit), [
-            {'name': self.percent_tax_1_incl.name,      'tax_base_amount': 100, 'price_unit': 21,      'tax_ids': [self.percent_tax_2.id]},
-            {'name': self.percent_tax_2.name,           'tax_base_amount': 221, 'price_unit': 26.52,   'tax_ids': []},
+        self.assertRecordValues(invoice.line_ids.filtered('tax_line_id').sorted(lambda x: -x.balance), [
+            {'name': self.percent_tax_1_incl.name,      'tax_base_amount': 100, 'balance': -21,      'tax_ids': [self.percent_tax_2.id]},
+            {'name': self.percent_tax_2.name,           'tax_base_amount': 221, 'balance': -26.52,   'tax_ids': []},
         ])
 
     def test_group_of_taxes(self):
@@ -165,9 +201,10 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             (100, self.percent_tax_2),
         ])
         invoice.action_post()
-        self.assertRecordValues(invoice.line_ids.filtered('tax_line_id').sorted(lambda x: x.price_unit), [
-            {'name': self.percent_tax_1_incl.name,      'tax_base_amount': 100, 'price_unit': 21,      'tax_ids': [self.percent_tax_2.id]},
-            {'name': self.percent_tax_2.name,           'tax_base_amount': 221, 'price_unit': 26.52,   'tax_ids': []},
+        self.assertRecordValues(invoice.line_ids.filtered('tax_line_id').sorted('balance'), [
+            {'name': self.percent_tax_1_incl.name,      'tax_base_amount': 100.0,   'balance': -21.0,   'tax_ids': [self.percent_tax_2.id]},
+            {'name': self.percent_tax_2.name,           'tax_base_amount': 121.0,   'balance': -14.52,  'tax_ids': []},
+            {'name': self.percent_tax_2.name,           'tax_base_amount': 100.0,   'balance': -12.0,   'tax_ids': []},
         ])
 
     def _create_tax_tag(self, tag_name):
@@ -184,9 +221,8 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
         ref_base_tag = self._create_tax_tag('refund_base')
         ref_tax_tag = self._create_tax_tag('refund_tax')
 
-        user_type = self.env.ref('account.data_account_type_current_assets')
-        account_1 = self.env['account.account'].create({'name': 'test1', 'code': 'test1', 'user_type_id': user_type.id})
-        account_2 = self.env['account.account'].create({'name': 'test2', 'code': 'test2', 'user_type_id': user_type.id})
+        account_1 = self.env['account.account'].create({'name': 'test1', 'code': 'test1', 'account_type': 'asset_current'})
+        account_2 = self.env['account.account'].create({'name': 'test2', 'code': 'test2', 'account_type': 'asset_current'})
 
         tax = self.env['account.tax'].create({
             'name': "Tax with account",
@@ -195,7 +231,6 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             'amount': 42,
             'invoice_repartition_line_ids': [
                 (0,0, {
-                    'factor_percent': 100,
                     'repartition_type': 'base',
                     'tag_ids': [(4, inv_base_tag.id, 0)],
                 }),
@@ -216,7 +251,6 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             ],
             'refund_repartition_line_ids': [
                 (0,0, {
-                    'factor_percent': 100,
                     'repartition_type': 'base',
                     'tag_ids': [(4, ref_base_tag.id, 0)],
                 }),
@@ -241,7 +275,7 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
         invoice.action_post()
 
         self.assertEqual(len(invoice.line_ids), 4, "There should be 4 account move lines created for the invoice: payable, base and 2 tax lines")
-        inv_base_line = invoice.line_ids.filtered(lambda x: not x.tax_repartition_line_id and x.account_id.user_type_id.type != 'receivable')
+        inv_base_line = invoice.line_ids.filtered(lambda x: not x.tax_repartition_line_id and x.account_id.account_type != 'asset_receivable')
         self.assertEqual(len(inv_base_line), 1, "There should be only one base line generated")
         self.assertEqual(abs(inv_base_line.balance), 100, "Base amount should be 100")
         self.assertEqual(inv_base_line.tax_tag_ids, inv_base_tag, "Base line should have received base tag")
@@ -257,7 +291,7 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
         refund.action_post()
 
         self.assertEqual(len(refund.line_ids), 4, "There should be 4 account move lines created for the refund: payable, base and 2 tax lines")
-        ref_base_line = refund.line_ids.filtered(lambda x: not x.tax_repartition_line_id and x.account_id.user_type_id.type != 'receivable')
+        ref_base_line = refund.line_ids.filtered(lambda x: not x.tax_repartition_line_id and x.account_id.account_type != 'asset_receivable')
         self.assertEqual(len(ref_base_line), 1, "There should be only one base line generated")
         self.assertEqual(abs(ref_base_line.balance), 100, "Base amount should be 100")
         self.assertEqual(ref_base_line.tax_tag_ids, ref_base_tag, "Base line should have received base tag")
@@ -296,7 +330,7 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
         }])
         # change price unit, everything should change as well
         with Form(invoice) as invoice_form:
-            with invoice_form.line_ids.edit(0) as line_edit:
+            with invoice_form.invoice_line_ids.edit(0) as line_edit:
                 line_edit.price_unit = 200
 
         self.assertRecordValues(invoice.line_ids.filtered('tax_line_id'), [{
@@ -314,24 +348,20 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             'invoice_repartition_line_ids': [
                 (0, 0, {
                     'repartition_type': 'base',
-                    'factor_percent': 100.0,
                     'tag_ids': [(6, 0, self.base_tag_pos.ids)],
                 }),
                 (0, 0, {
                     'repartition_type': 'tax',
-                    'factor_percent': 100.0,
                     'tag_ids': [(6, 0, self.tax_tag_pos.ids)],
                 }),
             ],
             'refund_repartition_line_ids': [
                 (0, 0, {
                     'repartition_type': 'base',
-                    'factor_percent': 100.0,
                     'tag_ids': [(6, 0, self.base_tag_neg.ids)],
                 }),
                 (0, 0, {
                     'repartition_type': 'tax',
-                    'factor_percent': 100.0,
                     'tag_ids': [(6, 0, self.tax_tag_neg.ids)],
                 }),
             ],
@@ -353,7 +383,6 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             credit_line.tax_ids.clear()
             credit_line.tax_ids.add(sale_tax)
 
-            self.assertTrue(credit_line.recompute_tax_line)
 
         # Balance the journal entry.
         with move_form.line_ids.new() as credit_line:
@@ -364,9 +393,9 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
         move = move_form.save()
 
         self.assertRecordValues(move.line_ids.sorted('balance'), [
-            {'balance': -1100.0,    'tax_ids': [],              'tax_tag_ids': [],                      'tax_base_amount': 0,       'tax_repartition_line_id': False},
-            {'balance': 100.0,      'tax_ids': [],              'tax_tag_ids': self.tax_tag_neg.ids,    'tax_base_amount': 1000,    'tax_repartition_line_id': ref_tax_rep_ln.id},
-            {'balance': 1000.0,     'tax_ids': sale_tax.ids,    'tax_tag_ids': self.base_tag_neg.ids,   'tax_base_amount': 0,       'tax_repartition_line_id': False},
+            {'balance': -1100.0,    'tax_ids': [],              'tax_tag_ids': [],                      'tax_base_amount': 0,       'tax_repartition_line_id': False,               'tax_tag_invert': False},
+            {'balance': 100.0,      'tax_ids': [],              'tax_tag_ids': self.tax_tag_neg.ids,    'tax_base_amount': 1000,    'tax_repartition_line_id': ref_tax_rep_ln.id,   'tax_tag_invert': False},
+            {'balance': 1000.0,     'tax_ids': sale_tax.ids,    'tax_tag_ids': self.base_tag_neg.ids,   'tax_base_amount': 0,       'tax_repartition_line_id': False,               'tax_tag_invert': False},
         ])
 
         # === Tax in credit ===
@@ -382,8 +411,6 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             credit_line.tax_ids.clear()
             credit_line.tax_ids.add(sale_tax)
 
-            self.assertTrue(credit_line.recompute_tax_line)
-
         # Balance the journal entry.
         with move_form.line_ids.new() as debit_line:
             debit_line.name = 'balance'
@@ -393,9 +420,9 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
         move = move_form.save()
 
         self.assertRecordValues(move.line_ids.sorted('balance'), [
-            {'balance': -1000.0,    'tax_ids': sale_tax.ids,    'tax_tag_ids': self.base_tag_neg.ids,   'tax_base_amount': 0,       'tax_repartition_line_id': False},
-            {'balance': -100.0,     'tax_ids': [],              'tax_tag_ids': self.tax_tag_neg.ids,    'tax_base_amount': 1000,    'tax_repartition_line_id': inv_tax_rep_ln.id},
-            {'balance': 1100.0,     'tax_ids': [],              'tax_tag_ids': [],                      'tax_base_amount': 0,       'tax_repartition_line_id': False},
+            {'balance': -1000.0,    'tax_ids': sale_tax.ids,    'tax_tag_ids': self.base_tag_pos.ids,   'tax_base_amount': 0,       'tax_repartition_line_id': False,                'tax_tag_invert': True},
+            {'balance': -100.0,     'tax_ids': [],              'tax_tag_ids': self.tax_tag_pos.ids,    'tax_base_amount': 1000,    'tax_repartition_line_id': inv_tax_rep_ln.id,    'tax_tag_invert': True},
+            {'balance': 1100.0,     'tax_ids': [],              'tax_tag_ids': [],                      'tax_base_amount': 0,       'tax_repartition_line_id': False,                'tax_tag_invert': False},
         ])
 
     def test_misc_journal_entry_tax_tags_purchase(self):
@@ -407,24 +434,20 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             'invoice_repartition_line_ids': [
                 (0, 0, {
                     'repartition_type': 'base',
-                    'factor_percent': 100.0,
                     'tag_ids': [(6, 0, self.base_tag_pos.ids)],
                 }),
                 (0, 0, {
                     'repartition_type': 'tax',
-                    'factor_percent': 100.0,
                     'tag_ids': [(6, 0, self.tax_tag_pos.ids)],
                 }),
             ],
             'refund_repartition_line_ids': [
                 (0, 0, {
                     'repartition_type': 'base',
-                    'factor_percent': 100.0,
                     'tag_ids': [(6, 0, self.base_tag_neg.ids)],
                 }),
                 (0, 0, {
                     'repartition_type': 'tax',
-                    'factor_percent': 100.0,
                     'tag_ids': [(6, 0, self.tax_tag_neg.ids)],
                 }),
             ],
@@ -446,8 +469,6 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             credit_line.tax_ids.clear()
             credit_line.tax_ids.add(purch_tax)
 
-            self.assertTrue(credit_line.recompute_tax_line)
-
         # Balance the journal entry.
         with move_form.line_ids.new() as credit_line:
             credit_line.name = 'balance'
@@ -457,9 +478,9 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
         move = move_form.save()
 
         self.assertRecordValues(move.line_ids.sorted('balance'), [
-            {'balance': -1100.0,    'tax_ids': [],              'tax_tag_ids': [],                      'tax_base_amount': 0,       'tax_repartition_line_id': False},
-            {'balance': 100.0,      'tax_ids': [],              'tax_tag_ids': self.tax_tag_pos.ids,    'tax_base_amount': 1000,    'tax_repartition_line_id': inv_tax_rep_ln.id},
-            {'balance': 1000.0,     'tax_ids': purch_tax.ids,   'tax_tag_ids': self.base_tag_pos.ids,   'tax_base_amount': 0,       'tax_repartition_line_id': False},
+            {'balance': -1100.0,    'tax_ids': [],              'tax_tag_ids': [],                      'tax_base_amount': 0,       'tax_repartition_line_id': False,                'tax_tag_invert': False},
+            {'balance': 100.0,      'tax_ids': [],              'tax_tag_ids': self.tax_tag_pos.ids,    'tax_base_amount': 1000,    'tax_repartition_line_id': inv_tax_rep_ln.id,    'tax_tag_invert': False},
+            {'balance': 1000.0,     'tax_ids': purch_tax.ids,   'tax_tag_ids': self.base_tag_pos.ids,   'tax_base_amount': 0,       'tax_repartition_line_id': False,                'tax_tag_invert': False},
         ])
 
         # === Tax in credit ===
@@ -475,8 +496,6 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
             credit_line.tax_ids.clear()
             credit_line.tax_ids.add(purch_tax)
 
-            self.assertTrue(credit_line.recompute_tax_line)
-
         # Balance the journal entry.
         with move_form.line_ids.new() as debit_line:
             debit_line.name = 'balance'
@@ -486,9 +505,120 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
         move = move_form.save()
 
         self.assertRecordValues(move.line_ids.sorted('balance'), [
-            {'balance': -1000.0,    'tax_ids': purch_tax.ids,   'tax_tag_ids': self.base_tag_pos.ids,   'tax_base_amount': 0,       'tax_repartition_line_id': False},
-            {'balance': -100.0,     'tax_ids': [],              'tax_tag_ids': self.tax_tag_pos.ids,    'tax_base_amount': 1000,    'tax_repartition_line_id': ref_tax_rep_ln.id},
-            {'balance': 1100.0,     'tax_ids': [],              'tax_tag_ids': [],                      'tax_base_amount': 0,       'tax_repartition_line_id': False},
+            {'balance': -1000.0,    'tax_ids': purch_tax.ids,   'tax_tag_ids': self.base_tag_neg.ids,   'tax_base_amount': 0,       'tax_repartition_line_id': False,                'tax_tag_invert': True},
+            {'balance': -100.0,     'tax_ids': [],              'tax_tag_ids': self.tax_tag_neg.ids,    'tax_base_amount': 1000,    'tax_repartition_line_id': ref_tax_rep_ln.id,    'tax_tag_invert': True},
+            {'balance': 1100.0,     'tax_ids': [],              'tax_tag_ids': [],                      'tax_base_amount': 0,       'tax_repartition_line_id': False,                'tax_tag_invert': False},
+        ])
+
+    def test_misc_entry_tax_group_signs(self):
+        """ Tests sign inversion of the tags on misc operations made with tax
+        groups.
+        """
+        def _create_group_of_taxes(tax_type):
+            # We use asymmetric tags between the child taxes to avoid shadowing errors
+            child1_sale_tax = self.env['account.tax'].create({
+                'sequence': 1,
+                'name': 'child1_%s' % tax_type,
+                'type_tax_use': 'none',
+                'amount_type': 'percent',
+                'amount': 5,
+                'invoice_repartition_line_ids': [
+                    (0, 0, {
+                        'repartition_type': 'base',
+                        'tag_ids': [(6, 0, self.base_tag_pos.ids)],
+                    }),
+                    (0, 0, {
+                        'repartition_type': 'tax',
+                        'tag_ids': [(6, 0, self.tax_tag_pos.ids)],
+                    }),
+                ],
+                'refund_repartition_line_ids': [
+                    (0, 0, {'repartition_type': 'base'}),
+                    (0, 0, {'repartition_type': 'tax'}),
+                ],
+            })
+            child2_sale_tax = self.env['account.tax'].create({
+                'sequence': 2,
+                'name': 'child2_%s' % tax_type,
+                'type_tax_use': 'none',
+                'amount_type': 'percent',
+                'amount': 10,
+                'invoice_repartition_line_ids': [
+                    (0, 0, {'repartition_type': 'base'}),
+                    (0, 0, {'repartition_type': 'tax'}),
+                ],
+                'refund_repartition_line_ids': [
+                    (0, 0, {
+                        'repartition_type': 'base',
+                        'tag_ids': [(6, 0, self.base_tag_neg.ids)],
+                    }),
+                    (0, 0, {
+                        'repartition_type': 'tax',
+                        'tag_ids': [(6, 0, self.tax_tag_neg.ids)],
+                    }),
+                ],
+            })
+            return self.env['account.tax'].create({
+                'name': 'group_%s' % tax_type,
+                'type_tax_use': tax_type,
+                'amount_type': 'group',
+                'amount': 10,
+                'children_tax_ids':[(6,0,[child1_sale_tax.id, child2_sale_tax.id])]
+            })
+
+        def _create_misc_operation(tax, tax_field):
+            with Form(self.env['account.move'], view='account.view_move_form') as move_form:
+                for line_field in ('debit', 'credit'):
+                    line_amount = tax_field == line_field and 1000 or 1150
+                    with move_form.line_ids.new() as line_form:
+                        line_form.name = '%s_line' % line_field
+                        line_form.account_id = self.company_data['default_account_revenue']
+                        line_form.debit = line_field == 'debit' and line_amount or 0
+                        line_form.credit = line_field == 'credit' and line_amount or 0
+
+                        if tax_field == line_field:
+                            line_form.tax_ids.clear()
+                            line_form.tax_ids.add(tax)
+
+            return move_form.save()
+
+        sale_group = _create_group_of_taxes('sale')
+        purchase_group = _create_group_of_taxes('purchase')
+
+        # Sale tax on debit: use refund repartition
+        debit_sale_move = _create_misc_operation(sale_group, 'debit')
+        self.assertRecordValues(debit_sale_move.line_ids.sorted('balance'), [
+            {'balance': -1150.0,    'tax_ids': [],                  'tax_tag_ids': [],                      'tax_base_amount': 0},
+            {'balance': 50.0,       'tax_ids': [],                  'tax_tag_ids': [],                      'tax_base_amount': 1000},
+            {'balance': 100.0,      'tax_ids': [],                  'tax_tag_ids': self.tax_tag_neg.ids,    'tax_base_amount': 1000},
+            {'balance': 1000.0,     'tax_ids': sale_group.ids,      'tax_tag_ids': self.base_tag_neg.ids,   'tax_base_amount': 0},
+        ])
+
+        # Sale tax on credit: use invoice repartition
+        credit_sale_move = _create_misc_operation(sale_group, 'credit')
+        self.assertRecordValues(credit_sale_move.line_ids.sorted('balance'), [
+            {'balance': -1000.0,    'tax_ids': sale_group.ids,      'tax_tag_ids': self.base_tag_pos.ids,   'tax_base_amount': 0},
+            {'balance': -100.0,     'tax_ids': [],                  'tax_tag_ids': [],                      'tax_base_amount': 1000},
+            {'balance': -50.0,      'tax_ids': [],                  'tax_tag_ids': self.tax_tag_pos.ids,    'tax_base_amount': 1000},
+            {'balance': 1150.0,     'tax_ids': [],                  'tax_tag_ids': [],                      'tax_base_amount': 0},
+        ])
+
+        # Purchase tax on debit: use invoice repartition
+        debit_purchase_move = _create_misc_operation(purchase_group, 'debit')
+        self.assertRecordValues(debit_purchase_move.line_ids.sorted('balance'), [
+            {'balance': -1150.0,    'tax_ids': [],                  'tax_tag_ids': [],                      'tax_base_amount': 0},
+            {'balance': 50.0,       'tax_ids': [],                  'tax_tag_ids': self.tax_tag_pos.ids,    'tax_base_amount': 1000},
+            {'balance': 100.0,      'tax_ids': [],                  'tax_tag_ids': [],                      'tax_base_amount': 1000},
+            {'balance': 1000.0,     'tax_ids': purchase_group.ids,  'tax_tag_ids': self.base_tag_pos.ids,   'tax_base_amount': 0},
+        ])
+
+        # Purchase tax on credit: use refund repartition
+        credit_purchase_move = _create_misc_operation(purchase_group, 'credit')
+        self.assertRecordValues(credit_purchase_move.line_ids.sorted('balance'), [
+            {'balance': -1000.0,    'tax_ids': purchase_group.ids,  'tax_tag_ids': self.base_tag_neg.ids,   'tax_base_amount': 0},
+            {'balance': -100.0,     'tax_ids': [],                  'tax_tag_ids': self.tax_tag_neg.ids,    'tax_base_amount': 1000},
+            {'balance': -50.0,      'tax_ids': [],                  'tax_tag_ids': [],                      'tax_base_amount': 1000},
+            {'balance': 1150.0,     'tax_ids': [],                  'tax_tag_ids': [],                      'tax_base_amount': 0},
         ])
 
     def test_tax_calculation_foreign_currency_large_quantity(self):
@@ -542,53 +672,65 @@ class TestInvoiceTaxes(AccountTestInvoicingCommon):
         ], currency_id=self.currency_data['currency'], invoice_payment_term_id=self.pay_terms_a)
         invoice.action_post()
 
-    def test_changing_tax_to_caba_after_create_invoice(self):
-        tax_waiting_account = self.env['account.account'].create({
-            'name': 'TAX_WAIT',
-            'code': 'TWAIT',
-            'user_type_id': self.env.ref('account.data_account_type_current_liabilities').id,
-            'reconcile': True,
-            'company_id': self.company_data['company'].id,
-        })
-        sale_tax = self.company_data['default_tax_sale']
-        invoice = self._create_invoice([
-            (100, sale_tax),
-        ])
-        # turn on cash basis on the same tax that was used
-        self.company_data['company'].tax_exigibility = True
-        sale_tax.write({
-            'tax_exigibility': 'on_payment',
-            'cash_basis_transition_account_id': tax_waiting_account.id,
-        })
-        invoice.action_post()
-
-        tax_lines = invoice.line_ids.filtered('tax_line_id')
-        self.assertEqual(len(tax_lines), 1, 'Should have only 1 tax line')
-        self.assertNotEqual(tax_lines.price_unit, 0.0)
-        self.assertEqual(tax_lines.tax_base_amount, 100.0)
-        self.assertEqual(tax_lines.account_id.name, 'TAX_WAIT')
-
-    def test_recompute_tax_when_change_currency(self):
-        """
-        - In a multi currency company, create an invoice with the domestic
-          currency ($);
-        - Add a product (100$) with taxes;
-        -  Change the currency to a foreign one (€);
-        - The product don't change of amount but only of currency (500€);
-        - Save the invoice, and print it.
-        """
-        invoice = self._create_invoice([
-            (100, self.percent_tax_1),
-        ])
-        tax_line = invoice.line_ids.filtered('tax_line_id')
+    def test_tax_calculation_multi_currency(self):
         self.env['res.currency.rate'].create({
-            'name': fields.Date.to_string(fields.Date.today()),
-            'rate': 4.0,
+            'name': '2018-01-01',
+            'rate': 0.273748,
             'currency_id': self.currency_data['currency'].id,
             'company_id': self.env.company.id,
         })
+        self.currency_data['currency'].rounding = 0.01
+
+        invoice = self.env['account.move'].create({
+            'move_type': 'out_invoice',
+            'partner_id': self.partner_a.id,
+            'currency_id': self.currency_data['currency'].id,
+            'invoice_date': '2018-01-01',
+            'date': '2018-01-01',
+            'invoice_line_ids': [(0, 0, {
+                'name': 'xxxx',
+                'quantity': 1,
+                'price_unit': 155.32,
+                'tax_ids': [(6, 0, self.percent_tax_1.ids)],
+            })]
+        })
+
+        self.assertRecordValues(invoice.line_ids.filtered('tax_line_id'), [{
+            'tax_base_amount': 567.38,      # 155.32 * 1 / (1 / 0.273748)
+            'balance': -119.16,             # tax_base_amount * 0.21
+        }])
+
+        self.assertRecordValues(invoice.line_ids.filtered(lambda l: not l.name), [{
+            'balance': 686.54,
+        }])
+
         with Form(invoice) as invoice_form:
             invoice_form.currency_id = self.currency_data['currency']
 
-        self.assertEqual(tax_line.tax_base_amount, 25, 'Should convert tax base amount to company currency')
-        self.assertEqual(tax_line.balance, -5.25, 'Should convert balance to company currency')
+        self.assertRecordValues(invoice.line_ids.filtered('tax_line_id'), [{
+            'tax_base_amount': 567.38,
+            'balance': -119.16,
+        }])
+
+        self.assertRecordValues(invoice.line_ids.filtered(lambda l: l.account_id.account_type == 'asset_receivable'), [{
+            'balance': 686.54,
+        }])
+
+    def test_fixed_tax_with_zero_price(self):
+        fixed_tax = self.env['account.tax'].create({
+            'name': 'Test 5 fixed',
+            'amount_type': 'fixed',
+            'amount': 5,
+        })
+        invoice = self._create_invoice([
+            (0, fixed_tax),
+        ])
+        self.assertRecordValues(invoice.line_ids.filtered('tax_line_id'), [{
+            'credit': 5.0,
+            'debit': 0,
+        }])
+        invoice.invoice_line_ids.quantity = 2
+        self.assertRecordValues(invoice.line_ids.filtered('tax_line_id'), [{
+            'credit': 10.0,
+            'debit': 0,
+        }])
