@@ -1,29 +1,28 @@
 # -*- coding: utf-8 -*-
 from odoo.addons.account.tests.common import AccountTestInvoicingCommon
 from odoo.tests import tagged
-from odoo.tests.common import Form
+from odoo.tests.common import Form, TransactionCase
 from odoo import fields, api, SUPERUSER_ID
-from odoo.exceptions import ValidationError
+from odoo.exceptions import ValidationError, UserError
 from odoo.tools import mute_logger
 
 from dateutil.relativedelta import relativedelta
+from freezegun import freeze_time
 from functools import reduce
 import json
 import psycopg2
 
 
-@tagged('post_install', '-at_install')
-class TestSequenceMixin(AccountTestInvoicingCommon):
-
+class TestSequenceMixinCommon(AccountTestInvoicingCommon):
     @classmethod
     def setUpClass(cls, chart_template_ref=None):
         super().setUpClass(chart_template_ref=chart_template_ref)
         cls.test_move = cls.create_move()
 
     @classmethod
-    def create_move(cls, move_type=None, date=None, journal=None, name=None, post=False):
+    def create_move(cls, date=None, journal=None, name=None, post=False):
         move = cls.env['account.move'].create({
-            'move_type': move_type or 'entry',
+            'move_type': 'entry',
             'date': date or '2016-01-01',
             'line_ids': [
                 (0, None, {
@@ -41,6 +40,9 @@ class TestSequenceMixin(AccountTestInvoicingCommon):
             move.action_post()
         return move
 
+
+@tagged('post_install', '-at_install')
+class TestSequenceMixin(TestSequenceMixinCommon):
     def test_sequence_change_date(self):
         """Change the sequence when we change the date iff it has never been posted."""
         # Check setup
@@ -168,21 +170,21 @@ class TestSequenceMixin(AccountTestInvoicingCommon):
             for i in range(6)
         )
         (invoice + invoice2 + refund + refund2).write({
-            'journal_id': self.company_data['default_journal_sale'],
+            'journal_id': self.company_data['default_journal_sale'].id,
             'partner_id': 1,
             'invoice_date': '2016-01-01',
         })
         (invoice + invoice2).move_type = 'out_invoice'
         (refund + refund2).move_type = 'out_refund'
-        all = (entry + entry2 + invoice + invoice2 + refund + refund2)
-        all.name = False
-        all.action_post()
+        all_moves = (entry + entry2 + invoice + invoice2 + refund + refund2)
+        all_moves.name = False
+        all_moves.action_post()
         self.assertEqual(entry.name, 'MISC/2016/01/0002')
         self.assertEqual(entry2.name, 'MISC/2016/01/0003')
-        self.assertEqual(invoice.name, 'INV/2016/01/0001')
-        self.assertEqual(invoice2.name, 'INV/2016/01/0002')
-        self.assertEqual(refund.name, 'RINV/2016/01/0001')
-        self.assertEqual(refund2.name, 'RINV/2016/01/0002')
+        self.assertEqual(invoice.name, 'INV/2016/00001')
+        self.assertEqual(invoice2.name, 'INV/2016/00002')
+        self.assertEqual(refund.name, 'RINV/2016/00001')
+        self.assertEqual(refund2.name, 'RINV/2016/00002')
 
     def test_journal_sequence_groupby_compute(self):
         """The grouping optimization is correctly done."""
@@ -213,7 +215,7 @@ class TestSequenceMixin(AccountTestInvoicingCommon):
         )
 
         journals[0].code = 'OLD'
-        journals.flush()
+        journals.flush_recordset()
         journal_same_code = self.env['account.journal'].create([{
             'name': 'Journal0',
             'code': 'J0',
@@ -233,22 +235,22 @@ class TestSequenceMixin(AccountTestInvoicingCommon):
     def test_journal_override_sequence_regex(self):
         """There is a possibility to override the regex and change the order of the paramters."""
         self.create_move(date='2020-01-01', name='00000876-G 0002/2020')
-        next = self.create_move(date='2020-01-01')
-        next.action_post()
-        self.assertEqual(next.name, '00000876-G 0002/2021')  # Wait, I didn't want this!
+        next_move = self.create_move(date='2020-01-01')
+        next_move.action_post()
+        self.assertEqual(next_move.name, '00000876-G 0002/2021')  # Wait, I didn't want this!
 
-        next.button_draft()
-        next.name = False
-        next.journal_id.sequence_override_regex = r'^(?P<seq>\d*)(?P<suffix1>.*?)(?P<year>(\d{4})?)(?P<suffix2>)$'
-        next.action_post()
-        self.assertEqual(next.name, '00000877-G 0002/2020')  # Pfew, better!
-        next = self.create_move(date='2020-01-01')
-        next.action_post()
-        self.assertEqual(next.name, '00000878-G 0002/2020')
+        next_move.button_draft()
+        next_move.name = False
+        next_move.journal_id.sequence_override_regex = r'^(?P<seq>\d*)(?P<suffix1>.*?)(?P<year>(\d{4})?)(?P<suffix2>)$'
+        next_move.action_post()
+        self.assertEqual(next_move.name, '00000877-G 0002/2020')  # Pfew, better!
+        next_move = self.create_move(date='2020-01-01')
+        next_move.action_post()
+        self.assertEqual(next_move.name, '00000878-G 0002/2020')
 
-        next = self.create_move(date='2017-05-02')
-        next.action_post()
-        self.assertEqual(next.name, '00000001-G 0002/2017')
+        next_move = self.create_move(date='2017-05-02')
+        next_move.action_post()
+        self.assertEqual(next_move.name, '00000001-G 0002/2017')
 
     def test_journal_sequence_ordering(self):
         """Entries are correctly sorted when posting multiple at once."""
@@ -278,7 +280,7 @@ class TestSequenceMixin(AccountTestInvoicingCommon):
         self.assertEqual(copies[5].name, 'XMISC/2019/00004')
 
         # Can't have twice the same name
-        with self.assertRaises(ValidationError):
+        with self.assertRaises(psycopg2.DatabaseError), mute_logger('odoo.sql_db'), self.env.cr.savepoint():
             copies[0].name = 'XMISC/2019/00001'
 
         # Lets remove the order by date
@@ -347,26 +349,98 @@ class TestSequenceMixin(AccountTestInvoicingCommon):
         reset_never = set_sequence(self.test_move.date + relativedelta(years=2), 'MISC/00100')
         test_date(reset_never.date, 'MISC/00101')  # Keep the new prefix in 2018
 
-    def test_sequence_concurency(self):
-        """Computing the same name in concurent transactions is not allowed."""
-        with self.env.registry.cursor() as cr0,\
-                self.env.registry.cursor() as cr1,\
-                self.env.registry.cursor() as cr2:
-            env0 = api.Environment(cr0, SUPERUSER_ID, {})
-            env1 = api.Environment(cr1, SUPERUSER_ID, {})
-            env2 = api.Environment(cr2, SUPERUSER_ID, {})
+    def test_resequence_clash(self):
+        """Resequence doesn't clash when it uses a name set in the same batch
+        but that will be overriden later."""
+        moves = self.env['account.move']
+        for i in range(3):
+            moves += self.create_move(name=str(i))
+        moves.action_post()
 
-            journal = env0['account.journal'].create({
+        mistake = moves[1]
+        mistake.button_draft()
+        mistake.posted_before = False
+        mistake.with_context(force_delete=True).unlink()
+        moves -= mistake
+
+        self.env['account.resequence.wizard'].create({
+            'move_ids': moves.ids,
+            'first_name': '2',
+        }).resequence()
+
+    @freeze_time('2021-10-01 00:00:00')
+    def test_change_journal_on_first_account_move(self):
+        """Changing the journal on the first move is allowed"""
+        journal = self.env['account.journal'].create({
+            'name': 'awesome journal',
+            'type': 'general',
+            'code': 'AJ',
+        })
+        move = self.env['account.move'].create({})
+        self.assertEqual(move.name, 'MISC/2021/10/0001')
+        with Form(move) as move_form:
+            move_form.journal_id = journal
+        self.assertEqual(move.name, 'AJ/2021/10/0001')
+
+
+@tagged('post_install', '-at_install')
+class TestSequenceMixinDeletion(TestSequenceMixinCommon):
+    @classmethod
+    def setUpClass(cls, chart_template_ref=None):
+        super().setUpClass(chart_template_ref=chart_template_ref)
+        journal = cls.env['account.journal'].create({
+            'name': 'Test sequences - deletion',
+            'code': 'SEQDEL',
+            'type': 'general',
+        })
+
+        cls.move_1_1 = cls.create_move('2021-01-01', journal, name='TOTO/2021/01/0001', post=True)
+        cls.move_1_2 = cls.create_move('2021-01-02', journal, post=True)
+        cls.move_1_3 = cls.create_move('2021-01-03', journal, post=True)
+        cls.move_2_1 = cls.create_move('2021-02-01', journal, post=True)
+        cls.move_draft = cls.create_move('2021-02-02', journal, post=False)
+        cls.move_2_2 = cls.create_move('2021-02-03', journal, post=True)
+        cls.move_3_1 = cls.create_move('2021-02-10', journal, name='TURLUTUTU/21/02/001', post=True)
+
+    def test_sequence_deletion_1(self):
+        """The last element of a sequence chain should always be deletable if in draft state.
+
+        Trying to delete another part of the chain shouldn't work.
+        """
+
+        # A draft move without any name can always be deleted.
+        self.move_draft.unlink()
+
+        # The last element of each sequence chain should allow deletion.
+        # Everything should be deletable if we follow this order (a bit randomized on purpose)
+        for move in (self.move_1_3, self.move_1_2, self.move_3_1, self.move_2_2, self.move_2_1, self.move_1_1):
+            move.button_draft()
+            move.unlink()
+
+    def test_sequence_deletion_2(self):
+        """Can delete in batch."""
+        all_moves = (self.move_1_3 + self.move_1_2 + self.move_3_1 + self.move_2_2 + self.move_2_1 + self.move_1_1)
+        all_moves.button_draft()
+        all_moves.unlink()
+
+
+@tagged('post_install', '-at_install')
+class TestSequenceMixinConcurrency(TransactionCase):
+    def setUp(self):
+        super().setUp()
+        with self.env.registry.cursor() as cr:
+            env = api.Environment(cr, SUPERUSER_ID, {})
+            journal = env['account.journal'].create({
                 'name': 'concurency_test',
                 'code': 'CT',
                 'type': 'general',
             })
-            account = env0['account.account'].create({
+            account = env['account.account'].create({
                 'code': 'CT',
                 'name': 'CT',
-                'user_type_id': env0.ref('account.data_account_type_fixed_assets').id,
+                'account_type': 'asset_fixed',
             })
-            moves = env0['account.move'].create([{
+            moves = env['account.move'].create([{
                 'journal_id': journal.id,
                 'date': fields.Date.from_string('2016-01-01'),
                 'line_ids': [(0, 0, {'name': 'name', 'account_id': account.id})]
@@ -374,23 +448,72 @@ class TestSequenceMixin(AccountTestInvoicingCommon):
             moves.name = '/'
             moves[0].action_post()
             self.assertEqual(moves.mapped('name'), ['CT/2016/01/0001', '/', '/'])
-            env0.cr.commit()
+            env.cr.commit()
+        self.data = {
+            'move_ids': moves.ids,
+            'account_id': account.id,
+            'journal_id': journal.id,
+            'envs': [
+                api.Environment(self.env.registry.cursor(), SUPERUSER_ID, {}),
+                api.Environment(self.env.registry.cursor(), SUPERUSER_ID, {}),
+                api.Environment(self.env.registry.cursor(), SUPERUSER_ID, {}),
+            ],
+        }
+        self.addCleanup(self.cleanUp)
 
-            # start the transactions here on cr2 to simulate concurency with cr1
-            env2.cr.execute('SELECT 1')
-
-            move = env1['account.move'].browse(moves[1].id)
-            move.action_post()
-            env1.cr.commit()
-
-            move = env2['account.move'].browse(moves[2].id)
-            with self.assertRaises(psycopg2.OperationalError), env2.cr.savepoint(), mute_logger('odoo.sql_db'):
-                move.action_post()
-
-            self.assertEqual(moves.mapped('name'), ['CT/2016/01/0001', 'CT/2016/01/0002', '/'])
+    def cleanUp(self):
+        with self.env.registry.cursor() as cr:
+            env = api.Environment(cr, SUPERUSER_ID, {})
+            moves = env['account.move'].browse(self.data['move_ids'])
             moves.button_draft()
             moves.posted_before = False
             moves.unlink()
+            journal = env['account.journal'].browse(self.data['journal_id'])
             journal.unlink()
+            account = env['account.account'].browse(self.data['account_id'])
             account.unlink()
-            env0.cr.commit()
+            env.cr.commit()
+        for env in self.data['envs']:
+            env.cr.close()
+
+    def test_sequence_concurency(self):
+        """Computing the same name in concurent transactions is not allowed."""
+        env0, env1, env2 = self.data['envs']
+
+        # start the transactions here on cr1 to simulate concurency with cr2
+        env1.cr.execute('SELECT 1')
+
+        # post in cr2
+        move = env2['account.move'].browse(self.data['move_ids'][1])
+        move.action_post()
+        env2.cr.commit()
+
+        # try to post in cr1, the retry sould find the right number
+        move = env1['account.move'].browse(self.data['move_ids'][2])
+        move.action_post()
+        env1.cr.commit()
+
+        # check the values
+        moves = env0['account.move'].browse(self.data['move_ids'])
+        self.assertEqual(moves.mapped('name'), ['CT/2016/01/0001', 'CT/2016/01/0002', 'CT/2016/01/0003'])
+
+    def test_sequence_concurency_no_useless_lock(self):
+        """Do not lock needlessly when the sequence is not computed"""
+        env0, env1, env2 = self.data['envs']
+
+        # start the transactions here on cr1 to simulate concurency with cr2
+        env1.cr.execute('SELECT 1')
+
+        # get the last sequence in cr1 (for instance opening a form view)
+        move = env2['account.move'].browse(self.data['move_ids'][1])
+        move.highest_name
+        env2.cr.commit()
+
+        # post in cr1, should work even though cr2 read values
+        move = env1['account.move'].browse(self.data['move_ids'][2])
+        move.action_post()
+        env1.cr.commit()
+
+        # check the values
+        moves = env0['account.move'].browse(self.data['move_ids'])
+        self.assertEqual(moves.mapped('name'), ['CT/2016/01/0001', '/', 'CT/2016/01/0002'])
