@@ -3,7 +3,6 @@ from odoo.addons.account.tests.common import AccountTestInvoicingCommon
 from odoo.tests import tagged
 
 
-@tagged('post_install', '-at_install')
 class TestTaxCommon(AccountTestInvoicingCommon):
 
     @classmethod
@@ -99,25 +98,15 @@ class TestTaxCommon(AccountTestInvoicingCommon):
             'amount': 0,
             'sequence': 8,
             'invoice_repartition_line_ids': [
-                (0,0, {
-                    'factor_percent': 100,
-                    'repartition_type': 'base',
-                }),
-
-                (0,0, {
-                    'factor_percent': 100,
+                (0, 0, {'repartition_type': 'base'}),
+                (0, 0, {
                     'repartition_type': 'tax',
                     'account_id': some_account.id,
                 }),
             ],
             'refund_repartition_line_ids': [
-                (0,0, {
-                    'factor_percent': 100,
-                    'repartition_type': 'base',
-                }),
-
-                (0,0, {
-                    'factor_percent': 100,
+                (0, 0, {'repartition_type': 'base'}),
+                (0, 0, {
                     'repartition_type': 'tax',
                     'account_id': some_account.id,
                 }),
@@ -1046,6 +1035,42 @@ class TestTax(TestTaxCommon):
             res3
         )
 
+    def test_is_base_affected(self):
+        taxes = self.env['account.tax'].create([{
+            'name': 'test_is_base_affected%s' % i,
+            'amount_type': 'percent',
+            'amount': amount,
+            'include_base_amount': include_base_amount,
+            'is_base_affected': is_base_affected,
+            'sequence': i,
+        } for i, amount, include_base_amount, is_base_affected in [
+            (0, 6, True, True),
+            (1, 6, True, False),
+            (2, 10, False, True),
+        ]])
+
+        compute_all_results = taxes.compute_all(100.0)
+
+        # Check the balance of the generated move lines
+        self._check_compute_all_results(
+            123.2,      # 'total_included'
+            100.0,      # 'total_excluded'
+            [
+                # base, amount
+                # -------------------------
+                (100.0, 6.0),
+                (100.0, 6.0),
+                (112.0, 11.2),
+                # -------------------------
+            ],
+            compute_all_results,
+        )
+
+        # Check the tax_ids on tax lines
+        expected_tax_ids_list = [taxes[2].ids, taxes[2].ids, []]
+        tax_ids_list = [tax_line['tax_ids'] for tax_line in compute_all_results['taxes']]
+        self.assertEqual(tax_ids_list, expected_tax_ids_list, "Only a tax affected by previous taxes should have tax_ids set on its tax line when used after an 'include_base_amount' tax.")
+
     def test_mixing_price_included_excluded_with_affect_base(self):
         tax_10_fix = self.env['account.tax'].create({
             'name': "tax_10_fix",
@@ -1073,3 +1098,73 @@ class TestTax(TestTaxCommon):
             ],
             (tax_10_fix + tax_21).compute_all(1210),
         )
+
+    def test_price_included_repartition_sum_0(self):
+        """ Tests the case where a tax with a non-zero value has a sum
+        of tax repartition factors of zero and is included in price. It
+        shouldn't behave in the same way as a 0% tax.
+        """
+        test_tax = self.env['account.tax'].create({
+            'name': "Definitely not a 0% tax",
+            'amount_type': 'percent',
+            'amount': 42,
+            'price_include': True,
+            'invoice_repartition_line_ids': [
+                (0, 0, {'repartition_type': 'base'}),
+
+                (0, 0, {'repartition_type': 'tax'}),
+
+                (0, 0, {
+                    'factor_percent': -100,
+                    'repartition_type': 'tax',
+                }),
+            ],
+            'refund_repartition_line_ids': [
+                (0, 0, {'repartition_type': 'base'}),
+
+                (0, 0, {'repartition_type': 'tax'}),
+
+                (0, 0, {
+                    'factor_percent': -100,
+                    'repartition_type': 'tax',
+                }),
+            ],
+        })
+
+        compute_all_res = test_tax.compute_all(100)
+        self._check_compute_all_results(
+            100,         # 'total_included'
+            100,         # 'total_excluded'
+            [
+                # base , amount
+                # ---------------
+                (100, 42),
+                (100, -42),
+                # ---------------
+            ],
+            compute_all_res
+        )
+
+    def test_parse_name_search(self):
+        list_ten_fixed_tax = self.env["account.tax"]
+        ten_fixed_tax = self.env["account.tax"].create(
+            {"name": "Ten Fixed tax", "amount_type": "fixed", "amount": 10}
+        )
+        list_ten_fixed_tax |= ten_fixed_tax
+        ten_fixed_tax_tix = self.env["account.tax"].create(
+            {"name": "Ten Fixed tax tix", "amount_type": "fixed", "amount": 10}
+        )
+        list_ten_fixed_tax |= ten_fixed_tax_tix
+
+        self.assertListEqual(
+            [x[0] for x in self.env["account.tax"].name_search("tix")],
+            list_ten_fixed_tax.ids,
+        )
+        self.assertListEqual(
+            [x[0] for x in self.env["account.tax"].name_search("\"tix\"")],
+            ten_fixed_tax_tix.ids,
+        )
+        self.assertListEqual(
+            [x[0] for x in self.env["account.tax"].name_search("Ten \"tix\"")],
+            ten_fixed_tax_tix.ids,
+        )
