@@ -5,15 +5,11 @@
 from . import account_automatic_entry_wizard
 from . import account_unreconcile
 from . import account_validate_account_move
-from . import pos_box
 from . import account_move_reversal
-from . import account_report_common
-from . import account_report_common_journal
-from . import account_report_print_journal
 from . import account_resequence
 from . import setup_wizards
-from . import wizard_tax_adjustments
 from . import account_invoice_send
 from . import base_document_layout
 from . import account_payment_register
 from . import account_tour_upload_bill
+from . import accrued_orders
