@@ -5,7 +5,7 @@
     'name': 'Check Printing Base',
     'version': '1.0',
     'category': 'Accounting/Accounting',
-    'summary': 'Check printing commons',
+    'summary': 'Check printing basic features',
     'description': """
 This module offers the basic functionalities to make payments by printing checks.
 It must be used as a dependency for modules that provide country-specific check templates.
@@ -23,5 +23,6 @@ The check settings are located in the accounting journals configuration page.
         'wizard/print_prenumbered_checks_views.xml'
     ],
     'installable': True,
-    'auto_install': False,
+    'post_init_hook': 'create_check_sequence_on_bank_journals',
+    'license': 'LGPL-3',
 }
