@@ -15,7 +15,6 @@ governements, etc.)
     'depends' : ['account'],
     'data': [
         'security/ir.model.access.csv',
-        'wizard/account_invoice_send_views.xml',
         'views/account_edi_document_views.xml',
         'views/account_move_views.xml',
         'views/account_payment_views.xml',
@@ -23,6 +22,6 @@ governements, etc.)
         'data/cron.xml'
     ],
     'installable': True,
-    'application': False,
     'auto_install': True,
+    'license': 'LGPL-3',
 }
