@@ -17,11 +17,10 @@ Odoo database.
     },
     'data': [
         'security/ir.model.access.csv',
-    ],
-    'demo': [
-        'data/config_demo.xml',
+        'security/account_edi_proxy_client_security.xml',
+        'views/account_edi_proxy_user_views.xml',
     ],
     'installable': True,
-    'application': False,
     'license': 'LGPL-3',
+    'post_init_hook': '_create_demo_config_param',
 }
