@@ -4,21 +4,22 @@
     'name': 'Extended Addresses',
     'summary': 'Add extra fields on addresses',
     'sequence': '19',
+    'version': '1.1',
     'category': 'Hidden',
-    'complexity': 'easy',
     'description': """
 Extended Addresses Management
 =============================
 
-This module holds all extra fields one may need to manage accurately addresses.
+This module provides the ability to choose a city from a list (in specific countries).
 
-For example, in legal reports, some countries need to split the street into several fields,
-with the street name, the house number, and room number.
+It is primarily used for EDIs that might need a special city code.
         """,
     'data': [
+        'security/ir.model.access.csv',
         'views/base_address_extended.xml',
-        'data/base_address_extended_data.xml',
+        'views/res_city_view.xml',
+        'views/res_country_view.xml',
     ],
     'depends': ['base'],
-    'post_init_hook': '_update_street_format',
+    'license': 'LGPL-3',
 }
