@@ -19,6 +19,16 @@ trigger an automatic reminder email.
     'data': [
         'security/ir.model.access.csv',
         'data/base_automation_data.xml',
-        'views/base_automation_view.xml',
+        'views/base_automation_views.xml',
     ],
+    'assets': {
+        'web.assets_backend': [
+            'base_automation/static/src/js/**/*',
+            'base_automation/static/src/xml/*.xml',
+        ],
+        'web.qunit_suite_tests': [
+            'base_automation/static/tests/**/*',
+        ],
+    },
+    'license': 'LGPL-3',
 }
