@@ -13,13 +13,21 @@ Lets the user create a custom dashboard.
 
 Allows users to create custom dashboard.
     """,
-    'depends': ['base', 'web'],
+    'depends': ['spreadsheet_dashboard'],
     'data': [
         'security/ir.model.access.csv',
         'views/board_views.xml',
-        'views/board_templates.xml',
-    ],
-    'qweb': ['static/src/xml/board.xml'],
-    'application': True,
+        ],
+    'application': False,
+    'assets': {
+        'web.assets_backend': [
+            'board/static/src/**/*.scss',
+            'board/static/src/**/*.js',
+            'board/static/src/**/*.xml',
+        ],
+        'web.qunit_suite_tests': [
+            'board/static/tests/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
