@@ -30,15 +30,17 @@ RRULE_FREQ_TO_SELECT = {
 }
 
 RRULE_WEEKDAY_TO_FIELD = {
-    rrule.MO.weekday: 'mo',
-    rrule.TU.weekday: 'tu',
-    rrule.WE.weekday: 'we',
-    rrule.TH.weekday: 'th',
-    rrule.FR.weekday: 'fr',
-    rrule.SA.weekday: 'sa',
-    rrule.SU.weekday: 'su',
+    rrule.MO.weekday: 'mon',
+    rrule.TU.weekday: 'tue',
+    rrule.WE.weekday: 'wed',
+    rrule.TH.weekday: 'thu',
+    rrule.FR.weekday: 'fri',
+    rrule.SA.weekday: 'sat',
+    rrule.SU.weekday: 'sun',
 }
 
+RRULE_WEEKDAYS = {'SUN': 'SU', 'MON': 'MO', 'TUE': 'TU', 'WED': 'WE', 'THU': 'TH', 'FRI': 'FR', 'SAT': 'SA'}
+
 RRULE_TYPE_SELECTION = [
     ('daily', 'Days'),
     ('weekly', 'Weeks'),
@@ -58,13 +60,13 @@ MONTH_BY_SELECTION = [
 ]
 
 WEEKDAY_SELECTION = [
-    ('MO', 'Monday'),
-    ('TU', 'Tuesday'),
-    ('WE', 'Wednesday'),
-    ('TH', 'Thursday'),
-    ('FR', 'Friday'),
-    ('SA', 'Saturday'),
-    ('SU', 'Sunday'),
+    ('MON', 'Monday'),
+    ('TUE', 'Tuesday'),
+    ('WED', 'Wednesday'),
+    ('THU', 'Thursday'),
+    ('FRI', 'Friday'),
+    ('SAT', 'Saturday'),
+    ('SUN', 'Sunday'),
 ]
 
 BYDAY_SELECTION = [
@@ -104,13 +106,13 @@ class RecurrenceRule(models.Model):
     end_type = fields.Selection(END_TYPE_SELECTION, default='count')
     interval = fields.Integer(default=1)
     count = fields.Integer(default=1)
-    mo = fields.Boolean()
-    tu = fields.Boolean()
-    we = fields.Boolean()
-    th = fields.Boolean()
-    fr = fields.Boolean()
-    sa = fields.Boolean()
-    su = fields.Boolean()
+    mon = fields.Boolean()
+    tue = fields.Boolean()
+    wed = fields.Boolean()
+    thu = fields.Boolean()
+    fri = fields.Boolean()
+    sat = fields.Boolean()
+    sun = fields.Boolean()
     month_by = fields.Selection(MONTH_BY_SELECTION, default='date')
     day = fields.Integer(default=1)
     weekday = fields.Selection(WEEKDAY_SELECTION, string='Weekday')
@@ -118,14 +120,20 @@ class RecurrenceRule(models.Model):
     until = fields.Date('Repeat Until')
 
     _sql_constraints = [
-        ('month_day', "CHECK (rrule_type != 'monthly' OR month_by != 'day' OR day >= 1 AND day <= 31)", "The day must be between 1 and 31"),
+        ('month_day',
+         "CHECK (rrule_type != 'monthly' "
+                "OR month_by != 'day' "
+                "OR day >= 1 AND day <= 31 "
+                "OR weekday in %s AND byday in %s)"
+                % (tuple(wd[0] for wd in WEEKDAY_SELECTION), tuple(bd[0] for bd in BYDAY_SELECTION)),
+         "The day must be between 1 and 31"),
     ]
 
     @api.depends('rrule')
     def _compute_name(self):
         for recurrence in self:
             period = dict(RRULE_TYPE_SELECTION)[recurrence.rrule_type]
-            every = _("Every %(count)s %(period)s, ", count=recurrence.interval, period=period)
+            every = _("Every %(count)s %(period)s", count=recurrence.interval, period=period)
 
             if recurrence.end_type == 'count':
                 end = _("for %s events", recurrence.count)
@@ -134,19 +142,25 @@ class RecurrenceRule(models.Model):
             else:
                 end = ''
 
-            if recurrence.rrule_type == 'weeky':
+            if recurrence.rrule_type == 'weekly':
                 weekdays = recurrence._get_week_days()
-                weekday_fields = (self._fields[weekday_to_field(w)] for w in weekdays)
-                on = _("on %s,") % ", ".join([field.string for field in weekday_fields])
+                # Convert Weekday object
+                weekdays = [str(w) for w in weekdays]
+                # We need to get the day full name from its three first letters.
+                week_map = {v: k for k, v in RRULE_WEEKDAYS.items()}
+                weekday_short = [week_map[w] for w in weekdays]
+                day_strings = [d[1] for d in WEEKDAY_SELECTION if d[0] in weekday_short]
+                on = _("on %s") % ", ".join([day_name for day_name in day_strings])
             elif recurrence.rrule_type == 'monthly':
                 if recurrence.month_by == 'day':
-                    weekday_label = dict(BYDAY_SELECTION)[recurrence.byday]
-                    on = _("on the %(position)s %(weekday)s, ", position=recurrence.byday, weekday=weekday_label)
+                    position_label = dict(BYDAY_SELECTION)[recurrence.byday]
+                    weekday_label = dict(WEEKDAY_SELECTION)[recurrence.weekday]
+                    on = _("on the %(position)s %(weekday)s", position=position_label, weekday=weekday_label)
                 else:
-                    on = _("day %s, ", recurrence.day)
+                    on = _("day %s", recurrence.day)
             else:
                 on = ''
-            recurrence.name = every + on + end
+            recurrence.name = ' '.join(filter(lambda s: s, [every, on, end]))
 
     @api.depends('calendar_event_ids.start')
     def _compute_dtstart(self):
@@ -160,7 +174,7 @@ class RecurrenceRule(models.Model):
 
     @api.depends(
         'byday', 'until', 'rrule_type', 'month_by', 'interval', 'count', 'end_type',
-        'mo', 'tu', 'we', 'th', 'fr', 'sa', 'su', 'day', 'weekday')
+        'mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun', 'day', 'weekday')
     def _compute_rrule(self):
         for recurrence in self:
             recurrence.rrule = recurrence._rrule_serialize()
@@ -185,7 +199,14 @@ class RecurrenceRule(models.Model):
         ranges_to_create = (event_range for event_range in ranges if event_range not in existing_ranges)
         return synced_events, ranges_to_create
 
-    def _apply_recurrence(self, specific_values_creation=None, no_send_edit=False):
+    def _select_new_base_event(self):
+        """
+        when the base event is no more available (archived, deleted, etc.), a new one should be selected
+        """
+        for recurrence in self:
+            recurrence.base_event_id = recurrence._get_first_event()
+
+    def _apply_recurrence(self, specific_values_creation=None, no_send_edit=False, generic_values_creation=None):
         """Create missing events in the recurrence and detach events which no longer
         follow the recurrence rules.
         :return: detached events
@@ -202,7 +223,7 @@ class RecurrenceRule(models.Model):
             if specific_values_creation:
                 ranges = set([(x[1], x[2]) for x in specific_values_creation if x[0] == recurrence.id])
             else:
-                ranges = set(recurrence._get_ranges(event.start, duration))
+                ranges = recurrence._range_calculation(event, duration)
 
             events_to_keep, ranges = recurrence._reconcile_events(ranges)
             keep |= events_to_keep
@@ -212,6 +233,8 @@ class RecurrenceRule(models.Model):
                 value = dict(base_values, start=start, stop=stop, recurrence_id=recurrence.id, follow_recurrence=True)
                 if (recurrence.id, start, stop) in specific_values_creation:
                     value.update(specific_values_creation[(recurrence.id, start, stop)])
+                if generic_values_creation and recurrence.id in generic_values_creation:
+                    value.update(generic_values_creation[recurrence.id])
                 values += [value]
             event_vals += values
 
@@ -302,9 +325,9 @@ class RecurrenceRule(models.Model):
     def _rrule_parse(self, rule_str, date_start):
         # LUL TODO clean this mess
         data = {}
-        day_list = ['mo', 'tu', 'we', 'th', 'fr', 'sa', 'su']
+        day_list = ['mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun']
 
-        if 'Z' in rule_str and not date_start.tzinfo:
+        if 'Z' in rule_str and date_start and not date_start.tzinfo:
             date_start = pytz.utc.localize(date_start)
         rule = rrule.rrulestr(rule_str, dtstart=date_start)
 
@@ -345,11 +368,14 @@ class RecurrenceRule(models.Model):
             data['end_type'] = 'forever'
         return data
 
+    def _get_lang_week_start(self):
+        lang = self.env['res.lang']._lang_get(self.env.user.lang)
+        week_start = int(lang.week_start)  # lang.week_start ranges from '1' to '7'
+        return rrule.weekday(week_start - 1) # rrule expects an int from 0 to 6
+
     def _get_start_of_period(self, dt):
         if self.rrule_type == 'weekly':
-            lang = self.env['res.lang']._lang_get(self.env.user.lang)
-            week_start = int(lang.week_start)  # lang.week_start ranges from '1' to '7'
-            week_start = rrule.weekday(week_start - 1)  # expects an int from 0 to 6
+            week_start = self._get_lang_week_start()
             start = dt + relativedelta(weekday=week_start(-1))
         elif self.rrule_type == 'monthly':
             start = dt + relativedelta(day=1)
@@ -374,6 +400,28 @@ class RecurrenceRule(models.Model):
                 synced_events |= recurrence.calendar_event_ids.filtered(lambda e: e.start in starts)
         return self.calendar_event_ids - synced_events
 
+    def _range_calculation(self, event, duration):
+        """ Calculate the range of recurrence when applying the recurrence
+        The following issues are taken into account:
+            start of period is sometimes in the past (weekly or monthly rule).
+            We can easily filter these range values but then the count value may be wrong...
+            In that case, we just increase the count value, recompute the ranges and dismiss the useless values
+        """
+        self.ensure_one()
+        original_count = self.end_type == 'count' and self.count
+        ranges = set(self._get_ranges(event.start, duration))
+        future_events = set((x, y) for x, y in ranges if x.date() >= event.start.date() and y.date() >= event.start.date())
+        if original_count and len(future_events) < original_count:
+            # Rise count number because some past values will be dismissed.
+            self.count = (2*original_count) - len(future_events)
+            ranges = set(self._get_ranges(event.start, duration))
+            # We set back the occurrence number to its original value
+            self.count = original_count
+        # Remove ranges of events occurring in the past
+        ranges = set((x, y) for x, y in ranges if x.date() >= event.start.date() and y.date() >= event.start.date())
+        return ranges
+
+
     def _get_ranges(self, start, event_duration):
         starts = self._get_occurrences(start)
         return ((start, start + event_duration) for start in starts)
@@ -429,13 +477,13 @@ class RecurrenceRule(models.Model):
         return tuple(
             rrule.weekday(weekday_index)
             for weekday_index, weekday in {
-                rrule.MO.weekday: self.mo,
-                rrule.TU.weekday: self.tu,
-                rrule.WE.weekday: self.we,
-                rrule.TH.weekday: self.th,
-                rrule.FR.weekday: self.fr,
-                rrule.SA.weekday: self.sa,
-                rrule.SU.weekday: self.su,
+                rrule.MO.weekday: self.mon,
+                rrule.TU.weekday: self.tue,
+                rrule.WE.weekday: self.wed,
+                rrule.TH.weekday: self.thu,
+                rrule.FR.weekday: self.fri,
+                rrule.SA.weekday: self.sat,
+                rrule.SU.weekday: self.sun,
             }.items() if weekday
         )
 
@@ -455,12 +503,13 @@ class RecurrenceRule(models.Model):
         if freq == 'monthly' and self.month_by == 'date':  # e.g. every 15th of the month
             rrule_params['bymonthday'] = self.day
         elif freq == 'monthly' and self.month_by == 'day':  # e.g. every 2nd Monday in the month
-            rrule_params['byweekday'] = getattr(rrule, self.weekday)(int(self.byday))  # e.g. MO(+2) for the second Monday of the month
+            rrule_params['byweekday'] = getattr(rrule, RRULE_WEEKDAYS[self.weekday])(int(self.byday))  # e.g. MO(+2) for the second Monday of the month
         elif freq == 'weekly':
             weekdays = self._get_week_days()
             if not weekdays:
                 raise UserError(_("You have to choose at least one day in the week"))
             rrule_params['byweekday'] = weekdays
+            rrule_params['wkst'] = self._get_lang_week_start()
 
         if self.end_type == 'count':  # e.g. stop after X occurence
             rrule_params['count'] = min(self.count, MAX_RECURRENT_EVENT)
