@@ -3,8 +3,6 @@
 
 from . import controllers
 from . import models
+from . import populate
 from . import report
 from . import wizard
-
-from odoo import api, SUPERUSER_ID
-
