@@ -13,8 +13,12 @@ class LostReason(models.Model):
     leads_count = fields.Integer('Leads Count', compute='_compute_leads_count')
 
     def _compute_leads_count(self):
-        lead_data = self.env['crm.lead'].with_context(active_test=False).read_group([('lost_reason', 'in', self.ids)], ['lost_reason'], ['lost_reason'])
-        mapped_data = dict((data['lost_reason'][0], data['lost_reason_count']) for data in lead_data)
+        lead_data = self.env['crm.lead'].with_context(active_test=False)._read_group(
+            [('lost_reason_id', 'in', self.ids)],
+            ['lost_reason_id'],
+            ['lost_reason_id']
+        )
+        mapped_data = dict((data['lost_reason_id'][0], data['lost_reason_id_count']) for data in lead_data)
         for reason in self:
             reason.leads_count = mapped_data.get(reason.id, 0)
 
@@ -22,7 +26,7 @@ class LostReason(models.Model):
         return {
             'name': _('Leads'),
             'view_mode': 'tree,form',
-            'domain': [('lost_reason', 'in', self.ids)],
+            'domain': [('lost_reason_id', 'in', self.ids)],
             'res_model': 'crm.lead',
             'type': 'ir.actions.act_window',
             'context': {'create': False, 'active_test': False},
