@@ -2,6 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import api, models
+from odoo.addons.base.models.ir_model import MODULE_UNINSTALL_FLAG
 
 
 class IrConfigParameter(models.Model):
@@ -10,7 +11,7 @@ class IrConfigParameter(models.Model):
     def write(self, vals):
         result = super(IrConfigParameter, self).write(vals)
         if any(record.key == "crm.pls_fields" for record in self):
-            self.flush()
+            self.env.flush_all()
             self.env.registry.setup_models(self.env.cr)
         return result
 
@@ -18,14 +19,14 @@ class IrConfigParameter(models.Model):
     def create(self, vals_list):
         records = super(IrConfigParameter, self).create(vals_list)
         if any(record.key == "crm.pls_fields" for record in records):
-            self.flush()
+            self.env.flush_all()
             self.env.registry.setup_models(self.env.cr)
         return records
 
     def unlink(self):
         pls_emptied = any(record.key == "crm.pls_fields" for record in self)
         result = super(IrConfigParameter, self).unlink()
-        if pls_emptied:
-            self.flush()
+        if pls_emptied and not self._context.get(MODULE_UNINSTALL_FLAG):
+            self.env.flush_all()
             self.env.registry.setup_models(self.env.cr)
-        return pls_emptied
+        return result
