@@ -3,9 +3,10 @@
 
 from . import delivery_carrier
 from . import delivery_grid
-from . import product_packaging
+from . import delivery_zip_prefix
 from . import product_template
 from . import sale_order
 from . import partner
-from . import stock_picking
 from . import stock_move
+from . import stock_picking
+from . import stock_package_type
