@@ -4,10 +4,13 @@
 import logging
 import pytz
 
-from odoo import _, api, fields, models
+from datetime import timedelta
+
+from odoo import _, api, Command, fields, models
 from odoo.addons.base.models.res_partner import _tz_get
-from odoo.tools import format_datetime
-from odoo.exceptions import ValidationError
+from odoo.tools import format_datetime, is_html_empty
+from odoo.exceptions import UserError, ValidationError
+from odoo.tools.misc import formatLang
 from odoo.tools.translate import html_translate
 
 _logger = logging.getLogger(__name__)
@@ -24,68 +27,57 @@ class EventType(models.Model):
     _description = 'Event Template'
     _order = 'sequence, id'
 
+    def _default_event_mail_type_ids(self):
+        return [(0, 0,
+                 {'notification_type': 'mail',
+                  'interval_nbr': 0,
+                  'interval_unit': 'now',
+                  'interval_type': 'after_sub',
+                  'template_ref': 'mail.template, %i' % self.env.ref('event.event_subscription').id,
+                 }),
+                (0, 0,
+                 {'notification_type': 'mail',
+                  'interval_nbr': 1,
+                  'interval_unit': 'hours',
+                  'interval_type': 'before_event',
+                  'template_ref': 'mail.template, %i' % self.env.ref('event.event_reminder').id,
+                 }),
+                (0, 0,
+                 {'notification_type': 'mail',
+                  'interval_nbr': 3,
+                  'interval_unit': 'days',
+                  'interval_type': 'before_event',
+                  'template_ref': 'mail.template, %i' % self.env.ref('event.event_reminder').id,
+                 })]
+
     name = fields.Char('Event Template', required=True, translate=True)
+    note = fields.Html(string='Note')
     sequence = fields.Integer()
     # tickets
-    use_ticket = fields.Boolean('Ticketing')
-    event_type_ticket_ids = fields.One2many(
-        'event.type.ticket', 'event_type_id',
-        string='Tickets', compute='_compute_event_type_ticket_ids',
-        readonly=False, store=True)
+    event_type_ticket_ids = fields.One2many('event.type.ticket', 'event_type_id', string='Tickets')
     tag_ids = fields.Many2many('event.tag', string="Tags")
     # registration
     has_seats_limitation = fields.Boolean('Limited Seats')
     seats_max = fields.Integer(
-        'Maximum Registrations', compute='_compute_default_registration',
+        'Maximum Registrations', compute='_compute_seats_max',
         readonly=False, store=True,
         help="It will select this default maximum value when you choose this event")
     auto_confirm = fields.Boolean(
         'Automatically Confirm Registrations', default=True,
         help="Events and registrations will automatically be confirmed "
              "upon creation, easing the flow for simple events.")
-    # location
-    use_timezone = fields.Boolean('Use Default Timezone')
     default_timezone = fields.Selection(
         _tz_get, string='Timezone', default=lambda self: self.env.user.tz or 'UTC')
     # communication
-    use_mail_schedule = fields.Boolean(
-        'Automatically Send Emails', default=True)
     event_type_mail_ids = fields.One2many(
-        'event.type.mail', 'event_type_id',
-        string='Mail Schedule', compute='_compute_event_type_mail_ids',
-        readonly=False, store=True)
-
-    @api.depends('use_mail_schedule')
-    def _compute_event_type_mail_ids(self):
-        for template in self:
-            if not template.use_mail_schedule:
-                template.event_type_mail_ids = [(5, 0)]
-            elif not template.event_type_mail_ids:
-                template.event_type_mail_ids = [(0, 0, {
-                    'notification_type': 'mail',
-                    'interval_unit': 'now',
-                    'interval_type': 'after_sub',
-                    'template_id': self.env.ref('event.event_subscription').id,
-                }), (0, 0, {
-                    'notification_type': 'mail',
-                    'interval_nbr': 10,
-                    'interval_unit': 'days',
-                    'interval_type': 'before_event',
-                    'template_id': self.env.ref('event.event_reminder').id,
-                })]
-
-    @api.depends('use_ticket')
-    def _compute_event_type_ticket_ids(self):
-        for template in self:
-            if not template.use_ticket:
-                template.event_type_ticket_ids = [(5, 0)]
-            elif not template.event_type_ticket_ids:
-                template.event_type_ticket_ids = [(0, 0, {
-                    'name': _('Registration'),
-                })]
+        'event.type.mail', 'event_type_id', string='Mail Schedule',
+        default=_default_event_mail_type_ids)
+    # ticket reports
+    ticket_instructions = fields.Html('Ticket Instructions', translate=True,
+        help="This information will be printed on your tickets.")
 
     @api.depends('has_seats_limitation')
-    def _compute_default_registration(self):
+    def _compute_seats_max(self):
         for template in self:
             if not template.has_seats_limitation:
                 template.seats_max = 0
@@ -98,15 +90,30 @@ class EventEvent(models.Model):
     _inherit = ['mail.thread', 'mail.activity.mixin']
     _order = 'date_begin'
 
+    @api.model
+    def default_get(self, fields_list):
+        result = super().default_get(fields_list)
+        if 'date_begin' in fields_list and 'date_begin' not in result:
+            now = fields.Datetime.now()
+            # Round the datetime to the nearest half hour (e.g. 08:17 => 08:30 and 08:37 => 09:00)
+            result['date_begin'] = now.replace(second=0, microsecond=0) + timedelta(minutes=-now.minute % 30)
+        if 'date_end' in fields_list and 'date_end' not in result and result.get('date_begin'):
+            result['date_end'] = result['date_begin'] + timedelta(days=1)
+        return result
+
     def _get_default_stage_id(self):
-        event_stages = self.env['event.stage'].search([])
-        return event_stages[0] if event_stages else False
+        return self.env['event.stage'].search([], limit=1)
 
     def _default_description(self):
-        return self.env['ir.ui.view']._render_template('event.event_default_descripton')
+        # avoid template branding with rendering_bundle=True
+        return self.env['ir.ui.view'].with_context(rendering_bundle=True) \
+            ._render_template('event.event_default_descripton')
+
+    def _default_event_mail_ids(self):
+        return self.env['event.type']._default_event_mail_type_ids()
 
     name = fields.Char(string='Event', translate=True, required=True)
-    note = fields.Text(string='Note')
+    note = fields.Html(string='Note', store=True, compute="_compute_note", readonly=False)
     description = fields.Html(string='Description', translate=html_translate, sanitize_attributes=False, sanitize_form=False, default=_default_description)
     active = fields.Boolean(default=True)
     user_id = fields.Many2one(
@@ -128,38 +135,38 @@ class EventEvent(models.Model):
         'event.tag', string="Tags", readonly=False,
         store=True, compute="_compute_tag_ids")
     # Kanban fields
-    kanban_state = fields.Selection([('normal', 'In Progress'), ('done', 'Done'), ('blocked', 'Blocked')], default='normal')
+    kanban_state = fields.Selection([('normal', 'In Progress'), ('done', 'Done'), ('blocked', 'Blocked')], default='normal', copy=False)
     kanban_state_label = fields.Char(
         string='Kanban State Label', compute='_compute_kanban_state_label',
         store=True, tracking=True)
     stage_id = fields.Many2one(
         'event.stage', ondelete='restrict', default=_get_default_stage_id,
-        group_expand='_read_group_stage_ids', tracking=True)
+        group_expand='_read_group_stage_ids', tracking=True, copy=False)
     legend_blocked = fields.Char(related='stage_id.legend_blocked', string='Kanban Blocked Explanation', readonly=True)
     legend_done = fields.Char(related='stage_id.legend_done', string='Kanban Valid Explanation', readonly=True)
     legend_normal = fields.Char(related='stage_id.legend_normal', string='Kanban Ongoing Explanation', readonly=True)
     # Seats and computation
     seats_max = fields.Integer(
-        string='Maximum Attendees Number',
+        string='Maximum Attendees',
         compute='_compute_seats_max', readonly=False, store=True,
         help="For each event you can define a maximum registration of seats(number of attendees), above this numbers the registrations are not accepted.")
-    seats_limited = fields.Boolean('Maximum Attendees', required=True, compute='_compute_seats_limited',
-                                   readonly=False, store=True)
+    seats_limited = fields.Boolean('Limit Attendees', required=True, compute='_compute_seats_limited',
+                                   precompute=True, readonly=False, store=True)
     seats_reserved = fields.Integer(
-        string='Reserved Seats',
-        store=True, readonly=True, compute='_compute_seats')
+        string='Number of Registrations',
+        store=False, readonly=True, compute='_compute_seats')
     seats_available = fields.Integer(
         string='Available Seats',
-        store=True, readonly=True, compute='_compute_seats')
+        store=False, readonly=True, compute='_compute_seats')
     seats_unconfirmed = fields.Integer(
-        string='Unconfirmed Seat Reservations',
-        store=True, readonly=True, compute='_compute_seats')
+        string='Unconfirmed Registrations',
+        store=False, readonly=True, compute='_compute_seats')
     seats_used = fields.Integer(
-        string='Number of Participants',
-        store=True, readonly=True, compute='_compute_seats')
+        string='Number of Attendees',
+        store=False, readonly=True, compute='_compute_seats')
     seats_expected = fields.Integer(
         string='Number of Expected Attendees',
-        compute_sudo=True, readonly=True, compute='_compute_seats_expected')
+        store=False, readonly=True, compute='_compute_seats')
     # Registration fields
     auto_confirm = fields.Boolean(
         string='Autoconfirmation', compute='_compute_auto_confirm', readonly=False, store=True,
@@ -168,6 +175,10 @@ class EventEvent(models.Model):
     event_ticket_ids = fields.One2many(
         'event.event.ticket', 'event_id', string='Event Ticket', copy=True,
         compute='_compute_event_ticket_ids', readonly=False, store=True)
+    event_registrations_started = fields.Boolean(
+        'Registrations started', compute='_compute_event_registrations_started',
+        help="registrations have started if the current datetime is after the earliest starting date of tickets."
+    )
     event_registrations_open = fields.Boolean(
         'Registration open', compute='_compute_event_registrations_open', compute_sudo=True,
         help="Registrations are open if:\n"
@@ -177,31 +188,36 @@ class EventEvent(models.Model):
     event_registrations_sold_out = fields.Boolean(
         'Sold Out', compute='_compute_event_registrations_sold_out', compute_sudo=True,
         help='The event is sold out if no more seats are available on event. If ticketing is used and all tickets are sold out, the event will be sold out.')
-    start_sale_date = fields.Date(
+    start_sale_datetime = fields.Datetime(
         'Start sale date', compute='_compute_start_sale_date',
         help='If ticketing is used, contains the earliest starting sale date of tickets.')
+
     # Date fields
     date_tz = fields.Selection(
         _tz_get, string='Timezone', required=True,
-        compute='_compute_date_tz', readonly=False, store=True)
+        compute='_compute_date_tz', precompute=True, readonly=False, store=True)
     date_begin = fields.Datetime(string='Start Date', required=True, tracking=True)
     date_end = fields.Datetime(string='End Date', required=True, tracking=True)
     date_begin_located = fields.Char(string='Start Date Located', compute='_compute_date_begin_tz')
     date_end_located = fields.Char(string='End Date Located', compute='_compute_date_end_tz')
     is_ongoing = fields.Boolean('Is Ongoing', compute='_compute_is_ongoing', search='_search_is_ongoing')
     is_one_day = fields.Boolean(compute='_compute_field_is_one_day')
+    is_finished = fields.Boolean(compute='_compute_is_finished', search='_search_is_finished')
     # Location and communication
     address_id = fields.Many2one(
         'res.partner', string='Venue', default=lambda self: self.env.company.partner_id.id,
         tracking=True, domain="['|', ('company_id', '=', False), ('company_id', '=', company_id)]")
+    address_search = fields.Many2one(
+        'res.partner', string='Address', compute='_compute_address_search', search='_search_address_search')
+    address_inline = fields.Char(
+        string='Venue (formatted for one line uses)', compute='_compute_address_inline',
+        compute_sudo=True)
     country_id = fields.Many2one(
         'res.country', 'Country', related='address_id.country_id', readonly=False, store=True)
-    # badge fields
-    badge_front = fields.Html(string='Badge Front')
-    badge_back = fields.Html(string='Badge Back')
-    badge_innerleft = fields.Html(string='Badge Inner Left')
-    badge_innerright = fields.Html(string='Badge Inner Right')
-    event_logo = fields.Html(string='Event Logo')
+    # ticket reports
+    ticket_instructions = fields.Html('Ticket Instructions', translate=True,
+        compute='_compute_ticket_instructions', store=True, readonly=False,
+        help="This information will be printed on your tickets.")
 
     @api.depends('stage_id', 'kanban_state')
     def _compute_kanban_state_label(self):
@@ -213,7 +229,7 @@ class EventEvent(models.Model):
             else:
                 event.kanban_state_label = event.stage_id.legend_done
 
-    @api.depends('seats_max', 'registration_ids.state')
+    @api.depends('seats_max', 'registration_ids.state', 'registration_ids.active')
     def _compute_seats(self):
         """ Determine reserved, available, reserved but unconfirmed and used seats. """
         # initialize fields to 0
@@ -230,32 +246,41 @@ class EventEvent(models.Model):
         if self.ids:
             query = """ SELECT event_id, state, count(event_id)
                         FROM event_registration
-                        WHERE event_id IN %s AND state IN ('draft', 'open', 'done')
+                        WHERE event_id IN %s AND state IN ('draft', 'open', 'done') AND active = true
                         GROUP BY event_id, state
                     """
-            self.env['event.registration'].flush(['event_id', 'state'])
+            self.env['event.registration'].flush_model(['event_id', 'state', 'active'])
             self._cr.execute(query, (tuple(self.ids),))
             res = self._cr.fetchall()
             for event_id, state, num in res:
                 results[event_id][state_field[state]] = num
 
-        # compute seats_available
+        # compute seats_available and expected
         for event in self:
             event.update(results.get(event._origin.id or event.id, base_vals))
             if event.seats_max > 0:
                 event.seats_available = event.seats_max - (event.seats_reserved + event.seats_used)
 
-    @api.depends('seats_unconfirmed', 'seats_reserved', 'seats_used')
-    def _compute_seats_expected(self):
-        for event in self:
             event.seats_expected = event.seats_unconfirmed + event.seats_reserved + event.seats_used
 
-    @api.depends('date_tz', 'start_sale_date', 'date_end', 'seats_available', 'seats_limited', 'event_ticket_ids.sale_available')
+    @api.depends('date_tz', 'start_sale_datetime')
+    def _compute_event_registrations_started(self):
+        for event in self:
+            event = event._set_tz_context()
+            if event.start_sale_datetime:
+                current_datetime = fields.Datetime.context_timestamp(event, fields.Datetime.now())
+                start_sale_datetime = fields.Datetime.context_timestamp(event, event.start_sale_datetime)
+                event.event_registrations_started = (current_datetime >= start_sale_datetime)
+            else:
+                event.event_registrations_started = True
+
+    @api.depends('date_tz', 'event_registrations_started', 'date_end', 'seats_available', 'seats_limited', 'seats_max',
+                 'event_ticket_ids.sale_available')
     def _compute_event_registrations_open(self):
         """ Compute whether people may take registrations for this event
 
           * event.date_end -> if event is done, registrations are not open anymore;
-          * event.start_sale_date -> lowest start date of tickets (if any; start_sale_date
+          * event.start_sale_datetime -> lowest start date of tickets (if any; start_sale_datetime
             is False if no ticket are defined, see _compute_start_sale_date);
           * any ticket is available for sale (seats available) if any;
           * seats are unlimited or seats are available;
@@ -264,30 +289,32 @@ class EventEvent(models.Model):
             event = event._set_tz_context()
             current_datetime = fields.Datetime.context_timestamp(event, fields.Datetime.now())
             date_end_tz = event.date_end.astimezone(pytz.timezone(event.date_tz or 'UTC')) if event.date_end else False
-            event.event_registrations_open = (event.start_sale_date <= current_datetime.date() if event.start_sale_date else True) and \
+            event.event_registrations_open = event.event_registrations_started and \
                 (date_end_tz >= current_datetime if date_end_tz else True) and \
-                (not event.seats_limited or event.seats_available) and \
+                (not event.seats_limited or not event.seats_max or event.seats_available) and \
                 (not event.event_ticket_ids or any(ticket.sale_available for ticket in event.event_ticket_ids))
 
-    @api.depends('event_ticket_ids.start_sale_date')
+    @api.depends('event_ticket_ids.start_sale_datetime')
     def _compute_start_sale_date(self):
         """ Compute the start sale date of an event. Currently lowest starting sale
         date of tickets if they are used, of False. """
         for event in self:
-            start_dates = [ticket.start_sale_date for ticket in event.event_ticket_ids if not ticket.is_expired]
-            event.start_sale_date = min(start_dates) if start_dates and all(start_dates) else False
+            start_dates = [ticket.start_sale_datetime for ticket in event.event_ticket_ids if not ticket.is_expired]
+            event.start_sale_datetime = min(start_dates) if start_dates and all(start_dates) else False
 
-    @api.depends('event_ticket_ids.sale_available')
+    @api.depends('event_ticket_ids.sale_available', 'seats_available', 'seats_limited')
     def _compute_event_registrations_sold_out(self):
+        """Note that max seats limits for events and sum of limits for all its tickets may not be
+        equal to enable flexibility.
+        E.g. max 20 seats for ticket A, 20 seats for ticket B
+            * With max 20 seats for the event
+            * Without limit set on the event (=40, but the customer didn't explicitly write 40)
+        """
         for event in self:
-            if event.seats_limited and not event.seats_available:
-                event.event_registrations_sold_out = True
-            elif event.event_ticket_ids:
-                event.event_registrations_sold_out = not any(
-                    ticket.seats_available > 0 if ticket.seats_limited else True for ticket in event.event_ticket_ids
-                )
-            else:
-                event.event_registrations_sold_out = False
+            event.event_registrations_sold_out = (
+                (event.seats_limited and event.seats_max and not event.seats_available)
+                or (event.event_ticket_ids and all(ticket.is_sold_out for ticket in event.event_ticket_ids))
+            )
 
     @api.depends('date_tz', 'date_begin')
     def _compute_date_begin_tz(self):
@@ -315,9 +342,9 @@ class EventEvent(models.Model):
 
     def _search_is_ongoing(self, operator, value):
         if operator not in ['=', '!=']:
-            raise ValueError(_('This operator is not supported'))
+            raise UserError(_('This operator is not supported'))
         if not isinstance(value, bool):
-            raise ValueError(_('Value should be True or False (not %s)'), value)
+            raise UserError(_('Value should be True or False (not %s)') % value)
         now = fields.Datetime.now()
         if (operator == '=' and value) or (operator == '!=' and not value):
             domain = [('date_begin', '<=', now), ('date_end', '>', now)]
@@ -336,14 +363,59 @@ class EventEvent(models.Model):
             end_tz = fields.Datetime.context_timestamp(event, event.date_end)
             event.is_one_day = (begin_tz.date() == end_tz.date())
 
+    @api.depends('date_end')
+    def _compute_is_finished(self):
+        for event in self:
+            if not event.date_end:
+                event.is_finished = False
+                continue
+            event = event._set_tz_context()
+            current_datetime = fields.Datetime.context_timestamp(event, fields.Datetime.now())
+            datetime_end = fields.Datetime.context_timestamp(event, event.date_end)
+            event.is_finished = datetime_end <= current_datetime
+
+    def _search_is_finished(self, operator, value):
+        if operator not in ['=', '!=']:
+            raise ValueError(_('This operator is not supported'))
+        if not isinstance(value, bool):
+            raise ValueError(_('Value should be True or False (not %s)'), value)
+        now = fields.Datetime.now()
+        if (operator == '=' and value) or (operator == '!=' and not value):
+            domain = [('date_end', '<=', now)]
+        else:
+            domain = [('date_end', '>', now)]
+        event_ids = self.env['event.event']._search(domain)
+        return [('id', 'in', event_ids)]
+
     @api.depends('event_type_id')
     def _compute_date_tz(self):
         for event in self:
-            if event.event_type_id.use_timezone and event.event_type_id.default_timezone:
+            if event.event_type_id.default_timezone:
                 event.date_tz = event.event_type_id.default_timezone
             if not event.date_tz:
                 event.date_tz = self.env.user.tz or 'UTC'
 
+    @api.depends('address_id')
+    def _compute_address_search(self):
+        for event in self:
+            event.address_search = event.address_id
+
+    def _search_address_search(self, operator, value):
+        if operator != 'ilike' or not isinstance(value, str):
+            raise NotImplementedError(_('Operation not supported.'))
+
+        address_ids = self.env['res.partner']._search([
+            '|', '|', '|', '|', '|',
+            ('street', 'ilike', value),
+            ('street2', 'ilike', value),
+            ('city', 'ilike', value),
+            ('zip', 'ilike', value),
+            ('state_id', 'ilike', value),
+            ('country_id', 'ilike', value),
+        ])
+
+        return [('address_id', 'in', address_ids)]
+
     # seats
 
     @api.depends('event_type_id')
@@ -393,19 +465,22 @@ class EventEvent(models.Model):
         """
         for event in self:
             if not event.event_type_id and not event.event_mail_ids:
-                event.event_mail_ids = False
+                event.event_mail_ids = self._default_event_mail_ids()
                 continue
 
             # lines to keep: those with already sent emails or registrations
-            mails_toremove = event._origin.event_mail_ids.filtered(lambda mail: not mail.mail_sent and not(mail.mail_registration_ids))
-            command = [(3, mail.id) for mail in mails_toremove]
-            if event.event_type_id.use_mail_schedule:
-                command += [
-                    (0, 0, {
-                        attribute_name: line[attribute_name] if not isinstance(line[attribute_name], models.BaseModel) else line[attribute_name].id
-                        for attribute_name in self.env['event.type.mail']._get_event_mail_fields_whitelist()
-                    }) for line in event.event_type_id.event_type_mail_ids
-                ]
+            mails_to_remove = event.event_mail_ids.filtered(
+                lambda mail: not(mail._origin.mail_done) and not(mail._origin.mail_registration_ids)
+            )
+            command = [Command.unlink(mail.id) for mail in mails_to_remove]
+
+            # lines to add: those which do not have the exact copy available in lines to keep
+            if event.event_type_id.event_type_mail_ids:
+                mails_to_keep_vals = {mail._prepare_event_mail_values() for mail in event.event_mail_ids - mails_to_remove}
+                for mail in event.event_type_id.event_type_mail_ids:
+                    mail_values = mail._prepare_event_mail_values()
+                    if mail_values not in mails_to_keep_vals:
+                        command.append(Command.create(mail_values._asdict()))
             if command:
                 event.event_mail_ids = command
 
@@ -432,39 +507,61 @@ class EventEvent(models.Model):
           * type lines are added;
 
         Note that updating event_ticket_ids triggers _compute_start_sale_date
-        (start_sale_date computation) so ensure result to avoid cache miss.
+        (start_sale_datetime computation) so ensure result to avoid cache miss.
         """
-        if self.ids or self._origin.ids:
-            # lines to keep: those with already sent emails or registrations
-            tickets_tokeep_ids = self.env['event.registration'].search(
-                [('event_id', 'in', self.ids or self._origin.ids)]
-            ).event_ticket_id.ids
-        else:
-            tickets_tokeep_ids = []
         for event in self:
             if not event.event_type_id and not event.event_ticket_ids:
                 event.event_ticket_ids = False
                 continue
 
             # lines to keep: those with existing registrations
-            if tickets_tokeep_ids:
-                tickets_toremove = event._origin.event_ticket_ids.filtered(lambda ticket: ticket.id not in tickets_tokeep_ids)
-                command = [(3, ticket.id) for ticket in tickets_toremove]
-            else:
-                command = [(5, 0)]
-            if event.event_type_id.use_ticket:
+            tickets_to_remove = event.event_ticket_ids.filtered(lambda ticket: not ticket._origin.registration_ids)
+            command = [Command.unlink(ticket.id) for ticket in tickets_to_remove]
+            if event.event_type_id.event_type_ticket_ids:
                 command += [
-                    (0, 0, {
+                    Command.create({
                         attribute_name: line[attribute_name] if not isinstance(line[attribute_name], models.BaseModel) else line[attribute_name].id
                         for attribute_name in self.env['event.type.ticket']._get_event_ticket_fields_whitelist()
                     }) for line in event.event_type_id.event_type_ticket_ids
                 ]
             event.event_ticket_ids = command
 
-    @api.constrains('seats_max', 'seats_available', 'seats_limited')
-    def _check_seats_limit(self):
-        if any(event.seats_limited and event.seats_max and event.seats_available < 0 for event in self):
-            raise ValidationError(_('No more available seats.'))
+    @api.depends('event_type_id')
+    def _compute_note(self):
+        for event in self:
+            if event.event_type_id and not is_html_empty(event.event_type_id.note):
+                event.note = event.event_type_id.note
+
+    @api.depends('event_type_id')
+    def _compute_ticket_instructions(self):
+        for event in self:
+            if is_html_empty(event.ticket_instructions) and not \
+               is_html_empty(event.event_type_id.ticket_instructions):
+                event.ticket_instructions = event.event_type_id.ticket_instructions
+
+    @api.depends('address_id')
+    def _compute_address_inline(self):
+        """Use venue address if available, otherwise its name, finally ''. """
+        for event in self:
+            if (event.address_id.contact_address or '').strip():
+                event.address_inline = ', '.join(
+                    frag.strip()
+                    for frag in event.address_id.contact_address.split('\n') if frag.strip()
+                )
+            else:
+                event.address_inline = event.address_id.name or ''
+
+    @api.constrains('seats_max', 'seats_limited', 'registration_ids')
+    def _check_seats_availability(self, minimal_availability=0):
+        sold_out_events = []
+        for event in self:
+            if event.seats_limited and event.seats_max and event.seats_available < minimal_availability:
+                sold_out_events.append(
+                    (_('- "%(event_name)s": Missing %(nb_too_many)i seats.',
+                        event_name=event.name, nb_too_many=-event.seats_available)))
+        if sold_out_events:
+            raise ValidationError(_('There are not enough seats available for:')
+                                  + '\n%s\n' % '\n'.join(sold_out_events))
 
     @api.constrains('date_begin', 'date_end')
     def _check_closing_date(self):
@@ -472,56 +569,66 @@ class EventEvent(models.Model):
             if event.date_end < event.date_begin:
                 raise ValidationError(_('The closing date cannot be earlier than the beginning date.'))
 
-    @api.depends('name', 'date_begin', 'date_end')
-    def name_get(self):
-        result = []
-        for event in self:
-            date_begin = fields.Datetime.from_string(event.date_begin)
-            date_end = fields.Datetime.from_string(event.date_end)
-            dates = [fields.Date.to_string(fields.Datetime.context_timestamp(event, dt)) for dt in [date_begin, date_end] if dt]
-            dates = sorted(set(dates))
-            result.append((event.id, '%s (%s)' % (event.name, ' - '.join(dates))))
-        return result
-
     @api.model
     def _read_group_stage_ids(self, stages, domain, order):
         return self.env['event.stage'].search([])
 
-    @api.model
-    def create(self, vals):
-        # Temporary fix for ``seats_limited`` and ``date_tz`` required fields
-        vals.update(self._sync_required_computed(vals))
-
-        res = super(EventEvent, self).create(vals)
-        if res.organizer_id:
-            res.message_subscribe([res.organizer_id.id])
-        res.flush()
-        return res
+    @api.model_create_multi
+    def create(self, vals_list):
+        events = super(EventEvent, self).create(vals_list)
+        for res in events:
+            if res.organizer_id:
+                res.message_subscribe([res.organizer_id.id])
+        self.env.flush_all()
+        return events
 
     def write(self, vals):
+        if 'stage_id' in vals and 'kanban_state' not in vals:
+            # reset kanban state when changing stage
+            vals['kanban_state'] = 'normal'
         res = super(EventEvent, self).write(vals)
         if vals.get('organizer_id'):
             self.message_subscribe([vals['organizer_id']])
         return res
 
+    def name_get(self):
+        """Adds ticket seats availability if requested by context."""
+        if not self.env.context.get('name_with_seats_availability'):
+            return super().name_get()
+        res = []
+        for event in self:
+            # event or its tickets are sold out
+            if event.event_registrations_sold_out:
+                name = _('%(event_name)s (Sold out)', event_name=event.name)
+            elif event.seats_limited and event.seats_max:
+                name = _(
+                    '%(event_name)s (%(count)s seats remaining)',
+                    event_name=event.name,
+                    count=formatLang(self.env, event.seats_available, digits=0),
+                )
+            else:
+                name = event.name
+            res.append((event.id, name))
+        return res
+
     @api.returns('self', lambda value: value.id)
     def copy(self, default=None):
         self.ensure_one()
         default = dict(default or {}, name=_("%s (copy)") % (self.name))
         return super(EventEvent, self).copy(default)
 
-    def _sync_required_computed(self, values):
-        # TODO: See if the change to seats_limited affects this ?
-        """ Call compute fields in cache to find missing values for required fields
-        (seats_limited and date_tz) in case they are not given in values """
-        missing_fields = list(set(['seats_limited', 'date_tz']).difference(set(values.keys())))
-        if missing_fields and values:
-            cache_event = self.new(values)
-            cache_event._compute_seats_limited()
-            cache_event._compute_date_tz()
-            return dict((fname, cache_event[fname]) for fname in missing_fields)
-        else:
-            return {}
+    @api.model
+    def _get_mail_message_access(self, res_ids, operation, model_name=None):
+        if (
+            operation == 'create'
+            and self.env.user.has_group('event.group_event_registration_desk')
+            and (not model_name or model_name == 'event.event')
+        ):
+            # allow the registration desk users to post messages on Event
+            # can not be done with "_mail_post_access" otherwise public user will be
+            # able to post on published Event (see website_event)
+            return 'read'
+        return super(EventEvent, self)._get_mail_message_access(res_ids, operation, model_name)
 
     def _set_tz_context(self):
         self.ensure_one()
@@ -533,9 +640,9 @@ class EventEvent(models.Model):
         into the first next (by sequence) stage defined as "Ended"
         (if they are not already in an ended stage)
         """
-        first_ended_stage = self.env['event.stage'].search([('pipe_end', '=', True)], order='sequence')
+        first_ended_stage = self.env['event.stage'].search([('pipe_end', '=', True)], limit=1, order='sequence')
         if first_ended_stage:
-            self.write({'stage_id': first_ended_stage[0].id})
+            self.write({'stage_id': first_ended_stage.id})
 
     def mail_attendees(self, template_id, force_send=False, filter_func=lambda self: self.state != 'cancel'):
         for event in self:
@@ -555,11 +662,11 @@ class EventEvent(models.Model):
             cal_event = cal.add('vevent')
 
             cal_event.add('created').value = fields.Datetime.now().replace(tzinfo=pytz.timezone('UTC'))
-            cal_event.add('dtstart').value = fields.Datetime.from_string(event.date_begin).replace(tzinfo=pytz.timezone('UTC'))
-            cal_event.add('dtend').value = fields.Datetime.from_string(event.date_end).replace(tzinfo=pytz.timezone('UTC'))
+            cal_event.add('dtstart').value = event.date_begin.astimezone(pytz.timezone(event.date_tz))
+            cal_event.add('dtend').value = event.date_end.astimezone(pytz.timezone(event.date_tz))
             cal_event.add('summary').value = event.name
             if event.address_id:
-                cal_event.add('location').value = event.sudo().address_id.contact_address
+                cal_event.add('location').value = event.address_inline
 
             result[event.id] = cal.serialize().encode('utf-8')
         return result
