@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import api, fields, models
 
+
 class ResConfigSettings(models.TransientModel):
     _inherit = 'res.config.settings'
 
@@ -10,10 +12,11 @@ class ResConfigSettings(models.TransientModel):
     module_website_event_track = fields.Boolean("Tracks and Agenda")
     module_website_event_track_live = fields.Boolean("Live Mode")
     module_website_event_track_quiz = fields.Boolean("Quiz on Tracks")
-    module_website_event_track_exhibitor = fields.Boolean("Advanced Sponsors")
+    module_website_event_exhibitor = fields.Boolean("Advanced Sponsors")
     module_website_event_questions = fields.Boolean("Registration Survey")
     module_event_barcode = fields.Boolean("Barcode")
     module_website_event_sale = fields.Boolean("Online Ticketing")
+    module_event_booth = fields.Boolean("Booth Management")
 
     @api.onchange('module_website_event_track')
     def _onchange_module_website_event_track(self):
@@ -24,4 +27,3 @@ class ResConfigSettings(models.TransientModel):
             if not config.module_website_event_track:
                 config.module_website_event_track_live = False
                 config.module_website_event_track_quiz = False
-                config.module_website_event_track_exhibitor = False
