@@ -5,12 +5,13 @@
     'name': 'Event CRM',
     'version': '1.0',
     'category': 'Marketing/Events',
-    'website': 'https://www.odoo.com/page/events',
+    'website': 'https://www.odoo.com/app/events',
     'description': "Create leads from event registrations.",
     'depends': ['event', 'crm'],
     'data': [
         'security/event_crm_security.xml',
         'security/ir.model.access.csv',
+        'data/crm_lead_merge_template.xml',
         'views/crm_lead_views.xml',
         'views/event_registration_views.xml',
         'views/event_lead_rule_views.xml',
