@@ -9,13 +9,9 @@
     'depends': ['event', 'sms'],
     'data': [
         'data/sms_data.xml',
-        'views/event_views.xml',
-        'views/event_mail_views.xml',
         'security/ir.model.access.csv',
         'security/sms_security.xml',
     ],
-    'demo': [
-    ],
     'installable': True,
     'auto_install': True,
     'license': 'LGPL-3',
