@@ -3,6 +3,7 @@
 
 from . import test_hr_employee
 from . import test_channel
-from . import test_hr_flow
 from . import test_self_user_access
 from . import test_multi_company
+from . import test_resource
+from . import test_ui
