@@ -12,19 +12,15 @@ class User(models.Model):
     attendance_state = fields.Selection(related='employee_id.attendance_state')
     last_check_in = fields.Datetime(related='employee_id.last_attendance_id.check_in')
     last_check_out = fields.Datetime(related='employee_id.last_attendance_id.check_out')
+    total_overtime = fields.Float(related='employee_id.total_overtime')
 
-    def __init__(self, pool, cr):
-        """ Override of __init__ to add access rights.
-            Access rights are disabled by default, but allowed
-            on some specific fields defined in self.SELF_{READ/WRITE}ABLE_FIELDS.
-        """
-        attendance_readable_fields = [
+    @property
+    def SELF_READABLE_FIELDS(self):
+        return super().SELF_READABLE_FIELDS + [
             'hours_last_month',
             'hours_last_month_display',
             'attendance_state',
             'last_check_in',
-            'last_check_out'
+            'last_check_out',
+            'total_overtime'
         ]
-        super(User, self).__init__(pool, cr)
-        # duplicate list to avoid modifying the original reference
-        type(self).SELF_READABLE_FIELDS = type(self).SELF_READABLE_FIELDS + attendance_readable_fields
