@@ -8,17 +8,18 @@ from odoo.exceptions import UserError
 class HrDepartureWizard(models.TransientModel):
     _inherit = 'hr.departure.wizard'
 
-    set_date_end = fields.Boolean(string="Set Contract End Date", default=True)
+    set_date_end = fields.Boolean(string="Set Contract End Date", default=lambda self: self.env.user.user_has_groups('hr_contract.group_hr_contract_manager'),
+        help="Set the end date on the current contract.")
 
     def action_register_departure(self):
         """If set_date_end is checked, set the departure date as the end date to current running contract,
         and cancel all draft contracts"""
-        current_contract = self.employee_id.contract_id
+        current_contract = self.sudo().employee_id.contract_id
         if current_contract and current_contract.date_start > self.departure_date:
             raise UserError(_("Departure date can't be earlier than the start date of current contract."))
 
         super(HrDepartureWizard, self).action_register_departure()
         if self.set_date_end:
-            self.employee_id.contract_ids.filtered(lambda c: c.state == 'draft').write({'state': 'cancel'})
+            self.sudo().employee_id.contract_ids.filtered(lambda c: c.state == 'draft').write({'state': 'cancel'})
             if current_contract:
-                self.employee_id.contract_id.write({'date_end': self.departure_date})
+                self.sudo().employee_id.contract_id.write({'date_end': self.departure_date})
