@@ -1,9 +1,13 @@
 # -*- coding: utf-8 -*-
 
 from . import hr_employee
+from . import account_move
 from . import account_move_line
+from . import account_payment
 from . import hr_department
 from . import hr_expense
 from . import product_template
 from . import res_config_settings
 from . import account_journal_dashboard
+from . import res_company
+from . import analytic
