@@ -5,3 +5,6 @@ from . import employee
 from . import res_users
 from . import fleet_vehicle_assignation_log
 from . import fleet_vehicle
+from . import fleet_vehicle_log_contract
+from . import fleet_vehicle_log_services
+from . import fleet_vehicle_odometer
