@@ -5,9 +5,16 @@ from . import test_access_rights
 from . import test_automatic_leave_dates
 from . import test_allocation_access_rights
 from . import test_holidays_flow
+from . import test_hr_holidays_cancel_leave
+from . import test_hr_holidays_tour
 from . import test_hr_leave_type
 from . import test_accrual_allocations
 from . import test_change_department
 from . import test_leave_requests
 from . import test_out_of_office
 from . import test_company_leave
+from . import test_res_partner
+from . import test_stress_days
+from . import test_global_leaves
+from . import test_uninstall
+from . import test_holidays_calendar
