@@ -12,72 +12,74 @@ from odoo.tools import mute_logger
 
 @tests.tagged('access_rights', 'post_install', '-at_install')
 class TestHrHolidaysAccessRightsCommon(TestHrHolidaysCommon):
-    def setUp(self):
-        super(TestHrHolidaysAccessRightsCommon, self).setUp()
-        self.leave_type = self.env['hr.leave.type'].create({
+    @classmethod
+    def setUpClass(cls):
+        super(TestHrHolidaysAccessRightsCommon, cls).setUpClass()
+        cls.leave_type = cls.env['hr.leave.type'].create({
             'name': 'Unlimited',
             'leave_validation_type': 'hr',
-            'allocation_type': 'no',
-            'validity_start': False,
+            'requires_allocation': 'no',
         })
-        self.rd_dept.manager_id = False
-        self.hr_dept.manager_id = False
-        self.employee_emp.parent_id = False
-        self.employee_leave = self.env['hr.leave'].with_user(self.user_employee_id).create({
+        cls.rd_dept.manager_id = False
+        cls.hr_dept.manager_id = False
+        cls.employee_emp.parent_id = False
+        cls.employee_leave = cls.env['hr.leave'].with_user(cls.user_employee_id).create({
             'name': 'Test',
-            'holiday_status_id': self.leave_type.id,
-            'department_id': self.employee_emp.department_id.id,
-            'employee_id': self.employee_emp.id,
-            'date_from': datetime.now(),
-            'date_to': datetime.now() + relativedelta(days=1),
+            'holiday_status_id': cls.leave_type.id,
+            'department_id': cls.employee_emp.department_id.id,
+            'employee_id': cls.employee_emp.id,
+            'date_from': datetime.now() + relativedelta(days=30),
+            'date_to': datetime.now() + relativedelta(days=31),
             'number_of_days': 1,
         })
 
-        self.lt_no_validation = self.env['hr.leave.type'].create({
+        cls.lt_no_validation = cls.env['hr.leave.type'].create({
             'name': 'Validation = no_validation',
             'leave_validation_type': 'hr',
-            'allocation_type': 'no',
-            'validity_start': False,
+            'requires_allocation': 'no',
         })
 
-        self.lt_validation_hr = self.env['hr.leave.type'].create({
+        cls.lt_validation_hr = cls.env['hr.leave.type'].create({
             'name': 'Validation = HR',
             'leave_validation_type': 'hr',
-            'allocation_type': 'no',
-            'validity_start': False,
+            'requires_allocation': 'no',
         })
 
-        self.lt_validation_manager = self.env['hr.leave.type'].create({
+        cls.lt_validation_manager = cls.env['hr.leave.type'].create({
             'name': 'Validation = manager',
             'leave_validation_type': 'hr',
-            'allocation_type': 'no',
-            'validity_start': False,
+            'requires_allocation': 'no',
         })
 
-        self.lt_validation_both = self.env['hr.leave.type'].create({
+        cls.lt_validation_both = cls.env['hr.leave.type'].create({
             'name': 'Validation = both',
             'leave_validation_type': 'hr',
-            'allocation_type': 'no',
-            'validity_start': False,
+            'requires_allocation': 'no',
         })
 
-        self.draft_status = [
-            self.lt_validation_hr,
-            self.lt_validation_manager,
-            self.lt_validation_both
+        cls.draft_status = [
+            cls.lt_validation_hr,
+            cls.lt_validation_manager,
+            cls.lt_validation_both
         ]
 
-        self.confirm_status = [
-            self.lt_no_validation,
-            self.lt_validation_hr,
-            self.lt_validation_manager,
-            self.lt_validation_both
+        cls.confirm_status = [
+            cls.lt_no_validation,
+            cls.lt_validation_hr,
+            cls.lt_validation_manager,
+            cls.lt_validation_both
         ]
 
+        # Here we only test access rights, prevent any conflict with
+        # existing stress days - they are tested someplace else.
+        cls.env['hr.leave.stress.day'].search([]).unlink()
+
     def request_leave(self, user_id, date_from, number_of_days, values=None):
         values = dict(values or {}, **{
             'date_from': date_from,
+            'request_date_from': date_from,
             'date_to': date_from + relativedelta(days=number_of_days),
+            'request_date_to': date_from + relativedelta(days=number_of_days),
             'number_of_days': number_of_days,
         })
         return self.env['hr.leave'].with_user(user_id).create(values)
@@ -501,7 +503,7 @@ class TestAccessRightsRead(TestHrHolidaysAccessRightsCommon):
             'number_of_days': 1,
         })
         with self.assertRaises(AccessError), self.cr.savepoint():
-            other_leave.invalidate_cache(['name'])
+            other_leave.invalidate_model(['name'])
             name = other_leave.with_user(self.user_employee_id).name
 
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
@@ -715,28 +717,79 @@ class TestAccessRightsWrite(TestHrHolidaysAccessRightsCommon):
 
     # TODO Can always cancel with great powers comes great responbilities
 
+class TestAccessRightsUnlink(TestHrHolidaysAccessRightsCommon):
+
+    # base.group_user
+
+    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
+    def test_leave_unlink_draft_by_user(self):
+        """ A simple user may delete its leave in draft state in the future"""
+        values = {
+            'name': 'Random Leave',
+            'employee_id': self.employee_emp.id,
+            'holiday_status_id': self.leave_type.id,
+            'state': 'draft',
+        }
+        leave = self.request_leave(self.user_employee_id, datetime.now() + relativedelta(days=5), 1, values)
+        leave.with_user(self.user_employee.id).unlink()
+
+    def test_leave_unlink_confirm_by_user(self):
+        """ A simple user may delete its leave in confirm state in the future"""
+        values = {
+            'name': 'Random Leave',
+            'employee_id': self.employee_emp.id,
+            'holiday_status_id': self.leave_type.id,
+            'state': 'confirm',
+        }
+        leave = self.request_leave(self.user_employee_id, datetime.now() + relativedelta(days=5), 1, values)
+        leave.with_user(self.user_employee.id).unlink()
+
+    def test_leave_unlink_confirm_in_past_by_user(self):
+        """ A simple user cannot delete its leave in the past"""
+        values = {
+            'name': 'Random Leave',
+            'employee_id': self.employee_emp.id,
+            'holiday_status_id': self.leave_type.id,
+            'state': 'confirm',
+        }
+        leave = self.request_leave(self.user_employee_id, datetime.now() + relativedelta(days=-4), 1, values)
+        with self.assertRaises(UserError), self.cr.savepoint():
+            leave.with_user(self.user_employee.id).unlink()
+
+    def test_leave_unlink_validate_by_user(self):
+        """ A simple user cannot delete its leave in validate state"""
+        values = {
+            'name': 'Random Leave',
+            'employee_id': self.employee_emp.id,
+            'holiday_status_id': self.leave_type.id,
+        }
+        leave = self.request_leave(self.user_employee_id, datetime.now() + relativedelta(days=5), 1, values)
+        leave.with_user(self.user_hrmanager_id).write({'state': 'validate'})
+        with self.assertRaises(UserError), self.cr.savepoint():
+            leave.with_user(self.user_employee.id).unlink()
 
 class TestMultiCompany(TestHrHolidaysCommon):
 
-    def setUp(self):
-        super(TestMultiCompany, self).setUp()
-        self.new_company = self.env['res.company'].create({
+    @classmethod
+    def setUpClass(cls):
+        super(TestMultiCompany, cls).setUpClass()
+        cls.new_company = cls.env['res.company'].create({
             'name': 'Crocodile Dundee Company',
         })
-        self.leave_type = self.env['hr.leave.type'].create({
+        cls.leave_type = cls.env['hr.leave.type'].create({
             'name': 'Unlimited - Company New',
-            'company_id': self.new_company.id,
+            'company_id': cls.new_company.id,
             'leave_validation_type': 'hr',
-            'allocation_type': 'no',
+            'requires_allocation': 'no',
         })
-        self.rd_dept.manager_id = False
-        self.hr_dept.manager_id = False
+        cls.rd_dept.manager_id = False
+        cls.hr_dept.manager_id = False
 
-        self.employee_leave = self.env['hr.leave'].create({
+        cls.employee_leave = cls.env['hr.leave'].create({
             'name': 'Test',
-            'holiday_status_id': self.leave_type.id,
-            'department_id': self.employee_emp.department_id.id,
-            'employee_id': self.employee_emp.id,
+            'holiday_status_id': cls.leave_type.id,
+            'department_id': cls.employee_emp.department_id.id,
+            'employee_id': cls.employee_emp.id,
             'date_from': datetime.now(),
             'date_to': datetime.now() + relativedelta(days=1),
             'number_of_days': 1,
@@ -745,7 +798,7 @@ class TestMultiCompany(TestHrHolidaysCommon):
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_leave_access_other_company_user(self):
         employee_leave = self.employee_leave.with_user(self.user_employee)
-        employee_leave.invalidate_cache(['name'])
+        employee_leave.invalidate_model(['name'])
         with self.assertRaises(AccessError):
             employee_leave.name
 
@@ -755,7 +808,7 @@ class TestMultiCompany(TestHrHolidaysCommon):
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_leave_access_other_company_officer(self):
         employee_leave_hruser = self.employee_leave.with_user(self.user_hruser)
-        employee_leave_hruser.invalidate_cache(['name'])
+        employee_leave_hruser.invalidate_model(['name'])
         with self.assertRaises(AccessError):
             employee_leave_hruser.name
 
@@ -765,7 +818,7 @@ class TestMultiCompany(TestHrHolidaysCommon):
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_leave_access_other_company_manager(self):
         employee_leave_hrmanager = self.employee_leave.with_user(self.user_hrmanager)
-        employee_leave_hrmanager.invalidate_cache(['name'])
+        employee_leave_hrmanager.invalidate_model(['name'])
         with self.assertRaises(AccessError):
             employee_leave_hrmanager.name
 
