@@ -23,12 +23,13 @@ class HrEmployee(models.Model):
         ])
         return [('id', 'in', employees.ids)]
 
-    @api.model
-    def create(self, vals):
-        new_employee = super(HrEmployee, self).create(vals)
-        if new_employee.applicant_id:
-            new_employee.applicant_id.message_post_with_view(
-                        'hr_recruitment.applicant_hired_template',
-                        values={'applicant': new_employee.applicant_id},
-                        subtype_id=self.env.ref("hr_recruitment.mt_applicant_hired").id)
-        return new_employee
+    @api.model_create_multi
+    def create(self, vals_list):
+        employees = super().create(vals_list)
+        for employee in employees:
+            if employee.applicant_id:
+                employee.applicant_id.message_post_with_view(
+                    'hr_recruitment.applicant_hired_template',
+                    values={'applicant': employee.applicant_id},
+                    subtype_id=self.env.ref("hr_recruitment.mt_applicant_hired").id)
+        return employees
