@@ -5,22 +5,24 @@
     'name': 'Skills Certification',
     'category': 'Hidden',
     'version': '1.0',
-    'summary': 'Add certification to resumé of your employees',
+    'summary': 'Add certification to resume of your employees',
     'description':
         """
 Certification and Skills for HR
 ===============================
 
-This module adds certification to resumé for employees.
+This module adds certification to resume for employees.
         """,
     'depends': ['hr_skills', 'survey'],
     'data': [
         'views/hr_templates.xml',
         'data/hr_resume_data.xml',
     ],
-    'qweb': [
-        'static/src/xml/resume_templates.xml',
-    ],
     'auto_install': True,
+    'assets': {
+        'web.assets_backend': [
+            'hr_skills_survey/static/src/xml/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
