@@ -4,10 +4,11 @@
 from . import controllers
 from . import models
 from . import report
-from . import wizard
 
 from odoo import api, fields, SUPERUSER_ID, _
 
+from odoo.addons.project import _check_exists_collaborators_for_project_sharing
+
 
 def create_internal_project(cr, registry):
     env = api.Environment(cr, SUPERUSER_ID, {})
@@ -19,20 +20,7 @@ def create_internal_project(cr, registry):
     admin = env.ref('base.user_admin', raise_if_not_found=False)
     if not admin:
         return
-    project_vals = []
-    for company in env['res.company'].search([]):
-        company = company.with_company(company)
-        project_vals += [{
-            'name': _('Internal'),
-            'allow_timesheets': True,
-            'company_id': company.id,
-            'task_ids': [(0, 0, {
-                'name': name,
-                'company_id': company.id,
-            }) for name in [_('Training'), _('Meeting')]]
-        }]
-    project_ids = env['project.project'].create(project_vals)
-
+    project_ids = env['res.company'].search([])._create_internal_project_task()
     env['account.analytic.line'].create([{
         'name': _("Analysis"),
         'user_id': admin.id,
@@ -41,3 +29,18 @@ def create_internal_project(cr, registry):
         'project_id': task.project_id.id,
         'task_id': task.id,
     } for task in project_ids.task_ids.filtered(lambda t: t.company_id in admin.employee_ids.company_id)])
+
+    _check_exists_collaborators_for_project_sharing(env)
+
+def _uninstall_hook(cr, registry):
+    env = api.Environment(cr, SUPERUSER_ID, {})
+    act_window = env.ref('project.open_view_project_all', raise_if_not_found=False)
+    if act_window and act_window.domain and 'is_internal_project' in act_window.domain:
+        act_window.domain = []
+
+    # archive the internal projects
+    project_ids = env['res.company'].search([('internal_project_id', '!=', False)]).mapped('internal_project_id')
+    if project_ids:
+        project_ids.write({'active': False})
+
+    env['ir.model.data'].search([('name', 'ilike', 'internal_project_default_stage')]).unlink()
