@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import hr_employee
 from . import hr_timesheet
 from . import ir_http
+from . import ir_ui_menu
 from . import res_company
 from . import res_config_settings
 from . import project
+from . import project_collaborator
 from . import uom
