@@ -5,7 +5,7 @@
     'name': 'IoT Box Homepage',
     'category': 'Sales/Point of Sale',
     'sequence': 6,
-    'website': 'https://www.odoo.com/page/point-of-sale-hardware',
+    'website': 'https://www.odoo.com/app/point-of-sale-hardware',
     'summary': 'A homepage for the IoT Box',
     'description': """
 IoT Box Homepage
@@ -20,4 +20,5 @@ regular Odoo interface anymore.
 
 """,
     'installable': False,
+    'license': 'LGPL-3',
 }
