@@ -3,6 +3,7 @@
 
 {
     'name': 'Germany - Accounting',
+    "version": "2.0",
     'author': 'openbig.org',
     'website': 'http://www.openbig.org',
     'category': 'Accounting/Localizations',
@@ -16,12 +17,12 @@ German accounting chart and localization.
         'account',
         'base_iban',
         'base_vat',
+        'l10n_din5008',
     ],
     'data': [
         'data/account_account_tags_data.xml',
-        'data/menuitem_data.xml',
         'views/account_view.xml',
-        'report/din5008_report.xml',
-        'data/report_layout.xml',
+        'views/res_company_views.xml',
     ],
+    'license': 'LGPL-3',
 }
