@@ -2,7 +2,6 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import datev
-from . import base_document_layout
 from . import chart_template
 from . import ir_actions_report
-from . import account_move
+from . import res_company
