@@ -2,10 +2,12 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 from collections import defaultdict
 from urllib3.util.ssl_ import create_urllib3_context, DEFAULT_CIPHERS
+from urllib3.contrib.pyopenssl import inject_into_urllib3
 from OpenSSL.crypto import load_certificate, load_privatekey, FILETYPE_PEM
 from zeep.transports import Transport
 
-from odoo import fields
+from odoo import fields, models, _
+from odoo.exceptions import UserError
 from odoo.tools import html_escape
 
 import math
@@ -13,11 +15,8 @@ import json
 import requests
 import zeep
 
-from odoo import models, _
-
 
 # Custom patches to perform the WSDL requests.
-
 EUSKADI_CIPHERS = f"{DEFAULT_CIPHERS}:!DH"
 
 
@@ -26,6 +25,7 @@ class PatchedHTTPAdapter(requests.adapters.HTTPAdapter):
 
     def init_poolmanager(self, *args, **kwargs):
         # OVERRIDE
+        inject_into_urllib3()
         kwargs['ssl_context'] = create_urllib3_context(ciphers=EUSKADI_CIPHERS)
         return super().init_poolmanager(*args, **kwargs)
 
@@ -37,17 +37,16 @@ class PatchedHTTPAdapter(requests.adapters.HTTPAdapter):
         # still made without checking temporary files exist.
         super().cert_verify(conn, url, verify, None)
         conn.cert_file = cert
-        conn.key_file = cert
+        conn.key_file = None
 
     def get_connection(self, url, proxies=None):
         # OVERRIDE
         # Patch the OpenSSLContext to decode the certificate in-memory.
         conn = super().get_connection(url, proxies=proxies)
-
         context = conn.conn_kw['ssl_context']
 
         def patched_load_cert_chain(l10n_es_odoo_certificate, keyfile=None, password=None):
-            cert_file, key_file, dummy = l10n_es_odoo_certificate._decode_certificate()
+            cert_file, key_file, _certificate = l10n_es_odoo_certificate.sudo()._decode_certificate()
             cert_obj = load_certificate(FILETYPE_PEM, cert_file)
             pkey_obj = load_privatekey(FILETYPE_PEM, key_file)
 
@@ -68,8 +67,8 @@ class AccountEdiFormat(models.Model):
 
     def _l10n_es_edi_get_invoices_tax_details_info(self, invoice, filter_invl_to_apply=None):
 
-        def grouping_key_generator(tax_values):
-            tax = tax_values['tax_id']
+        def grouping_key_generator(base_line, tax_values):
+            tax = tax_values['tax_repartition_line'].tax_id
             return {
                 'applied_tax_amount': tax.amount,
                 'l10n_es_type': tax.l10n_es_type,
@@ -77,9 +76,9 @@ class AccountEdiFormat(models.Model):
                 'l10n_es_bien_inversion': tax.l10n_es_bien_inversion,
             }
 
-        def filter_to_apply(tax_values):
+        def filter_to_apply(base_line, tax_values):
             # For intra-community, we do not take into account the negative repartition line
-            return tax_values['tax_repartition_line_id'].factor_percent > 0.0
+            return tax_values['tax_repartition_line'].factor_percent > 0.0
 
         def full_filter_invl_to_apply(invoice_line):
             if 'ignore' in invoice_line.tax_ids.flatten_taxes_hierarchy().mapped('l10n_es_type'):
@@ -91,7 +90,7 @@ class AccountEdiFormat(models.Model):
             filter_invl_to_apply=full_filter_invl_to_apply,
             filter_to_apply=filter_to_apply,
         )
-        sign = -1 if invoice.is_sale_document() else 1
+        sign = -1 if invoice.move_type in ('out_refund', 'in_refund') else 1
 
         tax_details_info = defaultdict(dict)
 
@@ -99,7 +98,7 @@ class AccountEdiFormat(models.Model):
         # on the same invoice, this can be deduced globally.
 
         recargo_tax_details = {} # Mapping between main tax and recargo tax details
-        invoice_lines = invoice.invoice_line_ids.filtered(lambda x: not x.display_type)
+        invoice_lines = invoice.invoice_line_ids.filtered(lambda x: x.display_type not in ('line_note', 'line_section'))
         if filter_invl_to_apply:
             invoice_lines = invoice_lines.filtered(filter_invl_to_apply)
         for line in invoice_lines:
@@ -110,7 +109,7 @@ class AccountEdiFormat(models.Model):
                 if not recargo_tax_details.get(recargo_main_tax):
                     recargo_tax_details[recargo_main_tax] = [
                         x for x in tax_details['tax_details'].values()
-                        if x['group_tax_details'][0]['tax_id'] == recargo_tax[0]
+                        if x['group_tax_details'][0]['tax_repartition_line'].tax_id == recargo_tax[0]
                     ][0]
 
         tax_amount_deductible = 0.0
@@ -134,7 +133,7 @@ class AccountEdiFormat(models.Model):
                         'CuotaRepercutida': round(math.copysign(tax_values['tax_amount'], base_amount), 2),
                     }
 
-                    recargo = recargo_tax_details.get(tax_values['group_tax_details'][0]['tax_id'])
+                    recargo = recargo_tax_details.get(tax_values['group_tax_details'][0]['tax_repartition_line'].tax_id)
                     if recargo:
                         tax_info['CuotaRecargoEquivalencia'] = round(sign * recargo['tax_amount'], 2)
                         tax_info['TipoRecargoEquivalencia'] = recargo['applied_tax_amount']
@@ -201,7 +200,7 @@ class AccountEdiFormat(models.Model):
                         })
                     if tax_values['l10n_es_bien_inversion']:
                         tax_info['BienInversion'] = 'S'
-                    recargo = recargo_tax_details.get(tax_values['group_tax_details'][0]['tax_id'])
+                    recargo = recargo_tax_details.get(tax_values['group_tax_details'][0]['tax_repartition_line'].tax_id)
                     if recargo:
                         tax_info['CuotaRecargoEquivalencia'] = round(sign * recargo['tax_amount'], 2)
                         tax_info['TipoRecargoEquivalencia'] = recargo['applied_tax_amount']
@@ -229,7 +228,10 @@ class AccountEdiFormat(models.Model):
         if (not partner.country_id or partner.country_id.code == 'ES') and partner.vat:
             # ES partner with VAT.
             partner_info['NIF'] = partner.vat[2:] if partner.vat.startswith('ES') else partner.vat
-        elif partner.country_id.code in eu_country_codes:
+            if self.env.context.get('error_1117'):
+                partner_info['IDOtro'] = {'IDType': '07', 'ID': IDOtro_ID}
+
+        elif partner.country_id.code in eu_country_codes and partner.vat:
             # European partner.
             partner_info['IDOtro'] = {'IDType': '02', 'ID': IDOtro_ID}
         else:
@@ -301,10 +303,11 @@ class AccountEdiFormat(models.Model):
                 else:
                     invoice_node['FechaRegContable'] = fields.Date.context_today(self).strftime('%d-%m-%Y')
 
-                if not com_partner.country_id or com_partner.country_id.code == 'ES':
+                country_code = com_partner.country_id.code
+                if not country_code or country_code == 'ES' or country_code not in eu_country_codes:
                     invoice_node['ClaveRegimenEspecialOTrascendencia'] = '01'
                 else:
-                    invoice_node['ClaveRegimenEspecialOTrascendencia'] = '09'
+                    invoice_node['ClaveRegimenEspecialOTrascendencia'] = '09' # For Intra-Com
 
             if invoice.move_type == 'out_invoice':
                 invoice_node['TipoFactura'] = 'F2' if is_simplified else 'F1'
@@ -319,7 +322,7 @@ class AccountEdiFormat(models.Model):
 
             # === Taxes ===
 
-            sign = -1 if invoice.is_sale_document() else 1
+            sign = -1 if invoice.move_type in ('out_refund', 'in_refund') else 1
 
             if invoice.is_sale_document():
                 # Customer invoices
@@ -351,6 +354,11 @@ class AccountEdiFormat(models.Model):
                         invoice_node.setdefault('TipoDesglose', {})
                         invoice_node['TipoDesglose'].setdefault('DesgloseTipoOperacion', {})
                         invoice_node['TipoDesglose']['DesgloseTipoOperacion']['Entrega'] = tax_details_info_consu_vals['tax_details_info']
+                    if not invoice_node.get('TipoDesglose'):
+                        raise UserError(_(
+                            "In case of a foreign customer, you need to configure the tax scope on taxes:\n%s",
+                            "\n".join(invoice.line_ids.tax_ids.mapped('name'))
+                        ))
 
                     invoice_node['ImporteTotal'] = round(sign * (
                         tax_details_info_service_vals['tax_details']['base_amount']
@@ -398,9 +406,15 @@ class AccountEdiFormat(models.Model):
 
     def _l10n_es_edi_web_service_aeat_vals(self, invoices):
         if invoices[0].is_sale_document():
-            return {'url': 'https://www2.agenciatributaria.gob.es/static_files/common/internet/dep/aplicaciones/es/aeat/ssii_1_1/fact/ws/SuministroFactEmitidas.wsdl'}
+            return {
+                'url': 'https://www2.agenciatributaria.gob.es/static_files/common/internet/dep/aplicaciones/es/aeat/ssii_1_1/fact/ws/SuministroFactEmitidas.wsdl',
+                'test_url': 'https://prewww1.aeat.es/wlpl/SSII-FACT/ws/fe/SiiFactFEV1SOAP',
+            }
         else:
-            return {'url': 'https://www2.agenciatributaria.gob.es/static_files/common/internet/dep/aplicaciones/es/aeat/ssii_1_1/fact/ws/SuministroFactRecibidas.wsdl'}
+            return {
+                'url': 'https://www2.agenciatributaria.gob.es/static_files/common/internet/dep/aplicaciones/es/aeat/ssii_1_1/fact/ws/SuministroFactRecibidas.wsdl',
+                'test_url': 'https://prewww1.aeat.es/wlpl/SSII-FACT/ws/fr/SiiFactFRV1SOAP',
+            }
 
     def _l10n_es_edi_web_service_bizkaia_vals(self, invoices):
         if invoices[0].is_sale_document():
@@ -429,7 +443,7 @@ class AccountEdiFormat(models.Model):
     def _l10n_es_edi_call_web_service_sign(self, invoices, info_list):
         company = invoices.company_id
 
-        # All are sharing the same value, see '_get_batch_key'.
+        # All are sharing the same value.
         csv_number = invoices.mapped('l10n_es_edi_csv')[0]
 
         # Set registration date
@@ -471,24 +485,24 @@ class AccountEdiFormat(models.Model):
         if company.l10n_es_edi_test_env and connection_vals.get('test_url'):
             serv._binding_options['address'] = connection_vals['test_url']
 
-        msg = ''
+        error_msg = None
         try:
             if invoices[0].is_sale_document():
                 res = serv.SuministroLRFacturasEmitidas(header, info_list)
             else:
                 res = serv.SuministroLRFacturasRecibidas(header, info_list)
         except requests.exceptions.SSLError as error:
-            msg = _("The SSL certificate could not be validated.")
+            error_msg = _("The SSL certificate could not be validated.")
         except zeep.exceptions.Error as error:
-            msg = _("Networking error:\n%s") % error
+            error_msg = _("Networking error:\n%s", error)
         except Exception as error:
-            msg = str(error)
-        finally:
-            if msg:
-                return {inv: {
-                    'error': msg,
-                    'blocking_level': 'warning',
-                } for inv in invoices}
+            error_msg = str(error)
+
+        if error_msg:
+            return {inv: {
+                'error': error_msg,
+                'blocking_level': 'warning',
+            } for inv in invoices}
 
         # Process response.
 
@@ -528,7 +542,7 @@ class AccountEdiFormat(models.Model):
                             inv = candidate
                             break
                         if partner_info.get('IDOtro') and all(getattr(respl_partner_info.IDOtro, k) == v
-                                                           for k, v in partner_info['IDOtro'].items()):
+                                                              for k, v in partner_info['IDOtro'].items()):
                             inv = candidate
                             break
 
@@ -550,6 +564,11 @@ class AccountEdiFormat(models.Model):
                 results[inv] = {'success': True}
                 inv.message_post(body=_("We saw that this invoice was sent correctly before, but we did not treat "
                                         "the response.  Make sure it is not because of a wrong configuration."))
+
+            elif respl.CodigoErrorRegistro == 1117 and not self.env.context.get('error_1117'):
+                return self.with_context(error_1117=True)._post_invoice_edi(invoices)
+
+
             else:
                 results[inv] = {
                     'error': _("[%s] %s", respl.CodigoErrorRegistro, respl.DescripcionErrorRegistro),
@@ -562,31 +581,26 @@ class AccountEdiFormat(models.Model):
     # EDI OVERRIDDEN METHODS
     # -------------------------------------------------------------------------
 
-    def _is_required_for_invoice(self, invoice):
-        # OVERRIDE
-        if self.code != 'es_sii':
-            return super()._is_required_for_invoice(invoice)
+    def _l10n_es_edi_sii_xml_invoice_content(self, invoice):
+        return json.dumps(self._l10n_es_edi_get_invoices_info(invoice)).encode()
 
-        return invoice.l10n_es_edi_is_required
+    def _get_move_applicability(self, move):
+        # EXTENDS account_edi
+        self.ensure_one()
+        if self.code != 'es_sii':
+            return super()._get_move_applicability(move)
+
+        if move.l10n_es_edi_is_required:
+            return {
+                'post': self._l10n_es_edi_sii_post_invoices,
+                'post_batching': lambda invoice: (invoice.move_type, invoice.l10n_es_edi_csv),
+                'edi_content': self._l10n_es_edi_sii_xml_invoice_content,
+            }
 
     def _needs_web_services(self):
         # OVERRIDE
         return self.code == 'es_sii' or super()._needs_web_services()
 
-    def _support_batching(self, move=None, state=None, company=None):
-        # OVERRIDE
-        if self.code != 'es_sii':
-            return super()._support_batching(move=move, state=state, company=company)
-
-        return state == 'to_send' and move.is_invoice()
-
-    def _get_batch_key(self, move, state):
-        # OVERRIDE
-        if self.code != 'es_sii':
-            return super()._get_batch_key(move, state)
-
-        return move.move_type, move.l10n_es_edi_csv
-
     def _check_move_configuration(self, move):
         # OVERRIDE
         res = super()._check_move_configuration(move)
@@ -595,9 +609,7 @@ class AccountEdiFormat(models.Model):
 
         if not move.company_id.vat:
             res.append(_("VAT number is missing on company %s", move.company_id.display_name))
-        if not move.partner_id.vat:
-            res.append(_("VAT number needs to be configured on the partner %s", move.partner_id.display_name))
-        for line in move.invoice_line_ids.filtered(lambda line: not line.display_type):
+        for line in move.invoice_line_ids.filtered(lambda line: line.display_type not in ('line_note', 'line_section')):
             taxes = line.tax_ids.flatten_taxes_hierarchy()
             recargo_count = taxes.mapped('l10n_es_type').count('recargo')
             retention_count = taxes.mapped('l10n_es_type').count('retencion')
@@ -628,11 +640,7 @@ class AccountEdiFormat(models.Model):
 
         return journal.country_code == 'ES'
 
-    def _post_invoice_edi(self, invoices, test_mode=False):
-        # OVERRIDE
-        if self.code != 'es_sii':
-            return super()._post_invoice_edi(invoices, test_mode=test_mode)
-
+    def _l10n_es_edi_sii_post_invoices(self, invoices):
         # Ensure a certificate is available.
         certificate = invoices.company_id.l10n_es_edi_certificate_id
         if not certificate:
@@ -653,10 +661,7 @@ class AccountEdiFormat(models.Model):
         info_list = self._l10n_es_edi_get_invoices_info(invoices)
 
         # Call the web service.
-        if test_mode:
-            res = {inv: {'success': True} for inv in invoices}
-        else:
-            res = self._l10n_es_edi_call_web_service_sign(invoices, info_list)
+        res = self._l10n_es_edi_call_web_service_sign(invoices, info_list)
 
         for inv in invoices:
             if res.get(inv, {}).get('success'):
@@ -668,5 +673,5 @@ class AccountEdiFormat(models.Model):
                     'res_model': inv._name,
                     'res_id': inv.id,
                 })
-                res[inv] = {'attachment': attachment}
+                res[inv]['attachment'] = attachment
         return res
