@@ -4,11 +4,16 @@ from .common import TestEsEdiCommon
 import json
 
 from freezegun import freeze_time
+from unittest.mock import patch
 
 from odoo.tests import tagged
 
 
-@tagged('post_install', '-at_install')
+def mocked_l10n_es_edi_call_web_service_sign(edi_format, invoices, info_list):
+    return {inv: {'success': True} for inv in invoices}
+
+
+@tagged('post_install_l10n', 'post_install', '-at_install')
 class TestEdiXmls(TestEsEdiCommon):
 
     @classmethod
@@ -20,7 +25,9 @@ class TestEdiXmls(TestEsEdiCommon):
         })
 
     def test_010_out_invoice_s_iva10b_s_iva21s(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 partner_id=self.partner_a.id,
                 invoice_line_ids=[
@@ -37,7 +44,7 @@ class TestEdiXmls(TestEsEdiCommon):
             self.assertEqual(json_file, {
                 'IDFactura': {
                     'IDEmisorFactura': {'NIF': '59962470K'},
-                    'NumSerieFacturaEmisor': 'INV/2019/01/0001',
+                    'NumSerieFacturaEmisor': 'INV/2019/00001',
                     'FechaExpedicionFacturaEmisor': '01-01-2019',
                 },
                 'PeriodoLiquidacion': {'Ejercicio': '2019', 'Periodo': '01'},
@@ -90,7 +97,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_020_out_invoice_s_iva10b_s_iva0_ns(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 partner_id=self.partner_b.id,
                 invoice_line_ids=[
@@ -107,7 +116,7 @@ class TestEdiXmls(TestEsEdiCommon):
             self.assertEqual(json_file, {
                 'IDFactura': {
                     'IDEmisorFactura': {'NIF': '59962470K'},
-                    'NumSerieFacturaEmisor': 'INV/2019/01/0001',
+                    'NumSerieFacturaEmisor': 'INV/2019/00001',
                     'FechaExpedicionFacturaEmisor': '01-01-2019',
                 },
                 'PeriodoLiquidacion': {'Ejercicio': '2019', 'Periodo': '01'},
@@ -139,7 +148,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_030_out_invoice_s_iva10b_s_req014_s_iva21s_s_req52(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 partner_id=self.partner_a.id,
                 invoice_line_ids=[
@@ -162,7 +173,7 @@ class TestEdiXmls(TestEsEdiCommon):
             self.assertEqual(json_file, {
                 'IDFactura': {
                     'IDEmisorFactura': {'NIF': '59962470K'},
-                    'NumSerieFacturaEmisor': 'INV/2019/01/0001',
+                    'NumSerieFacturaEmisor': 'INV/2019/00001',
                     'FechaExpedicionFacturaEmisor': '01-01-2019',
                 },
                 'PeriodoLiquidacion': {'Ejercicio': '2019', 'Periodo': '01'},
@@ -219,7 +230,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_040_out_refund_s_iva10b_s_iva10b_s_iva21s(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='out_refund',
                 partner_id=self.partner_a.id,
@@ -238,7 +251,7 @@ class TestEdiXmls(TestEsEdiCommon):
             self.assertEqual(json_file, {
                 'IDFactura': {
                     'IDEmisorFactura': {'NIF': '59962470K'},
-                    'NumSerieFacturaEmisor': 'RINV/2019/01/0001',
+                    'NumSerieFacturaEmisor': 'RINV/2019/00001',
                     'FechaExpedicionFacturaEmisor': '01-01-2019',
                 },
                 'PeriodoLiquidacion': {'Ejercicio': '2019', 'Periodo': '01'},
@@ -292,7 +305,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_050_out_invoice_s_iva0_sp_i_s_iva0_ic(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 partner_id=self.partner_a.id,
                 invoice_line_ids=[
@@ -309,7 +324,7 @@ class TestEdiXmls(TestEsEdiCommon):
             self.assertEqual(json_file, {
                 'IDFactura': {
                     'IDEmisorFactura': {'NIF': '59962470K'},
-                    'NumSerieFacturaEmisor': 'INV/2019/01/0001',
+                    'NumSerieFacturaEmisor': 'INV/2019/00001',
                     'FechaExpedicionFacturaEmisor': '01-01-2019',
                 },
                 'PeriodoLiquidacion': {'Ejercicio': '2019', 'Periodo': '01'},
@@ -347,7 +362,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_060_out_refund_s_iva0_sp_i_s_iva0_ic(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='out_refund',
                 partner_id=self.partner_a.id,
@@ -365,7 +382,7 @@ class TestEdiXmls(TestEsEdiCommon):
             self.assertEqual(json_file, {
                 'IDFactura': {
                     'IDEmisorFactura': {'NIF': '59962470K'},
-                    'NumSerieFacturaEmisor': 'RINV/2019/01/0001',
+                    'NumSerieFacturaEmisor': 'RINV/2019/00001',
                     'FechaExpedicionFacturaEmisor': '01-01-2019',
                 },
                 'PeriodoLiquidacion': {'Ejercicio': '2019', 'Periodo': '01'},
@@ -404,7 +421,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_070_out_invoice_s_iva_e_s_iva0_e(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 partner_id=self.partner_a.id,
                 invoice_line_ids=[
@@ -421,7 +440,7 @@ class TestEdiXmls(TestEsEdiCommon):
             self.assertEqual(json_file, {
                 'IDFactura': {
                     'IDEmisorFactura': {'NIF': '59962470K'},
-                    'NumSerieFacturaEmisor': 'INV/2019/01/0001',
+                    'NumSerieFacturaEmisor': 'INV/2019/00001',
                     'FechaExpedicionFacturaEmisor': '01-01-2019',
                 },
                 'PeriodoLiquidacion': {'Ejercicio': '2019', 'Periodo': '01'},
@@ -459,7 +478,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_080_out_refund_s_iva0_sp_i_s_iva0_ic(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='out_refund',
                 partner_id=self.partner_a.id,
@@ -477,7 +498,7 @@ class TestEdiXmls(TestEsEdiCommon):
             self.assertEqual(json_file, {
                 'IDFactura': {
                     'IDEmisorFactura': {'NIF': '59962470K'},
-                    'NumSerieFacturaEmisor': 'RINV/2019/01/0001',
+                    'NumSerieFacturaEmisor': 'RINV/2019/00001',
                     'FechaExpedicionFacturaEmisor': '01-01-2019',
                 },
                 'PeriodoLiquidacion': {'Ejercicio': '2019', 'Periodo': '01'},
@@ -516,7 +537,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_085_out_refund_s_iva0_sp_i_s_iva0_ic_multi_currency(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='out_refund',
                 partner_id=self.partner_a.id,
@@ -535,7 +558,7 @@ class TestEdiXmls(TestEsEdiCommon):
             self.assertEqual(json_file, {
                 'IDFactura': {
                     'IDEmisorFactura': {'NIF': '59962470K'},
-                    'NumSerieFacturaEmisor': 'RINV/2019/01/0001',
+                    'NumSerieFacturaEmisor': 'RINV/2019/00001',
                     'FechaExpedicionFacturaEmisor': '01-01-2019',
                 },
                 'PeriodoLiquidacion': {'Ejercicio': '2019', 'Periodo': '01'},
@@ -574,7 +597,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_090_in_invoice_p_iva10_bc_p_irpf19_p_iva21_sc_p_irpf19(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='in_invoice',
                 ref='sup0001',
@@ -624,7 +649,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_100_in_refund_p_iva10_bc(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='in_refund',
                 ref='sup0001',
@@ -665,7 +692,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_110_in_invoice_p_iva10_bc_p_req014_p_iva21_sc_p_req52(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='in_invoice',
                 ref='sup0001',
@@ -727,7 +756,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_120_in_invoice_p_iva21_sp_ex(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='in_invoice',
                 ref='sup0001',
@@ -770,7 +801,9 @@ class TestEdiXmls(TestEsEdiCommon):
 
     def test_130_in_invoice_p_iva0_ns_p_iva10_bc(self):
         # TODO make it work
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='in_invoice',
                 ref='sup0001',
@@ -815,7 +848,9 @@ class TestEdiXmls(TestEsEdiCommon):
 
     def test_140_out_invoice_s_iva10b_s_irpf1(self):
         # TODO: debug
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 partner_id=self.partner_b.id,
                 invoice_line_ids=[
@@ -835,7 +870,7 @@ class TestEdiXmls(TestEsEdiCommon):
             self.assertEqual(json_file, {
                 'IDFactura': {
                     'IDEmisorFactura': {'NIF': '59962470K'},
-                    'NumSerieFacturaEmisor': 'INV/2019/01/0001',
+                    'NumSerieFacturaEmisor': 'INV/2019/00001',
                     'FechaExpedicionFacturaEmisor': '01-01-2019',
                 },
                 'PeriodoLiquidacion': {'Ejercicio': '2019', 'Periodo': '01'},
@@ -867,7 +902,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_150_in_invoice_p_iva10_bc_p_irpf1(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='in_invoice',
                 ref='sup0001',
@@ -916,7 +953,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_160_in_refund_p_iva10_bc_p_irpf1(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='in_refund',
                 ref='sup0001',
@@ -966,7 +1005,9 @@ class TestEdiXmls(TestEsEdiCommon):
             })
 
     def test_165_in_refund_p_iva10_bc_p_irpf1_multi_currency(self):
-        with freeze_time(self.frozen_today):
+        with freeze_time(self.frozen_today), \
+             patch('odoo.addons.l10n_es_edi_sii.models.account_edi_format.AccountEdiFormat._l10n_es_edi_call_web_service_sign',
+                   new=mocked_l10n_es_edi_call_web_service_sign):
             invoice = self.create_invoice(
                 move_type='in_refund',
                 ref='sup0001',
