@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import api, models, fields
+from odoo import api, models
 
 
 class AccountChartTemplate(models.Model):
@@ -11,7 +11,10 @@ class AccountChartTemplate(models.Model):
     def _prepare_all_journals(self, acc_template_ref, company, journals_dict=None):
         journal_data = super(AccountChartTemplate, self)._prepare_all_journals(
             acc_template_ref, company, journals_dict)
+        if company.account_fiscal_country_id.code != 'FR':
+            return journal_data
+
         for journal in journal_data:
-            if journal['type'] in ('sale', 'purchase') and company.country_id.code == "FR":
+            if journal['type'] in ('sale', 'purchase'):
                 journal.update({'refund_sequence': True})
         return journal_data
