@@ -19,11 +19,6 @@ Install some generic chart of accounts.
         'data/account.account.template.csv',
         'data/l10n_generic_coa_post.xml',
     ],
-    'demo': [
-        'demo/account_bank_statement_demo.xml',
-        'demo/account_invoice_demo.xml',
-        'demo/account_reconcile_model.xml',
-    ],
     'uninstall_hook': 'uninstall_hook',
     'license': 'LGPL-3',
 }
