@@ -1,40 +1,32 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-# Copyright (C) 2014 InnOpen Group Kft (<http://www.innopen.eu>).
-
 {
-    'name': 'Hungarian - Accounting',
-    'version': '2.0',
+    'name': 'Hungary - Accounting',
+    'version': '3.0',
     'category': 'Accounting/Localizations/Account Charts',
     'description': """
-
-Base module for Hungarian localization
-==========================================
-
-This module consists of:
-
- - Generic Hungarian chart of accounts
- - Hungarian taxes
- - Hungarian Bank information
- """,
-    'author': 'InnOpen Group Kft',
-    'website': 'http://www.innopen.eu',
-    'depends': ['account'],
+        Accounting chart and localization for Hungary
+    """,
+    'depends': [
+        'account', 'base_vat', 'l10n_multilang'
+    ],
     'data': [
-        'data/l10n_hu_chart_data.xml',
+        'data/account_chart_template_data.xml',
         'data/account.account.template.csv',
-        'data/account.tax.group.csv',
+        'data/account.group.template.csv',
+        'data/account_tax_group.xml',
         'data/account_tax_report_data.xml',
         'data/account_tax_template_data.xml',
-        'data/account.fiscal.position.template.csv',
-        'data/account.fiscal.position.tax.template.csv',
+        'data/account_fiscal_position_template.xml',
+        'data/account_fiscal_position_tax_template.xml',
         'data/res.bank.csv',
-        'data/account_chart_template_data.xml',
+        'data/l10n_hu_chart_data.xml',
         'data/account_chart_template_configure_data.xml',
     ],
     'demo': [
         'demo/demo_company.xml',
     ],
+    'post_init_hook': 'load_translations',
     'license': 'LGPL-3',
 }
