@@ -10,7 +10,7 @@ class AccountMove(models.Model):
     @api.onchange('purchase_vendor_bill_id', 'purchase_id')
     def _onchange_purchase_auto_complete(self):
         purchase_order_id = self.purchase_vendor_bill_id.purchase_order_id or self.purchase_id
-        if purchase_order_id and self.l10n_in_company_country_code == 'IN':
+        if purchase_order_id and purchase_order_id.country_code == 'IN':
             journal_id = self.purchase_vendor_bill_id.purchase_order_id.l10n_in_journal_id or self.purchase_id.l10n_in_journal_id
             if journal_id:
                 self.journal_id = journal_id
