@@ -3,7 +3,7 @@
 
 {
     'name': 'Italy - Accounting',
-    'version': '0.2',
+    'version': '0.4',
     'depends': [
         'account',
         'base_iban',
@@ -19,15 +19,20 @@ Italian accounting chart and localization.
     'category': 'Accounting/Localizations/Account Charts',
     'website': 'http://www.odoo.com/',
     'data': [
-        'data/l10n_it_chart_data.xml',
+        'data/account_account_tag.xml',
+        'data/account_chart_template.xml',
         'data/account.account.template.csv',
+        'data/account_chart_template_accounts.xml',
         'data/account.tax.group.csv',
+        'data/account_tax_report_data.xml',
         'data/account_tax_template.xml',
         'data/account.fiscal.position.template.csv',
-        'data/account.chart.template.csv',
+        'data/account_fiscal_position_tax_template_data.xml',
         'data/account_chart_template_data.xml',
+        'data/report_invoice.xml'
     ],
     'demo': [
         'demo/demo_company.xml',
     ],
+    'license': 'LGPL-3',
 }
