@@ -5,7 +5,7 @@
 
 {
     'name': 'Netherlands - Accounting',
-    'version': '3.0',
+    'version': '3.2',
     'category': 'Accounting/Localizations/Account Charts',
     'author': 'Onestein',
     'website': 'http://www.onestein.eu',
@@ -13,25 +13,26 @@
         'account',
         'base_iban',
         'base_vat',
-        'base_address_extended',
     ],
     'data': [
         'data/account_account_tag.xml',
         'data/account_chart_template.xml',
         'data/account.account.template.csv',
         'data/account_chart_template_post_data.xml',
-        'data/account_data.xml',
+        'data/account_tax_group_data.xml',
         'data/account_tax_report_data.xml',
         'data/account_tax_template.xml',
         'data/account_fiscal_position_template.xml',
         'data/account_fiscal_position_tax_template.xml',
         'data/account_fiscal_position_account_template.xml',
         'data/account_chart_template_data.xml',
-        'data/menuitem.xml',
+        'views/res_partner_views.xml',
+        'views/res_company_views.xml',
+        'views/res_config_settings_view.xml',
     ],
     'demo': [
         'demo/demo_company.xml',
     ],
-    'auto_install': False,
     'installable': True,
+    'license': 'LGPL-3',
 }
