@@ -3,3 +3,5 @@
 
 from . import account_journal
 from . import account_move
+from . import res_partner
+from . import res_company
