@@ -5,12 +5,14 @@
     'summary': "PCGE Simplified",
     'category': 'Accounting/Localizations/Account Charts',
     'author': 'Vauxoo, Odoo',
+    'website': 'https://www.odoo.com/documentation/16.0/applications/finance/accounting/fiscal_localizations/localizations/peru.html',
     'license': 'LGPL-3',
     'depends': [
         'base_vat',
         'base_address_extended',
-        'base_address_city',
         'l10n_latam_base',
+        'l10n_latam_invoice_document',
+        'account_debit_note',
     ],
     'data': [
         'security/ir.model.access.csv',
@@ -19,8 +21,10 @@
         'data/account.group.template.csv',
         'data/account.account.template.csv',
         'data/l10n_pe_chart_post_data.xml',
+        'data/account_tax_group_data.xml',
         'data/account_tax_data.xml',
         'data/fiscal_position_data.xml',
+        'data/l10n_latam_document_type_data.xml',
         'data/account_chart_template_data.xml',
         'data/res.city.csv',
         'data/l10n_pe.res.city.district.csv',
