@@ -19,9 +19,13 @@
            'data/l10n_pt_chart_data.xml',
            'data/account_chart_template_data.xml',
            'data/account_fiscal_position_template_data.xml',
-           'data/account_data.xml',
+           'data/account_tax_group_data.xml',
+           'data/account_tax_report.xml',
            'data/account_tax_data.xml',
            'data/account_chart_template_configure_data.xml',
            ],
+    'demo': [
+        'demo/demo_company.xml',
+    ],
     'license': 'LGPL-3',
 }
