@@ -6,7 +6,7 @@
 {
     'name': 'Singapore - Accounting',
     'author': 'Tech Receptives',
-    'version': '2.0',
+    'version': '2.1',
     'category': 'Accounting/Localizations/Account Charts',
     'description': """
 Singapore accounting chart and localization.
@@ -21,7 +21,7 @@ This module add, for accounting:
     'depends': ['base', 'account'],
     'data': [
         'data/l10n_sg_chart_data.xml',
-        'data/account_data.xml',
+        'data/account_tax_group_data.xml',
         'data/account_tax_report_data.xml',
         'data/account_tax_data.xml',
         'data/account_chart_template_data.xml',
@@ -29,6 +29,9 @@ This module add, for accounting:
         'views/res_company_view.xml',
         'views/res_partner_view.xml',
     ],
+    'demo': [
+        'demo/demo_company.xml',
+    ],
     'post_init_hook': '_preserve_tag_on_taxes',
     'license': 'LGPL-3',
 }
