@@ -15,13 +15,14 @@ Thai accounting chart and localization.
     'website': 'http://almacom.co.th/',
     'depends': ['account'],
     'data': [
-        'data/account_data.xml',
+        'data/account_tax_group_data.xml',
         'data/l10n_th_chart_data.xml',
         'data/account.account.template.csv',
         'data/l10n_th_chart_post_data.xml',
         'data/account_tax_report_data.xml',
         'data/account_tax_template_data.xml',
         'data/account_chart_template_data.xml',
+        'views/report_invoice.xml',
     ],
     'demo': [
         'demo/demo_company.xml',
