@@ -2,6 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo.tests.common import tagged, HttpCase
+from odoo import Command
 
 
 @tagged('-at_install', 'post_install', 'mail_composer')
@@ -10,14 +11,14 @@ class TestMailFullComposer(HttpCase):
     def test_full_composer_tour(self):
         self.env['mail.template'].create({
             'name': 'Test template',
-            'partner_to': '${object.id}',
-            'lang': '${object.lang}',
+            'partner_to': '{{ object.id }}',
+            'lang': '{{ object.lang }}',
             'auto_delete': True,
             'model_id': self.ref('base.model_res_partner'),
         })
         testuser = self.env['res.users'].create({
             'email': 'testuser@testuser.com',
-            'groups_id': [(6, 0, [self.ref('base.group_user'), self.ref('base.group_partner_manager')])],
+            'groups_id': [Command.set([self.ref('base.group_user'), self.ref('base.group_partner_manager')])],
             'name': 'Test User',
             'login': 'testuser',
             'password': 'testuser',
