@@ -6,21 +6,21 @@
     'version': '1.2',
     'category': 'Productivity/Discuss',
     'summary': 'Add OdooBot in discussions',
-    'description': "",
-    'website': 'https://www.odoo.com/page/discuss',
+    'website': 'https://www.odoo.com/app/discuss',
     'depends': ['mail'],
     'auto_install': True,
     'installable': True,
-    'application': False,
     'data': [
-        'views/assets.xml',
         'views/res_users_views.xml',
         'data/mailbot_data.xml',
     ],
     'demo': [
         'data/mailbot_demo.xml',
     ],
-    'qweb': [
-        'static/src/bugfix/bugfix.xml',
-    ],
+    'assets': {
+        'web.assets_backend': [
+            'mail_bot/static/src/scss/odoobot_style.scss',
+        ],
+    },
+    'license': 'LGPL-3',
 }
