@@ -6,6 +6,7 @@ from datetime import date, datetime, timedelta
 
 from odoo import api, fields, models, SUPERUSER_ID, _
 from odoo.exceptions import UserError
+from odoo.osv import expression
 from odoo.tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT
 
 
@@ -40,7 +41,7 @@ class MaintenanceEquipmentCategory(models.Model):
         default=lambda self: self.env.company)
     technician_user_id = fields.Many2one('res.users', 'Responsible', tracking=True, default=lambda self: self.env.uid)
     color = fields.Integer('Color Index')
-    note = fields.Text('Comments', translate=True)
+    note = fields.Html('Comments', translate=True)
     equipment_ids = fields.One2many('maintenance.equipment', 'category_id', string='Equipments', copy=False)
     equipment_count = fields.Integer(string="Equipment", compute='_compute_equipment_count')
     maintenance_ids = fields.One2many('maintenance.request', 'category_id', copy=False)
@@ -52,22 +53,22 @@ class MaintenanceEquipmentCategory(models.Model):
     fold = fields.Boolean(string='Folded in Maintenance Pipe', compute='_compute_fold', store=True)
 
     def _compute_equipment_count(self):
-        equipment_data = self.env['maintenance.equipment'].read_group([('category_id', 'in', self.ids)], ['category_id'], ['category_id'])
+        equipment_data = self.env['maintenance.equipment']._read_group([('category_id', 'in', self.ids)], ['category_id'], ['category_id'])
         mapped_data = dict([(m['category_id'][0], m['category_id_count']) for m in equipment_data])
         for category in self:
             category.equipment_count = mapped_data.get(category.id, 0)
 
     def _compute_maintenance_count(self):
-        maintenance_data = self.env['maintenance.request'].read_group([('category_id', 'in', self.ids)], ['category_id'], ['category_id'])
+        maintenance_data = self.env['maintenance.request']._read_group([('category_id', 'in', self.ids)], ['category_id'], ['category_id'])
         mapped_data = dict([(m['category_id'][0], m['category_id_count']) for m in maintenance_data])
         for category in self:
             category.maintenance_count = mapped_data.get(category.id, 0)
 
-    def unlink(self):
+    @api.ondelete(at_uninstall=False)
+    def _unlink_except_contains_maintenance_requests(self):
         for category in self:
             if category.equipment_ids or category.maintenance_ids:
                 raise UserError(_("You cannot delete an equipment category containing equipments or maintenance requests."))
-        return super(MaintenanceEquipmentCategory, self).unlink()
 
     def _alias_get_creation_values(self):
         values = super(MaintenanceEquipmentCategory, self)._alias_get_creation_values()
@@ -103,11 +104,9 @@ class MaintenanceEquipment(models.Model):
     def _name_search(self, name, args=None, operator='ilike', limit=100, name_get_uid=None):
         args = args or []
         equipment_ids = []
-        if name:
+        if name and operator not in expression.NEGATIVE_TERM_OPERATORS and operator != '=':
             equipment_ids = self._search([('name', '=', name)] + args, limit=limit, access_rights_uid=name_get_uid)
-        if not equipment_ids:
-            equipment_ids = self._search([('name', operator, name)] + args, limit=limit, access_rights_uid=name_get_uid)
-        return equipment_ids
+        return equipment_ids or super()._name_search(name, args, operator, limit, name_get_uid)
 
     name = fields.Char('Equipment Name', required=True, translate=True)
     company_id = fields.Many2one('res.company', string='Company',
@@ -125,7 +124,7 @@ class MaintenanceEquipment(models.Model):
     assign_date = fields.Date('Assigned Date', tracking=True)
     effective_date = fields.Date('Effective Date', default=fields.Date.context_today, required=True, help="Date at which the equipment became effective. This date will be used to compute the Mean Time Between Failure.")
     cost = fields.Float('Cost')
-    note = fields.Text('Note')
+    note = fields.Html('Note')
     warranty_date = fields.Date('Warranty Expiration Date')
     color = fields.Integer('Color Index')
     scrap_date = fields.Date('Scrap Date')
@@ -200,12 +199,13 @@ class MaintenanceEquipment(models.Model):
         ('serial_no', 'unique(serial_no)', "Another asset already exists with this serial number!"),
     ]
 
-    @api.model
-    def create(self, vals):
-        equipment = super(MaintenanceEquipment, self).create(vals)
-        if equipment.owner_user_id:
-            equipment.message_subscribe(partner_ids=[equipment.owner_user_id.partner_id.id])
-        return equipment
+    @api.model_create_multi
+    def create(self, vals_list):
+        equipments = super().create(vals_list)
+        for equipment in equipments:
+            if equipment.owner_user_id:
+                equipment.message_subscribe(partner_ids=[equipment.owner_user_id.partner_id.id])
+        return equipments
 
     def write(self, vals):
         if vals.get('owner_user_id'):
@@ -220,9 +220,9 @@ class MaintenanceEquipment(models.Model):
         category_ids = categories._search([], order=order, access_rights_uid=SUPERUSER_ID)
         return categories.browse(category_ids)
 
-    def _create_new_request(self, date):
+    def _prepare_maintenance_request_vals(self, date):
         self.ensure_one()
-        self.env['maintenance.request'].create({
+        return {
             'name': _('Preventive Maintenance - %s', self.name),
             'request_date': date,
             'schedule_date': date,
@@ -234,7 +234,13 @@ class MaintenanceEquipment(models.Model):
             'maintenance_team_id': self.maintenance_team_id.id,
             'duration': self.maintenance_duration,
             'company_id': self.company_id.id or self.env.company.id
-            })
+        }
+
+    def _create_new_request(self, date):
+        self.ensure_one()
+        vals = self._prepare_maintenance_request_vals(date)
+        maintenance_requests = self.env['maintenance.request'].create(vals)
+        return maintenance_requests
 
     @api.model
     def _cron_generate_requests(self):
@@ -280,7 +286,7 @@ class MaintenanceRequest(models.Model):
     name = fields.Char('Subjects', required=True)
     company_id = fields.Many2one('res.company', string='Company',
         default=lambda self: self.env.company)
-    description = fields.Text('Description')
+    description = fields.Html('Description')
     request_date = fields.Date('Request Date', tracking=True, default=fields.Date.context_today,
                                help="Date requested for the maintenance to happen")
     owner_user_id = fields.Many2one('res.users', string='Created by User', default=lambda s: s.env.uid)
@@ -331,16 +337,21 @@ class MaintenanceRequest(models.Model):
         if not self.user_id or not self.equipment_id or (self.user_id and not self.equipment_id.technician_user_id):
             self.user_id = self.category_id.technician_user_id
 
-    @api.model
-    def create(self, vals):
+    @api.model_create_multi
+    def create(self, vals_list):
         # context: no_log, because subtype already handle this
-        request = super(MaintenanceRequest, self).create(vals)
-        if request.owner_user_id or request.user_id:
-            request._add_followers()
-        if request.equipment_id and not request.maintenance_team_id:
-            request.maintenance_team_id = request.equipment_id.maintenance_team_id
-        request.activity_update()
-        return request
+        maintenance_requests = super().create(vals_list)
+        for request in maintenance_requests:
+            if request.owner_user_id or request.user_id:
+                request._add_followers()
+            if request.equipment_id and not request.maintenance_team_id:
+                request.maintenance_team_id = request.equipment_id.maintenance_team_id
+            if request.close_date and not request.stage_id.done:
+                request.close_date = False
+            if not request.close_date and request.stage_id.done:
+                request.close_date = fields.Date.today()
+        maintenance_requests.activity_update()
+        return maintenance_requests
 
     def write(self, vals):
         # Overridden to reset the kanban_state to normal whenever
@@ -352,7 +363,9 @@ class MaintenanceRequest(models.Model):
             self._add_followers()
         if 'stage_id' in vals:
             self.filtered(lambda m: m.stage_id.done).write({'close_date': fields.Date.today()})
+            self.filtered(lambda m: not m.stage_id.done).write({'close_date': False})
             self.activity_feedback(['maintenance.mail_act_maintenance_request'])
+            self.activity_update()
         if vals.get('user_id') or vals.get('schedule_date'):
             self.activity_update()
         if vals.get('equipment_id'):
@@ -373,8 +386,10 @@ class MaintenanceRequest(models.Model):
                 new_user_id=request.user_id.id or request.owner_user_id.id or self.env.uid)
             if not updated:
                 if request.equipment_id:
-                    note = _('Request planned for <a href="#" data-oe-model="%s" data-oe-id="%s">%s</a>') % (
-                        request.equipment_id._name, request.equipment_id.id, request.equipment_id.display_name)
+                    note = _(
+                        'Request planned for %s',
+                        request.equipment_id._get_html_link()
+                    )
                 else:
                     note = False
                 request.activity_schedule(
@@ -422,12 +437,12 @@ class MaintenanceTeam(models.Model):
     @api.depends('request_ids.stage_id.done')
     def _compute_todo_requests(self):
         for team in self:
-            team.todo_request_ids = team.request_ids.filtered(lambda e: e.stage_id.done==False)
+            team.todo_request_ids = self.env['maintenance.request'].search([('maintenance_team_id', '=', team.id), ('stage_id.done', '=', False)])
             team.todo_request_count = len(team.todo_request_ids)
-            team.todo_request_count_date = len(team.todo_request_ids.filtered(lambda e: e.schedule_date != False))
-            team.todo_request_count_high_priority = len(team.todo_request_ids.filtered(lambda e: e.priority == '3'))
-            team.todo_request_count_block = len(team.todo_request_ids.filtered(lambda e: e.kanban_state == 'blocked'))
-            team.todo_request_count_unscheduled = len(team.todo_request_ids.filtered(lambda e: not e.schedule_date))
+            team.todo_request_count_date = self.env['maintenance.request'].search_count([('maintenance_team_id', '=', team.id), ('schedule_date', '!=', False)])
+            team.todo_request_count_high_priority = self.env['maintenance.request'].search_count([('maintenance_team_id', '=', team.id), ('priority', '=', '3')])
+            team.todo_request_count_block = self.env['maintenance.request'].search_count([('maintenance_team_id', '=', team.id), ('kanban_state', '=', 'blocked')])
+            team.todo_request_count_unscheduled = self.env['maintenance.request'].search_count([('maintenance_team_id', '=', team.id), ('schedule_date', '=', False)])
 
     @api.depends('equipment_ids')
     def _compute_equipment(self):
