@@ -1,15 +1,22 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from . import ir_mail_server
+from . import ir_model
 from . import link_tracker
+from . import mailing_contact_subscription
 from . import mailing_contact
 from . import mailing_list
 from . import mailing_trace
 from . import mailing
+from . import mailing_filter
 from . import mail_mail
 from . import mail_render_mixin
 from . import mail_thread
-from . import res_config_settings
-from . import res_users
-from . import utm
 from . import res_company
+from . import res_config_settings
+from . import res_partner
+from . import res_users
+from . import utm_campaign
+from . import utm_medium
+from . import utm_source
