@@ -15,7 +15,7 @@ from . import res_company
 from . import stock_move
 from . import stock_orderpoint
 from . import stock_picking
-from . import stock_production_lot
+from . import stock_lot
 from . import stock_rule
 from . import stock_scrap
 from . import stock_warehouse
