@@ -1,53 +1,59 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo.tests import Form
+from odoo.tests import Form, tagged
 from odoo.addons.mrp.tests.common import TestMrpCommon
 
 
+@tagged('post_install', '-at_install')
 class TestMultistepManufacturingWarehouse(TestMrpCommon):
 
-    def setUp(self):
-        super(TestMultistepManufacturingWarehouse, self).setUp()
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        # Required for `uom_id` to be visible in the view
+        cls.env.user.groups_id += cls.env.ref('uom.group_uom')
+        # Required for `manufacture_steps` to be visible in the view
+        cls.env.user.groups_id += cls.env.ref('stock.group_adv_location')
         # Create warehouse
-        self.customer_location = self.env['ir.model.data'].xmlid_to_res_id('stock.stock_location_customers')
-        warehouse_form = Form(self.env['stock.warehouse'])
+        cls.customer_location = cls.env['ir.model.data']._xmlid_to_res_id('stock.stock_location_customers')
+        warehouse_form = Form(cls.env['stock.warehouse'])
         warehouse_form.name = 'Test Warehouse'
         warehouse_form.code = 'TWH'
-        self.warehouse = warehouse_form.save()
+        cls.warehouse = warehouse_form.save()
 
-        self.uom_unit = self.env.ref('uom.product_uom_unit')
+        cls.uom_unit = cls.env.ref('uom.product_uom_unit')
 
         # Create manufactured product
-        product_form = Form(self.env['product.product'])
+        product_form = Form(cls.env['product.product'])
         product_form.name = 'Stick'
-        product_form.uom_id = self.uom_unit
-        product_form.uom_po_id = self.uom_unit
-        product_form.type = 'product'
+        product_form.uom_id = cls.uom_unit
+        product_form.uom_po_id = cls.uom_unit
+        product_form.detailed_type = 'product'
         product_form.route_ids.clear()
-        product_form.route_ids.add(self.warehouse.manufacture_pull_id.route_id)
-        product_form.route_ids.add(self.warehouse.mto_pull_id.route_id)
-        self.finished_product = product_form.save()
+        product_form.route_ids.add(cls.warehouse.manufacture_pull_id.route_id)
+        product_form.route_ids.add(cls.warehouse.mto_pull_id.route_id)
+        cls.finished_product = product_form.save()
 
         # Create raw product for manufactured product
-        product_form = Form(self.env['product.product'])
+        product_form = Form(cls.env['product.product'])
         product_form.name = 'Raw Stick'
-        product_form.type = 'product'
-        product_form.uom_id = self.uom_unit
-        product_form.uom_po_id = self.uom_unit
-        self.raw_product = product_form.save()
+        product_form.detailed_type = 'product'
+        product_form.uom_id = cls.uom_unit
+        product_form.uom_po_id = cls.uom_unit
+        cls.raw_product = product_form.save()
 
         # Create bom for manufactured product
-        bom_product_form = Form(self.env['mrp.bom'])
-        bom_product_form.product_id = self.finished_product
-        bom_product_form.product_tmpl_id = self.finished_product.product_tmpl_id
+        bom_product_form = Form(cls.env['mrp.bom'])
+        bom_product_form.product_id = cls.finished_product
+        bom_product_form.product_tmpl_id = cls.finished_product.product_tmpl_id
         bom_product_form.product_qty = 1.0
         bom_product_form.type = 'normal'
         with bom_product_form.bom_line_ids.new() as bom_line:
-            bom_line.product_id = self.raw_product
+            bom_line.product_id = cls.raw_product
             bom_line.product_qty = 2.0
 
-        self.bom = bom_product_form.save()
+        cls.bom = bom_product_form.save()
 
     def _check_location_and_routes(self):
         # Check manufacturing pull rule.
@@ -68,7 +74,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         self._check_location_and_routes()
         # Check locations of existing pull rule
         self.assertFalse(self.warehouse.pbm_route_id.rule_ids, 'only the update of global manufacture route should happen.')
-        self.assertEqual(self.warehouse.manufacture_pull_id.location_id.id, self.warehouse.lot_stock_id.id)
+        self.assertEqual(self.warehouse.manufacture_pull_id.location_dest_id.id, self.warehouse.lot_stock_id.id)
 
     def test_01_warehouse_twostep_manufacturing(self):
         """ Warehouse testing for picking before manufacturing """
@@ -76,7 +82,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
             warehouse.manufacture_steps = 'pbm'
         self._check_location_and_routes()
         self.assertEqual(len(self.warehouse.pbm_route_id.rule_ids), 2)
-        self.assertEqual(self.warehouse.manufacture_pull_id.location_id.id, self.warehouse.lot_stock_id.id)
+        self.assertEqual(self.warehouse.manufacture_pull_id.location_dest_id.id, self.warehouse.lot_stock_id.id)
 
     def test_02_warehouse_twostep_manufacturing(self):
         """ Warehouse testing for picking ans store after manufacturing """
@@ -84,7 +90,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
             warehouse.manufacture_steps = 'pbm_sam'
         self._check_location_and_routes()
         self.assertEqual(len(self.warehouse.pbm_route_id.rule_ids), 3)
-        self.assertEqual(self.warehouse.manufacture_pull_id.location_id.id, self.warehouse.sam_loc_id.id)
+        self.assertEqual(self.warehouse.manufacture_pull_id.location_dest_id.id, self.warehouse.sam_loc_id.id)
 
     def test_manufacturing_3_steps(self):
         """ Test MO/picking before manufacturing/picking after manufacturing
@@ -132,13 +138,13 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         with Form(self.warehouse) as warehouse:
             warehouse.manufacture_steps = 'pbm_sam'
             warehouse.delivery_steps = 'pick_pack_ship'
-        self.warehouse.flush()
+        self.warehouse.flush_model()
         self.env.ref('stock.route_warehouse0_mto').active = True
         self.env['stock.quant']._update_available_quantity(self.raw_product, self.warehouse.lot_stock_id, 4.0)
         picking_customer = self.env['stock.picking'].create({
             'location_id': self.warehouse.wh_output_stock_loc_id.id,
             'location_dest_id': self.customer_location,
-            'partner_id': self.env['ir.model.data'].xmlid_to_res_id('base.res_partner_4'),
+            'partner_id': self.env['ir.model.data']._xmlid_to_res_id('base.res_partner_4'),
             'picking_type_id': self.warehouse.out_type_id.id,
         })
         self.env['stock.move'].create({
@@ -174,7 +180,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
 
         self.assertTrue(picking_stock_preprod)
         self.assertTrue(picking_stock_postprod)
-        self.assertEqual(picking_stock_preprod.state, 'confirmed')
+        self.assertEqual(picking_stock_preprod.state, 'assigned')
         self.assertEqual(picking_stock_postprod.state, 'waiting')
         self.assertEqual(picking_stock_preprod.origin, production_order.name, 'The pre-prod origin should be the MO name')
         self.assertEqual(picking_stock_postprod.origin, 'SOURCEDOCUMENT', 'The post-prod origin should be the SO name')
@@ -205,7 +211,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
             ('location_dest_id', '=', self.warehouse.wh_pack_stock_loc_id.id),
             ('picking_type_id', '=', self.warehouse.pick_type_id.id)
         ]).picking_id
-        self.assertEqual(picking_stock_pick.move_lines.move_orig_ids.picking_id, picking_stock_postprod)
+        self.assertEqual(picking_stock_pick.move_ids.move_orig_ids.picking_id, picking_stock_postprod)
 
     def test_cancel_propagation(self):
         """ Test cancelling moves in a 'picking before
@@ -213,12 +219,12 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         cancel depends on the default values on each rule of the chain.
         """
         self.warehouse.manufacture_steps = 'pbm_sam'
-        self.warehouse.flush()
+        self.warehouse.flush_model()
         self.env['stock.quant']._update_available_quantity(self.raw_product, self.warehouse.lot_stock_id, 4.0)
         picking_customer = self.env['stock.picking'].create({
             'location_id': self.warehouse.lot_stock_id.id,
             'location_dest_id': self.customer_location,
-            'partner_id': self.env['ir.model.data'].xmlid_to_res_id('base.res_partner_4'),
+            'partner_id': self.env['ir.model.data']._xmlid_to_res_id('base.res_partner_4'),
             'picking_type_id': self.warehouse.out_type_id.id,
         })
         self.env['stock.move'].create({
@@ -250,7 +256,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
 
         self.assertTrue(move_stock_preprod)
         self.assertTrue(move_stock_postprod)
-        self.assertEqual(move_stock_preprod.state, 'confirmed')
+        self.assertEqual(move_stock_preprod.state, 'assigned')
         self.assertEqual(move_stock_postprod.state, 'waiting')
 
         move_stock_preprod._action_cancel()
@@ -297,7 +303,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         production = production_form.save()
 
         production.action_confirm()
-
+        production.is_locked = False
         production_form = Form(production)
         with production_form.move_raw_ids.new() as move:
             move.product_id = new_product
@@ -309,6 +315,169 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         self.assertEqual(len(pbm_move), 2)
         self.assertTrue(new_product in pbm_move.product_id)
 
+    def test_3_steps_and_byproduct(self):
+        """ Suppose a warehouse with Manufacture option set to '3 setps' and a product P01 with a reordering rule.
+        Suppose P01 has a BoM and this BoM mentions that when some P01 are produced, some P02 are produced too.
+        This test ensures that when a MO is generated thanks to the reordering rule, 2 pickings are also
+        generated:
+            - One to bring the components
+            - Another to return the P01 and P02 produced
+        """
+        warehouse = self.warehouse
+        warehouse.manufacture_steps = 'pbm_sam'
+        warehouse_stock_location = warehouse.lot_stock_id
+        pre_production_location = warehouse.pbm_loc_id
+        post_production_location = warehouse.sam_loc_id
+
+        one_unit_uom = self.env.ref('uom.product_uom_unit')
+        [two_units_uom, four_units_uom] = self.env['uom.uom'].create([{
+            'name': 'x%s' % i,
+            'category_id': self.ref('uom.product_uom_categ_unit'),
+            'uom_type': 'bigger',
+            'factor_inv': i,
+        } for i in [2, 4]])
+
+        finished_product = self.env['product.product'].create({
+            'name': 'Super Product',
+            'route_ids': [(4, self.ref('mrp.route_warehouse0_manufacture'))],
+            'type': 'product',
+        })
+        secondary_product = self.env['product.product'].create({
+            'name': 'Secondary',
+            'type': 'product',
+        })
+        component = self.env['product.product'].create({
+            'name': 'Component',
+            'type': 'consu',
+        })
+
+        self.env['mrp.bom'].create({
+            'product_tmpl_id': finished_product.product_tmpl_id.id,
+            'product_qty': 1,
+            'product_uom_id': two_units_uom.id,
+            'bom_line_ids': [(0, 0, {
+                'product_id': component.id,
+                'product_qty': 1,
+                'product_uom_id': one_unit_uom.id,
+            })],
+            'byproduct_ids': [(0, 0, {
+                'product_id': secondary_product.id,
+                'product_qty': 1,
+                'product_uom_id': four_units_uom.id,
+            })],
+        })
+
+        self.env['stock.warehouse.orderpoint'].create({
+            'warehouse_id': warehouse.id,
+            'location_id': warehouse_stock_location.id,
+            'product_id': finished_product.id,
+            'product_min_qty': 2,
+            'product_max_qty': 2,
+        })
+
+        self.env['procurement.group'].run_scheduler()
+        mo = self.env['mrp.production'].search([('product_id', '=', finished_product.id)])
+        pickings = mo.picking_ids
+        self.assertEqual(len(pickings), 2)
+
+        preprod_picking = pickings[0] if pickings[0].location_id == warehouse_stock_location else pickings[1]
+        self.assertEqual(preprod_picking.location_id, warehouse_stock_location)
+        self.assertEqual(preprod_picking.location_dest_id, pre_production_location)
+
+        postprod_picking = pickings - preprod_picking
+        self.assertEqual(postprod_picking.location_id, post_production_location)
+        self.assertEqual(postprod_picking.location_dest_id, warehouse_stock_location)
+
+        byproduct_postprod_move = self.env['stock.move'].search([
+            ('product_id', '=', secondary_product.id),
+            ('location_id', '=', post_production_location.id),
+            ('location_dest_id', '=', warehouse_stock_location.id),
+        ])
+        self.assertEqual(byproduct_postprod_move.state, 'waiting')
+        self.assertEqual(byproduct_postprod_move.group_id.name, mo.name)
+
+    def test_manufacturing_3_steps_trigger_reordering_rules(self):
+        with Form(self.warehouse) as warehouse:
+            warehouse.manufacture_steps = 'pbm_sam'
+
+        with Form(self.raw_product) as p:
+            p.route_ids.clear()
+            p.route_ids.add(self.warehouse.manufacture_pull_id.route_id)
+
+        # Create an additional BoM for component
+        product_form = Form(self.env['product.product'])
+        product_form.name = 'Wood'
+        product_form.detailed_type = 'product'
+        product_form.uom_id = self.uom_unit
+        product_form.uom_po_id = self.uom_unit
+        self.wood_product = product_form.save()
+
+        # Create bom for manufactured product
+        bom_product_form = Form(self.env['mrp.bom'])
+        bom_product_form.product_id = self.raw_product
+        bom_product_form.product_tmpl_id = self.raw_product.product_tmpl_id
+        bom_product_form.product_qty = 1.0
+        bom_product_form.type = 'normal'
+        with bom_product_form.bom_line_ids.new() as bom_line:
+            bom_line.product_id = self.wood_product
+            bom_line.product_qty = 1.0
+
+        bom_product_form.save()
+
+        self.env['stock.quant']._update_available_quantity(
+            self.finished_product, self.warehouse.lot_stock_id, -1.0)
+
+        rr_form = Form(self.env['stock.warehouse.orderpoint'])
+        rr_form.product_id = self.wood_product
+        rr_form.location_id = self.warehouse.lot_stock_id
+        rr_form.save()
+
+        rr_form = Form(self.env['stock.warehouse.orderpoint'])
+        rr_form.product_id = self.finished_product
+        rr_form.location_id = self.warehouse.lot_stock_id
+        rr_finish = rr_form.save()
+
+        rr_form = Form(self.env['stock.warehouse.orderpoint'])
+        rr_form.product_id = self.raw_product
+        rr_form.location_id = self.warehouse.lot_stock_id
+        rr_form.save()
+
+        self.env['procurement.group'].run_scheduler()
+
+        pickings_component = self.env['stock.picking'].search(
+            [('product_id', '=', self.wood_product.id)])
+        self.assertTrue(pickings_component)
+        self.assertTrue(rr_finish.name in pickings_component.origin)
+
+    def test_2_steps_and_additional_moves(self):
+        """ Suppose a 2-steps configuration. If a user adds a product to an existing draft MO and then
+        confirms it, the associated picking should includes this new product"""
+        self.warehouse.manufacture_steps = 'pbm'
+
+        mo_form = Form(self.env['mrp.production'])
+        mo_form.product_id = self.bom.product_id
+        mo_form.picking_type_id = self.warehouse.manu_type_id
+        mo = mo_form.save()
+
+        component_move = mo.move_raw_ids[0]
+        mo.with_context(default_raw_material_production_id=mo.id).move_raw_ids = [
+            [0, 0, {
+                'location_id': component_move.location_id.id,
+                'location_dest_id': component_move.location_dest_id.id,
+                'picking_type_id': component_move.picking_type_id.id,
+                'product_id': self.product_2.id,
+                'name': self.product_2.display_name,
+                'product_uom_qty': 1,
+                'product_uom': self.product_2.uom_id.id,
+                'warehouse_id': component_move.warehouse_id.id,
+                'raw_material_production_id': mo.id,
+            }]
+        ]
+
+        mo.action_confirm()
+
+        self.assertEqual(self.bom.bom_line_ids.product_id + self.product_2, mo.picking_ids.move_ids.product_id)
+
     def test_manufacturing_complex_product_3_steps(self):
         """ Test MO/picking after manufacturing a complex product which uses
         manufactured components. Ensure that everything is created and picked
@@ -318,28 +487,24 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         self.warehouse.mto_pull_id.route_id.active = True
         # Creating complex product which trigger another manifacture
 
-        product_form = Form(self.env['product.product'])
-        product_form.name = 'Arrow'
-        product_form.type = 'product'
-        product_form.route_ids.clear()
-        product_form.route_ids.add(self.warehouse.manufacture_pull_id.route_id)
-        product_form.route_ids.add(self.warehouse.mto_pull_id.route_id)
-        self.complex_product = product_form.save()
+        routes = self.warehouse.manufacture_pull_id.route_id + self.warehouse.mto_pull_id.route_id
+        self.complex_product = self.env['product.product'].create({
+            'name': 'Arrow',
+            'type': 'product',
+            'route_ids': [(6, 0, routes.ids)],
+        })
 
-        ## Create raw product for manufactured product
-        product_form = Form(self.env['product.product'])
-        product_form.name = 'Raw Iron'
-        product_form.type = 'product'
-        product_form.uom_id = self.uom_unit
-        product_form.uom_po_id = self.uom_unit
-        self.raw_product_2 = product_form.save()
+        # Create raw product for manufactured product
+        self.raw_product_2 = self.env['product.product'].create({
+            'name': 'Raw Iron',
+            'type': 'product',
+            'uom_id': self.uom_unit.id,
+            'uom_po_id': self.uom_unit.id,
+        })
 
-        with Form(self.finished_product) as finished_product:
-            finished_product.route_ids.clear()
-            finished_product.route_ids.add(self.warehouse.manufacture_pull_id.route_id)
-            finished_product.route_ids.add(self.warehouse.mto_pull_id.route_id)
+        self.finished_product.route_ids = [(6, 0, routes.ids)]
 
-        ## Create bom for manufactured product
+        # Create bom for manufactured product
         bom_product_form = Form(self.env['mrp.bom'])
         bom_product_form.product_id = self.complex_product
         bom_product_form.product_tmpl_id = self.complex_product.product_tmpl_id
@@ -395,113 +560,138 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         self.assertEqual(len(sfp_pickings), 2)
         self.assertEqual(sfp_pickings.mapped('group_id'), subproduction.procurement_group_id)
 
-        ## Move Raw Stick - Stock -> Preprocessing
+        # Move Raw Stick - Stock -> Preprocessing
         picking = sfp_pickings[0]
-        self.assertEqual(len(picking.move_lines), 1)
-        picking.move_lines[0].product_id = self.raw_product
+        self.assertEqual(len(picking.move_ids), 1)
+        picking.move_ids[0].product_id = self.raw_product
 
-        ## Move SFP - PostProcessing -> Stock
+        # Move SFP - PostProcessing -> Stock
         picking = sfp_pickings[1]
-        self.assertEqual(len(picking.move_lines), 1)
-        picking.move_lines[0].product_id = self.finished_product
+        self.assertEqual(len(picking.move_ids), 1)
+        picking.move_ids[0].product_id = self.finished_product
 
         # Main production 2 pickings, 1 group
         pickings = production.picking_ids.sorted('id')
         self.assertEqual(len(pickings), 2)
         self.assertEqual(pickings.mapped('group_id'), production.procurement_group_id)
 
-        ## Move 2 components Stock -> Preprocessing
+        # Move 2 components Stock -> Preprocessing
         picking = pickings[0]
-        self.assertEqual(len(picking.move_lines), 2)
-        picking.move_lines[0].product_id = self.finished_product
-        picking.move_lines[1].product_id = self.raw_product_2
+        self.assertEqual(len(picking.move_ids), 2)
+        picking.move_ids[0].product_id = self.finished_product
+        picking.move_ids[1].product_id = self.raw_product_2
 
-        ## Move FP PostProcessing -> Stock
+        # Move FP PostProcessing -> Stock
         picking = pickings[1]
-        self.assertEqual(len(picking.move_lines), 1)
+        self.assertEqual(len(picking.move_ids), 1)
         picking.product_id = self.complex_product
 
-    def test_3_steps_and_byproduct(self):
-        """ Suppose a warehouse with Manufacture option set to '3 setps' and a product P01 with a reordering rule.
-        Suppose P01 has a BoM and this BoM mentions that when some P01 are produced, some P02 are produced too.
-        This test ensures that when a MO is generated thanks to the reordering rule, 2 pickings are also
-        generated:
-            - One to bring the components
-            - Another to return the P01 and P02 produced
-        """
-        warehouse = self.warehouse
-        warehouse.manufacture_steps = 'pbm_sam'
-        warehouse_stock_location = warehouse.lot_stock_id
-        pre_production_location = warehouse.pbm_loc_id
-        post_production_location = warehouse.sam_loc_id
+    def test_child_parent_relationship_on_backorder_creation(self):
+        """ Test Child Mo and Source Mo in 2/3-step production for reorder
+            rules in backorder using order points with the help of run scheduler """
 
-        one_unit_uom = self.env['ir.model.data'].xmlid_to_object('uom.product_uom_unit')
-        [two_units_uom, four_units_uom] = self.env['uom.uom'].create([{
-            'name': 'x%s' % i,
-            'category_id': self.ref('uom.product_uom_categ_unit'),
-            'uom_type': 'bigger',
-            'factor_inv': i,
-        } for i in [2, 4]])
+        with Form(self.warehouse) as warehouse:
+            warehouse.manufacture_steps = 'pbm_sam'
 
-        finished_product = self.env['product.product'].create({
-            'name': 'Super Product',
-            'route_ids': [(4, self.ref('mrp.route_warehouse0_manufacture'))],
-            'type': 'product',
-        })
-        secondary_product = self.env['product.product'].create({
-            'name': 'Secondary',
-            'type': 'product',
-        })
-        component = self.env['product.product'].create({
-            'name': 'Component',
-            'type': 'consu',
-        })
-
-        bom = self.env['mrp.bom'].create({
-            'product_tmpl_id': finished_product.product_tmpl_id.id,
-            'product_qty': 1,
-            'product_uom_id': two_units_uom.id,
-            'bom_line_ids': [(0, 0, {
-                'product_id': component.id,
-                'product_qty': 1,
-                'product_uom_id': one_unit_uom.id,
-            })],
-            'byproduct_ids': [(0, 0, {
-                'product_id': secondary_product.id,
-                'product_qty': 1,
-                'product_uom_id': four_units_uom.id,
-            })],
-        })
-
-        orderpoint = self.env['stock.warehouse.orderpoint'].create({
-            'warehouse_id': warehouse.id,
-            'location_id': warehouse_stock_location.id,
-            'product_id': finished_product.id,
-            'product_min_qty': 2,
-            'product_max_qty': 2,
-        })
+        rr_form = Form(self.env['stock.warehouse.orderpoint'])
+        rr_form.product_id = self.finished_product
+        rr_form.product_min_qty = 20
+        rr_form.product_max_qty = 40
+        rr_form.save()
 
         self.env['procurement.group'].run_scheduler()
-        mo = self.env['mrp.production'].search([('product_id', '=', finished_product.id)])
-        pickings = mo.picking_ids
-        self.assertEqual(len(pickings), 2)
 
-        preprod_picking = pickings[0] if pickings[0].location_id == warehouse_stock_location else pickings[1]
-        self.assertEqual(preprod_picking.location_id, warehouse_stock_location)
-        self.assertEqual(preprod_picking.location_dest_id, pre_production_location)
+        mo = self.env['mrp.production'].search([('product_id', '=', self.finished_product.id)])
+        mo_form = Form(mo)
+        mo_form.qty_producing = 20
+        mo = mo_form.save()
 
-        postprod_picking = pickings - preprod_picking
-        self.assertEqual(postprod_picking.location_id, post_production_location)
-        self.assertEqual(postprod_picking.location_dest_id, warehouse_stock_location)
+        action = mo.button_mark_done()
+        backorder = Form(self.env['mrp.production.backorder'].with_context(**action['context']))
+        backorder.save().action_backorder()
 
-        postprod_SML = postprod_picking.move_lines
-        self.assertEqual(len(postprod_SML), 2)
-        self.assertEqual(postprod_SML.location_id, post_production_location)
-        self.assertEqual(postprod_SML.location_dest_id, warehouse_stock_location)
+        self.assertEqual(mo.mrp_production_child_count, 0, "Children MOs counted as existing where there should be none")
+        self.assertEqual(mo.mrp_production_source_count, 0, "Source MOs counted as existing where there should be none")
+        self.assertEqual(mo.mrp_production_backorder_count, 2)
 
-        finished_product_SML = postprod_SML[0] if postprod_SML[0].product_id == finished_product else postprod_SML[1]
-        secondary_product_SML = postprod_SML - finished_product_SML
-        self.assertEqual(finished_product_SML.product_uom.id, one_unit_uom.id)
-        self.assertEqual(finished_product_SML.product_uom_qty, 2)
-        self.assertEqual(secondary_product_SML.product_uom.id, four_units_uom.id)
-        self.assertEqual(secondary_product_SML.product_uom_qty, 1)
+    def test_source_location_on_merge_mo_3_steps(self):
+        """Check that default values are correct after merging mos when 3-step manufacturing"""
+
+        with Form(self.warehouse) as warehouse:
+            warehouse.manufacture_steps = 'pbm_sam'
+
+        # picking with non default location
+        picking_type = self.env['stock.picking.type'].create({
+            'name': 'Manufacturing',
+            'code': 'mrp_operation',
+            'warehouse_id': warehouse.id,
+            'default_location_src_id': self.warehouse.pbm_loc_id.copy().id,
+            'default_location_dest_id': self.warehouse.sam_loc_id.copy().id,
+            'sequence_code': 'TMP',
+            'sequence_id': self.env['ir.sequence'].create({
+                'code': 'mrp.production',
+                'name': 'tmp_production_sequence',
+            }).id,
+        })
+
+        mo1_form = Form(self.env['mrp.production'])
+        mo1_form.product_id = self.finished_product
+        mo1_form.picking_type_id = picking_type
+        mo1 = mo1_form.save()
+        mo1.action_confirm()
+
+        mo2_form = Form(self.env['mrp.production'])
+        mo2_form.product_id = self.finished_product
+        mo2_form.picking_type_id = picking_type
+        mo2 = mo2_form.save()
+        mo2.action_confirm()
+
+        action = (mo1 + mo2).action_merge()
+        mo = self.env[action['res_model']].browse(action['res_id'])
+
+        self.assertEqual(picking_type.default_location_src_id, mo.move_raw_ids.location_id,
+            "The default source location of the merged mo should be the same as the 1st of the original MOs")
+        self.assertEqual(picking_type, mo.picking_type_id,
+            "The operation type of the merged mo should be the same as the 1st of the original MOs")
+
+    def test_manufacturing_bom_from_reordering_rules(self):
+        """
+            Check that the manufacturing order is created with the BoM set in the reording rule:
+                - Create a product with 2 bill of materials,
+                - Create an orderpoint for this product specifying the 2nd BoM that must be used,
+                - Check that the MO has been created with the 2nd BoM
+        """
+        manufacturing_route = self.env['stock.rule'].search([
+            ('action', '=', 'manufacture')]).route_id
+        with Form(self.warehouse) as warehouse:
+            warehouse.manufacture_steps = 'pbm_sam'
+        finished_product = self.env['product.product'].create({
+            'name': 'Product',
+            'type': 'product',
+            'route_ids': manufacturing_route,
+        })
+        self.env['mrp.bom'].create({
+            'product_tmpl_id': finished_product.product_tmpl_id.id,
+            'product_qty': 1,
+            'product_uom_id': finished_product.uom_id.id,
+            'type': 'normal',
+        })
+        bom_2 = self.env['mrp.bom'].create({
+            'product_tmpl_id': finished_product.product_tmpl_id.id,
+            'product_qty': 1,
+            'product_uom_id': finished_product.uom_id.id,
+            'type': 'normal',
+        })
+        self.env['stock.warehouse.orderpoint'].create({
+            'name': 'Orderpoint for P1',
+            'product_id': self.finished_product.id,
+            'product_min_qty': 1,
+            'product_max_qty': 1,
+            'route_id': manufacturing_route.id,
+            'bom_id': bom_2.id,
+        })
+        self.env['procurement.group'].run_scheduler()
+        mo = self.env['mrp.production'].search([('product_id', '=', self.finished_product.id)])
+        self.assertEqual(len(mo), 1)
+        self.assertEqual(mo.product_qty, 1.0)
+        self.assertEqual(mo.bom_id, bom_2)
