@@ -11,4 +11,9 @@
     'depends': ['mrp_subcontracting', 'mrp_account'],
     'installable': True,
     'auto_install': True,
+    'license': 'LGPL-3',
+    'data': [
+        'security/mrp_subcontracting_account_security.xml',
+        'security/ir.model.access.csv',
+    ],
 }
