@@ -10,16 +10,10 @@ class StockPicking(models.Model):
 
     def action_view_stock_valuation_layers(self):
         action = super(StockPicking, self).action_view_stock_valuation_layers()
-        subcontracted_productions = self._get_subcontracted_productions()
+        subcontracted_productions = self._get_subcontract_production()
         if not subcontracted_productions:
             return action
         domain = action['domain']
         domain_subcontracting = [('id', 'in', (subcontracted_productions.move_raw_ids | subcontracted_productions.move_finished_ids).stock_valuation_layer_ids.ids)]
         domain = OR([domain, domain_subcontracting])
         return dict(action, domain=domain)
-
-    def _prepare_subcontract_mo_vals(self, subcontract_move, bom):
-        vals = super(StockPicking, self)._prepare_subcontract_mo_vals(subcontract_move, bom)
-        if bom.product_tmpl_id.cost_method in ('fifo', 'average'):
-            vals = dict(vals, extra_cost=subcontract_move._get_price_unit())
-        return vals
