@@ -5,8 +5,7 @@
     'name': 'Notes',
     'version': '1.0',
     'category': 'Productivity/Notes',
-    'description': "",
-    'website': 'https://www.odoo.com/page/notes',
+    'website': 'https://www.odoo.com/app/notes',
     'summary': 'Organize your work with memos',
     'sequence': 260,
     'depends': [
@@ -15,22 +14,27 @@
     'data': [
         'security/note_security.xml',
         'security/ir.model.access.csv',
-        'data/mail_activity_data.xml',
+        'data/mail_activity_type_data.xml',
         'data/note_data.xml',
         'data/res_users_data.xml',
         'views/note_views.xml',
-        'views/note_templates.xml',
-    ],
+        ],
     'demo': [
         'data/note_demo.xml',
     ],
-    'qweb': [
-        'static/src/xml/systray.xml',
-    ],
-    'test': [
-    ],
     'installable': True,
     'application': True,
-    'auto_install': False,
+    'assets': {
+        'mail.assets_messaging': [
+            'note/static/src/models/*.js',
+        ],
+        'web.assets_backend': [
+            'note/static/src/components/**/*',
+            'note/static/src/scss/note.scss',
+        ],
+        'web.qunit_suite_tests': [
+            'note/static/tests/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
