@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import payment_acquirer_onboarding_wizard
 from . import payment_link_wizard
+from . import payment_onboarding_wizard
