@@ -1,10 +1,14 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import models
 from . import controllers
-from odoo.addons.payment.models.payment_acquirer import create_missing_journal_for_acquirers
-from odoo.addons.payment import reset_payment_provider
+from . import models
+
+from odoo.addons.payment import setup_provider, reset_payment_provider
+
+
+def post_init_hook(cr, registry):
+    setup_provider(cr, registry, 'paypal')
+
 
 def uninstall_hook(cr, registry):
     reset_payment_provider(cr, registry, 'paypal')
