@@ -3,7 +3,7 @@
 
 {
     'name': 'Phone Numbers Validation',
-    'version': '2.0',
+    'version': '2.1',
     'summary': 'Validate and format phone numbers',
     'sequence': '9999',
     'category': 'Hidden',
@@ -17,12 +17,8 @@ according to a destination country.
 It also adds phone blacklist management through a specific model storing
 blacklisted phone numbers.
 
-It adds two mixins :
-
-  * phone.validation.mixin: parsing / formatting helpers on records, to be
-    used for example in number fields onchange;
-  * mail.thread.phone: handle sanitation and blacklist of records numbers;
-""",
+It adds mail.thread.phone mixin that handles sanitation and blacklist of
+records numbers. """,
     'data': [
         'security/ir.model.access.csv',
         'views/phone_blacklist_views.xml',
@@ -33,4 +29,5 @@ It adds two mixins :
         'mail',
     ],
     'auto_install': True,
+    'license': 'LGPL-3',
 }
