@@ -1,53 +1,65 @@
 odoo.define('point_of_sale.Chrome', function(require) {
     'use strict';
 
-    const { useState, useRef, useContext } = owl.hooks;
-    const { debounce } = owl.utils;
-    const { loadCSS } = require('web.ajax');
-    const { useListener } = require('web.custom_hooks');
-    const { CrashManager } = require('web.CrashManager');
-    const { BarcodeEvents } = require('barcodes.BarcodeEvents');
+    const { loadCSS } = require('@web/core/assets');
+    const { useListener } = require("@web/core/utils/hooks");
+    const BarcodeParser = require('barcodes.BarcodeParser');
     const PosComponent = require('point_of_sale.PosComponent');
     const NumberBuffer = require('point_of_sale.NumberBuffer');
-    const PopupControllerMixin = require('point_of_sale.PopupControllerMixin');
     const Registries = require('point_of_sale.Registries');
     const IndependentToOrderScreen = require('point_of_sale.IndependentToOrderScreen');
-    const contexts = require('point_of_sale.PosContext');
+    const { identifyError, batched } = require('point_of_sale.utils');
+    const { odooExceptionTitleMap } = require("@web/core/errors/error_dialogs");
+    const { ConnectionLostError, ConnectionAbortedError, RPCError } = require('@web/core/network/rpc_service');
+    const { useBus } = require("@web/core/utils/hooks");
+    const { debounce } = require("@web/core/utils/timing");
+    const { Transition } = require("@web/core/transition");
 
-    // This is kind of a trick.
-    // We get a reference to the whole exports so that
-    // when we create an instance of one of the classes,
-    // we instantiate the extended one.
-    const models = require('point_of_sale.models');
+    const {
+        onError,
+        onMounted,
+        onWillDestroy,
+        useExternalListener,
+        useRef,
+        useState,
+        useSubEnv,
+        reactive,
+    } = owl;
 
     /**
      * Chrome is the root component of the PoS App.
      */
-    class Chrome extends PopupControllerMixin(PosComponent) {
-        constructor() {
-            super(...arguments);
+    class Chrome extends PosComponent {
+        setup() {
+            super.setup();
+            useExternalListener(window, 'beforeunload', this._onBeforeUnload);
             useListener('show-main-screen', this.__showScreen);
             useListener('toggle-debug-widget', debounce(this._toggleDebugWidget, 100));
+            useListener('toggle-mobile-searchbar', this._toggleMobileSearchBar);
             useListener('show-temp-screen', this.__showTempScreen);
             useListener('close-temp-screen', this.__closeTempScreen);
             useListener('close-pos', this._closePos);
-            useListener('loading-skip-callback', () => this._loadingSkipCallback());
+            useListener('loading-skip-callback', () => this.env.proxy.stop_searching());
             useListener('play-sound', this._onPlaySound);
             useListener('set-sync-status', this._onSetSyncStatus);
+            useListener('show-notification', this._onShowNotification);
+            useListener('close-notification', this._onCloseNotification);
+            useListener('connect-to-proxy', this.connect_to_proxy);
+            useBus(this.env.posbus, 'start-cash-control', this.openCashControl);
             NumberBuffer.activate();
 
-            this.chromeContext = useContext(contexts.chrome);
-
             this.state = useState({
                 uiState: 'LOADING', // 'LOADING' | 'READY' | 'CLOSING'
                 debugWidgetIsShown: true,
+                mobileSearchBarIsShown: false,
                 hasBigScrollBars: false,
                 sound: { src: null },
-            });
-
-            this.loading = useState({
-                message: 'Loading',
-                skipButtonIsShown: false,
+                notification: {
+                    isShown: false,
+                    message: '',
+                    duration: 2000,
+                },
+                loadingSkipButtonIsShown: false,
             });
 
             this.mainScreen = useState({ name: null, component: null });
@@ -59,37 +71,45 @@ odoo.define('point_of_sale.Chrome', function(require) {
             this.progressbar = useRef('progressbar');
 
             this.previous_touch_y_coordinate = -1;
-        }
 
-        // OVERLOADED METHODS //
+            const pos = reactive(this.env.pos, batched(() => this.render(true)))
+            useSubEnv({ pos });
 
-        mounted() {
-            // remove default webclient handlers that induce click delay
-            $(document).off();
-            $(window).off();
-            $('html').off();
-            $('body').off();
-            // The above lines removed the bindings, but we really need them for the barcode
-            BarcodeEvents.start();
-        }
-        willUnmount() {
-            BarcodeEvents.stop();
-        }
-        destroy() {
-            super.destroy(...arguments);
-            this.env.pos.destroy();
-        }
-        catchError(error) {
-            console.error(error);
+            onMounted(() => {
+                // remove default webclient handlers that induce click delay
+                $(document).off();
+                $(window).off();
+                $('html').off();
+                $('body').off();
+            });
+
+            onWillDestroy(() => {
+                this.env.pos.destroy();
+            });
+
+            onError((error) => {
+                // error is an OwlError object.
+                console.error(error.cause);
+            });
+
+            this.props.setupIsDone(this);
         }
 
         // GETTERS //
 
-        get clientScreenButtonIsShown() {
-            return (
-                this.env.pos.config.use_proxy && this.env.pos.config.iface_customer_facing_display
-            );
+        get customerFacingDisplayButtonIsShown() {
+            return this.env.pos.config.iface_customer_facing_display;
         }
+
+        /**
+         * Used to give the `state.mobileSearchBarIsShown` value to main screen props
+         */
+        get mainScreenPropsFielded() {
+            return Object.assign({}, this.mainScreenProps, {
+                mobileSearchBarIsShown: this.state.mobileSearchBarIsShown,
+            });
+        }
+
         /**
          * Startup screen can be based on pos config so the startup screen
          * is only determined after pos data is completely loaded.
@@ -113,43 +133,22 @@ odoo.define('point_of_sale.Chrome', function(require) {
          */
         async start() {
             try {
-                // Instead of passing chrome to the instantiation the PosModel,
-                // we inject functions needed by pos.
-                // This way, we somehow decoupled Chrome from PosModel.
-                // We can then test PosModel independently from Chrome by supplying
-                // mocked version of these default attributes.
-                const posModelDefaultAttributes = {
-                    env: this.env,
-                    rpc: this.rpc.bind(this),
-                    session: this.env.session,
-                    do_action: this.props.webClient.do_action.bind(this.props.webClient),
-                    setLoadingMessage: this.setLoadingMessage.bind(this),
-                    showLoadingSkip: this.showLoadingSkip.bind(this),
-                    setLoadingProgress: this.setLoadingProgress.bind(this),
-                };
-                this.env.pos = new models.PosModel(posModelDefaultAttributes);
-                await this.env.pos.ready;
+                await this.env.pos.load_server_data();
+                await this.setupBarcodeParser();
+                if(this.env.pos.config.use_proxy){
+                    await this.connect_to_proxy();
+                }
+                // Load the saved `env.pos.toRefundLines` from localStorage when
+                // the PosGlobalState is ready.
+                Object.assign(this.env.pos.toRefundLines, this.env.pos.db.load('TO_REFUND_LINES') || {});
                 this._buildChrome();
                 this._closeOtherTabs();
-                this.env.pos.set(
-                    'selectedCategoryId',
-                    this.env.pos.config.iface_start_categ_id
-                        ? this.env.pos.config.iface_start_categ_id[0]
-                        : 0
-                );
+                this.env.pos.selectedCategoryId = this.env.pos.config.start_category && this.env.pos.config.iface_start_categ_id
+                    ? this.env.pos.config.iface_start_categ_id[0]
+                    : 0;
                 this.state.uiState = 'READY';
-                this.env.pos.on('change:selectedOrder', this._showSavedScreen, this);
                 this._showStartScreen();
-                if (_.isEmpty(this.env.pos.db.product_by_category_id)) {
-                    this._loadDemoData();
-                }
-                setTimeout(() => {
-                    // push order in the background, no need to await
-                    this.env.pos.push_orders();
-                    // Allow using the app even if not all the images are loaded.
-                    // Basically, preload the images in the background.
-                    this._preloadImages();
-                });
+                setTimeout(() => this._runBackgroundTasks());
             } catch (error) {
                 let title = 'Unknown Error',
                     body;
@@ -171,7 +170,11 @@ odoo.define('point_of_sale.Chrome', function(require) {
                     }
                 } else if (error instanceof Error) {
                     title = error.message;
-                    body = error.stack;
+                    if (error.cause) {
+                        body = error.cause.message;
+                    } else {
+                        body = error.stack;
+                    }
                 }
 
                 await this.showPopup('ErrorTracebackPopup', {
@@ -182,21 +185,87 @@ odoo.define('point_of_sale.Chrome', function(require) {
             }
         }
 
+        _runBackgroundTasks() {
+            // push order in the background, no need to await
+            this.env.pos.push_orders();
+            // Allow using the app even if not all the images are loaded.
+            // Basically, preload the images in the background.
+            this._preloadImages();
+            if (this.env.pos.config.limited_partners_loading && this.env.pos.config.partner_load_background) {
+                // Wrap in fresh reactive: none of the reads during loading should subscribe to anything
+                reactive(this.env.pos).loadPartnersBackground();
+            }
+            if (this.env.pos.config.limited_products_loading && this.env.pos.config.product_load_background) {
+                // Wrap in fresh reactive: none of the reads during loading should subscribe to anything
+                reactive(this.env.pos).loadProductsBackground().then(() => {
+                    this.render(true);
+                });
+            }
+        }
+
+        setupBarcodeParser() {
+            if (!this.env.pos.company.nomenclature_id) {
+                const errorMessage = this.env._t("The barcode nomenclature setting is not configured. " +
+                    "Make sure to configure it on your Point of Sale configuration settings");
+                throw new Error(this.env._t("Missing barcode nomenclature"), { cause: { message: errorMessage } });
+
+            }
+            const barcode_parser = new BarcodeParser({ nomenclature_id: this.env.pos.company.nomenclature_id });
+            this.env.barcode_reader.set_barcode_parser(barcode_parser);
+            return barcode_parser.is_loaded();
+        }
+
+        connect_to_proxy() {
+            return new Promise((resolve, reject) => {
+                this.env.barcode_reader.disconnect_from_proxy();
+                this.state.loadingSkipButtonIsShown = true;
+                this.env.proxy.autoconnect({
+                    force_ip: this.env.pos.config.proxy_ip || undefined,
+                    progress: function(prog){},
+                }).then(
+                    () => {
+                        if (this.env.pos.config.iface_scan_via_proxy) {
+                            this.env.barcode_reader.connect_to_proxy();
+                        }
+                        resolve();
+                    },
+                    (statusText, url) => {
+                        // this should reject so that it can be captured when we wait for pos.ready
+                        // in the chrome component.
+                        // then, if it got really rejected, we can show the error.
+                        if (statusText == 'error' && window.location.protocol == 'https:') {
+                            reject({
+                                title: this.env._t('HTTPS connection to IoT Box failed'),
+                                body: _.str.sprintf(
+                                    this.env._t('Make sure you are using IoT Box v18.12 or higher. Navigate to %s to accept the certificate of your IoT Box.'),
+                                    url
+                                ),
+                                popup: 'alert',
+                            });
+                        } else {
+                            resolve();
+                        }
+                    }
+                );
+            });
+        }
+
+        openCashControl() {
+            if (this.shouldShowCashControl()) {
+                this.showPopup('CashOpeningPopup');
+            }
+        }
+
+        shouldShowCashControl() {
+            return this.env.pos.config.cash_control && this.env.pos.pos_session.state == 'opening_control';
+        }
+
         // EVENT HANDLERS //
 
         _showStartScreen() {
             const { name, props } = this.startScreen;
             this.showScreen(name, props);
         }
-        /**
-         * Show the screen saved in the order when the `selectedOrder` of pos is changed.
-         * @param {models.PosModel} pos
-         * @param {models.Order} newSelectedOrder
-         */
-        _showSavedScreen(pos, newSelectedOrder) {
-            const { name, props } = this._getSavedScreen(newSelectedOrder);
-            this.showScreen(name, props);
-        }
         _getSavedScreen(order) {
             return order.get_screen_data();
         }
@@ -209,6 +278,7 @@ odoo.define('point_of_sale.Chrome', function(require) {
         }
         __closeTempScreen() {
             this.tempScreen.isShown = false;
+            this.tempScreen.name = null;
         }
         __showScreen({ detail: { name, props = {} } }) {
             const component = this.constructor.components[name];
@@ -217,10 +287,7 @@ odoo.define('point_of_sale.Chrome', function(require) {
             this.mainScreen.component = component;
             this.mainScreenProps = props;
 
-            // 2. Set some options
-            this.chromeContext.showOrderSelector = !component.hideOrderSelector;
-
-            // 3. Save the screen to the order.
+            // 2. Save the screen to the order.
             //  - This screen is shown when the order is selected.
             if (!(component.prototype instanceof IndependentToOrderScreen) && name !== "ReprintReceiptScreen") {
                 this._setScreenData(name, props);
@@ -256,7 +323,7 @@ odoo.define('point_of_sale.Chrome', function(require) {
                     window.location = '/web#action=point_of_sale.action_client_pos_menu';
                 } catch (error) {
                     console.warn(error);
-                    const reason = this.env.pos.get('failed')
+                    const reason = this.env.pos.failed
                         ? this.env._t(
                               'Some orders could not be submitted to ' +
                                   'the server due to configuration errors. ' +
@@ -277,8 +344,7 @@ odoo.define('point_of_sale.Chrome', function(require) {
                     });
                     if (confirmed) {
                         this.state.uiState = 'CLOSING';
-                        this.loading.skipButtonIsShown = false;
-                        this.setLoadingMessage(this.env._t('Closing ...'));
+                        this.state.loadingSkipButtonIsShown = false;
                         window.location = '/web#action=point_of_sale.action_client_pos_menu';
                     }
                 }
@@ -287,6 +353,13 @@ odoo.define('point_of_sale.Chrome', function(require) {
         _toggleDebugWidget() {
             this.state.debugWidgetIsShown = !this.state.debugWidgetIsShown;
         }
+        _toggleMobileSearchBar({ detail: isSearchBarEnabled }) {
+            if (isSearchBarEnabled !== null) {
+                this.state.mobileSearchBarIsShown = isSearchBarEnabled;
+            } else {
+                this.state.mobileSearchBarIsShown = !this.state.mobileSearchBarIsShown;
+            }
+        }
         _onPlaySound({ detail: name }) {
             let src;
             if (name === 'error') {
@@ -297,33 +370,23 @@ odoo.define('point_of_sale.Chrome', function(require) {
             this.state.sound.src = src;
         }
         _onSetSyncStatus({ detail: { status, pending }}) {
-            this.env.pos.set('synch', { status, pending });
+            this.env.pos.synch.status = status;
+            this.env.pos.synch.pending = pending;
         }
-
-        // TO PASS AS PARAMETERS //
-
-        setLoadingProgress(fac) {
-            if (this.progressbar.el) {
-                this.progressbar.el.style.width = `${Math.floor(fac * 100)}%`;
-            }
+        _onShowNotification({ detail: { message, duration } }) {
+            this.state.notification.isShown = true;
+            this.state.notification.message = message;
+            this.state.notification.duration = duration;
         }
-        setLoadingMessage(msg, progress) {
-            this.loading.message = msg;
-            if (typeof progress !== 'undefined') {
-                this.setLoadingProgress(progress);
-            }
+        _onCloseNotification() {
+            this.state.notification.isShown = false;
         }
         /**
-         * Show Skip button in the loading screen and allow to assign callback
-         * when the button is pressed.
-         *
-         * @param {Function} callback function to call when Skip button is pressed.
+         * Save `env.pos.toRefundLines` in localStorage on beforeunload - closing the
+         * browser, reloading or going to other page.
          */
-        showLoadingSkip(callback) {
-            if (callback) {
-                this.loading.skipButtonIsShown = true;
-                this._loadingSkipCallback = callback;
-            }
+        _onBeforeUnload() {
+            this.env.pos.db.save('TO_REFUND_LINES', this.env.pos.toRefundLines);
         }
 
         get isTicketScreenShown() {
@@ -331,31 +394,15 @@ odoo.define('point_of_sale.Chrome', function(require) {
         }
 
         // MISC METHODS //
-
-        async _loadDemoData() {
-            const { confirmed } = await this.showPopup('ConfirmPopup', {
-                title: this.env._t('You do not have any products'),
-                body: this.env._t(
-                    'Would you like to load demo data?'
-                ),
-            });
-            if (confirmed) {
-                await this.rpc({
-                    'route': '/pos/load_onboarding_data',
-                });
-                this.env.pos.load_server_data();
-            }
-        }
-
         _preloadImages() {
             for (let product of this.env.pos.db.get_product_by_category(0)) {
                 const image = new Image();
-                image.src = `/web/image?model=product.product&field=image_128&id=${product.id}&write_date=${product.write_date}&unique=1`;
+                image.src = `/web/image?model=product.product&field=image_128&id=${product.id}&unique=${product.__last_update}`;
             }
             for (let category of Object.values(this.env.pos.db.category_by_id)) {
                 if (category.id == 0) continue;
                 const image = new Image();
-                image.src = `/web/image?model=pos.category&field=image_128&id=${category.id}&write_date=${category.write_date}&unique=1`;
+                image.src = `/web/image?model=pos.category&field=image_128&id=${category.id}&unique=${category.write_date}`;
             }
             const staticImages = ['backspace.png', 'bc-arrow-big.png'];
             for (let imageName of staticImages) {
@@ -377,39 +424,6 @@ odoo.define('point_of_sale.Chrome', function(require) {
             }
 
             this._disableBackspaceBack();
-            this._replaceCrashmanager();
-        }
-        // replaces the error handling of the existing crashmanager which
-        // uses jquery dialog to display the error, to use the pos popup
-        // instead
-        _replaceCrashmanager() {
-            var self = this;
-            CrashManager.include({
-                show_warning: function (error) {
-                    if (self.env.pos) {
-                        // self == this component
-                        self.showPopup('ErrorPopup', {
-                            title: error.data.title.toString(),
-                            body: error.data.message,
-                        });
-                    } else {
-                        // this == CrashManager instance
-                        this._super(error);
-                    }
-                },
-                show_error: function (error) {
-                    if (self.env.pos) {
-                        // self == this component
-                        self.showPopup('ErrorTracebackPopup', {
-                            title: error.type,
-                            body: error.message + '\n' + error.data.debug + '\n',
-                        });
-                    } else {
-                        // this == CrashManager instance
-                        this._super(error);
-                    }
-                },
-            });
         }
         // prevent backspace from performing a 'back' navigation
         _disableBackspaceBack() {
@@ -445,8 +459,80 @@ odoo.define('point_of_sale.Chrome', function(require) {
                 false
             );
         }
+        showCashMoveButton() {
+            return this.env.pos && this.env.pos.config && this.env.pos.config.cash_control;
+        }
+
+        // UNEXPECTED ERROR HANDLING //
+
+        /**
+         * This method is used to handle unexpected errors. It is registered to
+         * the `error_handlers` service when this component is properly mounted.
+         * See `onMounted` hook of the `ChromeAdapter` component.
+         * @param {*} env
+         * @param {UncaughtClientError | UncaughtPromiseError} error
+         * @param {*} originalError
+         * @returns {boolean}
+         */
+        errorHandler(env, error, originalError) {
+            if (!env.pos) return false;
+            const errorToHandle = identifyError(originalError);
+            // Assume that the unhandled falsey rejections can be ignored.
+            if (errorToHandle) {
+                this._errorHandler(error, errorToHandle);
+            }
+            return true;
+        }
+
+        _errorHandler(error, errorToHandle) {
+            if (errorToHandle instanceof RPCError) {
+                const { message, data } = errorToHandle;
+                if (odooExceptionTitleMap.has(errorToHandle.exceptionName)) {
+                    const title = odooExceptionTitleMap.get(errorToHandle.exceptionName).toString();
+                    this.showPopup('ErrorPopup', { title, body: data.message });
+                } else {
+                    this.showPopup('ErrorTracebackPopup', {
+                        title: message,
+                        body: data.message + '\n' + data.debug + '\n',
+                    });
+                }
+            } else if (errorToHandle instanceof ConnectionLostError) {
+                this.showPopup('OfflineErrorPopup', {
+                    title: this.env._t('Connection is lost'),
+                    body: this.env._t('Check the internet connection then try again.'),
+                });
+            } else if (errorToHandle instanceof ConnectionAbortedError) {
+                this.showPopup('OfflineErrorPopup', {
+                    title: this.env._t('Connection is aborted'),
+                    body: this.env._t('Check the internet connection then try again.'),
+                });
+            } else if (errorToHandle instanceof Error) {
+                // If `errorToHandle` is a normal Error (such as TypeError),
+                // the annotated traceback can be found from `error`.
+                this.showPopup('ErrorTracebackPopup', {
+                    // Hopefully the message is translated.
+                    title: `${errorToHandle.name}: ${errorToHandle.message}`,
+                    body: error.traceback,
+                });
+            } else {
+                // Hey developer. It's your fault that the error reach here.
+                // Please, throw an Error object in order to get stack trace of the error.
+                // At least we can find the file that throws the error when you look
+                // at the console.
+                this.showPopup('ErrorPopup', {
+                    title: this.env._t('Unknown Error'),
+                    body: this.env._t('Unable to show information about this error.'),
+                });
+                console.error('Unknown error. Unable to show information about this error.', errorToHandle);
+            }
+        }
     }
     Chrome.template = 'Chrome';
+    Object.defineProperty(Chrome, "components", {
+        get () {
+            return Object.assign({ Transition }, PosComponent.components);
+        }
+    })
 
     Registries.Component.add(Chrome);
 
