@@ -1,18 +1,21 @@
 odoo.define('point_of_sale.TextInputPopup', function(require) {
     'use strict';
 
-    const { useState, useRef } = owl.hooks;
     const AbstractAwaitablePopup = require('point_of_sale.AbstractAwaitablePopup');
     const Registries = require('point_of_sale.Registries');
+    const { _lt } = require('@web/core/l10n/translation');
+
+    const { onMounted, useRef, useState } = owl;
 
     // formerly TextInputPopupWidget
     class TextInputPopup extends AbstractAwaitablePopup {
-        constructor() {
-            super(...arguments);
+        setup() {
+            super.setup();
             this.state = useState({ inputValue: this.props.startingValue });
             this.inputRef = useRef('input');
+            onMounted(this.onMounted);
         }
-        mounted() {
+        onMounted() {
             this.inputRef.el.focus();
         }
         getPayload() {
@@ -21,11 +24,12 @@ odoo.define('point_of_sale.TextInputPopup', function(require) {
     }
     TextInputPopup.template = 'TextInputPopup';
     TextInputPopup.defaultProps = {
-        confirmText: 'Ok',
-        cancelText: 'Cancel',
+        confirmText: _lt('Ok'),
+        cancelText: _lt('Cancel'),
         title: '',
         body: '',
         startingValue: '',
+        placeholder: '',
     };
 
     Registries.Component.add(TextInputPopup);
