@@ -3,15 +3,16 @@ odoo.define('point_of_sale.PaymentScreen', function (require) {
 
     const { parse } = require('web.field_utils');
     const PosComponent = require('point_of_sale.PosComponent');
-    const { useErrorHandlers } = require('point_of_sale.custom_hooks');
+    const { useErrorHandlers, useAsyncLockedMethod } = require('point_of_sale.custom_hooks');
     const NumberBuffer = require('point_of_sale.NumberBuffer');
-    const { useListener } = require('web.custom_hooks');
+    const { useListener } = require("@web/core/utils/hooks");
     const Registries = require('point_of_sale.Registries');
-    const { onChangeOrder } = require('point_of_sale.custom_hooks');
+    const { isConnectionError } = require('point_of_sale.utils');
+    const utils = require('web.utils');
 
     class PaymentScreen extends PosComponent {
-        constructor() {
-            super(...arguments);
+        setup() {
+            super.setup();
             useListener('delete-payment-line', this.deletePaymentLine);
             useListener('select-payment-line', this.selectPaymentLine);
             useListener('new-payment-line', this.addNewPaymentLine);
@@ -20,19 +21,41 @@ odoo.define('point_of_sale.PaymentScreen', function (require) {
             useListener('send-payment-cancel', this._sendPaymentCancel);
             useListener('send-payment-reverse', this._sendPaymentReverse);
             useListener('send-force-done', this._sendForceDone);
-            NumberBuffer.use({
+            useListener('validate-order', () => this.validateOrder(false));
+            this.payment_methods_from_config = this.env.pos.payment_methods.filter(method => this.env.pos.config.payment_method_ids.includes(method.id));
+            NumberBuffer.use(this._getNumberBufferConfig);
+            useErrorHandlers();
+            this.payment_interface = null;
+            this.error = false;
+            this.validateOrder = useAsyncLockedMethod(this.validateOrder);
+        }
+
+        showMaxValueError() {
+            this.showPopup('ErrorPopup', {
+                title: this.env._t('Maximum value reached'),
+                body: this.env._t('The amount cannot be higher than the due amount if you don\'t have a cash payment method configured.')
+            });
+        }
+        get _getNumberBufferConfig() {
+            let config = {
                 // The numberBuffer listens to this event to update its state.
                 // Basically means 'update the buffer when this event is triggered'
                 nonKeyboardInputEvent: 'input-from-numpad',
                 // When the buffer is updated, trigger this event.
                 // Note that the component listens to it.
                 triggerAtInput: 'update-selected-paymentline',
-            });
-            onChangeOrder(this._onPrevOrder, this._onNewOrder);
-            useErrorHandlers();
-            this.payment_interface = null;
-            this.error = false;
-            this.payment_methods_from_config = this.env.pos.payment_methods.filter(method => this.env.pos.config.payment_method_ids.includes(method.id));
+            };
+            // Check if pos has a cash payment method
+            const hasCashPaymentMethod = this.payment_methods_from_config.some(
+                (method) => method.type === 'cash'
+            );
+
+            if (!hasCashPaymentMethod) {
+                config['maxValue'] = this.currentOrder.get_due();
+                config['maxValueReached'] = this.showMaxValueError.bind(this);
+            }
+
+            return config;
         }
         get currentOrder() {
             return this.env.pos.get_order();
@@ -43,35 +66,32 @@ odoo.define('point_of_sale.PaymentScreen', function (require) {
         get selectedPaymentLine() {
             return this.currentOrder.selected_paymentline;
         }
-        async selectClient() {
+        async selectPartner() {
             // IMPROVEMENT: This code snippet is repeated multiple times.
             // Maybe it's better to create a function for it.
-            const currentClient = this.currentOrder.get_client();
-            const { confirmed, payload: newClient } = await this.showTempScreen(
-                'ClientListScreen',
-                { client: currentClient }
+            const currentPartner = this.currentOrder.get_partner();
+            const { confirmed, payload: newPartner } = await this.showTempScreen(
+                'PartnerListScreen',
+                { partner: currentPartner }
             );
             if (confirmed) {
-                this.currentOrder.set_client(newClient);
-                this.currentOrder.updatePricelist(newClient);
+                this.currentOrder.set_partner(newPartner);
+                this.currentOrder.updatePricelist(newPartner);
             }
         }
         addNewPaymentLine({ detail: paymentMethod }) {
             // original function: click_paymentmethods
-            if (this.currentOrder.electronic_payment_in_progress()) {
+            let result = this.currentOrder.add_paymentline(paymentMethod);
+            if (result){
+                NumberBuffer.reset();
+                return true;
+            }
+            else{
                 this.showPopup('ErrorPopup', {
                     title: this.env._t('Error'),
                     body: this.env._t('There is already an electronic payment in progress.'),
                 });
                 return false;
-            } else {
-                this.currentOrder.add_paymentline(paymentMethod);
-                NumberBuffer.reset();
-                this.payment_interface = paymentMethod.payment_terminal;
-                if (this.payment_interface) {
-                    this.currentOrder.selected_paymentline.set_payment_status('pending');
-                }
-                return true;
             }
         }
         _updateSelectedPaymentline() {
@@ -80,8 +100,9 @@ odoo.define('point_of_sale.PaymentScreen', function (require) {
             }
             if (!this.selectedPaymentLine) return; // do nothing if no selected payment line
             // disable changing amount on paymentlines with running or done payments on a payment terminal
+            const payment_terminal = this.selectedPaymentLine.payment_method.payment_terminal;
             if (
-                this.payment_interface &&
+                payment_terminal &&
                 !['pending', 'retry'].includes(this.selectedPaymentLine.get_payment_status())
             ) {
                 return;
@@ -95,30 +116,32 @@ odoo.define('point_of_sale.PaymentScreen', function (require) {
         toggleIsToInvoice() {
             // click_invoice
             this.currentOrder.set_to_invoice(!this.currentOrder.is_to_invoice());
-            this.render();
+            this.render(true);
         }
         openCashbox() {
-            this.env.pos.proxy.printer.open_cashbox();
+            this.env.proxy.printer.open_cashbox();
         }
         async addTip() {
             // click_tip
             const tip = this.currentOrder.get_tip();
             const change = this.currentOrder.get_change();
-            let value = tip.toFixed(this.env.pos.decimals);
-
-            if (tip === 0 && change > 0) {
-                value = change;
-            }
+            let value = tip === 0 && change > 0 ? change : tip;
 
             const { confirmed, payload } = await this.showPopup('NumberPopup', {
                 title: tip ? this.env._t('Change Tip') : this.env._t('Add Tip'),
                 startingValue: value,
+                isInputSelected: true,
             });
 
             if (confirmed) {
                 this.currentOrder.set_tip(parse.float(payload));
             }
         }
+        toggleIsToShip() {
+            // click_ship
+            this.currentOrder.set_to_ship(!this.currentOrder.is_to_ship());
+            this.render(true);
+        }
         deletePaymentLine(event) {
             var self = this;
             const { cid } = event.detail;
@@ -132,13 +155,13 @@ odoo.define('point_of_sale.PaymentScreen', function (require) {
                 line.payment_method.payment_terminal.send_payment_cancel(this.currentOrder, cid).then(function() {
                     self.currentOrder.remove_paymentline(line);
                     NumberBuffer.reset();
-                    self.render();
+                    self.render(true);
                 })
             }
             else if (line.get_payment_status() !== 'waitingCancel') {
                 this.currentOrder.remove_paymentline(line);
                 NumberBuffer.reset();
-                this.render();
+                this.render(true);
             }
         }
         selectPaymentLine(event) {
@@ -146,7 +169,7 @@ odoo.define('point_of_sale.PaymentScreen', function (require) {
             const line = this.paymentLines.find((line) => line.cid === cid);
             this.currentOrder.select_paymentline(line);
             NumberBuffer.reset();
-            this.render();
+            this.render(true);
         }
         async validateOrder(isForceValidate) {
             if(this.env.pos.config.cash_rounding) {
@@ -167,92 +190,180 @@ odoo.define('point_of_sale.PaymentScreen', function (require) {
             }
         }
         async _finalizeValidation() {
-            if ((this.currentOrder.is_paid_with_cash() || this.currentOrder.get_change()) && this.env.pos.config.iface_cashdrawer) {
-                this.env.pos.proxy.printer.open_cashbox();
+            if ((this.currentOrder.is_paid_with_cash() || this.currentOrder.get_change()) && this.env.pos.config.iface_cashdrawer && this.env.proxy && this.env.proxy.printer) {
+                this.env.proxy.printer.open_cashbox();
             }
 
             this.currentOrder.initialize_validation_date();
+            for (let line of this.paymentLines) {
+                if (!line.amount === 0) {
+                     this.currentOrder.remove_paymentline(line);
+                }
+            }
             this.currentOrder.finalized = true;
 
-            let syncedOrderBackendIds = [];
+            let syncOrderResult, hasError;
 
             try {
-                if (this.currentOrder.is_to_invoice()) {
-                    syncedOrderBackendIds = await this.env.pos.push_and_invoice_order(
-                        this.currentOrder
+                // 1. Save order to server.
+                syncOrderResult = await this.env.pos.push_single_order(this.currentOrder);
+
+                // 2. Invoice.
+                if (this.shouldDownloadInvoice() && this.currentOrder.is_to_invoice()) {
+                    if (syncOrderResult.length) {
+                        await this.env.legacyActionManager.do_action('account.account_invoices', {
+                            additional_context: {
+                                active_ids: [syncOrderResult[0].account_move],
+                            },
+                        });
+                    } else {
+                        throw { code: 401, message: 'Backend Invoice', data: { order: this.currentOrder } };
+                    }
+                }
+
+                // 3. Post process.
+                if (syncOrderResult.length && this.currentOrder.wait_for_push_order()) {
+                    const postPushResult = await this._postPushOrderResolve(
+                        this.currentOrder,
+                        syncOrderResult.map((res) => res.id)
                     );
-                } else {
-                    syncedOrderBackendIds = await this.env.pos.push_single_order(this.currentOrder);
+                    if (!postPushResult) {
+                        this.showPopup('ErrorPopup', {
+                            title: this.env._t('Error: no internet connection.'),
+                            body: this.env._t('Some, if not all, post-processing after syncing order failed.'),
+                        });
+                    }
                 }
             } catch (error) {
-                if (error.code == 700)
+                if (error.code == 700 || error.code == 701)
                     this.error = true;
-                if (error instanceof Error) {
-                    throw error;
-                } else {
+
+                if ('code' in error) {
+                    // We started putting `code` in the rejected object for invoicing error.
+                    // We can continue with that convention such that when the error has `code`,
+                    // then it is an error when invoicing. Besides, _handlePushOrderError was
+                    // introduce to handle invoicing error logic.
                     await this._handlePushOrderError(error);
+                } else {
+                    // We don't block for connection error. But we rethrow for any other errors.
+                    if (isConnectionError(error)) {
+                        this.showPopup('OfflineErrorPopup', {
+                            title: this.env._t('Connection Error'),
+                            body: this.env._t('Order is not synced. Check your internet connection'),
+                        });
+                    } else {
+                        throw error;
+                    }
                 }
-            }
-            if (syncedOrderBackendIds.length && this.currentOrder.wait_for_push_order()) {
-                const result = await this._postPushOrderResolve(
-                    this.currentOrder,
-                    syncedOrderBackendIds
-                );
-                if (!result) {
-                    await this.showPopup('ErrorPopup', {
-                        title: 'Error: no internet connection.',
-                        body: error,
+            } finally {
+                // Always show the next screen regardless of error since pos has to
+                // continue working even offline.
+                this.showScreen(this.nextScreen);
+                // Remove the order from the local storage so that when we refresh the page, the order
+                // won't be there
+                this.env.pos.db.remove_unpaid_order(this.currentOrder);
+
+                // Ask the user to sync the remaining unsynced orders.
+                if (!hasError && syncOrderResult && this.env.pos.db.get_orders().length) {
+                    const { confirmed } = await this.showPopup('ConfirmPopup', {
+                        title: this.env._t('Remaining unsynced orders'),
+                        body: this.env._t(
+                            'There are unsynced orders. Do you want to sync these orders?'
+                        ),
                     });
+                    if (confirmed) {
+                        // NOTE: Not yet sure if this should be awaited or not.
+                        // If awaited, some operations like changing screen
+                        // might not work.
+                        this.env.pos.push_orders();
+                    }
                 }
             }
-
-            this.showScreen(this.nextScreen);
-
-            // If we succeeded in syncing the current order, and
-            // there are still other orders that are left unsynced,
-            // we ask the user if he is willing to wait and sync them.
-            if (syncedOrderBackendIds.length && this.env.pos.db.get_orders().length) {
-                const { confirmed } = await this.showPopup('ConfirmPopup', {
-                    title: this.env._t('Remaining unsynced orders'),
-                    body: this.env._t(
-                        'There are unsynced orders. Do you want to sync these orders?'
-                    ),
-                });
-                if (confirmed) {
-                    // NOTE: Not yet sure if this should be awaited or not.
-                    // If awaited, some operations like changing screen
-                    // might not work.
-                    this.env.pos.push_orders();
-                }
-            }
+        }
+        /**
+         * This method is meant to be overriden by localization that do not want to print the invoice pdf
+         * every time they create an account move. For example, it can be overriden like this:
+         * ```
+         * shouldDownloadInvoice() {
+         *     const currentCountry = ...
+         *     if (currentCountry.code === 'FR') {
+         *         return false;
+         *     } else {
+         *         return super.shouldDownloadInvoice(); // or this._super(...arguments) depending on the odoo version.
+         *     }
+         * }
+         * ```
+         * @returns {boolean} true if the invoice pdf should be downloaded
+         */
+        shouldDownloadInvoice() {
+            return true;
         }
         get nextScreen() {
             return !this.error? 'ReceiptScreen' : 'ProductScreen';
         }
         async _isOrderValid(isForceValidate) {
-            if (this.currentOrder.get_orderlines().length === 0) {
+            if (this.currentOrder.get_orderlines().length === 0 && this.currentOrder.is_to_invoice()) {
                 this.showPopup('ErrorPopup', {
                     title: this.env._t('Empty Order'),
                     body: this.env._t(
-                        'There must be at least one product in your order before it can be validated'
+                        'There must be at least one product in your order before it can be validated and invoiced.'
                     ),
                 });
                 return false;
             }
 
-            if (this.currentOrder.is_to_invoice() && !this.currentOrder.get_client()) {
+            if (this.currentOrder.electronic_payment_in_progress()) {
+                this.showPopup('ErrorPopup', {
+                    title: this.env._t('Pending Electronic Payments'),
+                    body: this.env._t(
+                        'There is at least one pending electronic payment.\n' +
+                        'Please finish the payment with the terminal or ' +
+                        'cancel it then remove the payment line.'
+                    ),
+                });
+                return false;
+            }
+
+            const splitPayments = this.paymentLines.filter(payment => payment.payment_method.split_transactions)
+            if (splitPayments.length && !this.currentOrder.get_partner()) {
+                const paymentMethod = splitPayments[0].payment_method
+                const { confirmed } = await this.showPopup('ConfirmPopup', {
+                    title: this.env._t('Customer Required'),
+                    body: _.str.sprintf(this.env._t('Customer is required for %s payment method.'), paymentMethod.name),
+                });
+                if (confirmed) {
+                    this.selectPartner();
+                }
+                return false;
+            }
+
+            if ((this.currentOrder.is_to_invoice() || this.currentOrder.is_to_ship()) && !this.currentOrder.get_partner()) {
                 const { confirmed } = await this.showPopup('ConfirmPopup', {
                     title: this.env._t('Please select the Customer'),
                     body: this.env._t(
-                        'You need to select the customer before you can invoice an order.'
+                        'You need to select the customer before you can invoice or ship an order.'
                     ),
                 });
                 if (confirmed) {
-                    this.selectClient();
+                    this.selectPartner();
                 }
                 return false;
             }
 
+            let partner = this.currentOrder.get_partner()
+            if (this.currentOrder.is_to_ship() && !(partner.name && partner.street && partner.city && partner.country_id)) {
+                this.showPopup('ErrorPopup', {
+                    title: this.env._t('Incorrect address for shipping'),
+                    body: this.env._t('The selected customer needs an address.'),
+                });
+                return false;
+            }
+
+            if (this.currentOrder.get_total_with_tax() != 0 && this.currentOrder.get_paymentlines().length === 0) {
+                this.showNotification(this.env._t('Select a payment method to validate the order.'));
+                return false;
+            }
+
             if (!this.currentOrder.is_paid() || this.invoicing) {
                 return false;
             }
@@ -313,6 +424,8 @@ odoo.define('point_of_sale.PaymentScreen', function (require) {
                 return false;
             }
 
+            if (!this.currentOrder._isValidEmptyOrder()) return false;
+
             return true;
         }
         async _postPushOrderResolve(order, order_server_ids) {
@@ -331,6 +444,14 @@ odoo.define('point_of_sale.PaymentScreen', function (require) {
             if (isPaymentSuccessful) {
                 line.set_payment_status('done');
                 line.can_be_reversed = payment_terminal.supports_reversals;
+                // Automatically validate the order when after an electronic payment,
+                // the current order is fully paid and due is zero.
+                if (
+                    this.currentOrder.is_paid() &&
+                    utils.float_is_zero(this.currentOrder.get_due(), this.env.pos.currency.decimal_places)
+                ) {
+                    this.trigger('validate-order');
+                }
             } else {
                 line.set_payment_status('retry');
             }
@@ -361,19 +482,6 @@ odoo.define('point_of_sale.PaymentScreen', function (require) {
         async _sendForceDone({ detail: line }) {
             line.set_payment_status('done');
         }
-        _onPrevOrder(prevOrder) {
-            prevOrder.off('change', null, this);
-            prevOrder.paymentlines.off('change', null, this);
-            if (prevOrder) {
-                prevOrder.stop_electronic_payment();
-            }
-        }
-        async _onNewOrder(newOrder) {
-            newOrder.on('change', this.render, this);
-            newOrder.paymentlines.on('change', this.render, this);
-            NumberBuffer.reset();
-            await this.render();
-        }
     }
     PaymentScreen.template = 'PaymentScreen';
 
