@@ -4,44 +4,50 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
     const PosComponent = require('point_of_sale.PosComponent');
     const ControlButtonsMixin = require('point_of_sale.ControlButtonsMixin');
     const NumberBuffer = require('point_of_sale.NumberBuffer');
-    const { useListener } = require('web.custom_hooks');
+    const { useListener } = require("@web/core/utils/hooks");
     const Registries = require('point_of_sale.Registries');
-    const { onChangeOrder, useBarcodeReader } = require('point_of_sale.custom_hooks');
-    const { useState } = owl.hooks;
+    const { useBarcodeReader } = require('point_of_sale.custom_hooks');
+    const { isConnectionError } = require('point_of_sale.utils');
     const { parse } = require('web.field_utils');
+    const { _lt } = require('@web/core/l10n/translation');
+
+    const { onMounted, useState } = owl;
 
     class ProductScreen extends ControlButtonsMixin(PosComponent) {
-        constructor() {
-            super(...arguments);
+        setup() {
+            super.setup();
             useListener('update-selected-orderline', this._updateSelectedOrderline);
-            useListener('new-orderline-selected', this._newOrderlineSelected);
+            useListener('select-line', this._selectLine);
             useListener('set-numpad-mode', this._setNumpadMode);
             useListener('click-product', this._clickProduct);
-            useListener('click-customer', this._onClickCustomer);
+            useListener('click-partner', this.onClickPartner);
             useListener('click-pay', this._onClickPay);
             useBarcodeReader({
                 product: this._barcodeProductAction,
+                quantity: this._barcodeProductAction,
                 weight: this._barcodeProductAction,
                 price: this._barcodeProductAction,
-                client: this._barcodeClientAction,
+                client: this._barcodePartnerAction,
                 discount: this._barcodeDiscountAction,
                 error: this._barcodeErrorAction,
-            })
-            onChangeOrder(null, (newOrder) => newOrder && this.render());
+                gs1: this._barcodeGS1Action,
+            });
             NumberBuffer.use({
                 nonKeyboardInputEvent: 'numpad-click-input',
                 triggerAtInput: 'update-selected-orderline',
                 useWithBarcode: true,
             });
-            let status = this.showCashBoxOpening()
-            this.state = useState({ cashControl: status, numpadMode: 'quantity' });
-            this.mobile_pane = this.props.mobile_pane || 'right';
+            onMounted(this.onMounted);
+            // Call `reset` when the `onMounted` callback in `NumberBuffer.use` is done.
+            // We don't do this in the `mounted` lifecycle method because it is called before
+            // the callbacks in `onMounted` hook.
+            onMounted(() => NumberBuffer.reset());
+            this.state = useState({
+                mobile_pane: this.props.mobile_pane || 'right',
+            });
         }
-        mounted() {
-            this.env.pos.on('change:selectedClient', this.render, this);
-        }
-        willUnmount() {
-            this.env.pos.off('change:selectedClient', null, this);
+        onMounted() {
+            this.env.posbus.trigger('start-cash-control');
         }
         /**
          * To be overridden by modules that checks availability of
@@ -51,22 +57,17 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
         get isScaleAvailable() {
             return true;
         }
-        get client() {
-            return this.env.pos.get_client();
+        get partner() {
+            return this.currentOrder ? this.currentOrder.get_partner() : null;
         }
         get currentOrder() {
             return this.env.pos.get_order();
         }
-        showCashBoxOpening() {
-            if(this.env.pos.config.cash_control && this.env.pos.pos_session.state == 'opening_control')
-                return true;
-            return false;
-        }
-        async _getAddProductOptions(product) {
+        async _getAddProductOptions(product, code) {
             let price_extra = 0.0;
             let draftPackLotLines, weight, description, packLotLinesToEdit;
 
-            if (this.env.pos.config.product_configurator && _.some(product.attribute_line_ids, (id) => id in this.env.pos.attributes_by_ptal_id)) {
+            if (_.some(product.attribute_line_ids, (id) => id in this.env.pos.attributes_by_ptal_id)) {
                 let attributes = _.map(product.attribute_line_ids, (id) => this.env.pos.attributes_by_ptal_id[id])
                                   .filter((attr) => attr !== undefined);
                 let { confirmed, payload } = await this.showPopup('ProductConfiguratorPopup', {
@@ -98,24 +99,36 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
                         packLotLinesToEdit = [];
                     }
                 }
-                const { confirmed, payload } = await this.showPopup('EditListPopup', {
-                    title: this.env._t('Lot/Serial Number(s) Required'),
-                    isSingleItem: isAllowOnlyOneLot,
-                    array: packLotLinesToEdit,
-                });
-                if (confirmed) {
-                    // Segregate the old and new packlot lines
+                // if the lot information exists in the barcode, we don't need to ask it from the user.
+                if (code && code.type === 'lot') {
+                    // consider the old and new packlot lines
                     const modifiedPackLotLines = Object.fromEntries(
-                        payload.newArray.filter(item => item.id).map(item => [item.id, item.text])
+                        packLotLinesToEdit.filter(item => item.id).map(item => [item.id, item.text])
                     );
-                    const newPackLotLines = payload.newArray
-                        .filter(item => !item.id)
-                        .map(item => ({ lot_name: item.text }));
-
+                    const newPackLotLines = [
+                        { lot_name: code.code },
+                    ];
                     draftPackLotLines = { modifiedPackLotLines, newPackLotLines };
                 } else {
-                    // We don't proceed on adding product.
-                    return;
+                    const { confirmed, payload } = await this.showPopup('EditListPopup', {
+                        title: this.env._t('Lot/Serial Number(s) Required'),
+                        isSingleItem: isAllowOnlyOneLot,
+                        array: packLotLinesToEdit,
+                    });
+                    if (confirmed) {
+                        // Segregate the old and new packlot lines
+                        const modifiedPackLotLines = Object.fromEntries(
+                            payload.newArray.filter(item => item.id).map(item => [item.id, item.text])
+                        );
+                        const newPackLotLines = payload.newArray
+                            .filter(item => !item.id)
+                            .map(item => ({ lot_name: item.text }));
+
+                        draftPackLotLines = { modifiedPackLotLines, newPackLotLines };
+                    } else {
+                        // We don't proceed on adding product.
+                        return;
+                    }
                 }
             }
 
@@ -137,8 +150,15 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
                 }
             }
 
+            if (code && this.env.pos.db.product_packaging_by_barcode[code.code]) {
+                weight = this.env.pos.db.product_packaging_by_barcode[code.code].qty;
+            }
+
             return { draftPackLotLines, quantity: weight, description, price_extra };
         }
+        async _addProduct(product, options) {
+            this.currentOrder.add_product(product, options);
+        }
         async _clickProduct(event) {
             if (!this.currentOrder) {
                 this.env.pos.add_new_order();
@@ -148,20 +168,42 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
             // Do not add product if options is undefined.
             if (!options) return;
             // Add the product after having the extra information.
-            this.currentOrder.add_product(product, options);
+            await this._addProduct(product, options);
             NumberBuffer.reset();
         }
         _setNumpadMode(event) {
             const { mode } = event.detail;
             NumberBuffer.capture();
             NumberBuffer.reset();
-            this.state.numpadMode = mode;
+            this.env.pos.numpadMode = mode;
+        }
+        _selectLine() {
+            NumberBuffer.reset();
         }
         async _updateSelectedOrderline(event) {
-            if(this.state.numpadMode === 'quantity' && this.env.pos.disallowLineQuantityChange()) {
-                let order = this.env.pos.get_order();
-                let selectedLine = order.get_selected_orderline();
-                let lastId = order.orderlines.last().cid;
+            const order = this.env.pos.get_order();
+            const selectedLine = order.get_selected_orderline();
+            // This validation must not be affected by `disallowLineQuantityChange`
+            if (selectedLine && selectedLine.isTipLine() && this.env.pos.numpadMode !== "price") {
+                /**
+                 * You can actually type numbers from your keyboard, while a popup is shown, causing
+                 * the number buffer storage to be filled up with the data typed. So we force the
+                 * clean-up of that buffer whenever we detect this illegal action.
+                 */
+                NumberBuffer.reset();
+                if (event.detail.key === "Backspace") {
+                    this._setValue("remove");
+                } else {
+                    this.showPopup("ErrorPopup", {
+                        title: this.env._t("Cannot modify a tip"),
+                        body: this.env._t("Customer tips, cannot be modified directly"),
+                    });
+                }
+            } else if (this.env.pos.numpadMode === 'quantity' && this.env.pos.disallowLineQuantityChange()) {
+                if(!order.orderlines.length)
+                    return;
+                let orderlines = order.orderlines;
+                let lastId = orderlines.length !== 0 && orderlines.at(orderlines.length - 1).cid;
                 let currentQuantity = this.env.pos.get_order().get_selected_orderline().get_quantity();
 
                 if(selectedLine.noDecrease) {
@@ -182,33 +224,70 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
                 let { buffer } = event.detail;
                 let val = buffer === null ? 'remove' : buffer;
                 this._setValue(val);
+                if (val == 'remove') {
+                    NumberBuffer.reset();
+                    this.env.pos.numpadMode = 'quantity';
+                }
             }
         }
-        async _newOrderlineSelected() {
-            NumberBuffer.reset();
-        }
         _setValue(val) {
             if (this.currentOrder.get_selected_orderline()) {
-                if (this.state.numpadMode === 'quantity') {
-                    this.currentOrder.get_selected_orderline().set_quantity(val);
-                } else if (this.state.numpadMode === 'discount') {
+                if (this.env.pos.numpadMode === 'quantity') {
+                    const result = this.currentOrder.get_selected_orderline().set_quantity(val);
+                    if (!result) NumberBuffer.reset();
+                } else if (this.env.pos.numpadMode === 'discount') {
                     this.currentOrder.get_selected_orderline().set_discount(val);
-                } else if (this.state.numpadMode === 'price') {
+                } else if (this.env.pos.numpadMode === 'price') {
                     var selected_orderline = this.currentOrder.get_selected_orderline();
                     selected_orderline.price_manually_set = true;
                     selected_orderline.set_unit_price(val);
                 }
-                if (this.env.pos.config.iface_customer_facing_display) {
-                    this.env.pos.send_current_order_to_customer_facing_display();
-                }
             }
         }
-        async _barcodeProductAction(code) {
-            const product = this.env.pos.db.get_product_by_barcode(code.base_code)
+        async _getProductByBarcode(code) {
+            let product = this.env.pos.db.get_product_by_barcode(code.base_code);
             if (!product) {
-                return this._barcodeErrorAction(code);
+                // find the barcode in the backend
+                let foundProductIds = [];
+                const foundPackagings = [];
+                try {
+                    const { product_id = [], packaging = [] } = await this.rpc({
+                        model: 'pos.session',
+                        method: 'find_product_by_barcode',
+                        args: [odoo.pos_session_id, code.base_code],
+                        context: this.env.session.user_context,
+                    });
+                    foundProductIds.push(...product_id);
+                    foundPackagings.push(...packaging);
+                } catch (error) {
+                    if (isConnectionError(error)) {
+                        return this.showPopup('OfflineErrorPopup', {
+                            title: this.env._t('Network Error'),
+                            body: this.env._t("Product is not loaded. Tried loading the product from the server but there is a network error."),
+                        });
+                    } else {
+                        throw error;
+                    }
+                }
+                if (foundProductIds.length) {
+                    await this.env.pos._addProducts(foundProductIds, false);
+                    if (foundPackagings.length) {
+                        this.env.pos.db.add_packagings(foundPackagings);
+                    }
+                    // assume that the result is unique.
+                    product = this.env.pos.db.get_product_by_id(foundProductIds[0]);
+                } else {
+                    return this._barcodeErrorAction(code);
+                }
             }
-            const options = await this._getAddProductOptions(product);
+            return product
+        }
+        async _barcodeProductAction(code) {
+            const product = await this._getProductByBarcode(code);
+            if (!product) {
+                return;
+            }
+            const options = await this._getAddProductOptions(product, code);
             // Do not proceed on adding the product when no options is returned.
             // This is consistent with _clickProduct.
             if (!options) return;
@@ -221,7 +300,7 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
                         price_manually_set: true,
                     },
                 });
-            } else if (code.type === 'weight') {
+            } else if (code.type === 'weight' || code.type === 'quantity') {
                 Object.assign(options, {
                     quantity: code.value,
                     merge: false,
@@ -232,18 +311,15 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
                     merge: false,
                 });
             }
-            this.currentOrder.add_product(product,  options)
+            this.currentOrder.add_product(product,  options);
+            NumberBuffer.reset();
         }
-        _barcodeClientAction(code) {
+        _barcodePartnerAction(code) {
             const partner = this.env.pos.db.get_partner_by_barcode(code.code);
             if (partner) {
-                if (this.currentOrder.get_client() !== partner) {
-                    this.currentOrder.set_client(partner);
-                    this.currentOrder.set_pricelist(
-                        _.findWhere(this.env.pos.pricelists, {
-                            id: partner.property_product_pricelist[0],
-                        }) || this.env.pos.default_pricelist
-                    );
+                if (this.currentOrder.get_partner() !== partner) {
+                    this.currentOrder.set_partner(partner);
+                    this.currentOrder.updatePricelist(partner);
                 }
                 return true;
             }
@@ -256,6 +332,26 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
                 last_orderline.set_discount(code.value);
             }
         }
+        async _parseElementsFromGS1(parsed_results) {
+            const productBarcode = parsed_results.find(element => element.type === 'product');
+            const lotBarcode = parsed_results.find(element => element.type === 'lot');
+            const product = await this._getProductByBarcode(productBarcode);
+            return { product, lotBarcode, customProductOptions: {} }
+        }
+        /**
+         * Add a product to the current order using the product identifier and lot number from parsed results.
+         * This function retrieves the product identifier and lot number from the `parsed_results` parameter.
+         * It then uses these values to retrieve the product and add it to the current order.
+         */
+        async _barcodeGS1Action(parsed_results) {
+            const { product, lotBarcode, customProductOptions } = await this._parseElementsFromGS1(parsed_results)
+            if (!product) {
+                return;
+            }
+            const options = await this._getAddProductOptions(product, lotBarcode);
+            await this.currentOrder.add_product(product, { ...options, ...customProductOptions });
+            NumberBuffer.reset();
+        }
         // IMPROVEMENT: The following two methods should be in PosScreenComponent?
         // Why? Because once we start declaring barcode actions in different
         // screens, these methods will also be declared over and over.
@@ -269,6 +365,11 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
                 return code.code;
             }
         }
+        async _displayAllControlPopup() {
+            await this.showPopup('ControlButtonPopup', {
+                controlButtons: this.controlButtons
+            });
+        }
         /**
          * override this method to perform procedure if the scale is not available.
          * @see isScaleAvailable
@@ -279,7 +380,7 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
                 startingValue: 0,
                 title: this.env._t('Set the new quantity'),
             });
-            let newQuantity = inputNumber !== "" ? parse.float(inputNumber) : null;
+            let newQuantity = inputNumber && inputNumber !== "" ? parse.float(inputNumber) : null;
             if (confirmed && newQuantity !== null) {
                 let order = this.env.pos.get_order();
                 let selectedLine = this.env.pos.get_order().get_selected_orderline();
@@ -296,35 +397,55 @@ odoo.define('point_of_sale.ProductScreen', function(require) {
                     newLine.set_quantity( - decreasedQuantity, true);
                     order.add_orderline(newLine);
                 }
+                return true;
             }
+            return false;
         }
-        async _onClickCustomer() {
-            // IMPROVEMENT: This code snippet is very similar to selectClient of PaymentScreen.
-            const currentClient = this.currentOrder.get_client();
-            const { confirmed, payload: newClient } = await this.showTempScreen(
-                'ClientListScreen',
-                { client: currentClient }
+        async onClickPartner() {
+            // IMPROVEMENT: This code snippet is very similar to selectPartner of PaymentScreen.
+            const currentPartner = this.currentOrder.get_partner();
+            if (currentPartner && this.currentOrder.getHasRefundLines()) {
+                this.showPopup('ErrorPopup', {
+                    title: this.env._t("Can't change customer"),
+                    body: _.str.sprintf(
+                        this.env._t(
+                            "This order already has refund lines for %s. We can't change the customer associated to it. Create a new order for the new customer."
+                        ),
+                        currentPartner.name
+                    ),
+                });
+                return;
+            }
+            const { confirmed, payload: newPartner } = await this.showTempScreen(
+                'PartnerListScreen',
+                { partner: currentPartner }
             );
             if (confirmed) {
-                this.currentOrder.set_client(newClient);
-                this.currentOrder.updatePricelist(newClient);
+                this.currentOrder.set_partner(newPartner);
+                this.currentOrder.updatePricelist(newPartner);
             }
         }
-        _onClickPay() {
-            this.showScreen('PaymentScreen');
+        async _onClickPay() {
+            if (this.env.pos.get_order().orderlines.some(line => line.get_product().tracking !== 'none' && !line.has_valid_product_lot()) && (this.env.pos.picking_type.use_create_lots || this.env.pos.picking_type.use_existing_lots)) {
+                const { confirmed } = await this.showPopup('ConfirmPopup', {
+                    title: this.env._t('Some Serial/Lot Numbers are missing'),
+                    body: this.env._t('You are trying to sell products with serial/lot numbers, but some of them are not set.\nWould you like to proceed anyway?'),
+                    confirmText: this.env._t('Yes'),
+                    cancelText: this.env._t('No')
+                });
+                if (confirmed) {
+                    this.showScreen('PaymentScreen');
+                }
+            } else {
+                this.showScreen('PaymentScreen');
+            }
         }
         switchPane() {
-            if (this.mobile_pane === "left") {
-                this.mobile_pane = "right";
-                this.render();
-            }
-            else {
-                this.mobile_pane = "left";
-                this.render();
-            }
+            this.state.mobile_pane = this.state.mobile_pane === "left" ? "right" : "left";
         }
     }
     ProductScreen.template = 'ProductScreen';
+    ProductScreen.numpadActionName = _lt('Payment');
 
     Registries.Component.add(ProductScreen);
 
