@@ -8,6 +8,18 @@ var utils = require('web.utils');
  * - persistent : must stay between reloads ( orders )
  */
 
+
+/**
+ * cache the data in memory to avoid roundtrips to the localstorage
+ *
+ * NOTE/TODO: Originally, this is a prop of PosDB. However, if we keep it that way,
+ * caching will result to infinite loop to calling the reactive callbacks.
+ * Another way to solve the infinite loop is to move the instance of PosDB to env.
+ * But I'm not sure if there is anything inside the object that needs to be observed,
+ * so I guess this strategy is good enough for the moment.
+ */
+const CACHE = {};
+
 var PosDB = core.Class.extend({
     name: 'openerp_pos_db', //the prefix of the localstorage data
     limit: 100,  // the maximum number of results returned by a search
@@ -15,22 +27,24 @@ var PosDB = core.Class.extend({
         options = options || {};
         this.name = options.name || this.name;
         this.limit = options.limit || this.limit;
-        
+
         if (options.uuid) {
             this.name = this.name + '_' + options.uuid;
         }
 
-        //cache the data in memory to avoid roundtrips to the localstorage
-        this.cache = {};
-
         this.product_by_id = {};
         this.product_by_barcode = {};
         this.product_by_category_id = {};
+        this.product_packaging_by_barcode = {};
 
         this.partner_sorted = [];
         this.partner_by_id = {};
         this.partner_by_barcode = {};
+        // FIXME before master: partner_search_string is no longer used but is kept for partial
+        // compatibility with customizations. The string is no longer useful but we don't want
+        // a custo to crash when calling a method (eg .split()) on it.
         this.partner_search_string = "";
+        this.partner_search_strings = {};
         this.partner_write_date = null;
 
         this.category_by_id = {};
@@ -42,7 +56,7 @@ var PosDB = core.Class.extend({
         this.category_search_string = {};
     },
 
-    /** 
+    /**
      * sets an uuid to prevent conflict in locally stored data between multiple PoS Configs. By
      * using the uuid of the config the local storage from other configs will not get effected nor
      * loaded in sessions that don't belong to them.
@@ -54,8 +68,8 @@ var PosDB = core.Class.extend({
     },
 
     /* returns the category object from its id. If you pass a list of id as parameters, you get
-     * a list of category objects. 
-     */  
+     * a list of category objects.
+     */
     get_category_by_id: function(categ_id){
         if(categ_id instanceof Array){
             var list = [];
@@ -72,7 +86,7 @@ var PosDB = core.Class.extend({
             return this.category_by_id[categ_id];
         }
     },
-    /* returns a list of the category's child categories ids, or an empty list 
+    /* returns a list of the category's child categories ids, or an empty list
      * if a category has no childs */
     get_category_childs_ids: function(categ_id){
         return this.category_childs[categ_id] || [];
@@ -88,7 +102,7 @@ var PosDB = core.Class.extend({
         return this.category_parent[categ_id] || this.root_category_id;
     },
     /* adds categories definitions to the database. categories is a list of categories objects as
-     * returned by the openerp server. Categories must be inserted before the products or the 
+     * returned by the openerp server. Categories must be inserted before the products or the
      * product/ categories association may (will) not work properly */
     add_categories: function(categories){
         var self = this;
@@ -138,13 +152,13 @@ var PosDB = core.Class.extend({
     },
     /* loads a record store from the database. returns default if nothing is found */
     load: function(store,deft){
-        if(this.cache[store] !== undefined){
-            return this.cache[store];
+        if(CACHE[store] !== undefined){
+            return CACHE[store];
         }
         var data = localStorage[this.name + '_' + store];
         if(data !== undefined && data !== ""){
             data = JSON.parse(data);
-            this.cache[store] = data;
+            CACHE[store] = data;
             return data;
         }else{
             return deft;
@@ -153,7 +167,7 @@ var PosDB = core.Class.extend({
     /* saves a record store to the database */
     save: function(store,data){
         localStorage[this.name + '_' + store] = JSON.stringify(data);
-        this.cache[store] = data;
+        CACHE[store] = data;
     },
     _product_search_string: function(product){
         var str = product.display_name;
@@ -169,13 +183,13 @@ var PosDB = core.Class.extend({
         if (product.description_sale) {
             str += '|' + product.description_sale;
         }
-        str  = product.id + ':' + str.replace(/:/g,'') + '\n';
+        str  = product.id + ':' + str.replace(/[\n:]/g,'') + '\n';
         return str;
     },
     add_products: function(products){
         var stored_categories = this.product_by_category_id;
 
-        if(!products instanceof Array){
+        if(!(products instanceof Array)){
             products = [products];
         }
         for(var i = 0, len = products.length; i < len; i++){
@@ -211,11 +225,18 @@ var PosDB = core.Class.extend({
                 }
             }
             this.product_by_id[product.id] = product;
-            if(product.barcode){
+            if(product.barcode && product.active){
                 this.product_by_barcode[product.barcode] = product;
             }
         }
     },
+    add_packagings: function(productPackagings){
+        productPackagings.forEach(productPackaging => {
+            if (productPackaging.product_id[0] in this.product_by_id) {
+                this.product_packaging_by_barcode[productPackaging.barcode] = productPackaging;
+            }
+        });
+    },
     _partner_search_string: function(partner){
         var str =  partner.name || '';
         if(partner.barcode){
@@ -236,11 +257,14 @@ var PosDB = core.Class.extend({
         if(partner.vat){
             str += '|' + partner.vat;
         }
-        str = '' + partner.id + ':' + str.replace(':','') + '\n';
+        if(partner.parent_name){
+            str += '|' + partner.parent_name;
+        }
+        str = '' + partner.id + ':' + str.replace(':', '').replace(/\n/g, ' ') + '\n';
         return str;
     },
     add_partners: function(partners){
-        var updated_count = 0;
+        var updated = {};
         var new_write_date = '';
         var partner;
         for(var i = 0, len = partners.length; i < len; i++){
@@ -255,45 +279,56 @@ var PosDB = core.Class.extend({
                 // FIXME: The write_date is stored with milisec precision in the database
                 // but the dates we get back are only precise to the second. This means when
                 // you read partners modified strictly after time X, you get back partners that were
-                // modified X - 1 sec ago. 
+                // modified X - 1 sec ago.
                 continue;
-            } else if ( new_write_date < partner.write_date ) { 
+            } else if ( new_write_date < partner.write_date ) {
                 new_write_date  = partner.write_date;
             }
             if (!this.partner_by_id[partner.id]) {
                 this.partner_sorted.push(partner.id);
+            } else {
+                const oldPartner = this.partner_by_id[partner.id];
+                if (oldPartner.barcode) {
+                    delete this.partner_by_barcode[oldPartner.barcode];
+                }
             }
+            if (partner.barcode) {
+                this.partner_by_barcode[partner.barcode] = partner;
+            }
+            updated[partner.id] = partner;
             this.partner_by_id[partner.id] = partner;
-
-            updated_count += 1;
         }
 
         this.partner_write_date = new_write_date || this.partner_write_date;
 
-        if (updated_count) {
-            // If there were updates, we need to completely 
-            // rebuild the search string and the barcode indexing
+        const updatedChunks = new Set();
+        const CHUNK_SIZE = 100;
+        for (const id in updated) {
+            const chunkId = Math.floor(id / CHUNK_SIZE);
+            if (updatedChunks.has(chunkId)) {
+                // another partner in this chunk was updated and we already rebuild the chunk
+                continue;
+            }
+            updatedChunks.add(chunkId);
+            // If there were updates, we need to rebuild the search string for this chunk
+            let searchString = "";
 
-            this.partner_search_string = "";
-            this.partner_by_barcode = {};
-
-            for (var id in this.partner_by_id) {
-                partner = this.partner_by_id[id];
-
-                if(partner.barcode){
-                    this.partner_by_barcode[partner.barcode] = partner;
+            for (let id = chunkId * CHUNK_SIZE; id < (chunkId + 1) * CHUNK_SIZE; id++) {
+                if (!(id in this.partner_by_id)) {
+                    continue;
                 }
+                const partner = this.partner_by_id[id];
                 partner.address = (partner.street ? partner.street + ', ': '') +
                                   (partner.zip ? partner.zip + ', ': '') +
                                   (partner.city ? partner.city + ', ': '') +
                                   (partner.state_id ? partner.state_id[1] + ', ': '') +
                                   (partner.country_id ? partner.country_id[1]: '');
-                this.partner_search_string += this._partner_search_string(partner);
+                searchString += this._partner_search_string(partner);
             }
 
-            this.partner_search_string = utils.unaccent(this.partner_search_string);
+            this.partner_search_strings[chunkId] = utils.unaccent(searchString);
         }
-        return updated_count;
+        return Object.keys(updated).length;
     },
     get_partner_write_date: function(){
         return this.partner_write_date || "1970-01-01 00:00:00";
@@ -317,17 +352,19 @@ var PosDB = core.Class.extend({
             query = query.replace(/[\[\]\(\)\+\*\?\.\-\!\&\^\$\|\~\_\{\}\:\,\\\/]/g,'.');
             query = query.replace(/ /g,'.+');
             var re = RegExp("([0-9]+):.*?"+utils.unaccent(query),"gi");
-        }catch(e){
+        }catch(_e){
             return [];
         }
         var results = [];
-        for(var i = 0; i < this.limit; i++){
-            var r = re.exec(this.partner_search_string);
+        const searchStrings = Object.values(this.partner_search_strings).reverse();
+        let searchString = searchStrings.pop();
+        while (searchString && results.length < this.limit) {
+            var r = re.exec(searchString);
             if(r){
                 var id = Number(r[1]);
                 results.push(this.get_partner_by_id(id));
-            }else{
-                break;
+            } else {
+                searchString = searchStrings.pop();
             }
         }
         return results;
@@ -354,30 +391,33 @@ var PosDB = core.Class.extend({
     get_product_by_barcode: function(barcode){
         if(this.product_by_barcode[barcode]){
             return this.product_by_barcode[barcode];
-        } else {
-            return undefined;
+        } else if (this.product_packaging_by_barcode[barcode]) {
+            return this.product_by_id[this.product_packaging_by_barcode[barcode].product_id[0]];
         }
+        return undefined;
     },
     get_product_by_category: function(category_id){
         var product_ids  = this.product_by_category_id[category_id];
         var list = [];
         if (product_ids) {
             for (var i = 0, len = Math.min(product_ids.length, this.limit); i < len; i++) {
-                list.push(this.product_by_id[product_ids[i]]);
+                const product = this.product_by_id[product_ids[i]];
+                if (!(product.active && product.available_in_pos)) continue;
+                list.push(product);
             }
         }
         return list;
     },
     /* returns a list of products with :
      * - a category that is or is a child of category_id,
-     * - a name, package or barcode containing the query (case insensitive) 
+     * - a name, package or barcode containing the query (case insensitive)
      */
     search_product_in_category: function(category_id, query){
         try {
             query = query.replace(/[\[\]\(\)\+\*\?\.\-\!\&\^\$\|\~\_\{\}\:\,\\\/]/g,'.');
             query = query.replace(/ /g,'.+');
             var re = RegExp("([0-9]+):.*?"+utils.unaccent(query),"gi");
-        }catch(e){
+        }catch(_e){
             return [];
         }
         var results = [];
@@ -385,7 +425,9 @@ var PosDB = core.Class.extend({
             var r = re.exec(this.category_search_string[category_id]);
             if(r){
                 var id = Number(r[1]);
-                results.push(this.get_product_by_id(id));
+                const product = this.get_product_by_id(id);
+                if (!(product.active && product.available_in_pos)) continue;
+                results.push(product);
             }else{
                 break;
             }
@@ -397,13 +439,10 @@ var PosDB = core.Class.extend({
      * or one of its child categories.
      */
     is_product_in_category: function(category_ids, product_id) {
-        if (!(category_ids instanceof Array)) {
-            category_ids = [category_ids];
-        }
-        var cat = this.get_product_by_id(product_id).pos_categ_id[0];
+        let cat = this.get_product_by_id(product_id).pos_categ_id[0];
         while (cat) {
-            for (var i = 0; i < category_ids.length; i++) {
-                if (cat == category_ids[i]) {   // The == is important, ids may be strings
+            for (let cat_id of category_ids) {
+                if (cat == cat_id) {   // The == is important, ids may be strings
                     return true;
                 }
             }
@@ -499,19 +538,13 @@ var PosDB = core.Class.extend({
      * @return {array<object>} list of orders.
      */
     get_unpaid_orders_to_sync: function(ids){
-        var saved = this.load('unpaid_orders',[]);
-        var orders = [];
-        saved.forEach(function(o) {
-            if (ids.includes(o.id) && (o.data.server_id || o.data.lines.length)){
-                orders.push(o);
-            }
-        });
-        return orders;
+        const savedOrders = this.load('unpaid_orders',[]);
+        return savedOrders.filter(order => ids.includes(order.id) && (order.data.server_id || order.data.lines.length || order.data.statement_ids.length));
     },
     /**
      * Add a given order to the orders to be removed from the server.
      *
-     * If an order is removed from a table it also has to be removed from the server to prevent it from reapearing 
+     * If an order is removed from a table it also has to be removed from the server to prevent it from reapearing
      * after syncing. This function will add the server_id of the order to a list of orders still to be removed.
      * @param {object} order object.
      */
@@ -535,22 +568,14 @@ var PosDB = core.Class.extend({
      */
     set_ids_removed_from_server: function(ids){
         var to_remove = this.load('unpaid_orders_to_remove',[]);
-        
+
         to_remove = _.filter(to_remove, function(id){
             return !ids.includes(id);
         });
         this.save('unpaid_orders_to_remove', to_remove);
     },
-    set_cashier: function(cashier) {
-        // Always update if the user is the same as before
-        this.save('cashier', cashier || null);
-    },
-    get_cashier: function() {
-        return this.load('cashier');
-    }
 });
 
 return PosDB;
 
 });
-
