@@ -4,34 +4,53 @@
 from odoo import tools
 from odoo.api import Environment
 from odoo.tools import DEFAULT_SERVER_DATE_FORMAT
+from odoo.addons.account.tests.common import AccountTestInvoicingHttpCommon
 from datetime import date, timedelta
 
 import odoo.tests
 
 
-class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
+class TestPointOfSaleHttpCommon(AccountTestInvoicingHttpCommon):
 
-    def setUp(self):
-        super().setUp()
-        env = self.env(user=self.env.ref('base.user_admin'))
+    @classmethod
+    def setUpClass(cls, chart_template_ref=None):
+        super().setUpClass(chart_template_ref=chart_template_ref)
 
+        env = cls.env
+        cls.env.user.groups_id += env.ref('point_of_sale.group_pos_manager')
         journal_obj = env['account.journal']
         account_obj = env['account.account']
-        main_company = env.ref('base.main_company')
-        self.main_pos_config = env.ref('point_of_sale.pos_config_main')
+        main_company = cls.company_data['company']
+
+        account_receivable = account_obj.create({'code': 'X1012',
+                                                 'name': 'Account Receivable - Test',
+                                                 'account_type': 'asset_receivable',
+                                                 'reconcile': True})
+        env.company.account_default_pos_receivable_account_id = account_receivable
+        env['ir.property']._set_default('property_account_receivable_id', 'res.partner', account_receivable, main_company)
+        # Pricelists are set below, do not take demo data into account
+        env['ir.property'].sudo().search([('name', '=', 'property_product_pricelist')]).unlink()
+
+        cls.bank_journal = journal_obj.create({
+            'name': 'Bank Test',
+            'type': 'bank',
+            'company_id': main_company.id,
+            'code': 'BNK',
+            'sequence': 10,
+        })
+
+        env['pos.payment.method'].create({
+            'name': 'Bank',
+            'journal_id': cls.bank_journal.id,
+        })
+        cls.main_pos_config = env['pos.config'].create({
+            'name': 'Shop',
+        })
 
         env['res.partner'].create({
             'name': 'Deco Addict',
         })
 
-        account_receivable = account_obj.create({'code': 'X1012',
-                                                 'name': 'Account Receivable - Test',
-                                                 'user_type_id': env.ref('account.data_account_type_receivable').id,
-                                                 'reconcile': True})
-        self.env.company.account_default_pos_receivable_account_id = account_receivable
-
-        self.env['ir.property']._set_default('property_account_receivable_id', 'res.partner', account_receivable, main_company)
-
         cash_journal = journal_obj.create({
             'name': 'Cash Test',
             'type': 'cash',
@@ -41,10 +60,10 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
         })
 
         # Archive all existing product to avoid noise during the tours
-        all_pos_product = self.env['product.product'].search([('available_in_pos', '=', True)])
-        discount = self.env.ref('point_of_sale.product_product_consumable')
-        self.tip = self.env.ref('point_of_sale.product_product_tip')
-        (all_pos_product - discount - self.tip)._write({'active': False})
+        all_pos_product = env['product.product'].search([('available_in_pos', '=', True)])
+        discount = env.ref('point_of_sale.product_product_consumable')
+        cls.tip = env.ref('point_of_sale.product_product_tip')
+        (all_pos_product - discount - cls.tip)._write({'active': False})
 
         # In DESKS categ: Desk Pad
         pos_categ_desks = env.ref('point_of_sale.pos_category_desks')
@@ -56,7 +75,7 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
         pos_categ_chairs = env.ref('point_of_sale.pos_category_chairs')
 
         # test an extra price on an attribute
-        pear = env['product.product'].create({
+        cls.whiteboard_pen = env['product.product'].create({
             'name': 'Whiteboard Pen',
             'available_in_pos': True,
             'list_price': 1.20,
@@ -65,45 +84,45 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
             'to_weight': True,
             'pos_categ_id': pos_categ_misc.id,
         })
-        wall_shelf = env['product.product'].create({
+        cls.wall_shelf = env['product.product'].create({
             'name': 'Wall Shelf Unit',
             'available_in_pos': True,
             'list_price': 1.98,
             'taxes_id': False,
         })
-        small_shelf = env['product.product'].create({
+        cls.small_shelf = env['product.product'].create({
             'name': 'Small Shelf',
             'available_in_pos': True,
             'list_price': 2.83,
             'taxes_id': False,
         })
-        magnetic_board = env['product.product'].create({
+        cls.magnetic_board = env['product.product'].create({
             'name': 'Magnetic Board',
             'available_in_pos': True,
             'list_price': 1.98,
             'taxes_id': False,
         })
-        monitor_stand = env['product.product'].create({
+        cls.monitor_stand = env['product.product'].create({
             'name': 'Monitor Stand',
             'available_in_pos': True,
             'list_price': 3.19,
             'taxes_id': False,
         })
-        desk_pad = env['product.product'].create({
+        cls.desk_pad = env['product.product'].create({
             'name': 'Desk Pad',
             'available_in_pos': True,
             'list_price': 1.98,
             'taxes_id': False,
             'pos_categ_id': pos_categ_desks.id,
         })
-        letter_tray = env['product.product'].create({
+        cls.letter_tray = env['product.product'].create({
             'name': 'Letter Tray',
             'available_in_pos': True,
             'list_price': 4.80,
             'taxes_id': False,
             'pos_categ_id': pos_categ_chairs.id,
         })
-        desk_organizer = env['product.product'].create({
+        cls.desk_organizer = env['product.product'].create({
             'name': 'Desk Organizer',
             'available_in_pos': True,
             'list_price': 5.10,
@@ -124,7 +143,7 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
             'attribute_id': attribute.id,
         })
         line = env['product.template.attribute.line'].create({
-            'product_tmpl_id': pear.product_tmpl_id.id,
+            'product_tmpl_id': cls.whiteboard_pen.product_tmpl_id.id,
             'attribute_id': attribute.id,
             'value_ids': [(6, 0, attribute_value.ids)]
         })
@@ -201,12 +220,12 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
                 'compute_price': 'fixed',
                 'fixed_price': 2,
                 'applied_on': '0_product_variant',
-                'product_id': wall_shelf.id,
+                'product_id': cls.wall_shelf.id,
             }), (0, 0, {
                 'compute_price': 'fixed',
                 'fixed_price': 13.95,  # test for issues like in 7f260ab517ebde634fc274e928eb062463f0d88f
                 'applied_on': '0_product_variant',
-                'product_id': small_shelf.id,
+                'product_id': cls.small_shelf.id,
             })],
         })
 
@@ -216,17 +235,17 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
                 'compute_price': 'percentage',
                 'percent_price': 100,
                 'applied_on': '0_product_variant',
-                'product_id': wall_shelf.id,
+                'product_id': cls.wall_shelf.id,
             }), (0, 0, {
                 'compute_price': 'percentage',
                 'percent_price': 99,
                 'applied_on': '0_product_variant',
-                'product_id': small_shelf.id,
+                'product_id': cls.small_shelf.id,
             }), (0, 0, {
                 'compute_price': 'percentage',
                 'percent_price': 0,
                 'applied_on': '0_product_variant',
-                'product_id': magnetic_board.id,
+                'product_id': cls.magnetic_board.id,
             })],
         })
 
@@ -237,33 +256,33 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
                 'price_discount': 6,
                 'price_surcharge': 5,
                 'applied_on': '0_product_variant',
-                'product_id': wall_shelf.id,
+                'product_id': cls.wall_shelf.id,
             }), (0, 0, {
                 # .99 prices
                 'compute_price': 'formula',
                 'price_surcharge': -0.01,
                 'price_round': 1,
                 'applied_on': '0_product_variant',
-                'product_id': small_shelf.id,
+                'product_id': cls.small_shelf.id,
             }), (0, 0, {
                 'compute_price': 'formula',
                 'price_min_margin': 10,
                 'price_max_margin': 100,
                 'applied_on': '0_product_variant',
-                'product_id': magnetic_board.id,
+                'product_id': cls.magnetic_board.id,
             }), (0, 0, {
                 'compute_price': 'formula',
                 'price_surcharge': 10,
                 'price_max_margin': 5,
                 'applied_on': '0_product_variant',
-                'product_id': monitor_stand.id,
+                'product_id': cls.monitor_stand.id,
             }), (0, 0, {
                 'compute_price': 'formula',
                 'price_discount': -100,
                 'price_min_margin': 5,
                 'price_max_margin': 20,
                 'applied_on': '0_product_variant',
-                'product_id': desk_pad.id,
+                'product_id': cls.desk_pad.id,
             })],
         })
 
@@ -274,13 +293,13 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
                 'fixed_price': 1,
                 'applied_on': '0_product_variant',
                 'min_quantity': 2,
-                'product_id': wall_shelf.id,
+                'product_id': cls.wall_shelf.id,
             }), (0, 0, {
                 'compute_price': 'fixed',
                 'fixed_price': 2,
                 'applied_on': '0_product_variant',
                 'min_quantity': 1,
-                'product_id': wall_shelf.id,
+                'product_id': cls.wall_shelf.id,
             }), (0, 0, {
                 'compute_price': 'fixed',
                 'fixed_price': 2,
@@ -296,7 +315,7 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
                 'compute_price': 'fixed',
                 'fixed_price': 1,
                 'applied_on': '1_product',
-                'product_tmpl_id': wall_shelf.product_tmpl_id.id,
+                'product_tmpl_id': cls.wall_shelf.product_tmpl_id.id,
             }), (0, 0, {
                 'compute_price': 'fixed',
                 'fixed_price': 2,
@@ -409,23 +428,12 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
         excluded_pricelist = env['product.pricelist'].create({
             'name': 'Not loaded'
         })
-        res_partner_18 = self.env['res.partner'].create({
+        res_partner_18 = env['res.partner'].create({
             'name': 'Lumber Inc',
             'is_company': True,
         })
         res_partner_18.property_product_pricelist = excluded_pricelist
 
-        partner = self.env['res.partner'].create({
-            'name': 'TEST PARTNER',
-            'email': 'test@partner.com',
-        })
-
-        # set the company currency to USD, otherwise it will assume
-        # euro's. this will cause issues as the sales journal is in
-        # USD, because of this all products would have a different
-        # price
-        main_company.currency_id = env.ref('base.USD')
-
         test_sale_journal = journal_obj.create({'name': 'Sales Journal - Test',
                                                 'code': 'TSJ',
                                                 'type': 'sale',
@@ -437,9 +445,9 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
         src_tax = env['account.tax'].create({'name': "SRC", 'amount': 10})
         dst_tax = env['account.tax'].create({'name': "DST", 'amount': 5})
 
-        letter_tray.taxes_id = [(6, 0, [src_tax.id])]
+        cls.letter_tray.taxes_id = [(6, 0, [src_tax.id])]
 
-        self.main_pos_config.write({
+        cls.main_pos_config.write({
             'tax_regime_selection': True,
             'fiscal_position_ids': [(0, 0, {
                                             'name': "FP-POS-2M",
@@ -452,23 +460,17 @@ class TestPointOfSaleHttpCommon(odoo.tests.HttpCase):
             'journal_id': test_sale_journal.id,
             'invoice_journal_id': test_sale_journal.id,
             'payment_method_ids': [(0, 0, { 'name': 'Cash',
-                                            'is_cash_count': True,
-                                            'cash_journal_id': cash_journal.id,
+                                            'journal_id': cash_journal.id,
                                             'receivable_account_id': account_receivable.id,
             })],
             'use_pricelist': True,
             'pricelist_id': public_pricelist.id,
             'available_pricelist_ids': [(4, pricelist.id) for pricelist in all_pricelists],
-            'module_pos_loyalty': False,
         })
 
         # Change the default sale pricelist of customers,
         # so the js tests can expect deterministically this pricelist when selecting a customer.
-        env['ir.property']._set_default(
-            "property_product_pricelist",
-            "res.partner",
-            public_pricelist,
-        )
+        env['ir.property']._set_default("property_product_pricelist", "res.partner", public_pricelist, main_company)
 
 
 @odoo.tests.tagged('post_install', '-at_install')
@@ -481,7 +483,7 @@ class TestUi(TestPointOfSaleHttpCommon):
         })
 
         # open a session, the /pos/ui controller will redirect to it
-        self.main_pos_config.open_session_cb(check_coa=False)
+        self.main_pos_config.open_ui()
 
         # needed because tests are run before the module is marked as
         # installed. In js web will only load qweb coming from modules
@@ -489,11 +491,11 @@ class TestUi(TestPointOfSaleHttpCommon):
         # this you end up with js, css but no qweb.
         self.env['ir.module.module'].search([('name', '=', 'point_of_sale')], limit=1).state = 'installed'
 
-        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'pos_pricelist', login="admin")
-        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'pos_basic_order', login="admin")
-        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'ProductScreenTour', login="admin")
-        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'PaymentScreenTour', login="admin")
-        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'ReceiptScreenTour', login="admin")
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'pos_pricelist', login="accountman")
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'pos_basic_order', login="accountman")
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'ProductScreenTour', login="accountman")
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'PaymentScreenTour', login="accountman")
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'ReceiptScreenTour', login="accountman")
 
         for order in self.env['pos.order'].search([]):
             self.assertEqual(order.state, 'paid', "Validated order has payment of " + str(order.amount_paid) + " and total of " + str(order.amount_total))
@@ -503,23 +505,184 @@ class TestUi(TestPointOfSaleHttpCommon):
         self.assertEqual(email_count, 1)
 
     def test_02_pos_with_invoiced(self):
-        self.main_pos_config.open_session_cb(check_coa=False)
-        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'ChromeTour', login="admin")
+        self.main_pos_config.open_ui()
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'ChromeTour', login="accountman")
         n_invoiced = self.env['pos.order'].search_count([('state', '=', 'invoiced')])
         n_paid = self.env['pos.order'].search_count([('state', '=', 'paid')])
         self.assertEqual(n_invoiced, 1, 'There should be 1 invoiced order.')
         self.assertEqual(n_paid, 2, 'There should be 2 paid order.')
 
-    def test_03_order_management(self):
-        self.main_pos_config.write({ 'manage_orders': True, 'module_account': True })
-        self.main_pos_config.open_session_cb(check_coa=False)
-        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'OrderManagementScreenTour', login="admin")
-
     def test_04_product_configurator(self):
-        self.main_pos_config.write({ 'product_configurator': True })
-        self.main_pos_config.open_session_cb(check_coa=False)
-        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config, 'ProductConfiguratorTour', login="admin")
+
+        self.main_pos_config.open_ui()
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config, 'ProductConfiguratorTour', login="accountman")
 
     def test_05_ticket_screen(self):
-        self.main_pos_config.open_session_cb(check_coa=False)
-        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'TicketScreenTour', login="admin")
+        self.main_pos_config.open_ui()
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'TicketScreenTour', login="accountman")
+
+    def test_fixed_tax_negative_qty(self):
+        """ Assert the negative amount of a negative-quantity orderline
+            with zero-amount product with fixed tax.
+        """
+
+        # setup the zero-amount product
+        tax_received_account = self.env['account.account'].create({
+            'name': 'TAX_BASE',
+            'code': 'TBASE',
+            'account_type': 'asset_current',
+            'company_id': self.env.company.id,
+        })
+        fixed_tax = self.env['account.tax'].create({
+            'name': 'fixed amount tax',
+            'amount_type': 'fixed',
+            'amount': 1,
+            'invoice_repartition_line_ids': [
+                (0, 0, {'repartition_type': 'base'}),
+                (0, 0, {
+                    'repartition_type': 'tax',
+                    'account_id': tax_received_account.id,
+                }),
+            ],
+        })
+        zero_amount_product = self.env['product.product'].create({
+            'name': 'Zero Amount Product',
+            'available_in_pos': True,
+            'list_price': 0,
+            'taxes_id': [(6, 0, [fixed_tax.id])],
+        })
+
+        # Make an order with the zero-amount product from the frontend.
+        # We need to do this because of the fix in the "compute_all" port.
+        self.main_pos_config.write({'iface_tax_included': 'total'})
+        self.main_pos_config.open_ui()
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'FixedTaxNegativeQty', login="accountman")
+        pos_session = self.main_pos_config.current_session_id
+
+        # Close the session and check the session journal entry.
+        pos_session.action_pos_session_validate()
+
+        lines = pos_session.move_id.line_ids.sorted('balance')
+
+        # order in the tour is paid using the bank payment method.
+        bank_pm = self.main_pos_config.payment_method_ids.filtered(lambda pm: pm.name == 'Bank')
+
+        self.assertEqual(lines[0].account_id, bank_pm.receivable_account_id or self.env.company.account_default_pos_receivable_account_id)
+        self.assertAlmostEqual(lines[0].balance, -1)
+        self.assertEqual(lines[1].account_id, zero_amount_product.categ_id.property_account_income_categ_id)
+        self.assertAlmostEqual(lines[1].balance, 0)
+        self.assertEqual(lines[2].account_id, tax_received_account)
+        self.assertAlmostEqual(lines[2].balance, 1)
+
+    def test_change_without_cash_method(self):
+        #create bank payment method
+        bank_pm = self.env['pos.payment.method'].create({
+            'name': 'Bank',
+            'receivable_account_id': self.env.company.account_default_pos_receivable_account_id.id,
+            'is_cash_count': False,
+            'split_transactions': False,
+            'company_id': self.env.company.id,
+        })
+        self.main_pos_config.write({'payment_method_ids': [(6, 0, bank_pm.ids)]})
+        self.main_pos_config.open_ui()
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'PaymentScreenTour2', login="accountman")
+
+    def test_rounding_up(self):
+        rouding_method = self.env['account.cash.rounding'].create({
+            'name': 'Rounding up',
+            'rounding': 0.05,
+            'rounding_method': 'UP',
+        })
+
+        self.env['product.product'].create({
+            'name': 'Product Test',
+            'available_in_pos': True,
+            'list_price': 1.98,
+            'taxes_id': False,
+        })
+
+        self.main_pos_config.write({
+            'rounding_method': rouding_method.id,
+            'cash_rounding': True,
+        })
+
+        self.main_pos_config.open_ui()
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'PaymentScreenRoundingUp', login="accountman")
+
+    def test_rounding_down(self):
+        rouding_method = self.env['account.cash.rounding'].create({
+            'name': 'Rounding down',
+            'rounding': 0.05,
+            'rounding_method': 'DOWN',
+        })
+
+        self.env['product.product'].create({
+            'name': 'Product Test',
+            'available_in_pos': True,
+            'list_price': 1.98,
+            'taxes_id': False,
+        })
+
+        self.main_pos_config.write({
+            'rounding_method': rouding_method.id,
+            'cash_rounding': True,
+        })
+
+        self.main_pos_config.open_ui()
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'PaymentScreenRoundingDown', login="accountman")
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'PaymentScreenTotalDueWithOverPayment', login="accountman")
+
+    def test_rounding_half_up(self):
+        rouding_method = self.env['account.cash.rounding'].create({
+            'name': 'Rounding HALF-UP',
+            'rounding': 0.5,
+            'rounding_method': 'HALF-UP',
+        })
+
+        self.env['product.product'].create({
+            'name': 'Product Test 1.2',
+            'available_in_pos': True,
+            'list_price': 1.2,
+            'taxes_id': False,
+        })
+
+        self.env['product.product'].create({
+            'name': 'Product Test 1.25',
+            'available_in_pos': True,
+            'list_price': 1.25,
+            'taxes_id': False,
+        })
+
+        self.env['product.product'].create({
+            'name': 'Product Test 1.4',
+            'available_in_pos': True,
+            'list_price': 1.4,
+            'taxes_id': False,
+        })
+
+        self.main_pos_config.write({
+            'rounding_method': rouding_method.id,
+            'cash_rounding': True,
+        })
+
+        self.main_pos_config.open_ui()
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'PaymentScreenRoundingHalfUp', login="accountman")
+
+    def test_pos_closing_cash_details(self):
+        """Test if the cash closing details correctly show the cash difference
+           if there is a difference at the opening of the PoS session. This also test if the accounting
+           move are correctly created for the opening cash difference.
+           e.g. If the previous session was closed with 100$ and the opening count is 50$,
+           the closing popup should show a difference of 50$.
+        """
+        self.main_pos_config.open_ui()
+        current_session = self.main_pos_config.current_session_id
+        current_session.post_closing_cash_details(100)
+        current_session.close_session_from_ui()
+
+        self.main_pos_config.open_ui()
+        self.start_tour("/pos/ui?config_id=%d" % self.main_pos_config.id, 'CashClosingDetails', login="accountman")
+        #check accounting move for the pos opening cash difference
+        pos_session = self.main_pos_config.current_session_id
+        self.assertEqual(len(pos_session.statement_line_ids), 1)
+        self.assertEqual(pos_session.statement_line_ids[0].amount, -10)
