@@ -3,29 +3,43 @@ odoo.define('pos_restaurant.TableGuestsButton', function(require) {
 
     const PosComponent = require('point_of_sale.PosComponent');
     const ProductScreen = require('point_of_sale.ProductScreen');
-    const { useListener } = require('web.custom_hooks');
+    const { useListener } = require("@web/core/utils/hooks");
     const Registries = require('point_of_sale.Registries');
 
     class TableGuestsButton extends PosComponent {
-        constructor() {
-            super(...arguments);
+        setup() {
+            super.setup();
             useListener('click', this.onClick);
         }
         get currentOrder() {
             return this.env.pos.get_order();
         }
         get nGuests() {
-            return this.currentOrder ? this.currentOrder.get_customer_count() : 0;
+            return this.currentOrder ? this.currentOrder.getCustomerCount() : 0;
         }
         async onClick() {
             const { confirmed, payload: inputNumber } = await this.showPopup('NumberPopup', {
                 startingValue: this.nGuests,
                 cheap: true,
                 title: this.env._t('Guests ?'),
+                isInputSelected: true
             });
 
             if (confirmed) {
-                this.env.pos.get_order().set_customer_count(parseInt(inputNumber, 10) || 1);
+                const guestCount = parseInt(inputNumber, 10) || 1;
+                // Set the maximum number possible for an integer
+                const max_capacity = 2**31 - 1;
+                if (guestCount > max_capacity) {
+                    await this.showPopup('ErrorPopup', {
+                        title: this.env._t('Blocked action'),
+                        body: _.str.sprintf(
+                            this.env._t('You cannot put a number that exceeds %s '),
+                            max_capacity,
+                        ),
+                    });
+                    return;
+                }
+                this.env.pos.get_order().setCustomerCount(guestCount);
             }
         }
     }
