@@ -3,16 +3,17 @@ odoo.define('pos_restaurant.TransferOrderButton', function(require) {
 
     const PosComponent = require('point_of_sale.PosComponent');
     const ProductScreen = require('point_of_sale.ProductScreen');
-    const { useListener } = require('web.custom_hooks');
+    const { useListener } = require("@web/core/utils/hooks");
     const Registries = require('point_of_sale.Registries');
 
     class TransferOrderButton extends PosComponent {
-        constructor() {
-            super(...arguments);
+        setup() {
+            super.setup();
             useListener('click', this.onClick);
         }
-        onClick() {
-            this.env.pos.transfer_order_to_different_table();
+        async onClick() {
+            this.env.pos.setCurrentOrderToTransfer();
+            this.showScreen('FloorScreen');
         }
     }
     TransferOrderButton.template = 'TransferOrderButton';
