@@ -6,20 +6,25 @@ odoo.define('pos_restaurant.ReceiptScreen', function(require) {
 
     const PosResReceiptScreen = ReceiptScreen =>
         class extends ReceiptScreen {
-            /**
-             * @override
-             */
+            //@override
+            _addNewOrder() {
+                if (!this.env.pos.config.iface_floorplan) {
+                    super._addNewOrder();
+                }
+            }
+            //@override
             get nextScreen() {
-                if (
-                    this.env.pos.config.module_pos_restaurant &&
-                    this.env.pos.config.iface_floorplan
-                ) {
+                if (this.env.pos.config.iface_floorplan) {
                     const table = this.env.pos.table;
                     return { name: 'FloorScreen', props: { floor: table ? table.floor : null } };
                 } else {
                     return super.nextScreen;
                 }
             }
+            onBackToFloorButtonClick() {
+                // If we're here and the order is paid, we can remove it from the orders
+                this.env.pos.removeOrder(this.currentOrder);
+            }
         };
 
     Registries.Component.extend(ReceiptScreen, PosResReceiptScreen);
