@@ -13,7 +13,7 @@ _logger = logging.getLogger(__name__)
 class PosRestaurantAdyenController(AdyenController):
 
     @http.route()
-    def adyen_notification(self, **post):
+    def adyen_webhook(self, **post):
         if post.get('eventCode') in ['CAPTURE', 'AUTHORISATION_ADJUSTMENT'] and post.get('success') != 'true':
                 _logger.warning('%s for transaction_id %s failed', post.get('eventCode'), post.get('originalReference'))
-        return super(PosRestaurantAdyenController, self).adyen_notification(**post)
+        return super(PosRestaurantAdyenController, self).adyen_webhook(**post)
