@@ -11,7 +11,7 @@ class PosPaymentMethod(models.Model):
 
     def _get_adyen_endpoints(self):
         return {
-            **super(PosPaymentMethod, self).get_adyen_endpoints(),
+            **super(PosPaymentMethod, self)._get_adyen_endpoints(),
             'adjust': 'https://pal-%s.adyen.com/pal/servlet/Payment/v52/adjustAuthorisation',
             'capture': 'https://pal-%s.adyen.com/pal/servlet/Payment/v52/capture',
         }
