@@ -5,4 +5,5 @@ from . import pos_config
 from . import pos_order
 from . import crm_team
 from . import pos_session
-from . import account_move
+from . import sale_order
+from . import res_config_settings
