@@ -12,15 +12,17 @@ Please refer to Sale Matrix or Purchase Matrix for the use of this module.
     'version': '1.0',
     'depends': ['account'],
     # Account dependency for section_and_note widget.
-    'qweb': [
-        "static/src/xml/product_matrix.xml",
-    ],
     'data': [
-        'views/assets.xml',
         'views/matrix_templates.xml',
     ],
     'demo': [
         'data/product_matrix_demo.xml',
     ],
+    'assets': {
+        'web.assets_backend': [
+            'product_matrix/static/src/scss/product_matrix.scss',
+            'product_matrix/static/src/xml/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
