@@ -15,6 +15,6 @@ def post_init(cr, registry):
     for hr_leave_type in env['hr.leave.type'].search([('timesheet_generate', '=', True), ('timesheet_project_id', '=', False)]):
         company = hr_leave_type.company_id or env.company
         hr_leave_type.write({
-            'timesheet_project_id': company.leave_timesheet_project_id.id,
+            'timesheet_project_id': company.internal_project_id.id,
             'timesheet_task_id': company.leave_timesheet_task_id.id,
         })
