@@ -3,6 +3,8 @@
 
 from . import models
 from . import report
+from . import populate
+from . import wizard
 
 from odoo import api, SUPERUSER_ID
 
