@@ -17,3 +17,5 @@ from . import test_replenish_wizard
 from . import test_reordering_rule
 from . import test_move_cancel_propagation
 from . import test_product_template
+from . import test_routes
+from . import test_purchase_stock_accrued_entries
