@@ -2,12 +2,14 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 import re
 from datetime import datetime, timedelta
+from freezegun import freeze_time
 
 from odoo.tools import DEFAULT_SERVER_DATETIME_FORMAT
 from odoo.addons.stock_account.tests.test_anglo_saxon_valuation_reconciliation_common import ValuationReconciliationTestCommon
 from odoo.tests import Form, tagged
 
 
+@freeze_time("2021-01-14 09:12:15")
 @tagged('post_install', '-at_install')
 class TestPurchaseOrder(ValuationReconciliationTestCommon):
 
@@ -41,13 +43,13 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
 
     def test_00_purchase_order_flow(self):
         # Ensure product_id_2 doesn't have res_partner_1 as supplier
-        if self.partner_a in self.product_id_2.seller_ids.mapped('name'):
-            id_to_remove = self.product_id_2.seller_ids.filtered(lambda r: r.name == self.partner_a).ids[0] if self.product_id_2.seller_ids.filtered(lambda r: r.name == self.partner_a) else False
+        if self.partner_a in self.product_id_2.seller_ids.partner_id:
+            id_to_remove = self.product_id_2.seller_ids.filtered(lambda r: r.partner_id == self.partner_a).ids[0] if self.product_id_2.seller_ids.filtered(lambda r: r.partner_id == self.partner_a) else False
             if id_to_remove:
                 self.product_id_2.write({
                     'seller_ids': [(2, id_to_remove, False)],
                 })
-        self.assertFalse(self.product_id_2.seller_ids.filtered(lambda r: r.name == self.partner_a), 'Purchase: the partner should not be in the list of the product suppliers')
+        self.assertFalse(self.product_id_2.seller_ids.filtered(lambda r: r.partner_id == self.partner_a), 'Purchase: the partner should not be in the list of the product suppliers')
 
         self.po = self.env['purchase.order'].create(self.po_vals)
         self.assertTrue(self.po, 'Purchase: no purchase order created')
@@ -59,7 +61,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         self.assertEqual(self.po.state, 'purchase', 'Purchase: PO state should be "Purchase"')
         self.assertEqual(self.po.invoice_status, 'to invoice', 'Purchase: PO invoice_status should be "Waiting Invoices"')
 
-        self.assertTrue(self.product_id_2.seller_ids.filtered(lambda r: r.name == self.partner_a), 'Purchase: the partner should be in the list of the product suppliers')
+        self.assertTrue(self.product_id_2.seller_ids.filtered(lambda r: r.partner_id == self.partner_a), 'Purchase: the partner should be in the list of the product suppliers')
 
         seller = self.product_id_2._select_seller(partner_id=self.partner_a, quantity=2.0, date=self.po.date_planned, uom_id=self.product_id_2.uom_po_id)
         price_unit = seller.price if seller else 0.0
@@ -67,7 +69,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
             price_unit = seller.currency_id._convert(price_unit, self.po.currency_id, self.po.company_id, self.po.date_order)
         self.assertEqual(price_unit, 250.0, 'Purchase: the price of the product for the supplier should be 250.0.')
 
-        self.assertEqual(self.po.picking_count, 1, 'Purchase: one picking should be created"')
+        self.assertEqual(self.po.incoming_picking_count, 1, 'Purchase: one picking should be created"')
         self.picking = self.po.picking_ids[0]
         self.picking.move_line_ids.write({'qty_done': 5.0})
         self.picking.button_validate()
@@ -75,7 +77,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
 
         move_form = Form(self.env['account.move'].with_context(default_move_type='in_invoice'))
         move_form.partner_id = self.partner_a
-        move_form.purchase_id = self.po
+        move_form.purchase_vendor_bill_id = self.env['purchase.bill.union'].browse(-self.po.id)
         self.invoice = move_form.save()
 
         self.assertEqual(self.po.order_line.mapped('qty_invoiced'), [5.0, 5.0], 'Purchase: all products should be invoiced"')
@@ -98,7 +100,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         # Confirm the purchase order
         self.po.button_confirm()
         self.assertEqual(self.po.state, 'purchase', 'Purchase: PO state should be "Purchase')
-        self.assertEqual(self.po.picking_count, 1, 'Purchase: one picking should be created"')
+        self.assertEqual(self.po.incoming_picking_count, 1, 'Purchase: one picking should be created"')
         self.picking = self.po.picking_ids[0]
         self.picking.move_line_ids.write({'qty_done': 5.0})
         self.picking.button_validate()
@@ -108,7 +110,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         move_form = Form(self.env['account.move'].with_context(default_move_type='in_invoice'))
         move_form.invoice_date = move_form.date
         move_form.partner_id = self.partner_a
-        move_form.purchase_id = self.po
+        move_form.purchase_vendor_bill_id = self.env['purchase.bill.union'].browse(-self.po.id)
         self.invoice = move_form.save()
         self.invoice.action_post()
 
@@ -139,6 +141,13 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         move_form = Form(self.env['account.move'].with_context(default_move_type='in_refund'))
         move_form.invoice_date = move_form.date
         move_form.partner_id = self.partner_a
+        # Not supposed to see/change the purchase order of a refund invoice by default
+        # <field name="purchase_id" invisible="1"/>
+        # <label for="purchase_vendor_bill_id" string="Auto-Complete" class="oe_edit_only"
+        #         attrs="{'invisible': ['|', ('state','!=','draft'), ('move_type', '!=', 'in_invoice')]}" />
+        # <field name="purchase_vendor_bill_id" nolabel="1"
+        #         attrs="{'invisible': ['|', ('state','!=','draft'), ('move_type', '!=', 'in_invoice')]}"
+        move_form._view['modifiers']['purchase_id']['invisible'] = False
         move_form.purchase_id = self.po
         self.invoice = move_form.save()
         move_form = Form(self.invoice)
@@ -212,7 +221,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
 
         # A new move of 10 unit (15 - 5 units)
         self.assertEqual(po1.order_line.qty_received, 5)
-        self.assertEqual(po1.picking_ids[-1].move_lines.product_qty, 10)
+        self.assertEqual(po1.picking_ids[-1].move_ids.product_qty, 10)
 
     def test_04_update_date_planned(self):
         today = datetime.today().replace(hour=9, microsecond=0)
@@ -229,8 +238,10 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
             ('res_id', '=', po.id),
         ])
         self.assertTrue(activity)
-        self.assertIn(
-            '<p> partner_a modified receipt dates for the following products:</p><p> \xa0 - Large Desk from %s to %s </p><p>Those dates have been updated accordingly on the receipt %s.</p>' % (today.date(), tomorrow.date(), po.picking_ids.name),
+        self.assertEqual(
+            '<p>partner_a modified receipt dates for the following products:</p>\n'
+            '<p> - Large Desk from %s to %s</p>\n'
+            '<p>Those dates have been updated accordingly on the receipt %s.</p>' % (today.date(), tomorrow.date(), po.picking_ids.name),
             activity.note,
         )
 
@@ -243,8 +254,12 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         old_date = po.order_line[1].date_planned
         po._update_date_planned_for_lines([(po.order_line[1], tomorrow)])
         self.assertEqual(po.order_line[1].date_planned, old_date)
-        self.assertIn(
-            '<p> partner_a modified receipt dates for the following products:</p><p> \xa0 - Large Desk from %s to %s </p><p> \xa0 - Conference Chair from %s to %s </p><p>Those dates couldn’t be modified accordingly on the receipt %s which had already been validated.</p>' % (today.date(), tomorrow.date(), today.date(), tomorrow.date(), po.picking_ids.name),
+        self.assertEqual(
+            '<p>partner_a modified receipt dates for the following products:</p>\n'
+            '<p> - Large Desk from %s to %s</p>\n'
+            '<p> - Conference Chair from %s to %s</p>\n'
+            '<p>Those dates couldn’t be modified accordingly on the receipt %s which had already been validated.</p>' % (
+                today.date(), tomorrow.date(), today.date(), tomorrow.date(), po.picking_ids.name),
             activity.note,
         )
 
@@ -294,7 +309,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         self.assertEqual(expected_rate, po.on_time_rate)
 
         # Create a purchase order with 80% qty received for company B
-        # The On-Time Delivery Rate shouldn't be shared accross multiple companies
+        # The On-Time Delivery Rate shouldn't be shared across multiple companies
         self.env.user.write({
             'company_id': company_b.id,
             'company_ids': [(6, 0, [company_b.id])],
@@ -325,7 +340,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         # 1. Compute `res.partner.on_time_rate` regular non-stored comptued field
         partner_on_time_rate = po.partner_id.on_time_rate
         # 2. Invalidate the cache for that record and field, so it's not reused in the next step.
-        po.partner_id.invalidate_cache(fnames=["on_time_rate"], ids=po.partner_id.ids)
+        po.partner_id.invalidate_recordset(["on_time_rate"])
         # 3. Compute the related field `purchase.order.on_time_rate`
         po_on_time_rate = po.on_time_rate
         # 4. Check both are equals.
@@ -361,20 +376,63 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         picking.button_validate()
         self.assertEqual(po.order_line.mapped('qty_received'), [4.0], 'Purchase: no conversion error on receipt in different uom"')
 
-    def test_message_qty_already_received(self):
-        _product = self.env['product.product'].create({
-            'name': 'TempProduct',
-            'type': 'consu',
-            'company_id': self.env.user.company_id.id,
+    def test_05_po_update_qty_stock_move_merge(self):
+        """ This test ensures that changing product quantity when unit price has high decimal precision
+            merged with the original instead of creating a new return
+        """
+
+        unit_price_precision = self.env['decimal.precision'].search([('name', '=', 'Product Price')])
+        unit_price_precision.digits = 6
+
+        tax = self.env["account.tax"].create({
+            "name": "Dummy Tax",
+            "amount": "5.00",
+            "type_tax_use": "purchase",
         })
 
+        super_product = self.env['product.product'].create({
+            'name': 'Super Product',
+            'type': 'product',
+            'categ_id': self.stock_account_product_categ.id,
+            'standard_price': 9.876543,
+        })
+
+        purchase_order = self.env['purchase.order'].create({
+            'partner_id': self.env.ref('base.res_partner_3').id,
+            'order_line': [(0, 0, {
+                'name': super_product.name,
+                'product_id': super_product.id,
+                'product_qty': 7,
+                'product_uom': super_product.uom_id.id,
+                'price_unit': super_product.standard_price,
+                'taxes_id': [(4, tax.id)],
+            })],
+        })
+
+        purchase_order.button_confirm()
+        self.assertEqual(purchase_order.state, 'purchase')
+        self.assertEqual(len(purchase_order.picking_ids), 1)
+        self.assertEqual(len(purchase_order.picking_ids.move_line_ids), 1)
+        self.assertEqual(purchase_order.picking_ids.move_line_ids.reserved_qty, 7)
+
+
+        purchase_order.order_line.product_qty = 4
+        # updating quantity shouldn't create a seperate stock move
+        # the new stock move (-3) should be merged with the previous
+        purchase_order.button_confirm()
+        self.assertEqual(len(purchase_order.picking_ids), 1)
+        self.assertEqual(len(purchase_order.picking_ids.move_line_ids), 1)
+        self.assertEqual(purchase_order.picking_ids.move_line_ids.reserved_qty, 4)
+
+    def test_message_qty_already_received(self):
+        self.env.user.write({'company_id': self.company_data['company'].id})
+
         _purchase_order = self.env['purchase.order'].create({
-            'company_id': self.env.user.company_id.id,
             'partner_id': self.partner_a.id,
             'order_line': [
                 (0, 0, {
-                    'name': _product.name,
-                    'product_id': _product.id,
+                    'name': self.product_id_2.name,
+                    'product_id': self.product_id_2.id,
                     'product_qty': 25.0,
                     'price_unit': 250.0,
                 })],
@@ -383,22 +441,117 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         _purchase_order.button_confirm()
 
         first_picking = _purchase_order.picking_ids[0]
-        first_picking.move_lines.quantity_done = 5
+        first_picking.move_ids.quantity_done = 5
         backorder_wizard_dict = first_picking.button_validate()
         backorder_wizard = Form(self.env[backorder_wizard_dict['res_model']].with_context(backorder_wizard_dict['context'])).save()
         backorder_wizard.process()
 
         second_picking = _purchase_order.picking_ids[1]
-        second_picking.move_lines.quantity_done = 5
+        second_picking.move_ids.quantity_done = 5
         backorder_wizard_dict = second_picking.button_validate()
         backorder_wizard = Form(self.env[backorder_wizard_dict['res_model']].with_context(backorder_wizard_dict['context'])).save()
         backorder_wizard.process()
 
         third_picking = _purchase_order.picking_ids[2]
-        third_picking.move_lines.quantity_done = 5
+        third_picking.move_ids.quantity_done = 5
         backorder_wizard_dict = third_picking.button_validate()
         backorder_wizard = Form(self.env[backorder_wizard_dict['res_model']].with_context(backorder_wizard_dict['context'])).save()
         backorder_wizard.process()
 
         _message_content = _purchase_order.message_ids.mapped("body")[0]
         self.assertIsNotNone(re.search(r"Received Quantity: 5.0 -&gt; 10.0", _message_content), "Already received quantity isn't correctly taken into consideration")
+
+    def test_pol_description(self):
+        """
+        Suppose a product with several sellers, all with the same partner. On the purchase order, the product
+        description should be based on the correct seller
+        """
+        self.env.user.write({'company_id': self.company_data['company'].id})
+
+        product = self.env['product.product'].create({
+            'name': 'Super Product',
+            'seller_ids': [(0, 0, {
+                'partner_id': self.partner_a.id,
+                'min_qty': 1,
+                'price': 10,
+                'product_code': 'C01',
+                'product_name': 'Name01',
+                'sequence': 1,
+            }), (0, 0, {
+                'partner_id': self.partner_a.id,
+                'min_qty': 20,
+                'price': 2,
+                'product_code': 'C02',
+                'product_name': 'Name02',
+                'sequence': 2,
+            })]
+        })
+
+        orderpoint_form = Form(self.env['stock.warehouse.orderpoint'])
+        orderpoint_form.product_id = product
+        orderpoint_form.product_min_qty = 1
+        orderpoint_form.product_max_qty = 0.000
+        orderpoint_form.save()
+
+        self.env['procurement.group'].run_scheduler()
+
+        pol = self.env['purchase.order.line'].search([('product_id', '=', product.id)])
+        self.assertEqual(pol.name, "[C01] Name01")
+
+        with Form(pol.order_id) as po_form:
+            with po_form.order_line.edit(0) as pol_form:
+                pol_form.product_qty = 25
+        self.assertEqual(pol.name, "[C02] Name02")
+
+    def test_packaging_and_qty_decrease(self):
+        packaging = self.env['product.packaging'].create({
+            'name': "Super Packaging",
+            'product_id': self.product_a.id,
+            'qty': 10.0,
+        })
+
+        po_form = Form(self.env['purchase.order'])
+        po_form.partner_id = self.partner_a
+        with po_form.order_line.new() as line:
+            line.product_id = self.product_a
+            line.product_qty = 10
+        po = po_form.save()
+        po.button_confirm()
+
+        self.assertEqual(po.order_line.product_packaging_id, packaging)
+
+        with Form(po) as po_form:
+            with po_form.order_line.edit(0) as line:
+                line.product_qty = 8
+
+        self.assertEqual(po.picking_ids.move_ids.product_uom_qty, 8)
+
+    def test_putaway_strategy_in_backorder(self):
+        stock_location = self.company_data['default_warehouse'].lot_stock_id
+        sub_loc_01 = self.env['stock.location'].create([{
+            'name': 'Sub Location 1',
+            'usage': 'internal',
+            'location_id': stock_location.id,
+        }])
+        self.env["stock.putaway.rule"].create({
+            "location_in_id": stock_location.id,
+            "location_out_id": sub_loc_01.id,
+            "product_id": self.product_a.id,
+        })
+        po = self.env['purchase.order'].create({
+            'partner_id': self.partner_a.id,
+            'order_line': [
+                (0, 0, {
+                    'product_id': self.product_a.id,
+                    'product_qty': 2.0,
+                })],
+        })
+        po.button_confirm()
+        picking = po.picking_ids
+        self.assertEqual(po.state, "purchase")
+        self.assertEqual(picking.move_line_ids_without_package.location_dest_id.id, sub_loc_01.id)
+        picking.move_line_ids_without_package.write({'qty_done': 1})
+        res_dict = picking.button_validate()
+        self.env[res_dict['res_model']].with_context(res_dict['context']).process()
+        backorder = picking.backorder_ids
+        self.assertEqual(backorder.move_line_ids_without_package.location_dest_id.id, sub_loc_01.id)
