@@ -19,8 +19,14 @@ associated to every resource. It also manages the leaves of every resource.
         'security/ir.model.access.csv',
         'security/resource_security.xml',
         'views/resource_views.xml',
-        'views/resource.xml',
-    ],
-    'demo': [
     ],
+    'assets': {
+        'web.assets_backend': [
+            'resource/static/src/**/*',
+        ],
+        'web.qunit_suite_tests': [
+            'resource/static/tests/components/*.js',
+        ],
+    },
+    'license': 'LGPL-3',
 }
