@@ -10,6 +10,7 @@ var _t = core._t;
 var VariantMixin = {
     events: {
         'change .css_attribute_color input': '_onChangeColorAttribute',
+        'change .o_variant_pills input' :'_onChangePillsAttribute',
         'change .main_product:not(.in_cart) input.js_quantity': 'onChangeAddQuantity',
         'change [data-attribute_exclusions]': 'onChangeVariant'
     },
@@ -49,32 +50,67 @@ var VariantMixin = {
      * @returns {Deferred}
      */
     _getCombinationInfo: function (ev) {
-        var self = this;
-
         if ($(ev.target).hasClass('variant_custom_value')) {
             return Promise.resolve();
         }
 
-        var $parent = $(ev.target).closest('.js_product');
-        var qty = $parent.find('input[name="add_qty"]').val();
-        var combination = this.getSelectedVariantValues($parent);
-        var parentCombination = $parent.find('ul[data-attribute_exclusions]').data('attribute_exclusions').parent_combination;
-        var productTemplateId = parseInt($parent.find('.product_template_id').val());
+        const $parent = $(ev.target).closest('.js_product');
+        if(!$parent.length){
+            return Promise.resolve();
+        }
+        const combination = this.getSelectedVariantValues($parent);
+        let parentCombination;
 
-        self._checkExclusions($parent, combination);
+        if ($parent.hasClass('main_product')) {
+            parentCombination = $parent.find('ul[data-attribute_exclusions]').data('attribute_exclusions').parent_combination;
+            const $optProducts = $parent.parent().find(`[data-parent-unique-id='${$parent.data('uniqueId')}']`);
+
+            for (const optionalProduct of $optProducts) {
+                const $currentOptionalProduct = $(optionalProduct);
+                const childCombination = this.getSelectedVariantValues($currentOptionalProduct);
+                const productTemplateId = parseInt($currentOptionalProduct.find('.product_template_id').val());
+                ajax.jsonRpc(this._getUri('/sale/get_combination_info'), 'call', {
+                    'product_template_id': productTemplateId,
+                    'product_id': this._getProductId($currentOptionalProduct),
+                    'combination': childCombination,
+                    'add_qty': parseInt($currentOptionalProduct.find('input[name="add_qty"]').val()),
+                    'pricelist_id': this.pricelistId || false,
+                    'parent_combination': combination,
+                    ...this._getOptionalCombinationInfoParam($currentOptionalProduct),
+                }).then((combinationData) => {
+                    this._onChangeCombination(ev, $currentOptionalProduct, combinationData);
+                    this._checkExclusions($currentOptionalProduct, childCombination, combinationData.parent_exclusions);
+                });
+            }
+        } else {
+            parentCombination = this.getSelectedVariantValues(
+                $parent.parent().find('.js_product.in_cart.main_product')
+            );
+        }
 
         return ajax.jsonRpc(this._getUri('/sale/get_combination_info'), 'call', {
-            'product_template_id': productTemplateId,
+            'product_template_id': parseInt($parent.find('.product_template_id').val()),
             'product_id': this._getProductId($parent),
             'combination': combination,
-            'add_qty': parseInt(qty),
+            'add_qty': parseInt($parent.find('input[name="add_qty"]').val()),
             'pricelist_id': this.pricelistId || false,
             'parent_combination': parentCombination,
-        }).then(function (combinationData) {
-            self._onChangeCombination(ev, $parent, combinationData);
+            ...this._getOptionalCombinationInfoParam($parent),
+        }).then((combinationData) => {
+            this._onChangeCombination(ev, $parent, combinationData);
+            this._checkExclusions($parent, combination, combinationData.parent_exclusions);
         });
     },
 
+    /**
+     * Hook to add optional info to the combination info call.
+     *
+     * @param {$.Element} $product
+     */
+    _getOptionalCombinationInfoParam($product) {
+        return {};
+    },
+
     /**
      * Will add the "custom value" input for this attribute value if
      * the attribute value is configured as "custom" (see product_attribute_value.is_custom)
@@ -105,23 +141,19 @@ var VariantMixin = {
                               .data('custom_product_template_attribute_value_id') !== parseInt(attributeValueId)) {
                     $variantContainer.find('.variant_custom_value').remove();
 
+                    const previousCustomValue = $customInput.attr("previous_custom_value");
                     var $input = $('<input>', {
                         type: 'text',
                         'data-custom_product_template_attribute_value_id': attributeValueId,
                         'data-attribute_value_name': attributeValueName,
-                        class: 'variant_custom_value form-control'
+                        class: 'variant_custom_value form-control mt-2'
                     });
 
-                    var isRadioInput = $target.is('input[type=radio]') &&
-                        $target.closest('label.css_attribute_color').length === 0;
-
-                    if (isRadioInput && $customInput.data('is_single_and_custom') !== 'True') {
-                        $input.addClass('custom_value_radio');
-                        $target.closest('div').after($input);
-                    } else {
-                        $input.attr('placeholder', attributeValueName);
-                        $input.addClass('custom_value_own_line');
-                        $variantContainer.append($input);
+                    $input.attr('placeholder', attributeValueName);
+                    $input.addClass('custom_value_radio');
+                    $variantContainer.append($input);
+                    if (previousCustomValue) {
+                        $input.val(previousCustomValue);
                     }
                 }
             } else {
@@ -160,8 +192,8 @@ var VariantMixin = {
     onChangeAddQuantity: function (ev) {
         var $parent;
 
-        if ($(ev.currentTarget).closest('.oe_optional_products_modal').length > 0){
-            $parent = $(ev.currentTarget).closest('.oe_optional_products_modal');
+        if ($(ev.currentTarget).closest('.oe_advanced_configurator_modal').length > 0){
+            $parent = $(ev.currentTarget).closest('.oe_advanced_configurator_modal');
         } else if ($(ev.currentTarget).closest('form').length > 0){
             $parent = $(ev.currentTarget).closest('form');
         }  else {
@@ -177,10 +209,9 @@ var VariantMixin = {
      * @param {$.Element} $container
      */
     triggerVariantChange: function ($container) {
-        var self = this;
         $container.find('ul[data-attribute_exclusions]').trigger('change');
         $container.find('input.js_variant_change:checked, select.js_variant_change').each(function () {
-            self.handleCustomValues($(this));
+            VariantMixin.handleCustomValues($(this));
         });
     },
 
@@ -231,12 +262,13 @@ var VariantMixin = {
 
         $container.find(variantsValuesSelectors.join(',')).each(function (){
             var $variantValueInput = $(this);
+            var singleNoCustom = $variantValueInput.data('is_single') && !$variantValueInput.data('is_custom');
 
             if ($variantValueInput.is('select')){
                 $variantValueInput = $variantValueInput.find('option[value=' + $variantValueInput.val() + ']');
             }
 
-            if ($variantValueInput.length !== 0){
+            if ($variantValueInput.length !== 0 && !singleNoCustom){
                 noVariantAttributeValues.push({
                     'custom_product_template_attribute_value_id': $variantValueInput.data('value_id'),
                     'attribute_value_name': $variantValueInput.data('value_name'),
@@ -288,7 +320,6 @@ var VariantMixin = {
      * @returns {Promise} the promise that will be resolved with a {integer} productId
      */
     selectOrCreateProduct: function ($container, productId, productTemplateId, useAjax) {
-        var self = this;
         productId = parseInt(productId);
         productTemplateId = parseInt(productTemplateId);
         var productReady = Promise.resolve();
@@ -298,14 +329,17 @@ var VariantMixin = {
             var params = {
                 product_template_id: productTemplateId,
                 product_template_attribute_value_ids:
-                    JSON.stringify(self.getSelectedVariantValues($container)),
+                    JSON.stringify(VariantMixin.getSelectedVariantValues($container)),
             };
 
             var route = '/sale/create_product_variant';
             if (useAjax) {
                 productReady = ajax.jsonRpc(route, 'call', params);
-            } else {
+            } else if (Boolean(this._rpc)) {
+                // HACK to combine owl and non owl calls
                 productReady = this._rpc({route: route, params: params});
+            } else {
+                productReady = this.rpc(route, params);
             }
         }
 
@@ -331,18 +365,22 @@ var VariantMixin = {
      * @private
      * @param {$.Element} $parent the parent container to apply exclusions
      * @param {Array} combination the selected combination of product attribute values
+     * @param {Array} parentExclusions the exclusions induced by the variant selection of the parent product
+     * For example chair cannot have steel legs if the parent Desk doesn't have steel legs
      */
-    _checkExclusions: function ($parent, combination) {
+    _checkExclusions: function ($parent, combination, parentExclusions) {
         var self = this;
         var combinationData = $parent
             .find('ul[data-attribute_exclusions]')
             .data('attribute_exclusions');
 
+        if (parentExclusions && combinationData.parent_exclusions) {
+            combinationData.parent_exclusions = parentExclusions;
+        }
         $parent
-            .find('option, input, label')
+            .find('option, input, label, .o_variant_pills')
             .removeClass('css_not_available')
-            .prop('disabled', false)
-            .attr('title', '')
+            .attr('title', function () { return $(this).data('value_name') || ''; })
             .data('excluded-by', '');
 
         // exclusion rules: array of ptav
@@ -365,6 +403,52 @@ var VariantMixin = {
                 }
             });
         }
+        // combination exclusions: array of array of ptav
+        // for example a product with 3 variation and one specific variation is disabled (archived)
+        //  requires the first 2 to be selected for the third to be disabled
+        if (combinationData.archived_combinations) {
+            combinationData.archived_combinations.forEach((excludedCombination) => {
+                const ptavCommon = excludedCombination.filter((ptav) => combination.includes(ptav));
+                if (
+                    !!ptavCommon
+                    && (combination.length === excludedCombination.length)
+                    && (ptavCommon.length === combination.length)
+                ) {
+                    // Selected combination is archived, all attributes must be disabled from each other
+                    combination.forEach((ptav) => {
+                        combination.forEach((ptavOther) => {
+                            if (ptav === ptavOther) {
+                                return;
+                            }
+                            self._disableInput(
+                                $parent,
+                                ptav,
+                                ptavOther,
+                                combinationData.mapped_attribute_names,
+                            );
+                        })
+                    })
+                } else if (
+                    !!ptavCommon
+                    && (combination.length === excludedCombination.length)
+                    && (ptavCommon.length === (combination.length - 1))
+                ) {
+                    // In this case we only need to disable the remaining ptav
+                    const disabledPtav = excludedCombination.find((ptav) => !combination.includes(ptav));
+                    excludedCombination.forEach((ptav) => {
+                        if (ptav === disabledPtav) {
+                            return;
+                        }
+                        self._disableInput(
+                            $parent,
+                            disabledPtav,
+                            ptav,
+                            combinationData.mapped_attribute_names,
+                        )
+                    });
+                }
+            });
+        }
 
         // parent exclusions (tell which attributes are excluded from parent)
         _.each(combinationData.parent_exclusions, function (exclusions, excluded_by){
@@ -414,7 +498,7 @@ var VariantMixin = {
             .find('option[value=' + attributeValueId + '], input[value=' + attributeValueId + ']');
         $input.addClass('css_not_available');
         $input.closest('label').addClass('css_not_available');
-        $input.prop('disabled', true);
+        $input.closest('.o_variant_pills').addClass('css_not_available');
 
         if (excludedBy && attributeNames) {
             var $target = $input.is('option') ? $input : $input.closest('label').add($input);
@@ -572,6 +656,11 @@ var VariantMixin = {
      */
     _toggleDisable: function ($parent, isCombinationPossible) {
         $parent.toggleClass('css_not_available', !isCombinationPossible);
+        if ($parent.hasClass('in_cart')) {
+            const primaryButton = $parent.parents('.modal-content').find('.modal-footer .btn-primary');
+            primaryButton.prop('disabled', !isCombinationPossible);
+            primaryButton.toggleClass('disabled', !isCombinationPossible);
+        }
     },
     /**
      * Updates the product image.
@@ -622,6 +711,14 @@ var VariantMixin = {
             .addClass("active");
     },
 
+    _onChangePillsAttribute: function (ev) {
+        var $parent = $(ev.target).closest('.js_product');
+        $parent.find('.o_variant_pills')
+            .removeClass("active")
+            .filter(':has(input:checked)')
+            .addClass("active");
+    },
+
     /**
      * Extension point for website_sale
      *
