@@ -1,9 +1,17 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import api, fields, models
+from odoo import models
 
 
+class StockMove(models.Model):
+    _inherit = 'stock.move'
+
+    def _prepare_procurement_values(self):
+        res = super()._prepare_procurement_values()
+        res['analytic_account_id'] = self.sale_line_id.order_id.analytic_account_id
+        return res
+
 
 class StockMoveLine(models.Model):
     _inherit = 'stock.move.line'
