@@ -11,13 +11,17 @@
     'version': '1.0',
     'depends': ['sale', 'product_matrix', 'sale_product_configurator'],
     'data': [
-        'views/assets.xml',
         'views/product_template_views.xml',
-        'views/sale_views.xml',
+        'views/sale_order_views.xml',
         'report/sale_report_templates.xml',
     ],
     'demo': [
         'data/product_matrix_demo.xml'
     ],
+    'assets': {
+        'web.assets_backend': [
+            'sale_product_matrix/static/src/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
