@@ -13,10 +13,19 @@ This module allows to generate a project/task from sales orders.
     'data': [
         'security/ir.model.access.csv',
         'security/sale_project_security.xml',
+        'report/project_report_views.xml',
         'views/product_views.xml',
         'views/project_task_views.xml',
         'views/sale_order_views.xml',
+        'views/sale_project_portal_templates.xml',
+        'views/project_sharing_views.xml',
+        'views/project_views.xml',
     ],
+    'assets': {
+        'web.assets_backend': [
+            'sale_project/static/src/components/project_right_side_panel/**/*',
+        ],
+    },
     'auto_install': True,
     'license': 'LGPL-3',
 }
