@@ -4,4 +4,8 @@
 from . import product_template
 from . import res_company
 from . import sale_order
+from . import sale_order_line
+from . import sale_order_option
 from . import sale_order_template
+from . import sale_order_template_line
+from . import sale_order_template_option
