@@ -3,8 +3,12 @@
 
 from . import test_anglo_saxon_valuation
 from . import test_anglo_saxon_valuation_reconciliation
+from . import test_anglosaxon_account
 from . import test_sale_stock
 from . import test_sale_stock_lead_time
 from . import test_sale_stock_report
 from . import test_sale_order_dates
 from . import test_sale_stock_multicompany
+from . import test_sale_stock_accrued_entries
+from . import test_sale_stock_access_rights
+from . import test_create_perf
