@@ -22,19 +22,11 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         })
 
     def _inv_adj_two_units(self):
-        inventory = self.env['stock.inventory'].create({
-            'name': 'test',
-            'location_ids': [(4, self.company_data['default_warehouse'].lot_stock_id.id)],
-            'product_ids': [(4, self.product.id)],
-        })
-        inventory.action_start()
-        self.env['stock.inventory.line'].create({
-            'inventory_id': inventory.id,
+        self.env['stock.quant'].with_context(inventory_mode=True).create({
+            'product_id': self.product.id,  # tracking serial
+            'inventory_quantity': 2,
             'location_id': self.company_data['default_warehouse'].lot_stock_id.id,
-            'product_id': self.product.id,
-            'product_qty': 2,
-        })
-        inventory.action_validate()
+        }).action_apply_inventory()
 
     def _so_and_confirm_two_units(self):
         sale_order = self.env['sale.order'].create({
@@ -134,7 +126,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order = self._so_and_confirm_two_units()
 
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 1
+        sale_order.picking_ids.move_ids.quantity_done = 1
         wiz = sale_order.picking_ids.button_validate()
         wiz = Form(self.env[wiz['res_model']].with_context(wiz['context'])).save()
         wiz.process()
@@ -167,7 +159,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         self.product.standard_price = 14.0
 
         # deliver the backorder
-        sale_order.picking_ids[0].move_lines.quantity_done = 1
+        sale_order.picking_ids[0].move_ids.quantity_done = 1
         sale_order.picking_ids[0].button_validate()
 
         # change the standard price to 16
@@ -206,7 +198,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order = self._so_and_confirm_two_units()
 
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 1
+        sale_order.picking_ids.move_ids.quantity_done = 1
         wiz = sale_order.picking_ids.button_validate()
         wiz = Form(self.env[wiz['res_model']].with_context(wiz['context'])).save()
         wiz.process()
@@ -215,7 +207,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         self.product.standard_price = 14.0
 
         # deliver the backorder
-        sale_order.picking_ids.filtered('backorder_id').move_lines.quantity_done = 1
+        sale_order.picking_ids.filtered('backorder_id').move_ids.quantity_done = 1
         sale_order.picking_ids.filtered('backorder_id').button_validate()
 
         # Invoice the sale order.
@@ -273,7 +265,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order = self._so_and_confirm_two_units()
 
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 1
+        sale_order.picking_ids.move_ids.quantity_done = 1
         wiz = sale_order.picking_ids.button_validate()
         wiz = Form(self.env[wiz['res_model']].with_context(wiz['context'])).save()
         wiz.process()
@@ -306,7 +298,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         self.product.standard_price = 14.0
 
         # deliver the backorder
-        sale_order.picking_ids[0].move_lines.quantity_done = 1
+        sale_order.picking_ids[0].move_ids.quantity_done = 1
         sale_order.picking_ids[0].button_validate()
 
         # change the standard price to 16
@@ -345,7 +337,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order = self._so_and_confirm_two_units()
 
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 1
+        sale_order.picking_ids.move_ids.quantity_done = 1
         wiz = sale_order.picking_ids.button_validate()
         wiz = Form(self.env[wiz['res_model']].with_context(wiz['context'])).save()
         wiz.process()
@@ -354,7 +346,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         self.product.standard_price = 14.0
 
         # deliver the backorder
-        sale_order.picking_ids.filtered('backorder_id').move_lines.quantity_done = 1
+        sale_order.picking_ids.filtered('backorder_id').move_ids.quantity_done = 1
         sale_order.picking_ids.filtered('backorder_id').button_validate()
 
         # Invoice the sale order.
@@ -425,7 +417,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order = self._so_and_confirm_two_units()
 
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 1
+        sale_order.picking_ids.move_ids.quantity_done = 1
         wiz = sale_order.picking_ids.button_validate()
         wiz = Form(self.env[wiz['res_model']].with_context(wiz['context'])).save()
         wiz.process()
@@ -463,7 +455,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order = self._so_and_confirm_two_units()
 
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 2
+        sale_order.picking_ids.move_ids.quantity_done = 2
         sale_order.picking_ids.button_validate()
 
         # Invoice the sale order.
@@ -510,7 +502,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         so.action_confirm()
 
         pick = so.picking_ids
-        pick.move_lines.write({'quantity_done': 5})
+        pick.move_ids.write({'quantity_done': 5})
         pick.button_validate()
 
         product.standard_price = 40
@@ -523,7 +515,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         res = return_wiz.create_returns()
 
         return_pick = self.env['stock.picking'].browse(res['res_id'])
-        return_pick.move_lines.write({'quantity_done': 1})
+        return_pick.move_ids.write({'quantity_done': 1})
         return_pick.button_validate()
 
         picking = self.env['stock.picking'].create({
@@ -580,7 +572,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order = self._so_and_confirm_two_units()
 
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 1
+        sale_order.picking_ids.move_ids.quantity_done = 1
         wiz = sale_order.picking_ids.button_validate()
         wiz = Form(self.env[wiz['res_model']].with_context(wiz['context'])).save()
         wiz.process()
@@ -617,7 +609,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         # Create and confirm a sale order for 2@12
         sale_order = self._so_and_confirm_two_units()
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 2
+        sale_order.picking_ids.move_ids.quantity_done = 2
         sale_order.picking_ids.button_validate()
 
         # Invoice the sale order.
@@ -640,6 +632,74 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         self.assertEqual(income_aml.debit, 0)
         self.assertEqual(income_aml.credit, 24)
 
+    def test_avco_partially_owned_and_delivered_invoice_post_delivery(self):
+        """
+        Standard price set to 10. Sale order 2@12. One of the delivered
+        products was owned by an external partner. Invoice after full delivery.
+        """
+        self.product.categ_id.property_cost_method = 'average'
+        self.product.invoice_policy = 'delivery'
+        self.product.standard_price = 10
+
+        self.env['stock.quant']._update_available_quantity(self.product, self.company_data['default_warehouse'].lot_stock_id, 1, owner_id=self.partner_b)
+        self.env['stock.quant']._update_available_quantity(self.product, self.company_data['default_warehouse'].lot_stock_id, 1)
+
+        # Create and confirm a sale order for 2@12
+        sale_order = self._so_and_confirm_two_units()
+        # Deliver both products (there should be two SML)
+        sale_order.picking_ids.move_line_ids.qty_done = 1
+        sale_order.picking_ids.button_validate()
+
+        # Invoice one by one
+        invoice01 = sale_order._create_invoices()
+        with Form(invoice01) as invoice_form:
+            with invoice_form.invoice_line_ids.edit(0) as line_form:
+                line_form.quantity = 1
+        invoice01.action_post()
+
+        invoice02 = sale_order._create_invoices()
+        invoice02.action_post()
+
+        # COGS should ignore the owned product
+        self.assertRecordValues(invoice01.line_ids, [
+            # pylint: disable=bad-whitespace
+            {'account_id': self.company_data['default_account_revenue'].id,     'debit': 0,     'credit': 12},
+            {'account_id': self.company_data['default_account_receivable'].id,  'debit': 12,    'credit': 0},
+            {'account_id': self.company_data['default_account_stock_out'].id,   'debit': 0,     'credit': 10},
+            {'account_id': self.company_data['default_account_expense'].id,     'debit': 10,    'credit': 0},
+        ])
+        self.assertRecordValues(invoice02.line_ids, [
+            # pylint: disable=bad-whitespace
+            {'account_id': self.company_data['default_account_revenue'].id,     'debit': 0,     'credit': 12},
+            {'account_id': self.company_data['default_account_receivable'].id,  'debit': 12,    'credit': 0},
+        ])
+
+    def test_avco_fully_owned_and_delivered_invoice_post_delivery(self):
+        """
+        Standard price set to 10. Sale order 2@12. The products are owned by an
+        external partner. Invoice after full delivery.
+        """
+        self.product.categ_id.property_cost_method = 'average'
+        self.product.invoice_policy = 'delivery'
+        self.product.standard_price = 10
+
+        self.env['stock.quant']._update_available_quantity(self.product, self.company_data['default_warehouse'].lot_stock_id, 2, owner_id=self.partner_b)
+
+        sale_order = self._so_and_confirm_two_units()
+        sale_order.picking_ids.move_line_ids.qty_done = 2
+        sale_order.picking_ids.button_validate()
+
+        invoice = sale_order._create_invoices()
+        invoice.action_post()
+
+        # COGS should not exist because the products are owned by an external partner
+        amls = invoice.line_ids
+        self.assertRecordValues(amls, [
+            # pylint: disable=bad-whitespace
+            {'account_id': self.company_data['default_account_revenue'].id,     'debit': 0,     'credit': 24},
+            {'account_id': self.company_data['default_account_receivable'].id,  'debit': 24,    'credit': 0},
+        ])
+
     # -------------------------------------------------------------------------
     # FIFO Ordered
     # -------------------------------------------------------------------------
@@ -686,7 +746,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order = self._so_and_confirm_two_units()
 
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 1
+        sale_order.picking_ids.move_ids.quantity_done = 1
         wiz = sale_order.picking_ids.button_validate()
         wiz = Form(self.env[wiz['res_model']].with_context(wiz['context'])).save()
         wiz.process()
@@ -729,7 +789,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order = self._so_and_confirm_two_units()
 
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 2
+        sale_order.picking_ids.move_ids.quantity_done = 2
         sale_order.picking_ids.button_validate()
 
         # Invoice the sale order.
@@ -782,7 +842,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order = self._so_and_confirm_two_units()
 
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 1
+        sale_order.picking_ids.move_ids.quantity_done = 1
         wiz = sale_order.picking_ids.button_validate()
         wiz = Form(self.env[wiz['res_model']].with_context(wiz['context'])).save()
         wiz.process()
@@ -826,7 +886,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order = self._so_and_confirm_two_units()
 
         # Deliver one.
-        sale_order.picking_ids.move_lines.quantity_done = 2
+        sale_order.picking_ids.move_ids.quantity_done = 2
         sale_order.picking_ids.button_validate()
 
         # Invoice the sale order.
@@ -885,7 +945,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order.action_confirm()
 
         # Deliver 10
-        sale_order.picking_ids.move_lines.quantity_done = 10
+        sale_order.picking_ids.move_ids.quantity_done = 10
         sale_order.picking_ids.button_validate()
 
         # Make the second receipt
@@ -973,7 +1033,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
                 })],
         })
         sale_order.action_confirm()
-        sale_order.picking_ids.move_lines.quantity_done = 1
+        sale_order.picking_ids.move_ids.quantity_done = 1
         sale_order.picking_ids.button_validate()
         invoice = sale_order._create_invoices()
         invoice.action_post()
@@ -992,7 +1052,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
                 })],
         })
         sale_order.action_confirm()
-        sale_order.picking_ids.move_lines.quantity_done = 6
+        sale_order.picking_ids.move_ids.quantity_done = 6
         sale_order.picking_ids.button_validate()
         invoice = sale_order._create_invoices()
         invoice.action_post()
@@ -1039,7 +1099,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         sale_order.action_confirm()
 
         # Deliver 10
-        sale_order.picking_ids.move_lines.quantity_done = 10
+        sale_order.picking_ids.move_ids.quantity_done = 10
         sale_order.picking_ids.button_validate()
 
         # Invoice the sale order.
@@ -1061,7 +1121,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         in_move_2._action_done()
 
         # check the last anglo saxon move line
-        revalued_anglo_expense_amls = sale_order.picking_ids.mapped('move_lines.stock_valuation_layer_ids')[-1].stock_move_id.account_move_ids[-1].mapped('line_ids')
+        revalued_anglo_expense_amls = sale_order.picking_ids.move_ids.stock_valuation_layer_ids[-1].stock_move_id.account_move_ids[-1].line_ids
         revalued_cogs_aml = revalued_anglo_expense_amls.filtered(lambda aml: aml.account_id == self.company_data['default_account_expense'])
         self.assertEqual(revalued_cogs_aml.debit, 4, 'Price difference should have correctly reflected in expense account.')
 
@@ -1087,7 +1147,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
 
         # Create, confirm and deliver a sale order for 2@12 (SO1)
         so_1 = self._so_and_confirm_two_units()
-        so_1.picking_ids.move_lines.quantity_done = 2
+        so_1.picking_ids.move_ids.quantity_done = 2
         so_1.picking_ids.button_validate()
 
         # Return 1 from SO1
@@ -1100,7 +1160,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         stock_return_picking_action = stock_return_picking.create_returns()
         return_pick = self.env['stock.picking'].browse(stock_return_picking_action['res_id'])
         return_pick.action_assign()
-        return_pick.move_lines.quantity_done = 1
+        return_pick.move_ids.quantity_done = 1
         return_pick._action_done()
 
         # Create, confirm and deliver a sale order for 1@12 (SO2)
@@ -1117,7 +1177,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
                 })],
         })
         so_2.action_confirm()
-        so_2.picking_ids.move_lines.quantity_done = 1
+        so_2.picking_ids.move_ids.quantity_done = 1
         so_2.picking_ids.button_validate()
 
         # Receive 1@20
@@ -1144,7 +1204,7 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
         stock_redeliver_picking_action = stock_redeliver_picking.create_returns()
         redeliver_pick = self.env['stock.picking'].browse(stock_redeliver_picking_action['res_id'])
         redeliver_pick.action_assign()
-        redeliver_pick.move_lines.quantity_done = 1
+        redeliver_pick.move_ids.quantity_done = 1
         redeliver_pick._action_done()
 
         # Invoice the sale orders
@@ -1213,37 +1273,37 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
                 })],
         })
         so_1.action_confirm()
-        so_1.picking_ids.move_lines.quantity_done = 12
+        so_1.picking_ids.move_ids.quantity_done = 12
         so_1.picking_ids.button_validate()
 
         # Invoice the sale order.
         invoice_1 = so_1._create_invoices()
         invoice_1.action_post()
-        """
-        Invoice 1
 
-        Correct Journal Items
+        # Invoice 1
 
-        Name                            Debit       Credit
+        # Correct Journal Items
+
+        # Name                            Debit       Credit
+
+        # Product Sales                    0.00$      18.00$
+        # Account Receivable              18.00$       0.00$
+        # Default Account Stock Out        0.00$      24.00$
+        # Expenses                        24.00$       0.00$
 
-        Product Sales                    0.00$      18.00$
-        Account Receivable              18.00$       0.00$
-        Default Account Stock Out        0.00$      24.00$
-        Expenses                        24.00$       0.00$
-        """
         aml = invoice_1.line_ids
         # Product Sales
-        self.assertEqual(aml[0].debit,   0,0)
-        self.assertEqual(aml[0].credit, 18,0)
+        self.assertEqual(aml[0].debit, 0.0)
+        self.assertEqual(aml[0].credit, 18.0)
         # Account Receivable
-        self.assertEqual(aml[1].debit,  18,0)
-        self.assertEqual(aml[1].credit,  0,0)
+        self.assertEqual(aml[1].debit, 18.0)
+        self.assertEqual(aml[1].credit, 0.0)
         # Default Account Stock Out
-        self.assertEqual(aml[2].debit,   0,0)
-        self.assertEqual(aml[2].credit, 24,0)
+        self.assertEqual(aml[2].debit, 0.0)
+        self.assertEqual(aml[2].credit, 24.0)
         # Expenses
-        self.assertEqual(aml[3].debit,  24,0)
-        self.assertEqual(aml[3].credit,  0,0)
+        self.assertEqual(aml[3].debit, 24.0)
+        self.assertEqual(aml[3].credit, 0.0)
 
         # Create stock move 1
         in_move_1 = self.env['stock.move'].create({
@@ -1273,35 +1333,397 @@ class TestAngloSaxonValuation(ValuationReconciliationTestCommon):
                 })],
         })
         so_2.action_confirm()
-        so_2.picking_ids.move_lines.quantity_done = 12
+        so_2.picking_ids.move_ids.quantity_done = 12
         so_2.picking_ids.button_validate()
 
         # Invoice the sale order.
         invoice_2 = so_2._create_invoices()
         invoice_2.action_post()
 
-        """
-        Invoice 2
+        # Invoice 2
 
-        Correct Journal Items
+        # Correct Journal Items
 
-        Name                            Debit       Credit
+        # Name                            Debit       Credit
+
+        # Product Sales                    0.00$       18.0$
+        # Account Receivable              18.00$        0.0$
+        # Default Account Stock Out        0.00$       12.0$
+        # Expenses                        12.00$        0.0$
 
-        Product Sales                    0.00$       18.0$
-        Account Receivable              18.00$        0.0$
-        Default Account Stock Out        0.00$       12.0$
-        Expenses                        12.00$        0.0$
-        """
         aml = invoice_2.line_ids
         # Product Sales
-        self.assertEqual(aml[0].debit,   0,0)
-        self.assertEqual(aml[0].credit, 18,0)
+        self.assertEqual(aml[0].debit, 0.0)
+        self.assertEqual(aml[0].credit, 18.0)
         # Account Receivable
-        self.assertEqual(aml[1].debit,  18,0)
-        self.assertEqual(aml[1].credit,  0,0)
+        self.assertEqual(aml[1].debit, 18.0)
+        self.assertEqual(aml[1].credit, 0.0)
         # Default Account Stock Out
-        self.assertEqual(aml[2].debit,   0,0)
-        self.assertEqual(aml[2].credit, 12,0)
+        self.assertEqual(aml[2].debit, 0.0)
+        self.assertEqual(aml[2].credit, 12.0)
         # Expenses
-        self.assertEqual(aml[3].debit,  12,0)
-        self.assertEqual(aml[3].credit,  0,0)
+        self.assertEqual(aml[3].debit, 12.0)
+        self.assertEqual(aml[3].credit, 0.0)
+
+    def test_fifo_return_and_credit_note(self):
+        """
+        When posting a credit note for a returned product, the value of the anglo-saxo lines
+        should be based on the returned product's value
+        """
+        self.product.categ_id.property_cost_method = 'fifo'
+
+        # Receive one @10, one @20 and one @60
+        in_moves = self.env['stock.move'].create([{
+            'name': 'IN move @%s' % p,
+            'product_id': self.product.id,
+            'location_id': self.env.ref('stock.stock_location_suppliers').id,
+            'location_dest_id': self.company_data['default_warehouse'].lot_stock_id.id,
+            'product_uom': self.product.uom_id.id,
+            'product_uom_qty': 1,
+            'price_unit': p,
+        } for p in [10, 20, 60]])
+        in_moves._action_confirm()
+        in_moves.quantity_done = 1
+        in_moves._action_done()
+
+        # Sell 3 units
+        so = self.env['sale.order'].create({
+            'partner_id': self.partner_a.id,
+            'order_line': [
+                (0, 0, {
+                    'name': self.product.name,
+                    'product_id': self.product.id,
+                    'product_uom_qty': 3.0,
+                    'product_uom': self.product.uom_id.id,
+                    'price_unit': 100,
+                    'tax_id': False,
+                })],
+        })
+        so.action_confirm()
+
+        # Deliver 1@10, then 1@20 and then 1@60
+        pickings = []
+        picking = so.picking_ids
+        while picking:
+            pickings.append(picking)
+            picking.move_ids.quantity_done = 1
+            action = picking.button_validate()
+            if isinstance(action, dict):
+                wizard = Form(self.env[action['res_model']].with_context(action['context'])).save()
+                wizard.process()
+            picking = picking.backorder_ids
+
+        invoice = so._create_invoices()
+        invoice.action_post()
+
+        # Receive one @100
+        in_moves = self.env['stock.move'].create({
+            'name': 'IN move @100',
+            'product_id': self.product.id,
+            'location_id': self.env.ref('stock.stock_location_suppliers').id,
+            'location_dest_id': self.company_data['default_warehouse'].lot_stock_id.id,
+            'product_uom': self.product.uom_id.id,
+            'product_uom_qty': 1,
+            'price_unit': 100,
+        })
+        in_moves._action_confirm()
+        in_moves.quantity_done = 1
+        in_moves._action_done()
+
+        # Return the second picking (i.e. 1@20)
+        ctx = {'active_id': pickings[1].id, 'active_model': 'stock.picking'}
+        return_wizard = Form(self.env['stock.return.picking'].with_context(ctx)).save()
+        return_picking_id, dummy = return_wizard._create_returns()
+        return_picking = self.env['stock.picking'].browse(return_picking_id)
+        return_picking.move_ids.quantity_done = 1
+        return_picking.button_validate()
+
+        # Add a credit note for the returned product
+        ctx = {'active_model': 'account.move', 'active_ids': invoice.ids}
+        refund_wizard = self.env['account.move.reversal'].with_context(ctx).create({
+            'refund_method': 'refund',
+            'journal_id': invoice.journal_id.id,
+        })
+        action = refund_wizard.reverse_moves()
+        reverse_invoice = self.env['account.move'].browse(action['res_id'])
+        with Form(reverse_invoice) as reverse_invoice_form:
+            with reverse_invoice_form.invoice_line_ids.edit(0) as line:
+                line.quantity = 1
+        reverse_invoice.action_post()
+
+        amls = reverse_invoice.line_ids
+        stock_out_aml = amls.filtered(lambda aml: aml.account_id == self.company_data['default_account_stock_out'])
+        self.assertEqual(stock_out_aml.debit, 20, 'Should be to the value of the returned product')
+        self.assertEqual(stock_out_aml.credit, 0)
+        cogs_aml = amls.filtered(lambda aml: aml.account_id == self.company_data['default_account_expense'])
+        self.assertEqual(cogs_aml.debit, 0)
+        self.assertEqual(cogs_aml.credit, 20, 'Should be to the value of the returned product')
+
+    def test_fifo_return_and_create_invoice(self):
+        """
+        When creating an invoice for a returned product, the value of the anglo-saxo lines
+        should be based on the returned product's value
+        """
+        self.product.categ_id.property_cost_method = 'fifo'
+        self.product.invoice_policy = 'delivery'
+
+        # Receive one @10, one @20 and one @60
+        in_moves = self.env['stock.move'].create([{
+            'name': 'IN move @%s' % p,
+            'product_id': self.product.id,
+            'location_id': self.env.ref('stock.stock_location_suppliers').id,
+            'location_dest_id': self.company_data['default_warehouse'].lot_stock_id.id,
+            'product_uom': self.product.uom_id.id,
+            'product_uom_qty': 1,
+            'price_unit': p,
+        } for p in [10, 20, 60]])
+        in_moves._action_confirm()
+        in_moves.quantity_done = 1
+        in_moves._action_done()
+
+        # Sell 3 units
+        so = self.env['sale.order'].create({
+            'partner_id': self.partner_a.id,
+            'order_line': [
+                (0, 0, {
+                    'name': self.product.name,
+                    'product_id': self.product.id,
+                    'product_uom_qty': 3.0,
+                    'product_uom': self.product.uom_id.id,
+                    'price_unit': 100,
+                    'tax_id': False,
+                })],
+        })
+        so.action_confirm()
+
+        # Deliver 1@10, then 1@20 and then 1@60
+        pickings = []
+        picking = so.picking_ids
+        while picking:
+            pickings.append(picking)
+            picking.move_ids.quantity_done = 1
+            action = picking.button_validate()
+            if isinstance(action, dict):
+                wizard = Form(self.env[action['res_model']].with_context(action['context'])).save()
+                wizard.process()
+            picking = picking.backorder_ids
+
+        invoice = so._create_invoices()
+        invoice.action_post()
+
+        # Receive one @100
+        in_moves = self.env['stock.move'].create({
+            'name': 'IN move @100',
+            'product_id': self.product.id,
+            'location_id': self.env.ref('stock.stock_location_suppliers').id,
+            'location_dest_id': self.company_data['default_warehouse'].lot_stock_id.id,
+            'product_uom': self.product.uom_id.id,
+            'product_uom_qty': 1,
+            'price_unit': 100,
+        })
+        in_moves._action_confirm()
+        in_moves.quantity_done = 1
+        in_moves._action_done()
+
+        # Return the second picking (i.e. 1@20)
+        ctx = {'active_id': pickings[1].id, 'active_model': 'stock.picking'}
+        return_wizard = Form(self.env['stock.return.picking'].with_context(ctx)).save()
+        return_picking_id, dummy = return_wizard._create_returns()
+        return_picking = self.env['stock.picking'].browse(return_picking_id)
+        return_picking.move_ids.quantity_done = 1
+        return_picking.button_validate()
+
+        # Create a new invoice for the returned product
+        ctx = {'active_model': 'sale.order', 'active_ids': so.ids}
+        create_invoice_wizard = self.env['sale.advance.payment.inv'].with_context(ctx).create({'advance_payment_method': 'delivered'})
+        create_invoice_wizard.create_invoices()
+        reverse_invoice = so.invoice_ids[-1]
+        with Form(reverse_invoice) as reverse_invoice_form:
+            with reverse_invoice_form.invoice_line_ids.edit(0) as line:
+                line.quantity = 1
+        reverse_invoice.action_post()
+
+        amls = reverse_invoice.line_ids
+        stock_out_aml = amls.filtered(lambda aml: aml.account_id == self.company_data['default_account_stock_out'])
+        self.assertEqual(stock_out_aml.debit, 20, 'Should be to the value of the returned product')
+        self.assertEqual(stock_out_aml.credit, 0)
+        cogs_aml = amls.filtered(lambda aml: aml.account_id == self.company_data['default_account_expense'])
+        self.assertEqual(cogs_aml.debit, 0)
+        self.assertEqual(cogs_aml.credit, 20, 'Should be to the value of the returned product')
+
+    def test_fifo_several_invoices_reset_repost(self):
+        self.product.categ_id.property_cost_method = 'fifo'
+        self.product.invoice_policy = 'delivery'
+
+        svl_values = [10, 15, 65]
+        total_value = sum(svl_values)
+        in_moves = self.env['stock.move'].create([{
+            'name': 'IN move @%s' % p,
+            'product_id': self.product.id,
+            'location_id': self.env.ref('stock.stock_location_suppliers').id,
+            'location_dest_id': self.company_data['default_warehouse'].lot_stock_id.id,
+            'product_uom': self.product.uom_id.id,
+            'product_uom_qty': 1,
+            'price_unit': p,
+        } for p in svl_values])
+        in_moves._action_confirm()
+        in_moves.quantity_done = 1
+        in_moves._action_done()
+
+        so = self.env['sale.order'].create({
+            'partner_id': self.partner_a.id,
+            'order_line': [
+                (0, 0, {
+                    'name': self.product.name,
+                    'product_id': self.product.id,
+                    'product_uom_qty': 3.0,
+                    'product_uom': self.product.uom_id.id,
+                    'price_unit': 100,
+                    'tax_id': False,
+                })],
+        })
+        so.action_confirm()
+
+        # Deliver one by one, so it creates an out-SVL each time.
+        # Then invoice the delivered quantity
+        invoices = self.env['account.move']
+        picking = so.picking_ids
+        while picking:
+            picking.move_ids.quantity_done = 1
+            action = picking.button_validate()
+            if isinstance(action, dict):
+                wizard = Form(self.env[action['res_model']].with_context(action['context'])).save()
+                wizard.process()
+            picking = picking.backorder_ids
+
+            invoice = so._create_invoices()
+            invoice.action_post()
+            invoices |= invoice
+
+        out_account = self.product.categ_id.property_stock_account_output_categ_id
+        invoice01, _invoice02, invoice03 = invoices
+        cogs = invoices.line_ids.filtered(lambda l: l.account_id == out_account)
+        self.assertEqual(cogs.mapped('credit'), svl_values)
+
+        # Reset and repost each invoice
+        for i, inv in enumerate(invoices):
+            inv.button_draft()
+            inv.action_post()
+            cogs = invoices.line_ids.filtered(lambda l: l.account_id == out_account)
+            self.assertEqual(cogs.mapped('credit'), svl_values, 'Incorrect values while posting again invoice %s' % (i + 1))
+
+        # Reset and repost all invoices (we only check the total value as the
+        # distribution changes but does not really matter)
+        invoices.button_draft()
+        invoices.action_post()
+        cogs = invoices.line_ids.filtered(lambda l: l.account_id == out_account)
+        self.assertEqual(sum(cogs.mapped('credit')), total_value)
+
+        # Reset and repost few invoices (we only check the total value as the
+        # distribution changes but does not really matter)
+        (invoice01 | invoice03).button_draft()
+        (invoice01 | invoice03).action_post()
+        cogs = invoices.line_ids.filtered(lambda l: l.account_id == out_account)
+        self.assertEqual(sum(cogs.mapped('credit')), total_value)
+
+    def test_fifo_reverse_and_create_new_invoice(self):
+        """
+        FIFO automated
+        Receive 1@10, 1@50
+        Deliver 1
+        Post the invoice, add a credit note with option 'new draft inv'
+        Post the second invoice
+        COGS should be based on the delivered product
+
+        Note: This test will also ensure that a user who only has access to
+        account app can post such an invoice
+        """
+        self.product.categ_id.property_cost_method = 'fifo'
+
+        accountman = self.env['res.users'].create({
+            'name': 'Super Accountman',
+            'login': 'super_accountman',
+            'password': 'super_accountman',
+            'groups_id': [(6, 0, self.env.ref('account.group_account_invoice').ids)],
+        })
+
+        in_moves = self.env['stock.move'].create([{
+            'name': 'IN move @%s' % p,
+            'product_id': self.product.id,
+            'location_id': self.env.ref('stock.stock_location_suppliers').id,
+            'location_dest_id': self.company_data['default_warehouse'].lot_stock_id.id,
+            'product_uom': self.product.uom_id.id,
+            'product_uom_qty': 1,
+            'price_unit': p,
+        } for p in [10, 50]])
+        in_moves._action_confirm()
+        in_moves.quantity_done = 1
+        in_moves._action_done()
+
+        so = self.env['sale.order'].create({
+            'partner_id': self.partner_a.id,
+            'order_line': [
+                (0, 0, {
+                    'name': self.product.name,
+                    'product_id': self.product.id,
+                    'product_uom_qty': 1.0,
+                    'product_uom': self.product.uom_id.id,
+                    'price_unit': 100,
+                    'tax_id': False,
+                })],
+        })
+        so.action_confirm()
+
+        picking = so.picking_ids
+        picking.move_ids.quantity_done = 1.0
+        picking.button_validate()
+
+        invoice01 = so._create_invoices()
+
+        # Clear the cache to ensure access rights
+        self.env.invalidate_all()
+        invoice01.with_user(accountman.id).action_post()
+
+        move_reversal = self.env['account.move.reversal'].with_context(active_model="account.move", active_ids=invoice01.ids).create({
+            'refund_method': 'modify',
+            'journal_id': invoice01.journal_id.id,
+        })
+        reversal = move_reversal.reverse_moves()
+        invoice02 = self.env['account.move'].browse(reversal['res_id'])
+        invoice02.action_post()
+
+        amls = invoice02.line_ids
+        stock_out_aml = amls.filtered(lambda aml: aml.account_id == self.company_data['default_account_stock_out'])
+        self.assertEqual(stock_out_aml.debit, 0)
+        self.assertEqual(stock_out_aml.credit, 10)
+        cogs_aml = amls.filtered(lambda aml: aml.account_id == self.company_data['default_account_expense'])
+        self.assertEqual(cogs_aml.debit, 10)
+        self.assertEqual(cogs_aml.credit, 0)
+
+    def test_fifo_edit_svl_without_reinvoice(self):
+        """Edit SVL move line after delivering. Check no reinvoicing occurs."""
+        self.product.categ_id.property_cost_method = 'fifo'
+        self.product.invoice_policy = 'delivery'
+        self.product.standard_price = 10
+        self.product.expense_policy = 'cost'
+
+        self._fifo_in_one_eight_one_ten()
+
+        # Create and confirm a sale order for 2@12
+        sale_order = self._so_and_confirm_two_units()
+        self.assertEqual(len(sale_order.order_line), 1)
+        self.assertEqual(sale_order.order_line.product_uom_qty, 2.0)
+
+        # Deliver one.
+        sale_order.picking_ids.move_ids.quantity_done = 2
+        sale_order.picking_ids.button_validate()
+        svl_am = sale_order.order_line.move_ids.stock_valuation_layer_ids.account_move_id
+        svl_am.button_draft()
+        svl_line = svl_am.line_ids.filtered(lambda aml: aml.account_id == self.company_data['default_account_stock_out'])
+
+        svl_line.write({'analytic_distribution': {sale_order.analytic_account_id.id: 100}})
+        svl_am.action_post()
+
+        # Check no reinvoice line addded to the sale order
+        self.assertEqual(len(sale_order.order_line), 1)
+        self.assertEqual(sale_order.order_line.product_uom_qty, 2.0)
