@@ -5,6 +5,7 @@ from datetime import date, timedelta
 from odoo.fields import Date
 from odoo.tools import float_is_zero
 from odoo.exceptions import UserError
+from odoo.addons.hr_timesheet.tests.test_timesheet import TestCommonTimesheet
 from odoo.addons.sale_timesheet.tests.common import TestCommonSaleTimesheet
 from odoo.tests import tagged
 
@@ -35,23 +36,15 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
             'pricelist_id': self.company_data['default_pricelist'].id,
         })
         so_line_ordered_project_only = self.env['sale.order.line'].create({
-            'name': self.product_order_timesheet4.name,
             'product_id': self.product_order_timesheet4.id,
             'product_uom_qty': 10,
-            'product_uom': self.product_order_timesheet4.uom_id.id,
-            'price_unit': self.product_order_timesheet4.list_price,
             'order_id': sale_order.id,
         })
         so_line_ordered_global_project = self.env['sale.order.line'].create({
-            'name': self.product_order_timesheet2.name,
             'product_id': self.product_order_timesheet2.id,
             'product_uom_qty': 50,
-            'product_uom': self.product_order_timesheet2.uom_id.id,
-            'price_unit': self.product_order_timesheet2.list_price,
             'order_id': sale_order.id,
         })
-        so_line_ordered_project_only.product_id_change()
-        so_line_ordered_global_project.product_id_change()
         sale_order.action_confirm()
         task_serv2 = self.env['project.task'].search([('sale_line_id', '=', so_line_ordered_global_project.id)])
         project_serv1 = self.env['project.project'].search([('sale_line_id', '=', so_line_ordered_project_only.id)])
@@ -95,7 +88,7 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
             'employee_id': self.employee_user.id,
         })
         self.assertEqual(so_line_ordered_project_only.qty_delivered, 0.0, 'Timesheet directly on project does not increase delivered quantity on so line')
-        self.assertEqual(timesheet3.timesheet_invoice_type, 'non_billable_project', "Timesheets without task shoulbe be 'no project found'")
+        self.assertEqual(timesheet3.timesheet_invoice_type, 'non_billable', "Timesheets without SO should be be 'non-billable'")
         self.assertFalse(timesheet3.timesheet_invoice_id, "The timesheet should not be linked to the invoice, since we are in ordered quantity")
 
         # log timesheet on task in global project (higher than the initial ordrered qty)
@@ -111,11 +104,8 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
 
         # add so line with produdct "create task in new project".
         so_line_ordered_task_in_project = self.env['sale.order.line'].create({
-            'name': self.product_order_timesheet3.name,
             'product_id': self.product_order_timesheet3.id,
             'product_uom_qty': 3,
-            'product_uom': self.product_order_timesheet3.uom_id.id,
-            'price_unit': self.product_order_timesheet3.list_price,
             'order_id': sale_order.id,
         })
 
@@ -173,23 +163,15 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
             'pricelist_id': self.company_data['default_pricelist'].id,
         })
         so_line_deliver_global_project = self.env['sale.order.line'].create({
-            'name': self.product_delivery_timesheet2.name,
             'product_id': self.product_delivery_timesheet2.id,
             'product_uom_qty': 50,
-            'product_uom': self.product_delivery_timesheet2.uom_id.id,
-            'price_unit': self.product_delivery_timesheet2.list_price,
             'order_id': sale_order.id,
         })
         so_line_deliver_task_project = self.env['sale.order.line'].create({
-            'name': self.product_delivery_timesheet3.name,
             'product_id': self.product_delivery_timesheet3.id,
             'product_uom_qty': 20,
-            'product_uom': self.product_delivery_timesheet3.uom_id.id,
-            'price_unit': self.product_delivery_timesheet3.list_price,
             'order_id': sale_order.id,
         })
-        so_line_deliver_global_project.product_id_change()
-        so_line_deliver_task_project.product_id_change()
 
         # confirm SO
         sale_order.action_confirm()
@@ -252,11 +234,8 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
 
         # add a line on SO
         so_line_deliver_only_project = self.env['sale.order.line'].create({
-            'name': self.product_delivery_timesheet4.name,
             'product_id': self.product_delivery_timesheet4.id,
             'product_uom_qty': 5,
-            'product_uom': self.product_delivery_timesheet4.uom_id.id,
-            'price_unit': self.product_delivery_timesheet4.list_price,
             'order_id': sale_order.id,
         })
         self.assertEqual(len(sale_order.project_ids), 2, "No new project should have been created by the SO, when selling 'project only' product, since it reuse the one from 'new task in new project'.")
@@ -270,7 +249,7 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
         })
         self.assertTrue(float_is_zero(so_line_deliver_only_project.qty_delivered, precision_digits=2), "Timesheeting on project should not incremented the delivered quantity on the SO line")
         self.assertEqual(sale_order.invoice_status, 'to invoice', 'Sale Timesheet: "invoice on delivery" timesheets should have quantity to invoice')
-        self.assertEqual(timesheet3.timesheet_invoice_type, 'non_billable_project', "Timesheets without task shoulbe be 'no project found'")
+        self.assertEqual(timesheet3.timesheet_invoice_type, 'billable_time', "Timesheets with an amount > 0 should be 'billable time'")
         self.assertFalse(timesheet3.timesheet_invoice_id, "The timesheet3 should not be linked to the invoice yet")
 
         # let's log some timesheets on the task (new task/new project)
@@ -307,19 +286,13 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
             'pricelist_id': self.company_data['default_pricelist'].id,
         })
         so_line_manual_global_project = self.env['sale.order.line'].create({
-            'name': self.product_delivery_manual2.name,
             'product_id': self.product_delivery_manual2.id,
             'product_uom_qty': 50,
-            'product_uom': self.product_delivery_manual2.uom_id.id,
-            'price_unit': self.product_delivery_manual2.list_price,
             'order_id': sale_order.id,
         })
         so_line_manual_only_project = self.env['sale.order.line'].create({
-            'name': self.product_delivery_manual4.name,
             'product_id': self.product_delivery_manual4.id,
             'product_uom_qty': 20,
-            'product_uom': self.product_delivery_manual4.uom_id.id,
-            'price_unit': self.product_delivery_manual4.list_price,
             'order_id': sale_order.id,
         })
 
@@ -350,14 +323,14 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
 
         self.assertEqual(len(sale_order.project_ids), 2, "One project should have been created by the SO, when confirmed + the one coming from SO line 1 'task in global project'.")
         self.assertEqual(so_line_manual_global_project.task_id.sale_line_id, so_line_manual_global_project, "Task from a milestone product should be linked to its SO line too")
-        self.assertEqual(timesheet1.timesheet_invoice_type, 'billable_fixed', "Milestone timesheet goes in billable fixed category")
+        self.assertEqual(timesheet1.timesheet_invoice_type, 'billable_manual', "Milestone timesheet goes in billable manual category")
         self.assertTrue(float_is_zero(so_line_manual_global_project.qty_delivered, precision_digits=2), "Milestone Timesheeting should not incremented the delivered quantity on the SO line")
         self.assertEqual(so_line_manual_global_project.qty_to_invoice, 0.0, "Manual service should not be affected by timesheet on their created task.")
         self.assertEqual(so_line_manual_only_project.qty_to_invoice, 0.0, "Manual service should not be affected by timesheet on their created project.")
         self.assertEqual(sale_order.invoice_status, 'no', 'Sale Timesheet: "invoice on delivery" should not need to be invoiced on so confirmation')
 
-        self.assertEqual(timesheet1.timesheet_invoice_type, 'billable_fixed', "Timesheets linked to SO line with ordered product shoulbe be billable fixed since it is a milestone")
-        self.assertEqual(timesheet2.timesheet_invoice_type, 'non_billable_project', "Timesheets without task shoulbe be 'no project found'")
+        self.assertEqual(timesheet1.timesheet_invoice_type, 'billable_manual', "Timesheets linked to SO line with ordered product shoulbe be billable fixed since it is a prepaid product.")
+        self.assertEqual(timesheet2.timesheet_invoice_type, 'non_billable', "Timesheets without SO should be be 'non-billable'")
         self.assertFalse(timesheet1.timesheet_invoice_id, "The timesheet1 should not be linked to the invoice")
         self.assertFalse(timesheet2.timesheet_invoice_id, "The timesheet2 should not be linked to the invoice")
 
@@ -400,23 +373,15 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
             'display_type': 'line_section',
         })
         so_line_deliver_global_project = self.env['sale.order.line'].create({
-            'name': self.product_delivery_timesheet2.name,
             'product_id': self.product_delivery_timesheet2.id,
             'product_uom_qty': 50,
-            'product_uom': self.product_delivery_timesheet2.uom_id.id,
-            'price_unit': self.product_delivery_timesheet2.list_price,
             'order_id': sale_order.id,
         })
         so_line_deliver_task_project = self.env['sale.order.line'].create({
-            'name': self.product_delivery_timesheet3.name,
             'product_id': self.product_delivery_timesheet3.id,
             'product_uom_qty': 20,
-            'product_uom': self.product_delivery_timesheet3.uom_id.id,
-            'price_unit': self.product_delivery_timesheet3.list_price,
             'order_id': sale_order.id,
         })
-        so_line_deliver_global_project.product_id_change()
-        so_line_deliver_task_project.product_id_change()
 
         # confirm SO
         sale_order.action_confirm()
@@ -491,6 +456,8 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
         wizard.create_invoices()
 
         self.assertTrue(sale_order.invoice_ids, 'One invoice should be created because the timesheet logged is between the period defined in wizard')
+        self.assertTrue(all(line.invoice_status == "to invoice" for line in sale_order.order_line if line.qty_delivered != line.qty_invoiced),
+                        "All lines that still have some quantity to be invoiced should have an invoice status of 'to invoice', regardless if they were considered for previous invoicing, but didn't belong to the timesheet domain")
 
         invoice = sale_order.invoice_ids[0]
         self.assertEqual(so_line_deliver_global_project.qty_invoiced, timesheet1.unit_amount)
@@ -526,18 +493,20 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
 
     def test_transfert_project(self):
         """ Transfert task with timesheet to another project. """
+        self.env.user.employee_id = self.env['hr.employee'].create({'user_id': self.env.uid})
         Timesheet = self.env['account.analytic.line']
         Task = self.env['project.task']
         today = Date.context_today(self.env.user)
 
-        task = Task.with_context(default_project_id=self.project_global.id).create({
+        task = Task.with_context(default_project_id=self.project_template.id).create({
             'name': 'first task',
-            'partner_id': self.partner_a.id,
+            'partner_id': self.partner_b.id,
             'planned_hours': 10,
+            'sale_line_id': self.so.order_line[0].id
         })
 
         Timesheet.create({
-            'project_id': self.project_global.id,
+            'project_id': self.project_template.id,
             'task_id': task.id,
             'name': 'my first timesheet',
             'unit_amount': 4,
@@ -545,51 +514,11 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
 
         timesheet_count1 = Timesheet.search_count([('project_id', '=', self.project_global.id)])
         timesheet_count2 = Timesheet.search_count([('project_id', '=', self.project_template.id)])
-        self.assertEqual(timesheet_count1, 1, "One timesheet in project_global")
-        self.assertEqual(timesheet_count2, 0, "No timesheet in project_template")
+        self.assertEqual(timesheet_count1, 0, "No timesheet in project_global")
+        self.assertEqual(timesheet_count2, 1, "One timesheet in project_template")
         self.assertEqual(len(task.timesheet_ids), 1, "The timesheet should be linked to task")
 
         # change project of task, as the timesheet is not yet invoiced, the timesheet will change his project
-        task.write({
-            'project_id': self.project_template.id
-        })
-
-        timesheet_count1 = Timesheet.search_count([('project_id', '=', self.project_global.id)])
-        timesheet_count2 = Timesheet.search_count([('project_id', '=', self.project_template.id)])
-        self.assertEqual(timesheet_count1, 0, "No timesheet in project_global")
-        self.assertEqual(timesheet_count2, 1, "One timesheet in project_template")
-        self.assertEqual(len(task.timesheet_ids), 1, "The timesheet still should be linked to task")
-
-        wizard = self.env['project.task.create.sale.order'].with_context(active_id=task.id, active_model='project.task').create({
-            'product_id': self.product_delivery_timesheet3.id
-        })
-
-        # We create the SO and the invoice
-        action = wizard.action_create_sale_order()
-        sale_order = self.env['sale.order'].browse(action['res_id'])
-        self.context = {
-            'active_model': 'sale.order',
-            'active_ids': [sale_order.id],
-            'active_id': sale_order.id,
-            'default_journal_id': self.company_data['default_journal_sale'].id
-        }
-        wizard = self.env['sale.advance.payment.inv'].with_context(self.context).create({
-            'advance_payment_method': 'delivered',
-            'date_start_invoice_timesheet': today - timedelta(days=4),
-            'date_end_invoice_timesheet': today
-        })
-        wizard.create_invoices()
-
-        Timesheet.create({
-            'project_id': self.project_template.id,
-            'task_id': task.id,
-            'name': 'my second timesheet',
-            'unit_amount': 6,
-        })
-
-        self.assertEqual(Timesheet.search_count([('project_id', '=', self.project_template.id)]), 2, "2 timesheets in project_template")
-
-        # change project of task, the timesheet not yet invoiced will change its project. The timesheet already invoiced will not change his project.
         task.write({
             'project_id': self.project_global.id
         })
@@ -597,5 +526,273 @@ class TestSaleTimesheet(TestCommonSaleTimesheet):
         timesheet_count1 = Timesheet.search_count([('project_id', '=', self.project_global.id)])
         timesheet_count2 = Timesheet.search_count([('project_id', '=', self.project_template.id)])
         self.assertEqual(timesheet_count1, 1, "One timesheet in project_global")
-        self.assertEqual(timesheet_count2, 1, "Still one timesheet in project_template")
-        self.assertEqual(len(task.timesheet_ids), 2, "The 2 timesheet still should be linked to task")
+        self.assertEqual(timesheet_count2, 0, "No timesheet in project_template")
+        self.assertEqual(len(task.timesheet_ids), 1, "The timesheet still should be linked to task")
+
+        # Create an invoice
+        context = {
+            'active_model': 'sale.order',
+            'active_ids': [self.so.id],
+            'active_id': self.so.id,
+            'default_journal_id': self.company_data['default_journal_sale'].id
+        }
+        wizard = self.env['sale.advance.payment.inv'].with_context(context).create({
+            'advance_payment_method': 'delivered',
+            'date_start_invoice_timesheet': today - timedelta(days=4),
+            'date_end_invoice_timesheet': today,
+        })
+        wizard.create_invoices()
+
+        Timesheet.create({
+            'project_id': self.project_global.id,
+            'task_id': task.id,
+            'name': 'my second timesheet',
+            'unit_amount': 6,
+        })
+
+        self.assertEqual(Timesheet.search_count([('project_id', '=', self.project_global.id)]), 2, "2 timesheets in project_global")
+
+        # change project of task, the timesheet not yet invoiced will change its project. The timesheet already invoiced will not change his project.
+        task.write({
+            'project_id': self.project_template.id
+        })
+
+        timesheet_count1 = Timesheet.search_count([('project_id', '=', self.project_global.id)])
+        timesheet_count2 = Timesheet.search_count([('project_id', '=', self.project_template.id)])
+        self.assertEqual(timesheet_count1, 1, "Still one timesheet in project_global")
+        self.assertEqual(timesheet_count2, 1, "One timesheet in project_template")
+        self.assertEqual(len(task.timesheet_ids), 2, "The 2 timesheets still should be linked to task")
+
+    def test_change_customer_and_SOL_after_invoiced_timesheet(self):
+        sale_order1 = self.env['sale.order'].create({
+            'partner_id': self.partner_a.id,
+            'partner_invoice_id': self.partner_a.id,
+            'partner_shipping_id': self.partner_a.id,
+            'pricelist_id': self.company_data['default_pricelist'].id,
+        })
+        sale_order2 = self.env['sale.order'].create({
+            'partner_id': self.partner_b.id,
+            'partner_invoice_id': self.partner_b.id,
+            'partner_shipping_id': self.partner_b.id,
+            'pricelist_id': self.company_data['default_pricelist'].id,
+        })
+        so1_product_global_project_so_line = self.env['sale.order.line'].create({
+            'product_id': self.product_delivery_timesheet2.id,
+            'product_uom_qty': 50,
+            'order_id': sale_order1.id,
+        })
+        so2_product_global_project_so_line = self.env['sale.order.line'].create({
+            'product_id': self.product_delivery_timesheet2.id,
+            'product_uom_qty': 20,
+            'order_id': sale_order2.id,
+        })
+
+        sale_order1.action_confirm()
+        sale_order2.action_confirm()
+
+        task_so1 = self.env['project.task'].search([('sale_line_id', '=', so1_product_global_project_so_line.id)])
+        task_so2 = self.env['project.task'].search([('sale_line_id', '=', so2_product_global_project_so_line.id)])
+
+        self.assertEqual(self.partner_a, task_so1.partner_id, "The Customer of the first task should be equal to partner_a.")
+        self.assertEqual(self.partner_b, task_so2.partner_id, "The Customer of the second task should be equal to partner_b.")
+        self.assertEqual(sale_order1.partner_id, task_so1.partner_id, "The Customer of the first task should be equal to the Customer of the first Sales Order.")
+        self.assertEqual(sale_order2.partner_id, task_so2.partner_id, "The Customer of the second task should be equal to the Customer of the second Sales Order.")
+
+        task_so1_timesheet1 = self.env['account.analytic.line'].create({
+            'name': 'Test Line1',
+            'project_id': task_so1.project_id.id,
+            'task_id': task_so1.id,
+            'unit_amount': 5,
+            'employee_id': self.employee_user.id,
+        })
+
+        invoice = sale_order1._create_invoices()
+        invoice.action_post()
+
+        self.assertEqual(self.partner_a, task_so1_timesheet1.partner_id, "The Task's Timesheet entry should have the same partner than on the task 1 and Sales Order 1.")
+
+        task_so1_timesheet2 = self.env['account.analytic.line'].create({
+            'name': 'Test Line2',
+            'project_id': task_so1.project_id.id,
+            'task_id': task_so1.id,
+            'unit_amount': 3,
+            'employee_id': self.employee_user.id,
+        })
+
+        task_so1.write({
+            'partner_id': self.partner_b.id,
+            'sale_line_id': so2_product_global_project_so_line.id,
+        })
+
+        self.assertEqual(self.partner_a, task_so1_timesheet1.partner_id, "The Task's first Timesheet entry should not have changed as it was already invoiced (its partner should still be partner_a).")
+        self.assertEqual(self.partner_b, task_so1_timesheet2.partner_id, "The Task's second Timesheet entry should have its partner changed, as it was not invoiced and the Task's partner/customer changed.")
+        self.assertEqual(so1_product_global_project_so_line, task_so1_timesheet1.so_line, "The Task's first Timesheet entry should not have changed as it was already invoiced (its so_line should still be equal to the first Sales Order line).")
+        self.assertEqual(so2_product_global_project_so_line, task_so1_timesheet2.so_line, "The Task's second Timesheet entry should have it's so_line changed, as the Sales Order Item of the Task changed, and this entry was not invoiced.")
+
+    def test_timesheet_upsell(self):
+        """ Test timesheet upselling and email """
+
+        sale_order = self.env['sale.order'].with_context(mail_notrack=True, mail_create_nolog=True).create({
+            'partner_id': self.partner_a.id,
+            'partner_invoice_id': self.partner_a.id,
+            'partner_shipping_id': self.partner_a.id,
+            'user_id': self.user_employee_company_B.id,
+        })
+        # create SO and confirm it
+        uom_days = self.env.ref('uom.product_uom_day')
+        sale_order_line = self.env['sale.order.line'].create({
+            'order_id': sale_order.id,
+            'product_id': self.product_order_timesheet3.id,
+            'product_uom': uom_days.id,
+        })
+        sale_order.action_confirm()
+        task = sale_order_line.task_id
+
+        # let's log some timesheets
+        self.env['account.analytic.line'].create({
+            'name': 'Test Line',
+            'project_id': task.project_id.id,
+            'task_id': task.id,
+            'unit_amount': 8,
+            'employee_id': self.employee_manager.id,
+        })
+
+        sale_order._create_invoices()
+        last_message_id = self.env['mail.message'].search([('model', '=', 'sale.order'), ('res_id', '=', sale_order.id)], limit=1).id or 0
+        self.env['account.analytic.line'].create({
+            'name': 'Test Line',
+            'project_id': task.project_id.id,
+            'task_id': task.id,
+            'unit_amount': 5,
+            'employee_id': self.employee_user.id,
+        })
+
+        self.assertEqual(sale_order.invoice_status, 'upselling', 'Sale Timesheet: "invoice on delivery" timesheets should not modify the invoice_status of the so')
+        message_sent = self.env['mail.message'].search([
+            ('id', '>', last_message_id),
+            ('subject', 'like', 'Upsell'),
+            ('model', '=', 'sale.order'),
+            ('res_id', '=', sale_order.id),
+        ])
+
+        self.assertEqual(len(message_sent), 1, 'Sale Timesheet: An email should always be sent to the saleperson when the state of the sale order change to upselling')
+
+        self.env['account.analytic.line'].create({
+            'name': 'Test Line',
+            'project_id': task.project_id.id,
+            'task_id': task.id,
+            'unit_amount': 5,
+            'employee_id': self.employee_user.id,
+        })
+
+        message_sent = self.env['mail.message'].search([
+            ('id', '>', last_message_id),
+            ('subject', 'like', 'Upsell'),
+            ('model', '=', 'sale.order'),
+            ('res_id', '=', sale_order.id),
+        ])
+        self.assertEqual(len(message_sent), 1, 'Sale Timesheet: An email should only be sent to the saleperson when the state of the sale order change to upselling')
+
+    def test_unlink_timesheet(self):
+        sale_order = self.env['sale.order'].create({
+            'partner_id': self.partner_a.id,
+            'partner_invoice_id': self.partner_a.id,
+            'partner_shipping_id': self.partner_a.id,
+            'pricelist_id': self.company_data['default_pricelist'].id,
+        })
+        so_line = self.env['sale.order.line'].create({
+            'name': self.product_delivery_timesheet2.name,
+            'product_id': self.product_delivery_timesheet2.id,
+            'product_uom_qty': 50,
+            'product_uom': self.product_delivery_timesheet2.uom_id.id,
+            'price_unit': self.product_delivery_timesheet2.list_price,
+            'order_id': sale_order.id,
+        })
+        sale_order.action_confirm()
+        task = so_line.task_id
+
+        # let's log some timesheets
+        analytic_line = self.env['account.analytic.line'].create({
+            'name': 'Test Line',
+            'project_id': task.project_id.id,
+            'task_id': task.id,
+            'unit_amount': 50,
+            'employee_id': self.employee_manager.id,
+        })
+
+        move = sale_order._create_invoices()
+        self.assertEqual(analytic_line.timesheet_invoice_id, move, "The timesheet should be linked to move")
+
+        move.with_context(check_move_validity=False).line_ids[0].unlink()
+        self.assertFalse(analytic_line.timesheet_invoice_id, "The timesheet should have been unlinked from move")
+
+    def test_sale_order_with_multiple_project_templates(self):
+        """Test when creating multiple projects for one sale order every project has its own allocated hours"""
+        sale_order = self.env['sale.order'].with_context(tracking_disable=True).create({
+            'partner_id': self.partner_a.id,
+            'partner_invoice_id': self.partner_a.id,
+            'partner_shipping_id': self.partner_a.id,
+        })
+
+        project_template_1, project_template_2 = self.env['project.project'].create([
+            {'name': 'Template 1'},
+            {'name': 'Template 1'},
+        ])
+
+        product_1, product_2 = self.env['product.product'].create([
+            {
+                'name': "Service with template 1",
+                'standard_price': 10,
+                'list_price': 20,
+                'type': 'service',
+                'invoice_policy': 'order',
+                'uom_id': self.uom_hour.id,
+                'uom_po_id': self.uom_hour.id,
+                'default_code': 'c1',
+                'service_tracking': 'task_in_project',
+                'project_id': False,  # will create a project,
+                'project_template_id': project_template_1.id,
+            }, {
+                'name': "Service with template 2",
+                'standard_price': 10,
+                'list_price': 20,
+                'type': 'service',
+                'invoice_policy': 'order',
+                'uom_id': self.uom_hour.id,
+                'uom_po_id': self.uom_hour.id,
+                'default_code': 'c2',
+                'service_tracking': 'task_in_project',
+                'project_id': False,  # will create a project,
+                'project_template_id': project_template_2.id,
+            },
+        ])
+
+        sale_order_line_template_1, sale_order_line_template_2 = self.env['sale.order.line'].create([
+            {
+                'order_id': sale_order.id,
+                'name': product_1.name,
+                'product_id': product_1.id,
+                'product_uom_qty': 10,
+                'product_uom': product_1.uom_id.id,
+                'price_unit': product_1.list_price,
+            }, {
+                'order_id': sale_order.id,
+                'name': product_2.name,
+                'product_id': product_2.id,
+                'product_uom_qty': 5,
+                'product_uom': product_2.uom_id.id,
+                'price_unit': product_2.list_price,
+            },
+        ])
+
+        sale_order.action_confirm()
+        self.assertEqual(10, sale_order_line_template_1.project_id.allocated_hours)
+        self.assertEqual(5, sale_order_line_template_2.project_id.allocated_hours)
+
+
+class TestSaleTimesheetView(TestCommonTimesheet):
+    def test_get_view_timesheet_encode_uom(self):
+        """ Test the label of timesheet time spent fields according to the company encoding timesheet uom """
+        self.assert_get_view_timesheet_encode_uom([
+            ('sale_timesheet.project_project_view_form', '//field[@name="display_cost"]', [None, 'Daily Cost']),
+        ])
