@@ -20,12 +20,19 @@ class MailThread(models.AbstractModel):
     def _compute_message_has_sms_error(self):
         res = {}
         if self.ids:
-            self._cr.execute(""" SELECT msg.res_id, COUNT(msg.res_id) FROM mail_message msg
-                                 RIGHT JOIN mail_message_res_partner_needaction_rel rel
-                                 ON rel.mail_message_id = msg.id AND rel.notification_type = 'sms' AND rel.notification_status in ('exception')
-                                 WHERE msg.author_id = %s AND msg.model = %s AND msg.res_id in %s AND msg.message_type != 'user_notification'
-                                 GROUP BY msg.res_id""",
-                             (self.env.user.partner_id.id, self._name, tuple(self.ids),))
+            self.env.cr.execute("""
+                    SELECT msg.res_id, COUNT(msg.res_id)
+                      FROM mail_message msg
+                INNER JOIN mail_notification notif
+                        ON notif.mail_message_id = msg.id
+                     WHERE notif.notification_type = 'sms'
+                       AND notif.notification_status = 'exception'
+                       AND notif.author_id = %(author_id)s
+                       AND msg.model = %(model_name)s
+                       AND msg.res_id in %(res_ids)s
+                       AND msg.message_type != 'user_notification'
+                  GROUP BY msg.res_id
+            """, {'author_id': self.env.user.partner_id.id, 'model_name': self._name, 'res_ids': tuple(self.ids)})
             res.update(self._cr.fetchall())
 
         for record in self:
@@ -53,13 +60,15 @@ class MailThread(models.AbstractModel):
         """
         partners = self.env['res.partner']
         for fname in self._sms_get_partner_fields():
-            partners |= self.mapped(fname)
+            partners = partners.union(*self.mapped(fname))  # ensure ordering
         return partners
 
     def _sms_get_number_fields(self):
         """ This method returns the fields to use to find the number to use to
         send an SMS on a record. """
-        return ['mobile']
+        if 'mobile' in self:
+            return ['mobile']
+        return []
 
     def _sms_get_recipients_info(self, force_field=False, partner_fallback=True):
         """" Get SMS recipient information on current record set. This method
@@ -144,24 +153,19 @@ class MailThread(models.AbstractModel):
                 }
         return result
 
-    def _message_sms_schedule_mass(self, body='', template=False, active_domain=None, **composer_values):
+    def _message_sms_schedule_mass(self, body='', template=False, **composer_values):
         """ Shortcut method to schedule a mass sms sending on a recordset.
 
         :param template: an optional sms.template record;
-        :param active_domain: bypass self.ids and apply composer on active_domain
-          instead;
         """
         composer_context = {
             'default_res_model': self._name,
             'default_composition_mode': 'mass',
             'default_template_id': template.id if template else False,
-            'default_body': body if body and not template else False,
+            'default_res_ids': self.ids,
         }
-        if active_domain is not None:
-            composer_context['default_use_active_domain'] = True
-            composer_context['default_active_domain'] = repr(active_domain)
-        else:
-            composer_context['default_res_ids'] = self.ids
+        if body and not template:
+            composer_context['default_body'] = body
 
         create_vals = {
             'mass_force_send': False,
@@ -178,7 +182,7 @@ class MailThread(models.AbstractModel):
 
         :param template: a valid sms.template record;
         :param template_xmlid: XML ID of an sms.template (if no template given);
-        :param template_fallback: plaintext (jinja-enabled) in case template
+        :param template_fallback: plaintext (inline_template-enabled) in case template
           and template xml id are falsy (for example due to deleted data);
         """
         self.ensure_one()
@@ -201,8 +205,8 @@ class MailThread(models.AbstractModel):
         :param partner_ids: if set is a record set of partners to notify;
         :param number_field: if set is a name of field to use on current record
           to compute a number to notify;
-        :param sms_numbers: see ``_notify_record_by_sms``;
-        :param sms_pid_to_number: see ``_notify_record_by_sms``;
+        :param sms_numbers: see ``_notify_thread_by_sms``;
+        :param sms_pid_to_number: see ``_notify_thread_by_sms``;
         """
         self.ensure_one()
         sms_pid_to_number = sms_pid_to_number if sms_pid_to_number is not None else {}
@@ -223,7 +227,7 @@ class MailThread(models.AbstractModel):
                     sms_numbers = [False]
 
         if subtype_id is False:
-            subtype_id = self.env['ir.model.data'].xmlid_to_res_id('mail.mt_note')
+            subtype_id = self.env['ir.model.data']._xmlid_to_res_id('mail.mt_note')
 
         return self.message_post(
             body=plaintext2html(html2plaintext(body)), partner_ids=partner_ids or [],  # TDE FIXME: temp fix otherwise crash mail_thread.py
@@ -234,23 +238,35 @@ class MailThread(models.AbstractModel):
 
     def _notify_thread(self, message, msg_vals=False, **kwargs):
         recipients_data = super(MailThread, self)._notify_thread(message, msg_vals=msg_vals, **kwargs)
-        self._notify_record_by_sms(message, recipients_data, msg_vals=msg_vals, **kwargs)
+        self._notify_thread_by_sms(message, recipients_data, msg_vals=msg_vals, **kwargs)
         return recipients_data
 
-    def _notify_record_by_sms(self, message, recipients_data, msg_vals=False,
+    def _notify_thread_by_sms(self, message, recipients_data, msg_vals=False,
                               sms_numbers=None, sms_pid_to_number=None,
-                              check_existing=False, put_in_queue=False, **kwargs):
+                              resend_existing=False, put_in_queue=False, **kwargs):
         """ Notification method: by SMS.
 
-        :param message: mail.message record to notify;
-        :param recipients_data: see ``_notify_thread``;
-        :param msg_vals: see ``_notify_thread``;
+        :param message: ``mail.message`` record to notify;
+        :param recipients_data: list of recipients information (based on res.partner
+          records), formatted like
+            [{'active': partner.active;
+              'id': id of the res.partner being recipient to notify;
+              'groups': res.group IDs if linked to a user;
+              'notif': 'inbox', 'email', 'sms' (SMS App);
+              'share': partner.partner_share;
+              'type': 'customer', 'portal', 'user;'
+             }, {...}].
+          See ``MailThread._notify_get_recipients``;
+        :param msg_vals: dictionary of values used to create the message. If given it
+          may be used to access values related to ``message`` without accessing it
+          directly. It lessens query count in some optimized use cases by avoiding
+          access message content in db;
 
         :param sms_numbers: additional numbers to notify in addition to partners
           and classic recipients;
         :param pid_to_number: force a number to notify for a given partner ID
               instead of taking its mobile / phone number;
-        :param check_existing: check for existing notifications to update based on
+        :param resend_existing: check for existing notifications to update based on
           mailed recipient, otherwise create new notifications;
         :param put_in_queue: use cron to send queued SMS instead of sending them
           directly;
@@ -261,7 +277,7 @@ class MailThread(models.AbstractModel):
         sms_all = self.env['sms.sms'].sudo()
 
         # pre-compute SMS data
-        body = msg_vals['body'] if msg_vals and msg_vals.get('body') else message.body
+        body = msg_vals['body'] if msg_vals and 'body' in msg_vals else message.body
         sms_base_vals = {
             'body': html2plaintext(body),
             'mail_message_id': message.id,
@@ -269,7 +285,7 @@ class MailThread(models.AbstractModel):
         }
 
         # notify from computed recipients_data (followers, specific recipients)
-        partners_data = [r for r in recipients_data['partners'] if r['notif'] == 'sms']
+        partners_data = [r for r in recipients_data if r['notif'] == 'sms']
         partner_ids = [r['id'] for r in partners_data]
         if partner_ids:
             for partner in self.env['res.partner'].sudo().browse(partner_ids):
@@ -294,7 +310,7 @@ class MailThread(models.AbstractModel):
                 partner_id=False,
                 number=n,
                 state='outgoing' if n else 'error',
-                error_code='' if n else 'sms_number_missing',
+                failure_type='' if n else 'sms_number_missing',
             ) for n in tocreate_numbers]
 
         # create sms and notification
@@ -302,7 +318,7 @@ class MailThread(models.AbstractModel):
         if sms_create_vals:
             sms_all |= self.env['sms.sms'].sudo().create(sms_create_vals)
 
-            if check_existing:
+            if resend_existing:
                 existing = self.env['mail.notification'].sudo().search([
                     '|', ('res_partner_id', 'in', partner_ids),
                     '&', ('res_partner_id', '=', False), ('sms_number', 'in', sms_numbers),
@@ -316,6 +332,7 @@ class MailThread(models.AbstractModel):
                         existing_numbers.append(n.sms_number)
 
             notif_create_values = [{
+                'author_id': message.author_id.id,
                 'mail_message_id': message.id,
                 'res_partner_id': sms.partner_id.id,
                 'sms_number': sms.number,
@@ -323,7 +340,7 @@ class MailThread(models.AbstractModel):
                 'sms_id': sms.id,
                 'is_read': True,  # discard Inbox notification
                 'notification_status': 'ready' if sms.state == 'outgoing' else 'exception',
-                'failure_type': '' if sms.state == 'outgoing' else sms.error_code,
+                'failure_type': '' if sms.state == 'outgoing' else sms.failure_type,
             } for sms in sms_all if (sms.partner_id and sms.partner_id.id not in existing_pids) or (not sms.partner_id and sms.number not in existing_numbers)]
             if notif_create_values:
                 self.env['mail.notification'].sudo().create(notif_create_values)
@@ -345,3 +362,11 @@ class MailThread(models.AbstractModel):
             sms_all.filtered(lambda sms: sms.state == 'outgoing').send(auto_commit=False, raise_exception=False)
 
         return True
+
+    @api.model
+    def notify_cancel_by_type(self, notification_type):
+        super().notify_cancel_by_type(notification_type)
+        if notification_type == 'sms':
+            # TDE CHECK: delete pending SMS
+            self._notify_cancel_by_type_generic('sms')
+        return True
