@@ -6,7 +6,7 @@ from odoo import models
 
 class ResPartner(models.Model):
     _name = 'res.partner'
-    _inherit = ['res.partner', 'mail.thread.phone']
+    _inherit = ['mail.thread.phone', 'res.partner']
 
     def _sms_get_default_partners(self):
         """ Override of mail.thread method.
@@ -14,8 +14,7 @@ class ResPartner(models.Model):
         """
         return self
 
-    def _sms_get_number_fields(self):
+    def _phone_get_number_fields(self):
         """ This method returns the fields to use to find the number to use to
         send an SMS on a record. """
-        # TDE note: should override _phone_get_number_fields but ok as sms override it
         return ['mobile', 'phone']
