@@ -19,8 +19,6 @@ class SendSMS(models.TransientModel):
 
         result['res_model'] = result.get('res_model') or self.env.context.get('active_model')
 
-        if not result.get('active_domain'):
-            result['active_domain'] = repr(self.env.context.get('active_domain', []))
         if not result.get('res_ids'):
             if not result.get('res_id') and self.env.context.get('active_ids') and len(self.env.context.get('active_ids')) > 1:
                 result['res_ids'] = repr(self.env.context.get('active_ids'))
@@ -35,18 +33,14 @@ class SendSMS(models.TransientModel):
         ('numbers', 'Send to numbers'),
         ('comment', 'Post on a document'),
         ('mass', 'Send SMS in batch')], string='Composition Mode',
-        compute='_compute_composition_mode', readonly=False, required=True, store=True)
+        compute='_compute_composition_mode', precompute=True, readonly=False, required=True, store=True)
     res_model = fields.Char('Document Model Name')
+    res_model_description = fields.Char('Document Model Description', compute='_compute_res_model_description')
     res_id = fields.Integer('Document ID')
     res_ids = fields.Char('Document IDs')
     res_ids_count = fields.Integer(
-        'Visible records count', compute='_compute_recipients_count', compute_sudo=False,
+        'Visible records count', compute='_compute_res_ids_count', compute_sudo=False,
         help='Number of recipients that will receive the SMS if sent in mass mode, without applying the Active Domain value')
-    use_active_domain = fields.Boolean('Use active domain')
-    active_domain = fields.Text('Active domain', readonly=True)
-    active_domain_count = fields.Integer(
-        'Active records count', compute='_compute_recipients_count', compute_sudo=False,
-        help='Number of records found when searching with the value in Active Domain')
     comment_single_recipient = fields.Boolean(
         'Single Mode', compute='_compute_comment_single_recipient', compute_sudo=False,
         help='Indicates if the SMS composer targets a single specific recipient')
@@ -62,7 +56,7 @@ class SendSMS(models.TransientModel):
     recipient_single_number_itf = fields.Char(
         'Recipient Number', compute='_compute_recipient_single',
         readonly=False, compute_sudo=False, store=True,
-        help='UX field allowing to edit the recipient number. If changed it will be stored onto the recipient.')
+        help='Phone number of the recipient. If changed, it will be recorded on recipient\'s profile.')
     recipient_single_valid = fields.Boolean("Is valid", compute='_compute_recipient_single_valid', compute_sudo=False)
     number_field_name = fields.Char('Number Field')
     numbers = fields.Char('Recipients (Numbers)')
@@ -71,33 +65,35 @@ class SendSMS(models.TransientModel):
     template_id = fields.Many2one('sms.template', string='Use Template', domain="[('model', '=', res_model)]")
     body = fields.Text(
         'Message', compute='_compute_body',
-        readonly=False, store=True, required=True)
+        precompute=True, readonly=False, store=True, required=True)
 
-    @api.depends('res_ids_count', 'active_domain_count')
+    @api.depends('res_ids_count')
     @api.depends_context('sms_composition_mode')
     def _compute_composition_mode(self):
         for composer in self:
             if self.env.context.get('sms_composition_mode') == 'guess' or not composer.composition_mode:
-                if composer.res_ids_count > 1 or (composer.use_active_domain and composer.active_domain_count > 1):
+                if composer.res_ids_count > 1:
                     composer.composition_mode = 'mass'
                 else:
                     composer.composition_mode = 'comment'
 
-    @api.depends('res_model', 'res_id', 'res_ids', 'active_domain')
-    def _compute_recipients_count(self):
+    @api.depends('res_model')
+    def _compute_res_model_description(self):
+        self.res_model_description = False
+        for composer in self.filtered('res_model'):
+            composer.res_model_description = self.env['ir.model']._get(composer.res_model).display_name
+
+    @api.depends('res_model', 'res_id', 'res_ids')
+    def _compute_res_ids_count(self):
         for composer in self:
             composer.res_ids_count = len(literal_eval(composer.res_ids)) if composer.res_ids else 0
-            if composer.res_model:
-                composer.active_domain_count = self.env[composer.res_model].search_count(literal_eval(composer.active_domain or '[]'))
-            else:
-                composer.active_domain_count = 0
 
     @api.depends('res_id', 'composition_mode')
     def _compute_comment_single_recipient(self):
         for composer in self:
             composer.comment_single_recipient = bool(composer.res_id and composer.composition_mode == 'comment')
 
-    @api.depends('res_model', 'res_id', 'res_ids', 'use_active_domain', 'composition_mode', 'number_field_name', 'sanitized_numbers')
+    @api.depends('res_model', 'res_id', 'res_ids', 'composition_mode', 'number_field_name', 'sanitized_numbers')
     def _compute_recipients(self):
         for composer in self:
             composer.recipient_valid_count = 0
@@ -113,7 +109,7 @@ class SendSMS(models.TransientModel):
                 composer.recipient_invalid_count = len([rid for rid, rvalues in res.items() if not rvalues['sanitized']])
             else:
                 composer.recipient_invalid_count = 0 if (
-                    composer.sanitized_numbers or (composer.composition_mode == 'mass' and composer.use_active_domain)
+                    composer.sanitized_numbers or composer.composition_mode == 'mass'
                 ) else 1
 
     @api.depends('res_model', 'number_field_name')
@@ -127,7 +123,7 @@ class SendSMS(models.TransientModel):
                 continue
             records.ensure_one()
             res = records._sms_get_recipients_info(force_field=composer.number_field_name, partner_fallback=False)
-            composer.recipient_single_description = res[records.id]['partner'].name or records.display_name
+            composer.recipient_single_description = res[records.id]['partner'].name or records._sms_get_default_partners().display_name
             composer.recipient_single_number = res[records.id]['number'] or ''
             if not composer.recipient_single_number_itf:
                 composer.recipient_single_number_itf = res[records.id]['number'] or ''
@@ -168,22 +164,6 @@ class SendSMS(models.TransientModel):
             elif record.template_id:
                 record.body = record.template_id.body
 
-    # ------------------------------------------------------------
-    # CRUD
-    # ------------------------------------------------------------
-
-    @api.model
-    def create(self, values):
-        # TDE FIXME: currently have to compute manually to avoid required issue, waiting VFE branch
-        if not values.get('body') or not values.get('composition_mode'):
-            values_wdef = self._add_missing_default_values(values)
-            cache_composer = self.new(values_wdef)
-            cache_composer._compute_body()
-            cache_composer._compute_composition_mode()
-            values['body'] = values.get('body') or cache_composer.body
-            values['composition_mode'] = values.get('composition_mode') or cache_composer.composition_mode
-        return super(SendSMS, self).create(values)
-
     # ------------------------------------------------------------
     # Actions
     # ------------------------------------------------------------
@@ -230,18 +210,23 @@ class SendSMS(models.TransientModel):
         # on the numbers in the database.
         records = records if records is not None else self._get_records()
         records.ensure_one()
-        if self.recipient_single_number_itf and self.recipient_single_number_itf != self.recipient_single_number:
+        if not self.number_field_name:
+            self.numbers = self.recipient_single_number_itf or self.recipient_single_number
+        elif self.recipient_single_number_itf and self.recipient_single_number_itf != self.recipient_single_number:
             records.write({self.number_field_name: self.recipient_single_number_itf})
         return self._action_send_sms_comment(records=records)
 
     def _action_send_sms_comment(self, records=None):
         records = records if records is not None else self._get_records()
-        subtype_id = self.env['ir.model.data'].xmlid_to_res_id('mail.mt_note')
+        subtype_id = self.env['ir.model.data']._xmlid_to_res_id('mail.mt_note')
 
         messages = self.env['mail.message']
+        all_bodies = self._prepare_body_values(records)
+
         for record in records:
-            messages |= record._message_sms(
-                self.body, subtype_id=subtype_id,
+            messages += record._message_sms(
+                all_bodies[record.id],
+                subtype_id=subtype_id,
                 number_field=self.number_field_name,
                 sms_numbers=self.sanitized_numbers.split(',') if self.sanitized_numbers else None)
         return messages
@@ -273,6 +258,11 @@ class SendSMS(models.TransientModel):
             return [r.id for r in records if recipients_info[r.id]['sanitized'] in bl_numbers]
         return []
 
+    def _get_optout_record_ids(self, records, recipients_info):
+        """ Compute opt-outed contacts, not necessarily blacklisted. Void by default
+        as no opt-out mechanism exist in SMS, see SMS Marketing. """
+        return []
+
     def _get_done_record_ids(self, records, recipients_info):
         """ Get a list of already-done records. Order of record set is used to
         spot duplicates so pay attention to it if necessary. """
@@ -300,6 +290,7 @@ class SendSMS(models.TransientModel):
         all_bodies = self._prepare_body_values(records)
         all_recipients = self._prepare_recipient_values(records)
         blacklist_ids = self._get_blacklist_record_ids(records, all_recipients)
+        optout_ids = self._get_optout_record_ids(records, all_recipients)
         done_ids = self._get_done_record_ids(records, all_recipients)
 
         result = {}
@@ -308,23 +299,26 @@ class SendSMS(models.TransientModel):
             sanitized = recipients['sanitized']
             if sanitized and record.id in blacklist_ids:
                 state = 'canceled'
-                error_code = 'sms_blacklist'
+                failure_type = 'sms_blacklist'
+            elif sanitized and record.id in optout_ids:
+                state = 'canceled'
+                failure_type = 'sms_optout'
             elif sanitized and record.id in done_ids:
                 state = 'canceled'
-                error_code = 'sms_duplicate'
+                failure_type = 'sms_duplicate'
             elif not sanitized:
-                state = 'error'
-                error_code = 'sms_number_format' if recipients['number'] else 'sms_number_missing'
+                state = 'canceled'
+                failure_type = 'sms_number_format' if recipients['number'] else 'sms_number_missing'
             else:
                 state = 'outgoing'
-                error_code = ''
+                failure_type = ''
 
             result[record.id] = {
                 'body': all_bodies[record.id],
                 'partner_id': recipients['partner'].id,
                 'number': sanitized if sanitized else recipients['number'],
                 'state': state,
-                'error_code': error_code,
+                'failure_type': failure_type,
             }
         return result
 
@@ -366,10 +360,7 @@ class SendSMS(models.TransientModel):
     def _get_records(self):
         if not self.res_model:
             return None
-        if self.use_active_domain:
-            active_domain = literal_eval(self.active_domain or '[]')
-            records = self.env[self.res_model].search(active_domain)
-        elif self.res_ids:
+        if self.res_ids:
             records = self.env[self.res_model].browse(literal_eval(self.res_ids))
         elif self.res_id:
             records = self.env[self.res_model].browse(self.res_id)
