@@ -20,7 +20,9 @@ class SnailmailLetterFormatError(models.TransientModel):
             ('error_code', '=', 'FORMAT_ERROR'),
         ])
         for letter in letters_to_resend:
-            letter.attachment_id.unlink()
+            old_attachment = letter.attachment_id
+            letter.attachment_id = False
+            old_attachment.unlink()
             letter.write({'cover': self.snailmail_cover})
             letter.snailmail_print()
 
