@@ -5,6 +5,7 @@ from . import controllers
 from . import models
 from . import report
 from . import wizard
+from . import populate
 
 
 from odoo import api, SUPERUSER_ID
@@ -28,3 +29,8 @@ def _assign_default_mail_template_picking_id(cr, registry):
         company_ids_without_default_mail_template_id.write({
             'stock_mail_confirmation_template_id': default_mail_template_id.id,
         })
+
+def uninstall_hook(cr, registry):
+    env = api.Environment(cr, SUPERUSER_ID, {})
+    picking_type_ids = env["stock.picking.type"].with_context({"active_test": False}).search([])
+    picking_type_ids.sequence_id.unlink()
