@@ -2,10 +2,10 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo.exceptions import UserError
-from odoo.tests.common import SavepointCase, Form
+from odoo.tests.common import TransactionCase, Form
 
 
-class TestMultiCompany(SavepointCase):
+class TestMultiCompany(TransactionCase):
     @classmethod
     def setUpClass(cls):
         super(TestMultiCompany, cls).setUpClass()
@@ -22,14 +22,20 @@ class TestMultiCompany(SavepointCase):
         cls.user_a = cls.env['res.users'].create({
             'name': 'user company a with access to company b',
             'login': 'user a',
-            'groups_id': [(6, 0, [group_user.id, group_stock_manager.id])],
+            'groups_id': [(6, 0, [
+                group_user.id,
+                group_stock_manager.id,
+            ])],
             'company_id': cls.company_a.id,
             'company_ids': [(6, 0, [cls.company_a.id, cls.company_b.id])]
         })
         cls.user_b = cls.env['res.users'].create({
-            'name': 'user company a with access to company b',
+            'name': 'user company b with access to company a',
             'login': 'user b',
-            'groups_id': [(6, 0, [group_user.id, group_stock_manager.id])],
+            'groups_id': [(6, 0, [
+                group_user.id,
+                group_stock_manager.id,
+            ])],
             'company_id': cls.company_b.id,
             'company_ids': [(6, 0, [cls.company_a.id, cls.company_b.id])]
         })
@@ -103,60 +109,45 @@ class TestMultiCompany(SavepointCase):
             shared_partner.with_user(self.user_b).property_stock_customer = self.stock_location_a
 
     def test_inventory_1(self):
-        """Create an inventory in Company A for a product limited to Company A and, as a user of company
-        B, start the inventory and set its counted quantity to 10 before validating. The inventory
-        lines and stock moves should belong to Company A. The inventory loss location used should be
-        the one of Company A.
+        """Create a quant (inventory adjustment) in Company A for a product limited to Company A and
+        as a user of company B, apply the inventory adjustment and set its counted quantity to 10
+        before validating. The quant and stock moves should belong to Company A.
         """
         product = self.env['product.product'].create({
             'type': 'product',
             'company_id': self.company_a.id,
             'name': 'Product limited to company A',
         })
-        inventory = self.env['stock.inventory'].with_user(self.user_a).create({})
-        self.assertEqual(inventory.company_id, self.company_a)
-        inventory.with_user(self.user_b).action_start()
-        inventory.with_user(self.user_b).line_ids = [(0, 0, {
-            'product_qty': 10,
-            'product_id': product.id,
+        inventory_quant = self.env['stock.quant'].with_user(self.user_a).with_context(inventory_mode=True).create({
             'location_id': self.stock_location_a.id,
-        })]
-        inventory.with_user(self.user_b).action_validate()
-        self.assertEqual(inventory.line_ids.company_id, self.company_a)
-        self.assertEqual(inventory.move_ids.company_id, self.company_a)
-        self.assertEqual(inventory.move_ids.location_id.company_id, self.company_a)
+            'product_id': product.id,
+            'inventory_quantity': 0
+        })
+        self.assertEqual(inventory_quant.company_id, self.company_a)
+        inventory_quant.with_user(self.user_b).inventory_quantity = 10
+        inventory_quant.with_user(self.user_b).action_apply_inventory()
+        last_move_id = self.env['stock.move'].search([('is_inventory', '=', True)])[-1]
+        self.assertEqual(inventory_quant.company_id, self.company_a)
+        self.assertEqual(last_move_id.company_id, self.company_a)
+        self.assertEqual(last_move_id.quantity_done, 10)
+        self.assertEqual(last_move_id.location_id.company_id, self.company_a)
 
     def test_inventory_2(self):
-        """Create an empty inventory in Company A and check it is not possible to use products limited
-        to Company B in it.
+        """Try to create a quant (inventory adjustment) in Company A and check it is not possible to use
+        products limited to Company B in it.
         """
         product = self.env['product.product'].create({
             'name': 'product limited to company b',
             'company_id': self.company_b.id,
             'type': 'product'
         })
-        inventory = self.env['stock.inventory'].with_user(self.user_a).create({})
-        inventory.with_user(self.user_a).action_start()
-        inventory.with_user(self.user_a).line_ids = [(0, 0, {
-            'product_id': product.id,
-            'product_qty': 10,
-            'location_id': self.stock_location_a.id,
-        })]
-        with self.assertRaises(UserError):
-            inventory.with_user(self.user_a).action_validate()
 
-    def test_inventory_3(self):
-        """As a user of Company A, check it is not possible to start an inventory adjustment for
-        a product limited to Company B.
-        """
-        product = self.env['product.product'].create({
-            'name': 'product limited to company b',
-            'company_id': self.company_b.id,
-            'type': 'product'
-        })
-        inventory = self.env['stock.inventory'].with_user(self.user_a).create({'product_ids': [(4, product.id)]})
         with self.assertRaises(UserError):
-            inventory.with_user(self.user_a).action_start()
+            self.env['stock.quant'].with_user(self.user_a).with_context(inventory_mode=True).create({
+                'location_id': self.stock_location_a.id,
+                'product_id': product.id,
+                'inventory_quantity': 10
+        })
 
     def test_picking_1(self):
         """As a user of Company A, create a picking and use a picking type of Company B, check the
@@ -181,12 +172,12 @@ class TestMultiCompany(SavepointCase):
             'tracking': 'lot',
             'name': 'product lot',
         })
-        self.env['stock.production.lot'].create({
+        self.env['stock.lot'].create({
             'name': 'lotA',
             'company_id': self.company_a.id,
             'product_id': product_lot.id,
         })
-        self.env['stock.production.lot'].create({
+        self.env['stock.lot'].create({
             'name': 'lotA',
             'company_id': self.company_b.id,
             'product_id': product_lot.id,
@@ -225,7 +216,7 @@ class TestMultiCompany(SavepointCase):
         self.assertEqual(move1.move_line_ids[0].company_id, self.company_a)
         picking.with_user(self.user_b).button_validate()
         self.assertEqual(picking.state, 'done')
-        created_serial = self.env['stock.production.lot'].search([
+        created_serial = self.env['stock.lot'].search([
             ('name', '=', 'receipt_serial')
         ])
         self.assertEqual(created_serial.company_id, self.company_a)
@@ -233,6 +224,8 @@ class TestMultiCompany(SavepointCase):
     def test_orderpoint_1(self):
         """As a user of company A, create an orderpoint for company B. Check itsn't possible to
         use a warehouse of companny A"""
+        # Required for `warehouse_id` and `location_id` to be visible in the view
+        self.user_a.groups_id += self.env.ref("stock.group_stock_multi_locations")
         product = self.env['product.product'].create({
             'type': 'product',
             'name': 'shared product',
@@ -252,6 +245,8 @@ class TestMultiCompany(SavepointCase):
         """As a user of Company A, check it is not possible to change the company on an existing
         orderpoint to Company B.
         """
+        # Required for `warehouse_id` and `location_id` to be visible in the view
+        self.user_a.groups_id += self.env.ref("stock.group_stock_multi_locations")
         product = self.env['product.product'].create({
             'type': 'product',
             'name': 'shared product',
@@ -266,6 +261,30 @@ class TestMultiCompany(SavepointCase):
         with self.assertRaises(UserError):
             orderpoint.company_id = self.company_b.id
 
+    def test_orderpoint_3(self):
+        warehouse_a1 = self.warehouse_a
+        # Create a second warehouse the company A
+        # to test the change of location when changing of warehouse within a same company
+        warehouse_a2 = self.env['stock.warehouse'].with_user(self.user_a).sudo().create({'name': 'foo', 'code': 'foo'})
+        product = self.env['product.product'].create({
+            'type': 'product',
+            'name': 'shared product',
+        })
+        orderpoint = self.env['stock.warehouse.orderpoint'].with_user(self.user_a).create({
+            'product_id': product.id,
+        })
+        self.assertEqual(orderpoint.warehouse_id, warehouse_a1)
+        self.assertEqual(orderpoint.location_id, warehouse_a1.lot_stock_id)
+
+        orderpoint.warehouse_id = warehouse_a2
+        self.assertEqual(orderpoint.location_id, warehouse_a2.lot_stock_id)
+
+        orderpoint.location_id = warehouse_a1.lot_stock_id
+        self.assertEqual(orderpoint.warehouse_id, warehouse_a1)
+
+        orderpoint.location_id = warehouse_a2.lot_stock_id
+        self.assertEqual(orderpoint.warehouse_id, warehouse_a2)
+
     def test_product_1(self):
         """ As an user of Company A, checks we can or cannot create new product
         depending of its `company_id`."""
@@ -418,14 +437,14 @@ class TestMultiCompany(SavepointCase):
             }).id,
         })
 
-        route = self.env['stock.location.route'].create({
+        route = self.env['stock.route'].create({
             'name': 'Push',
             'company_id': False,
             'rule_ids': [(0, False, {
                 'name': 'create a move to company b',
                 'company_id': self.company_b.id,
                 'location_src_id': intercom_location.id,
-                'location_id': self.stock_location_b.id,
+                'location_dest_id': self.stock_location_b.id,
                 'action': 'push',
                 'auto': 'manual',
                 'picking_type_id': self.warehouse_b.in_type_id.id,
@@ -502,7 +521,7 @@ class TestMultiCompany(SavepointCase):
         intercom_location.write({'active': True})
         partner = self.env['res.partner'].create({'name': 'Deco Addict'})
         self.warehouse_a.resupply_wh_ids = [(6, 0, [self.warehouse_b.id])]
-        resupply_route = self.env['stock.location.route'].search([('supplier_wh_id', '=', self.warehouse_b.id),
+        resupply_route = self.env['stock.route'].search([('supplier_wh_id', '=', self.warehouse_b.id),
                                                                   ('supplied_wh_id', '=', self.warehouse_a.id)])
         self.assertTrue(resupply_route, "Resupply route not found")
 
@@ -563,7 +582,7 @@ class TestMultiCompany(SavepointCase):
                          "Chained move created in transit location")
         self.assertEqual(move_wha_to_cus.state, "waiting")
         self.assertEqual(move_transit_to_wha.state, "waiting")
-        self.assertEqual(move_whb_to_transit.state, "confirmed")
+        self.assertEqual(move_whb_to_transit.state, "assigned")
 
         (move_wha_to_cus + move_whb_to_transit + move_transit_to_wha).picking_id.action_assign()
         self.assertEqual(move_wha_to_cus.state, "waiting")
