@@ -2,18 +2,19 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo.tests import Form
-from odoo.tests.common import SavepointCase
+from odoo.tests.common import TransactionCase
 from odoo.tools import float_round
 from odoo.exceptions import UserError
 
 
-class TestPackingCommon(SavepointCase):
+class TestPackingCommon(TransactionCase):
     @classmethod
     def setUpClass(cls):
         super(TestPackingCommon, cls).setUpClass()
         cls.stock_location = cls.env.ref('stock.stock_location_stock')
         cls.warehouse = cls.env['stock.warehouse'].search([('lot_stock_id', '=', cls.stock_location.id)], limit=1)
         cls.warehouse.write({'delivery_steps': 'pick_pack_ship'})
+        cls.warehouse.int_type_id.reservation_method = 'manual'
         cls.pack_location = cls.warehouse.wh_pack_stock_loc_id
         cls.ship_location = cls.warehouse.wh_output_stock_loc_id
         cls.customer_location = cls.env.ref('stock.stock_location_customers')
@@ -108,23 +109,22 @@ class TestPacking(TestPackingCommon):
         package_level = self.env['stock.package_level'].create({
             'package_id': pack.id,
             'picking_id': picking.id,
-            'location_dest_id': self.stock_location.id,
             'company_id': picking.company_id.id,
         })
         self.assertEqual(package_level.state, 'draft',
                           'The package_level should be in draft as it has no moves, move lines and is not confirmed')
         picking.action_confirm()
         self.assertEqual(len(picking.move_ids_without_package), 0)
-        self.assertEqual(len(picking.move_lines), 1,
+        self.assertEqual(len(picking.move_ids), 1,
                          'One move should be created when the package_level has been confirmed')
         self.assertEqual(len(package_level.move_ids), 1,
                           'The move should be in the package level')
         self.assertEqual(package_level.state, 'confirmed',
                           'The package level must be state confirmed when picking is confirmed')
         picking.action_assign()
-        self.assertEqual(len(picking.move_lines), 1,
+        self.assertEqual(len(picking.move_ids), 1,
                          'You still have only one move when the picking is assigned')
-        self.assertEqual(len(picking.move_lines.move_line_ids), 1,
+        self.assertEqual(len(picking.move_ids.move_line_ids), 1,
                          'The move  should have one move line which is the reservation')
         self.assertEqual(picking.move_line_ids.package_level_id.id, package_level.id,
                           'The move line created should be linked to the package level')
@@ -139,9 +139,9 @@ class TestPacking(TestPackingCommon):
         self.assertEqual(package_level.move_line_ids[0].qty_done, 20.0,
                           'All quantity in package must be procesed in move line')
         picking.button_validate()
-        self.assertEqual(len(picking.move_lines), 1,
+        self.assertEqual(len(picking.move_ids), 1,
                          'You still have only one move when the picking is assigned')
-        self.assertEqual(len(picking.move_lines.move_line_ids), 1,
+        self.assertEqual(len(picking.move_ids.move_line_ids), 1,
                          'The move  should have one move line which is the reservation')
         self.assertEqual(package_level.state, 'done', 'The package level must be in state done')
         self.assertEqual(pack.location_id.id, picking.location_dest_id.id,
@@ -149,6 +149,63 @@ class TestPacking(TestPackingCommon):
         self.assertEqual(pack.quant_ids[0].location_id.id, picking.location_dest_id.id,
                           'The quant must be in the destination location')
 
+    def test_pick_a_pack_cancel(self):
+        """Cancel a reserved operation with a not-done package level (is_done=False)."""
+        pack = self.env['stock.quant.package'].create({'name': 'The pack to pick'})
+        self.env['stock.quant']._update_available_quantity(self.productA, self.stock_location, 20.0, package_id=pack)
+        picking = self.env['stock.picking'].create({
+            'picking_type_id': self.warehouse.int_type_id.id,
+            'location_id': self.stock_location.id,
+            'location_dest_id': self.stock_location.id,
+            'state': 'draft',
+        })
+        picking.picking_type_id.show_entire_packs = True
+        package_level = self.env['stock.package_level'].create({
+            'package_id': pack.id,
+            'picking_id': picking.id,
+            'location_dest_id': self.stock_location.id,
+            'company_id': picking.company_id.id,
+        })
+        picking.action_confirm()
+        picking.action_assign()
+        self.assertEqual(package_level.state, 'assigned')
+        self.assertTrue(package_level.move_line_ids)
+        picking.action_cancel()
+        self.assertEqual(package_level.state, 'cancel')
+        self.assertFalse(package_level.move_line_ids)
+
+    def test_pick_a_pack_cancel_is_done(self):
+        """Cancel a reserved operation with a package level that is done (is_done=True)."""
+        pack = self.env['stock.quant.package'].create({'name': 'The pack to pick'})
+        self.env['stock.quant']._update_available_quantity(self.productA, self.stock_location, 20.0, package_id=pack)
+        picking = self.env['stock.picking'].create({
+            'picking_type_id': self.warehouse.int_type_id.id,
+            'location_id': self.stock_location.id,
+            'location_dest_id': self.stock_location.id,
+            'state': 'draft',
+        })
+        picking.picking_type_id.show_entire_packs = True
+        package_level = self.env['stock.package_level'].create({
+            'package_id': pack.id,
+            'picking_id': picking.id,
+            'location_dest_id': self.stock_location.id,
+            'company_id': picking.company_id.id,
+        })
+        picking.action_confirm()
+        picking.action_assign()
+        self.assertEqual(package_level.state, 'assigned')
+        self.assertTrue(package_level.move_line_ids)
+        # By setting the package_level as 'done', all related lines will be kept
+        # when cancelling the transfer
+        package_level.is_done = True
+        picking.action_cancel()
+        self.assertEqual(picking.state, 'cancel')
+        self.assertEqual(package_level.state, 'cancel')
+        self.assertTrue(package_level.move_line_ids)
+        self.assertTrue(
+            all(package_level.move_line_ids.mapped(lambda l: l.state == 'cancel'))
+        )
+
     def test_multi_pack_reservation(self):
         """ When we move entire packages, it is possible to have a multiple times
             the same package in package level list, we make sure that only one is reserved,
@@ -171,13 +228,11 @@ class TestPacking(TestPackingCommon):
         package_level = self.env['stock.package_level'].create({
             'package_id': pack.id,
             'picking_id': picking.id,
-            'location_dest_id': self.stock_location.id,
             'company_id': picking.company_id.id,
         })
         package_level = self.env['stock.package_level'].create({
             'package_id': pack.id,
             'picking_id': picking.id,
-            'location_dest_id': self.stock_location.id,
             'company_id': picking.company_id.id,
         })
         picking.action_confirm()
@@ -217,7 +272,7 @@ class TestPacking(TestPackingCommon):
         })
         picking = self.env['stock.picking'].create({
             'picking_type_id': self.warehouse.in_type_id.id,
-            'location_id': self.stock_location.id,
+            'location_id': self.customer_location.id,
             'location_dest_id': self.stock_location.id,
             'state': 'draft',
         })
@@ -239,7 +294,7 @@ class TestPacking(TestPackingCommon):
         picking.write({
             'move_line_ids': [(0, 0, {
                 'product_id': self.productB.id,
-                'product_uom_qty': 7.0,
+                'reserved_uom_qty': 7.0,
                 'qty_done': 7.0,
                 'product_uom_id': self.productB.uom_id.id,
                 'location_id': self.customer_location.id,
@@ -251,7 +306,7 @@ class TestPacking(TestPackingCommon):
         picking.write({
             'move_line_ids': [(0, 0, {
                 'product_id': self.productA.id,
-                'product_uom_qty': 5.0,
+                'reserved_uom_qty': 5.0,
                 'qty_done': 5.0,
                 'product_uom_id': self.productA.uom_id.id,
                 'location_id': self.customer_location.id,
@@ -308,7 +363,7 @@ class TestPacking(TestPackingCommon):
             **location_dict,
             **{
                 'picking_type_id': self.warehouse.in_type_id.id,
-                'move_lines': [(6, 0, [move.id])],
+                'move_ids': [(6, 0, [move.id])],
         }})
 
         picking.action_confirm()
@@ -461,9 +516,17 @@ class TestPacking(TestPackingCommon):
 
         # Cancels the internal transfer and creates a new one.
         internal_transfer.action_cancel()
-        internal_form = Form(self.env['stock.picking'])
+        # @api.depends('picking_type_id.show_operations')
+        # def _compute_show_operations(self):
+        #     ...
+        #     if self.env.context.get('force_detailed_view'):
+        #         picking.show_operations = True
+        internal_form = Form(self.env['stock.picking'].with_context(force_detailed_view=True))
         internal_form.picking_type_id = self.warehouse.int_type_id
-        internal_form.location_id = self.warehouse.wh_input_stock_loc_id
+        # The test specifically removes the ability to see the location fields
+        # grp_multi_loc = self.env.ref('stock.group_stock_multi_locations')
+        # self.env.user.write({'groups_id': [(3, grp_multi_loc.id)]})
+        # Hence, `internal_form.location_id` shouldn't be changed
         with internal_form.package_level_ids_details.new() as pack_line:
             pack_line.package_id = receipt_package.package_id
         internal_transfer = internal_form.save()
@@ -600,9 +663,17 @@ class TestPacking(TestPackingCommon):
 
         # Cancels the internal transfer and creates a new one.
         internal_transfer.action_cancel()
-        internal_form = Form(self.env['stock.picking'])
+        # @api.depends('picking_type_id.show_operations')
+        # def _compute_show_operations(self):
+        #     ...
+        #     if self.env.context.get('force_detailed_view'):
+        #         picking.show_operations = True
+        internal_form = Form(self.env['stock.picking'].with_context(force_detailed_view=True))
         internal_form.picking_type_id = self.warehouse.int_type_id
-        internal_form.location_id = self.warehouse.wh_input_stock_loc_id
+        # The test specifically removes the ability to see the location fields
+        # grp_multi_loc = self.env.ref('stock.group_stock_multi_locations')
+        # self.env.user.write({'groups_id': [(3, grp_multi_loc.id)]})
+        # Hence, `internal_form.location_id` shouldn't be changed
         with internal_form.package_level_ids_details.new() as pack_line:
             pack_line.package_id = receipt_package.package_id
         internal_transfer = internal_form.save()
@@ -634,7 +705,7 @@ class TestPacking(TestPackingCommon):
         """ Create a simple move in a delivery. Reserve the quantity but set as quantity done only a part.
         Call Put In Pack button. """
         self.productA.tracking = 'lot'
-        lot1 = self.env['stock.production.lot'].create({
+        lot1 = self.env['stock.lot'].create({
             'product_id': self.productA.id,
             'name': '00001',
             'company_id': self.warehouse.company_id.id
@@ -706,7 +777,7 @@ class TestPacking(TestPackingCommon):
         self.assertEqual(picking.state, "assigned")
         self.assertEqual(picking.package_level_ids.package_id, package)
 
-        move = picking.move_lines
+        move = picking.move_ids
         line = move.move_line_ids
 
         # change the result package and set a qty_done
@@ -738,6 +809,11 @@ class TestPacking(TestPackingCommon):
         self.warehouse.delivery_steps = 'ship_only'
         package = self.env["stock.quant.package"].create({"name": "Src Pack"})
         self.env['stock.quant']._update_available_quantity(self.productA, self.stock_location, 100, package_id=package)
+        # Required for `package_level_ids_details` to be visible in the view
+        # <page string="Detailed Operations" attrs="{'invisible': [('show_operations', '=', False)]}">
+        # <field name="package_level_ids_details"
+        #   attrs="{'invisible': ['|', ('picking_type_entire_packs', '=', False), ('show_operations', '=', False)]}"
+        self.warehouse.out_type_id.show_operations = True
         self.warehouse.out_type_id.show_entire_packs = True
         picking = self.env['stock.picking'].create({
             'location_id': self.stock_location.id,
@@ -753,8 +829,8 @@ class TestPacking(TestPackingCommon):
         with Form(picking) as picking_form:
             with picking_form.package_level_ids_details.new() as package_level:
                 package_level.package_id = package
-        self.assertEqual(len(picking.move_lines), 1, 'Should have only 1 stock move')
-        self.assertEqual(len(picking.move_lines), 1, 'Should have only 1 stock move')
+        self.assertEqual(len(picking.move_ids), 1, 'Should have only 1 stock move')
+        self.assertEqual(len(picking.move_ids), 1, 'Should have only 1 stock move')
         with Form(picking) as picking_form:
             with picking_form.package_level_ids_details.edit(0) as package_level:
                 package_level.is_done = True
@@ -793,12 +869,17 @@ class TestPacking(TestPackingCommon):
                 move.product_id = self.productA
                 move.product_uom_qty = 75
         picking.action_assign()
-        with Form(picking) as picking_form:
+        # @api.depends('picking_type_id.show_operations')
+        # def _compute_show_operations(self):
+        #     ...
+        #     if self.env.context.get('force_detailed_view'):
+        #         picking.show_operations = True
+        with Form(picking.with_context(force_detailed_view=True)) as picking_form:
             with picking_form.package_level_ids_details.new() as package_level:
                 package_level.package_id = package
         with Form(picking) as picking_form:
             picking_form.package_level_ids.remove(0)
-        self.assertEqual(len(picking.move_lines), 1, 'Should have only 1 stock move')
+        self.assertEqual(len(picking.move_ids), 1, 'Should have only 1 stock move')
 
     def test_picking_state_with_null_qty(self):
         receipt_form = Form(self.env['stock.picking'].with_context(default_immediate_transfer=False))
@@ -830,3 +911,421 @@ class TestPacking(TestPackingCommon):
         self.assertEqual(receipt.state, 'assigned')
         receipt.move_ids_without_package[1].product_uom_qty = 0
         self.assertEqual(receipt.state, 'assigned')
+
+    def test_2_steps_and_backorder(self):
+        """ When creating a backorder with a package, the latter should be reserved in the new picking. Moreover,
+         the initial picking shouldn't have any line about this package """
+        def create_picking(pick_type, from_loc, to_loc):
+            picking = self.env['stock.picking'].create({
+                'picking_type_id': pick_type.id,
+                'location_id': from_loc.id,
+                'location_dest_id': to_loc.id,
+            })
+            move_A, move_B = self.env['stock.move'].create([{
+                'name': self.productA.name,
+                'product_id': self.productA.id,
+                'product_uom_qty': 1,
+                'product_uom': self.productA.uom_id.id,
+                'picking_id': picking.id,
+                'location_id': from_loc.id,
+                'location_dest_id': to_loc.id,
+            }, {
+                'name': self.productB.name,
+                'product_id': self.productB.id,
+                'product_uom_qty': 1,
+                'product_uom': self.productB.uom_id.id,
+                'picking_id': picking.id,
+                'location_id': from_loc.id,
+                'location_dest_id': to_loc.id,
+            }])
+            picking.action_confirm()
+            picking.action_assign()
+            return picking, move_A, move_B
+
+        self.warehouse.delivery_steps = 'pick_ship'
+        pick_type = self.warehouse.pick_type_id
+        delivery_type = self.warehouse.out_type_id
+
+        self.env['stock.quant']._update_available_quantity(self.productA, self.stock_location, 1)
+        self.env['stock.quant']._update_available_quantity(self.productB, self.stock_location, 1)
+
+        picking, moveA, moveB = create_picking(pick_type, pick_type.default_location_src_id, pick_type.default_location_dest_id)
+        moveA.move_line_ids.qty_done = 1
+        picking.action_put_in_pack()
+        moveB.move_line_ids.qty_done = 1
+        picking.action_put_in_pack()
+        picking.button_validate()
+
+        # Required for `package_level_ids_details` to be visible in the view
+        # <page string="Detailed Operations" attrs="{'invisible': [('show_operations', '=', False)]}">
+        # <field name="package_level_ids_details"
+        #   attrs="{'invisible': ['|', ('picking_type_entire_packs', '=', False), ('show_operations', '=', False)]}"
+        delivery_type.show_operations = True
+        delivery_type.show_entire_packs = True
+        picking, _, _ = create_picking(delivery_type, delivery_type.default_location_src_id, self.customer_location)
+        packB = picking.package_level_ids[1]
+        with Form(picking) as picking_form:
+            with picking_form.package_level_ids_details.edit(0) as package_level:
+                package_level.is_done = True
+        action_data = picking.button_validate()
+        backorder_wizard = Form(self.env['stock.backorder.confirmation'].with_context(action_data['context'])).save()
+        backorder_wizard.process()
+        bo = self.env['stock.picking'].search([('backorder_id', '=', picking.id)])
+
+        self.assertNotIn(packB, picking.package_level_ids)
+        self.assertEqual(packB, bo.package_level_ids)
+        self.assertEqual(bo.package_level_ids.state, 'assigned')
+
+    def test_package_and_sub_location(self):
+        """
+        Suppose there are some products P available in shelf1, a child location of the pack location.
+        When moving these P to another child location of pack location, the source location of the
+        related package level should be shelf1
+        """
+        shelf1_location = self.env['stock.location'].create({
+            'name': 'shelf1',
+            'usage': 'internal',
+            'location_id': self.pack_location.id,
+        })
+        shelf2_location = self.env['stock.location'].create({
+            'name': 'shelf2',
+            'usage': 'internal',
+            'location_id': self.pack_location.id,
+        })
+
+        pack = self.env['stock.quant.package'].create({'name': 'Super Package'})
+        self.env['stock.quant']._update_available_quantity(self.productA, shelf1_location, 20.0, package_id=pack)
+
+        picking = self.env['stock.picking'].create({
+            'picking_type_id': self.warehouse.in_type_id.id,
+            'location_id': self.pack_location.id,
+            'location_dest_id': shelf2_location.id,
+        })
+        package_level = self.env['stock.package_level'].create({
+            'package_id': pack.id,
+            'picking_id': picking.id,
+            'company_id': picking.company_id.id,
+        })
+
+        self.assertEqual(package_level.location_id, shelf1_location)
+
+        picking.action_confirm()
+        package_level.is_done = True
+        picking.button_validate()
+
+        self.assertEqual(package_level.location_id, shelf1_location)
+
+    def test_pack_in_receipt_two_step_multi_putaway_02(self):
+        """
+        Suppose a product P, its weight is equal to 1kg
+        We have 100 x P on two pallets.
+        Receipt in two steps + Sub locations in WH/Stock + Storage Category
+        The Storage Category adds some constraints on weight/pallets capacity
+        """
+        warehouse = self.stock_location.warehouse_id
+        warehouse.reception_steps = "two_steps"
+        self.productA.weight = 1.0
+        self.env.user.write({'groups_id': [(4, self.env.ref('stock.group_stock_storage_categories').id)]})
+        self.env.user.write({'groups_id': [(4, self.env.ref('stock.group_stock_multi_locations').id)]})
+        # Required for `result_package_id` to be visible in the view
+        self.env.user.write({'groups_id': [(4, self.env.ref('stock.group_tracking_lot').id)]})
+
+        package_type = self.env['stock.package.type'].create({
+            'name': "Super Pallet",
+        })
+        package_01, package_02 = self.env['stock.quant.package'].create([{
+            'name': 'Pallet %s' % i,
+            'package_type_id': package_type.id,
+        } for i in [1, 2]])
+
+        # max 100kg (so 100 x P) and max 1 pallet -> we will work with pallets,
+        # so the pallet capacity constraint should be the effective one
+        stor_category = self.env['stock.storage.category'].create({
+            'name': 'Super Storage Category',
+            'max_weight': 100,
+            'package_capacity_ids': [(0, 0, {
+                'package_type_id': package_type.id,
+                'quantity': 1,
+            })]
+        })
+
+        # 3 sub locations with the storage category
+        # (the third location should never be used)
+        sub_loc_01, sub_loc_02, dummy = self.env['stock.location'].create([{
+            'name': 'Sub Location %s' % i,
+            'usage': 'internal',
+            'location_id': self.stock_location.id,
+            'storage_category_id': stor_category.id,
+        } for i in [1, 2, 3]])
+
+        self.env['stock.putaway.rule'].create({
+            'location_in_id': self.stock_location.id,
+            'location_out_id': self.stock_location.id,
+            'package_type_ids': [(4, package_type.id)],
+            'storage_category_id': stor_category.id,
+        })
+
+        # Receive 100 x P
+        receipt_picking = self.env['stock.picking'].create({
+            'picking_type_id': warehouse.in_type_id.id,
+            'location_id': self.env.ref('stock.stock_location_suppliers').id,
+            'location_dest_id': warehouse.wh_input_stock_loc_id.id,
+        })
+        self.env['stock.move'].create({
+            'name': self.productA.name,
+            'product_id': self.productA.id,
+            'product_uom': self.productA.uom_id.id,
+            'product_uom_qty': 100.0,
+            'picking_id': receipt_picking.id,
+            'location_id': receipt_picking.location_id.id,
+            'location_dest_id': receipt_picking.location_dest_id.id,
+        })
+        receipt_picking.action_confirm()
+
+        # Distribute the products on two pallets, one with 49 x P and a second
+        # one with 51 x P (to easy the debugging in case of trouble)
+        move_form = Form(receipt_picking.move_ids, view="stock.view_stock_move_operations")
+        with move_form.move_line_ids.edit(0) as line:
+            line.qty_done = 49
+            line.result_package_id = package_01
+        with move_form.move_line_ids.new() as line:
+            line.qty_done = 51
+            line.result_package_id = package_02
+        move_form.save()
+        receipt_picking.button_validate()
+
+        # We are in two-steps receipt -> check the internal picking
+        internal_picking = self.env['stock.picking'].search([], order='id desc', limit=1)
+        self.assertRecordValues(internal_picking.move_line_ids, [
+            {'reserved_uom_qty': 51, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_01.id},
+            {'reserved_uom_qty': 49, 'qty_done': 0, 'result_package_id': package_01.id, 'location_dest_id': sub_loc_02.id},
+        ])
+
+        # Change the constraints of the storage category:
+        # max 75kg (so 75 x P) and max 2 pallet -> this time, the weight
+        # constraint should be the effective one
+        stor_category.max_weight = 75
+        stor_category.package_capacity_ids.quantity = 2
+        internal_picking.do_unreserve()
+        internal_picking.action_assign()
+        self.assertRecordValues(internal_picking.move_line_ids, [
+            {'reserved_uom_qty': 51, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_01.id},
+            {'reserved_uom_qty': 49, 'qty_done': 0, 'result_package_id': package_01.id, 'location_dest_id': sub_loc_02.id},
+        ])
+
+        move_form = Form(internal_picking.move_ids, view="stock.view_stock_move_operations")
+        # lines order is reversed: [Pallet 02, Pallet 01]
+        with move_form.move_line_ids.edit(0) as line:
+            line.qty_done = 51
+        with move_form.move_line_ids.edit(1) as line:
+            line.qty_done = 49
+        move_form.save()
+        self.assertRecordValues(internal_picking.move_line_ids, [
+            {'reserved_uom_qty': 51, 'qty_done': 51, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_01.id},
+            {'reserved_uom_qty': 49, 'qty_done': 49, 'result_package_id': package_01.id, 'location_dest_id': sub_loc_02.id},
+        ])
+
+    def test_pack_in_receipt_two_step_multi_putaway_03(self):
+        """
+        Two sublocations (max 100kg, max 2 pallet)
+        Two products P1, P2, weight = 1kg
+        There are 10 x P1 on a pallet in the first sub location
+        Receive a pallet of 50 x P1 + 50 x P2 => because of weight constraint, should be redirected to the
+            second sub location
+        Then, same with max 200kg max 1 pallet => same result, this time because of pallet count constraint
+        """
+        warehouse = self.stock_location.warehouse_id
+        warehouse.reception_steps = "two_steps"
+        self.productA.weight = 1.0
+        self.productB.weight = 1.0
+        self.env.user.write({'groups_id': [(4, self.env.ref('stock.group_stock_storage_categories').id)]})
+        self.env.user.write({'groups_id': [(4, self.env.ref('stock.group_stock_multi_locations').id)]})
+        # Required for `result_package_id` to be visible in the view
+        self.env.user.write({'groups_id': [(4, self.env.ref('stock.group_tracking_lot').id)]})
+
+        package_type = self.env['stock.package.type'].create({
+            'name': "Super Pallet",
+        })
+        package_01, package_02 = self.env['stock.quant.package'].create([{
+            'name': 'Pallet %s' % i,
+            'package_type_id': package_type.id,
+        } for i in [1, 2]])
+
+        # max 100kg and max 2 pallets
+        stor_category = self.env['stock.storage.category'].create({
+            'name': 'Super Storage Category',
+            'max_weight': 100,
+            'package_capacity_ids': [(0, 0, {
+                'package_type_id': package_type.id,
+                'quantity': 2,
+            })]
+        })
+
+        # 3 sub locations with the storage category
+        # (the third location should never be used)
+        sub_loc_01, sub_loc_02, dummy = self.env['stock.location'].create([{
+            'name': 'Sub Location %s' % i,
+            'usage': 'internal',
+            'location_id': self.stock_location.id,
+            'storage_category_id': stor_category.id,
+        } for i in [1, 2, 3]])
+
+        self.env['stock.quant']._update_available_quantity(self.productA, sub_loc_01, 10, package_id=package_01)
+
+        self.env['stock.putaway.rule'].create({
+            'location_in_id': self.stock_location.id,
+            'location_out_id': self.stock_location.id,
+            'package_type_ids': [(4, package_type.id)],
+            'storage_category_id': stor_category.id,
+        })
+
+        # Receive 50 x P_A and 50 x P_B
+        receipt_picking = self.env['stock.picking'].create({
+            'picking_type_id': warehouse.in_type_id.id,
+            'location_id': self.env.ref('stock.stock_location_suppliers').id,
+            'location_dest_id': warehouse.wh_input_stock_loc_id.id,
+        })
+        self.env['stock.move'].create([{
+            'name': p.name,
+            'product_id': p.id,
+            'product_uom': p.uom_id.id,
+            'product_uom_qty': 50,
+            'picking_id': receipt_picking.id,
+            'location_id': receipt_picking.location_id.id,
+            'location_dest_id': receipt_picking.location_dest_id.id,
+        } for p in [self.productA, self.productB]])
+        receipt_picking.action_confirm()
+
+        move_form = Form(receipt_picking.move_ids[0], view="stock.view_stock_move_operations")
+        with move_form.move_line_ids.edit(0) as line:
+            line.qty_done = 50
+            line.result_package_id = package_02
+        move_form.save()
+        move_form = Form(receipt_picking.move_ids[1], view="stock.view_stock_move_operations")
+        with move_form.move_line_ids.edit(0) as line:
+            line.qty_done = 50
+            line.result_package_id = package_02
+        move_form.save()
+        receipt_picking.button_validate()
+
+        # We are in two-steps receipt -> check the internal picking
+        internal_picking = self.env['stock.picking'].search([], order='id desc', limit=1)
+        self.assertRecordValues(internal_picking.move_line_ids, [
+            {'product_id': self.productA.id, 'reserved_uom_qty': 50, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_02.id},
+            {'product_id': self.productB.id, 'reserved_uom_qty': 50, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_02.id},
+        ])
+
+        # Change the constraints of the storage category:
+        # max 200kg and max 1 pallet
+        stor_category.max_weight = 200
+        stor_category.package_capacity_ids.quantity = 1
+        internal_picking.do_unreserve()
+        internal_picking.action_assign()
+        self.assertRecordValues(internal_picking.move_line_ids, [
+            {'product_id': self.productA.id, 'reserved_uom_qty': 50, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_02.id},
+            {'product_id': self.productB.id, 'reserved_uom_qty': 50, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_02.id},
+        ])
+
+    def test_rounding_and_reserved_qty(self):
+        """
+        Basic use case: deliver a storable product put in two packages. This
+        test actually ensures that the process 'put in pack' handles some
+        possible issues with the floating point representation
+        """
+        self.env['stock.quant']._update_available_quantity(self.productA, self.stock_location, 0.4)
+
+        picking = self.env['stock.picking'].create({
+            'picking_type_id': self.warehouse.out_type_id.id,
+            'location_id': self.stock_location.id,
+            'location_dest_id': self.customer_location.id,
+            'move_ids': [(0, 0, {
+                'name': self.productA.name,
+                'product_id': self.productA.id,
+                'product_uom_qty': 0.4,
+                'product_uom': self.productA.uom_id.id,
+                'location_id': self.stock_location.id,
+                'location_dest_id': self.customer_location.id,
+                'picking_type_id': self.warehouse.out_type_id.id,
+            })],
+        })
+        picking.action_confirm()
+
+        picking.move_line_ids.qty_done = 0.3
+        picking.action_put_in_pack()
+
+        picking.move_line_ids.filtered(lambda ml: not ml.result_package_id).qty_done = 0.1
+        picking.action_put_in_pack()
+
+        quant = self.env['stock.quant'].search([('product_id', '=', self.productA.id), ('location_id', '=', self.stock_location.id)])
+        self.assertEqual(quant.available_quantity, 0)
+
+        picking.button_validate()
+        self.assertEqual(picking.state, 'done')
+        self.assertEqual(picking.move_ids.quantity_done, 0.4)
+        self.assertEqual(len(picking.move_line_ids.result_package_id), 2)
+
+    def test_put_out_of_pack_transfer(self):
+        """ When a transfer has multiple products all in the same package, removing a product from the destination package
+        (i.e. removing it from the package but still putting it in the same location) shouldn't remove it for other products. """
+        loc_1 = self.env['stock.location'].create({
+            'name': 'Location A',
+            'location_id': self.stock_location.id,
+        })
+        loc_2 = self.env['stock.location'].create({
+            'name': 'Location B',
+            'location_id': self.stock_location.id,
+        })
+        pack = self.env['stock.quant.package'].create({'name': 'New Package'})
+        self.env['stock.quant']._update_available_quantity(self.productA, loc_1, 5, package_id=pack)
+        self.env['stock.quant']._update_available_quantity(self.productB, loc_1, 4, package_id=pack)
+
+        picking = self.env['stock.picking'].create({
+            'location_id': loc_1.id,
+            'location_dest_id': loc_2.id,
+            'picking_type_id': self.warehouse.int_type_id.id,
+        })
+        moveA = self.env['stock.move'].create({
+            'name': self.productA.name,
+            'product_id': self.productA.id,
+            'product_uom_qty': 5,
+            'product_uom': self.productA.uom_id.id,
+            'picking_id': picking.id,
+            'location_id': loc_1.id,
+            'location_dest_id': loc_2.id,
+        })
+        moveB = self.env['stock.move'].create({
+            'name': self.productB.name,
+            'product_id': self.productB.id,
+            'product_uom_qty': 4,
+            'product_uom': self.productB.uom_id.id,
+            'picking_id': picking.id,
+            'location_id': loc_1.id,
+            'location_dest_id': loc_2.id,
+        })
+        # Check availabilities
+        picking.action_assign()
+        self.assertEqual(len(moveA.move_line_ids), 1, "A move line should have been created for the reservation of the package.")
+        self.assertEqual(moveA.move_line_ids.package_id.id, pack.id, "The package should have been reserved for both products.")
+        self.assertEqual(moveB.move_line_ids.package_id.id, pack.id, "The package should have been reserved for both products.")
+        pack_level = moveA.move_line_ids.package_level_id
+
+        # Remove the product A from the package in the destination.
+        moveA.move_line_ids.result_package_id = False
+        self.assertEqual(moveA.move_line_ids.result_package_id.id, False, "No package should be linked in the destination.")
+        self.assertEqual(moveA.move_line_ids.package_level_id.id, False, "Package level should have been unlinked from this move line.")
+        self.assertEqual(moveB.move_line_ids.result_package_id.id, pack.id, "Package should have stayed the same.")
+        self.assertEqual(moveB.move_line_ids.package_level_id.id, pack_level.id, "Package level should have stayed the same.")
+
+        # Validate the picking
+        moveA.move_line_ids.qty_done = 5
+        moveB.move_line_ids.qty_done = 4
+        picking.button_validate()
+
+        # Check that the quants have their expected location/package/quantities
+        quantA = self.env['stock.quant'].search([('product_id', '=', self.productA.id), ('location_id', '=', loc_2.id)])
+        quantB = self.env['stock.quant'].search([('product_id', '=', self.productB.id), ('location_id', '=', loc_2.id)])
+        self.assertEqual(pack.location_id.id, loc_2.id, "Package should have been moved to Location B.")
+        self.assertEqual(quantA.quantity, 5, "All 5 units of product A should be in location B")
+        self.assertEqual(quantA.package_id.id, False, "There should be no package for product A as it was removed in the move.")
+        self.assertEqual(quantB.quantity, 4, "All 4 units of product B should be in location B")
+        self.assertEqual(quantB.package_id.id, pack.id, "Product B should still be in the initial package.")
