@@ -2,15 +2,16 @@
 
 from odoo.addons.stock.tests.common2 import TestStockCommon
 from odoo.tests import Form
-from odoo.exceptions import AccessError
+from odoo.exceptions import UserError
 from odoo.tools import mute_logger
 
 
 class TestWarehouse(TestStockCommon):
 
-    def setUp(self):
-        super(TestWarehouse, self).setUp()
-        self.partner = self.env['res.partner'].create({'name': 'Deco Addict'})
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.partner = cls.env['res.partner'].create({'name': 'Deco Addict'})
 
     def test_inventory_product(self):
         self.product_1.type = 'product'
@@ -19,28 +20,29 @@ class TestWarehouse(TestStockCommon):
             'inventory_quantity': 50.0,
             'location_id': self.warehouse_1.lot_stock_id.id,
         })
-        inventory = self.env['stock.inventory'].with_user(self.user_stock_manager).create({
-            'name': 'Starting for product_1',
-            'location_ids': [(4, self.warehouse_1.lot_stock_id.id)],
-            'product_ids': [(4, self.product_1.id)],
-        })
-        inventory.action_start()
-        # As done in common.py, there is already an inventory line existing
-        self.assertEqual(len(inventory.line_ids), 1)
-        self.assertEqual(inventory.line_ids.theoretical_qty, 50.0)
-        self.assertEqual(inventory.line_ids.product_id, self.product_1)
-        self.assertEqual(inventory.line_ids.product_uom_id, self.product_1.uom_id)
+        product_1_quant.action_apply_inventory()
 
-        # Update the line, set to 35
-        inventory.line_ids.write({'product_qty': 35.0})
-        inventory.action_validate()
+        # Make sure the inventory was successful
+        move_in_id = self.env['stock.move'].search([('is_inventory', '=', True), ('product_id', '=', self.product_1.id)])
+        self.assertEqual(len(move_in_id), 1)
+        self.assertEqual(move_in_id.product_qty, 50.0)
+        self.assertEqual(product_1_quant.quantity, 50.0)
+        self.assertEqual(move_in_id.product_uom, self.product_1.uom_id)
+        self.assertEqual(move_in_id.state, 'done')
+
+        # Update the inventory, set to 35
+        product_1_quant.inventory_quantity = 35.0
+        product_1_quant.action_apply_inventory()
 
         # Check related move and quants
-        self.assertIn(inventory.name, inventory.move_ids.name)
-        self.assertEqual(inventory.move_ids.product_qty, 15.0)
-        self.assertEqual(inventory.move_ids.location_id, self.warehouse_1.lot_stock_id)
-        self.assertEqual(inventory.move_ids.location_dest_id, self.product_1.property_stock_inventory)  # Inventory loss
-        self.assertEqual(inventory.move_ids.state, 'done')
+        move_ids = self.env['stock.move'].search([('is_inventory', '=', True), ('product_id', '=', self.product_1.id)])
+        self.assertEqual(len(move_ids), 2)
+        move_out_id = move_ids[-1]
+        self.assertEqual(move_out_id.product_qty, 15.0)
+        self.assertEqual(move_out_id.location_id, self.warehouse_1.lot_stock_id)
+        self.assertEqual(move_out_id.location_dest_id, self.product_1.property_stock_inventory)  # Inventory loss
+        self.assertEqual(move_out_id.state, 'done')
+
         quants = self.env['stock.quant']._gather(self.product_1, self.product_1.property_stock_inventory)
         self.assertEqual(len(quants), 1)  # One quant created for inventory loss
 
@@ -82,15 +84,8 @@ class TestWarehouse(TestStockCommon):
             'new_quantity': 50.0,
         })
         # User has no right on quant, must raise an AccessError
-        with self.assertRaises(AccessError):
+        with self.assertRaises(UserError):
             inventory_wizard.change_product_qty()
-        # Check quantity wasn't updated
-        self.assertEqual(self.product_1.virtual_available, 0.0)
-        self.assertEqual(self.product_1.qty_available, 0.0)
-
-        # Check associated quants: 0 quant expected
-        quant = self.env['stock.quant'].search([('id', 'not in', self.existing_quants.ids)])
-        self.assertEqual(len(quant), 0)
 
     def test_basic_move(self):
         product = self.product_3.with_user(self.user_stock_manager)
@@ -172,7 +167,7 @@ class TestWarehouse(TestStockCommon):
             'location_dest_id': customer_location.id,
         })
         picking_out.action_confirm()
-        picking_out.move_lines.quantity_done = 1
+        picking_out.move_ids.quantity_done = 1
         picking_out._action_done()
 
         quant = self.env['stock.quant'].search([('product_id', '=', productA.id), ('location_id', '=', stock_location.id)])
@@ -185,7 +180,7 @@ class TestWarehouse(TestStockCommon):
         stock_return_picking_action = stock_return_picking.create_returns()
         return_pick = self.env['stock.picking'].browse(stock_return_picking_action['res_id'])
         return_pick.action_assign()
-        return_pick.move_lines.quantity_done = 1
+        return_pick.move_ids.quantity_done = 1
         return_pick._action_done()
 
         quant = self.env['stock.quant'].search([('product_id', '=', productA.id), ('location_id', '=', stock_location.id)])
@@ -215,29 +210,25 @@ class TestWarehouse(TestStockCommon):
             'location_dest_id': customer_location.id,
         })
         picking_out.action_confirm()
-        picking_out.move_lines.quantity_done = 1
+        picking_out.move_ids.quantity_done = 1
         picking_out._action_done()
 
         # Make an inventory adjustment to set the quantity to 0
-        inventory = self.env['stock.inventory'].create({
-            'name': 'Starting for product_1',
-            'location_ids': [(4, stock_location.id)],
-            'product_ids': [(4, productA.id)],
-        })
-        inventory.action_start()
-        self.assertEqual(len(inventory.line_ids), 1, "Wrong inventory lines generated.")
-        self.assertEqual(inventory.line_ids.theoretical_qty, -1, "Theoretical quantity should be -1.")
-        inventory.line_ids.product_qty = 0  # Put the quantity back to 0
-        inventory.action_validate()
+        quant = self.env['stock.quant'].search([('product_id', '=', productA.id), ('location_id', '=', stock_location.id)])
+        self.assertEqual(len(quant), 1, "Wrong number of quants created.")
+        self.assertEqual(quant.quantity, -1, "Theoretical quantity should be -1.")
+        # Put the quantity back to 0
+        quant.inventory_quantity = 0
+        quant.action_apply_inventory()
 
         # The inventory adjustment should have created one
-        self.assertEqual(len(inventory.move_ids), 1)
-        quantity = inventory.move_ids.mapped('product_qty')
-        self.assertEqual(quantity, [1], "Moves created with wrong quantity.")
-        location_ids = inventory.move_ids.mapped('location_id').ids
-        self.assertEqual(set(location_ids), {location_loss.id})
+        move = self.env['stock.move'].search([('product_id', '=', productA.id), ('is_inventory', '=', True)])
+        self.assertEqual(len(move), 1)
+        self.assertEqual(move.product_qty, 1, "Moves created with wrong quantity.")
+        self.assertEqual(move.location_id.id, location_loss.id)
 
         # There should be no quant in the stock location
+        self.env['stock.quant']._quant_tasks()
         quants = self.env['stock.quant'].search([('product_id', '=', productA.id), ('location_id', '=', stock_location.id)])
         self.assertEqual(sum(quants.mapped('quantity')), 0)
 
@@ -256,10 +247,12 @@ class TestWarehouse(TestStockCommon):
             'code': 'STK',
         })
 
+        distribution_partner = self.env['res.partner'].create({'name': 'Distribution Center'})
         warehouse_distribution = self.env['stock.warehouse'].create({
             'name': 'Dist.',
             'code': 'DIST',
-            'resupply_wh_ids': [(6, 0, [warehouse_stock.id])]
+            'resupply_wh_ids': [(6, 0, [warehouse_stock.id])],
+            'partner_id': distribution_partner.id,
         })
 
         warehouse_shop = self.env['stock.warehouse'].create({
@@ -316,6 +309,9 @@ class TestWarehouse(TestStockCommon):
         self.assertTrue(self.env['stock.move'].search([('location_dest_id', '=', warehouse_shop.lot_stock_id.id)]))
         self.assertTrue(self.env['stock.move'].search([('location_id', '=', warehouse_shop.lot_stock_id.id)]))
 
+        self.assertTrue(self.env['stock.picking'].search([('location_id', '=', self.env.company.internal_transit_location_id.id), ('partner_id', '=', distribution_partner.id)]))
+        self.assertTrue(self.env['stock.picking'].search([('location_dest_id', '=', self.env.company.internal_transit_location_id.id), ('partner_id', '=', distribution_partner.id)]))
+
     def test_mutiple_resupply_warehouse(self):
         """ Simulate the following situation:
         - 2 shops with stock are resupply by 2 distinct warehouses
@@ -388,17 +384,17 @@ class TestWarehouse(TestStockCommon):
         picking_stock_transit = self.env['stock.picking'].search([('location_id', '=', warehouse_distribution_namur.lot_stock_id.id)])
         self.assertTrue(picking_stock_transit)
         picking_stock_transit.action_assign()
-        picking_stock_transit.move_lines[0].quantity_done = 1.0
+        picking_stock_transit.move_ids[0].quantity_done = 1.0
         picking_stock_transit._action_done()
 
         picking_transit_shop_namur = self.env['stock.picking'].search([('location_dest_id', '=', warehouse_shop_namur.lot_stock_id.id)])
         self.assertTrue(picking_transit_shop_namur)
         picking_transit_shop_namur.action_assign()
-        picking_transit_shop_namur.move_lines[0].quantity_done = 1.0
+        picking_transit_shop_namur.move_ids[0].quantity_done = 1.0
         picking_transit_shop_namur._action_done()
 
         picking_out_namur.action_assign()
-        picking_out_namur.move_lines[0].quantity_done = 1.0
+        picking_out_namur.move_ids[0].quantity_done = 1.0
         picking_out_namur._action_done()
 
         # Check that the correct quantity has been provided to customer
@@ -432,17 +428,17 @@ class TestWarehouse(TestStockCommon):
         picking_stock_transit = self.env['stock.picking'].search([('location_id', '=', warehouse_distribution_wavre.lot_stock_id.id)])
         self.assertTrue(picking_stock_transit)
         picking_stock_transit.action_assign()
-        picking_stock_transit.move_lines[0].quantity_done = 1.0
+        picking_stock_transit.move_ids[0].quantity_done = 1.0
         picking_stock_transit._action_done()
 
         picking_transit_shop_wavre = self.env['stock.picking'].search([('location_dest_id', '=', warehouse_shop_wavre.lot_stock_id.id)])
         self.assertTrue(picking_transit_shop_wavre)
         picking_transit_shop_wavre.action_assign()
-        picking_transit_shop_wavre.move_lines[0].quantity_done = 1.0
+        picking_transit_shop_wavre.move_ids[0].quantity_done = 1.0
         picking_transit_shop_wavre._action_done()
 
         picking_out_wavre.action_assign()
-        picking_out_wavre.move_lines[0].quantity_done = 1.0
+        picking_out_wavre.move_ids[0].quantity_done = 1.0
         picking_out_wavre._action_done()
 
         # Check that the correct quantity has been provided to customer
@@ -508,6 +504,7 @@ class TestWarehouse(TestStockCommon):
 
         # Picking Type
         self.assertFalse(warehouse.in_type_id.active)
+        self.assertFalse(warehouse.in_type_id.show_operations)
         self.assertFalse(warehouse.out_type_id.active)
         self.assertFalse(warehouse.int_type_id.active)
         self.assertFalse(warehouse.pick_type_id.active)
@@ -532,12 +529,15 @@ class TestWarehouse(TestStockCommon):
 
         # Picking Type
         self.assertTrue(warehouse.in_type_id.active)
+        self.assertFalse(warehouse.in_type_id.show_operations)
         self.assertTrue(warehouse.out_type_id.active)
         self.assertTrue(warehouse.int_type_id.active)
         self.assertFalse(warehouse.pick_type_id.active)
         self.assertFalse(warehouse.pack_type_id.active)
 
     def test_toggle_active_warehouse_2(self):
+        # Required for `delivery_steps` to be visible in the view
+        self.env.user.groups_id += self.env.ref('stock.group_adv_location')
         wh = Form(self.env['stock.warehouse'])
         wh.name = "The attic of Willy"
         wh.code = "WIL"
@@ -557,7 +557,7 @@ class TestWarehouse(TestStockCommon):
             'warehouse_ids': [(4, self.warehouse_1.id)]
         })
 
-        route = Form(self.env['stock.location.route'])
+        route = Form(self.env['stock.route'])
         route.name = "Stair"
         route = route.save()
 
@@ -629,3 +629,23 @@ class TestWarehouse(TestStockCommon):
         wh.save()
         self.assertEqual(warehouse.int_type_id.barcode, 'CH-INTERNAL')
         self.assertEqual(warehouse.int_type_id.sequence_id.prefix, 'CH/INT/')
+
+    def test_location_warehouse(self):
+        """ Check that the closest warehouse is selected
+        in a warehouse within warehouse situation
+        """
+        wh = self.env.ref("stock.warehouse0")
+        test_warehouse = self.warehouse_1
+        location = test_warehouse.lot_stock_id
+        self.assertEqual(location.warehouse_id, test_warehouse)
+
+        test_warehouse.view_location_id.location_id = wh.lot_stock_id.id
+        wh.sequence = 100
+        test_warehouse.sequence = 1
+        location._compute_warehouse_id()
+        self.assertEqual(location.warehouse_id, test_warehouse)
+
+        wh.sequence = 1
+        test_warehouse.sequence = 100
+        location._compute_warehouse_id()
+        self.assertEqual(location.warehouse_id, test_warehouse)
