@@ -1,9 +1,15 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from . import product_label_layout
 from . import stock_assign_serial_numbers
 from . import stock_picking_return
 from . import stock_change_product_qty
+from . import stock_inventory_conflict
+from . import stock_inventory_warning
+from . import stock_inventory_adjustment_name
+from . import stock_label_type
+from . import stock_lot_label_layout
 from . import stock_scheduler_compute
 from . import stock_immediate_transfer
 from . import stock_backorder_confirmation
@@ -14,3 +20,5 @@ from . import product_replenish
 from . import stock_track_confirmation
 from . import stock_package_destination
 from . import stock_orderpoint_snooze
+from . import stock_request_count
+from . import stock_replenishment_info
