@@ -11,44 +11,45 @@ from odoo.tests import tagged, Form
 @tagged('post_install', '-at_install')
 class TestLandedCosts(TestStockLandedCostsCommon):
 
-    def setUp(self):
-        super(TestLandedCosts, self).setUp()
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
         # Create picking incoming shipment
-        self.picking_in = self.Picking.create({
-            'partner_id': self.supplier_id,
-            'picking_type_id': self.warehouse.in_type_id.id,
-            'location_id': self.supplier_location_id,
-            'location_dest_id': self.warehouse.lot_stock_id.id})
-        self.Move.create({
-            'name': self.product_refrigerator.name,
-            'product_id': self.product_refrigerator.id,
+        cls.picking_in = cls.Picking.create({
+            'partner_id': cls.supplier_id,
+            'picking_type_id': cls.warehouse.in_type_id.id,
+            'location_id': cls.supplier_location_id,
+            'location_dest_id': cls.warehouse.lot_stock_id.id})
+        cls.Move.create({
+            'name': cls.product_refrigerator.name,
+            'product_id': cls.product_refrigerator.id,
             'product_uom_qty': 5,
-            'product_uom': self.product_refrigerator.uom_id.id,
-            'picking_id': self.picking_in.id,
-            'location_id': self.supplier_location_id,
-            'location_dest_id': self.warehouse.lot_stock_id.id})
-        self.Move.create({
-            'name': self.product_oven.name,
-            'product_id': self.product_oven.id,
+            'product_uom': cls.product_refrigerator.uom_id.id,
+            'picking_id': cls.picking_in.id,
+            'location_id': cls.supplier_location_id,
+            'location_dest_id': cls.warehouse.lot_stock_id.id})
+        cls.Move.create({
+            'name': cls.product_oven.name,
+            'product_id': cls.product_oven.id,
             'product_uom_qty': 10,
-            'product_uom': self.product_oven.uom_id.id,
-            'picking_id': self.picking_in.id,
-            'location_id': self.supplier_location_id,
-            'location_dest_id': self.warehouse.lot_stock_id.id})
+            'product_uom': cls.product_oven.uom_id.id,
+            'picking_id': cls.picking_in.id,
+            'location_id': cls.supplier_location_id,
+            'location_dest_id': cls.warehouse.lot_stock_id.id})
         # Create picking outgoing shipment
-        self.picking_out = self.Picking.create({
-            'partner_id': self.customer_id,
-            'picking_type_id': self.warehouse.out_type_id.id,
-            'location_id': self.warehouse.lot_stock_id.id,
-            'location_dest_id': self.customer_location_id})
-        self.Move.create({
-            'name': self.product_refrigerator.name,
-            'product_id': self.product_refrigerator.id,
+        cls.picking_out = cls.Picking.create({
+            'partner_id': cls.customer_id,
+            'picking_type_id': cls.warehouse.out_type_id.id,
+            'location_id': cls.warehouse.lot_stock_id.id,
+            'location_dest_id': cls.customer_location_id})
+        cls.Move.create({
+            'name': cls.product_refrigerator.name,
+            'product_id': cls.product_refrigerator.id,
             'product_uom_qty': 2,
-            'product_uom': self.product_refrigerator.uom_id.id,
-            'picking_id': self.picking_out.id,
-            'location_id': self.warehouse.lot_stock_id.id,
-            'location_dest_id': self.customer_location_id})
+            'product_uom': cls.product_refrigerator.uom_id.id,
+            'picking_id': cls.picking_out.id,
+            'location_id': cls.warehouse.lot_stock_id.id,
+            'location_dest_id': cls.customer_location_id})
 
     def test_00_landed_costs_on_incoming_shipment(self):
         """ Test landed cost on incoming shipment """
@@ -93,11 +94,76 @@ class TestLandedCosts(TestStockLandedCostsCommon):
         self._validate_additional_landed_cost_lines(stock_landed_cost, valid_vals)
         # Validate the landed cost.
         stock_landed_cost.button_validate()
-        self.assertTrue(stock_landed_cost.account_move_id, 'Landed costs should be available account move lines')
-        account_entry = self.env['account.move.line'].read_group(
-            [('move_id', '=', stock_landed_cost.account_move_id.id)], ['debit', 'credit', 'move_id'], ['move_id'])[0]
-        self.assertEqual(account_entry['debit'], account_entry['credit'], 'Debit and credit are not equal')
-        self.assertEqual(account_entry['debit'], 430.0, 'Wrong Account Entry')
+
+        self.assertRecordValues(stock_landed_cost.account_move_id.line_ids, [
+            {'name': 'equal split - Refrigerator', 'balance': 5},
+            {'name': 'equal split - Refrigerator', 'balance': -5},
+            {'name': 'split by quantity - Refrigerator', 'balance': 50},
+            {'name': 'split by quantity - Refrigerator', 'balance': -50},
+            {'name': 'split by weight - Refrigerator', 'balance': 50},
+            {'name': 'split by weight - Refrigerator', 'balance': -50},
+            {'name': 'split by volume - Refrigerator', 'balance': 5},
+            {'name': 'split by volume - Refrigerator', 'balance': -5},
+            {'name': 'equal split - Microwave Oven', 'balance': 5},
+            {'name': 'equal split - Microwave Oven', 'balance': -5},
+            {'name': 'split by quantity - Microwave Oven', 'balance': 100},
+            {'name': 'split by quantity - Microwave Oven', 'balance': -100},
+            {'name': 'split by weight - Microwave Oven', 'balance': 200},
+            {'name': 'split by weight - Microwave Oven', 'balance': -200},
+            {'name': 'split by volume - Microwave Oven', 'balance': 15},
+            {'name': 'split by volume - Microwave Oven', 'balance': -15},
+        ])
+
+    def test_00_landed_costs_on_incoming_shipment_without_real_time(self):
+        chart_of_accounts = self.env.company.chart_template_id
+        generic_coa = self.env.ref('l10n_generic_coa.configurable_chart_template')
+        if chart_of_accounts != generic_coa:
+            raise unittest.SkipTest('Skip this test as it works only with %s (%s loaded)' % (generic_coa.name, chart_of_accounts.name))
+        # Test landed cost on incoming shipment
+        #
+        # (A) Purchase product
+
+        #         Services           Quantity       Weight      Volume
+        #         -----------------------------------------------------
+        #         1. Refrigerator         5            10          1
+        #         2. Oven                 10           20          1.5
+
+        # (B) Add some costs on purchase
+
+        #         Services           Amount     Split Method
+        #         -------------------------------------------
+        #         1.labour            10        By Equal
+        #         2.brokerage         150       By Quantity
+        #         3.transportation    250       By Weight
+        #         4.packaging         20        By Volume
+
+        self.product_refrigerator.write({"categ_id": self.categ_manual_periodic.id})
+        self.product_oven.write({"categ_id": self.categ_manual_periodic.id})
+        # Process incoming shipment
+        income_ship = self._process_incoming_shipment()
+        # Create landed costs
+        stock_landed_cost = self._create_landed_costs({
+            'equal_price_unit': 10,
+            'quantity_price_unit': 150,
+            'weight_price_unit': 250,
+            'volume_price_unit': 20}, income_ship)
+        # Compute landed costs
+        stock_landed_cost.compute_landed_cost()
+
+        valid_vals = {
+            'equal': 5.0,
+            'by_quantity_refrigerator': 50.0,
+            'by_quantity_oven': 100.0,
+            'by_weight_refrigerator': 50.0,
+            'by_weight_oven': 200,
+            'by_volume_refrigerator': 5.0,
+            'by_volume_oven': 15.0}
+
+        # Check valuation adjustment line recognized or not
+        self._validate_additional_landed_cost_lines(stock_landed_cost, valid_vals)
+        # Validate the landed cost.
+        stock_landed_cost.button_validate()
+        self.assertFalse(stock_landed_cost.account_move_id)
 
     def test_01_negative_landed_costs_on_incoming_shipment(self):
         """ Test negative landed cost on incoming shipment """
@@ -178,8 +244,8 @@ class TestLandedCosts(TestStockLandedCostsCommon):
         self.assertEqual(stock_negative_landed_cost.state, 'done', 'Negative landed costs should be in done state')
         self.assertTrue(stock_negative_landed_cost.account_move_id, 'Landed costs should be available account move lines')
         account_entry = self.env['account.move.line'].read_group(
-            [('move_id', '=', stock_negative_landed_cost.account_move_id.id)], ['debit', 'credit', 'move_id'], ['move_id'])[0]
-        self.assertEqual(account_entry['debit'], account_entry['credit'], 'Debit and credit are not equal')
+            [('move_id', '=', stock_negative_landed_cost.account_move_id.id)], ['balance', 'move_id'], ['move_id'])[0]
+        self.assertEqual(account_entry['balance'], 0, 'Move is not balanced')
         move_lines = [
             {'name': 'split by volume - Microwave Oven',                    'debit': 3.75,  'credit': 0.0},
             {'name': 'split by volume - Microwave Oven',                    'debit': 0.0,   'credit': 3.75},
@@ -302,12 +368,7 @@ class TestLandedCostsWithPurchaseAndInv(TestStockValuationLCCommon):
         self.env.company.anglo_saxon_accounting = True
         self.product1.product_tmpl_id.categ_id.property_cost_method = 'fifo'
         self.product1.product_tmpl_id.categ_id.property_valuation = 'real_time'
-        self.price_diff_account = self.env['account.account'].create({
-            'name': 'price diff account',
-            'code': 'price diff account',
-            'user_type_id': self.env.ref('account.data_account_type_current_assets').id,
-        })
-        self.product1.property_account_creditor_price_difference = self.price_diff_account
+        stock_valuation_account = self.company_data['default_account_stock_valuation']
 
         # Create PO
         po_form = Form(self.env['purchase.order'])
@@ -321,13 +382,13 @@ class TestLandedCostsWithPurchaseAndInv(TestStockValuationLCCommon):
 
         # Receive the goods
         receipt = order.picking_ids[0]
-        receipt.move_lines.quantity_done = 1
+        receipt.move_ids.quantity_done = 1
         receipt.button_validate()
 
         # Check SVL and AML
-        svl = self.env['stock.valuation.layer'].search([('stock_move_id', '=', receipt.move_lines.id)])
+        svl = self.env['stock.valuation.layer'].search([('stock_move_id', '=', receipt.move_ids.id)])
         self.assertAlmostEqual(svl.value, 455)
-        aml = self.env['account.move.line'].search([('account_id', '=', self.company_data['default_account_stock_valuation'].id)])
+        aml = self.env['account.move.line'].search([('account_id', '=', stock_valuation_account.id)])
         self.assertAlmostEqual(aml.debit, 455)
 
         # Create and validate LC
@@ -348,18 +409,19 @@ class TestLandedCostsWithPurchaseAndInv(TestStockValuationLCCommon):
 
         # Check LC, SVL and AML
         self.assertAlmostEqual(lc.valuation_adjustment_lines.final_cost, 554)
-        svl = self.env['stock.valuation.layer'].search([('stock_move_id', '=', receipt.move_lines.id)], order='id desc', limit=1)
+        svl = self.env['stock.valuation.layer'].search([('stock_move_id', '=', receipt.move_ids.id)], order='id desc', limit=1)
         self.assertAlmostEqual(svl.value, 99)
-        aml = self.env['account.move.line'].search([('account_id', '=', self.company_data['default_account_stock_valuation'].id)], order='id desc', limit=1)
+        aml = self.env['account.move.line'].search([('account_id', '=', stock_valuation_account.id)], order='id desc', limit=1)
         self.assertAlmostEqual(aml.debit, 99)
 
         # Create an invoice with the same price
         move_form = Form(self.env['account.move'].with_context(default_move_type='in_invoice'))
+        move_form.invoice_date = move_form.date
         move_form.partner_id = order.partner_id
-        move_form.purchase_id = order
+        move_form.purchase_vendor_bill_id = self.env['purchase.bill.union'].browse(-order.id)
         move = move_form.save()
         move.action_post()
 
-        # Check nothing was posted in the price difference account
-        price_diff_aml = self.env['account.move.line'].search([('account_id','=', self.price_diff_account.id), ('move_id', '=', move.id)])
-        self.assertEqual(len(price_diff_aml), 0, "No line should have been generated in the price difference account.")
+        # Check nothing was posted in the stock valuation account.
+        price_diff_aml = self.env['account.move.line'].search([('account_id', '=', stock_valuation_account.id), ('move_id', '=', move.id)])
+        self.assertEqual(len(price_diff_aml), 0, "No line should have been generated in the stock valuation account about the price difference.")
