@@ -65,7 +65,7 @@ class TestStockValuationLCCommon(TestStockLandedCostsCommon):
         lc.compute_landed_cost()
         lc.button_validate()
         return lc
-    
+
     def _make_in_move(self, product, quantity, unit_cost=None, create_picking=False):
         """ Helper to create and validate a receipt move.
         """
@@ -137,10 +137,11 @@ class TestStockValuationLCCommon(TestStockLandedCostsCommon):
 
 @tagged('-at_install', 'post_install')
 class TestStockValuationLCFIFO(TestStockValuationLCCommon):
-    def setUp(self):
-        super(TestStockValuationLCFIFO, self).setUp()
-        self.product1.product_tmpl_id.categ_id.property_cost_method = 'fifo'
-        self.product1.product_tmpl_id.categ_id.property_valuation = 'real_time'
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.product1.product_tmpl_id.categ_id.property_cost_method = 'fifo'
+        cls.product1.product_tmpl_id.categ_id.property_valuation = 'real_time'
 
     def test_normal_1(self):
         move1 = self._make_in_move(self.product1, 10, unit_cost=10, create_picking=True)
@@ -253,10 +254,11 @@ class TestStockValuationLCFIFO(TestStockValuationLCCommon):
 
 @tagged('-at_install', 'post_install')
 class TestStockValuationLCAVCO(TestStockValuationLCCommon):
-    def setUp(self):
-        super(TestStockValuationLCAVCO, self).setUp()
-        self.product1.product_tmpl_id.categ_id.property_cost_method = 'average'
-        self.product1.product_tmpl_id.categ_id.property_valuation = 'real_time'
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.product1.product_tmpl_id.categ_id.property_cost_method = 'average'
+        cls.product1.product_tmpl_id.categ_id.property_valuation = 'real_time'
 
     def test_normal_1(self):
         move1 = self._make_in_move(self.product1, 10, unit_cost=10, create_picking=True)
@@ -341,6 +343,7 @@ class TestStockValuationLCFIFOVB(TestStockValuationLCCommon):
         # Create a vendor bill for the RFQ
         action = rfq.action_create_invoice()
         vb = self.env['account.move'].browse(action['res_id'])
+        vb.invoice_date = vb.date
         vb.action_post()
 
         input_aml = self._get_stock_input_move_lines()[-1]
@@ -353,6 +356,7 @@ class TestStockValuationLCFIFOVB(TestStockValuationLCCommon):
         # Create a vendor bill for a landed cost product, post it and validate a landed cost
         # linked to this vendor bill. LC; 1@50
         lcvb = Form(self.env['account.move'].with_context(default_move_type='in_invoice'))
+        lcvb.invoice_date = lcvb.date
         lcvb.partner_id = self.vendor2
         with lcvb.invoice_line_ids.new() as inv_line:
             inv_line.product_id = self.productlc1
@@ -429,6 +433,7 @@ class TestStockValuationLCFIFOVB(TestStockValuationLCCommon):
         # Create a vendor bill for the RFQ and add to it the landed cost
         vb = Form(self.env['account.move'].with_context(default_move_type='in_invoice'))
         vb.partner_id = self.vendor1
+        vb.invoice_date = vb.date
         with vb.invoice_line_ids.new() as inv_line:
             inv_line.product_id = self.productlc1
             inv_line.price_unit = 50
@@ -481,6 +486,7 @@ class TestStockValuationLCFIFOVB(TestStockValuationLCCommon):
         # Create a vebdor bill for the RFQ
         action = rfq.action_create_invoice()
         vb = self.env['account.move'].browse(action['res_id'])
+        vb.invoice_date = vb.date
         vb.action_post()
 
         expense_aml = self._get_expense_move_lines()[-1]
@@ -495,6 +501,7 @@ class TestStockValuationLCFIFOVB(TestStockValuationLCCommon):
         # linked to this vendor bill. LC; 1@50
         lcvb = Form(self.env['account.move'].with_context(default_move_type='in_invoice'))
         lcvb.partner_id = self.vendor2
+        lcvb.invoice_date = lcvb.date
         with lcvb.invoice_line_ids.new() as inv_line:
             inv_line.product_id = self.productlc1
             inv_line.price_unit = 50
