@@ -17,6 +17,7 @@ class Survey(models.Model):
     and each page can display one or more questions. """
     _name = 'survey.survey'
     _description = 'Survey'
+    _order = 'create_date DESC'
     _rec_name = 'title'
     _inherit = ['mail.thread', 'mail.activity.mixin']
 
@@ -44,6 +45,15 @@ class Survey(models.Model):
 
         return False  # could not generate a code
 
+    @api.model
+    def default_get(self, fields_list):
+        result = super().default_get(fields_list)
+        # allows to propagate the text one write in a many2one widget after
+        # clicking on 'Create and Edit...' to the popup form.
+        if 'title' in fields_list and not result.get('title') and self.env.context.get('default_name'):
+            result['title'] = self.env.context.get('default_name')
+        return result
+
     # description
     title = fields.Char('Survey Title', required=True, translate=True)
     color = fields.Integer('Color Index', default=0)
@@ -53,29 +63,31 @@ class Survey(models.Model):
     description_done = fields.Html(
         "End Message", translate=True,
         help="This message will be displayed when survey is completed")
-    background_image = fields.Binary("Background Image")
+    background_image = fields.Image("Background Image")
+    background_image_url = fields.Char('Background Url', compute="_compute_background_image_url")
     active = fields.Boolean("Active", default=True)
-    state = fields.Selection(selection=[
-        ('draft', 'Draft'), ('open', 'In Progress'), ('closed', 'Closed')
-    ], string="Survey Stage", default='draft', required=True,
-        group_expand='_read_group_states')
+    user_id = fields.Many2one(
+        'res.users', string='Responsible',
+        domain=[('share', '=', False)], tracking=True,
+        default=lambda self: self.env.user)
     # questions
     question_and_page_ids = fields.One2many('survey.question', 'survey_id', string='Sections and Questions', copy=True)
     page_ids = fields.One2many('survey.question', string='Pages', compute="_compute_page_and_question_ids")
     question_ids = fields.One2many('survey.question', string='Questions', compute="_compute_page_and_question_ids")
+    question_count = fields.Integer('# Questions', compute="_compute_page_and_question_ids")
     questions_layout = fields.Selection([
-        ('one_page', 'One page with all the questions'),
+        ('page_per_question', 'One page per question'),
         ('page_per_section', 'One page per section'),
-        ('page_per_question', 'One page per question')],
-        string="Layout", required=True, default='one_page')
+        ('one_page', 'One page with all the questions')],
+        string="Pagination", required=True, default='page_per_question')
     questions_selection = fields.Selection([
         ('all', 'All questions'),
-        ('random', 'Randomized per section')],
-        string="Selection", required=True, default='all',
+        ('random', 'Randomized per Section')],
+        string="Question Selection", required=True, default='all',
         help="If randomized is selected, you can configure the number of random questions by section. This mode is ignored in live session.")
     progression_mode = fields.Selection([
-        ('percent', 'Percentage'),
-        ('number', 'Number')], string='Progression Mode', default='percent',
+        ('percent', 'Percentage left'),
+        ('number', 'Number')], string='Display Progress as', default='percent',
         help="If Number is selected, it will display the number of questions answered on the total number of question to answer.")
     # attendees
     user_input_ids = fields.One2many('survey.user_input', 'survey_id', string='User responses', readonly=True, groups='survey.group_survey_user')
@@ -85,22 +97,23 @@ class Survey(models.Model):
         ('token', 'Invited people only')], string='Access Mode',
         default='public', required=True)
     access_token = fields.Char('Access Token', default=lambda self: self._get_default_access_token(), copy=False)
-    users_login_required = fields.Boolean('Login Required', help="If checked, users have to login before answering even with a valid token.")
+    users_login_required = fields.Boolean('Require Login', help="If checked, users have to login before answering even with a valid token.")
     users_can_go_back = fields.Boolean('Users can go back', help="If checked, users can go back to previous pages.")
     users_can_signup = fields.Boolean('Users can signup', compute='_compute_users_can_signup')
     # statistics
     answer_count = fields.Integer("Registered", compute="_compute_survey_statistic")
     answer_done_count = fields.Integer("Attempts", compute="_compute_survey_statistic")
-    answer_score_avg = fields.Float("Avg Score %", compute="_compute_survey_statistic")
+    answer_score_avg = fields.Float("Avg Score (%)", compute="_compute_survey_statistic")
+    answer_duration_avg = fields.Float("Average Duration", compute="_compute_answer_duration_avg", help="Average duration of the survey (in hours)")
     success_count = fields.Integer("Success", compute="_compute_survey_statistic")
-    success_ratio = fields.Integer("Success Ratio", compute="_compute_survey_statistic")
+    success_ratio = fields.Integer("Success Ratio (%)", compute="_compute_survey_statistic")
     # scoring
     scoring_type = fields.Selection([
         ('no_scoring', 'No scoring'),
         ('scoring_with_answers', 'Scoring with answers at the end'),
         ('scoring_without_answers', 'Scoring without answers at the end')],
-        string="Scoring", required=True, default='no_scoring')
-    scoring_success_min = fields.Float('Success %', default=80.0)
+        string="Scoring", required=True, store=True, readonly=False, compute="_compute_scoring_type", precompute=True)
+    scoring_success_min = fields.Float('Required Score (%)', default=80.0)
     # attendees context: attempts and time limitation
     is_attempts_limited = fields.Boolean('Limited number of attempts', help="Check this option if you want to limit the number of attempts per user",
                                          compute="_compute_is_attempts_limited", store=True, readonly=False)
@@ -109,11 +122,11 @@ class Survey(models.Model):
     time_limit = fields.Float("Time limit (minutes)", default=10)
     # certification
     certification = fields.Boolean('Is a Certification', compute='_compute_certification',
-                                   readonly=False, store=True)
+                                   readonly=False, store=True, precompute=True)
     certification_mail_template_id = fields.Many2one(
-        'mail.template', 'Email Template',
+        'mail.template', 'Certified Email Template',
         domain="[('model', '=', 'survey.user_input')]",
-        help="Automated email sent to the user when he succeeds the certification, containing his certification document.")
+        help="Automated email sent to the user when they succeed the certification, containing their certification document.")
     certification_report_layout = fields.Selection([
         ('modern_purple', 'Modern Purple'),
         ('modern_blue', 'Modern Blue'),
@@ -128,8 +141,8 @@ class Survey(models.Model):
     #   - If the certification badge is set, show certification_badge_id_dummy in 'no create' mode.
     #       So it can be edited but not removed or replaced.
     certification_give_badge = fields.Boolean('Give Badge', compute='_compute_certification_give_badge',
-                                              readonly=False, store=True)
-    certification_badge_id = fields.Many2one('gamification.badge', 'Certification Badge')
+                                              readonly=False, store=True, copy=False)
+    certification_badge_id = fields.Many2one('gamification.badge', 'Certification Badge', copy=False)
     certification_badge_id_dummy = fields.Many2one(related='certification_badge_id', string='Certification Badge ')
     # live sessions
     session_state = fields.Selection([
@@ -166,10 +179,14 @@ class Survey(models.Model):
         ('attempts_limit_check', "CHECK( (is_attempts_limited=False) OR (attempts_limit is not null AND attempts_limit > 0) )",
             'The attempts limit needs to be a positive number if the survey has a limited number of attempts.'),
         ('badge_uniq', 'unique (certification_badge_id)', "The badge for each survey should be unique!"),
-        ('give_badge_check', "CHECK(certification_give_badge=False OR (certification_give_badge=True AND certification_badge_id is not null))",
-            'Certification badge must be configured if Give Badge is set.'),
     ]
 
+    @api.depends('background_image', 'access_token')
+    def _compute_background_image_url(self):
+        self.background_image_url = False
+        for survey in self.filtered(lambda survey: survey.background_image and survey.access_token):
+            survey.background_image_url = "/survey/%s/get_background_image" % survey.access_token
+
     def _compute_users_can_signup(self):
         signup_allowed = self.env['res.users'].sudo()._get_signup_invitation_scope() == 'b2c'
         for survey in self:
@@ -183,9 +200,9 @@ class Survey(models.Model):
         }
         stat = dict((cid, dict(default_vals, answer_score_avg_total=0.0)) for cid in self.ids)
         UserInput = self.env['survey.user_input']
-        base_domain = ['&', ('survey_id', 'in', self.ids), ('test_entry', '!=', True)]
+        base_domain = [('survey_id', 'in', self.ids)]
 
-        read_group_res = UserInput.read_group(base_domain, ['survey_id', 'state'], ['survey_id', 'state', 'scoring_percentage', 'scoring_success'], lazy=False)
+        read_group_res = UserInput._read_group(base_domain, ['survey_id', 'state'], ['survey_id', 'state', 'scoring_percentage', 'scoring_success'], lazy=False)
         for item in read_group_res:
             stat[item['survey_id'][0]]['answer_count'] += item['__count']
             stat[item['survey_id'][0]]['answer_score_avg_total'] += item['scoring_percentage']
@@ -194,19 +211,41 @@ class Survey(models.Model):
             if item['scoring_success']:
                 stat[item['survey_id'][0]]['success_count'] += item['__count']
 
-        for survey_id, values in stat.items():
-            avg_total = stat[survey_id].pop('answer_score_avg_total')
-            stat[survey_id]['answer_score_avg'] = avg_total / (stat[survey_id]['answer_done_count'] or 1)
-            stat[survey_id]['success_ratio'] = (stat[survey_id]['success_count'] / (stat[survey_id]['answer_done_count'] or 1.0))*100
+        for survey_stats in stat.values():
+            avg_total = survey_stats.pop('answer_score_avg_total')
+            survey_stats['answer_score_avg'] = avg_total / (survey_stats['answer_done_count'] or 1)
+            survey_stats['success_ratio'] = (survey_stats['success_count'] / (survey_stats['answer_done_count'] or 1.0))*100
 
         for survey in self:
             survey.update(stat.get(survey._origin.id, default_vals))
 
+    @api.depends('user_input_ids.survey_id', 'user_input_ids.start_datetime', 'user_input_ids.end_datetime')
+    def _compute_answer_duration_avg(self):
+        result_per_survey_id = {}
+        if self.ids:
+            self.env.cr.execute(
+                """SELECT survey_id,
+                          avg((extract(epoch FROM end_datetime)) - (extract (epoch FROM start_datetime)))
+                     FROM survey_user_input
+                    WHERE survey_id = any(%s) AND state = 'done'
+                          AND end_datetime IS NOT NULL
+                          AND start_datetime IS NOT NULL
+                 GROUP BY survey_id""",
+                [self.ids]
+            )
+            result_per_survey_id = dict(self.env.cr.fetchall())
+
+        for survey in self:
+            # as avg returns None if nothing found, set 0 if it's the case.
+            survey.answer_duration_avg = (result_per_survey_id.get(survey.id) or 0) / 3600
+
+
     @api.depends('question_and_page_ids')
     def _compute_page_and_question_ids(self):
         for survey in self:
             survey.page_ids = survey.question_and_page_ids.filtered(lambda question: question.is_page)
             survey.question_ids = survey.question_and_page_ids - survey.page_ids
+            survey.question_count = len(survey.question_ids)
 
     @api.depends('question_and_page_ids.is_conditional', 'users_login_required', 'access_mode')
     def _compute_is_attempts_limited(self):
@@ -224,7 +263,7 @@ class Survey(models.Model):
 
         for survey in self:
             answer_count = 0
-            input_count = self.env['survey.user_input'].read_group(
+            input_count = self.env['survey.user_input']._read_group(
                 [('survey_id', '=', survey.id),
                  ('is_session_answer', '=', True),
                  ('state', '!=', 'done'),
@@ -245,7 +284,7 @@ class Survey(models.Model):
         context of sessions, so it should not matter too much. """
         for survey in self:
             answer_count = 0
-            input_line_count = self.env['survey.user_input.line'].read_group(
+            input_line_count = self.env['survey.user_input.line']._read_group(
                 [('question_id', '=', survey.session_question_id.id),
                  ('survey_id', '=', survey.id),
                  ('create_date', '>=', survey.session_start_time)],
@@ -294,20 +333,24 @@ class Survey(models.Model):
                not survey.certification:
                 survey.certification_give_badge = False
 
-    def _read_group_states(self, values, domain, order):
-        selection = self.env['survey.survey'].fields_get(allfields=['state'])['state']['selection']
-        return [s[0] for s in selection]
+    @api.depends('certification')
+    def _compute_scoring_type(self):
+        for survey in self:
+            if survey.certification and survey.scoring_type not in ['scoring_without_answers', 'scoring_with_answers']:
+                survey.scoring_type = 'scoring_without_answers'
+            elif not survey.scoring_type:
+                survey.scoring_type = 'no_scoring'
 
     # ------------------------------------------------------------
     # CRUD
     # ------------------------------------------------------------
 
-    @api.model
-    def create(self, vals):
-        survey = super(Survey, self).create(vals)
-        if vals.get('certification_give_badge'):
-            survey.sudo()._create_certification_badge_trigger()
-        return survey
+    @api.model_create_multi
+    def create(self, vals_list):
+        surveys = super(Survey, self).create(vals_list)
+        for survey_sudo in surveys.filtered(lambda survey: survey.certification_give_badge).sudo():
+            survey_sudo._create_certification_badge_trigger()
+        return surveys
 
     def write(self, vals):
         result = super(Survey, self).write(vals)
@@ -315,9 +358,37 @@ class Survey(models.Model):
             return self.sudo()._handle_certification_badges(vals)
         return result
 
+    @api.returns('self', lambda value: value.id)
+    def copy(self, default=None):
+        """ Correctly copy the 'triggering_question_id' and 'triggering_answer_id' fields from the original
+        to the clone.
+        This needs to be done in post-processing to make sure we get references to the newly created
+        answers/questions from the copy instead of references to the answers/questions of the original.
+        This implementation assumes that the order of created questions/answers will be kept between
+        the original and the clone, using 'zip()' to match the records between the two.
+
+        Note that when question_ids is provided in the default parameter, it falls back to the standard copy.
+        """
+        self.ensure_one()
+        clone = super(Survey, self).copy(default)
+        if default and 'question_ids' in default:
+            return clone
+
+        questions_map = {src.id: dst.id for src, dst in zip(self.question_ids, clone.question_ids)}
+        answers_map = {
+            source_answer.id: copy_answer.id
+            for source_answer, copy_answer
+            in zip(self.question_ids.suggested_answer_ids, clone.question_ids.suggested_answer_ids)
+        }
+        for src, dst in zip(self.question_ids, clone.question_ids):
+            if src.is_conditional:
+                dst.triggering_question_id = questions_map.get(src.triggering_question_id.id)
+                dst.triggering_answer_id = answers_map.get(src.triggering_answer_id.id)
+        return clone
+
     def copy_data(self, default=None):
-        title = _("%s (copy)") % (self.title)
-        default = dict(default or {}, title=title)
+        new_defaults = {'title': _("%s (copy)") % (self.title)}
+        default = dict(new_defaults, **(default or {}))
         return super(Survey, self).copy_data(default)
 
     def toggle_active(self):
@@ -402,9 +473,7 @@ class Survey(models.Model):
                 raise exceptions.UserError(_('Creating test token is not allowed for you.'))
         else:
             if not self.active:
-                raise exceptions.UserError(_('Creating token for archived surveys is not allowed.'))
-            elif self.state == 'closed':
-                raise exceptions.UserError(_('Creating token for closed surveys is not allowed.'))
+                raise exceptions.UserError(_('Creating token for closed/archived surveys is not allowed.'))
             if self.access_mode == 'authentication':
                 # signup possible -> should have at least a partner to create an account
                 if self.users_can_signup and not user and not partner:
@@ -412,7 +481,7 @@ class Survey(models.Model):
                 # no signup possible -> should be a not public user (employee or portal users)
                 if not self.users_can_signup and (not user or user._is_public()):
                     raise exceptions.UserError(_('Creating token for external people is not allowed for surveys requesting authentication.'))
-            if self.access_mode == 'internal' and (not user or not user.has_group('base.group_user')):
+            if self.access_mode == 'internal' and (not user or not user._is_internal()):
                 raise exceptions.UserError(_('Creating token for anybody else than employees is not allowed for internal surveys.'))
             if check_attempts and not self._has_attempts_left(partner or (user and user.partner_id), email, invite_token):
                 raise exceptions.UserError(_('No attempts left.'))
@@ -512,11 +581,30 @@ class Survey(models.Model):
             if self.questions_selection == 'random' and not self.session_state:
                 result = user_input.predefined_question_ids
             else:
-                result = self.question_and_page_ids.filtered(
-                    lambda question: not question.is_page or not is_html_empty(question.description))
+                result = self._get_pages_and_questions_to_show()
 
         return result
 
+    def _get_pages_and_questions_to_show(self):
+        """
+        :return: survey.question recordset excluding invalid conditional questions and pages without description
+        """
+
+        self.ensure_one()
+        invalid_questions = self.env['survey.question']
+        questions_and_valid_pages = self.question_and_page_ids.filtered(
+            lambda question: not question.is_page or not is_html_empty(question.description))
+        for question in questions_and_valid_pages.filtered(lambda q: q.is_conditional).sorted():
+            trigger = question.triggering_question_id
+            if (trigger in invalid_questions
+                    or trigger.is_page
+                    or trigger.question_type not in ['simple_choice', 'multiple_choice']
+                    or not trigger.suggested_answer_ids
+                    or trigger.sequence > question.sequence
+                    or (trigger.sequence == question.sequence and trigger.id > question.id)):
+                invalid_questions |= question
+        return questions_and_valid_pages - invalid_questions
+
     def _get_next_page_or_question(self, user_input, page_or_question_id, go_back=False):
         """ Generalized logic to retrieve the next question or page to show on the survey.
         It's based on the page_or_question_id parameter, that is usually the currently displayed question/page.
@@ -584,6 +672,17 @@ class Survey(models.Model):
                     return section
             return Question
 
+    def _is_first_page_or_question(self, page_or_question):
+        """ This method checks if the given question or page is the first one to display.
+            If the first section of the survey as a description, this will be the first screen to display.
+            else, the first question will be the first screen to be displayed.
+            This methods is used for survey session management where the host should not be able to go back on the
+            first page or question."""
+        first_section_has_description = self.page_ids and not is_html_empty(self.page_ids[0].description)
+        is_first_page_or_question = (first_section_has_description and page_or_question == self.page_ids[0]) or \
+            (not first_section_has_description and page_or_question == self.question_ids[0])
+        return is_first_page_or_question
+
     def _is_last_page_or_question(self, user_input, page_or_question):
         """ This method checks if the given question or page is the last one.
         This includes conditional questions configuration. If the given question is normally not the last one but
@@ -690,9 +789,9 @@ class Survey(models.Model):
 
         if self.env.user.has_group('survey.group_survey_user'):
             self.sudo().write({'session_state': 'in_progress'})
-            self.sudo().flush(['session_state'])
+            self.sudo().flush_recordset(['session_state'])
 
-    def _get_session_next_question(self):
+    def _get_session_next_question(self, go_back):
         self.ensure_one()
 
         if not self.question_ids or not self.env.user.has_group('survey.group_survey_user'):
@@ -701,7 +800,7 @@ class Survey(models.Model):
         most_voted_answers = self._get_session_most_voted_answers()
         return self._get_next_page_or_question(
             most_voted_answers,
-            self.session_question_id.id if self.session_question_id else 0)
+            self.session_question_id.id if self.session_question_id else 0, go_back=go_back)
 
     def _get_session_most_voted_answers(self):
         """ In sessions of survey that has conditional questions, as the survey is passed at the same time by
@@ -801,15 +900,6 @@ class Survey(models.Model):
     # ACTIONS
     # ------------------------------------------------------------
 
-    def action_draft(self):
-        self.write({'state': 'draft'})
-
-    def action_open(self):
-        self.write({'state': 'open'})
-
-    def action_close(self):
-        self.write({'state': 'closed'})
-
     def action_send_survey(self):
         """ Open a window to compose an email, pre-filled with the survey message """
         # Ensure that this survey has at least one question.
@@ -823,7 +913,7 @@ class Survey(models.Model):
             if not self.page_ids.mapped('question_ids'):
                 raise UserError(_('You cannot send an invitation for a "One page per section" survey if the survey only contains empty sections.'))
 
-        if self.state == 'closed':
+        if not self.active:
             raise exceptions.UserError(_("You cannot send invitations for closed surveys."))
 
         template = self.env.ref('survey.mail_template_user_input_invite', raise_if_not_found=False)
@@ -833,10 +923,11 @@ class Survey(models.Model):
             default_survey_id=self.id,
             default_use_template=bool(template),
             default_template_id=template and template.id or False,
-            notif_layout='mail.mail_notification_light',
+            default_email_layout_xmlid='mail.mail_notification_light',
         )
         return {
             'type': 'ir.actions.act_window',
+            'name': _("Share a Survey"),
             'view_mode': 'form',
             'res_model': 'survey.invite',
             'target': 'new',
@@ -871,7 +962,7 @@ class Survey(models.Model):
         return {
             'type': 'ir.actions.act_url',
             'name': "Results of the Survey",
-            'target': 'self',
+            'target': 'new',
             'url': '/survey/results/%s' % self.id
         }
 
@@ -889,8 +980,7 @@ class Survey(models.Model):
         action = self.env['ir.actions.act_window']._for_xml_id('survey.action_survey_user_input')
         ctx = dict(self.env.context)
         ctx.update({'search_default_survey_id': self.ids[0],
-                    'search_default_completed': 1,
-                    'search_default_not_test': 1})
+                    'search_default_completed': 1})
         action['context'] = ctx
         return action
 
@@ -898,16 +988,14 @@ class Survey(models.Model):
         action = self.env['ir.actions.act_window']._for_xml_id('survey.action_survey_user_input')
         ctx = dict(self.env.context)
         ctx.update({'search_default_survey_id': self.ids[0],
-                    'search_default_scoring_success': 1,
-                    'search_default_not_test': 1})
+                    'search_default_scoring_success': 1})
         action['context'] = ctx
         return action
 
     def action_survey_user_input(self):
         action = self.env['ir.actions.act_window']._for_xml_id('survey.action_survey_user_input')
         ctx = dict(self.env.context)
-        ctx.update({'search_default_survey_id': self.ids[0],
-                    'search_default_not_test': 1})
+        ctx.update({'search_default_survey_id': self.ids[0]})
         action['context'] = ctx
         return action
 
@@ -915,14 +1003,14 @@ class Survey(models.Model):
         self.ensure_one()
         return {
             'type': 'ir.actions.act_url',
-            'target': '_blank',
-            'url': '/survey/%s/get_certification_preview' % (self.id)
+            'target': 'new',
+            'url': '/survey/%s/certification_preview' % (self.id)
         }
 
     def action_start_session(self):
         """ Sets the necessary fields for the session to take place and starts it.
         The write is sudo'ed because a survey user can start a session even if it's
-        not his own survey. """
+        not their own survey. """
 
         if not self.env.user.has_group('survey.group_survey_user'):
             raise AccessError(_('Only survey users can manage sessions.'))
@@ -942,20 +1030,20 @@ class Survey(models.Model):
         return {
             'type': 'ir.actions.act_url',
             'name': "Open Session Manager",
-            'target': 'self',
+            'target': 'new',
             'url': '/survey/session/manage/%s' % self.access_token
         }
 
     def action_end_session(self):
         """ The write is sudo'ed because a survey user can end a session even if it's
-        not his own survey. """
+        not their own survey. """
 
         if not self.env.user.has_group('survey.group_survey_user'):
             raise AccessError(_('Only survey users can manage sessions.'))
 
         self.sudo().write({'session_state': False})
         self.user_input_ids.sudo().write({'state': 'done'})
-        self.env['bus.bus'].sendone(self.access_token, {'type': 'end_session'})
+        self.env['bus.bus']._sendone(self.access_token, 'end_session', {})
 
     def get_start_url(self):
         return '/survey/start/%s' % self.access_token
@@ -983,11 +1071,12 @@ class Survey(models.Model):
                 ('state', '=', 'done'),
                 ('test_entry', '=', False)
             ]
-        count_data = self.env['survey.user_input'].sudo().read_group(user_input_domain, ['scoring_success', 'id:count_distinct'], ['scoring_success'])
+        count_data_success = self.env['survey.user_input'].sudo()._read_group(user_input_domain, ['scoring_success', 'id:count_distinct'], ['scoring_success'])
+        completed_count = self.env['survey.user_input'].sudo().search_count(user_input_domain + [('state', "=", "done")])
 
         scoring_success_count = 0
         scoring_failed_count = 0
-        for count_data_item in count_data:
+        for count_data_item in count_data_success:
             if count_data_item['scoring_success']:
                 scoring_success_count += count_data_item['scoring_success_count']
             else:
@@ -998,7 +1087,7 @@ class Survey(models.Model):
             'count': scoring_success_count,
             'color': '#2E7D32'
         }, {
-            'text': _('Missed'),
+            'text': _('Failed'),
             'count': scoring_failed_count,
             'color': '#C62828'
         }])
@@ -1006,7 +1095,11 @@ class Survey(models.Model):
         total = scoring_success_count + scoring_failed_count
         return {
             'global_success_rate': round((scoring_success_count / total) * 100, 1) if total > 0 else 0,
-            'global_success_graph': success_graph
+            'global_success_graph': success_graph,
+            'count_all': total,
+            'count_finished': completed_count,
+            'count_failed': scoring_failed_count,
+            'count_passed': total - scoring_failed_count
         }
 
     # ------------------------------------------------------------
@@ -1018,6 +1111,9 @@ class Survey(models.Model):
 
     def _create_certification_badge_trigger(self):
         self.ensure_one()
+        if not self.certification_badge_id:
+            raise ValueError(_('Certification Badge is not configured for the survey %(survey_name)s', survey_name=self.title))
+
         goal = self.env['gamification.goal.definition'].create({
             'name': self.title,
             'description': _("%s certification passed", self.title),
