@@ -27,7 +27,7 @@ class TestAccess(common.TestSurveyCommon):
         with self.assertRaises(AccessError):
             self.env['survey.survey'].create({'title': 'Test Survey 2'})
         with self.assertRaises(AccessError):
-            self.env['survey.question'].create({'title': 'My Page', 'sequence': 0, 'is_page': True, 'survey_id': self.survey.id})
+            self.env['survey.question'].create({'title': 'My Page', 'sequence': 0, 'is_page': True, 'question_type': False, 'survey_id': self.survey.id})
         with self.assertRaises(AccessError):
             self.env['survey.question'].create({'title': 'My Question', 'sequence': 1, 'page_id': self.page_0.id})
 
@@ -60,7 +60,7 @@ class TestAccess(common.TestSurveyCommon):
         with self.assertRaises(AccessError):
             self.env['survey.survey'].create({'title': 'Test Survey 2'})
         with self.assertRaises(AccessError):
-            self.env['survey.question'].create({'title': 'My Page', 'sequence': 0, 'is_page': True, 'survey_id': self.survey.id})
+            self.env['survey.question'].create({'title': 'My Page', 'sequence': 0, 'is_page': True, 'question_type': False, 'survey_id': self.survey.id})
         with self.assertRaises(AccessError):
             self.env['survey.question'].create({'title': 'My Question', 'sequence': 1, 'page_id': self.page_0.id})
 
@@ -93,7 +93,7 @@ class TestAccess(common.TestSurveyCommon):
         with self.assertRaises(AccessError):
             self.env['survey.survey'].create({'title': 'Test Survey 2'})
         with self.assertRaises(AccessError):
-            self.env['survey.question'].create({'title': 'My Page', 'sequence': 0, 'is_page': True, 'survey_id': self.survey.id})
+            self.env['survey.question'].create({'title': 'My Page', 'sequence': 0, 'is_page': True, 'question_type': False, 'survey_id': self.survey.id})
         with self.assertRaises(AccessError):
             self.env['survey.question'].create({'title': 'My Question', 'sequence': 1, 'page_id': self.page_0.id})
 
@@ -123,7 +123,7 @@ class TestAccess(common.TestSurveyCommon):
     def test_access_survey_survey_manager(self):
         # Create: all
         survey = self.env['survey.survey'].create({'title': 'Test Survey 2'})
-        self.env['survey.question'].create({'title': 'My Page', 'sequence': 0, 'is_page': True, 'survey_id': survey.id})
+        self.env['survey.question'].create({'title': 'My Page', 'sequence': 0, 'is_page': True, 'question_type': False, 'survey_id': survey.id})
         self.env['survey.question'].create({'title': 'My Question', 'sequence': 1, 'survey_id': survey.id})
 
         # Read: all
@@ -142,7 +142,7 @@ class TestAccess(common.TestSurveyCommon):
     def test_access_survey_survey_user(self):
         # Create: own only
         survey = self.env['survey.survey'].create({'title': 'Test Survey 2'})
-        self.env['survey.question'].create({'title': 'My Page', 'sequence': 0, 'is_page': True, 'survey_id': survey.id})
+        self.env['survey.question'].create({'title': 'My Page', 'sequence': 0, 'is_page': True, 'question_type': False, 'survey_id': survey.id})
         self.env['survey.question'].create({'title': 'My Question', 'sequence': 1, 'survey_id': survey.id})
 
         # Read: all
@@ -251,24 +251,24 @@ class TestAccess(common.TestSurveyCommon):
     @users('survey_user')
     def test_access_answers_survey_user(self):
         survey_own = self.env['survey.survey'].create({'title': 'Other'})
-        self.env['survey.question'].create({'title': 'Other', 'sequence': 0, 'is_page': True, 'survey_id': survey_own.id})
+        self.env['survey.question'].create({'title': 'Other', 'sequence': 0, 'is_page': True, 'question_type': False, 'survey_id': survey_own.id})
         question_own = self.env['survey.question'].create({'title': 'Other Question', 'sequence': 1, 'survey_id': survey_own.id})
 
         # Create: own survey only
         answer_own = self.env['survey.user_input'].create({'survey_id': survey_own.id})
-        answer_line_own = self.env['survey.user_input.line'].create({'question_id': question_own.id, 'answer_type': 'numerical_box', 'value_numerical_box': 3, 'user_input_id': answer_own.id})
+        with self.assertRaises(AccessError):
+            self.env['survey.user_input.line'].create({'question_id': question_own.id, 'answer_type': 'numerical_box', 'value_numerical_box': 3, 'user_input_id': answer_own.id})
 
         # Read: always
         answers = self.env['survey.user_input'].search([('survey_id', 'in', [survey_own.id, self.survey.id])])
         self.assertEqual(answers, answer_own | self.answer_0)
 
         answer_lines = self.env['survey.user_input.line'].search([('survey_id', 'in', [survey_own.id, self.survey.id])])
-        self.assertEqual(answer_lines, answer_line_own | self.answer_0_0 | self.answer_0_1)
+        self.assertEqual(answer_lines, self.answer_0_0 | self.answer_0_1)
 
         self.env['survey.user_input'].browse(answer_own.ids).read(['state'])
         self.env['survey.user_input'].browse(self.answer_0.ids).read(['state'])
 
-        self.env['survey.user_input.line'].browse(answer_line_own.ids).read(['value_numerical_box'])
         self.env['survey.user_input.line'].browse(self.answer_0_0.ids).read(['value_numerical_box'])
 
         # Create: own survey only (moved after read because DB not correctly rollbacked with assertRaises)
@@ -294,7 +294,7 @@ class TestAccess(common.TestSurveyCommon):
         admin = self.env.ref('base.user_admin')
         with self.with_user(admin.login):
             survey_other = self.env['survey.survey'].create({'title': 'Other'})
-            self.env['survey.question'].create({'title': 'Other', 'sequence': 0, 'is_page': True, 'survey_id': survey_other.id})
+            self.env['survey.question'].create({'title': 'Other', 'sequence': 0, 'is_page': True, 'question_type': False, 'survey_id': survey_other.id})
             question_other = self.env['survey.question'].create({'title': 'Other Question', 'sequence': 1, 'survey_id': survey_other.id})
             self.assertEqual(survey_other.create_uid, admin)
             self.assertEqual(question_other.create_uid, admin)
@@ -331,12 +331,9 @@ class TestSurveySecurityControllers(common.TestSurveyCommon, HttpCase):
     def test_survey_start_short(self):
         # avoid name clash with existing data
         surveys = self.env['survey.survey'].search([
-            ('state', '=', 'open'),
             ('session_state', 'in', ['ready', 'in_progress'])
         ])
-        surveys.write({'state': 'done'})
         self.survey.write({
-            'state': 'open',
             'session_state': 'ready',
             'session_code': '123456',
             'session_start_time': datetime.datetime.now(),
@@ -345,20 +342,20 @@ class TestSurveySecurityControllers(common.TestSurveyCommon, HttpCase):
         })
 
         # right short access token
-        response = self.url_open(f'/s/123456')
+        response = self.url_open('/s/123456')
         self.assertEqual(response.status_code, 200)
         self.assertIn('The session will begin automatically when the host starts', response.text)
 
         # `like` operator injection
-        response = self.url_open(f'/s/______')
+        response = self.url_open('/s/______')
         self.assertFalse(self.survey.title in response.text)
 
-        # right short token, but wrong state
-        self.survey.state = 'draft'
-        response = self.url_open(f'/s/123456')
+        # right short token, but closed survey
+        self.survey.action_archive()
+        response = self.url_open('/s/123456')
         self.assertFalse(self.survey.title in response.text)
 
         # right short token, but wrong `session_state`
-        self.survey.write({'state': 'open', 'session_state': False})
-        response = self.url_open(f'/s/123456')
+        self.survey.write({'session_state': False, 'active': True})
+        response = self.url_open('/s/123456')
         self.assertFalse(self.survey.title in response.text)
