@@ -3,4 +3,7 @@
 
 from . import common
 from . import test_event_crm
+from . import test_event_event
+from . import test_event_security
+from . import test_performance
 from . import test_wevent_register
