@@ -9,17 +9,28 @@
     'description': """This module contains tests related to mail. Those are
 present in a separate module as it contains models used only to perform
 tests independently to functional aspects of other models. """,
-    'depends': ['test_performance', 'mail'],
+    'depends': [
+        'mail',
+        'test_performance',
+    ],
     'data': [
         'security/ir.model.access.csv',
         'security/test_mail_security.xml',
         'data/data.xml',
+        'data/mail_template_data.xml',
         'data/subtype_data.xml',
-        'data/template_data.xml',
-    ],
-    'demo': [
     ],
+    'assets': {
+        'web.qunit_suite_tests': [
+            'test_mail/static/tests/*',
+        ],
+        'web.qunit_mobile_suite_tests': [
+            'test_mail/static/tests/mobile/activity_tests.js',
+        ],
+        'web.tests_assets': [
+            'test_mail/static/tests/helpers/*',
+        ],
+    },
     'installable': True,
-    'application': False,
     'license': 'LGPL-3',
 }
