@@ -2,9 +2,11 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo.addons.test_mail.tests.common import TestMailCommon
+from odoo.tests import tagged
 from odoo.tools import mute_logger
 
 
+@tagged('mail_followers')
 class TestInvite(TestMailCommon):
 
     @mute_logger('odoo.addons.mail.models.mail_mail')
@@ -26,8 +28,6 @@ class TestInvite(TestMailCommon):
         # check added followers and that emails were sent
         self.assertEqual(test_record.message_partner_ids,
                          test_partner | self.user_admin.partner_id)
-        self.assertEqual(test_record.message_follower_ids.mapped('channel_id'),
-                         self.env['mail.channel'])
         self.assertSentEmail(self.partner_employee, [test_partner])
         self.assertSentEmail(self.partner_employee, [self.partner_admin])
         self.assertEqual(len(self._mails), 2)
