@@ -7,18 +7,18 @@ from unittest.mock import patch
 from odoo.addons.mail.tests.common import mail_new_test_user
 from odoo.addons.test_mail.tests.common import TestMailCommon
 from odoo.addons.test_mail.models.test_mail_models import MailTestSimple
-from odoo.exceptions import AccessError
-from odoo.tools import mute_logger, formataddr
-from odoo.tests import tagged
+from odoo.exceptions import AccessError, UserError
+from odoo.tools import is_html_empty, mute_logger, formataddr
+from odoo.tests import tagged, users
 
 
+@tagged('mail_message')
 class TestMessageValues(TestMailCommon):
 
     @classmethod
     def setUpClass(cls):
         super(TestMessageValues, cls).setUpClass()
 
-        cls._init_mail_gateway()
         cls.alias_record = cls.env['mail.test.container'].with_context(cls._test_context).create({
             'name': 'Pigs',
             'alias_name': 'pigs',
@@ -26,8 +26,182 @@ class TestMessageValues(TestMailCommon):
         })
         cls.Message = cls.env['mail.message'].with_user(cls.user_employee)
 
+    @users('employee')
+    def test_empty_message(self):
+        """ Test that message is correctly considered as empty (see `_filter_empty()`).
+        Message considered as empty if:
+            - no body or empty body
+            - AND no subtype or no subtype description
+            - AND no tracking values
+            - AND no attachment
+
+        Check _update_content behavior when voiding messages (cleanup side
+        records: stars, notifications).
+        """
+        note_subtype = self.env.ref('mail.mt_note')
+        _attach_1 = self.env['ir.attachment'].with_user(self.user_employee).create({
+            'name': 'Attach1',
+            'datas': 'bWlncmF0aW9uIHRlc3Q=',
+            'res_id': 0,
+            'res_model': 'mail.compose.message',
+        })
+        record = self.env['mail.test.track'].create({'name': 'EmptyTesting'})
+        self.flush_tracking()
+        record.message_subscribe(partner_ids=self.partner_admin.ids, subtype_ids=note_subtype.ids)
+        message = record.message_post(
+            attachment_ids=_attach_1.ids,
+            body='Test',
+            message_type='comment',
+            subtype_id=note_subtype.id,
+        )
+        message.write({'starred_partner_ids': [(4, self.partner_admin.id)]})
+
+        # check content
+        self.assertEqual(len(message.attachment_ids), 1)
+        self.assertFalse(is_html_empty(message.body))
+        self.assertEqual(len(message.sudo().notification_ids), 1)
+        self.assertEqual(message.notified_partner_ids, self.partner_admin)
+        self.assertEqual(message.starred_partner_ids, self.partner_admin)
+        self.assertFalse(message.sudo().tracking_value_ids)
+
+        # Reset body case
+        record._message_update_content(message, '<p><br /></p>', attachment_ids=message.attachment_ids.ids)
+        self.assertTrue(is_html_empty(message.body))
+        self.assertFalse(message.sudo()._filter_empty(), 'Still having attachments')
+
+        # Subtype content
+        note_subtype.sudo().write({'description': 'Very important discussions'})
+        record._message_update_content(message, '', [])
+        self.assertFalse(message.attachment_ids)
+        self.assertEqual(message.notified_partner_ids, self.partner_admin)
+        self.assertEqual(message.starred_partner_ids, self.partner_admin)
+        self.assertFalse(message.sudo()._filter_empty(), 'Subtype with description')
+
+        # Completely void now
+        note_subtype.sudo().write({'description': ''})
+        self.assertEqual(message.sudo()._filter_empty(), message)
+        record._message_update_content(message, '', [])
+        self.assertFalse(message.notified_partner_ids)
+        self.assertFalse(message.starred_partner_ids)
+
+        # test tracking values
+        record.write({'user_id': self.user_admin.id})
+        self.flush_tracking()
+        tracking_message = record.message_ids[0]
+        self.assertFalse(tracking_message.attachment_ids)
+        self.assertTrue(is_html_empty(tracking_message.body))
+        self.assertFalse(tracking_message.subtype_id.description)
+        self.assertFalse(tracking_message.sudo()._filter_empty(), 'Has tracking values')
+        with self.assertRaises(UserError, msg='Tracking values prevent from updating content'):
+            record._message_update_content(tracking_message, '', [])
+
     @mute_logger('odoo.models.unlink')
-    def test_mail_message_values_no_document_values(self):
+    def test_mail_message_format(self):
+        record1 = self.env['mail.test.simple'].create({'name': 'Test1'})
+        message = self.env['mail.message'].create([{
+            'model': 'mail.test.simple',
+            'res_id': record1.id,
+        }])
+        res = message.message_format()
+        self.assertEqual(res[0].get('record_name'), 'Test1')
+
+        record1.write({"name": "Test2"})
+        res = message.message_format()
+        self.assertEqual(res[0].get('record_name'), 'Test2')
+
+    @mute_logger('odoo.models.unlink')
+    def test_mail_message_format_access(self):
+        """
+        User that doesn't have access to a record should still be able to fetch
+        the record_name inside message_format.
+        """
+        company_2 = self.env['res.company'].create({'name': 'Second Test Company'})
+        record1 = self.env['mail.test.multi.company'].create({
+            'name': 'Test1',
+            'company_id': company_2.id,
+        })
+        message = record1.message_post(body='', partner_ids=[self.user_employee.partner_id.id])
+        # We need to flush and invalidate the ORM cache since the record_name
+        # is already cached from the creation. Otherwise it will leak inside
+        # message_format.
+        self.env.flush_all()
+        self.env.invalidate_all()
+        res = message.with_user(self.user_employee).message_format()
+        self.assertEqual(res[0].get('record_name'), 'Test1')
+
+    def test_mail_message_values_body_base64_image(self):
+        msg = self.env['mail.message'].with_user(self.user_employee).create({
+            'body': 'taratata <img src="data:image/png;base64,iV/+OkI=" width="2"> <img src="data:image/png;base64,iV/+OkI=" width="2">',
+        })
+        self.assertEqual(len(msg.attachment_ids), 1)
+        self.assertEqual(
+            msg.body,
+            '<p>taratata <img src="/web/image/{attachment.id}?access_token={attachment.access_token}" alt="image0" width="2"> '
+            '<img src="/web/image/{attachment.id}?access_token={attachment.access_token}" alt="image0" width="2"></p>'.format(attachment=msg.attachment_ids[0])
+        )
+
+    @mute_logger('odoo.models.unlink')
+    @users('employee')
+    def test_mail_message_values_fromto_long_name(self):
+        """ Long headers may break in python if above 78 chars as folding is not
+        done correctly (see ``_notify_get_reply_to_formatted_email`` docstring
+        + commit linked to this test). """
+        # name would make it blow up: keep only email
+        test_record = self.env['mail.test.container'].browse(self.alias_record.ids)
+        test_record.write({
+            'name': 'Super Long Name That People May Enter "Even with an internal quoting of stuff"'
+        })
+        msg = self.env['mail.message'].create({
+            'model': test_record._name,
+            'res_id': test_record.id
+        })
+        reply_to_email = f"{test_record.alias_name}@{self.alias_domain}"
+        self.assertEqual(msg.reply_to, reply_to_email,
+                         'Reply-To: use only email when formataddr > 78 chars')
+
+        # name + company_name would make it blow up: keep record_name in formatting
+        test_record.write({'name': 'Name that would be more than 78 with company name'})
+        msg = self.env['mail.message'].create({
+            'model': test_record._name,
+            'res_id': test_record.id
+        })
+        self.assertEqual(msg.reply_to, formataddr((test_record.name, reply_to_email)),
+                         'Reply-To: use recordname as name in format if recordname + company > 78 chars')
+
+        # no record_name: keep company_name in formatting if ok
+        test_record.write({'name': ''})
+        msg = self.env['mail.message'].create({
+            'model': test_record._name,
+            'res_id': test_record.id
+        })
+        self.assertEqual(msg.reply_to, formataddr((self.env.user.company_id.name, reply_to_email)),
+                         'Reply-To: use company as name in format when no record name and still < 78 chars')
+
+        # no record_name and company_name make it blow up: keep only email
+        self.env.user.company_id.write({'name': 'Super Long Name That People May Enter "Even with an internal quoting of stuff"'})
+        msg = self.env['mail.message'].create({
+            'model': test_record._name,
+            'res_id': test_record.id
+        })
+        self.assertEqual(msg.reply_to, reply_to_email,
+                         'Reply-To: use only email when formataddr > 78 chars')
+
+        # whatever the record and company names, email is too long: keep only email
+        test_record.write({
+            'alias_name': 'Waaaay too long alias name that should make any reply-to blow the 78 characters limit',
+            'name': 'Short',
+        })
+        self.env.user.company_id.write({'name': 'Comp'})
+        sanitized_alias_name = 'waaaay-too-long-alias-name-that-should-make-any-reply-to-blow-the-78-characters-limit'
+        msg = self.env['mail.message'].create({
+            'model': test_record._name,
+            'res_id': test_record.id
+        })
+        self.assertEqual(msg.reply_to, f"{sanitized_alias_name}@{self.alias_domain}",
+                         'Reply-To: even a long email is ok as only formataddr is problematic')
+
+    @mute_logger('odoo.models.unlink')
+    def test_mail_message_values_fromto_no_document_values(self):
         msg = self.Message.create({
             'reply_to': 'test.reply@example.com',
             'email_from': 'test.from@example.com',
@@ -37,7 +211,7 @@ class TestMessageValues(TestMailCommon):
         self.assertEqual(msg.email_from, 'test.from@example.com')
 
     @mute_logger('odoo.models.unlink')
-    def test_mail_message_values_no_document(self):
+    def test_mail_message_values_fromto_no_document(self):
         msg = self.Message.create({})
         self.assertIn('-private', msg.message_id.split('@')[0], 'mail_message: message_id for a void message should be a "private" one')
         reply_to_name = self.env.user.company_id.name
@@ -63,7 +237,7 @@ class TestMessageValues(TestMailCommon):
         self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
     @mute_logger('odoo.models.unlink')
-    def test_mail_message_values_document_alias(self):
+    def test_mail_message_values_fromto_document_alias(self):
         msg = self.Message.create({
             'model': 'mail.test.container',
             'res_id': self.alias_record.id
@@ -100,7 +274,7 @@ class TestMessageValues(TestMailCommon):
         self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
     @mute_logger('odoo.models.unlink')
-    def test_mail_message_values_document_no_alias(self):
+    def test_mail_message_values_fromto_document_no_alias(self):
         test_record = self.env['mail.test.simple'].create({'name': 'Test', 'email_from': 'ignasse@example.com'})
 
         msg = self.Message.create({
@@ -114,7 +288,7 @@ class TestMessageValues(TestMailCommon):
         self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
     @mute_logger('odoo.models.unlink')
-    def test_mail_message_values_document_manual_alias(self):
+    def test_mail_message_values_fromto_document_manual_alias(self):
         test_record = self.env['mail.test.simple'].create({'name': 'Test', 'email_from': 'ignasse@example.com'})
         alias = self.env['mail.alias'].create({
             'alias_name': 'MegaLias',
@@ -135,54 +309,35 @@ class TestMessageValues(TestMailCommon):
         self.assertEqual(msg.reply_to, formataddr((reply_to_name, reply_to_email)))
         self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
-    def test_mail_message_values_no_auto_thread(self):
+    def test_mail_message_values_fromto_reply_to_force_new(self):
         msg = self.Message.create({
             'model': 'mail.test.container',
             'res_id': self.alias_record.id,
-            'no_auto_thread': True,
+            'reply_to_force_new': True,
         })
         self.assertIn('reply_to', msg.message_id.split('@')[0])
         self.assertNotIn('mail.test.container', msg.message_id.split('@')[0])
         self.assertNotIn('-%d-' % self.alias_record.id, msg.message_id.split('@')[0])
 
-    def test_mail_message_base64_image(self):
-        msg = self.env['mail.message'].with_user(self.user_employee).create({
-            'body': 'taratata <img src="data:image/png;base64,iV/+OkI=" width="2"> <img src="data:image/png;base64,iV/+OkI=" width="2">',
-        })
-        self.assertEqual(len(msg.attachment_ids), 1)
-        body = '<p>taratata <img src="/web/image/%s?access_token=%s" alt="image0" width="2"> <img src="/web/image/%s?access_token=%s" alt="image0" width="2"></p>'
-        body = body % (msg.attachment_ids[0].id, msg.attachment_ids[0].access_token, msg.attachment_ids[0].id, msg.attachment_ids[0].access_token)
-        self.assertEqual(msg.body, body)
-
 
+@tagged('mail_message')
 class TestMessageAccess(TestMailCommon):
 
     @classmethod
     def setUpClass(cls):
         super(TestMessageAccess, cls).setUpClass()
 
+        cls.user_employee_1 = mail_new_test_user(cls.env, login='tao', groups='base.group_user', name='Tao Lee')
         cls.user_public = mail_new_test_user(cls.env, login='bert', groups='base.group_public', name='Bert Tartignole')
         cls.user_portal = mail_new_test_user(cls.env, login='chell', groups='base.group_portal', name='Chell Gladys')
 
-        Channel = cls.env['mail.channel'].with_context(cls._test_context)
-        # Pigs: base group for tests
-        cls.group_pigs = Channel.create({
-            'name': 'Pigs',
-            'public': 'groups',
-            'group_public_id': cls.env.ref('base.group_user').id})
-        # Jobs: public group
-        cls.group_public = Channel.create({
-            'name': 'Jobs',
-            'description': 'NotFalse',
-            'public': 'public'})
-        # Private: private gtroup
-        cls.group_private = Channel.create({
-            'name': 'Private',
-            'public': 'private'})
+        cls.group_restricted_channel = cls.env['mail.channel'].browse(cls.env['mail.channel'].channel_create(name='Channel for Groups', group_id=cls.env.ref('base.group_user').id)['id'])
+        cls.public_channel = cls.env['mail.channel'].browse(cls.env['mail.channel'].channel_create(name='Public Channel', group_id=None)['id'])
+        cls.private_group = cls.env['mail.channel'].browse(cls.env['mail.channel'].create_group(partners_to=cls.user_employee_1.partner_id.ids, name="Group")['id'])
         cls.message = cls.env['mail.message'].create({
             'body': 'My Body',
             'model': 'mail.channel',
-            'res_id': cls.group_private.id,
+            'res_id': cls.private_group.id,
         })
 
     @mute_logger('odoo.addons.mail.models.mail_mail')
@@ -195,18 +350,18 @@ class TestMessageAccess(TestMailCommon):
             'partner_ids': [(6, 0, [self.user_public.partner_id.id])]})
         msg3 = self.env['mail.message'].create({
             'subject': '_ZTest', 'body': 'A Pigs', 'subtype_id': False,
-            'model': 'mail.channel', 'res_id': self.group_pigs.id})
+            'model': 'mail.channel', 'res_id': self.group_restricted_channel.id})
         msg4 = self.env['mail.message'].create({
             'subject': '_ZTest', 'body': 'A+P Pigs', 'subtype_id': self.ref('mail.mt_comment'),
-            'model': 'mail.channel', 'res_id': self.group_pigs.id,
+            'model': 'mail.channel', 'res_id': self.group_restricted_channel.id,
             'partner_ids': [(6, 0, [self.user_public.partner_id.id])]})
         msg5 = self.env['mail.message'].create({
             'subject': '_ZTest', 'body': 'A+E Pigs', 'subtype_id': self.ref('mail.mt_comment'),
-            'model': 'mail.channel', 'res_id': self.group_pigs.id,
+            'model': 'mail.channel', 'res_id': self.group_restricted_channel.id,
             'partner_ids': [(6, 0, [self.user_employee.partner_id.id])]})
         msg6 = self.env['mail.message'].create({
             'subject': '_ZTest', 'body': 'A Birds', 'subtype_id': self.ref('mail.mt_comment'),
-            'model': 'mail.channel', 'res_id': self.group_private.id})
+            'model': 'mail.channel', 'res_id': self.private_group.id})
         msg7 = self.env['mail.message'].with_user(self.user_employee).create({
             'subject': '_ZTest', 'body': 'B', 'subtype_id': self.ref('mail.mt_comment')})
         msg8 = self.env['mail.message'].with_user(self.user_employee).create({
@@ -217,11 +372,11 @@ class TestMessageAccess(TestMailCommon):
         messages = self.env['mail.message'].with_user(self.user_public).search([('subject', 'like', '_ZTest')])
         self.assertEqual(messages, msg2 | msg4)
 
-        # Test: Employee: 3 messages on Pigs Raoul can read (employee can read group with default values)
+        # Test: Employee: 3 messages on Channel Employee can read (employee can read group with default values)
         messages = self.env['mail.message'].with_user(self.user_employee).search([('subject', 'like', '_ZTest'), ('body', 'ilike', 'A')])
         self.assertEqual(messages, msg3 | msg4 | msg5)
 
-        # Test: Raoul: 3 messages on Pigs Raoul can read (employee can read group with default values), 0 on Birds (private group) + 2 messages as author
+        # Test: Employee: 3 messages on Channel Employee can read (employee can read group with default values), 0 on Birds (private group) + 2 messages as author
         messages = self.env['mail.message'].with_user(self.user_employee).search([('subject', 'like', '_ZTest')])
         self.assertEqual(messages, msg3 | msg4 | msg5 | msg7 | msg8)
 
@@ -233,8 +388,8 @@ class TestMessageAccess(TestMailCommon):
         messages = self.env['mail.message'].with_user(self.user_portal).search([('subject', 'like', '_ZTest')])
         self.assertFalse(messages)
 
-        # Test: Portal: 2 messages (public group with a subtype)
-        self.group_pigs.write({'public': 'public'})
+        # Test: Portal: 2 messages (public channel)
+        self.group_restricted_channel.write({'group_public_id': False})
         messages = self.env['mail.message'].with_user(self.user_portal).search([('subject', 'like', '_ZTest')])
         self.assertEqual(messages, msg4 | msg5)
 
@@ -253,18 +408,20 @@ class TestMessageAccess(TestMailCommon):
             self.message.with_user(self.user_portal).read(['body', 'message_type', 'subtype_id'])
 
     def test_mail_message_access_read_ok_portal(self):
-        self.message.write({'subtype_id': self.ref('mail.mt_comment'), 'res_id': self.group_public.id})
+        self.message.write({'subtype_id': self.ref('mail.mt_comment'), 'res_id': self.public_channel.id})
         self.message.with_user(self.user_portal).read(['body', 'message_type', 'subtype_id'])
 
     def test_mail_message_access_read_notification(self):
         attachment = self.env['ir.attachment'].create({
             'datas': base64.b64encode(b'My attachment'),
-            'name': 'doc.txt'})
+            'name': 'doc.txt',
+            'res_model': self.message._name,
+            'res_id': self.message.id})
         # attach the attachment to the message
         self.message.write({'attachment_ids': [(4, attachment.id)]})
         self.message.write({'partner_ids': [(4, self.user_employee.partner_id.id)]})
         self.message.with_user(self.user_employee).read()
-        # Test: Bert has access to attachment, ok because he can read message
+        # Test: Employee has access to attachment, ok because they can read message
         attachment.with_user(self.user_employee).read(['name', 'datas'])
 
     def test_mail_message_access_read_author(self):
@@ -272,44 +429,38 @@ class TestMessageAccess(TestMailCommon):
         self.message.with_user(self.user_employee).read()
 
     def test_mail_message_access_read_doc(self):
-        self.message.write({'model': 'mail.channel', 'res_id': self.group_public.id})
-        # Test: Bert reads the message, ok because linked to a doc he is allowed to read
-        self.message.with_user(self.user_employee).read()
-
-    def test_mail_message_access_read_crash_moderation(self):
-        # with self.assertRaises(AccessError):
-        self.message.write({'model': 'mail.channel', 'res_id': self.group_public.id, 'moderation_status': 'pending_moderation'})
-        # Test: Bert reads the message, ok because linked to a doc he is allowed to read
+        self.message.write({'model': 'mail.channel', 'res_id': self.public_channel.id})
+        # Test: Employee reads the message, ok because linked to a doc they are allowed to read
         self.message.with_user(self.user_employee).read()
 
     # --------------------------------------------------
     # CREATE
     # --------------------------------------------------
 
-    @mute_logger('odoo.addons.base.models.ir_model')
+    @mute_logger('odoo.addons.base.models.ir_model', 'odoo.addons.base.models.ir_rule')
     def test_mail_message_access_create_crash_public(self):
-        # Do: Bert creates a message on Pigs -> ko, no creation rights
+        # Public creates a message on Channel for groups -> ko, no enter rights
         with self.assertRaises(AccessError):
-            self.env['mail.message'].with_user(self.user_public).create({'model': 'mail.channel', 'res_id': self.group_pigs.id, 'body': 'Test'})
+            self.env['mail.message'].with_user(self.user_public).create({'model': 'mail.channel', 'res_id': self.group_restricted_channel.id, 'body': 'Test'})
 
-        # Do: Bert create a message on Jobs -> ko, no creation rights
+        # Public create a message on Public Channel -> ko, no creation rights
         with self.assertRaises(AccessError):
-            self.env['mail.message'].with_user(self.user_public).create({'model': 'mail.channel', 'res_id': self.group_public.id, 'body': 'Test'})
+            self.env['mail.message'].with_user(self.user_public).create({'model': 'mail.channel', 'res_id': self.public_channel.id, 'body': 'Test'})
 
     @mute_logger('odoo.models')
     def test_mail_message_access_create_crash(self):
-        # Do: Bert create a private message -> ko, no creation rights
+        # Do: Employee create a private message -> ko, no creation rights
         with self.assertRaises(AccessError):
-            self.env['mail.message'].with_user(self.user_employee).create({'model': 'mail.channel', 'res_id': self.group_private.id, 'body': 'Test'})
+            self.env['mail.message'].with_user(self.user_employee).create({'model': 'mail.channel', 'res_id': self.private_group.id, 'body': 'Test'})
 
     @mute_logger('odoo.models')
     def test_mail_message_access_create_doc(self):
         Message = self.env['mail.message'].with_user(self.user_employee)
-        # Do: Raoul creates a message on Jobs -> ok, write access to the related document
-        Message.create({'model': 'mail.channel', 'res_id': self.group_public.id, 'body': 'Test'})
-        # Do: Raoul creates a message on Priv -> ko, no write access to the related document
+        # Do: Employee creates a message on Public Channel -> ok, write access to the related document
+        Message.create({'model': 'mail.channel', 'res_id': self.public_channel.id, 'body': 'Test'})
+        # Do: Employee creates a message on Group -> ko, no write access to the related document
         with self.assertRaises(AccessError):
-            Message.create({'model': 'mail.channel', 'res_id': self.group_private.id, 'body': 'Test'})
+            Message.create({'model': 'mail.channel', 'res_id': self.private_group.id, 'body': 'Test'})
 
     def test_mail_message_access_create_private(self):
         self.env['mail.message'].with_user(self.user_employee).create({'body': 'Test'})
@@ -318,7 +469,7 @@ class TestMessageAccess(TestMailCommon):
         # TDE FIXME: should it really work ? not sure - catchall makes crash (aka, post will crash also)
         self.env['ir.config_parameter'].set_param('mail.catchall.domain', False)
         self.message.write({'partner_ids': [(4, self.user_employee.partner_id.id)]})
-        self.env['mail.message'].with_user(self.user_employee).create({'model': 'mail.channel', 'res_id': self.group_private.id, 'body': 'Test', 'parent_id': self.message.id})
+        self.env['mail.message'].with_user(self.user_employee).create({'model': 'mail.channel', 'res_id': self.private_group.id, 'body': 'Test', 'parent_id': self.message.id})
 
     def test_mail_message_access_create_wo_parent_access(self):
         """ Purpose is to test posting a message on a record whose first message / parent
@@ -353,205 +504,8 @@ class TestMessageAccess(TestMailCommon):
 
         new_mail = self.env['mail.mail'].sudo().search([
             ('mail_message_id', '=', new_msg.id),
-            ('references', '=', message.message_id),
+            ('references', '=', f'{message.message_id} {new_msg.message_id}'),
         ])
 
         self.assertTrue(new_mail)
         self.assertEqual(new_msg.parent_id, message)
-
-    # --------------------------------------------------
-    # WRITE
-    # --------------------------------------------------
-
-    def test_mail_message_access_write_moderation(self):
-        """ Only moderators can modify pending messages """
-        self.group_public.write({
-            'email_send': True,
-            'moderation': True,
-            'channel_partner_ids': [(4, self.partner_employee.id)],
-            'moderator_ids': [(4, self.user_employee.id)],
-        })
-        self.message.write({'model': 'mail.channel', 'res_id': self.group_public.id, 'moderation_status': 'pending_moderation'})
-        self.message.with_user(self.user_employee).write({'moderation_status': 'accepted'})
-
-    def test_mail_message_access_write_crash_moderation(self):
-        self.message.write({'model': 'mail.channel', 'res_id': self.group_public.id, 'moderation_status': 'pending_moderation'})
-        with self.assertRaises(AccessError):
-            self.message.with_user(self.user_employee).write({'moderation_status': 'accepted'})
-
-    @mute_logger('openerp.addons.mail.models.mail_mail')
-    def test_mark_all_as_read(self):
-        self.user_employee.notification_type = 'inbox'
-        emp_partner = self.user_employee.partner_id.with_user(self.user_employee)
-
-        group_private = self.env['mail.channel'].with_context({
-            'mail_create_nolog': True,
-            'mail_create_nosubscribe': True,
-            'mail_channel_noautofollow': True,
-        }).create({
-            'name': 'Private',
-            'description': 'Private James R.',
-            'public': 'private',
-            'alias_name': 'private',
-            'alias_contact': 'followers'}
-        ).with_context({'mail_create_nosubscribe': False})
-
-        # mark all as read clear needactions
-        msg1 = group_private.message_post(body='Test', message_type='comment', subtype_xmlid='mail.mt_comment', partner_ids=[emp_partner.id])
-        self._reset_bus()
-        emp_partner.env['mail.message'].mark_all_as_read(domain=[])
-        self.assertBusNotifications([(self.cr.dbname, 'res.partner', emp_partner.id)], [{ 'type': 'mark_as_read', 'message_ids': [msg1.id] }])
-        na_count = emp_partner.get_needaction_count()
-        self.assertEqual(na_count, 0, "mark all as read should conclude all needactions")
-
-        # mark all as read also clear inaccessible needactions
-        msg2 = group_private.message_post(body='Zest', message_type='comment', subtype_xmlid='mail.mt_comment', partner_ids=[emp_partner.id])
-        needaction_accessible = len(emp_partner.env['mail.message'].search([['needaction', '=', True]]))
-        self.assertEqual(needaction_accessible, 1, "a new message to a partner is readable to that partner")
-
-        msg2.sudo().partner_ids = self.env['res.partner']
-        emp_partner.env['mail.message'].search([['needaction', '=', True]])
-        needaction_length = len(emp_partner.env['mail.message'].search([['needaction', '=', True]]))
-        self.assertEqual(needaction_length, 1, "message should still be readable when notified")
-
-        na_count = emp_partner.get_needaction_count()
-        self.assertEqual(na_count, 1, "message not accessible is currently still counted")
-
-        self._reset_bus()
-        emp_partner.env['mail.message'].mark_all_as_read(domain=[])
-        self.assertBusNotifications([(self.cr.dbname, 'res.partner', emp_partner.id)], [{ 'type': 'mark_as_read', 'message_ids': [msg2.id] }])
-        na_count = emp_partner.get_needaction_count()
-        self.assertEqual(na_count, 0, "mark all read should conclude all needactions even inacessible ones")
-
-    @mute_logger('openerp.addons.mail.models.mail_mail')
-    def test_mark_all_as_read_share(self):
-        self.user_portal.notification_type = 'inbox'
-        portal_partner = self.user_portal.partner_id.with_user(self.user_portal)
-
-        # mark all as read clear needactions
-        self.group_pigs.message_post(body='Test', message_type='comment', subtype_xmlid='mail.mt_comment', partner_ids=[portal_partner.id])
-        portal_partner.env['mail.message'].mark_all_as_read(domain=[])
-        na_count = portal_partner.get_needaction_count()
-        self.assertEqual(na_count, 0, "mark all as read should conclude all needactions")
-
-        # mark all as read also clear inaccessible needactions
-        new_msg = self.group_pigs.message_post(body='Zest', message_type='comment', subtype_xmlid='mail.mt_comment', partner_ids=[portal_partner.id])
-        needaction_accessible = len(portal_partner.env['mail.message'].search([['needaction', '=', True]]))
-        self.assertEqual(needaction_accessible, 1, "a new message to a partner is readable to that partner")
-
-        new_msg.sudo().partner_ids = self.env['res.partner']
-        needaction_length = len(portal_partner.env['mail.message'].search([['needaction', '=', True]]))
-        self.assertEqual(needaction_length, 1, "message should still be readable when notified")
-
-        na_count = portal_partner.get_needaction_count()
-        self.assertEqual(na_count, 1, "message not accessible is currently still counted")
-
-        portal_partner.env['mail.message'].mark_all_as_read(domain=[])
-        na_count = portal_partner.get_needaction_count()
-        self.assertEqual(na_count, 0, "mark all read should conclude all needactions even inacessible ones")
-
-
-@tagged('moderation')
-class TestMessageModeration(TestMailCommon):
-
-    @classmethod
-    def setUpClass(cls):
-        super(TestMessageModeration, cls).setUpClass()
-
-        cls.channel_1 = cls.env['mail.channel'].create({
-            'name': 'Moderation_1',
-            'email_send': True,
-            'moderation': True
-        })
-        cls.user_employee.write({'moderation_channel_ids': [(6, 0, [cls.channel_1.id])]})
-        cls.user_portal = cls._create_portal_user()
-
-        # A pending moderation message needs to have field channel_ids empty. Moderators
-        # need to be able to notify a pending moderation message (in a channel they moderate).
-        cls.msg_c1_admin1 = cls._add_messages(cls.channel_1, 'Body11', author=cls.partner_admin, moderation_status='pending_moderation')
-        cls.msg_c1_admin2 = cls._add_messages(cls.channel_1, 'Body12', author=cls.partner_admin, moderation_status='pending_moderation')
-        cls.msg_c1_portal = cls._add_messages(cls.channel_1, 'Body21', author=cls.partner_portal, moderation_status='pending_moderation')
-
-    @mute_logger('odoo.models.unlink')
-    def test_moderate_accept(self):
-        self._reset_bus()
-        self.assertFalse(self.msg_c1_admin1.channel_ids | self.msg_c1_admin2.channel_ids | self.msg_c1_portal.channel_ids)
-
-        self.msg_c1_admin1.with_user(self.user_employee)._moderate('accept')
-        self.assertEqual(self.msg_c1_admin1.channel_ids, self.channel_1)
-        self.assertEqual(self.msg_c1_admin1.moderation_status, 'accepted')
-        self.assertEqual(self.msg_c1_admin2.moderation_status, 'pending_moderation')
-        self.assertBusNotifications([(self.cr.dbname, 'mail.channel', self.channel_1.id)])
-
-    @mute_logger('odoo.models.unlink')
-    def test_moderate_allow(self):
-        self._reset_bus()
-
-        self.msg_c1_admin1.with_user(self.user_employee)._moderate('allow')
-        self.assertEqual(self.msg_c1_admin1.channel_ids, self.channel_1)
-        self.assertEqual(self.msg_c1_admin2.channel_ids, self.channel_1)
-        self.assertEqual(self.msg_c1_admin1.moderation_status, 'accepted')
-        self.assertEqual(self.msg_c1_admin2.moderation_status, 'accepted')
-        self.assertBusNotifications([
-            (self.cr.dbname, 'mail.channel', self.channel_1.id),
-            (self.cr.dbname, 'mail.channel', self.channel_1.id)])
-
-    @mute_logger('odoo.models.unlink')
-    def test_moderate_reject(self):
-        with self.mock_mail_gateway():
-            (self.msg_c1_admin1 | self.msg_c1_portal).with_user(self.user_employee)._moderate_send_reject_email('Title', 'Message to author')
-            self.assertEqual(len(self._new_mails), 2)
-        for mail in self._new_mails:
-            self.assertEqual(mail.author_id, self.partner_employee)
-            self.assertEqual(mail.subject, 'Title')
-            self.assertEqual(mail.state, 'outgoing')
-        self.assertEqual(
-            set(self._new_mails.mapped('email_to')),
-            set([self.msg_c1_admin1.email_from, self.msg_c1_portal.email_from])
-        )
-        self.assertEqual(
-            set(self._new_mails.mapped('body_html')),
-            set(['<div>Message to author</div>\n%s\n' % self.msg_c1_admin1.body, '<div>Message to author</div>\n%s\n' % self.msg_c1_portal.body])
-        )  # TDE note: \n are added by append content to html, because why not
-
-    @mute_logger('odoo.models.unlink')
-    def test_moderate_discard(self):
-        self._reset_bus()
-        id1, id2, id3 = self.msg_c1_admin1.id, self.msg_c1_admin2.id, self.msg_c1_portal.id  # save ids because unlink will discard them
-        (self.msg_c1_admin1 | self.msg_c1_admin2 | self.msg_c1_portal).with_user(self.user_employee)._moderate_discard()
-
-        self.assertBusNotifications(
-            [(self.cr.dbname, 'res.partner', self.partner_admin.id),
-             (self.cr.dbname, 'res.partner', self.partner_employee.id),
-             (self.cr.dbname, 'res.partner', self.partner_portal.id)],
-            [{'type': 'deletion', 'message_ids': [id1, id2]},  # author of 2 messages
-             {'type': 'deletion', 'message_ids': [id1, id2, id3]},  # moderator
-             {'type': 'deletion', 'message_ids': [id3]}]  # author of 1 message
-        )
-
-    @mute_logger('odoo.models.unlink')
-    def test_notify_moderators(self):
-        # create pending messages in another channel to have two notification to push
-        channel_2 = self.env['mail.channel'].create({
-            'name': 'Moderation_1',
-            'email_send': True,
-            'moderation': True
-        })
-        self.user_admin.write({'moderation_channel_ids': [(6, 0, [channel_2.id])]})
-        self.msg_c2_portal = self._add_messages(channel_2, 'Body31', author=self.partner_portal, moderation_status='pending_moderation')
-
-        # one notification for each moderator: employee (channel1), admin (channel2)
-        with self.assertPostNotifications([{
-            'content': 'Hello %s' % self.partner_employee.name,
-            'message_type': 'user_notification', 'subtype': 'mail.mt_note',
-            'notif': [{
-                'partner': self.partner_employee,
-                'type': 'inbox'}]
-        }, {
-            'content': 'Hello %s' % self.partner_admin.name,
-            'message_type': 'user_notification', 'subtype': 'mail.mt_note',
-            'notif': [{
-                'partner': self.partner_admin,
-                'type': 'inbox'}]
-        }]):
-            self.env['mail.message']._notify_moderators()
