@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import mailing_mailing
 from . import mailing_models
+from . import mailing_models_cornercase
