@@ -4,9 +4,15 @@
 from . import test_controller_args
 from . import test_custom_snippet
 from . import test_error
+from . import test_fuzzy
+from . import test_image_upload_progress
 from . import test_is_multilang
+from . import test_media
 from . import test_multi_company
+from . import test_page
 from . import test_performance
+from . import test_redirect
 from . import test_reset_views
 from . import test_session
+from . import test_settings
 from . import test_views_during_module_operation
