@@ -4,39 +4,39 @@ odoo.define('web_editor.test_utils', function (require) {
 var ajax = require('web.ajax');
 var MockServer = require('web.MockServer');
 var testUtils = require('web.test_utils');
+var OdooEditorLib = require('@web_editor/js/editor/odoo-editor/src/OdooEditor');
 var Widget = require('web.Widget');
 var Wysiwyg = require('web_editor.wysiwyg');
 var options = require('web_editor.snippets.options');
 
 const COLOR_PICKER_TEMPLATE = `
-    <t t-name="web_editor.colorpicker">
-        <colorpicker>
-            <div class="o_colorpicker_section" data-name="theme" data-display="Theme Colors" data-icon-class="fa fa-flask">
-                <button data-color="o-color-1"/>
-                <button data-color="o-color-2"/>
-                <button data-color="o-color-3"/>
-                <button data-color="o-color-4"/>
-                <button data-color="o-color-5"/>
-            </div>
-            <div class="o_colorpicker_section" data-name="transparent_grayscale" data-display="Transparent Colors" data-icon-class="fa fa-eye-slash">
-                <button class="o_btn_transparent"/>
-                <button data-color="black-25"/>
-                <button data-color="black-50"/>
-                <button data-color="black-75"/>
-                <button data-color="white-25"/>
-                <button data-color="white-50"/>
-                <button data-color="white-75"/>
-            </div>
-            <div class="o_colorpicker_section" data-name="common" data-display="Common Colors" data-icon-class="fa fa-paint-brush"/>
-        </colorpicker>
-    </t>`;
+    <colorpicker>
+        <div class="o_colorpicker_section" data-name="theme" data-display="Theme Colors" data-icon-class="fa fa-flask">
+            <button data-color="o-color-1"/>
+            <button data-color="o-color-2"/>
+            <button data-color="o-color-3"/>
+            <button data-color="o-color-4"/>
+            <button data-color="o-color-5"/>
+        </div>
+        <div class="o_colorpicker_section" data-name="transparent_grayscale" data-display="Transparent Colors" data-icon-class="fa fa-eye-slash">
+            <button class="o_btn_transparent"/>
+            <button data-color="black-25"/>
+            <button data-color="black-50"/>
+            <button data-color="black-75"/>
+            <button data-color="white-25"/>
+            <button data-color="white-50"/>
+            <button data-color="white-75"/>
+        </div>
+        <div class="o_colorpicker_section" data-name="common" data-display="Common Colors" data-icon-class="fa fa-paint-brush"/>
+    </colorpicker>
+`;
 const SNIPPETS_TEMPLATE = `
     <h2 id="snippets_menu">Add blocks</h2>
     <div id="o_scroll">
         <div id="snippet_structure" class="o_panel">
             <div class="o_panel_header">First Panel</div>
             <div class="o_panel_body">
-                <div name="Separator" data-oe-type="snippet" data-oe-thumbnail="/website/static/src/img/snippets_thumbs/s_separator.png">
+                <div name="Separator" data-oe-type="snippet" data-oe-thumbnail="/web_editor/static/src/img/snippets_thumbs/s_hr.svg">
                     <div class="s_hr pt32 pb32">
                         <hr class="s_hr_1px s_hr_solid w-100 mx-auto"/>
                     </div>
@@ -92,11 +92,11 @@ MockServer.include({
      * @returns {Promise}
      */
     async _performRpc(route, args) {
-        if (args.model === "ir.ui.view") {
-            if (args.method === 'read_template' && args.args[0] === "web_editor.colorpicker") {
+        if (args.model === "ir.ui.view" && args.method === 'render_public_asset') {
+            if (args.args[0] === "web_editor.colorpicker") {
                 return COLOR_PICKER_TEMPLATE;
             }
-            if (args.method === 'render_public_asset' && args.args[0] === "web_editor.snippets") {
+            if (args.args[0] === "web_editor.snippets") {
                 return SNIPPETS_TEMPLATE;
             }
         }
@@ -192,12 +192,10 @@ function wysiwygData(data) {
                 },
             },
             records: [],
-            read_template(args) {
+            render_template(args) {
                 if (args[0] === 'web_editor.colorpicker') {
                     return COLOR_PICKER_TEMPLATE;
                 }
-            },
-            render_template(args) {
                 if (args[0] === 'web_editor.snippets') {
                     return SNIPPETS_TEMPLATE;
                 }
@@ -335,7 +333,6 @@ async function createWysiwyg(params) {
 /**
  * Char codes.
  */
-var dom = $.summernote.dom;
 var keyboardMap = {
     "8": "BACKSPACE",
     "9": "TAB",
@@ -405,7 +402,6 @@ var testKeyboard = function ($editable, assert, keyboardTests, addTests) {
         } else {
             keypress.key = keyboardMap[keypress.keyCode] || String.fromCharCode(keypress.keyCode);
         }
-        keypress.keyCode = keypress.keyCode;
         var event = $.Event("keydown", keypress);
         $target.trigger(event);
 
@@ -449,6 +445,24 @@ var testKeyboard = function ($editable, assert, keyboardTests, addTests) {
         $(target.tagName ? target : target.parentNode).trigger('mouseup');
     }
 
+    function nextPoint(point) {
+        var node, offset;
+        if (OdooEditorLib.nodeSize(point.node) === point.offset) {
+            node = point.node.parentNode;
+            offset = OdooEditorLib.childNodeIndex(point.node) + 1;
+        } else if (point.node.hasChildNodes()) {
+            node = point.node.childNodes[point.offset];
+            offset = 0;
+        } else {
+            node = point.node;
+            offset = point.offset + 1;
+        }
+        return {
+            node: node,
+            offset: offset
+        };
+    }
+
     function endOfAreaBetweenTwoNodes(point) {
         // move the position because some browser make the caret on the end of the previous area after normalize
         if (
@@ -456,9 +470,9 @@ var testKeyboard = function ($editable, assert, keyboardTests, addTests) {
             point.offset === point.node.textContent.length &&
             !/\S|\u00A0/.test(point.node.textContent)
         ) {
-            point = dom.nextPoint(dom.nextPoint(point));
+            point = nextPoint(nextPoint(point));
             while (point.node.tagName && point.node.textContent.length) {
-                point = dom.nextPoint(point);
+                point = nextPoint(point);
             }
         }
         return point;
@@ -474,7 +488,7 @@ var testKeyboard = function ($editable, assert, keyboardTests, addTests) {
             var def = testUtils.makeTestPromise();
             if (step.start) {
                 selectText(step.start, step.end);
-                if (!Wysiwyg.getRange($editable[0])) {
+                if (!Wysiwyg.getRange()) {
                     throw 'Wrong range! \n' +
                         'Test: ' + test.name + '\n' +
                         'Selection: ' + step.start + '" to "' + step.end + '"\n' +
@@ -483,7 +497,7 @@ var testKeyboard = function ($editable, assert, keyboardTests, addTests) {
             }
             setTimeout(function () {
                 if (step.keyCode || step.key) {
-                    var target = Wysiwyg.getRange($editable[0]).ec;
+                    var target = Wysiwyg.getRange().ec;
                     if (window.location.search.indexOf('notrycatch') !== -1) {
                         keydown(target, {
                             key: step.key,
@@ -559,14 +573,14 @@ var testKeyboard = function ($editable, assert, keyboardTests, addTests) {
             // test carret position
             if (test.test.start) {
                 var start = _select(test.test.start);
-                var range = Wysiwyg.getRange($editable[0]);
+                var range = Wysiwyg.getRange();
                 if ((range.sc !== range.ec || range.so !== range.eo) && !test.test.end) {
                     assert.ok(false, test.name + ": the carret is not colapsed and the 'end' selector in test is missing");
                     return;
                 }
                 var end = test.test.end ? _select(test.test.end) : start;
                 if (start.node && end.node) {
-                    range = Wysiwyg.getRange($editable[0]);
+                    range = Wysiwyg.getRange();
                     var startPoint = endOfAreaBetweenTwoNodes({
                         node: range.sc,
                         offset: range.so,
@@ -660,7 +674,7 @@ var keydown = function (key, $editable, options) {
         keyPress.key = keyboardMap[key] || String.fromCharCode(key);
         keyPress.keyCode = key;
     }
-    var range = Wysiwyg.getRange($editable[0]);
+    var range = Wysiwyg.getRange();
     if (!range) {
         console.error("Editor have not any range");
         return;
@@ -708,6 +722,164 @@ var textInput = function (target, char) {
     }
 };
 
+//--------------------------------------------------------------------------
+// Convert Inline
+//--------------------------------------------------------------------------
+
+const tableAttributes = {
+    cellspacing: 0,
+    cellpadding: 0,
+    border: 0,
+    width: '100%',
+    align: 'center',
+    role: 'presentation',
+};
+const tableAttributesString = Object.keys(tableAttributes).map(key => `${key}="${tableAttributes[key]}"`).join(' ');
+const tableStyles = {
+    'border-collapse': 'collapse',
+    'text-align': 'inherit',
+    'font-size': 'unset',
+    'line-height': 'unset',
+};
+const tableStylesString = Object.keys(tableStyles).map(key => `${key}: ${tableStyles[key]};`).join(' ');
+/**
+ * Take a matrix representing a grid and return an HTML string of the Bootstrap
+ * grid. The matrix is an array of rows, with each row being an array of cells.
+ * Each cell can be represented either by a 0 < number < 13 (col-#) or a falsy
+ * value (col). Each cell has its coordinates `(row index, column index)` as
+ * text content.
+ * Eg: [                        // <div class="container">
+ *      [                       //     <div class="row">
+ *          1,                  //         <div class="col-1">(0, 0)</div>
+ *          11,                 //         <div class="col-11">(0, 1)</div>
+ *      ],                      //     </div>
+ *      [                       //     <div class="row">
+ *          false,              //         <div class="col">(1, 0)</div>
+ *      ],                      //     </div>
+ * ]                            // </div>
+ *
+ * @param {Array<Array<Number|null>>} matrix
+ * @returns {string}
+ */
+function getGridHtml(matrix) {
+    return (
+        `<div class="container">` +
+        matrix.map((row, iRow) => (
+            `<div class="row">` +
+            row.map((col, iCol) => (
+                `<div class="${col ? 'col-' + col : 'col'}">(${iRow}, ${iCol})</div>`
+            )).join('') +
+            `</div>`
+        )).join('') +
+        `</div>`
+    );
+}
+function getTdHtml(colspan, text, containerWidth) {
+    return (
+        `<td colspan="${colspan}"${
+            containerWidth ? ' ' + `style="max-width: ${Math.round(containerWidth*colspan/12*100)/100}px;"`
+                           : ''}>` +
+            text +
+        `</td>`
+    );
+}
+/**
+ * Take a matrix representing a table and return an HTML string of the table.
+ * The matrix is an array of rows, with each row being an array of cells. Each
+ * cell is represented by a tuple of numbers [colspan, width (in percent)]. A
+ * cell can have a string as third value to represent its text content. The
+ * default text content of each cell is its coordinates `(row index, column
+ * index)`. If the cell has a number as third value, it will be used as the
+ * max-width of the cell (in pixels).
+ * Eg: [                        // <table> (note: extra attrs and styles apply)
+ *      [                       //   <tr>
+ *          [1, 8],             //     <td colspan="1" width="8%">(0, 0)</td>
+ *          [11, 92]            //     <td colspan="11" width="92%">(0, 1)</td>
+ *      ],                      //   </tr>
+ *      [                       //   <tr>
+ *          [2, 17, 'A'],       //     <td colspan="2" width="17%">A</td>
+ *          [10, 83],           //     <td colspan="10" width="83%">(1, 1)</td>
+ *      ],                      //   </tr>
+ * ]                            // </table>
+ *
+ * @param {Array<Array<Array<[Number, Number, string?, number?]>>>} matrix
+ * @param {Number} [containerWidth]
+ * @returns {string}
+ */
+function getTableHtml(matrix, containerWidth) {
+    return (
+        `<table ${tableAttributesString} style="width: 100% !important; ${tableStylesString}">` +
+        matrix.map((row, iRow) => (
+            `<tr>` +
+            row.map((col, iCol) => (
+                getTdHtml(col[0], typeof col[2] === 'string' ? col[2] : `(${iRow}, ${iCol})`, containerWidth)
+            )).join('') +
+            `</tr>`
+        )).join('') +
+        `</table>`
+    );
+}
+/**
+ * Take a number of rows and a number of columns (or number of columns per
+ * individual row) and return an HTML string of the corresponding grid. Every
+ * column is a regular Bootstrap "col" (no col-#).
+ * Eg: [2, 3] <=> getGridHtml([[false, false, false], [false, false, false]])
+ * Eg: [2, [2, 1]] <=> getGridHtml([[false, false], [false]])
+ *
+ * @see getGridHtml
+ * @param {Number} nRows
+ * @param {Number|Number[]} nCols
+ * @returns {string}
+ */
+function getRegularGridHtml(nRows, nCols) {
+    const matrix = new Array(nRows).fill().map((_, iRow) => (
+        new Array(Array.isArray(nCols) ? nCols[iRow] : nCols).fill()
+    ));
+    return getGridHtml(matrix);
+};
+/**
+ * Take a number of rows, a number of columns (or number of columns per
+ * individual row), a colspan (or colspan per individual row) and a width (or
+ * width per individual row, in percent), and return an HTML string of the
+ * corresponding table. Every cell in a row has the same colspan/width.
+ * Eg: [2, 2, 6, 50] <=> getTableHtml([[[6, 50], [6, 50]], [[6, 50], [6, 50]]])
+ * Eg: [2, [2, 1], [6, 12], [50, 100]] <=> getTableHtml([[[6, 50], [6, 50]], [[12, 100]]])
+ *
+ * @see getTableHtml
+ * @param {Number} nRows
+ * @param {Number|Number[]} nCols
+ * @param {Number|Number[]} colspan
+ * @param {Number|Number[]} width
+ * @param {Number} containerWidth
+ * @returns {string}
+ */
+function getRegularTableHtml(nRows, nCols, colspan, width, containerWidth) {
+    const matrix = new Array(nRows).fill().map((_, iRow) => (
+        new Array(Array.isArray(nCols) ? nCols[iRow] : nCols).fill().map(() => ([
+            Array.isArray(colspan) ? colspan[iRow] : colspan,
+            Array.isArray(width) ? width[iRow] : width,
+        ])))
+    );
+    return getTableHtml(matrix, containerWidth);
+}
+/**
+ * Take an HTML string and returns that string stripped from any HTML comments.
+ * By default, also removes the mso-hide class which is only there for outlook
+ * to hide elements when we use mso conditional comments.
+ *
+ * @param {string} html
+ * @param {boolean} [removeMsoHide=true]
+ * @returns {string}
+ */
+function removeComments(html, removeMsoHide=true) {
+    const cleanHtml = html.replace(/<!--(.*?)-->/g, '');
+    if (removeMsoHide) {
+        return cleanHtml.replaceAll(' class="mso-hide"', '').replace(/\s*mso-hide/g, '').replace(/mso-hide\s*/g, '');
+    } else {
+        return cleanHtml;
+    }
+}
+
 return {
     wysiwygData: wysiwygData,
     createWysiwyg: createWysiwyg,
@@ -716,6 +888,12 @@ return {
     keydown: keydown,
     patch: patch,
     unpatch: unpatch,
+    getGridHtml: getGridHtml,
+    getTableHtml: getTableHtml,
+    getRegularGridHtml: getRegularGridHtml,
+    getRegularTableHtml: getRegularTableHtml,
+    getTdHtml: getTdHtml,
+    removeComments: removeComments,
 };
 
 
