@@ -7,13 +7,15 @@ var local_storage = require('web.local_storage');
 var mixins = require('web.mixins');
 var utils = require('web_tour.utils');
 var TourStepUtils = require('web_tour.TourStepUtils');
-var RainbowMan = require('web.RainbowMan');
 var RunningTourActionHelper = require('web_tour.RunningTourActionHelper');
 var ServicesMixin = require('web.ServicesMixin');
 var session = require('web.session');
 var Tip = require('web_tour.Tip');
+const {Markup} = require('web.utils');
+const { config: transitionConfig } = require("@web/core/transition");
 
 var _t = core._t;
+const { markup } = require("@odoo/owl");
 
 var RUNNING_TOUR_TIMEOUT = 10000;
 
@@ -26,7 +28,7 @@ var do_before_unload = utils.do_before_unload;
 var get_jquery_element_from_selector = utils.get_jquery_element_from_selector;
 
 return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
-    init: function(parent, consumed_tours) {
+    init: function(parent, consumed_tours, disabled = false) {
         mixins.EventDispatcherMixin.init.call(this);
         this.setParent(parent);
 
@@ -37,7 +39,14 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
         this.consumed_tours = (consumed_tours || []).filter(tourName => {
             return !local_storage.getItem(get_debugging_key(tourName));
         });
+        this.disabled = disabled;
         this.running_tour = local_storage.getItem(get_running_key());
+        if (this.running_tour) {
+            // Transitions can cause DOM mutations which will cause the tour_service
+            // MutationObserver to wait longer before proceeding to the next step
+            // this can slow down tours
+            transitionConfig.disabled = true;
+        }
         this.running_step_delay = parseInt(local_storage.getItem(get_running_delay_key()), 10) || 0;
         this.edition = (_.last(session.server_version_info) === 'e') ? 'enterprise' : 'community';
         this._log = [];
@@ -55,6 +64,10 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
      *        the url to load when manually running the tour
      * @param {boolean} [options.rainbowMan=true]
      *        whether or not the rainbowman must be shown at the end of the tour
+     * @param {string} [options.fadeout]
+     *        Delay for rainbowman to disappear. 'fast' will make rainbowman dissapear, quickly,
+     *        'medium', 'slow' and 'very_slow' will wait little longer before disappearing, no
+     *        will keep rainbowman on screen until user clicks anywhere outside rainbowman
      * @param {boolean} [options.sequence=1000]
      *        priority sequence of the tour (lowest is first, tours with the same
      *        sequence will be executed in a non deterministic order).
@@ -63,11 +76,10 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
      * @param {string|function} [options.rainbowManMessage]
               text or function returning the text displayed under the rainbowman
               at the end of the tour.
-     * @param {string} [options.rainbowManFadeout]
      * @param {Object[]} steps - steps' descriptions, each step being an object
      *                     containing a tip description
      */
-    register() {
+    register: function() {
         var args = Array.prototype.slice.call(arguments);
         var last_arg = args[args.length - 1];
         var name = args[0];
@@ -83,13 +95,13 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
             url: options.url,
             rainbowMan: options.rainbowMan === undefined ? true : !!options.rainbowMan,
             rainbowManMessage: options.rainbowManMessage,
-            rainbowManFadeout: options.rainbowManFadeout,
+            fadeout: options.fadeout || 'medium',
             sequence: options.sequence || 1000,
             test: options.test,
             wait_for: options.wait_for || Promise.resolve(),
         };
         if (options.skip_enabled) {
-            tour.skip_link = '<p><span class="o_skip_tour">' + _t('Skip tour') + '</span></p>';
+            tour.skip_link = Markup`<p><span class="o_skip_tour">${_t('Skip tour')}</span></p>`;
             tour.skip_handler = function (tip) {
                 this._deactivate_tip(tip);
                 this._consume_tour(name);
@@ -125,6 +137,11 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
         });
     },
     _register: function (do_update, tour, name) {
+        const debuggingTour = local_storage.getItem(get_debugging_key(name));
+        if (this.disabled && !this.running_tour && !debuggingTour) {
+            this.consumed_tours.push(name);
+        }
+
         if (tour.ready) return Promise.resolve();
 
         const tour_is_consumed = this._isTourConsumed(name);
@@ -143,7 +160,6 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
 
             tour.ready = true;
 
-            const debuggingTour = local_storage.getItem(get_debugging_key(name));
             if (debuggingTour ||
                 (do_update && (this.running_tour === name ||
                               (!this.running_tour && !tour.test && !tour_is_consumed)))) {
@@ -153,7 +169,7 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
     },
     /**
      * Resets the given tour to its initial step, and prevent it from being
-     * marked as consumed at reload, by the include in tour_disable.js
+     * marked as consumed at reload.
      *
      * @param {string} tourName
      */
@@ -173,7 +189,7 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
         // redirect to its starting point (or /web by default)
         window.location.href = window.location.origin + (tour.url || '/web');
     },
-    run: function (tour_name, step_delay) {
+    run: async function (tour_name, step_delay) {
         console.log(_.str.sprintf("Preparing tour %s", tour_name));
         if (this.running_tour) {
             this._deactivate_tip(this.active_tooltips[this.running_tour]);
@@ -222,17 +238,17 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
     update: function (tour_name) {
         if (this.paused) return;
 
-        this.$modal_displayed = $('.modal:visible').last();
-
         tour_name = this.running_tour || tour_name;
         if (tour_name) {
             var tour = this.tours[tour_name];
             if (!tour || !tour.ready) return;
 
+            let self = this;
+            self._check_for_skipping_step(self.active_tooltips[tour_name], tour_name);
+
             if (this.running_tour && this.running_tour_timeout === undefined) {
                 this._set_running_tour_timeout(this.running_tour, this.active_tooltips[this.running_tour]);
             }
-            var self = this;
             setTimeout(function () {
                 self._check_for_tooltip(self.active_tooltips[tour_name], tour_name);
             });
@@ -243,11 +259,29 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
             let visibleTip = false;
             for (const tourName of sortedTooltips) {
                 var tip = this.active_tooltips[tourName];
+                this._check_for_skipping_step(tip, tourName)
                 tip.hidden = visibleTip;
                 visibleTip = this._check_for_tooltip(tip, tourName) || visibleTip;
             }
         }
     },
+    /**
+     *  Check if the current step of a tour needs to be skipped. If so, skip the step and update
+     *
+     * @param {Object} step
+     * @param {string} tour_name
+     */
+    _check_for_skipping_step: function (step, tour_name) {
+        if (step && step.skip_trigger) {
+            let $skip_trigger;
+            $skip_trigger = get_jquery_element_from_selector(step.skip_trigger);
+            let skipping = get_first_visible_element($skip_trigger).length;
+            if (skipping) {
+                this._to_next_step(tour_name);
+                this.update(tour_name);
+            }
+        }
+    },
     /**
      *  Check (and activate or update) a help tooltip for a tour.
      *
@@ -264,6 +298,7 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
             this._log.push("blockUI is preventing the tip to be consumed");
             return false;
         }
+        this.$modal_displayed = $('.modal:visible').last();
 
         var $trigger;
         if (tip.in_modal !== false && this.$modal_displayed.length) {
@@ -346,7 +381,7 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
         var tip_info = tip;
         if (tour.skip_link) {
             tip_info = _.extend(_.omit(tip_info, 'content'), {
-                content: tip.content + tour.skip_link,
+                content: Markup`${tip.content}${tour.skip_link}`,
                 event_handlers: [{
                     event: 'click',
                     selector: '.o_skip_tour',
@@ -422,12 +457,14 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
             if (message) {
                 message = typeof message === 'function' ? message() : message;
             } else {
-                message = _t('<strong><b>Good job!</b> You went through all steps of this tour.</strong>');
+                message = markup(_t('<strong><b>Good job!</b> You went through all steps of this tour.</strong>'));
             }
-            new RainbowMan({
-                message: message,
-                fadeout: this.tours[tour_name].rainbowManFadeout || 'medium',
-            }).appendTo(this.$body);
+            const fadeout = this.tours[tour_name].fadeout;
+            core.bus.trigger('show-effect', {
+                type: "rainbow_man",
+                message,
+                fadeout,
+            });
         }
         this.tours[tour_name].current_step = 0;
         local_storage.removeItem(get_step_key(tour_name));
@@ -442,7 +479,17 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
                 _.each(this._log, function (log) {
                     console.log(log);
                 });
-                console.log(document.body.parentElement.outerHTML);
+                let documentHTML = document.documentElement.outerHTML;
+                // Replace empty iframe tags with their content
+                const iframeEls = Array.from(document.body.querySelectorAll('iframe.o_iframe'));
+                if (iframeEls.length) {
+                    const matches = documentHTML.match(/<iframe[^>]+class=["'][^"']*\bo_iframe\b[^>]+><\/iframe>/g);
+                    for (let i = 0; i < matches.length; i++) {
+                        const iframeWithContent = matches[i].replace(/></, `>${iframeEls[i].contentDocument.documentElement.outerHTML}<`);
+                        documentHTML = documentHTML.replace(matches[i], `\n${iframeWithContent}\n`);
+                    }
+                }
+                console.log(documentHTML);
                 console.error(error); // will be displayed as error info
             } else {
                 console.log(_.str.sprintf("Tour %s succeeded", tour_name));
@@ -489,15 +536,34 @@ return core.Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
         }
 
         function exec() {
+            const anchorIsInDocument = tip.widget.$anchor[0].ownerDocument.contains(tip.widget.$anchor[0]);
+            const uiIsBlocked = $('body').hasClass('o_ui_blocked');
+            if (!anchorIsInDocument || uiIsBlocked) {
+                // trigger is no longer in the DOM, or UI is now blocked, so run the same step again
+                self._deactivate_tip(self.active_tooltips[tour_name]);
+                self._to_next_step(tour_name, 0);
+                self.update();
+                return;
+            }
             var action_helper = new RunningTourActionHelper(tip.widget);
             do_before_unload(self._consume_tip.bind(self, tip, tour_name));
 
             var tour = self.tours[tour_name];
             if (typeof tip.run === "function") {
-                tip.run.call(tip.widget, action_helper);
+                try {
+                    tip.run.call(tip.widget, action_helper);
+                } catch (e) {
+                    console.error(`Tour ${tour_name} failed at step ${self._describeTip(tip)}: ${e.message}`);
+                    throw e;
+                }
             } else if (tip.run !== undefined) {
                 var m = tip.run.match(/^([a-zA-Z0-9_]+) *(?:\(? *(.+?) *\)?)?$/);
-                action_helper[m[1]](m[2]);
+                try {
+                    action_helper[m[1]](m[2]);
+                } catch (e) {
+                    console.error(`Tour ${tour_name} failed at step ${self._describeTip(tip)}: ${e.message}`);
+                    throw e;
+                }
             } else if (tour.current_step === tour.steps.length - 1) {
                 console.log('Tour %s: ignoring action (auto) of last step', tour_name);
             } else {
