@@ -9,7 +9,17 @@
     'depends': ['base_setup', 'web_editor'],
     'data': [
         'views/res_config_settings_view.xml',
-        'views/web_unsplash_templates.xml',
-    ],
+        ],
     'auto_install': True,
+    'assets': {
+        'web.assets_frontend': [
+            'web_unsplash/static/src/js/unsplash_beacon.js',
+        ],
+        'web_editor.assets_media_dialog': [
+            'web_unsplash/static/src/components/media_dialog/*.js',
+            'web_unsplash/static/src/components/media_dialog/*.xml',
+            'web_unsplash/static/src/services/unsplash_service.js',
+        ],
+    },
+    'license': 'LGPL-3',
 }
