@@ -2,4 +2,6 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import backend
+from . import form
 from . import main
+from . import webclient
