@@ -5,25 +5,30 @@ import datetime
 import json
 import os
 import logging
-import pytz
+import re
 import requests
 import werkzeug.urls
 import werkzeug.utils
 import werkzeug.wrappers
 
 from itertools import islice
-from werkzeug import urls
+from lxml import etree
+from textwrap import shorten
+from werkzeug.exceptions import NotFound
 from xml.etree import ElementTree as ET
 
 import odoo
 
 from odoo import http, models, fields, _
-from odoo.http import request
-from odoo.tools import OrderedSet
+from odoo.exceptions import AccessError
+from odoo.http import request, SessionExpiredException
+from odoo.osv import expression
+from odoo.tools import OrderedSet, escape_psql, html_escape as escape
 from odoo.addons.http_routing.models.ir_http import slug, slugify, _guess_mimetype
-from odoo.addons.web.controllers.main import Binary
 from odoo.addons.portal.controllers.portal import pager as portal_pager
 from odoo.addons.portal.controllers.web import Home
+from odoo.addons.web.controllers.binary import Binary
+from odoo.addons.website.tools import get_base_domain
 
 logger = logging.getLogger(__name__)
 
@@ -69,20 +74,47 @@ class Website(Home):
 
     @http.route('/', type='http', auth="public", website=True, sitemap=True)
     def index(self, **kw):
+        """ The goal of this controller is to make sure we don't serve a 404 as
+        the website homepage. As this is the website entry point, serving a 404
+        is terrible.
+        There is multiple fallback mechanism to prevent that:
+        - If homepage URL is set (empty by default), serve the website.page
+        matching it
+        - If homepage URL is set (empty by default), serve the controller
+        matching it
+        - If homepage URL is not set, serve the `/` website.page
+        - Serve the first accessible menu as last resort. It should be relevant
+        content, at least better than a 404
+        - Serve 404
+        Most DBs will just have a website.page with '/' as URL and keep the
+        homepage_url setting empty.
+        """
         # prefetch all menus (it will prefetch website.page too)
         top_menu = request.website.menu_id
 
-        homepage = request.website.homepage_id
-        if homepage and (homepage.sudo().is_visible or request.env.user.has_group('base.group_user')) and homepage.url != '/':
-            return request.env['ir.http'].reroute(homepage.url)
+        homepage_url = request.website._get_cached('homepage_url')
+        if homepage_url and homepage_url != '/':
+            request.env['ir.http'].reroute(homepage_url)
 
+        # Check for page
         website_page = request.env['ir.http']._serve_page()
         if website_page:
             return website_page
-        else:
-            first_menu = top_menu and top_menu.child_id and top_menu.child_id.filtered(lambda menu: menu.is_visible)
-            if first_menu and first_menu[0].url not in ('/', '', '#') and (not (first_menu[0].url.startswith(('/?', '/#', ' ')))):
-                return request.redirect(first_menu[0].url)
+
+        # Check for controller
+        if homepage_url and homepage_url != '/':
+            try:
+                return request._serve_ir_http()
+            except (AccessError, NotFound, SessionExpiredException):
+                pass
+
+        # Fallback on first accessible menu
+        def is_reachable(menu):
+            return menu.is_visible and menu.url not in ('/', '', '#') and not menu.url.startswith(('/?', '/#', ' '))
+
+        reachable_menus = top_menu.child_id.filtered(is_reachable)
+        if reachable_menus:
+            return request.redirect(reachable_menus[0].url)
 
         raise request.not_found()
 
@@ -92,11 +124,8 @@ class Website(Home):
         session, AFTER landing on that website domain (if set) as this will be a
         different session.
         """
-        parse = werkzeug.urls.url_parse
-        safe_path = parse(path).path
-
         if not (request.env.user.has_group('website.group_multi_website')
-           and request.env.user.has_group('website.group_website_publisher')):
+           and request.env.user.has_group('website.group_website_restricted_editor')):
             # The user might not be logged in on the forced website, so he won't
             # have rights. We just redirect to the path as the user is already
             # on the domain (basically a no-op as it won't change domain or
@@ -104,19 +133,36 @@ class Website(Home):
             # Website 1 : 127.0.0.1 (admin)
             # Website 2 : 127.0.0.2 (not logged in)
             # Click on "Website 2" from Website 1
-            return request.redirect(safe_path)
+            return request.redirect(path)
 
         website = request.env['website'].browse(website_id)
 
         if not isredir and website.domain:
             domain_from = request.httprequest.environ.get('HTTP_HOST', '')
-            domain_to = parse(website._get_http_domain()).netloc
+            domain_to = get_base_domain(website.domain)
             if domain_from != domain_to:
                 # redirect to correct domain for a correct routing map
-                url_to = urls.url_join(website._get_http_domain(), '/website/force/%s?isredir=1&path=%s' % (website.id, safe_path))
+                url_to = werkzeug.urls.url_join(website.domain, '/website/force/%s?isredir=1&path=%s' % (website.id, path))
                 return request.redirect(url_to)
         website._force()
-        return request.redirect(safe_path)
+        return request.redirect(path)
+
+    @http.route(['/@/', '/@/<path:path>'], type='http', auth='public', website=True, sitemap=False, multilang=False)
+    def client_action_redirect(self, path='', **kw):
+        """ Redirect internal users to the backend preview of the requested path
+        URL (client action iframe).
+        Non internal users will be redirected to the regular frontend version of
+        that URL.
+        """
+        path = '/' + path
+        mode_edit = bool(kw.pop('enable_editor', False))
+        if kw:
+            path += '?' + werkzeug.urls.url_encode(kw)
+
+        if request.env.user._is_internal():
+            path = request.website.get_client_action_url(path, mode_edit)
+
+        return request.redirect(path)
 
     # ------------------------------------------------------
     # Login - overwrite of the web login so that regular users are redirected to the backend
@@ -128,8 +174,8 @@ class Website(Home):
         the frontend
         """
         if not redirect and request.params.get('login_success'):
-            if request.env['res.users'].browse(uid).has_group('base.group_user'):
-                redirect = b'/web?' + request.httprequest.query_string
+            if request.env['res.users'].browse(uid)._is_internal():
+                redirect = '/web?' + request.httprequest.query_string.decode()
             else:
                 redirect = '/my'
         return super()._login_redirect(uid, redirect=redirect)
@@ -150,13 +196,15 @@ class Website(Home):
     @http.route('/website/lang/<lang>', type='http', auth="public", website=True, multilang=False)
     def change_lang(self, lang, r='/', **kwargs):
         """ :param lang: supposed to be value of `url_code` field """
-        r = request.website._get_relative_url(r)
         if lang == 'default':
             lang = request.website.default_lang_id.url_code
             r = '/%s%s' % (lang, r or '/')
-        redirect = werkzeug.utils.redirect(r or ('/%s' % lang), 303)
         lang_code = request.env['res.lang']._lang_get_code(lang)
-        redirect.set_cookie('frontend_lang', lang_code)
+        # replace context with correct lang, to avoid that the url_for of request.redirect remove the
+        # default lang in case we switch from /fr -> /en with /en as default lang.
+        request.update_context(lang=lang_code)
+        redirect = request.redirect(r or ('/%s' % lang))
+        redirect.set_cookie('frontend_lang', lang_code, max_age=365 * 24 * 3600)
         return redirect
 
     @http.route(['/website/country_infos/<model("res.country"):country>'], type='json', auth="public", methods=['POST'], website=True)
@@ -178,7 +226,7 @@ class Website(Home):
 
         def create_sitemap(url, content):
             return Attachment.create({
-                'datas': base64.b64encode(content),
+                'raw': content.encode(),
                 'mimetype': mimetype,
                 'type': 'binary',
                 'name': url,
@@ -236,12 +284,20 @@ class Website(Home):
 
         return request.make_response(content, [('Content-Type', mimetype)])
 
-    @http.route('/website/info', type='http', auth="public", website=True, sitemap=True)
+    def sitemap_website_info(env, rule, qs):
+        website = env['website'].get_current_website()
+        if not (
+            website.viewref('website.website_info', False).active
+            and website.viewref('website.show_website_info', False).active
+        ):
+            # avoid 404 or blank page in sitemap
+            return False
+
+        if not qs or qs.lower() in '/website/info':
+            yield {'loc': '/website/info'}
+
+    @http.route('/website/info', type='http', auth="public", website=True, sitemap=sitemap_website_info)
     def website_info(self, **kwargs):
-        try:
-            request.website.get_template('website.website_info').name
-        except Exception as e:
-            return request.env['ir.http']._handle_exception(e)
         Module = request.env['ir.module.module'].sudo()
         apps = Module.search([('state', '=', 'installed'), ('application', '=', True)])
         l10n = Module.search([('state', '=', 'installed'), ('name', '=like', 'l10n_%')])
@@ -252,12 +308,25 @@ class Website(Home):
         }
         return request.render('website.website_info', values)
 
+    @http.route(['/website/configurator', '/website/configurator/<int:step>'], type='http', auth="user", website=True, multilang=False)
+    def website_configurator(self, step=1, **kwargs):
+        if not request.env.user.has_group('website.group_website_designer'):
+            raise werkzeug.exceptions.NotFound()
+        if request.website.configurator_done:
+            return request.redirect('/')
+        if request.env.lang != request.website.default_lang_id.code:
+            return request.redirect('/%s%s' % (request.website.default_lang_id.url_code, request.httprequest.path))
+        action_url = '/web#action=website.website_configurator&menu_id=%s' % request.env.ref('website.menu_website_configuration').id
+        if step > 1:
+            action_url += '&step=' + str(step)
+        return request.redirect(action_url)
+
     @http.route(['/website/social/<string:social>'], type='http', auth="public", website=True, sitemap=False)
     def social(self, social, **kwargs):
         url = getattr(request.website, 'social_%s' % social, False)
         if not url:
             raise werkzeug.exceptions.NotFound()
-        return request.redirect(url)
+        return request.redirect(url, local=False)
 
     @http.route('/website/get_suggested_links', type='json', auth="user", website=True)
     def get_suggested_link(self, needle, limit=10):
@@ -284,7 +353,7 @@ class Website(Home):
         for name, url, mod in current_website.get_suggested_controllers():
             if needle.lower() in name.lower() or needle.lower() in url.lower():
                 module_sudo = mod and request.env.ref('base.module_%s' % mod, False).sudo()
-                icon = mod and "<img src='%s' width='24px' class='mr-2 rounded' /> " % (module_sudo and module_sudo.icon or mod) or ''
+                icon = mod and "<img src='%s' width='24px' height='24px' class='mr-2 rounded' /> " % (module_sudo and module_sudo.icon or mod) or ''
                 suggested_controllers.append({
                     'value': url,
                     'label': '%s%s (%s)' % (icon, url, name),
@@ -299,58 +368,168 @@ class Website(Home):
         }
 
     @http.route('/website/snippet/filters', type='json', auth='public', website=True)
-    def get_dynamic_filter(self, filter_id, template_key, limit=None, search_domain=None):
+    def get_dynamic_filter(self, filter_id, template_key, limit=None, search_domain=None, with_sample=False):
         dynamic_filter = request.env['website.snippet.filter'].sudo().search(
             [('id', '=', filter_id)] + request.website.website_domain()
         )
-        return dynamic_filter and dynamic_filter.render(template_key, limit, search_domain) or ''
+        return dynamic_filter and dynamic_filter._render(template_key, limit, search_domain, with_sample) or []
 
     @http.route('/website/snippet/options_filters', type='json', auth='user', website=True)
-    def get_dynamic_snippet_filters(self):
+    def get_dynamic_snippet_filters(self, model_name=None, search_domain=None):
+        domain = request.website.website_domain()
+        if search_domain:
+            domain = expression.AND([domain, search_domain])
+        if model_name:
+            domain = expression.AND([
+                domain,
+                ['|', ('filter_id.model_id', '=', model_name), ('action_server_id.model_id.model', '=', model_name)]
+            ])
         dynamic_filter = request.env['website.snippet.filter'].sudo().search_read(
-            request.website.website_domain(), ['id', 'name', 'limit']
+            domain, ['id', 'name', 'limit', 'model_name'], order='id asc'
         )
         return dynamic_filter
 
     @http.route('/website/snippet/filter_templates', type='json', auth='public', website=True)
-    def get_dynamic_snippet_templates(self, filter_id=False):
-        # todo: if filter_id.model -> filter template
-        templates = request.env['ir.ui.view'].sudo().search_read(
-            [['key', 'ilike', '.dynamic_filter_template_'], ['type', '=', 'qweb']], ['key', 'name']
-        )
+    def get_dynamic_snippet_templates(self, filter_name=False):
+        domain = [['key', 'ilike', '.dynamic_filter_template_'], ['type', '=', 'qweb']]
+        if filter_name:
+            domain.append(['key', 'ilike', escape_psql('_%s_' % filter_name)])
+        templates = request.env['ir.ui.view'].sudo().search_read(domain, ['key', 'name', 'arch_db'])
+
+        for t in templates:
+            children = etree.fromstring(t.pop('arch_db')).getchildren()
+            attribs = children and children[0].attrib or {}
+            t['numOfEl'] = attribs.get('data-number-of-elements')
+            t['numOfElSm'] = attribs.get('data-number-of-elements-sm')
+            t['numOfElFetch'] = attribs.get('data-number-of-elements-fetch')
+            t['rowPerSlide'] = attribs.get('data-row-per-slide')
+            t['arrowPosition'] = attribs.get('data-arrow-position')
+            t['extraClasses'] = attribs.get('data-extra-classes')
+            t['thumb'] = attribs.get('data-thumb')
         return templates
 
-    # ------------------------------------------------------
-    # Edit
-    # ------------------------------------------------------
-
-    @http.route(['/website/pages', '/website/pages/page/<int:page>'], type='http', auth="user", website=True)
-    def pages_management(self, page=1, sortby='url', search='', **kw):
-        # only website_designer should access the page Management
-        if not request.env.user.has_group('website.group_website_designer'):
-            raise werkzeug.exceptions.NotFound()
-
-        Page = request.env['website.page']
-        searchbar_sortings = {
-            'url': {'label': _('Sort by Url'), 'order': 'url'},
-            'name': {'label': _('Sort by Name'), 'order': 'name'},
+    @http.route('/website/get_current_currency', type='json', auth="public", website=True)
+    def get_current_currency(self, **kwargs):
+        return {
+            'id': request.website.company_id.currency_id.id,
+            'symbol': request.website.company_id.currency_id.symbol,
+            'position': request.website.company_id.currency_id.position,
         }
-        # default sortby order
-        sort_order = searchbar_sortings.get(sortby, 'url')['order'] + ', website_id desc, id'
 
-        domain = request.website.website_domain()
-        if search:
-            domain += ['|', ('name', 'ilike', search), ('url', 'ilike', search)]
+    # --------------------------------------------------------------------------
+    # Search Bar
+    # --------------------------------------------------------------------------
 
-        pages = Page.search(domain, order=sort_order)
-        if sortby != 'url' or not request.env.user.has_group('website.group_multi_website'):
-            pages = pages.filtered(pages._is_most_specific_page)
-        pages_count = len(pages)
+    def _get_search_order(self, order):
+        # OrderBy will be parsed in orm and so no direct sql injection
+        # id is added to be sure that order is a unique sort key
+        order = order or 'name ASC'
+        return 'is_published desc, %s, id desc' % order
 
+    @http.route('/website/snippet/autocomplete', type='json', auth='public', website=True)
+    def autocomplete(self, search_type=None, term=None, order=None, limit=5, max_nb_chars=999, options=None):
+        """
+        Returns list of results according to the term and options
+
+        :param str search_type: indicates what to search within, 'all' matches all available types
+        :param str term: search term written by the user
+        :param str order:
+        :param int limit: number of results to consider, defaults to 5
+        :param int max_nb_chars: max number of characters for text fields
+        :param dict options: options map containing
+            allowFuzzy: enables the fuzzy matching when truthy
+            fuzzy (boolean): True when called after finding a name through fuzzy matching
+
+        :returns: dict (or False if no result) containing
+            - 'results' (list): results (only their needed field values)
+                    note: the monetary fields will be strings properly formatted and
+                    already containing the currency
+            - 'results_count' (int): the number of results in the database
+                    that matched the search query
+            - 'parts' (dict): presence of fields across all results
+            - 'fuzzy_search': search term used instead of requested search
+        """
+        order = self._get_search_order(order)
+        options = options or {}
+        results_count, search_results, fuzzy_term = request.website._search_with_fuzzy(search_type, term, limit, order, options)
+        if not results_count:
+            return {
+                'results': [],
+                'results_count': 0,
+                'parts': {},
+            }
+        term = fuzzy_term or term
+        search_results = request.website._search_render_results(search_results, limit)
+
+        mappings = []
+        results_data = []
+        for search_result in search_results:
+            results_data += search_result['results_data']
+            mappings.append(search_result['mapping'])
+        if search_type == 'all':
+            # Only supported order for 'all' is on name
+            results_data.sort(key=lambda r: r.get('name', ''), reverse='name desc' in order)
+        results_data = results_data[:limit]
+        result = []
+        for record in results_data:
+            mapping = record['_mapping']
+            mapped = {
+                '_fa': record.get('_fa'),
+            }
+            for mapped_name, field_meta in mapping.items():
+                value = record.get(field_meta.get('name'))
+                if not value:
+                    mapped[mapped_name] = ''
+                    continue
+                field_type = field_meta.get('type')
+                if field_type == 'text':
+                    if value and field_meta.get('truncate', True):
+                        value = shorten(value, max_nb_chars, placeholder='...')
+                    if field_meta.get('match') and value and term:
+                        pattern = '|'.join(map(re.escape, term.split()))
+                        if pattern:
+                            parts = re.split(f'({pattern})', value, flags=re.IGNORECASE)
+                            if len(parts) > 1:
+                                value = request.env['ir.ui.view'].sudo()._render_template(
+                                    "website.search_text_with_highlight",
+                                    {'parts': parts}
+                                )
+                                field_type = 'html'
+
+                if field_type not in ('image', 'binary') and ('ir.qweb.field.%s' % field_type) in request.env:
+                    opt = {}
+                    if field_type == 'monetary':
+                        opt['display_currency'] = options['display_currency']
+                    value = request.env[('ir.qweb.field.%s' % field_type)].value_to_html(value, opt)
+                mapped[mapped_name] = escape(value)
+            result.append(mapped)
+
+        return {
+            'results': result,
+            'results_count': results_count,
+            'parts': {key: True for mapping in mappings for key in mapping},
+            'fuzzy_search': fuzzy_term,
+        }
+
+    @http.route(['/pages', '/pages/page/<int:page>'], type='http', auth="public", website=True, sitemap=False)
+    def pages_list(self, page=1, search='', **kw):
+        options = {
+            'displayDescription': False,
+            'displayDetail': False,
+            'displayExtraDetail': False,
+            'displayExtraLink': False,
+            'displayImage': False,
+            'allowFuzzy': not kw.get('noFuzzy'),
+        }
         step = 50
+        pages_count, details, fuzzy_search_term = request.website._search_with_fuzzy(
+            "pages", search, limit=page * step, order='name asc, website_id desc, id',
+            options=options)
+        pages = details[0].get('results', request.env['website.page'])
+
         pager = portal_pager(
-            url="/website/pages",
-            url_args={'sortby': sortby},
+            url="/pages",
+            url_args={'search': search},
             total=pages_count,
             page=page,
             step=step
@@ -361,14 +540,63 @@ class Website(Home):
         values = {
             'pager': pager,
             'pages': pages,
-            'search': search,
-            'sortby': sortby,
-            'searchbar_sortings': searchbar_sortings,
+            'search': fuzzy_search_term or search,
+            'search_count': pages_count,
+            'original_search': fuzzy_search_term and search,
         }
-        return request.render("website.list_website_pages", values)
+        return request.render("website.list_website_public_pages", values)
 
-    @http.route(['/website/add/', '/website/add/<path:path>'], type='http', auth="user", website=True, methods=['POST'])
-    def pagenew(self, path="", noredirect=False, add_menu=False, template=False, **kwargs):
+    @http.route([
+        '/website/search',
+        '/website/search/page/<int:page>',
+        '/website/search/<string:search_type>',
+        '/website/search/<string:search_type>/page/<int:page>',
+    ], type='http', auth="public", website=True, sitemap=False)
+    def hybrid_list(self, page=1, search='', search_type='all', **kw):
+        if not search:
+            return request.render("website.list_hybrid")
+
+        options = {
+            'displayDescription': True,
+            'displayDetail': True,
+            'displayExtraDetail': True,
+            'displayExtraLink': True,
+            'displayImage': True,
+            'allowFuzzy': not kw.get('noFuzzy'),
+        }
+        data = self.autocomplete(search_type=search_type, term=search, order='name asc', limit=500, max_nb_chars=200, options=options)
+
+        results = data.get('results', [])
+        search_count = len(results)
+        parts = data.get('parts', {})
+
+        step = 50
+        pager = portal_pager(
+            url="/website/search/%s" % search_type,
+            url_args={'search': search},
+            total=search_count,
+            page=page,
+            step=step
+        )
+
+        results = results[(page - 1) * step:page * step]
+
+        values = {
+            'pager': pager,
+            'results': results,
+            'parts': parts,
+            'search': search,
+            'fuzzy_search': data.get('fuzzy_search'),
+            'search_count': search_count,
+        }
+        return request.render("website.list_hybrid", values)
+
+    # ------------------------------------------------------
+    # Edit
+    # ------------------------------------------------------
+
+    @http.route(['/website/add', '/website/add/<path:path>'], type='http', auth="user", website=True, methods=['POST'])
+    def pagenew(self, path="", add_menu=False, template=False, redirect=False, **kwargs):
         # for supported mimetype, get correct default template
         _, ext = os.path.splitext(path)
         ext_special_case = ext and ext in _guess_mimetype() and ext != '.html'
@@ -379,14 +607,21 @@ class Website(Home):
                 template = default_templ
 
         template = template and dict(template=template) or {}
+        website_id = kwargs.get('website_id')
+        if website_id:
+            website = request.env['website'].browse(int(website_id))
+            website._force()
         page = request.env['website'].new_page(path, add_menu=add_menu, **template)
         url = page['url']
-        if noredirect:
-            return werkzeug.wrappers.Response(url, mimetype='text/plain')
 
-        if ext_special_case:  # redirect non html pages to backend to edit
-            return werkzeug.utils.redirect('/web#id=' + str(page.get('view_id')) + '&view_type=form&model=ir.ui.view')
-        return werkzeug.utils.redirect(url + "?enable_editor=1")
+        if redirect:
+            if ext_special_case:  # redirect non html pages to backend to edit
+                return request.redirect('/web#id=' + str(page.get('view_id')) + '&view_type=form&model=ir.ui.view')
+            return request.redirect(request.env['website'].get_client_action_url(url, True))
+
+        if ext_special_case:
+            return json.dumps({'view_id': page.get('view_id')})
+        return json.dumps({'url': url})
 
     @http.route("/website/get_switchable_related_views", type="json", auth="user", website=True)
     def get_switchable_related_views(self, key):
@@ -394,13 +629,6 @@ class Website(Home):
         views = views.sorted(key=lambda v: (v.inherit_id.id, v.name))
         return views.with_context(display_website=False).read(['name', 'id', 'key', 'xml_id', 'active', 'inherit_id'])
 
-    @http.route('/website/toggle_switchable_view', type='json', auth='user', website=True)
-    def toggle_switchable_view(self, view_key):
-        if request.website.user_has_groups('website.group_website_designer'):
-            request.website.viewref(view_key).toggle_active()
-        else:
-            return werkzeug.exceptions.Forbidden()
-
     @http.route('/website/reset_template', type='http', auth='user', methods=['POST'], website=True, csrf=False)
     def reset_template(self, view_id, mode='soft', redirect='/', **kwargs):
         """ This method will try to reset a broken view.
@@ -442,7 +670,7 @@ class Website(Home):
 
     @http.route(['/website/get_seo_data'], type='json', auth="user", website=True)
     def get_seo_data(self, res_id, res_model):
-        if not request.env.user.has_group('website.group_website_publisher'):
+        if not request.env.user.has_group('website.group_website_restricted_editor'):
             raise werkzeug.exceptions.Forbidden()
 
         fields = ['website_meta_title', 'website_meta_description', 'website_meta_keywords', 'website_meta_og_img']
@@ -450,7 +678,7 @@ class Website(Home):
             fields.extend(['website_indexed', 'website_id'])
 
         record = request.env[res_model].browse(res_id)
-        res = record._read_format(fields)[0]
+        res = record.read(fields)[0]
         res['has_social_default_image'] = request.website.has_social_default_image
 
         if res_model not in ('website.page', 'ir.ui.view') and 'seo_name' in record:  # allow custom slugify
@@ -463,8 +691,9 @@ class Website(Home):
         if not request.website.google_search_console:
             logger.warning('Google Search Console not enable')
             raise werkzeug.exceptions.NotFound()
+        gsc = request.website.google_search_console
+        trusted = gsc[gsc.startswith('google') and len('google'):gsc.endswith('.html') and -len('.html') or None]
 
-        trusted = request.website.google_search_console.lstrip('google').rstrip('.html')
         if key != trusted:
             if key.startswith(trusted):
                 request.website.sudo().google_search_console = "google%s.html" % key
@@ -484,60 +713,46 @@ class Website(Home):
     # Themes
     # ------------------------------------------------------
 
-    def _get_customize_views(self, xml_ids):
-        View = request.env["ir.ui.view"].with_context(active_test=False)
-        if not xml_ids:
-            return View
-        domain = [("key", "in", xml_ids)] + request.website.website_domain()
-        return View.search(domain).filter_duplicate()
+    def _get_customize_data(self, keys, is_view_data):
+        model = 'ir.ui.view' if is_view_data else 'ir.asset'
+        Model = request.env[model].with_context(active_test=False)
+        domain = expression.AND([[("key", "in", keys)], request.website.website_domain()])
+        return Model.search(domain).filter_duplicate()
 
-    @http.route(['/website/theme_customize_get'], type='json', auth='user', website=True)
-    def theme_customize_get(self, xml_ids):
-        views = self._get_customize_views(xml_ids)
-        return views.filtered('active').mapped('key')
+    @http.route(['/website/theme_customize_data_get'], type='json', auth='user', website=True)
+    def theme_customize_data_get(self, keys, is_view_data):
+        records = self._get_customize_data(keys, is_view_data)
+        return records.filtered('active').mapped('key')
 
-    @http.route(['/website/theme_customize'], type='json', auth='user', website=True)
-    def theme_customize(self, enable=None, disable=None):
+    @http.route(['/website/theme_customize_data'], type='json', auth='user', website=True)
+    def theme_customize_data(self, is_view_data, enable=None, disable=None, reset_view_arch=False):
         """
-        Enables and/or disables views according to list of keys.
+        Enables and/or disables views/assets according to list of keys.
 
-        :param enable: list of views' keys to enable
-        :param disable: list of views' keys to disable
+        :param is_view_data: True = "ir.ui.view", False = "ir.asset"
+        :param enable: list of views/assets keys to enable
+        :param disable: list of views/assets keys to disable
+        :param reset_view_arch: restore the default template after disabling
         """
-        self._get_customize_views(disable).filtered('active').write({'active': False})
-        self._get_customize_views(enable).filtered(lambda x: not x.active).write({'active': True})
+        if disable:
+            records = self._get_customize_data(disable, is_view_data).filtered('active')
+            if reset_view_arch:
+                records.reset_arch(mode='hard')
+            records.write({'active': False})
+
+        if enable:
+            records = self._get_customize_data(enable, is_view_data)
+            records.filtered(lambda x: not x.active).write({'active': True})
 
     @http.route(['/website/theme_customize_bundle_reload'], type='json', auth='user', website=True)
     def theme_customize_bundle_reload(self):
         """
         Reloads asset bundles and returns their unique URLs.
         """
-        context = dict(request.context)
         return {
-            'web.assets_common': request.env['ir.qweb']._get_asset_link_urls('web.assets_common', options=context),
-            'web.assets_frontend': request.env['ir.qweb']._get_asset_link_urls('web.assets_frontend', options=context),
-            'website.assets_editor': request.env['ir.qweb']._get_asset_link_urls('website.assets_editor', options=context),
+            'web.assets_frontend': request.env['ir.qweb']._get_asset_link_urls('web.assets_frontend', request.session.debug),
         }
 
-    @http.route(['/website/make_scss_custo'], type='json', auth='user', website=True)
-    def make_scss_custo(self, url, values):
-        """
-        Params:
-            url (str):
-                the URL of the scss file to customize (supposed to be a variable
-                file which will appear in the assets_common bundle)
-
-            values (dict):
-                key,value mapping to integrate in the file's map (containing the
-                word hook). If a key is already in the file's map, its value is
-                overridden.
-
-        Returns:
-            boolean
-        """
-        request.env['web_editor.assets'].make_scss_customization(url, values)
-        return True
-
     # ------------------------------------------------------
     # Server actions
     # ------------------------------------------------------
@@ -574,11 +789,9 @@ class Website(Home):
         return request.redirect('/')
 
 
-# ------------------------------------------------------
-# Retrocompatibility routes
-# ------------------------------------------------------
-class WebsiteBinary(http.Controller):
+class WebsiteBinary(Binary):
 
+    # Retrocompatibility routes
     @http.route([
         '/website/image',
         '/website/image/<xmlid>',
@@ -588,7 +801,8 @@ class WebsiteBinary(http.Controller):
         '/website/image/<model>/<id>/<field>',
         '/website/image/<model>/<id>/<field>/<int:width>x<int:height>'
     ], type='http', auth="public", website=False, multilang=False)
-    def content_image(self, id=None, max_width=0, max_height=0, **kw):
+    # pylint: disable=redefined-builtin,invalid-name
+    def website_content_image(self, id=None, max_width=0, max_height=0, **kw):
         if max_width:
             kw['width'] = max_width
         if max_height:
@@ -598,8 +812,9 @@ class WebsiteBinary(http.Controller):
             kw['id'] = int(id)
             if unique:
                 kw['unique'] = unique
-        return Binary().content_image(**kw)
+        return self.content_image(**kw)
 
+    # TODO in master: move this route outside the WebsiteBinary class
     # if not icon provided in DOM, browser tries to access /favicon.ico, eg when opening an order pdf
     @http.route(['/favicon.ico'], type='http', auth='public', website=True, multilang=False, sitemap=False)
     def favicon(self, **kw):
