@@ -2,9 +2,14 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import assets
-from . import ir_actions
+from . import base_partner_merge
+from . import ir_actions_server
+from . import ir_asset
 from . import ir_attachment
+from . import ir_binary
 from . import ir_http
+from . import ir_model
+from . import ir_model_data
 from . import ir_module_module
 from . import ir_qweb
 from . import ir_qweb_fields
@@ -14,7 +19,6 @@ from . import website_menu
 from . import website_page
 from . import website_rewrite
 from . import ir_rule
-from . import ir_translation
 from . import ir_ui_view
 from . import res_company
 from . import res_partner
@@ -22,5 +26,7 @@ from . import res_users
 from . import res_config_settings
 from . import res_lang
 from . import theme_models
-from . import website_visitor
+from . import website_configurator_feature
+from . import website_form
 from . import website_snippet_filter
+from . import website_visitor
