@@ -3,7 +3,7 @@
 
 import logging
 import os
-from collections import OrderedDict
+from collections import defaultdict, OrderedDict
 
 from odoo import api, fields, models
 from odoo.addons.base.models.ir_model import MODULE_UNINSTALL_FLAG
@@ -21,6 +21,7 @@ class IrModuleModule(models.Model):
     # The order is important because of dependencies (page need view, menu need page)
     _theme_model_names = OrderedDict([
         ('ir.ui.view', 'theme.ir.ui.view'),
+        ('ir.asset', 'theme.ir.asset'),
         ('website.page', 'theme.website.page'),
         ('website.menu', 'theme.website.menu'),
         ('ir.attachment', 'theme.ir.attachment'),
@@ -74,6 +75,9 @@ class IrModuleModule(models.Model):
 
                     -> We want to upgrade every website using this theme.
         """
+        if request and request.db and request.context.get('apply_new_theme'):
+            self = self.with_context(apply_new_theme=True)
+
         for module in self:
             if module.name.startswith('theme_') and vals.get('state') == 'installed':
                 _logger.info('Module %s has been loaded as theme template (%s)' % (module.name, module.state))
@@ -152,7 +156,9 @@ class IrModuleModule(models.Model):
                 # special case for attachment
                 # if module B override attachment from dependence A, we update it
                 if not find and model_name == 'ir.attachment':
-                    find = rec.copy_ids.search([('key', '=', rec.key), ('website_id', '=', website.id)])
+                    # In master, a unique constraint over (theme_template_id, website_id)
+                    # will be introduced, thus ensuring unicity of 'find'
+                    find = rec.copy_ids.search([('key', '=', rec.key), ('website_id', '=', website.id), ("original_id", "=", False)])
 
                 if find:
                     imd = self.env['ir.model.data'].search([('model', '=', find._name), ('res_id', '=', find.id)])
@@ -182,14 +188,31 @@ class IrModuleModule(models.Model):
     def _post_copy(self, old_rec, new_rec):
         self.ensure_one()
         translated_fields = self._theme_translated_fields.get(old_rec._name, [])
+        cur_lang = self.env.lang or 'en_US'
+        old_rec.flush_recordset()
         for (src_field, dst_field) in translated_fields:
-            self._cr.execute("""INSERT INTO ir_translation (lang, src, name, res_id, state, value, type, module)
-                                SELECT t.lang, t.src, %s, %s, t.state, t.value, t.type, t.module
-                                FROM ir_translation t
-                                WHERE name = %s
-                                  AND res_id = %s
-                                ON CONFLICT DO NOTHING""",
-                             (dst_field, new_rec.id, src_field, old_rec.id))
+            __, src_fname = src_field.split(',')
+            dst_mname, dst_fname = dst_field.split(',')
+            if dst_mname != new_rec._name:
+                continue
+            old_field = old_rec._fields[src_fname]
+            old_translations = old_field._get_stored_translations(old_rec)
+            if not old_translations:
+                continue
+            if not callable(old_field.translate):
+                if old_rec[src_fname] == new_rec[dst_fname]:
+                    new_rec.update_field_translations(dst_fname, old_translations)
+            else:
+                old_translation_lang = old_translations.get(cur_lang) or old_translations.get('en_US')
+                # {from_lang_term: {lang: to_lang_term}
+                translation_dictionary = old_field.get_translation_dictionary(old_translation_lang, {
+                    lang: value for lang, value in old_translations.items() if lang != cur_lang})
+                # {lang: {old_term: new_term}
+                translations = defaultdict(dict)
+                for from_lang_term, to_lang_terms in translation_dictionary.items():
+                    for lang, to_lang_term in to_lang_terms.items():
+                        translations[lang][from_lang_term] = to_lang_term
+                new_rec.with_context(install_filename='dummy').update_field_translations(dst_fname, translations)
 
     def _theme_load(self, website):
         """
@@ -204,7 +227,15 @@ class IrModuleModule(models.Model):
             for model_name in self._theme_model_names:
                 module._update_records(model_name, website)
 
-            self.env['theme.utils'].with_context(website_id=website.id)._post_copy(module)
+            if self._context.get('apply_new_theme'):
+                # Both the theme install and upgrade flow ends up here.
+                # The _post_copy() is supposed to be called only when the theme
+                # is installed for the first time on a website.
+                # It will basically select some header and footer template.
+                # We don't want the system to select again the theme footer or
+                # header template when that theme is updated later. It could
+                # erase the change the user made after the theme install.
+                self.env['theme.utils'].with_context(website_id=website.id)._post_copy(module)
 
     def _theme_unload(self, website):
         """
@@ -358,13 +389,19 @@ class IrModuleModule(models.Model):
         website.theme_id = self
 
         # this will install 'self' if it is not installed yet
+        if request:
+            request.update_context(apply_new_theme=True)
         self._theme_upgrade_upstream()
 
         active_todo = self.env['ir.actions.todo'].search([('state', '=', 'open')], limit=1)
+        result = None
         if active_todo:
-            return active_todo.action_launch()
+            result = active_todo.action_launch()
         else:
-            return website.button_go_website(mode_edit=True)
+            result = website.button_go_website(mode_edit=True)
+        if result.get('tag') == 'website_preview' and result.get('context', {}).get('params', {}).get('enable_editor'):
+            result['context']['params']['with_loader'] = True
+        return result
 
     def button_remove_theme(self):
         """Remove the current theme of the current website."""
@@ -412,6 +449,21 @@ class IrModuleModule(models.Model):
                         'res_id': theme.id,
                     })
 
+    def get_themes_domain(self):
+        """Returns the 'ir.module.module' search domain matching all available themes."""
+        def get_id(model_id):
+            return self.env['ir.model.data']._xmlid_to_res_id(model_id)
+        return [
+            ('state', '!=', 'uninstallable'),
+            ('category_id', 'not in', [
+                get_id('base.module_category_hidden'),
+                get_id('base.module_category_theme_hidden'),
+            ]),
+            '|',
+            ('category_id', '=', get_id('base.module_category_theme')),
+            ('category_id.parent_id', '=', get_id('base.module_category_theme'))
+        ]
+
     def _check(self):
         super()._check()
         View = self.env['ir.ui.view']
@@ -421,3 +473,72 @@ class IrModuleModule(models.Model):
                 cow_view = View.browse(view_replay[0])
                 View._load_records_write_on_cow(cow_view, view_replay[1], view_replay[2])
             self.pool.website_views_to_adapt.clear()
+
+    @api.model
+    def _load_module_terms(self, modules, langs, overwrite=False):
+        """ Add missing website specific translation """
+        res = super()._load_module_terms(modules, langs, overwrite=overwrite)
+
+        if not langs or langs == ['en_US'] or not modules:
+            return res
+
+        # Add specific view translations
+
+        # use the translation dic of the generic to translate the specific
+        self.env.cr.flush()
+        cache = self.env.cache
+        View = self.env['ir.ui.view']
+        field = self.env['ir.ui.view']._fields['arch_db']
+        # assume there are not too many records
+        self.env.cr.execute(""" SELECT generic.arch_db, specific.arch_db, specific.id
+                          FROM ir_ui_view generic
+                         INNER JOIN ir_ui_view specific
+                            ON generic.key = specific.key
+                         WHERE generic.website_id IS NULL AND generic.type = 'qweb'
+                         AND specific.website_id IS NOT NULL
+            """)
+        for generic_arch_db, specific_arch_db, specific_id in self.env.cr.fetchall():
+            if not generic_arch_db:
+                continue
+            langs_update = (langs & generic_arch_db.keys()) - {'en_US'}
+            if not langs_update:
+                continue
+            # get dictionaries limited to the requested languages
+            generic_arch_db_en = generic_arch_db.get('en_US')
+            specific_arch_db_en = specific_arch_db.get('en_US')
+            generic_arch_db_update = {k: generic_arch_db[k] for k in langs_update}
+            specific_arch_db_update = {k: specific_arch_db.get(k, specific_arch_db_en) for k in langs_update}
+            generic_translation_dictionary = field.get_translation_dictionary(generic_arch_db_en, generic_arch_db_update)
+            specific_translation_dictionary = field.get_translation_dictionary(specific_arch_db_en, specific_arch_db_update)
+            # update specific_translation_dictionary
+            for term_en, specific_term_langs in specific_translation_dictionary.items():
+                if term_en not in generic_translation_dictionary:
+                    continue
+                for lang, generic_term_lang in generic_translation_dictionary[term_en].items():
+                    if overwrite or term_en == specific_term_langs[lang]:
+                        specific_term_langs[lang] = generic_term_lang
+            for lang in langs_update:
+                specific_arch_db[lang] = field.translate(
+                    lambda term: specific_translation_dictionary.get(term, {lang: None})[lang], specific_arch_db_en)
+            cache.update_raw(View.browse(specific_id), field, [specific_arch_db], dirty=True)
+
+        default_menu = self.env.ref('website.main_menu', raise_if_not_found=False)
+        if not default_menu:
+            return res
+
+        o_menu_name = [f"'{lang}', o_menu.name->>'{lang}'" for lang in langs if lang != 'en_US']
+        o_menu_name = 'jsonb_build_object(' + ', '.join(o_menu_name) + ')'
+        self.env.cr.execute(f"""
+                        UPDATE website_menu menu
+                           SET name = {'menu.name || ' + o_menu_name if overwrite else o_menu_name + ' || menu.name'}
+                          FROM website_menu o_menu
+                         INNER JOIN website_menu s_menu
+                            ON o_menu.name->>'en_US' = s_menu.name->>'en_US' AND o_menu.url = s_menu.url
+                         INNER JOIN website_menu root_menu
+                            ON s_menu.parent_id = root_menu.id AND root_menu.parent_id IS NULL
+                         WHERE o_menu.website_id IS NULL AND o_menu.parent_id = %s
+                           AND s_menu.website_id IS NOT NULL
+                           AND menu.id = s_menu.id
+            """, (default_menu.id,))
+
+        return res
