@@ -2,13 +2,11 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 import logging
-import os
 import uuid
 import werkzeug
 
 from odoo import api, fields, models
 from odoo import tools
-from odoo.addons import website
 from odoo.exceptions import AccessError
 from odoo.osv import expression
 from odoo.http import request
@@ -45,15 +43,42 @@ class View(models.Model):
         for view in self:
             view.first_page_id = self.env['website.page'].search([('view_id', '=', view.id)], limit=1)
 
+    @api.model_create_multi
+    def create(self, vals_list):
+        """
+        SOC for ir.ui.view creation. If a view is created without a website_id,
+        it should get one if one is present in the context. Also check that
+        an explicit website_id in create values matches the one in the context.
+        """
+        website_id = self.env.context.get('website_id', False)
+        if not website_id:
+            return super().create(vals_list)
+
+        for vals in vals_list:
+            if 'website_id' not in vals:
+                # Automatic addition of website ID during view creation if not
+                # specified but present in the context
+                vals['website_id'] = website_id
+            else:
+                # If website ID specified, automatic check that it is the same as
+                # the one in the context. Otherwise raise an error.
+                new_website_id = vals['website_id']
+                if not new_website_id:
+                    raise ValueError(f"Trying to create a generic view from a website {website_id} environment")
+                elif new_website_id != website_id:
+                    raise ValueError(f"Trying to create a view for website {new_website_id} from a website {website_id} environment")
+        return super().create(vals_list)
+
     def name_get(self):
-        if (not self._context.get('display_website') and not self.env.user.has_group('website.group_multi_website')) or \
-                not self._context.get('display_website'):
+        if not (self._context.get('display_key') or self._context.get('display_website')):
             return super(View, self).name_get()
 
         res = []
         for view in self:
             view_name = view.name
-            if view.website_id:
+            if self._context.get('display_key'):
+                view_name += ' <%s>' % view.key
+            if self._context.get('display_website') and view.website_id:
                 view_name += ' [%s]' % view.website_id.name
             res.append((view.id, view_name))
         return res
@@ -70,12 +95,17 @@ class View(models.Model):
         # We need to consider inactive views when handling multi-website cow
         # feature (to copy inactive children views, to search for specific
         # views, ...)
-        for view in self.with_context(active_test=False):
+        # Website-specific views need to be updated first because they might
+        # be relocated to new ids by the cow if they are involved in the
+        # inheritance tree.
+        for view in self.with_context(active_test=False).sorted(key='website_id', reverse=True):
             # Make sure views which are written in a website context receive
             # a value for their 'key' field
             if not view.key and not vals.get('key'):
                 view.with_context(no_cow=True).key = 'website.key_%s' % str(uuid.uuid4())[:6]
 
+            pages = view.page_ids
+
             # No need of COW if the view is already specific
             if view.website_id:
                 super(View, view).write(vals)
@@ -87,9 +117,8 @@ class View(models.Model):
             # but in reality the values were only meant to go on the specific
             # page. Invalidate all fields and not only those in vals because
             # other fields could have been changed implicitly too.
-            pages = view.page_ids
-            pages.flush(records=pages)
-            pages.invalidate_cache(ids=pages.ids)
+            pages.flush_recordset()
+            pages.invalidate_recordset()
 
             # If already a specific view for this generic view, write on it
             website_specific_view = view.search([
@@ -131,6 +160,14 @@ class View(models.Model):
 
         return True
 
+    def _load_records_write_on_cow(self, cow_view, inherit_id, values):
+        inherit_id = self.search([
+            ('key', '=', self.browse(inherit_id).key),
+            ('website_id', 'in', (False, cow_view.website_id.id)),
+        ], order='website_id', limit=1).id
+        values['inherit_id'] = inherit_id
+        cow_view.with_context(no_cow=True).write(values)
+
     def _create_all_specific_views(self, processed_modules):
         """ When creating a generic child view, we should
             also create that view under specific view trees (COW'd).
@@ -199,6 +236,10 @@ class View(models.Model):
             })
             page.menu_ids.filtered(lambda m: m.website_id.id == website.id).page_id = new_page.id
 
+    def _get_top_level_view(self):
+        self.ensure_one()
+        return self.inherit_id._get_top_level_view() if self.inherit_id else self
+
     @api.model
     def get_related_views(self, key, bundles=False):
         '''Make this only return most specific views for website.'''
@@ -255,8 +296,8 @@ class View(models.Model):
             return view_id if view_id._name == 'ir.ui.view' else self.env['ir.ui.view']
 
     @api.model
-    def _get_inheriting_views_arch_domain(self, model):
-        domain = super(View, self)._get_inheriting_views_arch_domain(model)
+    def _get_inheriting_views_domain(self):
+        domain = super(View, self)._get_inheriting_views_domain()
         current_website = self.env['website'].browse(self._context.get('website_id'))
         website_views_domain = current_website.website_domain()
         # when rendering for the website we have to include inactive views
@@ -266,11 +307,11 @@ class View(models.Model):
         return expression.AND([website_views_domain, domain])
 
     @api.model
-    def get_inheriting_views_arch(self, model):
+    def _get_inheriting_views(self):
         if not self._context.get('website_id'):
-            return super(View, self).get_inheriting_views_arch(model)
+            return super(View, self)._get_inheriting_views()
 
-        views = super(View, self.with_context(active_test=False)).get_inheriting_views_arch(model)
+        views = super(View, self.with_context(active_test=False))._get_inheriting_views()
         # prefer inactive website-specific views over active generic ones
         return views.filter_duplicate().filtered('active')
 
@@ -301,7 +342,7 @@ class View(models.Model):
 
     @api.model
     @tools.ormcache_context('self.env.uid', 'self.env.su', 'xml_id', keys=('website_id',))
-    def get_view_id(self, xml_id):
+    def _get_view_id(self, xml_id):
         """If a website_id is in the context and the given xml_id is not an int
         then try to get the id of the specific view for that website, but
         fallback to the id of the generic view if there is no specific.
@@ -313,8 +354,9 @@ class View(models.Model):
         Archived views are ignored (unless the active_test context is set, but
         then the ormcache_context will not work as expected).
         """
-        if 'website_id' in self._context and not isinstance(xml_id, int):
-            current_website = self.env['website'].browse(self._context.get('website_id'))
+        website_id = self._context.get('website_id')
+        if website_id and not isinstance(xml_id, int):
+            current_website = self.env['website'].browse(int(website_id))
             domain = ['&', ('key', '=', xml_id)] + current_website.website_domain()
 
             view = self.sudo().search(domain, order='website_id', limit=1)
@@ -322,23 +364,11 @@ class View(models.Model):
                 _logger.warning("Could not find view object with xml_id '%s'", xml_id)
                 raise ValueError('View %r in website %r not found' % (xml_id, self._context['website_id']))
             return view.id
-        return super(View, self.sudo()).get_view_id(xml_id)
+        return super(View, self.sudo())._get_view_id(xml_id)
 
-    @api.model
-    def read_template(self, xml_id):
-        view = self._view_obj(self.get_view_id(xml_id))
-        if view.visibility and view._handle_visibility(do_raise=False):
-            self = self.sudo()
-        return super(View, self).read_template(xml_id)
-
-    def _get_original_view(self):
-        """Given a view, retrieve the original view it was COW'd from.
-        The given view might already be the original one. In that case it will
-        (and should) return itself.
-        """
-        self.ensure_one()
-        domain = [('key', '=', self.key), ('model_data_id', '!=', None)]
-        return self.with_context(active_test=False).search(domain, limit=1)  # Useless limit has multiple xmlid should not be possible
+    @tools.ormcache('self.id')
+    def _get_cached_visibility(self):
+        return self.visibility
 
     def _handle_visibility(self, do_raise=True):
         """ Check the visibility set on the main view and raise 403 if you should not have access.
@@ -350,23 +380,25 @@ class View(models.Model):
 
         self = self.sudo()
 
-        if self.visibility and not request.env.user.has_group('website.group_website_designer'):
-            if (self.visibility == 'connected' and request.website.is_public_user()):
+        visibility = self._get_cached_visibility()
+
+        if visibility and not request.env.user.has_group('website.group_website_designer'):
+            if (visibility == 'connected' and request.website.is_public_user()):
                 error = werkzeug.exceptions.Forbidden()
-            elif self.visibility == 'password' and \
+            elif visibility == 'password' and \
                     (request.website.is_public_user() or self.id not in request.session.get('views_unlock', [])):
                 pwd = request.params.get('visibility_password')
                 if pwd and self.env.user._crypt_context().verify(
-                        pwd, self.sudo().visibility_password):
+                        pwd, self.visibility_password):
                     request.session.setdefault('views_unlock', list()).append(self.id)
                 else:
                     error = werkzeug.exceptions.Forbidden('website_visibility_password_required')
 
-            # elif self.visibility == 'restricted_group' and self.groups_id: or if groups_id set from backend
-            try:
-                self._check_view_access()
-            except AccessError:
-                error = werkzeug.exceptions.Forbidden()
+            if visibility not in ('password', 'connected'):
+                try:
+                    self._check_view_access()
+                except AccessError:
+                    error = werkzeug.exceptions.Forbidden()
 
         if error:
             if do_raise:
@@ -375,69 +407,15 @@ class View(models.Model):
                 return False
         return True
 
-    def _render(self, values=None, engine='ir.qweb', minimal_qcontext=False):
+    def _render_template(self, template, values=None):
         """ Render the template. If website is enabled on request, then extend rendering context with website values. """
-        self._handle_visibility(do_raise=True)
-        new_context = dict(self._context)
-        if request and getattr(request, 'is_frontend', False):
-
-            editable = request.website.is_publisher()
-            translatable = editable and self._context.get('lang') != request.website.default_lang_id.code
-            editable = not translatable and editable
-
-            # in edit mode ir.ui.view will tag nodes
-            if not translatable and not self.env.context.get('rendering_bundle'):
-                if editable:
-                    new_context = dict(self._context, inherit_branding=True)
-                elif request.env.user.has_group('website.group_website_publisher'):
-                    new_context = dict(self._context, inherit_branding_auto=True)
-            if values and 'main_object' in values:
-                if request.env.user.has_group('website.group_website_publisher'):
-                    func = getattr(values['main_object'], 'get_backend_menu_id', False)
-                    values['backend_menu_id'] = func and func() or self.env['ir.model.data'].xmlid_to_res_id('website.menu_website_configuration')
-
-        if self._context != new_context:
-            self = self.with_context(new_context)
-        return super(View, self)._render(values, engine=engine, minimal_qcontext=minimal_qcontext)
-
-    @api.model
-    def _prepare_qcontext(self):
-        """ Returns the qcontext : rendering context with website specific value (required
-            to render website layout template)
-        """
-        qcontext = super(View, self)._prepare_qcontext()
-
-        if request and getattr(request, 'is_frontend', False):
-            Website = self.env['website']
-            editable = request.website.is_publisher()
-            translatable = editable and self._context.get('lang') != request.env['ir.http']._get_default_lang().code
-            editable = not translatable and editable
-
-            cur = Website.get_current_website()
-            if self.env.user.has_group('website.group_website_publisher') and self.env.user.has_group('website.group_multi_website'):
-                qcontext['multi_website_websites_current'] = {'website_id': cur.id, 'name': cur.name, 'domain': cur._get_http_domain()}
-                qcontext['multi_website_websites'] = [
-                    {'website_id': website.id, 'name': website.name, 'domain': website._get_http_domain()}
-                    for website in Website.search([]) if website != cur
-                ]
-
-                cur_company = self.env.company
-                qcontext['multi_website_companies_current'] = {'company_id': cur_company.id, 'name': cur_company.name}
-                qcontext['multi_website_companies'] = [
-                    {'company_id': comp.id, 'name': comp.name}
-                    for comp in self.env.user.company_ids if comp != cur_company
-                ]
-
-            qcontext.update(dict(
-                main_object=self,
-                website=request.website,
-                is_view_active=request.website.is_view_active,
-                res_company=request.website.company_id.sudo(),
-                translatable=translatable,
-                editable=editable,
-            ))
-
-        return qcontext
+        view = self._get(template).sudo()
+        view._handle_visibility(do_raise=True)
+        if values is None:
+            values = {}
+        if 'main_object' not in values:
+            values['main_object'] = view
+        return super()._render_template(template, values=values)
 
     @api.model
     def get_default_lang_code(self):
@@ -448,13 +426,6 @@ class View(models.Model):
         else:
             return super(View, self).get_default_lang_code()
 
-    def redirect_to_page_manager(self):
-        return {
-            'type': 'ir.actions.act_url',
-            'url': '/website/pages',
-            'target': 'self',
-        }
-
     def _read_template_keys(self):
         return super(View, self)._read_template_keys() + ['website_id']
 
@@ -491,6 +462,14 @@ class View(models.Model):
                 self = website_specific_view
         super(View, self).save(value, xpath=xpath)
 
+    @api.model
+    def _get_allowed_root_attrs(self):
+        # Related to these options:
+        # background-video, background-shapes, parallax
+        return super()._get_allowed_root_attrs() + [
+            'data-bg-video-src', 'data-shape', 'data-scroll-background-ratio',
+        ]
+
     # --------------------------------------------------------------------------
     # Snippet saving
     # --------------------------------------------------------------------------
@@ -502,3 +481,14 @@ class View(models.Model):
         if website_id:
             res['website_id'] = website_id
         return res
+
+    def _update_field_translations(self, fname, translations, digest=None):
+        return super(View, self.with_context(no_cow=True))._update_field_translations(fname, translations, digest)
+
+    def _get_base_lang(self):
+        """ Returns the default language of the website as the base language if the record is bound to it """
+        self.ensure_one()
+        website = self.website_id
+        if website:
+            return website.default_lang_id.code
+        return super()._get_base_lang()
