@@ -27,7 +27,7 @@ options.registry.InnerChart = options.Class.extend({
     init: function () {
         this._super.apply(this, arguments);
         this.themeArray = ['o-color-1', 'o-color-2', 'o-color-3', 'o-color-4', 'o-color-5'];
-        this.style = window.getComputedStyle(document.documentElement);
+        this.style = window.getComputedStyle(this.$target[0].ownerDocument.documentElement);
     },
     /**
      * @override
@@ -118,6 +118,9 @@ options.registry.InnerChart = options.Class.extend({
             this._setDefaultSelectedInput();
             await this._reloadGraph();
         }
+        if (params.attributeName === 'minValue' || params.attributeName === 'maxValue') {
+            this._computeTicksMinMaxValue();
+        }
     },
 
     //--------------------------------------------------------------------------
@@ -148,6 +151,44 @@ options.registry.InnerChart = options.Class.extend({
         }
         return this._super(...arguments);
     },
+    /**
+     * Maintains the gap between the scale axis for the auto fit behavior if we
+     * used min/max config.
+     *
+     * @private
+     */
+    _computeTicksMinMaxValue() {
+        const dataset = this.$target[0].dataset;
+        let minValue = parseInt(dataset.minValue);
+        let maxValue = parseInt(dataset.maxValue);
+        if (!isNaN(maxValue)) {
+            // Reverse min max values when min value is greater than max value
+            if (maxValue < minValue) {
+                maxValue = minValue;
+                minValue = parseInt(dataset.maxValue);
+            } else if (maxValue === minValue) {
+                // If min value and max value are same for positive and negative
+                // number
+                minValue < 0 ? (maxValue = 0, minValue = 2 * minValue) : (minValue = 0, maxValue = 2 * maxValue);
+            }
+        } else {
+            // Find max value from each row/column data
+            const datasets = JSON.parse(dataset.data).datasets || [];
+            const dataValue = _.flatten(datasets.map(el => el.data.map(data => {
+                return !isNaN(parseInt(data)) ? parseInt(data) : 0;
+            })));
+            // When max value is not given and min value is greater than chart
+            // data values
+            if (minValue >= Math.max(...dataValue)) {
+                maxValue = minValue;
+                minValue = 0;
+            }
+        }
+        this.$target.attr({
+            'data-ticks-min': minValue,
+            'data-ticks-max': maxValue,
+        });
+    },
     /**
      * Sets and reloads the data on the canvas if it has changed.
      * Used in matrix related method.
@@ -203,6 +244,7 @@ options.registry.InnerChart = options.Class.extend({
         const rmbuttonEl = options.buildElement('we-button', null, {
             classes: ['o_we_text_danger', 'o_we_link', 'fa', 'fa-fw', 'fa-minus', ...classes],
         });
+        rmbuttonEl.title = classes.includes('o_we_matrix_remove_col') ? _t("Remove Serie") : _t("Remove Row");
         const newEl = document.createElement('td');
         newEl.appendChild(rmbuttonEl);
         return newEl;
@@ -256,7 +298,7 @@ options.registry.InnerChart = options.Class.extend({
      * @param {string} tag tag of the HTML Element (td/th)
      * @param {string} value The current value of the cell input
      * @param {string} backgroundColor The background Color of the data on the graph
-     * @param {string} borderColor The border Color of the the data on the graph
+     * @param {string} borderColor The border Color of the data on the graph
      * @returns {HTMLElement}
      */
     _makeCell: function (tag, value, backgroundColor, borderColor) {
@@ -437,18 +479,7 @@ options.registry.InnerChart = options.Class.extend({
      * @param {Event} ev
      */
     _onMatrixInputFocusOut: function (ev) {
-        // Sometimes, an input is focusout for internal reason (like an undo
-        // recording) then focused again manually in the same JS stack
-        // execution. In that case, the blur should not trigger an option
-        // selection as the user did not leave the input. We thus defer the blur
-        // handling to then check that the target is indeed still blurred before
-        // executing the actual option selection.
-        setTimeout(() => {
-            if (ev.currentTarget === document.activeElement) {
-                return;
-            }
-            this._reloadGraph();
-        });
+        this._reloadGraph();
     },
     /**
      * Set the selected cell/header and display the related remove button
