@@ -1,41 +1,53 @@
 odoo.define('website.s_image_gallery_options', function (require) {
 'use strict';
 
+const { MediaDialogWrapper } = require('@web_editor/components/media_dialog/media_dialog');
+const { ComponentWrapper } = require('web.OwlCompatibility');
 var core = require('web.core');
-var weWidgets = require('wysiwyg.widgets');
 var options = require('web_editor.snippets.options');
+const wUtils = require("website.utils");
 
 var _t = core._t;
 var qweb = core.qweb;
 
 options.registry.gallery = options.Class.extend({
-    xmlDependencies: ['/website/static/src/snippets/s_image_gallery/000.xml'],
-
     /**
      * @override
      */
     start: function () {
         var self = this;
+        // TODO In master: define distinct classes.
+        // Differentiate both instances of this class: we want to avoid
+        // registering the same event listener twice.
+        this.hasAddImages = this.el.querySelector("we-button[data-add-images]");
 
-        // The snippet should not be editable
-        this.$target.addClass('o_fake_not_editable').attr('contentEditable', false);
+        if (!this.hasAddImages) {
+            let layoutPromise;
+            const containerEl = this.$target[0].querySelector(":scope > .container, :scope > .container-fluid, :scope > .o_container_small");
+            if (containerEl.querySelector(":scope > *:not(div)")) {
+                layoutPromise = self._modeWithImageWait(null, self.getMode());
+            } else {
+                layoutPromise = Promise.resolve();
+            }
+            return layoutPromise.then(this._super.apply(this, arguments));
+        }
 
         // Make sure image previews are updated if images are changed
-        this.$target.on('save', 'img', function (ev) {
+        this.$target.on('image_changed.gallery', 'img', function (ev) {
             var $img = $(ev.currentTarget);
             var index = self.$target.find('.carousel-item.active').index();
-            self.$('.carousel:first li[data-target]:eq(' + index + ')')
+            self.$('.carousel:first li[data-bs-target]:eq(' + index + ')')
                 .css('background-image', 'url(' + $img.attr('src') + ')');
         });
 
         // When the snippet is empty, an edition button is the default content
         // TODO find a nicer way to do that to have editor style
-        this.$target.on('click', '.o_add_images', function (e) {
+        this.$target.on('click.gallery', '.o_add_images', function (e) {
             e.stopImmediatePropagation();
             self.addImages(false);
         });
 
-        this.$target.on('dropped', 'img', function (ev) {
+        this.$target.on('dropped.gallery', 'img', function (ev) {
             self.mode(null, self.getMode());
             if (!ev.target.height) {
                 $(ev.target).one('load', function () {
@@ -46,11 +58,6 @@ options.registry.gallery = options.Class.extend({
             }
         });
 
-        const $container = this.$('> .container, > .container-fluid, > .o_container_small');
-        if ($container.find('> *:not(div)').length) {
-            self.mode(null, self.getMode());
-        }
-
         return this._super.apply(this, arguments);
     },
     /**
@@ -77,6 +84,13 @@ options.registry.gallery = options.Class.extend({
             this.$target.removeAttr('style');
         }
     },
+    /**
+     * @override
+     */
+    destroy() {
+        this._super(...arguments);
+        this.$target.off('.gallery');
+    },
 
     //--------------------------------------------------------------------------
     // Options
@@ -90,29 +104,39 @@ options.registry.gallery = options.Class.extend({
     addImages: function (previewMode) {
         const $images = this.$('img');
         var $container = this.$('> .container, > .container-fluid, > .o_container_small');
-        var dialog = new weWidgets.MediaDialog(this, {multiImages: true, onlyImages: true, mediaWidth: 1920});
-        var lastImage = _.last(this._getImages());
-        var index = lastImage ? this._getIndex(lastImage) : -1;
-        return new Promise(resolve => {
-            dialog.on('save', this, function (attachments) {
-                for (var i = 0; i < attachments.length; i++) {
-                    $('<img/>', {
-                        class: $images.length > 0 ? $images[0].className : 'img img-fluid d-block ',
-                        src: attachments[i].image_src,
-                        'data-index': ++index,
-                        alt: attachments[i].description || '',
-                        'data-name': _t('Image'),
-                        style: $images.length > 0 ? $images[0].style.cssText : '',
-                    }).appendTo($container);
-                }
-                if (attachments.length > 0) {
-                    this.mode('reset', this.getMode());
-                    this.trigger_up('cover_update');
-                }
-            });
-            dialog.on('closed', this, () => resolve());
-            dialog.open();
+        const lastImage = _.last(this._getImages());
+        let index = lastImage ? this._getIndex(lastImage) : -1;
+        const dialog = new ComponentWrapper(this, MediaDialogWrapper, {
+            multiImages: true,
+            onlyImages: true,
+            save: images => {
+                // TODO In master: restore addImages Promise result.
+                this.trigger_up('snippet_edition_request', {exec: () => {
+                    let $newImageToSelect;
+                    for (const image of images) {
+                        const $img = $('<img/>', {
+                            class: $images.length > 0 ? $images[0].className : 'img img-fluid d-block ',
+                            src: image.src,
+                            'data-index': ++index,
+                            alt: image.alt || '',
+                            'data-name': _t('Image'),
+                            style: $images.length > 0 ? $images[0].style.cssText : '',
+                        }).appendTo($container);
+                        if (!$newImageToSelect) {
+                            $newImageToSelect = $img;
+                        }
+                    }
+                    if (images.length > 0) {
+                        return this._modeWithImageWait('reset', this.getMode()).then(() => {
+                            this.trigger_up('cover_update');
+                            // Triggers the re-rendering of the thumbnail
+                            $newImageToSelect.trigger('image_changed');
+                        });
+                    }
+                }});
+            },
         });
+        dialog.mount(this.el);
     },
     /**
      * Allows to change the number of columns when displaying images with a
@@ -124,6 +148,7 @@ options.registry.gallery = options.Class.extend({
         const nbColumns = parseInt(widgetValue || '1');
         this.$target.attr('data-columns', nbColumns);
 
+        // TODO In master return mode's result.
         this.mode(previewMode, this.getMode(), {}); // TODO improve
     },
     /**
@@ -155,7 +180,7 @@ options.registry.gallery = options.Class.extend({
         var $container = this._replaceContent($row);
 
         _.each(imgs, function (img, index) {
-            var $img = $(img);
+            const $img = $(img.cloneNode());
             var $col = $('<div/>', {class: colClass});
             $col.append($img).appendTo($row);
             if ((index + 1) % columns === 0) {
@@ -187,18 +212,47 @@ options.registry.gallery = options.Class.extend({
 
         // Dispatch images in columns by always putting the next one in the
         // smallest-height column
+        if (this._masonryAwaitImages) {
+            // TODO In master return promise.
+            this._masonryAwaitImagesPromise = new Promise(async resolve => {
+                for (const imgEl of imgs) {
+                    let min = Infinity;
+                    let smallestColEl;
+                    for (const colEl of cols) {
+                        const imgEls = colEl.querySelectorAll("img");
+                        const lastImgRect = imgEls.length && imgEls[imgEls.length - 1].getBoundingClientRect();
+                        const height = lastImgRect ? Math.round(lastImgRect.top + lastImgRect.height) : 0;
+                        if (height < min) {
+                            min = height;
+                            smallestColEl = colEl;
+                        }
+                    }
+                    smallestColEl.append(imgEl.cloneNode());
+                    await wUtils.onceAllImagesLoaded(this.$target);
+                }
+                resolve();
+            });
+            return;
+        }
+        // TODO Remove in master.
+        // Order might be wrong if images were not loaded yet.
         while (imgs.length) {
             var min = Infinity;
             var $lowest;
             _.each(cols, function (col) {
                 var $col = $(col);
                 var height = $col.is(':empty') ? 0 : $col.find('img').last().offset().top + $col.find('img').last().height() - self.$target.offset().top;
+                // Neutralize invisible sub-pixel height differences.
+                height = Math.round(height);
                 if (height < min) {
                     min = height;
                     $lowest = $col;
                 }
             });
-            $lowest.append(imgs.shift());
+            // Only on Chrome: appended images are sometimes invisible and not
+            // correctly loaded from cache, we use a clone of the image to force
+            // the loading.
+            $lowest.append(imgs.shift().cloneNode());
         }
     },
     /**
@@ -212,6 +266,13 @@ options.registry.gallery = options.Class.extend({
         this.$target
             .removeClass('o_nomode o_masonry o_grid o_slideshow')
             .addClass('o_' + widgetValue);
+        // Used to prevent the editor's "unbreakable protection mechanism" from
+        // restoring Image Wall adaptations (images removed > new images added
+        // to the container & layout updates) when adding new images to the
+        // snippet.
+        if (this.options.wysiwyg) {
+            this.options.wysiwyg.odooEditor.unbreakableStepUnactive();
+        }
         this[widgetValue]();
         this.trigger_up('cover_update');
         this._refreshPublicWidgets();
@@ -242,7 +303,7 @@ options.registry.gallery = options.Class.extend({
      */
     removeAllImages: function (previewMode) {
         var $addImg = $('<div>', {
-            class: 'alert alert-info css_editable_mode_display text-center',
+            class: 'alert alert-info css_non_editable_mode_hidden text-center',
         });
         var $text = $('<span>', {
             class: 'o_add_images',
@@ -265,7 +326,7 @@ options.registry.gallery = options.Class.extend({
             src: img.getAttribute('src'),
             alt: img.getAttribute('alt'),
         }));
-        var currentInterval = this.$target.find('.carousel:first').attr('data-interval');
+        var currentInterval = this.$target.find('.carousel:first').attr('data-bs-interval');
         var params = {
             images: images,
             index: 0,
@@ -298,12 +359,28 @@ options.registry.gallery = options.Class.extend({
      */
     notify: function (name, data) {
         this._super(...arguments);
+        // TODO Remove in master.
+        if (!this.hasAddImages) {
+            // In stable, the widget is instanciated twice. We do not want
+            // operations, especially moves, to be performed twice.
+            // We therefore ignore the requests from one of the instances.
+            return;
+        }
+        // TODO In master: nest in a snippet_edition_request to await mode.
         if (name === 'image_removed') {
             data.$image.remove(); // Force the removal of the image before reset
-            this.mode('reset', this.getMode());
+            // TODO In master: use async mode.
+            this.trigger_up('snippet_edition_request', {exec: () => {
+                return this._modeWithImageWait('reset', this.getMode());
+            }});
         } else if (name === 'image_index_request') {
             var imgs = this._getImages();
             var position = _.indexOf(imgs, data.$image[0]);
+            if (position === 0 && data.position === "prev") {
+                data.position = "last";
+            } else if (position === imgs.length - 1 && data.position === "next") {
+                data.position = "first";
+            }
             imgs.splice(position, 1);
             switch (data.position) {
                 case 'first':
@@ -327,24 +404,29 @@ options.registry.gallery = options.Class.extend({
                 $(img).attr('data-index', index);
             });
             const currentMode = this.getMode();
-            this.mode('reset', currentMode);
-            if (currentMode === 'slideshow') {
-                const $carousel = this.$target.find('.carousel');
-                $carousel.removeClass('slide');
-                $carousel.carousel(position);
-                this.$target.find('.carousel-indicators li').removeClass('active');
-                this.$target.find('.carousel-indicators li[data-slide-to="' + position + '"]').addClass('active');
-                this.trigger_up('activate_snippet', {
-                    $snippet: this.$target.find('.carousel-item.active img'),
-                    ifInactiveOptions: true,
+            // TODO In master: use async mode.
+            this.trigger_up('snippet_edition_request', {exec: () => {
+                return this._modeWithImageWait('reset', currentMode).then(() => {
+                    if (currentMode === 'slideshow') {
+                        const $carousel = this.$target.find('.carousel');
+                        $carousel.removeClass('slide');
+                        $carousel.carousel(position);
+                        this.$target.find('.carousel-indicators li').removeClass('active');
+                        this.$target.find('.carousel-indicators li[data-bs-slide-to="' + position + '"]').addClass('active');
+                        this.trigger_up('activate_snippet', {
+                            $snippet: this.$target.find('.carousel-item.active img'),
+                            ifInactiveOptions: true,
+                        });
+                        $carousel.addClass('slide');
+                    } else {
+                        const imageEl = this.$target[0].querySelector(`[data-index='${position}']`);
+                        this.trigger_up('activate_snippet', {
+                            $snippet: $(imageEl),
+                            ifInactiveOptions: true,
+                        });
+                    }
                 });
-                $carousel.addClass('slide');
-            } else {
-                this.trigger_up('activate_snippet', {
-                    $snippet: data.$image,
-                    ifInactiveOptions: true,
-                });
-            }
+            }});
         }
     },
 
@@ -358,10 +440,10 @@ options.registry.gallery = options.Class.extend({
     _adaptNavigationIDs: function () {
         var uuid = new Date().getTime();
         this.$target.find('.carousel').attr('id', 'slideshow_' + uuid);
-        _.each(this.$target.find('[data-slide], [data-slide-to]'), function (el) {
+        _.each(this.$target.find('[data-bs-slide], [data-bs-slide-to]'), function (el) {
             var $el = $(el);
-            if ($el.attr('data-target')) {
-                $el.attr('data-target', '#slideshow_' + uuid);
+            if ($el.attr('data-bs-target')) {
+                $el.attr('data-bs-target', '#slideshow_' + uuid);
             } else if ($el.attr('href')) {
                 $el.attr('href', '#slideshow_' + uuid);
             }
@@ -443,6 +525,25 @@ options.registry.gallery = options.Class.extend({
         $container.empty().append($content);
         return $container;
     },
+    /**
+     * Call mode while ensuring that all images are loaded.
+     *
+     * @see this.selectClass for parameters
+     * @returns {Promise}
+     */
+    _modeWithImageWait(previewMode, widgetValue, params) {
+        // TODO Remove in master.
+        let promise;
+        this._masonryAwaitImages = true;
+        try {
+            this.mode(previewMode, widgetValue, params);
+            promise = this._masonryAwaitImagesPromise;
+        } finally {
+            this._masonryAwaitImages = false;
+            this._masonryAwaitImagesPromise = undefined;
+        }
+        return promise || Promise.resolve();
+    },
 });
 
 options.registry.gallery_img = options.Class.extend({
