@@ -14,6 +14,7 @@ class EventRegistrationAnswer(models.Model):
     """ Represents the user input answer for a single event.question """
     _name = 'event.registration.answer'
     _description = 'Event Registration Answer'
+    _rec_names_search = ['value_answer_id', 'value_text_box']
 
     question_id = fields.Many2one(
         'event.question', ondelete='restrict', required=True,
@@ -28,3 +29,12 @@ class EventRegistrationAnswer(models.Model):
     _sql_constraints = [
         ('value_check', "CHECK(value_answer_id IS NOT NULL OR COALESCE(value_text_box, '') <> '')", "There must be a suggested value or a text value.")
     ]
+
+    # for displaying selected answers by attendees in attendees list view
+    def name_get(self):
+        return [
+            (reg.id,
+             reg.value_answer_id.name if reg.question_type == "simple_choice" else reg.value_text_box
+             )
+            for reg in self
+        ]
