@@ -11,13 +11,16 @@ import werkzeug.wrappers
 from datetime import datetime
 
 from odoo import http, tools, _
+from odoo.exceptions import AccessError
 from odoo.addons.http_routing.models.ir_http import slug
 from odoo.addons.website.models.ir_http import sitemap_qs2dom
 from odoo.addons.website_profile.controllers.main import WebsiteProfile
 from odoo.addons.portal.controllers.portal import _build_url_w_params
 
+from odoo.exceptions import UserError
 from odoo.http import request
 
+
 _logger = logging.getLogger(__name__)
 
 
@@ -35,7 +38,19 @@ class WebsiteForum(WebsiteProfile):
         if kwargs.get('forum'):
             values['forum'] = kwargs.get('forum')
         elif kwargs.get('forum_id'):
-            values['forum'] = request.env['forum.forum'].browse(kwargs.pop('forum_id'))
+            values['forum'] = request.env['forum.forum'].browse(int(kwargs.pop('forum_id')))
+        return values
+
+    def _prepare_mark_as_offensive_values(self, post, **kwargs):
+        offensive_reasons = request.env['forum.post.reason'].search([('reason_type', '=', 'offensive')])
+
+        values = self._prepare_user_values(**kwargs)
+        values.update({
+            'question': post,
+            'forum': post.forum_id,
+            'reasons': offensive_reasons,
+            'offensive': True,
+        })
         return values
 
     # Forum
@@ -46,35 +61,12 @@ class WebsiteForum(WebsiteProfile):
         domain = request.website.website_domain()
         forums = request.env['forum.forum'].search(domain)
         if len(forums) == 1:
-            return werkzeug.utils.redirect('/forum/%s' % slug(forums[0]), code=302)
+            return request.redirect('/forum/%s' % slug(forums[0]), code=302)
 
         return request.render("website_forum.forum_all", {
             'forums': forums
         })
 
-    @http.route('/forum/new', type='json', auth="user", methods=['POST'], website=True)
-    def forum_create(self, forum_name="New Forum", forum_mode="questions", forum_privacy="public", forum_privacy_group=False, add_menu=False):
-        forum = {
-            'name': forum_name,
-            'mode': forum_mode,
-            'privacy': forum_privacy,
-            'website_id': request.website.id,
-        }
-        if forum_privacy == 'private' and forum_privacy_group:
-            forum['authorized_group_id'] = forum_privacy_group
-        forum_id = request.env['forum.forum'].create(forum)
-        if add_menu:
-            group = [int(forum_privacy_group)] if forum_privacy == 'private' else [request.env.ref('base.group_portal').id, request.env.ref('base.group_user').id]
-            menu_id = request.env['website.menu'].create({
-                'name': forum_name,
-                'url': "/forum/%s" % slug(forum_id),
-                'parent_id': request.website.menu_id.id,
-                'website_id': request.website.id,
-                'group_ids': [(6, 0, group)]
-            })
-            forum_id.menu_id = menu_id
-        return "/forum/%s" % slug(forum_id)
-
     def sitemap_forum(env, rule, qs):
         Forum = env['forum.forum']
         dom = sitemap_qs2dom(qs, '/forum', Forum._rec_name)
@@ -90,46 +82,36 @@ class WebsiteForum(WebsiteProfile):
                  '''/forum/<model("forum.forum"):forum>/tag/<model("forum.tag"):tag>/questions/page/<int:page>''',
                  ], type='http', auth="public", website=True, sitemap=sitemap_forum)
     def questions(self, forum, tag=None, page=1, filters='all', my=None, sorting=None, search='', **post):
-        if not forum.can_access_from_current_website():
-            raise werkzeug.exceptions.NotFound()
-
         Post = request.env['forum.post']
 
-        domain = [('forum_id', '=', forum.id), ('parent_id', '=', False), ('state', '=', 'active'), ('can_view', '=', True)]
-        if search:
-            domain += ['|', ('name', 'ilike', search), ('content', 'ilike', search)]
-        if tag:
-            domain += [('tag_ids', 'in', tag.id)]
-        if filters == 'unanswered':
-            domain += [('child_ids', '=', False)]
-        elif filters == 'solved':
-            domain += [('has_validated_answer', '=', True)]
-        elif filters == 'unsolved':
-            domain += [('has_validated_answer', '=', False)]
-
-        user = request.env.user
-
-        if my == 'mine':
-            domain += [('create_uid', '=', user.id)]
-        elif my == 'followed':
-            domain += [('message_partner_ids', '=', user.partner_id.id)]
-        elif my == 'tagged':
-            domain += [('tag_ids.message_partner_ids', '=', user.partner_id.id)]
-        elif my == 'favourites':
-            domain += [('favourite_ids', '=', user.id)]
-
         if sorting:
             # check that sorting is valid
             # retro-compatibily for V8 and google links
             try:
+                sorting = werkzeug.urls.url_unquote_plus(sorting)
                 Post._generate_order_by(sorting, None)
-            except ValueError:
+            except (UserError, ValueError):
                 sorting = False
 
         if not sorting:
             sorting = forum.default_order
 
-        question_count = Post.search_count(domain)
+        options = {
+            'displayDescription': False,
+            'displayDetail': False,
+            'displayExtraDetail': False,
+            'displayExtraLink': False,
+            'displayImage': False,
+            'allowFuzzy': not post.get('noFuzzy'),
+            'forum': str(forum.id) if forum else None,
+            'tag': str(tag.id) if tag else None,
+            'filters': filters,
+            'my': my,
+        }
+        question_count, details, fuzzy_search_term = request.website._search_with_fuzzy("forum_posts_only", search,
+            limit=page * self._post_per_page, order=sorting, options=options)
+        question_ids = details[0].get('results', Post)
+        question_ids = question_ids[(page - 1) * self._post_per_page:page * self._post_per_page]
 
         if tag:
             url = "/forum/%s/tag/%s/questions" % (slug(forum), slug(tag))
@@ -145,24 +127,24 @@ class WebsiteForum(WebsiteProfile):
             url_args['filters'] = filters
         if my:
             url_args['my'] = my
-        pager = request.website.pager(url=url, total=question_count, page=page,
+        pager = tools.lazy(lambda: request.website.pager(url=url, total=question_count, page=page,
                                       step=self._post_per_page, scope=self._post_per_page,
-                                      url_args=url_args)
-
-        question_ids = Post.search(domain, limit=self._post_per_page, offset=pager['offset'], order=sorting)
+                                      url_args=url_args))
 
         values = self._prepare_user_values(forum=forum, searches=post, header={'ask_hide': not forum.active})
         values.update({
             'main_object': tag or forum,
-            'edit_in_backend': not tag,
+            'edit_in_backend': True,
             'question_ids': question_ids,
             'question_count': question_count,
+            'search_count': question_count,
             'pager': pager,
             'tag': tag,
             'filters': filters,
             'my': my,
             'sorting': sorting,
-            'search': search,
+            'search': fuzzy_search_term or search,
+            'original_search': fuzzy_search_term and search,
         })
         return request.render("website_forum.forum_index", values)
 
@@ -177,13 +159,16 @@ class WebsiteForum(WebsiteProfile):
         return request.render("website_forum.faq_karma", values)
 
     @http.route('/forum/get_tags', type='http', auth="public", methods=['GET'], website=True, sitemap=False)
-    def tag_read(self, query='', limit=25, **post):
+    def tag_read(self, forum_id, query='', limit=25, **post):
         data = request.env['forum.tag'].search_read(
-            domain=[('name', '=ilike', (query or '') + "%")],
+            domain=[('forum_id', '=', int(forum_id)), ('name', '=ilike', (query or '') + "%")],
             fields=['id', 'name'],
             limit=int(limit),
         )
-        return json.dumps(data)
+        return request.make_response(
+            json.dumps(data),
+            headers=[("Content-Type", "application/json")]
+        )
 
     @http.route(['/forum/<model("forum.forum"):forum>/tag', '/forum/<model("forum.forum"):forum>/tag/<string:tag_char>'], type='http', auth="public", website=True, sitemap=False)
     def tags(self, forum, tag_char=None, **post):
@@ -247,11 +232,12 @@ class WebsiteForum(WebsiteProfile):
 
         if question.parent_id:
             redirect_url = "/forum/%s/%s" % (slug(forum), slug(question.parent_id))
-            return werkzeug.utils.redirect(redirect_url, 301)
+            return request.redirect(redirect_url, 301)
         filters = 'question'
         values = self._prepare_user_values(forum=forum, searches=post)
         values.update({
             'main_object': question,
+            'edit_in_backend': True,
             'question': question,
             'can_bump': (question.forum_id.allow_bump and not question.child_count and (datetime.today() - question.write_date).days > 9),
             'header': {'question_data': True},
@@ -259,7 +245,7 @@ class WebsiteForum(WebsiteProfile):
             'reversed': reversed,
         })
         if (request.httprequest.referrer or "").startswith(request.httprequest.url_root):
-            values['back_button_url'] = request.httprequest.referrer
+            values['has_back_button_url'] = True
 
         # increment view counter
         question.sudo()._set_viewed()
@@ -297,27 +283,27 @@ class WebsiteForum(WebsiteProfile):
             if record.create_uid.id == request.uid:
                 answer = record
                 break
-        return werkzeug.utils.redirect("/forum/%s/post/%s/edit" % (slug(forum), slug(answer)))
+        return request.redirect("/forum/%s/post/%s/edit" % (slug(forum), slug(answer)))
 
     @http.route('/forum/<model("forum.forum"):forum>/question/<model("forum.post"):question>/close', type='http', auth="user", methods=['POST'], website=True)
     def question_close(self, forum, question, **post):
         question.close(reason_id=int(post.get('reason_id', False)))
-        return werkzeug.utils.redirect("/forum/%s/question/%s" % (slug(forum), slug(question)))
+        return request.redirect("/forum/%s/question/%s" % (slug(forum), slug(question)))
 
     @http.route('/forum/<model("forum.forum"):forum>/question/<model("forum.post"):question>/reopen', type='http', auth="user", methods=['POST'], website=True)
     def question_reopen(self, forum, question, **kwarg):
         question.reopen()
-        return werkzeug.utils.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
+        return request.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
 
     @http.route('/forum/<model("forum.forum"):forum>/question/<model("forum.post"):question>/delete', type='http', auth="user", methods=['POST'], website=True)
     def question_delete(self, forum, question, **kwarg):
         question.active = False
-        return werkzeug.utils.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
+        return request.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
 
     @http.route('/forum/<model("forum.forum"):forum>/question/<model("forum.post"):question>/undelete', type='http', auth="user", methods=['POST'], website=True)
     def question_undelete(self, forum, question, **kwarg):
         question.active = True
-        return werkzeug.utils.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
+        return request.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
 
     # Post
     # --------------------------------------------------
@@ -325,7 +311,7 @@ class WebsiteForum(WebsiteProfile):
     def forum_post(self, forum, **post):
         user = request.env.user
         if not user.email or not tools.single_email_re.match(user.email):
-            return werkzeug.utils.redirect("/forum/%s/user/%s/edit?email_required=1" % (slug(forum), request.session.uid))
+            return request.redirect("/forum/%s/user/%s/edit?email_required=1" % (slug(forum), request.session.uid))
         values = self._prepare_user_values(forum=forum, searches={}, header={'ask_hide': True}, new_question=True)
         return request.render("website_forum.new_question", values)
 
@@ -342,7 +328,7 @@ class WebsiteForum(WebsiteProfile):
         post_tag_ids = forum._tag_to_write_vals(post.get('post_tags', ''))
 
         if request.env.user.forum_waiting_posts_count:
-            return werkzeug.utils.redirect("/forum/%s/ask" % slug(forum))
+            return request.redirect("/forum/%s/ask" % slug(forum))
 
         new_question = request.env['forum.post'].create({
             'forum_id': forum.id,
@@ -351,7 +337,7 @@ class WebsiteForum(WebsiteProfile):
             'parent_id': post_parent and post_parent.id or False,
             'tag_ids': post_tag_ids
         })
-        return werkzeug.utils.redirect("/forum/%s/%s" % (slug(forum), post_parent and slug(post_parent) or new_question.id))
+        return request.redirect("/forum/%s/%s" % (slug(forum), post_parent and slug(post_parent) or new_question.id))
 
     @http.route('/forum/<model("forum.forum"):forum>/post/<model("forum.post"):post>/comment', type='http', auth="user", methods=['POST'], website=True)
     def post_comment(self, forum, post, **kwargs):
@@ -363,7 +349,7 @@ class WebsiteForum(WebsiteProfile):
                 body=body,
                 message_type='comment',
                 subtype_xmlid='mail.mt_comment')
-        return werkzeug.utils.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
+        return request.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
 
     @http.route('/forum/<model("forum.forum"):forum>/post/<model("forum.post"):post>/toggle_correct', type='json', auth="public", website=True)
     def post_toggle_correct(self, forum, post, **kwargs):
@@ -382,8 +368,8 @@ class WebsiteForum(WebsiteProfile):
         question = post.parent_id
         post.unlink()
         if question:
-            werkzeug.utils.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
-        return werkzeug.utils.redirect("/forum/%s" % slug(forum))
+            request.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
+        return request.redirect("/forum/%s" % slug(forum))
 
     @http.route('/forum/<model("forum.forum"):forum>/post/<model("forum.post"):post>/edit', type='http', auth="user", website=True)
     def post_edit(self, forum, post, **kwargs):
@@ -417,7 +403,7 @@ class WebsiteForum(WebsiteProfile):
         vals['tag_ids'] = forum._tag_to_write_vals(kwargs.get('post_tags', ''))
         post.write(vals)
         question = post.parent_id if post.parent_id else post
-        return werkzeug.utils.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
+        return request.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
 
     #  JSON utilities
     # --------------------------------------------------
@@ -515,7 +501,7 @@ class WebsiteForum(WebsiteProfile):
         elif post.state == 'offensive':
             url = "/forum/%s/offensive_posts" % (slug(forum))
         post.validate()
-        return werkzeug.utils.redirect(url)
+        return request.redirect(url)
 
     @http.route('/forum/<model("forum.forum"):forum>/post/<model("forum.post"):post>/refuse', type='http', auth="user", website=True)
     def post_refuse(self, forum, post, **kwargs):
@@ -528,17 +514,18 @@ class WebsiteForum(WebsiteProfile):
             return {'error': 'anonymous_user'}
         return post.flag()[0]
 
-    @http.route('/forum/<model("forum.forum"):forum>/post/<model("forum.post"):post>/ask_for_mark_as_offensive', type='http', auth="user", methods=['GET'], website=True)
-    def post_ask_for_mark_as_offensive(self, forum, post, **kwargs):
-        offensive_reasons = request.env['forum.post.reason'].search([('reason_type', '=', 'offensive')])
+    @http.route('/forum/<model("forum.post"):post>/ask_for_mark_as_offensive', type='json', auth="user", website=True)
+    def post_json_ask_for_mark_as_offensive(self, post, **kwargs):
+        if not post.can_moderate:
+            raise AccessError(_('%d karma required to mark a post as offensive.', post.forum_id.karma_moderate))
+        values = self._prepare_mark_as_offensive_values(post, **kwargs)
+        return request.env['ir.ui.view']._render_template('website_forum.mark_as_offensive', values)
 
-        values = self._prepare_user_values(forum=forum)
-        values.update({
-            'question': post,
-            'forum': forum,
-            'reasons': offensive_reasons,
-            'offensive': True,
-        })
+    @http.route('/forum/<model("forum.forum"):forum>/post/<model("forum.post"):post>/ask_for_mark_as_offensive', type='http', auth="user", methods=['GET'], website=True)
+    def post_http_ask_for_mark_as_offensive(self, forum, post, **kwargs):
+        if not post.can_moderate:
+            raise AccessError(_('%d karma required to mark a post as offensive.', forum.karma_moderate))
+        values = self._prepare_mark_as_offensive_values(post, **kwargs)
         return request.render("website_forum.close_post", values)
 
     @http.route('/forum/<model("forum.forum"):forum>/post/<model("forum.post"):post>/mark_as_offensive', type='http', auth="user", methods=["POST"], website=True)
@@ -549,7 +536,7 @@ class WebsiteForum(WebsiteProfile):
             url = "/forum/%s/%s/#answer-%s" % (slug(forum), post.parent_id.id, post.id)
         else:
             url = "/forum/%s/%s" % (slug(forum), slug(post))
-        return werkzeug.utils.redirect(url)
+        return request.redirect(url)
 
     # User
     # --------------------------------------------------
@@ -558,15 +545,15 @@ class WebsiteForum(WebsiteProfile):
         if partner_id:
             partner = request.env['res.partner'].sudo().search([('id', '=', partner_id)])
             if partner and partner.user_ids:
-                return werkzeug.utils.redirect("/forum/%s/user/%d" % (slug(forum), partner.user_ids[0].id))
-        return werkzeug.utils.redirect("/forum/%s" % slug(forum))
+                return request.redirect("/forum/%s/user/%d" % (slug(forum), partner.user_ids[0].id))
+        return request.redirect("/forum/%s" % slug(forum))
 
     # Profile
     # -----------------------------------
 
     @http.route(['/forum/<model("forum.forum"):forum>/user/<int:user_id>'], type='http', auth="public", website=True)
-    def view_user_forum_profile(self, forum, user_id, forum_origin, **post):
-        return werkzeug.utils.redirect('/profile/user/' + str(user_id) + '?forum_id=' + str(forum.id) + '&forum_origin=' + str(forum_origin))
+    def view_user_forum_profile(self, forum, user_id, forum_origin='/forum', **post):
+        return request.redirect('/profile/user/' + str(user_id) + '?forum_id=' + str(forum.id) + '&forum_origin=' + str(forum_origin))
 
     def _prepare_user_profile_values(self, user, **post):
         values = super(WebsiteForum, self)._prepare_user_profile_values(user, **post)
@@ -636,10 +623,10 @@ class WebsiteForum(WebsiteProfile):
                 down_votes = rec['vote_count']
 
         # Votes which given by users on others questions and answers.
-        vote_ids = Vote.search([('user_id', '=', user.id)])
+        vote_ids = Vote.search([('user_id', '=', user.id), ('forum_id', 'in', forums.ids)])
 
         # activity by user.
-        model, comment = Data.get_object_reference('mail', 'mt_comment')
+        comment = Data._xmlid_lookup('mail.mt_comment')[2]
         activities = Activity.search(
             [('res_id', 'in', (user_question_ids + user_answer_ids).ids), ('model', '=', 'forum.post'),
              ('subtype_id', '!=', comment)],
@@ -686,17 +673,17 @@ class WebsiteForum(WebsiteProfile):
     def convert_comment_to_answer(self, forum, post, comment, **kwarg):
         post = request.env['forum.post'].convert_comment_to_answer(comment.id)
         if not post:
-            return werkzeug.utils.redirect("/forum/%s" % slug(forum))
+            return request.redirect("/forum/%s" % slug(forum))
         question = post.parent_id if post.parent_id else post
-        return werkzeug.utils.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
+        return request.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
 
     @http.route('/forum/<model("forum.forum"):forum>/post/<model("forum.post"):post>/convert_to_comment', type='http', auth="user", methods=['POST'], website=True)
     def convert_answer_to_comment(self, forum, post, **kwarg):
         question = post.parent_id
         new_msg = post.convert_answer_to_comment()
         if not new_msg:
-            return werkzeug.utils.redirect("/forum/%s" % slug(forum))
-        return werkzeug.utils.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
+            return request.redirect("/forum/%s" % slug(forum))
+        return request.redirect("/forum/%s/%s" % (slug(forum), slug(question)))
 
     @http.route('/forum/<model("forum.forum"):forum>/post/<model("forum.post"):post>/comment/<model("mail.message"):comment>/delete', type='json', auth="user", website=True)
     def delete_comment(self, forum, post, comment, **kwarg):
