@@ -8,7 +8,7 @@
     'version': '1.0',
     'summary': 'Manage your online hiring process',
     'description': "This module allows to publish your available job positions on your website and keep track of application submissions easily. It comes as an add-on of *Recruitment* app.",
-    'depends': ['website_partner', 'hr_recruitment', 'website_mail', 'website_form'],
+    'depends': ['hr_recruitment', 'website_mail'],
     'data': [
         'security/ir.model.access.csv',
         'security/website_hr_recruitment_security.xml',
@@ -16,11 +16,26 @@
         'views/website_hr_recruitment_templates.xml',
         'views/hr_recruitment_views.xml',
         'views/hr_job_views.xml',
+        'views/website_pages_views.xml',
+        'views/snippets.xml',
     ],
     'demo': [
         'data/hr_job_demo.xml',
     ],
     'installable': True,
     'application': True,
+    'auto_install': ['hr_recruitment', 'website_mail'],
+    'assets': {
+        'web.assets_frontend': [
+            'website_hr_recruitment/static/src/scss/**/*',
+        ],
+        'website.assets_editor': [
+            'website_hr_recruitment/static/src/js/systray_items/new_content.js',
+            'website_hr_recruitment/static/src/js/website_hr_recruitment_editor.js',
+        ],
+        'web.assets_tests': [
+            'website_hr_recruitment/static/tests/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
