@@ -2,30 +2,37 @@ odoo.define('website_hr_recruitment.form', function (require) {
 'use strict';
 
 var core = require('web.core');
-var FormEditorRegistry = require('website_form.form_editor_registry');
+var FormEditorRegistry = require('website.form_editor_registry');
 
-var _t = core._t;
+const _lt = core._lt;
 
 FormEditorRegistry.add('apply_job', {
     formFields: [{
         type: 'char',
         modelRequired: true,
         name: 'partner_name',
-        string: 'Your Name',
+        fillWith: 'name',
+        string: _lt('Your Name'),
     }, {
         type: 'email',
         required: true,
+        fillWith: 'email',
         name: 'email_from',
-        string: 'Your Email',
+        string: _lt('Your Email'),
     }, {
         type: 'char',
         required: true,
-        name: 'partner_phone',
-        string: 'Phone Number',
+        fillWith: 'phone',
+        name: 'partner_mobile',
+        string: _lt('Phone Number'),
+    }, {
+        type: 'char',
+        name: 'linkedin_profile',
+        string: _lt('LinkedIn Profile'),
     }, {
         type: 'text',
         name: 'description',
-        string: 'Short Introduction',
+        string: _lt('Short Introduction'),
     }, {
         type: 'binary',
         custom: true,
@@ -35,12 +42,12 @@ FormEditorRegistry.add('apply_job', {
         name: 'job_id',
         type: 'many2one',
         relation: 'hr.job',
-        string: _t('Applied Job'),
+        string: _lt('Applied Job'),
     }, {
         name: 'department_id',
         type: 'many2one',
         relation: 'hr.department',
-        string: _t('Department'),
+        string: _lt('Department'),
     }],
     successPage: '/job-thank-you',
 });
