@@ -13,8 +13,8 @@ class WebsitePartnerPage(http.Controller):
         _, partner_id = unslug(partner_id)
         if partner_id:
             partner_sudo = request.env['res.partner'].sudo().browse(partner_id)
-            is_website_publisher = request.env['res.users'].has_group('website.group_website_publisher')
-            if partner_sudo.exists() and (partner_sudo.website_published or is_website_publisher):
+            is_website_restricted_editor = request.env['res.users'].has_group('website.group_website_restricted_editor')
+            if partner_sudo.exists() and (partner_sudo.website_published or is_website_restricted_editor):
                 values = {
                     'main_object': partner_sudo,
                     'partner': partner_sudo,
