@@ -1,11 +1,20 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-from odoo import api, SUPERUSER_ID
+from odoo import api, SUPERUSER_ID, _
 from . import controllers
 from . import models
 from . import wizard
 from . import report
 
+def _post_init_hook(cr, registry):
+    env = api.Environment(cr, SUPERUSER_ID, {})
+    terms_conditions = env['ir.config_parameter'].get_param('account.use_invoice_terms')
+    if not terms_conditions:
+        env['ir.config_parameter'].set_param('account.use_invoice_terms', True)
+    companies = env['res.company'].search([])
+    for company in companies:
+        company.terms_type = 'html'
+    env['website'].search([]).auth_signup_uninvited = 'b2c'
 
 def uninstall_hook(cr, registry):
     ''' Need to reenable the `product` pricelist multi-company rule that were
