@@ -2,6 +2,7 @@ odoo.define('website_sale.tour_shop_dynamic_variants', function (require) {
 'use strict';
 
 var tour = require('web_tour.tour');
+const tourUtils = require('website_sale.tour_utils');
 
 // This tour relies on a data created from the python test.
 tour.register('tour_shop_dynamic_variants', {
@@ -27,6 +28,7 @@ tour.register('tour_shop_dynamic_variants', {
         extra_trigger: 'body:has(input[type="hidden"][name="product_id"][value=0])',
         trigger: '#add_to_cart',
     },
+        tourUtils.goToCart(),
     {
         content: "check the variant is in the cart",
         trigger: 'td.td-product_name:contains(Dynamic Product (Dynamic Value 2))',
