@@ -1,12 +1,12 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import ir_http
 from . import gamification_challenge
 from . import slide_slide
 from . import slide_question
+from . import slide_embed
 from . import slide_channel
 from . import slide_channel_tag
+from . import slide_slide_resource
 from . import res_config_settings
 from . import website
 from . import res_users
