@@ -1,25 +1,25 @@
-odoo.define('website_slides.quiz', function (require) {
-    'use strict';
+/** @odoo-module **/
 
-    var publicWidget = require('web.public.widget');
-    var Dialog = require('web.Dialog');
-    var core = require('web.core');
-    var session = require('web.session');
+    import publicWidget from 'web.public.widget';
+    import Dialog from 'web.Dialog';
+    import  { qweb as QWeb, _t } from 'web.core';
+    import session from 'web.session';
+    import { Markup } from 'web.utils';
+    import CourseJoin from '@website_slides/js/slides_course_join';
+    import QuestionFormWidget from '@website_slides/js/slides_course_quiz_question_form';
+    import SlideQuizFinishModal from '@website_slides/js/slides_course_quiz_finish';
+    import { SlideCoursePage } from '@website_slides/js/slides_course_page';
 
-    var CourseJoinWidget = require('website_slides.course.join.widget').courseJoinWidget;
-    var QuestionFormWidget = require('website_slides.quiz.question.form');
-    var SlideQuizFinishModal = require('website_slides.quiz.finish');
+    import SlideEnroll from '@website_slides/js/slides_course_enroll_email';
 
-    var SlideEnrollDialog = require('website_slides.course.enroll').slideEnrollDialog;
-
-    var QWeb = core.qweb;
-    var _t = core._t;
+    const CourseJoinWidget = CourseJoin.courseJoinWidget;
+    const SlideEnrollDialog = SlideEnroll.slideEnrollDialog;
 
     /**
      * This widget is responsible of displaying quiz questions and propositions. Submitting the quiz will fetch the
      * correction and decorate the answers according to the result. Error message or modal can be displayed.
      *
-     * This widget can be attached to DOM rendered server-side by `website_slides.slide_type_quiz` or
+     * This widget can be attached to DOM rendered server-side by `website_slides.slide_category_quiz` or
      * used client side (Fullscreen).
      *
      * Triggered events are :
@@ -28,10 +28,6 @@ odoo.define('website_slides.quiz', function (require) {
      */
     var Quiz = publicWidget.Widget.extend({
         template: 'slide.slide.quiz',
-        xmlDependencies: [
-            '/website_slides/static/src/xml/slide_quiz.xml',
-            '/website_slides/static/src/xml/slide_course_join.xml'
-        ],
         events: {
             "click .o_wslides_quiz_answer": '_onAnswerClick',
             "click .o_wslides_js_lesson_quiz_submit": '_submitQuiz',
@@ -92,7 +88,7 @@ odoo.define('website_slides.quiz', function (require) {
          * Overridden to add custom rendering behavior upon start of the widget.
          *
          * If the user has answered the quiz before having joined the course, we check
-         * his answers (saved into his session) here as well.
+         * their answers (saved into their session) here as well.
          *
          * @override
          */
@@ -115,21 +111,25 @@ odoo.define('website_slides.quiz', function (require) {
         // Private
         //--------------------------------------------------------------------------
 
-        _alertShow: function (alertCode) {
+        _showErrorMessage: function (errorCode) {
             var message = _t('There was an error validating this quiz.');
-            if (alertCode === 'slide_quiz_incomplete') {
+            if (errorCode === 'slide_quiz_incomplete') {
                 message = _t('All questions must be answered !');
-            } else if (alertCode === 'slide_quiz_done') {
+            } else if (errorCode === 'slide_quiz_done') {
                 message = _t('This quiz is already done. Retaking it is not possible.');
-            } else if (alertCode === 'public_user') {
+            } else if (errorCode === 'public_user') {
                 message = _t('You must be logged to submit the quiz.');
             }
 
-            this.displayNotification({
-                type: 'warning',
-                message: message,
-                sticky: true
-            });
+            this.$('.o_wslides_js_quiz_submit_error')
+                .removeClass('d-none')
+                .find('.o_wslides_js_quiz_submit_error_text')
+                .text(message);
+        },
+
+        _hideErrorMessage: function () {
+            this.$('.o_wslides_js_quiz_submit_error')
+                .addClass('d-none');
         },
 
         /**
@@ -194,6 +194,7 @@ odoo.define('website_slides.quiz', function (require) {
                     'slide_id': self.slide.id,
                 }
             }).then(function (quiz_data) {
+                self.slide.sessionAnswers = quiz_data.session_answers;
                 self.quiz = {
                     questions: quiz_data.slide_questions || [],
                     questionsCount: quiz_data.slide_questions.length,
@@ -342,35 +343,45 @@ odoo.define('website_slides.quiz', function (require) {
          *
          * @private
          */
-        _submitQuiz: function () {
-            var self = this;
-
-            return this._rpc({
+         async _submitQuiz() {
+            const data = await this._rpc({
                 route: '/slides/slide/quiz/submit',
                 params: {
-                    slide_id: self.slide.id,
+                    slide_id: this.slide.id,
                     answer_ids: this._getQuizAnswers(),
                 }
-            }).then(function (data) {
-                if (data.error) {
-                    self._alertShow(data.error);
-                } else {
-                    self.quiz = _.extend(self.quiz, data);
-                    if (data.completed) {
-                        self._disableAnswers();
-                        new SlideQuizFinishModal(self, {
-                            quiz: self.quiz,
-                            hasNext: self.slide.hasNext,
-                            userId: self.userId
-                        }).open();
-                        self.slide.completed = true;
-                        self.trigger_up('slide_completed', {slide: self.slide, completion: data.channel_completion});
-                    }
-                    self._hideEditOptions();
-                    self._renderAnswersHighlightingAndComments();
-                    self._renderValidationInfo();
-                }
             });
+            if (data.error) {
+                this._showErrorMessage(data.error);
+                return;
+            } else {
+                this._hideErrorMessage();
+            }
+            Object.assign(this.quiz, data);
+            const {rankProgress, completed, channel_completion: completion} = this.quiz;
+            // two of the rankProgress properties are HTML messages, mark if set
+            if ('description' in rankProgress) {
+                rankProgress['description'] = Markup(rankProgress['description'] || '');
+                rankProgress['previous_rank']['motivational'] =
+                    Markup(rankProgress['previous_rank']['motivational'] || '');
+            }
+            if (completed) {
+                this._disableAnswers();
+                new SlideQuizFinishModal(this, {
+                    quiz: this.quiz,
+                    hasNext: this.slide.hasNext,
+                    userId: this.userId
+                }).open();
+                this.slide.completed = true;
+                this.trigger_up('slide_completed', {
+                    slideId: this.slide.id,
+                    channelCompletion: completion,
+                    completed: true,
+                });
+            }
+            this._hideEditOptions();
+            this._renderAnswersHighlightingAndComments();
+            this._renderValidationInfo();
         },
 
         /**
@@ -461,30 +472,25 @@ odoo.define('website_slides.quiz', function (require) {
          * @private
          */
         _saveQuizAnswersToSession: function () {
-            var quizAnswers = this._getQuizAnswers();
-            if (quizAnswers.length === this.quiz.questions.length) {
-                return this._rpc({
-                    route: '/slides/slide/quiz/save_to_session',
-                    params: {
-                        'quiz_answers': {'slide_id': this.slide.id, 'slide_answers': quizAnswers},
-                    }
-                });
-            } else {
-                this._alertShow('slide_quiz_incomplete');
-                return Promise.reject('The quiz is incomplete');
-            }
+            this._hideErrorMessage();
+
+            return this._rpc({
+                route: '/slides/slide/quiz/save_to_session',
+                params: {
+                    'quiz_answers': {'slide_id': this.slide.id, 'slide_answers': this._getQuizAnswers()},
+                }
+            });
         },
         /**
-        * After joining the course, we immediately submit the quiz and get the correction.
-        * This allows a smooth onboarding when the user is logged in and the course is public.
+        * After joining the course, we save the questions in the session
+        * and reload the page to update the view.
         *
         * @private
         */
        _afterJoin: function () {
-            this.isMember = true;
-            this._renderValidationInfo();
-            this._applySessionAnswers();
-            this._submitQuiz();
+            this._saveQuizAnswersToSession().then(() => {
+                window.location.reload();
+            });
        },
 
         /**
@@ -632,10 +638,6 @@ odoo.define('website_slides.quiz', function (require) {
      */
     var ConfirmationDialog = Dialog.extend({
         template: 'slide.quiz.confirm.deletion',
-        xmlDependencies: Dialog.prototype.xmlDependencies.concat(
-            ['/website_slides/static/src/xml/slide_quiz_create.xml']
-        ),
-
         /**
          * @override
          * @param parent
@@ -674,12 +676,11 @@ odoo.define('website_slides.quiz', function (require) {
         }
     });
 
-    publicWidget.registry.websiteSlidesQuizNoFullscreen = publicWidget.Widget.extend({
+    publicWidget.registry.websiteSlidesQuizNoFullscreen = SlideCoursePage.extend({
         selector: '.o_wslides_lesson_main', // selector of complete page, as we need slide content and aside content table
-        custom_events: {
+        custom_events: _.extend({}, SlideCoursePage.prototype.custom_events, {
             slide_go_next: '_onQuizNextSlide',
-            slide_completed: '_onQuizCompleted',
-        },
+        }),
 
         //----------------------------------------------------------------------
         // Public
@@ -690,36 +691,32 @@ odoo.define('website_slides.quiz', function (require) {
          * @param {Object} parent
          */
         start: function () {
-            var self = this;
-            this.quizWidgets = [];
-            var defs = [this._super.apply(this, arguments)];
-            this.$('.o_wslides_js_lesson_quiz').each(function () {
-                var slideData = $(this).data();
-                var channelData = self._extractChannelData(slideData);
+            const ret = this._super(...arguments);
+
+            const $quiz = this.$('.o_wslides_js_lesson_quiz');
+            if ($quiz.length) {
+                const slideData = $quiz.data();
+                const channelData = this._extractChannelData(slideData);
                 slideData.quizData = {
-                    questions: self._extractQuestionsAndAnswers(),
+                    questions: this._extractQuestionsAndAnswers(),
                     sessionAnswers: slideData.sessionAnswers || [],
                     quizKarmaMax: slideData.quizKarmaMax,
                     quizKarmaWon: slideData.quizKarmaWon || 0,
                     quizKarmaGain: slideData.quizKarmaGain,
                     quizAttemptsCount: slideData.quizAttemptsCount,
                 };
-                defs.push(new Quiz(self, slideData, channelData, slideData.quizData).attachTo($(this)));
-            });
-            return Promise.all(defs);
+
+                this.quiz = new Quiz(this, slideData, channelData, slideData.quizData);
+                this.quiz.attachTo($quiz);
+            } else {
+                this.quiz = null;
+            }
+            return ret;
         },
 
         //----------------------------------------------------------------------
         // Handlers
         //---------------------------------------------------------------------
-        _onQuizCompleted: function (ev) {
-            var slide = ev.data.slide;
-            var completion = ev.data.completion;
-            this.$('#o_wslides_lesson_aside_slide_check_' + slide.id).addClass('text-success fa-check').removeClass('text-600 fa-circle-o');
-            // need to use global selector as progress bar is outside this animation widget scope
-            $('.o_wslides_lesson_header .progress-bar').css('width', completion + "%");
-            $('.o_wslides_lesson_header .progress span').text(_.str.sprintf("%s %%", completion));
-        },
         _onQuizNextSlide: function () {
             var url = this.$('.o_wslides_js_lesson_quiz').data('next-slide-url');
             window.location.replace(url);
@@ -729,6 +726,63 @@ odoo.define('website_slides.quiz', function (require) {
         // Private
         //---------------------------------------------------------------------
 
+        /**
+         * Get the slide data from the elements in the DOM.
+         *
+         * We need this overwrite because a documentation in non-fullscreen view
+         * doesn't have the standard "done" button and so in that case the slide
+         * data can not be retrieved.
+         *
+         * @override
+         * @param {Integer} slideId
+         */
+        _getSlide: function (slideId) {
+            const slide = this._super(...arguments);
+            if (slide) {
+                return slide;
+            }
+            // A quiz in a documentation on non fullscreen view
+            return $(`.o_wslides_js_lesson_quiz[data-id="${slideId}"`).data();
+        },
+
+        /**
+         * After a slide has been marked as completed / uncompleted, update the state
+         * of this widget and reload the slide if needed (e.g. to re-show the questions
+         * of a quiz).
+         *
+         * @override
+         * @param {Object} slide
+         * @param {Boolean} completed
+         */
+        toggleCompletionButton: function (slide, completed = true) {
+            this._super(...arguments);
+
+            if (this.quiz && this.quiz.slide.id === slide.id && !completed && this.quiz.quiz.questionsCount) {
+                // The quiz has been marked as "Not Done", re-load the questions
+                this.quiz.quiz.answers = null;
+                this.quiz.quiz.sessionAnswers = null;
+                this.quiz.slide.completed = false;
+                this.quiz._fetchQuiz().then(() => {
+                    this.quiz.renderElement();
+                    this.quiz._renderValidationInfo();
+                });
+
+            }
+
+            // The quiz has been submitted in a documentation and in non fullscreen view,
+            // should update the button "Mark Done" to "Mark To Do"
+            const $doneButton = $('.o_wslides_done_button');
+            if ($doneButton.length && completed) {
+                $doneButton
+                    .removeClass('o_wslides_done_button disabled btn-primary text-white')
+                    .addClass('o_wslides_undone_button btn-light')
+                    .text(_t('Mark To Do'))
+                    .removeAttr('title')
+                    .removeAttr('aria-disabled')
+                    .attr('href', `/slides/slide/${slide.id}/set_uncompleted`);
+            }
+        },
+
         _extractChannelData: function (slideData) {
             return {
                 channelId: slideData.channelId,
@@ -767,9 +821,6 @@ odoo.define('website_slides.quiz', function (require) {
         },
     });
 
-    return {
-        Quiz: Quiz,
-        ConfirmationDialog: ConfirmationDialog,
-        websiteSlidesQuizNoFullscreen: publicWidget.registry.websiteSlidesQuizNoFullscreen
-    };
-});
+    export var Quiz = Quiz;
+    export var ConfirmationDialog = ConfirmationDialog;
+    export const websiteSlidesQuizNoFullscreen = publicWidget.registry.websiteSlidesQuizNoFullscreen;
