@@ -1,21 +1,21 @@
-odoo.define('website_slides.upload_modal', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var core = require('web.core');
-var Dialog = require('web.Dialog');
-var publicWidget = require('web.public.widget');
-var utils = require('web.utils');
-
-var QWeb = core.qweb;
-var _t = core._t;
+import { qweb as QWeb, _t } from 'web.core';
+import Dialog from 'web.Dialog';
+import publicWidget from 'web.public.widget';
+import utils from 'web.utils';
+import wUtils from 'website.utils';
 
 var SlideUploadDialog = Dialog.extend({
     template: 'website.slide.upload.modal',
     events: _.extend({}, Dialog.prototype.events, {
         'click .o_wslides_js_upload_install_button': '_onClickInstallModule',
-        'click .o_wslides_select_type': '_onClickSlideTypeIcon',
+        'click .o_wslides_select_category': '_onClickSlideCategoryIcon',
         'change input#upload': '_onChangeSlideUpload',
-        'change input#url': '_onChangeSlideUrl',
+        'change input#video_url': '_onChangeVideoUrl',
+        'change input#image_google_url': '_onChangeImageUrl',
+        'change input#document_google_url': '_onChangeDocumentUrl',
+        'change input[name="source_type"]': '_onChangeSourceType',
     }),
 
     /**
@@ -27,20 +27,27 @@ var SlideUploadDialog = Dialog.extend({
      */
     init: function (parent, options) {
         options = _.defaults(options || {}, {
-            title: _t("Upload a document"),
+            title: _t("Add Content"),
             size: 'medium',
         });
         this._super(parent, options);
         this._setup();
 
         this.channelID = parseInt(options.channelId, 10);
-        this.defaultCategoryID = parseInt(options.categoryId,10);
+        this.defaultCategoryID = parseInt(options.categoryId, 10);
         this.canUpload = options.canUpload === 'True';
         this.canPublish = options.canPublish === 'True';
         this.modulesToInstall = options.modulesToInstall ? JSON.parse(options.modulesToInstall.replace(/'/g, '"')) : null;
         this.modulesToInstallStatus = null;
 
-        this.set('state', '_select');
+        if (options.openModal && options.openModal in this.slide_category_data) {
+            // Sets the appropriate category's upload template if one has to be opened on load.
+            this.set('state', options.openModal);
+            this.set('defaultTemplate', this.slide_category_data[options.openModal]['template']);
+        } else {
+            this.set('state', '_select');
+            this.set('defaultTemplate', 'website.slide.upload.modal.select');
+        }
         this.on('change:state', this, this._onChangeType);
         this.set('can_submit_form', false);
         this.on('change:can_submit_form', this, this._onChangeCanSubmitForm);
@@ -51,6 +58,10 @@ var SlideUploadDialog = Dialog.extend({
     start: function () {
         var self = this;
         return this._super.apply(this, arguments).then(function () {
+            if (self.get('state') !== '_select') {
+                self.$modal.find('.modal-dialog').addClass('modal-lg');
+            }
+            self._bindSelect2Dropdown();
             self._resetModalButton();
         });
     },
@@ -63,10 +74,10 @@ var SlideUploadDialog = Dialog.extend({
      * @private
      * @param {string} message
      */
-    _alertDisplay: function (message) {
+    _alertDisplay: function (message, alertClass='alert-warning') {
         this._alertRemove();
         $('<div/>', {
-            "class": 'alert alert-warning',
+            "class": 'alert ' + alertClass,
             id: 'upload-alert',
             role: 'alert'
         }).text(message).insertBefore(this.$('form'));
@@ -102,17 +113,21 @@ var SlideUploadDialog = Dialog.extend({
             });
         }));
     },
-    _fetchUrlPreview: function (url) {
+    _fetchUrlPreview: function (url, slideCategory) {
         return this._rpc({
             route: '/slides/prepare_preview/',
             params: {
                 'url': url,
+                'slide_category': slideCategory,
                 'channel_id': this.channelID
             },
         });
     },
     _formSetFieldValue: function (fieldId, value) {
-        this.$('form').find('#'+fieldId).val(value);
+        var $formField = this.$('form').find('#'+fieldId);
+        if (!$formField.val()) {  // update only if the user did not assign a value manually
+            $formField.val(value);
+        }
     },
     _formGetFieldValue: function (fieldId) {
         return this.$('#'+fieldId).val();
@@ -123,51 +138,53 @@ var SlideUploadDialog = Dialog.extend({
         return form[0].checkValidity() && this.isValidUrl;
     },
     /**
-     * Extract values to submit from form, force the slide_type according to
+     * Extract values to submit from form, force the slide_category according to
      * filled values.
      *
      * @private
      */
-    _formValidateGetValues: function (forcePublished) {
-        var canvas = this.$('#data_canvas')[0];
-        var values = _.extend({
-            'channel_id': this.channelID,
-            'name': this._formGetFieldValue('name'),
-            'url': this._formGetFieldValue('url'),
-            'description': this._formGetFieldValue('description'),
-            'duration': this._formGetFieldValue('duration'),
-            'is_published': forcePublished,
-        }, this._getSelect2DropdownValues()); // add tags and category
-
-        // default slide_type (for webpage for instance)
-        if (_.contains(this.slide_type_data), this.get('state')) {
-            values['slide_type'] = this.get('state');
+    _formValidateGetValues: async function (forcePublished) {
+        var slideCategory = 'document';
+        // default slide_category (for article for instance)
+        if (_.contains(this.slide_category_data), this.get('state')) {
+            slideCategory = this.get('state');
         }
 
+        var sourceType = 'local_file';
+        if (slideCategory === 'video') {
+            sourceType = 'external';  // force external for videos
+        } else {
+            sourceType = this.$('input[name="source_type"]:checked').data('value');
+        }
+        var values = _.extend({
+            'channel_id': this.channelID,
+            'description': this._formGetFieldValue('description'),
+            'document_google_url': this._formGetFieldValue('document_google_url'),
+            'duration': this._formGetFieldValue('duration'),
+            'image_google_url': this._formGetFieldValue('image_google_url'),
+            'is_published': forcePublished,
+            'name': this._formGetFieldValue('name'),
+            'slide_category': slideCategory,
+            'source_type': sourceType,
+            'video_url': this._formGetFieldValue('video_url'),
+        }, this._getSelect2DropdownValues()); // add tags and category
+
+        var canvas = this.$('#data_canvas')[0];
         if (this.file.type === 'application/pdf') {
             _.extend(values, {
                 'image_1920': canvas.toDataURL().split(',')[1],
-                'slide_type': canvas.height > canvas.width ? 'document' : 'presentation',
-                'mime_type': this.file.type,
-                'datas': this.file.data
+                'slide_category': 'document',
+                'binary_content': this.file.data
             });
-        } else if (values['slide_type'] === 'webpage') {
+        } else if (values['slide_category'] === 'article') {
             _.extend(values, {
-                'mime_type': 'text/html',
-                'image_1920': this.file.type === 'image/svg+xml' ? this._svgToPng() : this.file.data,
+                'image_1920': this.file.type === 'image/svg+xml' ? await this._svgToPng() : this.file.data,
             });
         } else if (/^image\/.*/.test(this.file.type)) {
-            if (values['slide_type'] === 'presentation') {
-                _.extend(values, {
-                    'slide_type': 'infographic',
-                    'mime_type': this.file.type === 'image/svg+xml' ? 'image/png' : this.file.type,
-                    'datas': this.file.type === 'image/svg+xml' ? this._svgToPng() : this.file.data
-                });
-            } else {
-                _.extend(values, {
-                    'image_1920': this.file.type === 'image/svg+xml' ? this._svgToPng() : this.file.data,
-                });
-            }
+            _.extend(values, {
+                'slide_category': 'infographic',
+                'binary_content': this.file.type === 'image/svg+xml' ? await this._svgToPNG() : this.file.data,
+            });
         }
         return values;
     },
@@ -189,7 +206,7 @@ var SlideUploadDialog = Dialog.extend({
             if (! this.modulesToInstallStatus.installing) {
                 btnList.push({text: this.modulesToInstallStatus.failed ? _t("Retry") : _t("Install"), classes: 'btn-primary', click: this._onClickInstallModuleConfirm.bind(this)});
             }
-            btnList.push({text: _t("Discard"), classes: 'o_w_slide_go_back', click: this._onClickGoBack.bind(this)});
+            btnList.push({text: _t("Back"), classes: 'o_w_slide_go_back', click: this._onClickGoBack.bind(this)});
         } else if (state !== '_upload') { // no button when uploading
             if (this.canUpload) {
                 if (this.canPublish) {
@@ -199,7 +216,7 @@ var SlideUploadDialog = Dialog.extend({
                     btnList.push({text: _t("Save"), classes: 'btn-primary o_w_slide_upload', click: this._onClickFormSubmit.bind(this)});
                 }
             }
-            btnList.push({text: _t("Discard"), classes: 'o_w_slide_go_back', click: this._onClickGoBack.bind(this)});
+            btnList.push({text: _t("Back"), classes: 'o_w_slide_go_back', click: this._onClickGoBack.bind(this)});
         }
         return btnList;
     },
@@ -268,11 +285,11 @@ var SlideUploadDialog = Dialog.extend({
             formatNoMatches: false,
             selection_data: false,
             fetch_rpc_fnc: fetchFNC,
-            formatSelection: function (data) {
+            formatSelection: function (data, container, fmt) {
                 if (data.tag) {
                     data.text = data.tag;
                 }
-                return data.text;
+                return fmt(data.text);
             },
             createSearchChoice: function (term, data) {
                 var addedTags = $(this.opts.element).select2('data');
@@ -323,24 +340,29 @@ var SlideUploadDialog = Dialog.extend({
         return values;
     },
     /**
-     * Init the data relative to the support slide type to upload
+     * Init the data relative to the support slide category to upload
      *
      * @private
      */
     _setup: function () {
-        this.slide_type_data = {
-            presentation: {
+        this.slide_category_data = {
+            document: {
                 icon: 'fa-file-pdf-o',
-                label: _t('Presentation'),
-                template: 'website.slide.upload.modal.presentation',
+                label: _t('Document'),
+                template: 'website.slide.upload.modal.document',
             },
-            webpage: {
+            infographic: {
+                icon: 'fa-file-image-o',
+                label: _t('Image'),
+                template: 'website.slide.upload.modal.infographic',
+            },
+            article: {
                 icon: 'fa-file-text',
-                label: _t('Web Page'),
-                template: 'website.slide.upload.modal.webpage',
+                label: _t('Article'),
+                template: 'website.slide.upload.modal.article',
             },
             video: {
-                icon: 'fa-video-camera',
+                icon: 'fa-file-video-o',
                 label: _t('Video'),
                 template: 'website.slide.upload.modal.video',
             },
@@ -368,18 +390,92 @@ var SlideUploadDialog = Dialog.extend({
         this.$('.o_slide_preview').addClass('d-none');
     },
     /**
-     * @private
+     * TODO: Remove this part, as now SVG support in image resize tools is
+     * included?
+     * Python PIL does not support SVG, so converting SVG to PNG.
+     *
+     * @returns {Promise<string>}
      */
-    // TODO: Remove this part, as now SVG support in image resize tools is included
-    //Python PIL does not support SVG, so converting SVG to PNG
-    _svgToPng: function () {
-        var img = this.$el.find('img#slide-image')[0];
-        var canvas = document.createElement('canvas');
-        canvas.width = img.width;
-        canvas.height = img.height;
-        canvas.getContext('2d').drawImage(img, 0, 0);
-        return canvas.toDataURL('image/png').split(',')[1];
+    _svgToPNG: async function () {
+        const imgEl = this.$el.find('img#slide-image')[0];
+        const png = await wUtils.svgToPNG(imgEl);
+        return png.split(',')[1];
     },
+
+    /**
+     * When the URL changes for slides of categories infographic, document and video, we attempt to fetch
+     * some metadata on YouTube / Google Drive (such as a name, a title, a duration, ...).
+     *
+     * @param {string} url
+     * @param {string} slideCategory
+     */
+    _updateSlideMetadata: function (url, slideCategory) {
+        var self = this;
+        this._alertRemove();
+        this.isValidUrl = false;
+        this.set('can_submit_form', false);
+        this._setModalLoading(true);
+        this._fetchUrlPreview(url, slideCategory).then(function (data) {
+            self.set('can_submit_form', true);
+            if (data.error) {
+                self._alertDisplay(data.error);
+                self._hidePreviewColumn();
+            } else {
+                if (data.info) {
+                    self._alertDisplay(data.info, 'alert-info');
+                } else {
+                    self._alertRemove();
+                }
+
+                self.isValidUrl = true;
+
+                if (data.name) {
+                    self._formSetFieldValue('name', data.name);
+                    self.$('#slide-video-title')
+                        .text(data.name)
+                        .removeClass('d-none');
+                } else {
+                    self.$('#slide-video-title').addClass('d-none');
+                }
+
+                if (data.description) {
+                    self._formSetFieldValue('description', data.description);
+                }
+                if (data.completion_time) {
+                    // hours to minutes conversion
+                    self._formSetFieldValue('duration', Math.round(data.completion_time * 60));
+                }
+                if (data.image_url) {
+                    self.$('#slide-image').attr('src', data.image_url);
+                }
+
+                if (!data.name && !data.description && !data.image_url) {
+                    self._hidePreviewColumn();
+                } else {
+                    self._showPreviewColumn();
+                }
+            }
+
+            self._setModalLoading(false);
+        });
+    },
+
+    /**
+     * Typically used when loading slide metadata.
+     * Since the request result will change form values, it's better to wait for the return of the
+     * request to avoid having the user type text that will be overridden (such as the slide name
+     * for example).
+     *
+     * @param {boolean} loading true to mask the modal with a loading screen, false to remove it
+     */
+    _setModalLoading: function (loading) {
+        if (loading) {
+            this.$el.closest('.modal-content').append(QWeb.render('website.slide.upload.modal.loading'));
+        } else {
+            this.$el.closest('.modal-content').find('.o_wslides_slide_upload_loading').remove();
+        }
+    },
+
     //--------------------------------------------------------------------------
     // Handler
     //--------------------------------------------------------------------------
@@ -395,7 +491,7 @@ var SlideUploadDialog = Dialog.extend({
         } else if (currentType === '_import') {
             tmpl = 'website.slide.upload.modal.import';
         } else {
-            tmpl = this.slide_type_data[currentType]['template'];
+            tmpl = this.slide_category_data[currentType]['template'];
             this.$modal.find('.modal-dialog').addClass('modal-lg');
         }
         this.$('.o_w_slide_upload_modal_container').empty();
@@ -406,7 +502,7 @@ var SlideUploadDialog = Dialog.extend({
         if (currentType === '_import') {
             this.set_title(_t("New Certification"));
         } else {
-            this.set_title(_t("Upload a document"));
+            this.set_title(_t("Add Content"));
         }
     },
     _onChangeCanSubmitForm: function (ev) {
@@ -416,6 +512,28 @@ var SlideUploadDialog = Dialog.extend({
             this.$('.o_w_slide_upload').button('loading');
         }
     },
+    /**
+     * When the user selects 'local_file' or 'external' as source type, we display the 'upload'
+     * field or the 'document_google_url' / 'image_google_url' fields respectively.
+     * We also toggle the 'required' attribute the same way.
+     *
+     * @param {Event} ev the onchange event
+     */
+    _onChangeSourceType: function (ev) {
+        if (this.$('#source_type_local_file').is(':checked')) {
+            this.$('.o_wslides_js_slide_upload_local_file').removeClass('d-none');
+            this.$('.o_wslides_js_slide_upload_external').addClass('d-none');
+
+            this.$('#upload').attr('required', 'required');
+            this.$('#document_google_url, #image_google_url').removeAttr('required');
+        } else if (this.$('#source_type_external').is(':checked')) {
+            this.$('.o_wslides_js_slide_upload_external').removeClass('d-none');
+            this.$('.o_wslides_js_slide_upload_local_file').addClass('d-none');
+
+            this.$('#document_google_url, #image_google_url').attr('required', 'required');
+            this.$('#upload').removeAttr('required');
+        }
+    },
     _onChangeSlideUpload: function (ev) {
         var self = this;
         this._alertRemove();
@@ -485,7 +603,7 @@ var SlideUploadDialog = Dialog.extend({
                  * In the mean time, this small fix allows not refactoring all of this and can not
                  * cause much harm.
                  */
-                Util = window.pdfjsLib.Util;
+                window.Util = window.pdfjsLib.Util;
                 window.pdfjsLib.getDocument(new Uint8Array(buffer), null, passwordNeeded).then(function getPdf(pdf) {
                     self._formSetFieldValue('duration', (pdf._pdfInfo.numPages || 0) * 5);
                     pdf.getPage(1).then(function getFirstPage(page) {
@@ -521,31 +639,15 @@ var SlideUploadDialog = Dialog.extend({
             }
         }
     },
-    _onChangeSlideUrl: function (ev) {
-        var self = this;
-        var url = $(ev.target).val();
-        this._alertRemove();
-        this.isValidUrl = false;
-        this.set('can_submit_form', false);
-        this._fetchUrlPreview(url).then(function (data) {
-            self.set('can_submit_form', true);
-            if (data.error) {
-                self._alertDisplay(data.error);
-            } else {
-                if (data.completion_time) {
-                    // hours to minutes conversion
-                    self._formSetFieldValue('duration', Math.round(data.completion_time * 60));
-                }
-                self.$('#slide-image').attr('src', data.url_src);
-                self._formSetFieldValue('name', data.title);
-                self._formSetFieldValue('description', data.description);
-
-                self.isValidUrl = true;
-                self._showPreviewColumn();
-            }
-        });
+    _onChangeDocumentUrl: function (ev) {
+        this._updateSlideMetadata($(ev.target).val(), 'document');
+    },
+    _onChangeImageUrl: function (ev) {
+        this._updateSlideMetadata($(ev.target).val(), 'infographic');
+    },
+    _onChangeVideoUrl: function (ev) {
+        this._updateSlideMetadata($(ev.target).val(), 'video');
     },
-
     _onClickInstallModule: function (ev) {
         var $btn = $(ev.currentTarget);
         var moduleId = $btn.data('moduleId');
@@ -577,7 +679,12 @@ var SlideUploadDialog = Dialog.extend({
             method: 'button_immediate_install',
             args: [[this.modulesToInstallStatus.id]],
         }).then(function () {
-            window.location.href = window.location.origin + window.location.pathname + '?enable_slide_upload';
+            let redirectUrl = window.location.origin + window.location.pathname + '?enable_slide_upload';
+            if (self.modulesToInstallStatus.default_slide_category) {
+                redirectUrl += '=';
+                redirectUrl += self.modulesToInstallStatus.default_slide_category;
+            }
+            window.location.href = redirectUrl;
         }, function () {
             $el.text(_.str.sprintf(_t('Failed to install "%s".'), self.modulesToInstallStatus.name));
             self.modulesToInstallStatus.installing = false;
@@ -593,21 +700,27 @@ var SlideUploadDialog = Dialog.extend({
             this.modulesToInstallStatus = null;
         }
     },
-
-    _onClickFormSubmit: function (ev) {
-        var self = this;
-        var $btn = $(ev.currentTarget);
-        if (this._formValidate()) {
-            var values = this._formValidateGetValues($btn.hasClass('o_w_slide_upload_published')); // get info before changing state
-            var oldType = this.get('state');
-            this.set('state', '_upload');
-            return this._rpc({
-                route: '/slides/add_slide',
-                params: values,
-            }).then(function (data) {
-                self._onFormSubmitDone(data, oldType);
-            });
+    /**
+     * @private
+     * @returns {Promise}
+     */
+    _onClickFormSubmit: async function (ev) {
+        if (!this._formValidate()) {
+            return;
         }
+
+        // Get info before changing state
+        const forcePublished = $(ev.currentTarget).hasClass('o_w_slide_upload_published');
+        const values = await this._formValidateGetValues(forcePublished);
+
+        var oldType = this.get('state');
+        this.set('state', '_upload');
+
+        const data = await this._rpc({
+            route: '/slides/add_slide',
+            params: values,
+        });
+        this._onFormSubmitDone(data, oldType);
     },
 
     _onFormSubmitDone: function (data, oldType) {
@@ -615,14 +728,21 @@ var SlideUploadDialog = Dialog.extend({
             this.set('state', oldType);
             this._alertDisplay(data.error);
         } else {
-            window.location = data.url;
+            if (data.url.indexOf('enable_editor') >= 0) {
+                // If we need to enter edit mode, it should be done to the top
+                // window so that we endup refreshing the backend client action
+                // in edit mode.
+                window.top.location = data.url;
+            } else {
+                window.location = data.url;
+            }
         }
     },
 
-    _onClickSlideTypeIcon: function (ev) {
+    _onClickSlideCategoryIcon: function (ev) {
         var $elem = this.$(ev.currentTarget);
-        var slideType = $elem.data('slideType');
-        this.set('state', slideType);
+        var slideCategory = $elem.data('slideCategory');
+        this.set('state', slideCategory);
 
         this._bindSelect2Dropdown();  // rebind select2 at each modal body rendering
     },
@@ -630,19 +750,21 @@ var SlideUploadDialog = Dialog.extend({
 
 publicWidget.registry.websiteSlidesUpload = publicWidget.Widget.extend({
     selector: '.o_wslides_js_slide_upload',
-    xmlDependencies: ['/website_slides/static/src/xml/website_slides_upload.xml'],
     events: {
         'click': '_onUploadClick',
     },
 
     /**
+     * Automatically opens the upload dialog if requested from query string.
+     * If openModal is defined ( === '' ), opens the category selection dialog.
+     * If openModal is a category name, opens the category's upload dialog.
+     *
      * @override
      */
     start: function () {
-        // Automatically open the upload dialog if requested from query string
-        if (this.$el.attr('data-open-modal')) {
-            this.$el.removeAttr('data-open-modal');
+        if ('openModal' in this.$el.data()) {
             this._openDialog(this.$el);
+            this.$el.data('openModal', false);
         }
         return this._super.apply(this, arguments);
     },
@@ -652,8 +774,7 @@ publicWidget.registry.websiteSlidesUpload = publicWidget.Widget.extend({
     //--------------------------------------------------------------------------
 
     _openDialog: function ($element) {
-        var data = $element.data();
-        return new SlideUploadDialog(this, data).open();
+        return new SlideUploadDialog(this, $element.data()).open();
     },
 
     //--------------------------------------------------------------------------
@@ -670,9 +791,7 @@ publicWidget.registry.websiteSlidesUpload = publicWidget.Widget.extend({
     },
 });
 
-return {
+export default {
     SlideUploadDialog: SlideUploadDialog,
     websiteSlidesUpload: publicWidget.registry.websiteSlidesUpload
 };
-
-});
