@@ -3,15 +3,11 @@ odoo.define('website_slides_survey.fullscreen', function (require) {
 
 var core = require('web.core');
 var QWeb = core.qweb;
-var Fullscreen = require('website_slides.fullscreen');
+var Fullscreen = require('@website_slides/js/slides_course_fullscreen_player')[Symbol.for("default")];
 
 Fullscreen.include({
-    xmlDependencies: (Fullscreen.prototype.xmlDependencies || []).concat(
-        ["/website_slides_survey/static/src/xml/website_slides_fullscreen.xml"]
-    ),
-
     /**
-     * Extend the _renderSlide method so that slides of type "certification"
+     * Extend the _renderSlide method so that slides of category "certification"
      * are also taken into account and rendered correctly
      *
      * @private
@@ -20,7 +16,7 @@ Fullscreen.include({
     _renderSlide: function (){
         var def = this._super.apply(this, arguments);
         var $content = this.$('.o_wslides_fs_content');
-        if (this.get('slide').type === "certification"){
+        if (this.get('slide').category === "certification"){
             $content.html(QWeb.render('website.slides.fullscreen.certification',{widget: this}));
         }
         return Promise.all([def]);
