@@ -1,14 +1,16 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+import json
 import functools
 import itertools
 
 import psycopg2
 import pytz
 
-from odoo import api, fields, models, _
-from odoo.tools import ustr
+from odoo import api, Command, fields, models, _
+from odoo.tools import ustr, OrderedSet
+from odoo.tools.translate import code_translations, _lt
 
 REFERENCING_FIELDS = {None, 'id', '.id'}
 def only_ref_fields(record):
@@ -16,13 +18,13 @@ def only_ref_fields(record):
 def exclude_ref_fields(record):
     return {k: v for k, v in record.items() if k not in REFERENCING_FIELDS}
 
-CREATE = lambda values: (0, False, values)
-UPDATE = lambda id, values: (1, id, values)
-DELETE = lambda id: (2, id, False)
-FORGET = lambda id: (3, id, False)
-LINK_TO = lambda id: (4, id, False)
-DELETE_ALL = lambda: (5, False, False)
-REPLACE_WITH = lambda ids: (6, False, ids)
+# these lazy translations promise translations for ['yes', 'no', 'true', 'false']
+BOOLEAN_TRANSLATIONS = (
+    _lt('yes'),
+    _lt('no'),
+    _lt('true'),
+    _lt('false')
+)
 
 class ImportWarning(Warning):
     """ Used to send warnings upwards the stack during the import process """
@@ -31,7 +33,6 @@ class ImportWarning(Warning):
 class ConversionNotFound(ValueError):
     pass
 
-
 class IrFieldsConverter(models.AbstractModel):
     _name = 'ir.fields.converter'
     _description = 'Fields Converter'
@@ -49,6 +50,41 @@ class IrFieldsConverter(models.AbstractModel):
                 error_params = tuple(sanitize(v) for v in error_params)
         return error_type(error_msg % error_params, error_args)
 
+    def _get_import_field_path(self, field, value):
+        """ Rebuild field path for import error attribution to the right field.
+        This method uses the 'parent_fields_hierarchy' context key built during treatment of one2many fields
+        (_str_to_one2many). As the field to import is the last of the chain (child_id/child_id2/field_to_import),
+        we need to retrieve the complete hierarchy in case of error in order to assign the error to the correct
+        column in the import UI.
+
+        :param (str) field: field in which the value will be imported.
+        :param (str or list) value:
+            - str: in most of the case the value we want to import into a field is a string (or a number).
+            - list: when importing into a one2may field, all the records to import are regrouped into a list of dict.
+                E.g.: creating multiple partners: [{None: 'ChildA_1', 'type': 'Private address'}, {None: 'ChildA_2', 'type': 'Private address'}]
+                where 'None' is the name. (because we can find a partner by his name, we don't need to specify the field.)
+
+        The field_path value is computed based on the last field in the chain.
+        for example,
+            - path_field for 'Private address' at childA_1 is ['partner_id', 'type']
+            - path_field for 'childA_1' is ['partner_id']
+
+        So, by retrieving the correct field_path for each value to import, if errors are raised for those fields,
+        we can the link the errors to the correct header-field couple in the import UI.
+        """
+        field_path = [field]
+        parent_fields_hierarchy = self._context.get('parent_fields_hierarchy')
+        if parent_fields_hierarchy:
+            field_path = parent_fields_hierarchy + field_path
+
+        field_path_value = value
+        while isinstance(field_path_value, list):
+            key = list(field_path_value[0].keys())[0]
+            if key:
+                field_path.append(key)
+            field_path_value = field_path_value[0][key]
+        return field_path
+
     @api.model
     def for_model(self, model, fromtype=str):
         """ Returns a converter object for the model. A converter is a
@@ -57,6 +93,7 @@ class IrFieldsConverter(models.AbstractModel):
         records matching what :meth:`odoo.osv.orm.Model.write` expects.
 
         :param model: :class:`odoo.osv.orm.Model` for the conversion base
+        :param fromtype:
         :returns: a converter callable
         :rtype: (record: dict, logger: (field, error) -> None) -> dict
         """
@@ -70,6 +107,7 @@ class IrFieldsConverter(models.AbstractModel):
 
         def fn(record, log):
             converted = {}
+            import_file_context = self.env.context.get('import_file')
             for field, value in record.items():
                 if field in REFERENCING_FIELDS:
                     continue
@@ -84,7 +122,24 @@ class IrFieldsConverter(models.AbstractModel):
                             # uniform handling
                             w = ImportWarning(w)
                         log(field, w)
+                except (UnicodeEncodeError, UnicodeDecodeError) as e:
+                    log(field, ValueError(str(e)))
                 except ValueError as e:
+                    if import_file_context:
+                        # if the error is linked to a matching error, the error is a tuple
+                        # E.g.:("Value X cannot be found for field Y at row 1", {
+                        #   'more_info': {},
+                        #   'value': 'X',
+                        #   'field': 'Y',
+                        #   'field_path': child_id/Y,
+                        # })
+                        # In order to link the error to the correct header-field couple in the import UI, we need to add
+                        # the field path to the additional error info.
+                        # As we raise the deepest child in error, we need to add the field path only for the deepest
+                        # error in the import recursion. (if field_path is given, don't overwrite it)
+                        error_info = len(e.args) > 1 and e.args[1]
+                        if error_info and not error_info.get('field_path'):  # only raise the deepest child in error
+                            error_info['field_path'] = self._get_import_field_path(field, value)
                     log(field, e)
             return converted
 
@@ -122,11 +177,11 @@ class IrFieldsConverter(models.AbstractModel):
         it returns. The handling of a warning at the upper levels is the same
         as ``ValueError`` above.
 
+        :param model:
         :param field: field object to generate a value for
         :type field: :class:`odoo.fields.Field`
         :param fromtype: type to convert to something fitting for ``field``
         :type fromtype: type | str
-        :param context: odoo request context
         :return: a function (fromtype -> field.write_type), if a converter is found
         :rtype: Callable | None
         """
@@ -138,15 +193,25 @@ class IrFieldsConverter(models.AbstractModel):
             return None
         return functools.partial(converter, model, field)
 
+    def _str_to_json(self, model, field, value):
+        try:
+            return json.loads(value), []
+        except ValueError:
+            msg = _("'%s' does not seem to be a valid JSON for field '%%(field)s'")
+            raise self._format_import_error(ValueError, msg, value)
+
+    def _str_to_properties(self, model, field, value):
+        msg = _("Unable to import field type '%s'  ", field.type)
+        raise self._format_import_error(ValueError, msg)
+
     @api.model
     def _str_to_boolean(self, model, field, value):
         # all translatables used for booleans
-        true, yes, false, no = _(u"true"), _(u"yes"), _(u"false"), _(u"no")
         # potentially broken casefolding? What about locales?
         trues = set(word.lower() for word in itertools.chain(
             [u'1', u"true", u"yes"], # don't use potentially translated values
-            self._get_translations(['code'], u"true"),
-            self._get_translations(['code'], u"yes"),
+            self._get_boolean_translations(u"true"),
+            self._get_boolean_translations(u"yes"),
         ))
         if value.lower() in trues:
             return True, []
@@ -154,16 +219,19 @@ class IrFieldsConverter(models.AbstractModel):
         # potentially broken casefolding? What about locales?
         falses = set(word.lower() for word in itertools.chain(
             [u'', u"0", u"false", u"no"],
-            self._get_translations(['code'], u"false"),
-            self._get_translations(['code'], u"no"),
+            self._get_boolean_translations(u"false"),
+            self._get_boolean_translations(u"no"),
         ))
         if value.lower() in falses:
             return False, []
 
+        if field.name in self._context.get('import_skip_records', []):
+            return None, []
+
         return True, [self._format_import_error(
-            ImportWarning,
-            _(u"Unknown value '%s' for boolean field '%%(field)s', assuming '%s'"),
-            (value, yes),
+            ValueError,
+            _(u"Unknown value '%s' for boolean field '%%(field)s'"),
+            value,
             {'moreinfo': _(u"Use '1' for yes and '0' for no")}
         )]
 
@@ -248,17 +316,46 @@ class IrFieldsConverter(models.AbstractModel):
         return fields.Datetime.to_string(dt.astimezone(pytz.UTC)), []
 
     @api.model
-    def _get_translations(self, types, src):
-        types = tuple(types)
+    def _get_boolean_translations(self, src):
         # Cache translations so they don't have to be reloaded from scratch on
         # every row of the file
         tnx_cache = self._cr.cache.setdefault(self._name, {})
-        if tnx_cache.setdefault(types, {}) and src in tnx_cache[types]:
-            return tnx_cache[types][src]
+        if src in tnx_cache:
+            return tnx_cache[src]
 
-        Translations = self.env['ir.translation']
-        tnx = Translations.search([('type', 'in', types), ('src', '=', src)])
-        result = tnx_cache[types][src] = [t.value for t in tnx if t.value is not False]
+        values = OrderedSet()
+        for lang, __ in self.env['res.lang'].get_installed():
+            translations = code_translations.get_python_translations('base', lang)
+            if src in translations:
+                values.add(translations[src])
+
+        result = tnx_cache[src] = list(values)
+        return result
+
+    @api.model
+    def _get_selection_translations(self, field, src):
+        if not src:
+            return []
+        # Cache translations so they don't have to be reloaded from scratch on
+        # every row of the file
+        tnx_cache = self._cr.cache.setdefault(self._name, {})
+        if src in tnx_cache:
+            return tnx_cache[src]
+
+        values = OrderedSet()
+        self.env['ir.model.fields.selection'].flush_model()
+        query = """
+            SELECT s.name
+            FROM ir_model_fields_selection s
+            JOIN ir_model_fields f ON s.field_id = f.id
+            WHERE f.model = %s AND f.name = %s AND s.name->>'en_US' = %s
+        """
+        self.env.cr.execute(query, [field.model_name, field.name, src])
+        for (name,) in self.env.cr.fetchall():
+            name.pop('en_US')
+            values.update(name.values())
+
+        result = tnx_cache[src] = list(values)
         return result
 
     @api.model
@@ -269,10 +366,23 @@ class IrFieldsConverter(models.AbstractModel):
 
         for item, label in selection:
             label = ustr(label)
-            labels = [label] + self._get_translations(('selection', 'model', 'code'), label)
-            if value == str(item) or value in labels:
+            if callable(field.selection):
+                labels = [label]
+                for item2, label2 in field._description_selection(self.env):
+                    if item2 == item:
+                        labels.append(label2)
+                        break
+            else:
+                labels = [label] + self._get_selection_translations(field, label)
+            # case insensitive comparaison of string to allow to set the value even if the given 'value' param is not
+            # exactly (case sensitive) the same as one of the selection item.
+            if value.lower() == str(item).lower() or any(value.lower() == label.lower() for label in labels):
                 return item, []
 
+        if field.name in self._context.get('import_skip_records', []):
+            return None, []
+        elif field.name in self._context.get('import_set_empty_fields', []):
+            return False, []
         raise self._format_import_error(
             ValueError,
             _(u"Value '%s' not found in selection field '%%(field)s'"),
@@ -292,7 +402,6 @@ class IrFieldsConverter(models.AbstractModel):
                          ``id`` for an external id and ``.id`` for a database
                          id
         :param value: value of the reference to match to an actual record
-        :param context: OpenERP request context
         :return: a pair of the matched database identifier (if any), the
                  translated user-readable name for the field and the list of
                  warnings
@@ -354,14 +463,15 @@ class IrFieldsConverter(models.AbstractModel):
             if ids:
                 if len(ids) > 1:
                     warnings.append(ImportWarning(
-                        _(u"Found multiple matches for field '%%(field)s' (%d matches)")
-                        % (len(ids))))
+                        _(u"Found multiple matches for value '%s' in field '%%(field)s' (%d matches)")
+                        %(str(value).replace('%', '%%'), len(ids))))
                 id, _name = ids[0]
             else:
                 name_create_enabled_fields = self.env.context.get('name_create_enabled_fields') or {}
                 if name_create_enabled_fields.get(field.name):
                     try:
-                        id, _name = RelatedModel.name_create(name=value)
+                        with self.env.cr.savepoint():
+                            id, _name = RelatedModel.name_create(name=value)
                     except (Exception, psycopg2.IntegrityError):
                         error_msg = _(u"Cannot create new '%s' records from their name alone. Please create those records manually and try importing again.", RelatedModel._description)
         else:
@@ -371,16 +481,31 @@ class IrFieldsConverter(models.AbstractModel):
                 subfield
             )
 
-        if id is None:
+        set_empty = False
+        skip_record = False
+        if self.env.context.get('import_file'):
+            import_set_empty_fields = self.env.context.get('import_set_empty_fields') or []
+            field_path = "/".join((self.env.context.get('parent_fields_hierarchy', []) + [field.name]))
+            set_empty = field_path in import_set_empty_fields
+            skip_record = field_path in self.env.context.get('import_skip_records', [])
+        if id is None and not set_empty and not skip_record:
             if error_msg:
                 message = _("No matching record found for %(field_type)s '%(value)s' in field '%%(field)s' and the following error was encountered when we attempted to create one: %(error_message)s")
             else:
                 message = _("No matching record found for %(field_type)s '%(value)s' in field '%%(field)s'")
+
+            error_info_dict = {'moreinfo': action}
+            if self.env.context.get('import_file'):
+                # limit to 50 char to avoid too long error messages.
+                value = value[:50] if isinstance(value, str) else value
+                error_info_dict.update({'value': value, 'field_type': field_type})
+                if error_msg:
+                    error_info_dict['error_message'] = error_msg
             raise self._format_import_error(
                 ValueError,
                 message,
                 {'field_type': field_type, 'value': value, 'error_message': error_msg},
-                {'moreinfo': action})
+                error_info_dict)
         return id, field_type, warnings
 
     def _xmlid_to_record_id(self, xmlid, model):
@@ -457,10 +582,15 @@ class IrFieldsConverter(models.AbstractModel):
             ids.append(id)
             warnings.extend(ws)
 
+        if field.name in self._context.get('import_set_empty_fields', []) and any([id is None for id in ids]):
+            ids = [id for id in ids if id]
+        elif field.name in self._context.get('import_skip_records', []) and any([id is None for id in ids]):
+            return None, warnings
+
         if self._context.get('update_many2many'):
-            return [LINK_TO(id) for id in ids], warnings
+            return [Command.link(id) for id in ids], warnings
         else:
-            return [REPLACE_WITH(ids)], warnings
+            return [Command.set(ids)], warnings
 
     @api.model
     def _str_to_one2many(self, model, field, records):
@@ -487,12 +617,19 @@ class IrFieldsConverter(models.AbstractModel):
         def log(f, exception):
             if not isinstance(exception, Warning):
                 current_field_name = self.env[field.comodel_name]._fields[f].string
-                arg0 = exception.args[0] % {'field': '%(field)s/' + current_field_name}
+                arg0 = exception.args[0].replace('%(field)s', '%(field)s/' + current_field_name)
                 exception.args = (arg0, *exception.args[1:])
                 raise exception
             warnings.append(exception)
 
-        convert = self.with_context(name_create_enabled_fields=relative_name_create_enabled_fields).for_model(self.env[field.comodel_name])
+        # Complete the field hierarchy path
+        # E.g. For "parent/child/subchild", field hierarchy path for "subchild" is ['parent', 'child']
+        parent_fields_hierarchy = self._context.get('parent_fields_hierarchy', []) + [field.name]
+
+        convert = self.with_context(
+            name_create_enabled_fields=relative_name_create_enabled_fields,
+            parent_fields_hierarchy=parent_fields_hierarchy
+        ).for_model(self.env[field.comodel_name])
 
         for record in records:
             id = None
@@ -510,10 +647,10 @@ class IrFieldsConverter(models.AbstractModel):
                     writable['id'] = record['id']
 
             if id:
-                commands.append(LINK_TO(id))
-                commands.append(UPDATE(id, writable))
+                commands.append(Command.link(id))
+                commands.append(Command.update(id, writable))
             else:
-                commands.append(CREATE(writable))
+                commands.append(Command.create(writable))
 
         return commands, warnings
 
