@@ -1,23 +1,26 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import common
 from . import test_acl
 from . import test_api
+from . import test_barcode
 from . import test_base
 from . import test_basecase
 from . import test_cache
+from . import test_deprecation
 from . import test_db_cursor
 from . import test_expression
 from . import test_float
+from . import test_format_address_mixin
 from . import test_func
 from . import test_http_case
 from . import test_image
+from . import test_avatar_mixin
 from . import test_ir_actions
 from . import test_ir_attachment
 from . import test_ir_cron
-from . import test_ir_http
 from . import test_ir_filters
+from . import test_ir_mail_server
 from . import test_ir_model
 from . import test_ir_sequence
 from . import test_ir_sequence_date_range
@@ -26,9 +29,11 @@ from . import test_mail
 from . import test_menu
 from . import test_mimetypes
 from . import test_misc
+from . import test_module
 from . import test_orm
 from . import test_ormcache
 from . import test_osv
+from . import test_qweb_field
 from . import test_qweb
 from . import test_res_config
 from . import test_res_lang
@@ -38,11 +43,17 @@ from . import test_translate
 from . import test_user_has_group
 from . import test_views
 from . import test_xmlrpc
+from . import test_res_company
+from . import test_res_currency
 from . import test_res_partner
 from . import test_res_partner_bank
 from . import test_res_users
 from . import test_reports
+from . import test_test_retry
+from . import test_test_suite
 from . import test_tests_tags
 from . import test_form_create
 from . import test_cloc
+from . import test_profiler
 from . import test_pdf
+from . import test_neutralize
