@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import models
+from odoo import api, models, Command
 from odoo.addons.base.tests.common import SavepointCaseWithUserDemo
-from odoo.tools import mute_logger
+from odoo.tools import mute_logger, unique, lazy
 from odoo.exceptions import AccessError
 
 
@@ -30,7 +30,7 @@ class TestAPI(SavepointCaseWithUserDemo):
     @mute_logger('odoo.models')
     def test_00_query(self):
         """ Build a recordset, and check its contents. """
-        domain = [('name', 'ilike', 'j')]
+        domain = [('name', 'ilike', 'j'), ('id', 'in', self.partners.ids)]
         partners = self.env['res.partner'].search(domain)
 
         # partners is a collection of browse records
@@ -44,8 +44,8 @@ class TestAPI(SavepointCaseWithUserDemo):
     @mute_logger('odoo.models')
     def test_01_query_offset(self):
         """ Build a recordset with offset, and check equivalence. """
-        partners1 = self.env['res.partner'].search([], offset=10)
-        partners2 = self.env['res.partner'].search([])[10:]
+        partners1 = self.env['res.partner'].search([('id', 'in', self.partners.ids)], offset=5)
+        partners2 = self.env['res.partner'].search([('id', 'in', self.partners.ids)])[5:]
         self.assertIsRecordset(partners1, 'res.partner')
         self.assertIsRecordset(partners2, 'res.partner')
         self.assertEqual(list(partners1), list(partners2))
@@ -53,8 +53,8 @@ class TestAPI(SavepointCaseWithUserDemo):
     @mute_logger('odoo.models')
     def test_02_query_limit(self):
         """ Build a recordset with offset, and check equivalence. """
-        partners1 = self.env['res.partner'].search([], order='id asc', limit=10)
-        partners2 = self.env['res.partner'].search([], order='id asc')[:10]
+        partners1 = self.env['res.partner'].search([('id', 'in', self.partners.ids)], order='id asc', limit=5)
+        partners2 = self.env['res.partner'].search([('id', 'in', self.partners.ids)], order='id asc')[:5]
         self.assertIsRecordset(partners1, 'res.partner')
         self.assertIsRecordset(partners2, 'res.partner')
         self.assertEqual(list(partners1), list(partners2))
@@ -62,8 +62,8 @@ class TestAPI(SavepointCaseWithUserDemo):
     @mute_logger('odoo.models')
     def test_03_query_offset_limit(self):
         """ Build a recordset with offset and limit, and check equivalence. """
-        partners1 = self.env['res.partner'].search([], order='id asc', offset=3, limit=7)
-        partners2 = self.env['res.partner'].search([], order='id asc')[3:10]
+        partners1 = self.env['res.partner'].search([('id', 'in', self.partners.ids)], order='id asc', offset=3, limit=7)
+        partners2 = self.env['res.partner'].search([('id', 'in', self.partners.ids)], order='id asc')[3:10]
         self.assertIsRecordset(partners1, 'res.partner')
         self.assertIsRecordset(partners2, 'res.partner')
         self.assertEqual(list(partners1), list(partners2))
@@ -81,7 +81,7 @@ class TestAPI(SavepointCaseWithUserDemo):
     @mute_logger('odoo.models')
     def test_05_immutable(self):
         """ Check that a recordset remains the same, even after updates. """
-        domain = [('name', 'ilike', 'g')]
+        domain = [('name', 'ilike', 'g'), ('id', 'in', self.partners.ids)]
         partners = self.env['res.partner'].search(domain)
         self.assertTrue(partners)
         ids = partners.ids
@@ -102,24 +102,23 @@ class TestAPI(SavepointCaseWithUserDemo):
         self.assertIsRecord(user.partner_id, 'res.partner')
         self.assertIsRecordset(user.groups_id, 'res.groups')
 
-        partners = self.env['res.partner'].search([])
-        for name, field in partners._fields.items():
+        for name, field in self.partners._fields.items():
             if field.type == 'many2one':
-                for p in partners:
+                for p in self.partners:
                     self.assertIsRecord(p[name], field.comodel_name)
             elif field.type == 'reference':
-                for p in partners:
+                for p in self.partners:
                     if p[name]:
                         self.assertIsRecord(p[name], field.comodel_name)
             elif field.type in ('one2many', 'many2many'):
-                for p in partners:
+                for p in self.partners:
                     self.assertIsRecordset(p[name], field.comodel_name)
 
     @mute_logger('odoo.models')
     def test_07_null(self):
         """ Check behavior of null instances. """
         # select a partner without a parent
-        partner = self.env['res.partner'].search([('parent_id', '=', False)])[0]
+        partner = self.env['res.partner'].search([('parent_id', '=', False), ('id', 'in', self.partners.ids)])[0]
 
         # check partner and related null instances
         self.assertTrue(partner)
@@ -141,7 +140,7 @@ class TestAPI(SavepointCaseWithUserDemo):
     @mute_logger('odoo.models')
     def test_40_new_new(self):
         """ Call new-style methods in the new API style. """
-        partners = self.env['res.partner'].search([('name', 'ilike', 'g')])
+        partners = self.env['res.partner'].search([('name', 'ilike', 'g'), ('id', 'in', self.partners.ids)])
         self.assertTrue(partners)
 
         # call method write on partners itself, and check its effect
@@ -152,7 +151,7 @@ class TestAPI(SavepointCaseWithUserDemo):
     @mute_logger('odoo.models')
     def test_45_new_new(self):
         """ Call new-style methods on records (new API style). """
-        partners = self.env['res.partner'].search([('name', 'ilike', 'g')])
+        partners = self.env['res.partner'].search([('name', 'ilike', 'g'), ('id', 'in', self.partners.ids)])
         self.assertTrue(partners)
 
         # call method write on partner records, and check its effects
@@ -166,7 +165,7 @@ class TestAPI(SavepointCaseWithUserDemo):
     def test_50_environment(self):
         """ Test environment on records. """
         # partners and reachable records are attached to self.env
-        partners = self.env['res.partner'].search([('name', 'ilike', 'j')])
+        partners = self.env['res.partner'].search([('name', 'ilike', 'j'), ('id', 'in', self.partners.ids)])
         self.assertEqual(partners.env, self.env)
         for x in (partners, partners[0], partners[0].company_id):
             self.assertEqual(x.env, self.env)
@@ -177,8 +176,12 @@ class TestAPI(SavepointCaseWithUserDemo):
         partners[0].company_id.name
         partners[0].company_id.write({'name': 'Fools'})
 
-        # create an environment with the demo user
-        demo = self.env['res.users'].search([('login', 'ilike', 'demo')])[0]
+        # create an environment with a demo user
+        demo = self.env['res.users'].create({
+            'name': 'test_environment_demo',
+            'login': 'test_environment_demo',
+            'password': 'test_environment_demo',
+        })
         demo_env = self.env(user=demo)
         self.assertNotEqual(demo_env, self.env)
 
@@ -205,12 +208,23 @@ class TestAPI(SavepointCaseWithUserDemo):
             demo_partner.company_id.write({'name': 'Pricks'})
 
         # remove demo user from all groups
-        demo.write({'groups_id': [(5,)]})
+        demo.write({'groups_id': [Command.clear()]})
 
         # demo user can no longer access partner data
         with self.assertRaises(AccessError):
             demo_partner.company_id.name
 
+    @mute_logger('odoo.models')
+    def test_55_environment_lang(self):
+        """ Check the record env.lang behavior """
+        partner = self.partner_demo
+        self.env['res.lang']._activate_lang('fr_FR')
+        self.assertEqual(partner.with_context(lang=None).env.lang, None, 'None lang context should have None env.lang')
+        self.assertEqual(partner.with_context(lang='en_US').env.lang, 'en_US', 'en_US active lang context should have en_US env.lang')
+        self.assertEqual(partner.with_context(lang='fr_FR').env.lang, 'fr_FR', 'fr_FR active lang context should have fr_FR env.lang')
+        self.assertEqual(partner.with_context(lang='nl_NL').env.lang, None, 'Inactive lang context lang should have None env.lang')
+        self.assertEqual(partner.with_context(lang='Dummy').env.lang, None, 'Ilegal lang context should have None env.lang')
+
     @mute_logger('odoo.models')
     def test_60_cache(self):
         """ Check the record cache behavior """
@@ -224,7 +238,7 @@ class TestAPI(SavepointCaseWithUserDemo):
         for p in data:
             pids.append(Partners.create({
                 'name': p,
-                'child_ids': [(0, 0, {'name': c}) for c in data[p]],
+                'child_ids': [Command.create({'name': c}) for c in data[p]],
             }).id)
 
         partners = Partners.search([('id', 'in', pids)])
@@ -270,12 +284,12 @@ class TestAPI(SavepointCaseWithUserDemo):
         partner.country_id, partner.child_ids
         data = partner._convert_to_write(partner._cache)
         self.assertEqual(data['country_id'], partner.country_id.id)
-        self.assertEqual(data['child_ids'], [(6, 0, partner.child_ids.ids)])
+        self.assertEqual(data['child_ids'], [Command.set(partner.child_ids.ids)])
 
     @mute_logger('odoo.models')
     def test_60_prefetch(self):
         """ Check the record cache prefetching """
-        partners = self.env['res.partner'].search([], limit=models.PREFETCH_MAX)
+        partners = self.env['res.partner'].search([('id', 'in', self.partners.ids)], limit=models.PREFETCH_MAX)
         self.assertTrue(len(partners) > 1)
 
         # all the records in partners are ready for prefetching
@@ -310,7 +324,7 @@ class TestAPI(SavepointCaseWithUserDemo):
     @mute_logger('odoo.models')
     def test_60_prefetch_model(self):
         """ Check the prefetching model. """
-        partners = self.env['res.partner'].search([], limit=models.PREFETCH_MAX)
+        partners = self.env['res.partner'].search([('id', 'in', self.partners.ids)], limit=models.PREFETCH_MAX)
         self.assertTrue(partners)
 
         def same_prefetch(a, b):
@@ -322,7 +336,7 @@ class TestAPI(SavepointCaseWithUserDemo):
         # the recordset operations below use different prefetch sets
         diff_prefetch(partners, partners.browse())
         diff_prefetch(partners, partners[0])
-        diff_prefetch(partners, partners[:10])
+        diff_prefetch(partners, partners[:5])
 
         # the recordset operations below share the prefetch set
         same_prefetch(partners, partners.browse(partners.ids))
@@ -344,8 +358,8 @@ class TestAPI(SavepointCaseWithUserDemo):
         vals1 = {
             'name': 'Non-empty relational fields',
             'country_id': self.ref('base.be'),
-            'bank_ids': [(0, 0, {'acc_number': 'FOO42'})],
-            'category_id': [(4, self.partner_category.id)],
+            'bank_ids': [Command.create({'acc_number': 'FOO42'})],
+            'category_id': [Command.link(self.partner_category.id)],
         }
         partners = partners.create(vals0) + partners.create(vals1)
         for partner in partners:
@@ -379,11 +393,61 @@ class TestAPI(SavepointCaseWithUserDemo):
         self.assertIn('company_type', partner1._cache)
         self.assertNotIn('company_type', partner2._cache)
 
+    def test_60_reversed(self):
+        records = self.partners
+        self.assertGreater(len(records), 1)
+
+        # check order
+        self.assertEqual(list(reversed(records)), list(reversed(list(records))))
+
+        first = next(iter(records))
+        last = next(reversed(records))
+        self.assertEqual(first, records[0])
+        self.assertEqual(last, records[-1])
+
+        # check prefetching
+        prefetch_ids = records.ids
+        reversed_ids = [record.id for record in reversed(records)]
+
+        self.assertEqual(list(first._prefetch_ids), prefetch_ids)
+        self.assertEqual(list(last._prefetch_ids), reversed_ids)
+
+        self.assertEqual(list(reversed(first._prefetch_ids)), reversed_ids)
+        self.assertEqual(list(reversed(last._prefetch_ids)), prefetch_ids)
+
+        # check prefetching across many2one field
+        prefetch_ids = records.state_id.ids
+        reversed_ids = list(unique(
+            record.state_id.id
+            for record in reversed(records)
+            if record.state_id
+        ))
+
+        self.assertEqual(list(first.state_id._prefetch_ids), prefetch_ids)
+        self.assertEqual(list(last.state_id._prefetch_ids), reversed_ids)
+
+        self.assertEqual(list(reversed(first.state_id._prefetch_ids)), reversed_ids)
+        self.assertEqual(list(reversed(last.state_id._prefetch_ids)), prefetch_ids)
+
+        # check prefetching across x2many field
+        prefetch_ids = records.child_ids.ids
+        reversed_ids = list(unique(
+            child.id
+            for record in reversed(records)
+            for child in record.child_ids
+        ))
+
+        self.assertEqual(list(first.child_ids._prefetch_ids), prefetch_ids)
+        self.assertEqual(list(last.child_ids._prefetch_ids), reversed_ids)
+
+        self.assertEqual(list(reversed(first.child_ids._prefetch_ids)), reversed_ids)
+        self.assertEqual(list(reversed(last.child_ids._prefetch_ids)), prefetch_ids)
+
     @mute_logger('odoo.models')
     def test_70_one(self):
         """ Check method one(). """
         # check with many records
-        ps = self.env['res.partner'].search([('name', 'ilike', 'a')])
+        ps = self.env['res.partner'].search([('name', 'ilike', 'a'), ('id', 'in', self.partners.ids)])
         self.assertTrue(len(ps) > 1)
         with self.assertRaises(ValueError):
             ps.ensure_one()
@@ -400,15 +464,33 @@ class TestAPI(SavepointCaseWithUserDemo):
     @mute_logger('odoo.models')
     def test_80_contains(self):
         """ Test membership on recordset. """
-        p1 = self.env['res.partner'].search([('name', 'ilike', 'a')], limit=1).ensure_one()
-        ps = self.env['res.partner'].search([('name', 'ilike', 'a')])
+        p1 = self.partners[0]
+        ps = self.partners
         self.assertTrue(p1 in ps)
 
+        with self.assertRaisesRegex(TypeError, r"unsupported operand types in: 42 in res\.partner.*"):
+            42 in ps
+        with self.assertRaisesRegex(TypeError, r"inconsistent models in: ir\.ui\.menu.* in res\.partner.*"):
+            self.env['ir.ui.menu'] in ps
+
+    @mute_logger('odoo.models')
+    def test_80_lazy_contains(self):
+        """ Test membership on recordset. """
+        p1 = lazy(lambda: self.partners[0])
+        ps = lazy(lambda: self.partners)
+        self.assertTrue(p1 in ps)
+
+        with self.assertRaisesRegex(TypeError, r"unsupported operand types in: 42 in res\.partner.*"):
+            lazy(lambda: 42) in ps
+        with self.assertRaisesRegex(TypeError, r"inconsistent models in: ir\.ui\.menu.* in res\.partner.*"):
+            lazy(lambda: self.env['ir.ui.menu']) in ps
+
     @mute_logger('odoo.models')
     def test_80_set_operations(self):
         """ Check set operations on recordsets. """
-        pa = self.env['res.partner'].search([('name', 'ilike', 'a')])
-        pb = self.env['res.partner'].search([('name', 'ilike', 'b')])
+        pa = self.env['res.partner'].search([('name', 'ilike', 'a'), ('id', 'in', self.partners.ids)])
+        pb = self.env['res.partner'].search([('name', 'ilike', 'b'), ('id', 'in', self.partners.ids)])
+
         self.assertTrue(pa)
         self.assertTrue(pb)
         self.assertTrue(set(pa) & set(pb))
@@ -440,27 +522,85 @@ class TestAPI(SavepointCaseWithUserDemo):
         self.assertNotEqual(ps._name, ms._name)
         self.assertNotEqual(ps, ms)
 
+        with self.assertRaisesRegex(TypeError, r"unsupported operand types in: res\.partner.* \+ 'string'"):
+            ps + 'string'
+        with self.assertRaisesRegex(TypeError, r"inconsistent models in: res\.partner.* \+ ir\.ui\.menu.*"):
+            ps + ms
+        with self.assertRaisesRegex(TypeError, r"inconsistent models in: res\.partner.* - ir\.ui\.menu.*"):
+            ps - ms
+        with self.assertRaisesRegex(TypeError, r"inconsistent models in: res\.partner.* & ir\.ui\.menu.*"):
+            ps & ms
+        with self.assertRaisesRegex(TypeError, r"inconsistent models in: res\.partner.* \| ir\.ui\.menu.*"):
+            ps | ms
         with self.assertRaises(TypeError):
-            res = ps + ms
+            ps < ms
         with self.assertRaises(TypeError):
-            res = ps - ms
+            ps <= ms
         with self.assertRaises(TypeError):
-            res = ps & ms
+            ps > ms
         with self.assertRaises(TypeError):
-            res = ps | ms
+            ps >= ms
+
+    @mute_logger('odoo.models')
+    def test_80_lazy_set_operations(self):
+        """ Check set operations on recordsets. """
+        pa = lazy(lambda: self.env['res.partner'].search([('name', 'ilike', 'a'), ('id', 'in', self.partners.ids)]))
+        pb = lazy(lambda: self.env['res.partner'].search([('name', 'ilike', 'b'), ('id', 'in', self.partners.ids)]))
+
+        self.assertTrue(pa)
+        self.assertTrue(pb)
+        self.assertTrue(set(pa) & set(pb))
+
+        concat = pa + pb
+        self.assertEqual(list(concat), list(pa) + list(pb))
+        self.assertEqual(len(concat), len(pa) + len(pb))
+
+        difference = pa - pb
+        self.assertEqual(len(difference), len(set(difference)))
+        self.assertEqual(set(difference), set(pa) - set(pb))
+        self.assertLessEqual(difference, pa)
+
+        intersection = pa & pb
+        self.assertEqual(len(intersection), len(set(intersection)))
+        self.assertEqual(set(intersection), set(pa) & set(pb))
+        self.assertLessEqual(intersection, pa)
+        self.assertLessEqual(intersection, pb)
+
+        union = pa | pb
+        self.assertEqual(len(union), len(set(union)))
+        self.assertEqual(set(union), set(pa) | set(pb))
+        self.assertGreaterEqual(union, pa)
+        self.assertGreaterEqual(union, pb)
+
+        # one cannot mix different models with set operations
+        ps = pa
+        ms = lazy(lambda: self.env['ir.ui.menu'].search([]))
+        self.assertNotEqual(ps._name, ms._name)
+        self.assertNotEqual(ps, ms)
+
+        with self.assertRaisesRegex(TypeError, r"unsupported operand types in: res\.partner.* \+ 'string'"):
+            ps + 'string'
+        with self.assertRaisesRegex(TypeError, r"inconsistent models in: res\.partner.* \+ ir\.ui\.menu.*"):
+            ps + ms
+        with self.assertRaisesRegex(TypeError, r"inconsistent models in: res\.partner.* - ir\.ui\.menu.*"):
+            ps - ms
+        with self.assertRaisesRegex(TypeError, r"inconsistent models in: res\.partner.* & ir\.ui\.menu.*"):
+            ps & ms
+        with self.assertRaisesRegex(TypeError, r"inconsistent models in: res\.partner.* \| ir\.ui\.menu.*"):
+            ps | ms
         with self.assertRaises(TypeError):
-            res = ps < ms
+            ps < ms
         with self.assertRaises(TypeError):
-            res = ps <= ms
+            ps <= ms
         with self.assertRaises(TypeError):
-            res = ps > ms
+            ps > ms
         with self.assertRaises(TypeError):
-            res = ps >= ms
+            ps >= ms
 
     @mute_logger('odoo.models')
     def test_80_filter(self):
         """ Check filter on recordsets. """
-        ps = self.env['res.partner'].search([])
+        ps = self.partners
         customers = ps.browse([p.id for p in ps if p.employee])
 
         # filter on a single field
@@ -476,7 +616,7 @@ class TestAPI(SavepointCaseWithUserDemo):
     @mute_logger('odoo.models')
     def test_80_map(self):
         """ Check map on recordsets. """
-        ps = self.env['res.partner'].search([])
+        ps = self.partners
         parents = ps.browse()
         for p in ps:
             parents |= p.parent_id
@@ -506,7 +646,7 @@ class TestAPI(SavepointCaseWithUserDemo):
     @mute_logger('odoo.models')
     def test_80_sorted(self):
         """ Check sorted on recordsets. """
-        ps = self.env['res.partner'].search([])
+        ps = self.env['res.partner'].search([('id', 'in', self.partners.ids)])
 
         # sort by model order
         qs = ps[:len(ps) // 2] + ps[len(ps) // 2:]
@@ -520,3 +660,14 @@ class TestAPI(SavepointCaseWithUserDemo):
         # sort by inverse name, with a field name
         by_name_ids = [p.id for p in sorted(ps, key=lambda p: p.name, reverse=True)]
         self.assertEqual(ps.sorted('name', reverse=True).ids, by_name_ids)
+
+
+class TestExternalAPI(SavepointCaseWithUserDemo):
+
+    def test_call_kw(self):
+        """kwargs is not modified by the execution of the call"""
+        partner = self.env['res.partner'].create({'name': 'MyPartner1'})
+        args = (partner.ids, ['name'])
+        kwargs = {'context': {'test': True}}
+        api.call_kw(self.env['res.partner'], 'read', args, kwargs)
+        self.assertEqual(kwargs, {'context': {'test': True}})
