@@ -3,8 +3,8 @@
 
 import ast
 
-from odoo import SUPERUSER_ID
-from odoo.exceptions import UserError, ValidationError
+from odoo import SUPERUSER_ID, Command
+from odoo.exceptions import RedirectWarning, UserError, ValidationError
 from odoo.tests.common import TransactionCase, BaseCase
 from odoo.tools import mute_logger
 from odoo.tools.safe_eval import safe_eval, const_eval, expr_eval
@@ -366,8 +366,8 @@ class TestBase(TransactionCase):
                                       'phone': '1122334455',
                                       'email': 'info@sunhelm.com',
                                       'vat': 'BE0477472701',
-                                      'child_ids': [(4, p0.id),
-                                                    (0, 0, {'name': 'Alrik Greenthorn',
+                                      'child_ids': [Command.link(p0.id),
+                                                    Command.create({'name': 'Alrik Greenthorn',
                                                             'email': 'agr@sunhelm.com'})]})
         p1 = res_partner.create({'name': 'Otto Blackwood',
                                  'email': 'otto.blackwood@sunhelm.com',
@@ -376,7 +376,7 @@ class TestBase(TransactionCase):
                                   'email': 'ggr@sunhelm.com',
                                   'parent_id': p1.id})
         p2 = res_partner.search([('email', '=', 'agr@sunhelm.com')], limit=1)
-        sunhelm.write({'child_ids': [(0, 0, {'name': 'Ulrik Greenthorn',
+        sunhelm.write({'child_ids': [Command.create({'name': 'Ulrik Greenthorn',
                                              'email': 'ugr@sunhelm.com'})]})
         p3 = res_partner.search([('email', '=', 'ugr@sunhelm.com')], limit=1)
 
@@ -409,17 +409,17 @@ class TestBase(TransactionCase):
     def test_60_read_group(self):
         title_sir = self.env['res.partner.title'].create({'name': 'Sir...'})
         title_lady = self.env['res.partner.title'].create({'name': 'Lady...'})
-        test_users = [
+        user_vals_list = [
             {'name': 'Alice', 'login': 'alice', 'color': 1, 'function': 'Friend', 'date': '2015-03-28', 'title': title_lady.id},
-            {'name': 'Alice', 'login': 'alice2', 'color': 0, 'function': 'Friend',  'date': '2015-01-28', 'title': title_lady.id},
+            {'name': 'Alice', 'login': 'alice2', 'color': 0, 'function': 'Friend', 'date': '2015-01-28', 'title': title_lady.id},
             {'name': 'Bob', 'login': 'bob', 'color': 2, 'function': 'Friend', 'date': '2015-03-02', 'title': title_sir.id},
             {'name': 'Eve', 'login': 'eve', 'color': 3, 'function': 'Eavesdropper', 'date': '2015-03-20', 'title': title_lady.id},
             {'name': 'Nab', 'login': 'nab', 'color': -3, 'function': '5$ Wrench', 'date': '2014-09-10', 'title': title_sir.id},
             {'name': 'Nab', 'login': 'nab-she', 'color': 6, 'function': '5$ Wrench', 'date': '2014-01-02', 'title': title_lady.id},
         ]
         res_users = self.env['res.users']
-        user_ids = [res_users.create(vals).id for vals in test_users]
-        domain = [('id', 'in', user_ids)]
+        users = res_users.create(user_vals_list)
+        domain = [('id', 'in', users.ids)]
 
         # group on local char field without domain and without active_test (-> empty WHERE clause)
         groups_data = res_users.with_context(active_test=False).read_group([], fields=['login'], groupby=['login'], orderby='login DESC')
@@ -428,7 +428,7 @@ class TestBase(TransactionCase):
         # group on local char field with limit
         groups_data = res_users.read_group(domain, fields=['login'], groupby=['login'], orderby='login DESC', limit=3, offset=3)
         self.assertEqual(len(groups_data), 3, "Incorrect number of results when grouping on a field with limit")
-        self.assertEqual(['bob', 'alice2', 'alice'], [g['login'] for g in groups_data], 'Result mismatch')
+        self.assertEqual([g['login'] for g in groups_data], ['bob', 'alice2', 'alice'], 'Result mismatch')
 
         # group on inherited char field, aggregate on int field (second groupby ignored on purpose)
         groups_data = res_users.read_group(domain, fields=['name', 'color', 'function'], groupby=['function', 'login'])
@@ -440,66 +440,179 @@ class TestBase(TransactionCase):
 
         # group on inherited char field, reverse order
         groups_data = res_users.read_group(domain, fields=['name', 'color'], groupby='name', orderby='name DESC')
-        self.assertEqual(['Nab', 'Eve', 'Bob', 'Alice'], [g['name'] for g in groups_data], 'Incorrect ordering of the list')
+        self.assertEqual([g['name'] for g in groups_data], ['Nab', 'Eve', 'Bob', 'Alice'], 'Incorrect ordering of the list')
 
         # group on int field, default ordering
         groups_data = res_users.read_group(domain, fields=['color'], groupby='color')
-        self.assertEqual([-3, 0, 1, 2, 3, 6], [g['color'] for g in groups_data], 'Incorrect ordering of the list')
+        self.assertEqual([g['color'] for g in groups_data], [-3, 0, 1, 2, 3, 6], 'Incorrect ordering of the list')
 
         # multi group, second level is int field, should still be summed in first level grouping
         groups_data = res_users.read_group(domain, fields=['name', 'color'], groupby=['name', 'color'], orderby='name DESC')
-        self.assertEqual(['Nab', 'Eve', 'Bob', 'Alice'], [g['name'] for g in groups_data], 'Incorrect ordering of the list')
-        self.assertEqual([3, 3, 2, 1], [g['color'] for g in groups_data], 'Incorrect ordering of the list')
+        self.assertEqual([g['name'] for g in groups_data], ['Nab', 'Eve', 'Bob', 'Alice'], 'Incorrect ordering of the list')
+        self.assertEqual([g['color'] for g in groups_data], [3, 3, 2, 1], 'Incorrect ordering of the list')
 
         # group on inherited char field, multiple orders with directions
         groups_data = res_users.read_group(domain, fields=['name', 'color'], groupby='name', orderby='color DESC, name')
         self.assertEqual(len(groups_data), 4, "Incorrect number of results when grouping on a field")
-        self.assertEqual(['Eve', 'Nab', 'Bob', 'Alice'], [g['name'] for g in groups_data], 'Incorrect ordering of the list')
-        self.assertEqual([1, 2, 1, 2], [g['name_count'] for g in groups_data], 'Incorrect number of results')
+        self.assertEqual([g['name'] for g in groups_data], ['Eve', 'Nab', 'Bob', 'Alice'], 'Incorrect ordering of the list')
+        self.assertEqual([g['name_count'] for g in groups_data], [1, 2, 1, 2], 'Incorrect number of results')
 
         # group on inherited date column (res_partner.date) -> Year-Month, default ordering
         groups_data = res_users.read_group(domain, fields=['function', 'color', 'date'], groupby=['date'])
         self.assertEqual(len(groups_data), 4, "Incorrect number of results when grouping on a field")
-        self.assertEqual(['January 2014', 'September 2014', 'January 2015', 'March 2015'], [g['date'] for g in groups_data], 'Incorrect ordering of the list')
-        self.assertEqual([1, 1, 1, 3], [g['date_count'] for g in groups_data], 'Incorrect number of results')
+        self.assertEqual([g['date'] for g in groups_data], ['January 2014', 'September 2014', 'January 2015', 'March 2015'], 'Incorrect ordering of the list')
+        self.assertEqual([g['date_count'] for g in groups_data], [1, 1, 1, 3], 'Incorrect number of results')
+
+        # group on inherited date column (res_partner.date) specifying the :year -> Year default ordering
+        groups_data = res_users.read_group(domain, fields=['function', 'color', 'date'], groupby=['date:year'])
+        self.assertEqual(len(groups_data), 2, "Incorrect number of results when grouping on a field")
+        self.assertEqual([g['date:year'] for g in groups_data], ['2014', '2015'], 'Incorrect ordering of the list')
+        self.assertEqual([g['date_count'] for g in groups_data], [2, 4], 'Incorrect number of results')
 
         # group on inherited date column (res_partner.date) -> Year-Month, custom order
         groups_data = res_users.read_group(domain, fields=['function', 'color', 'date'], groupby=['date'], orderby='date DESC')
         self.assertEqual(len(groups_data), 4, "Incorrect number of results when grouping on a field")
-        self.assertEqual(['March 2015', 'January 2015', 'September 2014', 'January 2014'], [g['date'] for g in groups_data], 'Incorrect ordering of the list')
-        self.assertEqual([3, 1, 1, 1], [g['date_count'] for g in groups_data], 'Incorrect number of results')
+        self.assertEqual([g['date'] for g in groups_data], ['March 2015', 'January 2015', 'September 2014', 'January 2014'], 'Incorrect ordering of the list')
+        self.assertEqual([g['date_count'] for g in groups_data], [3, 1, 1, 1], 'Incorrect number of results')
 
         # group on inherited many2one (res_partner.title), default order
         groups_data = res_users.read_group(domain, fields=['function', 'color', 'title'], groupby=['title'])
         self.assertEqual(len(groups_data), 2, "Incorrect number of results when grouping on a field")
         # m2o is returned as a (id, label) pair
-        self.assertEqual([(title_lady.id, 'Lady...'), (title_sir.id, 'Sir...')], [g['title'] for g in groups_data], 'Incorrect ordering of the list')
-        self.assertEqual([4, 2], [g['title_count'] for g in groups_data], 'Incorrect number of results')
-        self.assertEqual([10, -1], [g['color'] for g in groups_data], 'Incorrect aggregation of int column')
+        self.assertEqual([g['title'] for g in groups_data], [(title_lady.id, 'Lady...'), (title_sir.id, 'Sir...')], 'Incorrect ordering of the list')
+        self.assertEqual([g['title_count'] for g in groups_data], [4, 2], 'Incorrect number of results')
+        self.assertEqual([g['color'] for g in groups_data], [10, -1], 'Incorrect aggregation of int column')
 
         # group on inherited many2one (res_partner.title), reversed natural order
         groups_data = res_users.read_group(domain, fields=['function', 'color', 'title'], groupby=['title'], orderby="title desc")
         self.assertEqual(len(groups_data), 2, "Incorrect number of results when grouping on a field")
         # m2o is returned as a (id, label) pair
         self.assertEqual([(title_sir.id, 'Sir...'), (title_lady.id, 'Lady...')], [g['title'] for g in groups_data], 'Incorrect ordering of the list')
-        self.assertEqual([2, 4], [g['title_count'] for g in groups_data], 'Incorrect number of results')
-        self.assertEqual([-1, 10], [g['color'] for g in groups_data], 'Incorrect aggregation of int column')
+        self.assertEqual([g['title_count'] for g in groups_data], [2, 4], 'Incorrect number of results')
+        self.assertEqual([g['color'] for g in groups_data], [-1, 10], 'Incorrect aggregation of int column')
 
         # group on inherited many2one (res_partner.title), multiple orders with m2o in second position
         groups_data = res_users.read_group(domain, fields=['function', 'color', 'title'], groupby=['title'], orderby="color desc, title desc")
         self.assertEqual(len(groups_data), 2, "Incorrect number of results when grouping on a field")
         # m2o is returned as a (id, label) pair
-        self.assertEqual([(title_lady.id, 'Lady...'), (title_sir.id, 'Sir...')], [g['title'] for g in groups_data], 'Incorrect ordering of the result')
-        self.assertEqual([4, 2], [g['title_count'] for g in groups_data], 'Incorrect number of results')
-        self.assertEqual([10, -1], [g['color'] for g in groups_data], 'Incorrect aggregation of int column')
+        self.assertEqual([g['title'] for g in groups_data], [(title_lady.id, 'Lady...'), (title_sir.id, 'Sir...')], 'Incorrect ordering of the result')
+        self.assertEqual([g['title_count'] for g in groups_data], [4, 2], 'Incorrect number of results')
+        self.assertEqual([g['color'] for g in groups_data], [10, -1], 'Incorrect aggregation of int column')
 
         # group on inherited many2one (res_partner.title), ordered by other inherited field (color)
         groups_data = res_users.read_group(domain, fields=['function', 'color', 'title'], groupby=['title'], orderby='color')
         self.assertEqual(len(groups_data), 2, "Incorrect number of results when grouping on a field")
         # m2o is returned as a (id, label) pair
-        self.assertEqual([(title_sir.id, 'Sir...'), (title_lady.id, 'Lady...')], [g['title'] for g in groups_data], 'Incorrect ordering of the list')
-        self.assertEqual([2, 4], [g['title_count'] for g in groups_data], 'Incorrect number of results')
-        self.assertEqual([-1, 10], [g['color'] for g in groups_data], 'Incorrect aggregation of int column')
+        self.assertEqual([g['title'] for g in groups_data], [(title_sir.id, 'Sir...'), (title_lady.id, 'Lady...')], 'Incorrect ordering of the list')
+        self.assertEqual([g['title_count'] for g in groups_data], [2, 4], 'Incorrect number of results')
+        self.assertEqual([g['color'] for g in groups_data], [-1, 10], 'Incorrect aggregation of int column')
+
+    def test_61_private_read_group(self):
+        """
+        the _read_group should behave exactly like read_group (public method) except for sorting the one2many on ID
+        instead of name, so avoiding the join on the "to many" table to get the name
+        """
+        title_sir = self.env['res.partner.title'].create({'name': 'Sir...'})
+        title_lady = self.env['res.partner.title'].create({'name': 'Lady...'})
+        user_vals_list = [
+            {'name': 'Alice', 'login': 'alice', 'color': 1, 'function': 'Friend', 'date': '2015-03-28', 'title': title_lady.id},
+            {'name': 'Alice', 'login': 'alice2', 'color': 0, 'function': 'Friend', 'date': '2015-01-28', 'title': title_lady.id},
+            {'name': 'Bob', 'login': 'bob', 'color': 2, 'function': 'Friend', 'date': '2015-03-02', 'title': title_sir.id},
+            {'name': 'Eve', 'login': 'eve', 'color': 3, 'function': 'Eavesdropper', 'date': '2015-03-20', 'title': title_lady.id},
+            {'name': 'Nab', 'login': 'nab', 'color': -3, 'function': '5$ Wrench', 'date': '2014-09-10', 'title': title_sir.id},
+            {'name': 'Nab', 'login': 'nab-she', 'color': 6, 'function': '5$ Wrench', 'date': '2014-01-02', 'title': title_lady.id},
+        ]
+        res_users = self.env['res.users']
+        users = res_users.create(user_vals_list)
+        domain = [('id', 'in', users.ids)]
+
+        # group on local char field without domain and without active_test (-> empty WHERE clause)
+        groups_data = res_users.with_context(active_test=False)._read_group([], fields=['login'], groupby=['login'], orderby='login DESC')
+        self.assertGreater(len(groups_data), 6, "Incorrect number of results when grouping on a field")
+
+        # group on local char field with limit
+        groups_data = res_users._read_group(domain, fields=['login'], groupby=['login'], orderby='login DESC', limit=3, offset=3)
+        self.assertEqual(len(groups_data), 3, "Incorrect number of results when grouping on a field with limit")
+        self.assertEqual(['bob', 'alice2', 'alice'], [g['login'] for g in groups_data], 'Result mismatch')
+
+        # group on inherited char field, aggregate on int field (second groupby ignored on purpose)
+        groups_data = res_users._read_group(domain, fields=['name', 'color', 'function'], groupby=['function', 'login'])
+        self.assertEqual(len(groups_data), 3, "Incorrect number of results when grouping on a field")
+        self.assertEqual([g['function'] for g in groups_data], ['5$ Wrench', 'Eavesdropper', 'Friend'], 'incorrect _read_group order')
+        for group_data in groups_data:
+            self.assertIn('color', group_data, "Aggregated data for the column 'color' is not present in _read_group return values")
+            self.assertEqual(group_data['color'], 3, "Incorrect sum for aggregated data for the column 'color'")
+
+        # group on inherited char field, reverse order
+        groups_data = res_users._read_group(domain, fields=['name', 'color'], groupby='name', orderby='name DESC')
+        self.assertEqual([g['name'] for g in groups_data], ['Nab', 'Eve', 'Bob', 'Alice'], 'Incorrect ordering of the list')
+
+        # group on int field, default ordering
+        groups_data = res_users._read_group(domain, fields=['color'], groupby='color')
+        self.assertEqual([g['color'] for g in groups_data], [-3, 0, 1, 2, 3, 6], 'Incorrect ordering of the list')
+
+        # multi group, second level is int field, should still be summed in first level grouping
+        groups_data = res_users._read_group(domain, fields=['name', 'color'], groupby=['name', 'color'], orderby='name DESC')
+        self.assertEqual([g['name'] for g in groups_data], ['Nab', 'Eve', 'Bob', 'Alice'], 'Incorrect ordering of the list')
+        self.assertEqual([g['color'] for g in groups_data], [3, 3, 2, 1], 'Incorrect ordering of the list')
+
+        # group on inherited char field, multiple orders with directions
+        groups_data = res_users._read_group(domain, fields=['name', 'color'], groupby='name', orderby='color DESC, name')
+        self.assertEqual(len(groups_data), 4, "Incorrect number of results when grouping on a field")
+        self.assertEqual([g['name'] for g in groups_data], ['Eve', 'Nab', 'Bob', 'Alice'], 'Incorrect ordering of the list')
+        self.assertEqual([g['name_count'] for g in groups_data], [1, 2, 1, 2], 'Incorrect number of results')
+
+        # group on inherited date column (res_partner.date) -> Year-Month, default ordering
+        groups_data = res_users._read_group(domain, fields=['function', 'color', 'date'], groupby=['date'])
+        self.assertEqual(len(groups_data), 4, "Incorrect number of results when grouping on a field")
+        self.assertEqual([g['date'] for g in groups_data], ['January 2014', 'September 2014', 'January 2015', 'March 2015'], 'Incorrect ordering of the list')
+        self.assertEqual([g['date_count'] for g in groups_data], [1, 1, 1, 3], 'Incorrect number of results')
+
+        # group on inherited date column (res_partner.date) specifying the :year -> Year default ordering
+        groups_data = res_users._read_group(domain, fields=['function', 'color', 'date'], groupby=['date:year'])
+        self.assertEqual(len(groups_data), 2, "Incorrect number of results when grouping on a field")
+        self.assertEqual([g['date:year'] for g in groups_data], ['2014', '2015'], 'Incorrect ordering of the list')
+        self.assertEqual([g['date_count'] for g in groups_data], [2, 4], 'Incorrect number of results')
+
+        # group on inherited date column (res_partner.date) -> Year-Month, custom order
+        groups_data = res_users._read_group(domain, fields=['function', 'color', 'date'], groupby=['date'], orderby='date DESC')
+        self.assertEqual(len(groups_data), 4, "Incorrect number of results when grouping on a field")
+        self.assertEqual([g['date'] for g in groups_data], ['March 2015', 'January 2015', 'September 2014', 'January 2014'], 'Incorrect ordering of the list')
+        self.assertEqual([g['date_count'] for g in groups_data], [3, 1, 1, 1], 'Incorrect number of results')
+
+        # group on inherited many2one (res_partner.title), default order
+        groups_data = res_users._read_group(domain, fields=['function', 'color', 'title'], groupby=['title'])
+        self.assertEqual(len(groups_data), 2, "Incorrect number of results when grouping on a field")
+        # m2o is returned as a (id, label) pair
+        # here the order of the titles is by ID
+        self.assertEqual([g['title'] for g in groups_data], [(title_sir.id, 'Sir...'), (title_lady.id, 'Lady...')], 'Incorrect ordering of the list')
+        self.assertEqual([g['title_count'] for g in groups_data], [2, 4], 'Incorrect number of results')
+        self.assertEqual([g['color'] for g in groups_data], [-1, 10], 'Incorrect aggregation of int column')
+
+        # group on inherited many2one (res_partner.title), reversed natural order
+        groups_data = res_users._read_group(domain, fields=['function', 'color', 'title'], groupby=['title'], orderby="title desc")
+        self.assertEqual(len(groups_data), 2, "Incorrect number of results when grouping on a field")
+        # m2o is returned as a (id, label) pair
+        # here the order of the titles is by ID DESC
+        self.assertEqual([g['title'] for g in groups_data], [(title_sir.id, 'Sir...'), (title_lady.id, 'Lady...')], 'Incorrect ordering of the list')
+        self.assertEqual([g['title_count'] for g in groups_data], [2, 4], 'Incorrect number of results')
+        self.assertEqual([g['color'] for g in groups_data], [-1, 10], 'Incorrect aggregation of int column')
+
+        # group on inherited many2one (res_partner.title), multiple orders with m2o in second position
+        groups_data = res_users._read_group(domain, fields=['function', 'color', 'title'], groupby=['title'], orderby="color desc, title desc")
+        self.assertEqual(len(groups_data), 2, "Incorrect number of results when grouping on a field")
+        # m2o is returned as a (id, label) pair
+        self.assertEqual([g['title'] for g in groups_data], [(title_lady.id, 'Lady...'), (title_sir.id, 'Sir...')], 'Incorrect ordering of the list')
+        self.assertEqual([g['title_count'] for g in groups_data], [4, 2], 'Incorrect number of results')
+        self.assertEqual([g['color'] for g in groups_data], [10, -1], 'Incorrect aggregation of int column')
+
+        # group on inherited many2one (res_partner.title), ordered by other inherited field (color)
+        groups_data = res_users._read_group(domain, fields=['function', 'color', 'title'], groupby=['title'], orderby='color')
+        self.assertEqual(len(groups_data), 2, "Incorrect number of results when grouping on a field")
+        # m2o is returned as a (id, label) pair
+        self.assertEqual([g['title'] for g in groups_data], [(title_sir.id, 'Sir...'), (title_lady.id, 'Lady...')], 'Incorrect ordering of the list')
+        self.assertEqual([g['title_count'] for g in groups_data], [2, 4], 'Incorrect number of results')
+        self.assertEqual([g['color'] for g in groups_data], [-1, 10], 'Incorrect aggregation of int column')
 
     def test_70_archive_internal_partners(self):
         test_partner = self.env['res.partner'].create({'name':'test partner'})
@@ -508,8 +621,10 @@ class TestBase(TransactionCase):
                                 'partner_id': test_partner.id,
                                 })
         # Cannot archive the partner
+        with self.assertRaises(RedirectWarning):
+            test_partner.with_user(self.env.ref('base.user_admin')).toggle_active()
         with self.assertRaises(ValidationError):
-            test_partner.toggle_active()
+            test_partner.with_user(self.env.ref('base.user_demo')).toggle_active()
 
         # Can archive the user but the partner stays active
         test_user.toggle_active()
@@ -554,8 +669,8 @@ class TestPartnerRecursion(TransactionCase):
         """ Indirect hacky write to create cycle in children """
         p3b = self.p1.create({'name': 'Elmtree Grand-Child 1.2', 'parent_id': self.p2.id})
         with self.assertRaises(ValidationError):
-            self.p2.write({'child_ids': [(1, self.p3.id, {'parent_id': p3b.id}),
-                                         (1, p3b.id, {'parent_id': self.p3.id})]})
+            self.p2.write({'child_ids': [Command.update(self.p3.id, {'parent_id': p3b.id}),
+                                         Command.update(p3b.id, {'parent_id': self.p3.id})]})
 
     def test_110_res_partner_recursion_multi_update(self):
         """ multi-write on several partners in same hierarchy must not trigger a false cycle detection """
@@ -605,7 +720,7 @@ class TestParentStore(TransactionCase):
         """ Duplicate the children then reassign them to the new parent (2nd method). """
         new_cat1 = self.cat1.copy()
         new_cat2 = self.cat2.copy()
-        new_cat0 = self.cat0.copy({'child_ids': [(6, 0, (new_cat1 + new_cat2).ids)]})
+        new_cat0 = self.cat0.copy({'child_ids': [Command.set((new_cat1 + new_cat2).ids)]})
         new_struct = new_cat0.search([('parent_id', 'child_of', new_cat0.id)])
         self.assertEqual(len(new_struct), 4, "After duplication, the new object must have the childs records")
         old_struct = new_cat0.search([('parent_id', 'child_of', self.cat0.id)])
@@ -617,7 +732,7 @@ class TestParentStore(TransactionCase):
         new_cat1 = self.cat1.copy()
         new_cat2 = self.cat2.copy()
         new_cat0 = self.cat0.copy({'child_ids': []})
-        new_cat0.write({'child_ids': [(4, new_cat1.id), (4, new_cat2.id)]})
+        new_cat0.write({'child_ids': [Command.link(new_cat1.id), Command.link(new_cat2.id)]})
         new_struct = new_cat0.search([('parent_id', 'child_of', new_cat0.id)])
         self.assertEqual(len(new_struct), 4, "After duplication, the new object must have the childs records")
         old_struct = new_cat0.search([('parent_id', 'child_of', self.cat0.id)])
@@ -649,8 +764,8 @@ class TestGroups(TransactionCase):
         # four groups with no cycle, check them all together
         a = self.env['res.groups'].create({'name': 'A'})
         b = self.env['res.groups'].create({'name': 'B'})
-        c = self.env['res.groups'].create({'name': 'G', 'implied_ids': [(6, 0, (a + b).ids)]})
-        d = self.env['res.groups'].create({'name': 'D', 'implied_ids': [(6, 0, c.ids)]})
+        c = self.env['res.groups'].create({'name': 'G', 'implied_ids': [Command.set((a + b).ids)]})
+        d = self.env['res.groups'].create({'name': 'D', 'implied_ids': [Command.set(c.ids)]})
         self.assertTrue((a + b + c + d)._check_m2m_recursion('implied_ids'))
 
         # create a cycle and check
@@ -662,6 +777,35 @@ class TestGroups(TransactionCase):
         b = a.copy()
         self.assertFalse(a.name == b.name)
 
+    def test_apply_groups(self):
+        a = self.env['res.groups'].create({'name': 'A'})
+        b = self.env['res.groups'].create({'name': 'B'})
+        c = self.env['res.groups'].create({'name': 'C', 'implied_ids': [Command.set(a.ids)]})
+
+        # C already implies A, we want both B+C to imply A
+        (b + c)._apply_group(a)
+
+        self.assertIn(a, b.implied_ids)
+        self.assertIn(a, c.implied_ids)
+
+    def test_remove_groups(self):
+        u = self.env['res.users'].create({'login': 'u', 'name': 'U'})
+
+        a = self.env['res.groups'].create({'name': 'A', 'users': [Command.set(u.ids)]})
+        b = self.env['res.groups'].create({'name': 'B', 'users': [Command.set(u.ids)]})
+        c = self.env['res.groups'].create({'name': 'C', 'implied_ids': [Command.set(a.ids)]})
+
+        # C already implies A, we want none of B+C to imply A
+        (b + c)._remove_group(a)
+
+        self.assertNotIn(a, b.implied_ids)
+        self.assertNotIn(a, c.implied_ids)
+
+        # Since B didn't imply A, removing A from the implied groups of (B+C)
+        # should not remove user U from A, even though C implied A, since C does
+        # not have U as a user
+        self.assertIn(u, a.users)
+
 
 class TestUsers(TransactionCase):
     def test_superuser(self):
