@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+import unittest
+from unittest.mock import patch
 
 import psycopg2
 
@@ -9,6 +11,7 @@ from odoo.models import BaseModel
 from odoo.tests.common import TransactionCase
 from odoo.tools import mute_logger
 from odoo.osv import expression
+from odoo import Command
 
 
 class TestExpression(SavepointCaseWithUserDemo):
@@ -17,12 +20,12 @@ class TestExpression(SavepointCaseWithUserDemo):
     def setUpClass(cls):
         super(TestExpression, cls).setUpClass()
         cls._load_partners_set()
+        cls.env['res.currency'].with_context({'active_test': False}).search([('name', 'in', ['EUR', 'USD'])]).write({'active': True})
 
-    def _search(self, obj, domain, init_domain=[]):
-        sql = obj.search(domain)
-        allobj = obj.search(init_domain)
-        fil = allobj.filtered_domain(domain)
-        self.assertEqual(sql, fil, "filtered_domain do not match SQL search for domain: "+str(domain))
+    def _search(self, model, domain, init_domain=None):
+        sql = model.search(domain, order="id")
+        fil = model.search(init_domain or [], order="id").filtered_domain(domain)
+        self.assertEqual(sql._ids, fil._ids, f"filtered_domain do not match SQL search for domain: {domain}")
         return sql
 
     def test_00_in_not_in_m2m(self):
@@ -32,9 +35,9 @@ class TestExpression(SavepointCaseWithUserDemo):
         cat_b = categories.create({'name': 'test_expression_category_B'})
 
         partners = self.env['res.partner']
-        a = partners.create({'name': 'test_expression_partner_A', 'category_id': [(6, 0, [cat_a.id])]})
-        b = partners.create({'name': 'test_expression_partner_B', 'category_id': [(6, 0, [cat_b.id])]})
-        ab = partners.create({'name': 'test_expression_partner_AB', 'category_id': [(6, 0, [cat_a.id, cat_b.id])]})
+        a = partners.create({'name': 'test_expression_partner_A', 'category_id': [Command.set([cat_a.id])]})
+        b = partners.create({'name': 'test_expression_partner_B', 'category_id': [Command.set([cat_b.id])]})
+        ab = partners.create({'name': 'test_expression_partner_AB', 'category_id': [Command.set([cat_a.id, cat_b.id])]})
         c = partners.create({'name': 'test_expression_partner_C'})
 
         # The tests.
@@ -99,7 +102,7 @@ class TestExpression(SavepointCaseWithUserDemo):
         }
         pids = {}
         for name, cat_ids in partners_config.items():
-            pids[name] = partners.create({'name': name, 'category_id': [(6, 0, cat_ids)]}).id
+            pids[name] = partners.create({'name': name, 'category_id': [Command.set(cat_ids)]}).id
 
         base_domain = [('id', 'in', list(pids.values()))]
 
@@ -114,6 +117,17 @@ class TestExpression(SavepointCaseWithUserDemo):
         test('not ilike', 'B', ['0', 'a'])
         test('not like', 'AB', ['0', 'a', 'b', 'a b'])
 
+    def test_09_hierarchy_filtered_domain(self):
+        Partner = self.env['res.partner']
+        p = Partner.create({'name': 'dummy'})
+
+        # hierarchy without parent
+        self.assertFalse(p.parent_id)
+        p2 = self._search(Partner, [('parent_id', 'child_of', p.id)], [('id', '=', p.id)])
+        self.assertEqual(p2, p)
+        p3 = self._search(Partner, [('parent_id', 'parent_of', p.id)], [('id', '=', p.id)])
+        self.assertEqual(p3, p)
+
     def test_10_hierarchy_in_m2m(self):
         Partner = self.env['res.partner']
         Category = self.env['res.partner.category']
@@ -185,6 +199,40 @@ class TestExpression(SavepointCaseWithUserDemo):
             cats = self._search(Category, [('id', 'parent_of', False)])
         self.assertEqual(len(cats), 0)
 
+    @mute_logger('odoo.models.unlink')
+    def test_10_hierarchy_access(self):
+        Partner = self.env['res.partner'].with_user(self.user_demo)
+        top = Partner.create({'name': 'Top'})
+        med = Partner.create({'name': 'Medium', 'parent_id': top.id})
+        bot = Partner.create({'name': 'Bottom', 'parent_id': med.id})
+
+        # restrict access of user Demo to partners Top and Bottom
+        accessible = top + bot
+        self.env['ir.rule'].search([]).unlink()
+        self.env['ir.rule'].create({
+            'name': 'partners rule',
+            'model_id': self.env['ir.model']._get('res.partner').id,
+            'domain_force': str([('id', 'in', accessible.ids)]),
+        })
+
+        # these searches should return the subset of accessible nodes that are
+        # in the given hierarchy
+        self.assertEqual(Partner.search([]), accessible)
+        self.assertEqual(Partner.search([('id', 'child_of', top.ids)]), accessible)
+        self.assertEqual(Partner.search([('id', 'parent_of', bot.ids)]), accessible)
+
+        # same kind of search from another model
+        Bank = self.env['res.partner.bank'].with_user(self.user_demo)
+        bank_top, _bank_med, bank_bot = Bank.create([
+            {'acc_number': '1', 'partner_id': top.id},
+            {'acc_number': '2', 'partner_id': med.id},
+            {'acc_number': '3', 'partner_id': bot.id},
+        ])
+
+        self.assertEqual(Bank.search([('partner_id', 'in', accessible.ids)]), bank_top + bank_bot)
+        self.assertEqual(Bank.search([('partner_id', 'child_of', top.ids)]), bank_top + bank_bot)
+        self.assertEqual(Bank.search([('partner_id', 'parent_of', bot.ids)]), bank_top + bank_bot)
+
     def test_10_eq_lt_gt_lte_gte(self):
         # test if less/greater than or equal operators work
         currency = self.env['res.currency'].search([], limit=1)
@@ -437,9 +485,15 @@ class TestExpression(SavepointCaseWithUserDemo):
 
         # test many2many operator with False
         partners = self._search(Partner, [('category_id', '=', False)])
+        self.assertTrue(partners)
         for partner in partners:
             self.assertFalse(partner.category_id)
 
+        partners = self._search(Partner, [('category_id', '!=', False)])
+        self.assertTrue(partners)
+        for partner in partners:
+            self.assertTrue(partner.category_id)
+
         # filtering on nonexistent value across x2many should return nothing
         partners = self._search(Partner, [('child_ids.city', '=', 'foo')])
         self.assertFalse(partners)
@@ -579,7 +633,7 @@ class TestExpression(SavepointCaseWithUserDemo):
         # Test2: inheritance + relational fields
         users = self._search(Users, [('child_ids.name', 'like', 'test_B')])
         self.assertEqual(users, b1, 'searching through inheritance failed')
-        
+
         # Special =? operator mean "is equal if right is set, otherwise always True"
         users = self._search(Users, [('name', 'like', 'test'), ('parent_id', '=?', False)])
         self.assertEqual(users, a + b1 + b2, '(x =? False) failed')
@@ -593,6 +647,17 @@ class TestExpression(SavepointCaseWithUserDemo):
         norm_domain = ['&', '&', '&'] + domain
         self.assertEqual(norm_domain, expression.normalize_domain(domain), "Non-normalized domains should be properly normalized")
 
+    def test_35_negating_thruty_leafs(self):
+        self.assertEqual(expression.distribute_not(['!', '!', expression.TRUE_LEAF]), [expression.TRUE_LEAF], "distribute_not applied wrongly")
+        self.assertEqual(expression.distribute_not(['!', '!', expression.FALSE_LEAF]), [expression.FALSE_LEAF], "distribute_not applied wrongly")
+        self.assertEqual(expression.distribute_not(['!', '!', '!', '!', expression.TRUE_LEAF]), [expression.TRUE_LEAF], "distribute_not applied wrongly")
+        self.assertEqual(expression.distribute_not(['!', '!', '!', '!', expression.FALSE_LEAF]), [expression.FALSE_LEAF], "distribute_not applied wrongly")
+
+        self.assertEqual(expression.distribute_not(['!', expression.TRUE_LEAF]), [expression.FALSE_LEAF], "distribute_not applied wrongly")
+        self.assertEqual(expression.distribute_not(['!', expression.FALSE_LEAF]), [expression.TRUE_LEAF], "distribute_not applied wrongly")
+        self.assertEqual(expression.distribute_not(['!', '!', '!', expression.TRUE_LEAF]), [expression.FALSE_LEAF], "distribute_not applied wrongly")
+        self.assertEqual(expression.distribute_not(['!', '!', '!', expression.FALSE_LEAF]), [expression.TRUE_LEAF], "distribute_not applied wrongly")
+
     def test_40_negating_long_expression(self):
         source = ['!', '&', ('user_id', '=', 4), ('partner_id', 'in', [1, 2])]
         expect = ['|', ('user_id', '!=', 4), ('partner_id', 'not in', [1, 2])]
@@ -614,13 +679,29 @@ class TestExpression(SavepointCaseWithUserDemo):
 
     def test_accent(self):
         if not self.registry.has_unaccent:
-            return
-        Company = self.env['res.company']
-        helene = Company.create({'name': u'Hélène'})
-        self.assertEqual(helene, Company.search([('name','ilike','Helene')]))
-        self.assertEqual(helene, Company.search([('name','ilike','hélène')]))
-        self.assertNotIn(helene, Company.search([('name','not ilike','Helene')]))
-        self.assertNotIn(helene, Company.search([('name','not ilike','hélène')]))
+            raise unittest.SkipTest("unaccent not enabled")
+
+        Model = self.env['res.partner.category']
+        helen = Model.create({'name': 'Hélène'})
+        self.assertEqual(helen, Model.search([('name', 'ilike', 'Helene')]))
+        self.assertEqual(helen, Model.search([('name', 'ilike', 'hélène')]))
+        self.assertNotIn(helen, Model.search([('name', 'not ilike', 'Helene')]))
+        self.assertNotIn(helen, Model.search([('name', 'not ilike', 'hélène')]))
+
+        hermione, nicostratus = Model.create([
+            {'name': 'Hermione', 'parent_id': helen.id},
+            {'name': 'Nicostratus', 'parent_id': helen.id}
+        ])
+        self.assertEqual(nicostratus.parent_path, f'{helen.id}/{nicostratus.id}/')
+
+        with patch('odoo.osv.expression.get_unaccent_wrapper') as w:
+            w().side_effect = lambda x: x
+            rs = Model.search([('parent_path', 'like', f'{helen.id}/%')], order='id asc')
+            self.assertEqual(rs, helen | hermione | nicostratus)
+            # the result of `get_unaccent_wrapper()` is the wrapper and that's
+            # what should not be called
+            w().assert_not_called()
+
 
     def test_pure_function(self):
         orig_false = expression.FALSE_DOMAIN.copy()
@@ -681,6 +762,10 @@ class TestExpression(SavepointCaseWithUserDemo):
             countries = self._search(Country, domain)
             self.assertEqual(countries, belgium)
 
+        countries = self._search(Country, [('name', 'not in', ['No country'])])
+        all_countries = self._search(Country, [])
+        self.assertEqual(countries, all_countries)
+
     @mute_logger('odoo.sql_db')
     def test_invalid(self):
         """ verify that invalid expressions are refused, even for magic fields """
@@ -707,8 +792,8 @@ class TestExpression(SavepointCaseWithUserDemo):
         vals = {
             'name': 'OpenERP Test',
             'active': False,
-            'category_id': [(6, 0, [self.partner_category.id])],
-            'child_ids': [(0, 0, {'name': 'address of OpenERP Test', 'country_id': self.ref("base.be")})],
+            'category_id': [Command.set([self.partner_category.id])],
+            'child_ids': [Command.create({'name': 'address of OpenERP Test', 'country_id': self.ref("base.be")})],
         }
         Partner.create(vals)
         partner = self._search(Partner, [('category_id', 'ilike', 'sellers'), ('active', '=', False)], [('active', '=', False)])
@@ -793,13 +878,23 @@ class TestExpression(SavepointCaseWithUserDemo):
         countries = countries.browse(reversed(countries._ids))
         self.assertEqual(countries.filtered_domain(domain)._ids, countries._ids)
 
+    def test_filtered_domain_order2(self):
+        countries = self.env['res.country'].search([])
+        # match the first two countries, in order
+        expected = countries[:2]
+        id1, id2 = expected._ids
+        domain = ['|', ('id', '=', id1), ('id', '=', id2)]
+        self.assertEqual(countries.filtered_domain(domain)._ids, expected._ids)
+        domain = ['|', ('id', '=', id2), ('id', '=', id1)]
+        self.assertEqual(countries.filtered_domain(domain)._ids, expected._ids)
+
 
 class TestExpression2(TransactionCase):
 
     def test_long_table_alias(self):
         # To test the 64 characters limit for table aliases in PostgreSQL
-        self.patch_order('res.users', 'partner_id')
-        self.patch_order('res.partner', 'commercial_partner_id,company_id,name')
+        self.patch(self.registry['res.users'], '_order', 'partner_id')
+        self.patch(self.registry['res.partner'], '_order', 'commercial_partner_id,company_id,name')
         self.env['res.users'].search([('name', '=', 'test')])
 
 
@@ -1020,7 +1115,7 @@ class TestQueries(TransactionCase):
                     )
                 )
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             Model.search(domain)
 
@@ -1032,7 +1127,7 @@ class TestQueries(TransactionCase):
             SELECT "res_partner".id
             FROM "res_partner"
             WHERE (("res_partner"."active" = %s) AND ("res_partner"."name"::text LIKE %s))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             Model.search([('name', 'like', 'foo')])
 
@@ -1049,12 +1144,25 @@ class TestQueries(TransactionCase):
         Model.search([('name', 'like', 'foo')])
 
         with self.assertQueries(['''
-            SELECT count(1)
+            SELECT COUNT(*)
             FROM "res_partner"
             WHERE (("res_partner"."active" = %s) AND ("res_partner"."name"::text LIKE %s))
         ''']):
             Model.search_count([('name', 'like', 'foo')])
 
+    def test_count_limit(self):
+        Model = self.env['res.partner']
+        Model.search([('name', 'like', 'foo')])
+
+        with self.assertQueries(['''
+            SELECT COUNT(*) FROM (
+                SELECT FROM "res_partner"
+                WHERE (("res_partner"."active" = %s) AND ("res_partner"."name"::text LIKE %s))
+                LIMIT 1
+            ) t
+        ''']):
+            Model.search_count([('name', 'like', 'foo')], limit=1)
+
     def test_translated_field(self):
         self.env['res.lang']._activate_lang('fr_FR')
         Model = self.env['res.partner.title'].with_context(lang='fr_FR')
@@ -1063,19 +1171,13 @@ class TestQueries(TransactionCase):
         with self.assertQueries(['''
             SELECT "res_partner_title".id
             FROM "res_partner_title"
-            LEFT JOIN "ir_translation" AS "res_partner_title__name" ON
-                ("res_partner_title"."id" = "res_partner_title__name"."res_id"
-                 AND "res_partner_title__name"."type" = 'model'
-                 AND "res_partner_title__name"."name" = %s
-                 AND "res_partner_title__name"."lang" = %s
-                 AND "res_partner_title__name"."value" != %s)
-            WHERE COALESCE("res_partner_title__name"."value", "res_partner_title"."name") LIKE %s
-            ORDER BY COALESCE("res_partner_title__name"."value", "res_partner_title"."name")
+            WHERE (COALESCE("res_partner_title"."name"->>'fr_FR', "res_partner_title"."name"->>'en_US') like %s)
+            ORDER BY COALESCE("res_partner_title"."name"->>'fr_FR', "res_partner_title"."name"->>'en_US')   
         ''']):
             Model.search([('name', 'like', 'foo')])
 
         with self.assertQueries(['''
-            SELECT COUNT(1)
+            SELECT COUNT(*)
             FROM "res_partner_title"
             WHERE ("res_partner_title"."id" = %s)
         ''']):
@@ -1108,6 +1210,38 @@ class TestQueries(TransactionCase):
         ''']):
             Model.search([])
 
+    def test_rec_names_search(self):
+        Model = self.env['ir.model']
+
+        # search on both 'name' and 'model'
+        self.assertEqual(Model._rec_names_search, ['name', 'model'])
+
+        Model.name_search('partner')
+        with self.assertQueries(['''
+            SELECT "ir_model".id
+            FROM "ir_model"
+            WHERE (
+                ("ir_model"."name"->>'en_US' ILIKE %s)
+                OR ("ir_model"."model"::text ILIKE %s)
+            )
+            ORDER BY "ir_model"."model"
+            LIMIT 100
+        ''']):
+            Model.name_search('foo')
+
+        Model.name_search('partner', operator='not ilike')
+        with self.assertQueries(['''
+            SELECT "ir_model".id
+            FROM "ir_model"
+            WHERE (
+                ("ir_model"."name" is NULL OR "ir_model"."name"->>'en_US' not ilike %s)
+                AND (("ir_model"."model"::text NOT ILIKE %s) OR "ir_model"."model" IS NULL)
+            )
+            ORDER BY "ir_model"."model"
+            LIMIT 100
+        ''']):
+            Model.name_search('foo', operator='not ilike')
+
 
 class TestMany2one(TransactionCase):
     def setUp(self):
@@ -1127,6 +1261,18 @@ class TestMany2one(TransactionCase):
         ''']):
             self.User.search([('name', 'like', 'foo')])
 
+        # the field supporting the inheritance should be auto_join, too
+        # TODO: use another model, since 'res.users' has explicit auto_join
+        with self.assertQueries(['''
+            SELECT "res_users".id
+            FROM "res_users"
+            LEFT JOIN "res_partner" AS "res_users__partner_id" ON
+                ("res_users"."partner_id" = "res_users__partner_id"."id")
+            WHERE ("res_users__partner_id"."name"::text LIKE %s)
+            ORDER BY "res_users__partner_id"."name", "res_users"."login"
+        ''']):
+            self.User.search([('partner_id.name', 'like', 'foo')])
+
     def test_regular(self):
         self.Partner.search([('company_id.partner_id.name', 'like', self.company.name)])
         self.Partner.search([('country_id.code', 'like', 'BE')])
@@ -1135,7 +1281,7 @@ class TestMany2one(TransactionCase):
             SELECT "res_partner".id
             FROM "res_partner"
             WHERE ("res_partner"."company_id" = %s)
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('company_id', '=', self.company.id)])
 
@@ -1146,9 +1292,8 @@ class TestMany2one(TransactionCase):
                 SELECT "res_company".id
                 FROM "res_company"
                 WHERE ("res_company"."name"::text like %s)
-                ORDER BY "res_company"."id"
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('company_id.name', 'like', self.company.name)])
 
@@ -1162,11 +1307,9 @@ class TestMany2one(TransactionCase):
                     SELECT "res_partner".id
                     FROM "res_partner"
                     WHERE ("res_partner"."name"::text LIKE %s)
-                    ORDER BY "res_partner"."id"
                 ))
-                ORDER BY "res_company"."id"
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('company_id.partner_id.name', 'like', self.company.name)])
 
@@ -1177,14 +1320,12 @@ class TestMany2one(TransactionCase):
                 SELECT "res_company".id
                 FROM "res_company"
                 WHERE ("res_company"."name"::text LIKE %s)
-                ORDER BY "res_company"."id"
             )) OR ("res_partner"."country_id" IN (
                 SELECT "res_country".id
                 FROM "res_country"
                 WHERE ("res_country"."code"::text LIKE %s)
-                ORDER BY "res_country"."id"
             )))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([
                 '|',
@@ -1201,14 +1342,29 @@ class TestMany2one(TransactionCase):
             WHERE ("res_partner"."company_id" IN (
                 SELECT "res_company".id
                 FROM "res_company"
-                WHERE ("res_company"."name"::text like %s)
-                ORDER BY "res_company"."id"
+                WHERE (("res_company"."active" = %s) AND ("res_company"."name"::text like %s))
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             company_ids = self.company._search([('name', 'like', self.company.name)], order='id')
             self.Partner.search([('company_id', 'in', company_ids)])
 
+        # special case, with a LIMIT to make ORDER BY necessary
+        with self.assertQueries(['''
+            SELECT "res_partner".id
+            FROM "res_partner"
+            WHERE ("res_partner"."company_id" IN (
+                SELECT "res_company".id
+                FROM "res_company"
+                WHERE (("res_company"."active" = %s) AND ("res_company"."name"::text like %s))
+                ORDER BY "res_company"."id"
+                LIMIT 1
+            ))
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
+        ''']):
+            company_ids = self.company._search([('name', 'like', self.company.name)], order='id', limit=1)
+            self.Partner.search([('company_id', 'in', company_ids)])
+
     def test_autojoin(self):
         # auto_join on the first many2one
         self.patch(self.Partner._fields['company_id'], 'auto_join', True)
@@ -1221,7 +1377,7 @@ class TestMany2one(TransactionCase):
             LEFT JOIN "res_company" AS "res_partner__company_id" ON
                 ("res_partner"."company_id" = "res_partner__company_id"."id")
             WHERE ("res_partner__company_id"."name"::text LIKE %s)
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('company_id.name', 'like', self.company.name)])
 
@@ -1234,9 +1390,8 @@ class TestMany2one(TransactionCase):
                 SELECT "res_partner".id
                 FROM "res_partner"
                 WHERE ("res_partner"."name"::text LIKE %s)
-                ORDER BY "res_partner"."id"
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('company_id.partner_id.name', 'like', self.company.name)])
 
@@ -1254,9 +1409,8 @@ class TestMany2one(TransactionCase):
                 LEFT JOIN "res_partner" AS "res_company__partner_id" ON
                     ("res_company"."partner_id" = "res_company__partner_id"."id")
                 WHERE ("res_company__partner_id"."name"::text LIKE %s)
-                ORDER BY "res_company"."id"
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('company_id.partner_id.name', 'like', self.company.name)])
 
@@ -1273,7 +1427,7 @@ class TestMany2one(TransactionCase):
             LEFT JOIN "res_partner" AS "res_partner__company_id__partner_id" ON
                 ("res_partner__company_id"."partner_id" = "res_partner__company_id__partner_id"."id")
             WHERE ("res_partner__company_id__partner_id"."name"::text LIKE %s)
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('company_id.partner_id.name', 'like', self.company.name)])
 
@@ -1295,7 +1449,7 @@ class TestMany2one(TransactionCase):
                 ("res_partner"."company_id" = "res_partner__company_id"."id")
             WHERE (("res_partner__company_id"."name"::text LIKE %s)
                 OR ("res_partner__country_id"."code"::text LIKE %s))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([
                 '|',
@@ -1313,12 +1467,23 @@ class TestMany2one(TransactionCase):
                 SELECT "res_company".id
                 FROM "res_company"
                 WHERE ("res_company"."name"::text LIKE %s)
-                ORDER BY "res_company"."sequence", "res_company"."name"
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('company_id', 'like', self.company.name)])
 
+        with self.assertQueries(['''
+                SELECT "res_partner".id
+                FROM "res_partner"
+                WHERE (("res_partner"."company_id" IN (
+                    SELECT "res_company".id
+                    FROM "res_company"
+                    WHERE (("res_company"."name"::text not like %s) OR "res_company"."name" IS NULL))
+                ) OR "res_partner"."company_id" IS NULL)
+                ORDER BY "res_partner"."display_name", "res_partner"."id"
+        ''']):
+            self.Partner.search([('company_id', 'not like', "blablabla")])
+
 
 class TestOne2many(TransactionCase):
     def setUp(self):
@@ -1327,9 +1492,9 @@ class TestOne2many(TransactionCase):
         self.partner = self.Partner.create({
             'name': 'Foo',
             'bank_ids': [
-                (0, 0, {'acc_number': '123', 'acc_type': 'bank'}),
-                (0, 0, {'acc_number': '456', 'acc_type': 'bank'}),
-                (0, 0, {'acc_number': '789', 'acc_type': 'bank'}),
+                Command.create({'acc_number': '123', 'acc_type': 'bank'}),
+                Command.create({'acc_number': '456', 'acc_type': 'bank'}),
+                Command.create({'acc_number': '789', 'acc_type': 'bank'}),
             ],
         })
 
@@ -1344,7 +1509,7 @@ class TestOne2many(TransactionCase):
             WHERE ("res_partner"."id" IN (
                 SELECT "partner_id" FROM "res_partner_bank" WHERE "id" IN %s
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('bank_ids', 'in', self.partner.bank_ids.ids)])
 
@@ -1355,9 +1520,8 @@ class TestOne2many(TransactionCase):
                 SELECT "res_partner_bank"."partner_id"
                 FROM "res_partner_bank"
                 WHERE ("res_partner_bank"."sanitized_acc_number"::text LIKE %s)
-                ORDER BY "res_partner_bank"."id"
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('bank_ids.sanitized_acc_number', 'like', '12')])
 
@@ -1371,11 +1535,9 @@ class TestOne2many(TransactionCase):
                     SELECT "res_partner_bank"."partner_id"
                     FROM "res_partner_bank"
                     WHERE ("res_partner_bank"."sanitized_acc_number"::text LIKE %s)
-                    ORDER BY "res_partner_bank"."id"
-                ))
-                ORDER BY "res_partner"."id"
+                )) AND "res_partner"."parent_id" IS NOT NULL
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('child_ids.bank_ids.sanitized_acc_number', 'like', '12')])
 
@@ -1392,7 +1554,7 @@ class TestOne2many(TransactionCase):
             WHERE ("res_partner"."id" IN (
                 SELECT "partner_id" FROM "res_partner_bank" WHERE "id" IN %s
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('bank_ids', 'in', self.partner.bank_ids.ids)])
 
@@ -1404,7 +1566,7 @@ class TestOne2many(TransactionCase):
                 FROM "res_partner_bank"
                 WHERE ("res_partner_bank"."sanitized_acc_number"::text LIKE %s)
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('bank_ids.sanitized_acc_number', 'like', '12')])
 
@@ -1420,7 +1582,7 @@ class TestOne2many(TransactionCase):
                 FROM "res_partner_bank"
                 WHERE ("res_partner_bank"."sanitized_acc_number"::text LIKE %s)
             )))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([
                 ('bank_ids.sanitized_acc_number', 'like', '12'),
@@ -1439,7 +1601,7 @@ class TestOne2many(TransactionCase):
                     WHERE ("res_partner_bank"."sanitized_acc_number"::text LIKE %s)
                 )) AND ("res_partner"."active" = %s))
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('child_ids.bank_ids.sanitized_acc_number', 'like', '12')])
 
@@ -1460,7 +1622,7 @@ class TestOne2many(TransactionCase):
                         SELECT "res_partner_bank"."partner_id"
                         FROM "res_partner_bank"
                         WHERE ((
-                            "res_partner_bank"."id" IN (%s,%s,%s)
+                            "res_partner_bank"."id" IN %s
                         ) AND (
                             "res_partner_bank"."sanitized_acc_number"::text LIKE %s
                         ))
@@ -1469,7 +1631,7 @@ class TestOne2many(TransactionCase):
                     ("res_partner"."name" != %s) OR "res_partner"."name" IS NULL
                 ))
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('child_ids.bank_ids.id', 'in', self.partner.bank_ids.ids)])
 
@@ -1495,7 +1657,7 @@ class TestOne2many(TransactionCase):
                     "res_partner"."active" = %s
                 ))
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('child_ids.state_id.country_id.code', 'like', 'US')])
 
@@ -1509,12 +1671,35 @@ class TestOne2many(TransactionCase):
                 SELECT "res_partner_bank"."partner_id"
                 FROM "res_partner_bank"
                 WHERE ("res_partner_bank"."sanitized_acc_number"::text LIKE %s)
-                ORDER BY "res_partner_bank"."sequence", "res_partner_bank"."id"
             ))
-            ORDER BY "res_partner"."display_name"
+            ORDER BY "res_partner"."display_name", "res_partner"."id"
         ''']):
             self.Partner.search([('bank_ids', 'like', '12')])
 
+    def test_empty(self):
+        self.Partner.search([('bank_ids', '!=', False)], order='id')
+        self.Partner.search([('bank_ids', '=', False)], order='id')
+
+        with self.assertQueries(['''
+            SELECT "res_partner".id
+            FROM "res_partner"
+            WHERE ("res_partner"."id" IN (
+                SELECT "partner_id" FROM "res_partner_bank" WHERE "partner_id" IS NOT NULL
+            ))
+            ORDER BY "res_partner"."id"
+        ''']):
+            self.Partner.search([('bank_ids', '!=', False)], order='id')
+
+        with self.assertQueries(['''
+            SELECT "res_partner".id
+            FROM "res_partner"
+            WHERE ("res_partner"."id" NOT IN (
+                SELECT "partner_id" FROM "res_partner_bank" WHERE "partner_id" IS NOT NULL
+            ))
+            ORDER BY "res_partner"."id"
+        ''']):
+            self.Partner.search([('bank_ids', '=', False)], order='id')
+
 
 class TestMany2many(TransactionCase):
     def setUp(self):
@@ -1533,46 +1718,64 @@ class TestMany2many(TransactionCase):
         with self.assertQueries(['''
             SELECT "res_users".id
             FROM "res_users"
-            WHERE ("res_users"."id" IN (
-                SELECT "uid" FROM "res_groups_users_rel" WHERE "gid" IN %s
-            ))
+            WHERE EXISTS (
+                SELECT 1 FROM "res_groups_users_rel" AS "res_users__groups_id"
+                WHERE "res_users__groups_id"."uid" = "res_users".id
+                AND "res_users__groups_id"."gid" IN %s
+            )
             ORDER BY "res_users"."id"
         ''']):
             self.User.search([('groups_id', 'in', group.ids)], order='id')
 
+        group_color = group.color
         with self.assertQueries(['''
             SELECT "res_users".id
             FROM "res_users"
-            WHERE ("res_users"."id" IN (
-                SELECT "uid" FROM "res_groups_users_rel" WHERE "gid" IN (
-                    SELECT "res_groups".id
-                    FROM "res_groups"
-                    WHERE ("res_groups"."color" = %s)
-                    ORDER BY "res_groups"."id"
-                )
-            ))
+            WHERE NOT EXISTS (
+                SELECT 1 FROM "res_groups_users_rel" AS "res_users__groups_id"
+                WHERE "res_users__groups_id"."uid" = "res_users".id
+                AND "res_users__groups_id"."gid" IN %s
+            )
             ORDER BY "res_users"."id"
         ''']):
-            self.User.search([('groups_id.color', '=', group.color)], order='id')
+            self.User.search([('groups_id', 'not in', group.ids)], order='id')
 
         with self.assertQueries(['''
             SELECT "res_users".id
             FROM "res_users"
-            WHERE ("res_users"."id" IN (
-                SELECT "uid" FROM "res_groups_users_rel" WHERE "gid" IN (
+            WHERE EXISTS (
+                SELECT 1 FROM "res_groups_users_rel" AS "res_users__groups_id"
+                WHERE "res_users__groups_id"."uid" = "res_users".id
+                AND "res_users__groups_id"."gid" IN (
                     SELECT "res_groups".id
                     FROM "res_groups"
-                    WHERE ("res_groups"."id" IN (
-                        SELECT "group_id" FROM "rule_group_rel" WHERE "rule_group_id" IN (
+                    WHERE ("res_groups"."color" = %s)
+                )
+            )
+            ORDER BY "res_users"."id"
+        ''']):
+            self.User.search([('groups_id.color', '=', group_color)], order='id')
+
+        with self.assertQueries(['''
+            SELECT "res_users".id
+            FROM "res_users"
+            WHERE EXISTS (
+                SELECT 1 FROM "res_groups_users_rel" AS "res_users__groups_id"
+                WHERE "res_users__groups_id"."uid" = "res_users".id
+                AND "res_users__groups_id"."gid" IN (
+                    SELECT "res_groups".id
+                    FROM "res_groups"
+                    WHERE EXISTS (
+                        SELECT 1 FROM "rule_group_rel" AS "res_groups__rule_groups"
+                        WHERE "res_groups__rule_groups"."group_id" = "res_groups".id
+                        AND "res_groups__rule_groups"."rule_group_id" IN (
                             SELECT "ir_rule".id
                             FROM "ir_rule"
                             WHERE ("ir_rule"."name"::text LIKE %s)
-                            ORDER BY "ir_rule"."id"
                         )
-                    ))
-                    ORDER BY "res_groups"."id"
+                    )
                 )
-            ))
+            )
             ORDER BY "res_users"."id"
         ''']):
             self.User.search([('groups_id.rule_groups.name', 'like', rule.name)], order='id')
@@ -1588,14 +1791,41 @@ class TestMany2many(TransactionCase):
         with self.assertQueries(['''
             SELECT "res_users".id
             FROM "res_users"
-            WHERE ("res_users"."id" IN (
-                SELECT "user_id" FROM "res_company_users_rel" WHERE "cid" IN (
+            WHERE EXISTS (
+                SELECT 1 FROM "res_company_users_rel" AS "res_users__company_ids"
+                WHERE "res_users__company_ids"."user_id" = "res_users".id
+                AND "res_users__company_ids"."cid" IN (
                     SELECT "res_company".id
                     FROM "res_company"
                     WHERE ("res_company"."name"::text LIKE %s)
-                    ORDER BY "res_company"."sequence", "res_company"."name"
                 )
-            ))
+            )
             ORDER BY "res_users"."id"
         ''']):
             self.User.search([('company_ids', 'like', self.company.name)], order='id')
+
+    def test_empty(self):
+        self.User.search([('groups_id', '!=', False)], order='id')
+        self.User.search([('groups_id', '=', False)], order='id')
+
+        with self.assertQueries(['''
+            SELECT "res_users".id
+            FROM "res_users"
+            WHERE EXISTS (
+                SELECT 1 FROM "res_groups_users_rel" AS "res_users__groups_id"
+                WHERE "res_users__groups_id"."uid" = "res_users".id
+            )
+            ORDER BY "res_users"."id"
+        ''']):
+            self.User.search([('groups_id', '!=', False)], order='id')
+
+        with self.assertQueries(['''
+            SELECT "res_users".id
+            FROM "res_users"
+            WHERE NOT EXISTS (
+                SELECT 1 FROM "res_groups_users_rel" AS "res_users__groups_id"
+                WHERE "res_users__groups_id"."uid" = "res_users".id
+            )
+            ORDER BY "res_users"."id"
+        ''']):
+            self.User.search([('groups_id', '=', False)], order='id')
