@@ -8,12 +8,14 @@ import email.message
 import re
 import threading
 
-from odoo.tests.common import BaseCase, SavepointCase, TransactionCase
+from odoo.addons.base.models.ir_mail_server import extract_rfc2822_addresses
+from odoo.tests.common import BaseCase, TransactionCase
 from odoo.tools import (
-    is_html_empty, html_sanitize, append_content_to_html, plaintext2html,
-    email_split,
+    is_html_empty, html_to_inner_content, html_sanitize, append_content_to_html, plaintext2html,
+    email_split, email_domain_normalize,
     misc, formataddr,
     prepend_html_content,
+    config,
 )
 
 from . import test_mail_examples
@@ -33,24 +35,6 @@ class TestSanitizer(BaseCase):
             html = html_sanitize(content)
             self.assertEqual(html, expected, 'html_sanitize is broken')
 
-    def test_mako(self):
-        cases = [
-            ('''<p>Some text</p>
-<% set signup_url = object.get_signup_url() %>
-% if signup_url:
-<p>
-    You can access this document and pay online via our Customer Portal:
-</p>''', '''<p>Some text</p>
-<% set signup_url = object.get_signup_url() %>
-% if signup_url:
-<p>
-    You can access this document and pay online via our Customer Portal:
-</p>''')
-        ]
-        for content, expected in cases:
-            html = html_sanitize(content, silent=False)
-            self.assertEqual(html, expected, 'html_sanitize: broken mako management')
-
     def test_evil_malicious_code(self):
         # taken from https://www.owasp.org/index.php/XSS_Filter_Evasion_Cheat_Sheet#Tests
         cases = [
@@ -195,6 +179,13 @@ class TestSanitizer(BaseCase):
         for ext in test_mail_examples.HOTMAIL_1_OUT:
             self.assertIn(ext, html)
 
+    def test_quote_outlook_html(self):
+        html = html_sanitize(test_mail_examples.QUOTE_OUTLOOK_HTML)
+        for ext in test_mail_examples.QUOTE_OUTLOOK_HTML_IN:
+            self.assertIn(ext, html)
+        for ext in test_mail_examples.QUOTE_OUTLOOK_HTML_OUT:
+            self.assertIn(ext, html)
+
     def test_quote_thunderbird_html(self):
         html = html_sanitize(test_mail_examples.QUOTE_THUNDERBIRD_HTML)
         for ext in test_mail_examples.QUOTE_THUNDERBIRD_HTML_IN:
@@ -202,6 +193,13 @@ class TestSanitizer(BaseCase):
         for ext in test_mail_examples.QUOTE_THUNDERBIRD_HTML_OUT:
             self.assertIn(ext, html)
 
+    def test_quote_yahoo_html(self):
+        html = html_sanitize(test_mail_examples.QUOTE_YAHOO_HTML)
+        for ext in test_mail_examples.QUOTE_YAHOO_HTML_IN:
+            self.assertIn(ext, html)
+        for ext in test_mail_examples.QUOTE_YAHOO_HTML_OUT:
+            self.assertIn(ext, html)
+
     def test_quote_basic_text(self):
         test_data = [
             (
@@ -309,6 +307,19 @@ class TestHtmlTools(BaseCase):
             html = plaintext2html(content, container_tag)
             self.assertEqual(html, expected, 'plaintext2html is broken')
 
+    def test_html_html_to_inner_content(self):
+        cases = [
+            ('<div><p>First <br/>Second <br/>Third Paragraph</p><p>--<br/>Signature paragraph with a <a href="./link">link</a></p></div>',
+             'First Second Third Paragraph -- Signature paragraph with a link'),
+            ('<p>Now =&gt; processing&nbsp;entities&#8203;and extra whitespace too.  </p>',
+             'Now =&gt; processing&nbsp;entities\u200band extra whitespace too.'),
+            ('<div>Look what happens with <p>unmatched tags</div>', 'Look what happens with unmatched tags'),
+            ('<div>Look what happens with <p unclosed tags</div> Are we good?', 'Look what happens with Are we good?')
+        ]
+        for content, expected in cases:
+            text = html_to_inner_content(content)
+            self.assertEqual(text, expected, 'html_html_to_inner_content is broken')
+
     def test_append_to_html(self):
         test_samples = [
             ('<!DOCTYPE...><HTML encoding="blah">some <b>content</b></HtMl>', '--\nYours truly', True, True, False,
@@ -328,11 +339,21 @@ class TestHtmlTools(BaseCase):
         for content in void_strings_samples:
             self.assertTrue(is_html_empty(content))
 
-        void_html_samples = ['<p><br></p>', '<p><br> </p>', '<p><br /></p >']
+        void_html_samples = [
+            '<p><br></p>', '<p><br> </p>', '<p><br /></p >',
+            '<p style="margin: 4px"></p>',
+            '<div style="margin: 4px"></div>',
+            '<p class="oe_testing"><br></p>',
+            '<p><span style="font-weight: bolder;"><font style="color: rgb(255, 0, 0);" class=" "></font></span><br></p>',
+        ]
         for content in void_html_samples:
             self.assertTrue(is_html_empty(content), 'Failed with %s' % content)
 
-        valid_html_samples = ['<p><br>1</p>', '<p>1<br > </p>']
+        valid_html_samples = [
+            '<p><br>1</p>', '<p>1<br > </p>', '<p style="margin: 4px">Hello World</p>',
+            '<div style="margin: 4px"><p>Hello World</p></div>',
+            '<p><span style="font-weight: bolder;"><font style="color: rgb(255, 0, 0);" class=" ">W</font></span><br></p>',
+        ]
         for content in valid_html_samples:
             self.assertFalse(is_html_empty(content))
 
@@ -425,9 +446,27 @@ class TestEmailTools(BaseCase):
                 with self.subTest(pair=pair, charset=charset):
                     self.assertEqual(formataddr(pair, charset), expected)
 
+    def test_extract_rfc2822_addresses(self):
+        tests = [
+            ('"Admin" <admin@example.com>', ['admin@example.com']),
+            ('"Admin" <admin@example.com>, Demo <demo@test.com>', ['admin@example.com', 'demo@test.com']),
+            ('admin@example.com', ['admin@example.com']),
+            ('"Admin" <admin@example.com>, Demo <malformed email>', ['admin@example.com']),
+            ('admin@éxample.com', ['admin@xn--xample-9ua.com']),
+            ('"admin@éxample.com" <admin@éxample.com>', ['admin@xn--xample-9ua.com']),
+        ]
 
-class EmailConfigCase(SavepointCase):
-    @patch.dict("odoo.tools.config.options", {"email_from": "settings@example.com"})
+        for (rfc2822_email, expected) in tests:
+            self.assertEqual(extract_rfc2822_addresses(rfc2822_email), expected)
+
+    def test_email_domain_normalize(self):
+        self.assertEqual(email_domain_normalize("Test.Com"), "test.com", "Should have normalized the domain")
+        self.assertEqual(email_domain_normalize("email@test.com"), False, "The domain is not valid, should return False")
+        self.assertEqual(email_domain_normalize(False), False, "The domain is not valid, should return False")
+
+
+class EmailConfigCase(TransactionCase):
+    @patch.dict(config.options, {"email_from": "settings@example.com"})
     def test_default_email_from(self, *args):
         """Email from setting is respected."""
         # ICP setting is more important
@@ -447,107 +486,43 @@ class EmailConfigCase(SavepointCase):
         )
         self.assertEqual(message["From"], "settings@example.com")
 
-    def test_email_from_rewrite(self):
-        get_email_from = self.env['ir.mail_server']._get_email_from
-        set_param = self.env['ir.config_parameter'].set_param
 
-        # Standard case, no setting
-        set_param('mail.force.smtp.from', False)
-        set_param('mail.dynamic.smtp.from', False)
-        set_param('mail.catchall.domain', 'odoo.example.com')
+class _FakeSMTP:
+    """SMTP stub"""
+    def __init__(self):
+        self.messages = []
+        self.from_filter = 'example.com'
 
-        email_from, return_path = get_email_from('admin@test.example.com')
-        self.assertEqual(email_from, 'admin@test.example.com')
-        self.assertFalse(return_path)
+    # Python 3 before 3.7.4
+    def sendmail(self, smtp_from, smtp_to_list, message_str,
+                 mail_options=(), rcpt_options=()):
+        self.messages.append(message_str)
 
-        email_from, return_path = get_email_from('"Admin" <admin@test.example.com>')
-        self.assertEqual(email_from, '"Admin" <admin@test.example.com>')
-        self.assertFalse(return_path)
-
-        # We always force the email FROM
-        set_param('mail.force.smtp.from', 'email_force@domain.com')
-        set_param('mail.dynamic.smtp.from', False)
-        set_param('mail.catchall.domain', 'odoo.example.com')
-
-        email_from, return_path = get_email_from('admin@test.example.com')
-        self.assertEqual(email_from, '"admin@test.example.com" <email_force@domain.com>')
-        self.assertEqual(return_path, 'email_force@domain.com')
-
-        email_from, return_path = get_email_from('"Admin" <admin@test.example.com>')
-        self.assertEqual(email_from, '"Admin (admin@test.example.com)" <email_force@domain.com>')
-        self.assertEqual(return_path, 'email_force@domain.com')
-
-        # We always force the email FROM (notification email contains a name part)
-        set_param('mail.force.smtp.from', '"Your notification bot" <email_force@domain.com>')
-        set_param('mail.dynamic.smtp.from', False)
-        set_param('mail.catchall.domain', 'odoo.example.com')
-
-        email_from, return_path = get_email_from('"Admin" <admin@test.example.com>')
-        self.assertEqual(email_from, '"Admin (admin@test.example.com)" <email_force@domain.com>',
-                         msg='Should drop the name part of the forced email')
-        self.assertEqual(return_path, 'email_force@domain.com')
-
-        # We dynamically force the email FROM
-        set_param('mail.force.smtp.from', False)
-        set_param('mail.dynamic.smtp.from', 'notification@odoo.example.com')
-        set_param('mail.catchall.domain', 'odoo.example.com')
-
-        email_from, return_path = get_email_from('"Admin" <admin@test.example.com>')
-        self.assertEqual(email_from, '"Admin (admin@test.example.com)" <notification@odoo.example.com>',
-                         msg='Domain is not the same as the catchall domain, we should force the email FROM')
-        self.assertEqual(return_path, 'notification@odoo.example.com')
-
-        email_from, return_path = get_email_from('"Admin" <admin@odoo.example.com>')
-        self.assertEqual(email_from, '"Admin" <admin@odoo.example.com>',
-                         msg='Domain is the same as the catchall domain, we should not force the email FROM')
-        self.assertFalse(return_path)
-
-        # We dynamically force the email FROM (notification email contains a name part)
-        set_param('mail.force.smtp.from', False)
-        set_param('mail.dynamic.smtp.from', '"Your notification bot" <notification@odoo.example.com>')
-        set_param('mail.catchall.domain', 'odoo.example.com')
-
-        email_from, return_path = get_email_from('"Admin" <admin@test.example.com>')
-        self.assertEqual(email_from, '"Admin (admin@test.example.com)" <notification@odoo.example.com>',
-                         msg='Domain is not the same as the catchall domain, we should force the email FROM')
-        self.assertEqual(return_path, 'notification@odoo.example.com')
-
-        email_from, return_path = get_email_from('"Admin" <admin@odoo.example.com>')
-        self.assertEqual(email_from, '"Admin" <admin@odoo.example.com>',
-                         msg='Domain is the same as the catchall domain, we should not force the email FROM')
-        self.assertFalse(return_path)
+    # Python 3.7.4+
+    def send_message(self, message, smtp_from, smtp_to_list,
+                     mail_options=(), rcpt_options=()):
+        self.messages.append(message.as_string())
 
 
 class TestEmailMessage(TransactionCase):
-    def test_as_string(self):
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls._fake_smtp = _FakeSMTP()
+
+    def build_email(self, **kwargs):
+        kwargs.setdefault('email_from', 'from@example.com')
+        kwargs.setdefault('email_to', 'to@example.com')
+        kwargs.setdefault('subject', 'subject')
+        return self.env['ir.mail_server'].build_email(**kwargs)
+
+    def send_email(self, msg):
+        with patch.object(threading.current_thread(), 'testing', False):
+            self.env['ir.mail_server'].send_email(msg, smtp_session=self._fake_smtp)
+        return self._fake_smtp.messages.pop()
+
+    def test_bpo_34424_35805(self):
         """Ensure all email sent are bpo-34424 and bpo-35805 free"""
-
-        message_truth = (
-            r'From: .+? <joe@example\.com>\r\n'
-            r'To: .+? <joe@example\.com>\r\n'
-            r'Message-Id: <[0-9a-z.-]+@[0-9a-z.-]+>\r\n'
-            r'References: (<[0-9a-z.-]+@[0-9a-z.-]+>\s*)+\r\n'
-            r'\r\n'
-        )
-
-        class FakeSMTP:
-            """SMTP stub"""
-            def __init__(this):
-                this.email_sent = False
-
-            # Python 3 before 3.7.4
-            def sendmail(this, smtp_from, smtp_to_list, message_str,
-                         mail_options=(), rcpt_options=()):
-                this.email_sent = True
-                self.assertRegex(message_str, message_truth)
-
-            # Python 3.7.4+
-            def send_message(this, message, smtp_from, smtp_to_list,
-                             mail_options=(), rcpt_options=()):
-                message_str = message.as_string()
-                this.email_sent = True
-                self.assertRegex(message_str, message_truth)
-
         msg = email.message.EmailMessage(policy=email.policy.SMTP)
         msg['From'] = '"Joé Doe" <joe@example.com>'
         msg['To'] = '"Joé Doe" <joe@example.com>'
@@ -556,7 +531,33 @@ class TestEmailMessage(TransactionCase):
         msg['Message-Id'] = '<929227342217024.1596730490.324691772460938-example-30661-some.reference@test-123.example.com>'
         msg['References'] = '<345227342212345.1596730777.324691772483620-example-30453-other.reference@test-123.example.com>'
 
-        smtp = FakeSMTP()
-        self.patch(threading.currentThread(), 'testing', False)
-        self.env['ir.mail_server'].send_email(msg, smtp_session=smtp)
-        self.assertTrue(smtp.email_sent)
+        msg_on_the_wire = self.send_email(msg)
+        self.assertEqual(msg_on_the_wire,
+            'From: =?utf-8?q?Jo=C3=A9?= Doe <joe@example.com>\r\n'
+            'To: =?utf-8?q?Jo=C3=A9?= Doe <joe@example.com>\r\n'
+            'Message-Id: <929227342217024.1596730490.324691772460938-example-30661-some.reference@test-123.example.com>\r\n'
+            'References: <345227342212345.1596730777.324691772483620-example-30453-other.reference@test-123.example.com>\r\n'
+            '\r\n'
+        )
+
+    def test_alternative_correct_order(self):
+        """
+        RFC-1521 7.2.3. The Multipart/alternative subtype
+        > the alternatives appear in an order of increasing faithfulness
+        > to the original content. In general, the best choice is the
+        > LAST part of a type supported by the recipient system's local
+        > environment.
+
+        Also, the MIME-Version header should be present in BOTH the
+        enveloppe AND the parts
+        """
+        msg = self.build_email(body='<p>Hello world</p>', subtype='html')
+        msg_on_the_wire = self.send_email(msg)
+
+        self.assertGreater(msg_on_the_wire.index('text/html'), msg_on_the_wire.index('text/plain'),
+            "The html part should be preferred (=appear after) to the text part")
+        self.assertEqual(msg_on_the_wire.count('==============='), 2 + 2, # +2 for the header and the footer
+            "There should be 2 parts: one text and one html")
+        self.assertEqual(msg_on_the_wire.count('MIME-Version: 1.0'), 3,
+            "There should be 3 headers MIME-Version: one on the enveloppe, "
+            "one on the html part, one on the text part")
