@@ -3,9 +3,10 @@
 
 import datetime
 from dateutil.relativedelta import relativedelta
+import os.path
 import pytz
 
-from odoo.tools import misc, date_utils, merge_sequences, remove_accents
+from odoo.tools import config, misc, date_utils, file_open, file_path, merge_sequences, remove_accents
 from odoo.tests.common import TransactionCase, BaseCase
 
 
@@ -239,10 +240,10 @@ class TestFormatLangDate(TransactionCase):
 
         # -- test `datetime`
         datetime_str = '2017-01-31 10:33:00'
-
         # Change languages and timezones
-        self.assertEqual(misc.format_datetime(lang.with_context(lang='fr_FR').env, datetime_str, tz='Europe/Brussels'), '31 janv. 2017 à 11:33:00')
-        self.assertEqual(misc.format_datetime(lang.with_context(lang='zh_CN').env, datetime_str, tz='America/New_York'), '2017\u5E741\u670831\u65E5 \u4E0A\u53485:33:00')  # '2017年1月31日 上午5:33:00'
+        datetime_us_str = misc.format_datetime(lang.with_context(lang='en_US').env, datetime_str, tz='Europe/Brussels')
+        self.assertNotEqual(misc.format_datetime(lang.with_context(lang='fr_FR').env, datetime_str, tz='Europe/Brussels'), datetime_us_str)
+        self.assertNotEqual(misc.format_datetime(lang.with_context(lang='zh_CN').env, datetime_str, tz='America/New_York'), datetime_us_str)
 
         # Change language, timezone and format
         self.assertEqual(misc.format_datetime(lang.with_context(lang='fr_FR').env, datetime_str, tz='America/New_York', dt_format='short'), '31/01/2017 05:33')
@@ -257,20 +258,36 @@ class TestFormatLangDate(TransactionCase):
         time_part_tz = datetime.time(16, 30, 22, tzinfo=pytz.timezone('US/Eastern'))  # 4:30 PM timezoned
 
         self.assertEqual(misc.format_time(lang.with_context(lang='fr_FR').env, time_part), '16:30:22')
-        self.assertEqual(misc.format_time(lang.with_context(lang='zh_CN').env, time_part), '\u4e0b\u53484:30:22')
+        self.assertEqual(misc.format_time(lang.with_context(lang='zh_CN').env, time_part, time_format="ah:m:ss"), '\u4e0b\u53484:30:22')
 
         # Check format in different languages
         self.assertEqual(misc.format_time(lang.with_context(lang='fr_FR').env, time_part, time_format='short'), '16:30')
-        self.assertEqual(misc.format_time(lang.with_context(lang='zh_CN').env, time_part, time_format='short'), '\u4e0b\u53484:30')
+        self.assertEqual(misc.format_time(lang.with_context(lang='zh_CN').env, time_part, time_format='ah:mm'), '\u4e0b\u53484:30')
 
         # Check timezoned time part
         self.assertIn(misc.format_time(lang.with_context(lang='fr_FR').env, time_part_tz, time_format='long'), ['16:30:22 -0504', '16:30:22 HNE'])
-        self.assertEqual(misc.format_time(lang.with_context(lang='zh_CN').env, time_part_tz, time_format='full'), '\u5317\u7f8e\u4e1c\u90e8\u6807\u51c6\u65f6\u95f4\u0020\u4e0b\u53484:30:22')
+        self.assertEqual(misc.format_time(lang.with_context(lang='zh_CN').env, time_part_tz, time_format='zzzz ah:mm:ss'), '\u5317\u7f8e\u4e1c\u90e8\u6807\u51c6\u65f6\u95f4\u0020\u4e0b\u53484:30:22')
+
+        #Check timezone conversion in format_time
+        self.assertEqual(misc.format_time(lang.with_context(lang='fr_FR').env, datetime_str, 'Europe/Brussels', time_format='long'), '11:33:00 +0100')
+        self.assertEqual(misc.format_time(lang.with_context(lang='fr_FR').env, datetime_str, 'US/Eastern', time_format='long'), '05:33:00 HNE')
 
         # Check given `lang_code` overwites context lang
-        self.assertEqual(misc.format_time(lang.with_context(lang='fr_FR').env, time_part, time_format='short', lang_code='zh_CN'), '\u4e0b\u53484:30')
+        self.assertEqual(misc.format_time(lang.with_context(lang='fr_FR').env, time_part, time_format='ah:mm', lang_code='zh_CN'), '\u4e0b\u53484:30')
         self.assertEqual(misc.format_time(lang.with_context(lang='zh_CN').env, time_part, time_format='medium', lang_code='fr_FR'), '16:30:22')
 
+    def test_02_tz(self):
+        self.env.user.tz = 'Europe/Brussels'
+        datetime_str = '2016-12-31 23:55:00'
+        date_datetime = datetime.datetime.strptime(datetime_str, "%Y-%m-%d %H:%M:%S")
+
+        # While London is still in 2016, Brussels is already in 2017
+        self.assertEqual(misc.format_date(self.env, date_datetime), '01/01/2017')
+
+        # Force London timezone
+        date_datetime = date_datetime.replace(tzinfo=pytz.UTC)
+        self.assertEqual(misc.format_date(self.env, date_datetime), '12/31/2016', "User's tz must be ignored when tz is specifed in datetime object")
+
 
 class TestCallbacks(BaseCase):
     def test_callback(self):
@@ -357,3 +374,98 @@ class TestRemoveAccents(BaseCase):
     def test_non_latin(self):
         self.assertEqual(remove_accents('العربية'), 'العربية')
         self.assertEqual(remove_accents('русский алфавит'), 'русскии алфавит')
+
+
+class TestAddonsFileAccess(BaseCase):
+
+    def assertCannotAccess(self, path, ExceptionType=FileNotFoundError, filter_ext=None):
+        with self.assertRaises(ExceptionType):
+            file_path(path, filter_ext=filter_ext)
+
+    def assertCanRead(self, path, needle='', mode='r', filter_ext=None):
+        with file_open(path, mode, filter_ext) as f:
+            self.assertIn(needle, f.read())
+
+    def assertCannotRead(self, path, ExceptionType=FileNotFoundError, filter_ext=None):
+        with self.assertRaises(ExceptionType):
+            file_open(path, filter_ext=filter_ext)
+
+    def test_file_path(self):
+        # absolute path
+        self.assertEqual(__file__, file_path(__file__))
+        self.assertEqual(__file__, file_path(__file__, filter_ext=None)) # means "no filter" too
+        self.assertEqual(__file__, file_path(__file__, filter_ext=('.py',)))
+
+        # directory target is ok
+        self.assertEqual(os.path.dirname(__file__), file_path(os.path.join(__file__, '..')))
+
+        # relative path
+        relpath = os.path.join(*(__file__.split(os.sep)[-3:])) # 'base/tests/test_misc.py'
+        self.assertEqual(__file__, file_path(relpath))
+        self.assertEqual(__file__, file_path(relpath, filter_ext=('.py',)))
+
+        # leading 'addons/' is ignored if present
+        self.assertTrue(file_path("addons/web/__init__.py"))
+        relpath = os.path.join('addons', relpath) # 'addons/base/tests/test_misc.py'
+        self.assertEqual(__file__, file_path(relpath))
+
+        # files in root_path are allowed
+        self.assertTrue(file_path('tools/misc.py'))
+
+        # errors when outside addons_paths
+        self.assertCannotAccess('/doesnt/exist')
+        self.assertCannotAccess('/tmp')
+        self.assertCannotAccess('../../../../../../../../../tmp')
+        self.assertCannotAccess(os.path.join(__file__, '../../../../../'))
+
+        # data_dir is forbidden
+        self.assertCannotAccess(config['data_dir'])
+
+        # errors for illegal extensions
+        self.assertCannotAccess(__file__, ValueError, filter_ext=('.png',))
+        # file doesnt exist but has wrong extension
+        self.assertCannotAccess(__file__.replace('.py', '.foo'), ValueError, filter_ext=('.png',))
+
+    def test_file_open(self):
+        # The needle includes UTF8 so we test reading non-ASCII files at the same time.
+        # This depends on the system locale and is harder to unit test, but if you manage to run the
+        # test with a non-UTF8 locale (`LC_ALL=fr_FR.iso8859-1 python3...`) it should not crash ;-)
+        test_needle = "A needle with non-ascii bytes: ♥"
+
+        # absolute path
+        self.assertCanRead(__file__, test_needle)
+        self.assertCanRead(__file__, test_needle.encode(), mode='rb')
+        self.assertCanRead(__file__, test_needle.encode(), mode='rb', filter_ext=('.py',))
+
+        # directory target *is* an error
+        with self.assertRaises(FileNotFoundError):
+            file_open(os.path.join(__file__, '..'))
+
+        # relative path
+        relpath = os.path.join(*(__file__.split(os.sep)[-3:])) # 'base/tests/test_misc.py'
+        self.assertCanRead(relpath, test_needle)
+        self.assertCanRead(relpath, test_needle.encode(), mode='rb')
+        self.assertCanRead(relpath, test_needle.encode(), mode='rb', filter_ext=('.py',))
+
+        # leading 'addons/' is ignored if present
+        self.assertCanRead("addons/web/__init__.py", "import")
+        relpath = os.path.join('addons', relpath) # 'addons/base/tests/test_misc.py'
+        self.assertCanRead(relpath, test_needle)
+
+        # files in root_path are allowed
+        self.assertCanRead('tools/misc.py')
+
+        # errors when outside addons_paths
+        self.assertCannotRead('/doesnt/exist')
+        self.assertCannotRead('')
+        self.assertCannotRead('/tmp')
+        self.assertCannotRead('../../../../../../../../../tmp')
+        self.assertCannotRead(os.path.join(__file__, '../../../../../'))
+
+        # data_dir is forbidden
+        self.assertCannotRead(config['data_dir'])
+
+        # errors for illegal extensions
+        self.assertCannotRead(__file__, ValueError, filter_ext=('.png',))
+        # file doesnt exist but has wrong extension
+        self.assertCannotRead(__file__.replace('.py', '.foo'), ValueError, filter_ext=('.png',))
