@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-from odoo import SUPERUSER_ID
+from odoo import SUPERUSER_ID, Command
 from odoo.exceptions import AccessError
 from odoo.tests import common, TransactionCase
 
@@ -14,7 +14,7 @@ class Feedback(TransactionCase):
         self.user = self.env['res.users'].create({
             'login': 'bob',
             'name': "Bob Bobman",
-            'groups_id': [(6, 0, self.group2.ids)],
+            'groups_id': [Command.set(self.group2.ids)],
         })
 
 
@@ -30,7 +30,7 @@ class TestSudo(Feedback):
             'login': 'demo2',
             'password': 'demo2',
             'partner_id': partner_demo.id,
-            'groups_id': [(6, 0, [self.env.ref('base.group_user').id, self.env.ref('base.group_partner_manager').id])],
+            'groups_id': [Command.set([self.env.ref('base.group_user').id, self.env.ref('base.group_partner_manager').id])],
         })
 
         # with_user(user)
@@ -107,8 +107,8 @@ class TestACLFeedback(Feedback):
         })
         self.record = self.env['test_access_right.some_obj'].create({'val': 5})
         # values are in cache, clear them up for the test
-        ACL.flush()
-        ACL.invalidate_cache()
+        self.env.flush_all()
+        self.env.invalidate_all()
 
     def test_no_groups(self):
         """ Operation is never allowed
@@ -167,7 +167,7 @@ class TestIRRuleFeedback(Feedback):
         res = self.env['ir.rule'].create({
             'name': name,
             'model_id': self.model.id,
-            'groups': [] if global_ else [(4, self.group2.id)],
+            'groups': [] if global_ else [Command.link(self.group2.id)],
             'domain_force': domain,
             'perm_read': False,
             'perm_write': False,
@@ -188,8 +188,8 @@ class TestIRRuleFeedback(Feedback):
 Contact your administrator to request access if necessary.""")
 
         # debug mode
-        self.env.ref('base.group_no_one').write({'users': [(4, self.user.id)]})
-        self.env.ref('base.group_user').write({'users': [(4, self.user.id)]})
+        self.env.ref('base.group_no_one').write({'users': [Command.link(self.user.id)]})
+        self.env.ref('base.group_user').write({'users': [Command.link(self.user.id)]})
         with self.assertRaises(AccessError) as ctx:
             self.record.write({'val': 1})
         self.assertEqual(
@@ -215,8 +215,8 @@ Contact your administrator to request access if necessary.""" % (self.record.dis
             p.with_user(self.user).write({'val': 1})
 
     def test_locals(self):
-        self.env.ref('base.group_no_one').write({'users': [(4, self.user.id)]})
-        self.env.ref('base.group_user').write({'users': [(4, self.user.id)]})
+        self.env.ref('base.group_no_one').write({'users': [Command.link(self.user.id)]})
+        self.env.ref('base.group_user').write({'users': [Command.link(self.user.id)]})
         self._make_rule('rule 0', '[("val", "=", 42)]')
         self._make_rule('rule 1', '[("val", "=", 78)]')
         with self.assertRaises(AccessError) as ctx:
@@ -236,8 +236,8 @@ Contact your administrator to request access if necessary.""" % (self.record.dis
         )
 
     def test_globals_all(self):
-        self.env.ref('base.group_no_one').write({'users': [(4, self.user.id)]})
-        self.env.ref('base.group_user').write({'users': [(4, self.user.id)]})
+        self.env.ref('base.group_no_one').write({'users': [Command.link(self.user.id)]})
+        self.env.ref('base.group_user').write({'users': [Command.link(self.user.id)]})
         self._make_rule('rule 0', '[("val", "=", 42)]', global_=True)
         self._make_rule('rule 1', '[("val", "=", 78)]', global_=True)
         with self.assertRaises(AccessError) as ctx:
@@ -260,8 +260,8 @@ Contact your administrator to request access if necessary.""" % (self.record.dis
         """ Global rules are AND-eded together, so when an access fails it
         might be just one of the rules, and we want an exact listing
         """
-        self.env.ref('base.group_no_one').write({'users': [(4, self.user.id)]})
-        self.env.ref('base.group_user').write({'users': [(4, self.user.id)]})
+        self.env.ref('base.group_no_one').write({'users': [Command.link(self.user.id)]})
+        self.env.ref('base.group_user').write({'users': [Command.link(self.user.id)]})
         self._make_rule('rule 0', '[("val", "=", 42)]', global_=True)
         self._make_rule('rule 1', '[(1, "=", 1)]', global_=True)
         with self.assertRaises(AccessError) as ctx:
@@ -280,8 +280,8 @@ Contact your administrator to request access if necessary.""" % (self.record.dis
         )
 
     def test_combination(self):
-        self.env.ref('base.group_no_one').write({'users': [(4, self.user.id)]})
-        self.env.ref('base.group_user').write({'users': [(4, self.user.id)]})
+        self.env.ref('base.group_no_one').write({'users': [Command.link(self.user.id)]})
+        self.env.ref('base.group_user').write({'users': [Command.link(self.user.id)]})
         self._make_rule('rule 0', '[("val", "=", 42)]', global_=True)
         self._make_rule('rule 1', '[(1, "=", 1)]', global_=True)
         self._make_rule('rule 2', '[(0, "=", 1)]')
@@ -303,12 +303,13 @@ This restriction is due to the following rules:
 Contact your administrator to request access if necessary.""" % (self.record.display_name, self.record.id, self.user.name, self.user.id)
         )
 
-    def test_warn_company(self):
+    def test_warn_company_no_access(self):
         """ If one of the failing rules mentions company_id, add a note that
-        this might be a multi-company issue.
+        this might be a multi-company issue, but the user doesn't access to this company
+        then no information about the company is showed.
         """
-        self.env.ref('base.group_no_one').write({'users': [(4, self.user.id)]})
-        self.env.ref('base.group_user').write({'users': [(4, self.user.id)]})
+        self.env.ref('base.group_no_one').write({'users': [Command.link(self.user.id)]})
+        self.env.ref('base.group_user').write({'users': [Command.link(self.user.id)]})
         self._make_rule('rule 0', "[('company_id', '=', user.company_id.id)]")
         self._make_rule('rule 1', '[("val", "=", 0)]', global_=True)
         with self.assertRaises(AccessError) as ctx:
@@ -328,35 +329,69 @@ Note: this might be a multi-company issue.
 Contact your administrator to request access if necessary.""" % (self.record.display_name, self.record.id, self.user.name, self.user.id)
         )
 
-    def test_read(self):
-        """ because of prefetching, read() goes through a different codepath
-        to apply rules
+    def test_warn_company_no_company_field(self):
+        """ If one of the failing rules mentions company_id, add a note that
+        this might be a multi-company issue, but the record doesn't have company_id field
+        then no information about the company is showed.
         """
-        self.env.ref('base.group_no_one').write({'users': [(4, self.user.id)]})
-        self.env.ref('base.group_user').write({'users': [(4, self.user.id)]})
-        self._make_rule('rule 0', "[('company_id', '=', user.company_id.id)]", attr='read')
-        self._make_rule('rule 1', '[("val", "=", 1)]', global_=True, attr='read')
+        self.env.ref('base.group_no_one').write({'users': [Command.link(self.user.id)]})
+        self.env.ref('base.group_user').write({'users': [Command.link(self.user.id)]})
+        ChildModel = self.env['test_access_right.child'].sudo()
+        self.env['ir.rule'].create({
+            'name': 'rule 0',
+            'model_id': self.env['ir.model'].search([('model', '=', ChildModel._name)]).id,
+            'groups': [],
+            'domain_force': '[("parent_id.company_id", "=", user.company_id.id)]',
+            'perm_read': True,
+        })
+        self.record.sudo().company_id = self.env['res.company'].create({'name': 'Brosse Inc.'})
+        self.user.sudo().company_ids = [Command.link(self.record.company_id.id)]
+        child_record = ChildModel.create({'parent_id': self.record.id}).with_user(self.user)
         with self.assertRaises(AccessError) as ctx:
-            _ = self.record.val
+            _ = child_record.parent_id
         self.assertEqual(
             ctx.exception.args[0],
-            """Due to security restrictions, you are not allowed to access 'Object For Test Access Right' (test_access_right.some_obj) records.
+            """Due to security restrictions, you are not allowed to access 'Object for testing company ir rule' (test_access_right.child) records.
 
 Records: %s (id=%s)
 User: %s (id=%s)
 
 This restriction is due to the following rules:
 - rule 0
-- rule 1
 
 Note: this might be a multi-company issue.
 
-Contact your administrator to request access if necessary.""" % (self.record.display_name, self.record.id, self.user.name, self.user.id)
+Contact your administrator to request access if necessary.""" % (child_record.display_name, child_record.id, self.user.name, self.user.id)
         )
 
+    def test_warn_company_access(self):
+        """ because of prefetching, read() goes through a different codepath
+        to apply rules
+        """
+        self.env.ref('base.group_no_one').write({'users': [Command.link(self.user.id)]})
+        self.env.ref('base.group_user').write({'users': [Command.link(self.user.id)]})
+        self.record.sudo().company_id = self.env['res.company'].create({'name': 'Brosse Inc.'})
+        self.user.sudo().company_ids = [Command.link(self.record.company_id.id)]
+        self._make_rule('rule 0', "[('company_id', '=', user.company_id.id)]", attr='read')
+        with self.assertRaises(AccessError) as ctx:
+            _ = self.record.val
+        self.assertEqual(
+            ctx.exception.args[0],
+            """Due to security restrictions, you are not allowed to access 'Object For Test Access Right' (test_access_right.some_obj) records.
+
+Records: %s (id=%s, company=%s)
+User: %s (id=%s)
+
+This restriction is due to the following rules:
+- rule 0
+
+Note: this might be a multi-company issue.
+
+Contact your administrator to request access if necessary.""" % (self.record.display_name, self.record.id, self.record.sudo().company_id.display_name, self.user.name, self.user.id)
+        )
         p = self.env['test_access_right.parent'].create({'obj_id': self.record.id})
-        p.flush()
-        p.invalidate_cache()
+        self.env.flush_all()
+        self.env.invalidate_all()
         with self.assertRaisesRegex(
             AccessError,
             r"Implicitly accessed through 'Object for testing related access rights' \(test_access_right.parent\)\.",
@@ -374,7 +409,7 @@ class TestFieldGroupFeedback(Feedback):
 
     def test_read(self):
         self.env.ref('base.group_no_one').write(
-            {'users': [(4, self.user.id)]})
+            {'users': [Command.link(self.user.id)]})
         with self.assertRaises(AccessError) as ctx:
             _ = self.record.forbidden
 
@@ -406,7 +441,7 @@ Fields:
 
     def test_write(self):
         self.env.ref('base.group_no_one').write(
-            {'users': [(4, self.user.id)]})
+            {'users': [Command.link(self.user.id)]})
 
         with self.assertRaises(AccessError) as ctx:
             self.record.write({'forbidden': 1, 'forbidden2': 2})
