@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-
+import contextlib
 import json
 import pkgutil
 import re
@@ -9,6 +9,7 @@ from odoo import fields
 from odoo.addons.base.tests.common import SavepointCaseWithUserDemo
 from odoo.tests import common
 from odoo.tools.misc import mute_logger
+from odoo.tools.translate import code_translations
 
 def message(msg, type='error', from_=0, to_=0, record=0, field='value', **kwargs):
     return dict(kwargs,
@@ -40,8 +41,13 @@ class ImporterCase(common.TransactionCase):
         super(ImporterCase, self).setUp()
         self.model = self.env[self.model_name]
         self.env['ir.model.data'].clear_caches()
+        self.cr.cache.clear()
 
     def import_(self, fields, rows, context=None):
+        context = context or {}
+        context.update({
+            'import_file': True
+        })
         return self.model.with_context(context or {}).load(fields, rows)
 
     def read(self, fields=('value',), domain=(), context=None):
@@ -72,19 +78,6 @@ class ImporterCase(common.TransactionCase):
         })
         return '__test__.' + name
 
-    def add_translations(self, name, type, code, *tnx):
-        self.env['res.lang']._activate_lang(code)
-        Translations = self.env['ir.translation']
-        for source, value in tnx:
-            Translations.create({
-                'name': name,
-                'lang': code,
-                'type': type,
-                'src': source,
-                'value': value,
-                'state': 'translated',
-            })
-
 
 class test_ids_stuff(ImporterCase):
     model_name = 'export.integer'
@@ -148,18 +141,14 @@ class test_boolean_field(ImporterCase):
         ], values(records))
 
     def test_falses(self):
-        for lang, source, value in [('fr_FR', 'no', u'non'),
-                                    ('de_DE', 'no', u'nein'),
-                                    ('ru_RU', 'no', u'нет'),
-                                    ('nl_BE', 'false', u'vals'),
-                                    ('lt_LT', 'false', u'klaidingas')]:
-            self.add_translations('test_import.py', 'code', lang, (source, value))
+        for lang in ['fr_FR', 'de_DE', 'ru_RU', 'nl_BE', 'lt_LT']:
+            self.env['res.lang']._activate_lang(lang)
         falses = [[u'0'], [u'no'], [u'false'], [u'FALSE'], [u''],
-                  [u'non'], # no, fr
-                  [u'nein'], # no, de
-                  [u'нет'], # no, ru
-                  [u'vals'], # false, nl
-                  [u'klaidingas'], # false, lt,
+                  [u'faux'], # false, fr
+                  [u'falsch'], # false, de
+                  [u'ложь'], # no, ru
+                  [u'onwaar'], # false, nl
+                  [u'ne'], # false, lt,
         ]
 
         result = self.import_(['value'], falses)
@@ -168,20 +157,23 @@ class test_boolean_field(ImporterCase):
         self.assertEqual([False] * len(falses), values(self.read()))
 
     def test_trues(self):
+        # Since importing wrong boolean values is now returning error, import should not return any ids if an error is raised.
         trues = [['None'], ['nil'], ['()'], ['f'], ['#f'],
                   # Problem: OpenOffice (and probably excel) output localized booleans
                   ['VRAI'], ['ok'], ['true'], ['yes'], ['1'], ]
         result = self.import_(['value'], trues)
-        self.assertEqual(len(result['ids']), 10)
-        self.assertEqual(result['messages'], [
-            message(u"Unknown value '%s' for boolean field 'Value', assuming 'yes'" % v[0],
-                    moreinfo=u"Use '1' for yes and '0' for no",
-                    type='warning', from_=i, to_=i, record=i)
-            for i, v in enumerate(trues)
-            if v[0] not in ('true', 'yes', '1')
-        ])
+        self.assertEqual(result['ids'], False)
+        self.assertEqual(result['messages'], [{
+            'rows': {'from': i, 'to': i}, 'type': 'error', 'record': i, 'field': 'value',
+            'message': "Unknown value '%s' for boolean field 'Value'" % v[0],
+            'moreinfo': "Use '1' for yes and '0' for no", 'field_name': 'Value'
+        } for i, v in enumerate(trues) if v[0] not in ('true', 'yes', '1')])
+
+        # Only correct boolean values are accepted.
+        result = self.import_(['value'], [['1'], ['yes'], ['true']])
+        self.assertEqual(len(result['ids']), 3)
         self.assertEqual(
-            [True] * 10,
+            [True] * 3,
             values(self.read()))
 
 
@@ -264,6 +256,7 @@ class test_integer_field(ImporterCase):
         result = self.import_(['value'], [['zorglub']])
         self.assertIs(result['ids'], False)
         self.assertEqual(result['messages'], [{
+            'field_name': 'Value',
             'type': 'error',
             'rows': {'from': 0, 'to': 0},
             'record': 0,
@@ -335,7 +328,7 @@ class test_float_field(ImporterCase):
         result = self.import_(['value'], [['foobar']])
         self.assertIs(result['ids'], False)
         self.assertEqual(result['messages'], [
-            message(u"'foobar' does not seem to be a number for field 'Value'")])
+            message(u"'foobar' does not seem to be a number for field 'Value'", field_name='Value')])
 
 class test_string_field(ImporterCase):
     model_name = 'export.string.bounded'
@@ -439,11 +432,11 @@ class test_text(ImporterCase):
 
 class test_selection(ImporterCase):
     model_name = 'export.selection'
-    translations_fr = [
-        ("Foo", "tete"),
-        ("Bar", "titi"),
-        ("Qux", "toto"),
-    ]
+    translations_fr = {
+        "Foo": "tete",
+        "Bar": "titi",
+        "Qux": "toto",
+    }
 
     def test_imported(self):
         result = self.import_(['value'], [
@@ -457,8 +450,11 @@ class test_selection(ImporterCase):
         self.assertEqual(['3', '2', '1', '2'], values(self.read()))
 
     def test_imported_translated(self):
-        self.add_translations(
-            'ir.model.fields.selection,name', 'model', 'fr_FR', *self.translations_fr)
+        self.env['res.lang']._activate_lang('fr_FR')
+        # add translations for selection
+        selections = self.env['ir.model.fields.selection'].search([('name', 'in', list(self.translations_fr))])
+        for selection in selections:
+            selection.with_context(lang='fr_FR').name = self.translations_fr[selection.name]
 
         result = self.import_(['value'], [
             ['toto'],
@@ -479,13 +475,14 @@ class test_selection(ImporterCase):
         self.assertIs(result['ids'], False)
         self.assertEqual(result['messages'], [message(
             u"Value 'Baz' not found in selection field 'Value'",
-            moreinfo="Foo Bar Qux 4".split())])
+            moreinfo="Foo Bar Qux 4".split(), field_name='Value', field_path=['value'])])
 
         result = self.import_(['value'], [['42']])
         self.assertIs(result['ids'], False)
         self.assertEqual(result['messages'], [message(
             u"Value '42' not found in selection field 'Value'",
-            moreinfo="Foo Bar Qux 4".split())])
+            moreinfo="Foo Bar Qux 4".split(),
+            field_name='Value', field_path=['value'])])
 
 
 class test_selection_with_default(ImporterCase):
@@ -516,12 +513,12 @@ class test_selection_with_default(ImporterCase):
 
 class test_selection_function(ImporterCase):
     model_name = 'export.selection.function'
-    translations_fr = [
-        ("Corge", "toto"),
-        ("Grault", "titi"),
-        ("Wheee", "tete"),
-        ("Moog", "tutu"),
-    ]
+    translations_fr = {
+        "Corge": "toto",
+        "Grault": "titi",
+        "Wheee": "tete",
+        "Moog": "tutu",
+    }
 
     def test_imported(self):
         """ import uses fields_get, so translates import label (may or may not
@@ -540,8 +537,10 @@ class test_selection_function(ImporterCase):
     def test_translated(self):
         """ Expects output of selection function returns translated labels
         """
-        self.add_translations(
-            'ir.model.fields.selection,name', 'model', 'fr_FR', *self.translations_fr)
+        self.env['res.lang']._activate_lang('fr_FR')
+        # hack into code translations
+        code_translations.python_translations[('test_impex', 'fr_FR')] = self.translations_fr
+        self.addCleanup(code_translations.python_translations.pop, ('test_impex', 'fr_FR'))
 
         result = self.import_(['value'], [
             ['titi'],
@@ -569,18 +568,17 @@ class test_m2o(ImporterCase):
 
         # preheat the oven
         for _ in range(5):
-            self.env.cr.execute('SAVEPOINT xxx')
-            self.import_(['value'], [[name1], [name1], [name2]])
-            self.env.cr.execute('ROLLBACK TO SAVEPOINT xxx')
-            self.env.cr.execute('RELEASE SAVEPOINT xxx')
+            with contextlib.closing(self.env.cr.savepoint(flush=False)):
+                self.import_(['value'], [[name1], [name1], [name2]])
 
-        # 1 x SAVEPOINT model_load
+        # 1 x SAVEPOINT load
         # 3 x name_search
-        # 1 x SAVEPOINT
+        # 1 x SAVEPOINT _load_records
         # 3 x insert
-        # 1 x RELEASE SAVEPOINT
-        # => 9
-        with self.assertQueryCount(9):
+        # 1 x RELEASE SAVEPOINT _load_records
+        # 1 x RELEASE SAVEPOINT load
+        # => 10
+        with self.assertQueryCount(8):
             result = self.import_(['value'], [
                 # import by name_get
                 [name1],
@@ -626,7 +624,7 @@ class test_m2o(ImporterCase):
         result = self.import_(['value'], [[name2]])
         self.assertEqual(
             result['messages'],
-            [message(u"Found multiple matches for field 'Value' (2 matches)",
+            [message(u"Found multiple matches for value 'export.integer:42' in field 'Value' (2 matches)",
                      type='warning')])
         self.assertEqual(len(result['ids']), 1)
         self.assertEqual([
@@ -650,7 +648,8 @@ class test_m2o(ImporterCase):
         self.assertEqual(result['messages'], [
             message(u"No matching record found for name '%s' in field 'Value'" % id,
                     from_=index, to_=index, record=index,
-                    moreinfo=moreaction(res_model='export.integer'))
+                    moreinfo=moreaction(res_model='export.integer'), field_name='Value', field_path=['value'],
+                    field_type='name', value=id)
             for index, id in enumerate([record1.id, record2.id, record1.id])])
         self.assertIs(result['ids'], False)
 
@@ -661,7 +660,8 @@ class test_m2o(ImporterCase):
         self.assertEqual(result['messages'], [
             message(u"Invalid database id 'foo' for the field 'Value'",
                     moreinfo=moreaction(res_model='ir.model.data',
-                                        domain=[('model','=','export.integer')]))
+                                        domain=[('model', '=', 'export.integer')]),
+                    field_name='Value', field_path=['value', '.id'])
         ])
         self.assertIs(result['ids'], False)
 
@@ -680,21 +680,25 @@ class test_m2o(ImporterCase):
         self.assertEqual(result['messages'], [message(
             u"No matching record found for name 'nameisnoexist:3' "
             u"in field 'Value'", moreinfo=moreaction(
-                res_model='export.integer'))])
+                res_model='export.integer'),
+            field_name='Value', field_path=['value'], field_type='name', value='nameisnoexist:3'
+        )])
         self.assertIs(result['ids'], False)
 
         result = self.import_(['value/id'], [['noxidhere']])
         self.assertEqual(result['messages'], [message(
             u"No matching record found for external id 'noxidhere' "
             u"in field 'Value'", moreinfo=moreaction(
-                res_model='ir.model.data', domain=[('model','=','export.integer')]))])
+                res_model='ir.model.data', domain=[('model', '=', 'export.integer')]),
+            field_name='Value', field_path=['value', 'id'], field_type="external id", value="noxidhere")])
         self.assertIs(result['ids'], False)
 
         result = self.import_(['value/.id'], [['66']])
         self.assertEqual(result['messages'], [message(
             u"No matching record found for database id '66' "
             u"in field 'Value'", moreinfo=moreaction(
-                res_model='ir.model.data', domain=[('model','=','export.integer')]))])
+                res_model='ir.model.data', domain=[('model', '=', 'export.integer')]),
+            field_name='Value', field_path=['value', '.id'], field_type="database id", value="66")])
         self.assertIs(result['ids'], False)
 
     def test_fail_multiple(self):
@@ -711,7 +715,8 @@ class test_m2o(ImporterCase):
         self.assertEqual(result['messages'], [message(
             u"No matching record found for name '101' "
             u"in field 'Value'", moreinfo=moreaction(
-                res_model='export.integer'))])
+                res_model='export.integer'),
+            field_name='Value', field_path=['value'], field_type='name', value=101)])
         self.assertIs(result['ids'], False)
         context = {
             'name_create_enabled_fields': {'value': True},
@@ -720,6 +725,20 @@ class test_m2o(ImporterCase):
         self.assertFalse(result['messages'])
         self.assertEqual(len(result['ids']), 1)
 
+class TestInvalidStrings(ImporterCase):
+    model_name = 'export.m2o.str'
+
+    @mute_logger('odoo.sql_db')
+    def test_fail_unpaired_surrogate(self):
+        result = self.import_(['child_id'], [['\uddff']])
+        self.assertTrue(result['messages'])
+        self.assertIn('surrogates', result['messages'][0]['message'])
+
+    @mute_logger('odoo.sql_db')
+    def test_fail_nul(self):
+        result = self.import_(['child_id'], [['\x00']])
+        self.assertTrue(result['messages'])
+        self.assertIn('NUL', result['messages'][0]['message'])
 
 class test_m2m(ImporterCase):
     model_name = 'export.many2many'
@@ -757,7 +776,8 @@ class test_m2m(ImporterCase):
         self.assertEqual(result['messages'], [message(
             u"No matching record found for database id '42' in field "
             u"'Value'", moreinfo=moreaction(
-                res_model='ir.model.data', domain=[('model','=','export.many2many.other')]))])
+                res_model='ir.model.data', domain=[('model', '=', 'export.many2many.other')]),
+            field_name='Value', field_path=['value', '.id'], field_type="database id", value='42')])
         self.assertIs(result['ids'], False)
 
     def test_xids(self):
@@ -783,7 +803,8 @@ class test_m2m(ImporterCase):
         self.assertEqual(result['messages'], [message(
             u"No matching record found for external id 'noxidforthat' in field"
             u" 'Value'", moreinfo=moreaction(
-                res_model='ir.model.data', domain=[('model','=','export.many2many.other')]))])
+                res_model='ir.model.data', domain=[('model', '=', 'export.many2many.other')]),
+            field_name='Value', field_path=['value', 'id'], field_type='external id', value='noxidforthat')])
         self.assertIs(result['ids'], False)
 
     def test_names(self):
@@ -811,7 +832,8 @@ class test_m2m(ImporterCase):
         self.assertEqual(result['messages'], [message(
             u"No matching record found for name 'wherethem2mhavenonames' in "
             u"field 'Value'", moreinfo=moreaction(
-                res_model='export.many2many.other'))])
+                res_model='export.many2many.other'),
+            field_name='Value', field_path=['value'], field_type="name", value='wherethem2mhavenonames')])
         self.assertIs(result['ids'], False)
 
     def test_import_to_existing(self):
@@ -844,8 +866,9 @@ class test_o2m(ImporterCase):
             ['const', 'value'],
             [['5', s]])
         self.assertEqual(result['messages'], [message(
-            u"No matching record found for name '%s' in field 'Value'" % s,
-            moreinfo=moreaction(res_model='export.one2many.child'))])
+            u"No matching record found for name '%s' in field 'Value'" % s[:50],
+            moreinfo=moreaction(res_model='export.one2many.child'),
+            field_name='Value', field_path=['value'], field_type='name', value=s[:50])])
         self.assertIs(result['ids'], False)
 
     def test_single(self):
@@ -906,8 +929,8 @@ class test_o2m(ImporterCase):
         result = self.import_(['value/parent_id'], [['noxidforthat']])
         self.assertEqual(result['messages'], [message(
             u"No matching record found for name 'noxidforthat' in field 'Value/Parent'",
-            moreinfo=moreaction(res_model='export.one2many')
-            )])
+            moreinfo=moreaction(res_model='export.one2many'),
+            field_name='Value', field_path=['value', 'parent_id'], field_type='name', value='noxidforthat')])
         self.assertIs(result['ids'], False)
 
     def test_link_inline(self):
@@ -998,7 +1021,8 @@ class test_o2m(ImporterCase):
         self.assertEqual(result['messages'], [message(
             u"No matching record found for name '101' "
             u"in field 'Value/M2O'", moreinfo=moreaction(
-                res_model='export.integer'))])
+                res_model='export.integer'),
+            field_name='Value', field_path=['value', 'm2o'], field_type='name', value=101)])
         self.assertEqual(result['ids'], False)
         context = {
             'name_create_enabled_fields': {'value/m2o': True},
@@ -1009,6 +1033,14 @@ class test_o2m(ImporterCase):
         [b] = self.browse()
         self.assertEqual(b.value.m2o.value, 101)
 
+    def test_escape_m2o_in_o2m(self):
+        result = self.import_(['value/m2o'], [['21%']])
+        self.assertEqual(result['messages'], [message(
+            u"No matching record found for name '21%' "
+            u"in field 'Value/M2O'", moreinfo=moreaction(
+                res_model='export.integer'),
+            field_name='Value', field_path=['value', 'm2o'], field_type='name', value='21%')])
+
 
 class test_o2m_multiple(ImporterCase):
     model_name = 'export.one2many.multiple'
@@ -1116,13 +1148,14 @@ class test_realworld(SavepointCaseWithUserDemo):
     def test_o2m_subfields_fail_by_implicit_id(self):
         self.env['ir.model.data'].clear_caches()
         Model = self.env['export.one2many.recursive']
-        result = Model.load(
+        result = Model.with_context(import_file=True).load(
             ['child/child1/parent_id'],
             [['5'],],
         )
         self.assertEqual(result['messages'], [message(
             u"No matching record found for name '5' in field 'Child/Child1/Parent'", field='child',
-            moreinfo=moreaction(res_model='export.one2many.multiple'))])
+            moreinfo=moreaction(res_model='export.one2many.multiple'),
+            field_name='Child', field_path=['child', 'child1', 'parent_id'], field_type='name', value='5')])
         self.assertIs(result['ids'], False)
 
 
@@ -1144,7 +1177,7 @@ class test_date(ImporterCase):
         self.assertEqual(result['messages'], [
             message(u"'not really a date' does not seem to be a valid date "
                     u"for field 'Value'",
-                    moreinfo=u"Use the format '2012-12-31'")])
+                    moreinfo=u"Use the format '2012-12-31'", field_name='Value', field_path=['value'])])
         self.assertIs(result['ids'], False)
 
 
@@ -1166,7 +1199,7 @@ class test_datetime(ImporterCase):
         self.assertEqual(result['messages'], [
             message(u"'not really a datetime' does not seem to be a valid "
                     u"datetime for field 'Value'",
-                    moreinfo=u"Use the format '2012-12-31 23:59:59'")])
+                    moreinfo=u"Use the format '2012-12-31 23:59:59'", field_name='Value', field_path=['value'])])
         self.assertIs(result['ids'], False)
 
     def test_checktz1(self):
