@@ -29,17 +29,17 @@ class TestECMAScriptVersion(lint_case.LintCase):
     def test_ecmascript_version(self):
         """ Test that there is no unsupported ecmascript in javascript files """
 
-        black_re = re.compile(r'summernote.+(intro\.js|outro.js)$')
-
         files_to_check = [
             p for p in self.iter_module_files('*.js')
-            if not 'static/test' in p
-            if not 'static/src/tests' in p
-            if not black_re.search(p)
+            if 'static/test' not in p
+            if 'static/src/tests' not in p
+            if 'static/lib/qweb/qweb.js' not in p   # because this file is not bundled at all
+            if 'py.js/lib/py.js' not in p           # because it is not "strict" compliant
+            if 'static/lib/epos-2.12.0.js' not in p # same
         ]
 
         _logger.info('Testing %s js files', len(files_to_check))
-        cmd = [es_check, MAX_ES_VERSION] + files_to_check
+        cmd = [es_check, MAX_ES_VERSION] + files_to_check + ['--module']
         process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         out, err = process.communicate()
         self.assertEqual(process.returncode, 0, msg=out.decode())
