@@ -5,6 +5,7 @@ from unittest.mock import patch
 
 from odoo.addons.base.tests.common import SavepointCaseWithUserDemo
 from odoo.tests import common, Form
+from odoo import Command
 
 def strip_prefix(prefix, names):
     size = len(prefix)
@@ -48,7 +49,7 @@ class TestOnChange(SavepointCaseWithUserDemo):
             'author': USER.id,
             'size': 0,
         }
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         result = self.Message.onchange(values, 'discussion', field_onchange)
         self.assertIn('name', result['value'])
         self.assertEqual(result['value']['name'], "[%s] %s" % (discussion.name, USER.name))
@@ -61,7 +62,7 @@ class TestOnChange(SavepointCaseWithUserDemo):
             'author': USER.id,
             'size': 0,
         }
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         result = self.Message.onchange(values, 'body', field_onchange)
         self.assertIn('size', result['value'])
         self.assertEqual(result['value']['size'], len(BODY))
@@ -75,7 +76,7 @@ class TestOnChange(SavepointCaseWithUserDemo):
             'author': USER.id,
             'size': 0,
         }
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         result = self.Message.onchange(values, 'body', field_onchange)
         self.assertNotIn('name', result['value'])
 
@@ -93,7 +94,7 @@ class TestOnChange(SavepointCaseWithUserDemo):
             'root_categ': False,
         }
 
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         result = Category.onchange(values, 'parent', field_onchange).get('value', {})
         self.assertIn('root_categ', result)
         self.assertEqual(result['root_categ'], root.name_get()[0])
@@ -101,7 +102,7 @@ class TestOnChange(SavepointCaseWithUserDemo):
         values.update(result)
         values['parent'] = False
 
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         result = Category.onchange(values, 'parent', field_onchange).get('value', {})
         self.assertIn('root_categ', result)
         self.assertIs(result['root_categ'], False)
@@ -130,10 +131,10 @@ class TestOnChange(SavepointCaseWithUserDemo):
             'moderator': False,
             'participants': [],
             'messages': [
-                (4, message1.id),
-                (4, message2.id),
-                (1, message2.id, {'body': "XYZ"}),
-                (0, 0, {
+                Command.link(message1.id),
+                Command.link(message2.id),
+                Command.update(message2.id, {'body': "XYZ"}),
+                Command.create({
                     'name': "[%s] %s" % ('', USER.name),
                     'body': "ABC",
                     'author': USER.id,
@@ -142,26 +143,26 @@ class TestOnChange(SavepointCaseWithUserDemo):
                 }),
             ],
         }
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         result = self.Discussion.onchange(values, 'name', field_onchange)
         self.assertIn('messages', result['value'])
         self.assertEqual(result['value']['messages'], [
-            (5,),
-            (1, message1.id, {
+            Command.clear(),
+            Command.update(message1.id, {
                 'name': "[%s] %s" % ("Foo", USER.name),
                 'body': "ABC",
                 'author': USER.name_get()[0],
                 'size': 3,
                 'important': False,
             }),
-            (1, message2.id, {
+            Command.update(message2.id, {
                 'name': "[%s] %s" % ("Foo", USER.name),
                 'body': "XYZ",          # this must be sent back
                 'author': USER.name_get()[0],
                 'size': 3,
                 'important': False,
             }),
-            (0, 0, {
+            Command.create({
                 'name': "[%s] %s" % ("Foo", USER.name),
                 'body': "ABC",
                 'author': USER.name_get()[0],
@@ -175,11 +176,11 @@ class TestOnChange(SavepointCaseWithUserDemo):
         values = dict(values, name='{generate_dummy_message}')
         result = self.Discussion.with_context(generate_dummy_message=True).onchange(values, 'name', one_level_fields)
         self.assertEqual(result['value']['messages'], [
-            (5,),
-            (4, message1.id),
-            (4, message2.id),
-            (0, 0, {}),
-            (0, 0, {}),
+            Command.clear(),
+            Command.link(message1.id),
+            Command.link(message2.id),
+            Command.create({}),
+            Command.create({}),
         ])
 
     def test_onchange_one2many_reference(self):
@@ -213,11 +214,11 @@ class TestOnChange(SavepointCaseWithUserDemo):
                 }),
             ],
         }
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         result = self.Discussion.onchange(values, 'name', field_onchange)
         self.assertIn('messages', result['value'])
         self.assertItemsEqual(result['value']['messages'], [
-            (5,),
+            (5, 0, 0),
             (0, REFERENCE, {
                 'name': "[%s] %s" % ("Foo", USER.name),
                 'body': BODY,
@@ -248,7 +249,7 @@ class TestOnChange(SavepointCaseWithUserDemo):
         self.assertEqual(values, {
             'name': partner1.name,
             'partner': partner1.id,
-            'lines': [(6, 0, [line1.id])],
+            'lines': [Command.set([line1.id])],
         })
 
         # modify 'partner'
@@ -259,25 +260,25 @@ class TestOnChange(SavepointCaseWithUserDemo):
         values = {
             'name': partner1.name,
             'partner': partner2.id,             # this one just changed
-            'lines': [(6, 0, [line1.id]),
-                      (0, 0, {'name': False, 'partner': False, 'tags': [(5,)]})],
+            'lines': [Command.set([line1.id]),
+                      Command.create({'name': False, 'partner': False, 'tags': [Command.clear()]})],
         }
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
 
         result = multi.onchange(values, 'partner', field_onchange)
         self.assertEqual(result['value'], {
             'name': partner2.name,
             'lines': [
-                (5,),
-                (1, line1.id, {
+                Command.clear(),
+                Command.update(line1.id, {
                     'name': partner2.name,
                     'partner': (partner2.id, partner2.name),
-                    'tags': [(5,)],
+                    'tags': [Command.clear()],
                 }),
-                (0, 0, {
+                Command.create({
                     'name': partner2.name,
                     'partner': (partner2.id, partner2.name),
-                    'tags': [(5,)],
+                    'tags': [Command.clear()],
                 }),
             ],
         })
@@ -286,39 +287,39 @@ class TestOnChange(SavepointCaseWithUserDemo):
         values = {
             'name': partner1.name,
             'partner': partner2.id,             # this one just changed
-            'lines': [(6, 0, [line1.id]),
-                      (0, 0, {'name': False,
+            'lines': [Command.set([line1.id]),
+                      Command.create({'name': False,
                               'partner': False,
-                              'tags': [(5,), (0, 0, {'name': 'Tag'})]})],
+                              'tags': [Command.clear(), Command.create({'name': 'Tag'})]})],
         }
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         result = multi.onchange(values, 'partner', field_onchange)
         expected_value = {
             'name': partner2.name,
             'lines': [
-                (5,),
-                (1, line1.id, {
+                Command.clear(),
+                Command.update(line1.id, {
                     'name': partner2.name,
                     'partner': (partner2.id, partner2.name),
-                    'tags': [(5,)],
+                    'tags': [Command.clear()],
                 }),
-                (0, 0, {
+                Command.create({
                     'name': partner2.name,
                     'partner': (partner2.id, partner2.name),
-                    'tags': [(5,), (0, 0, {'name': 'Tag'})],
+                    'tags': [Command.clear(), Command.create({'name': 'Tag'})],
                 }),
             ],
         }
         self.assertEqual(result['value'], expected_value)
 
         # ensure ID is not returned when asked and a many2many record is set to be created
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
 
         result = multi.onchange(values, 'partner', dict(field_onchange, **{'lines.tags.id': None}))
         self.assertEqual(result['value'], expected_value)
 
         # ensure inverse of one2many field is not returned
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
 
         result = multi.onchange(values, 'partner', dict(field_onchange, **{'lines.multi': None}))
         self.assertEqual(result['value'], expected_value)
@@ -343,17 +344,17 @@ class TestOnChange(SavepointCaseWithUserDemo):
         values = {
             'name': discussion.name,
             'moderator': demo.id,
-            'categories': [(4, cat.id) for cat in discussion.categories],
-            'messages': [(4, msg.id) for msg in discussion.messages],
-            'participants': [(4, usr.id) for usr in discussion.participants],
+            'categories': [Command.link(cat.id) for cat in discussion.categories],
+            'messages': [Command.link(msg.id) for msg in discussion.messages],
+            'participants': [Command.link(usr.id) for usr in discussion.participants],
         }
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         result = discussion.onchange(values, 'moderator', field_onchange)
 
         self.assertIn('participants', result['value'])
         self.assertItemsEqual(
             result['value']['participants'],
-            [(5,)] + [(4, user.id) for user in discussion.participants + demo],
+            [Command.clear()] + [Command.link(user.id) for user in discussion.participants + demo],
         )
 
     def test_onchange_default(self):
@@ -367,13 +368,13 @@ class TestOnChange(SavepointCaseWithUserDemo):
         self.env['ir.default'].set('test_new_api.foo', 'value2', 666, condition='value1=42')
 
         # setting 'value1' to 42 should trigger the change of 'value2'
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         values = {'name': 'X', 'value1': 42, 'value2': False}
         result = Foo.onchange(values, 'value1', field_onchange)
         self.assertEqual(result['value'], {'value2': 666})
 
         # setting 'value1' to 24 should not trigger the change of 'value2'
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         values = {'name': 'X', 'value1': 24, 'value2': False}
         result = Foo.onchange(values, 'value1', field_onchange)
         self.assertEqual(result['value'], {})
@@ -400,16 +401,16 @@ class TestOnChange(SavepointCaseWithUserDemo):
         self.assertEqual(field_onchange.get('messages'), '1')
 
         self.assertEqual(len(discussion.messages), 3)
-        messages = [(4, msg.id) for msg in discussion.messages]
+        messages = [Command.link(msg.id) for msg in discussion.messages]
         messages[0] = (1, messages[0][1], {'body': 'test onchange'})
         lines = ["%s:%s" % (m.name, m.body) for m in discussion.messages]
         lines[0] = "%s:%s" % (discussion.messages[0].name, 'test onchange')
         values = {
             'name': discussion.name,
             'moderator': demo.id,
-            'categories': [(4, cat.id) for cat in discussion.categories],
+            'categories': [Command.link(cat.id) for cat in discussion.categories],
             'messages': messages,
-            'participants': [(4, usr.id) for usr in discussion.participants],
+            'participants': [Command.link(usr.id) for usr in discussion.participants],
             'message_concat': False,
         }
         result = discussion.onchange(values, 'messages', field_onchange)
@@ -423,9 +424,7 @@ class TestOnChange(SavepointCaseWithUserDemo):
 
         # mimic UI behaviour, so we get subfields
         # (we need at least subfield: 'important_emails.important')
-        view_info = self.Discussion.fields_view_get(
-            view_id=self.env.ref('test_new_api.discussion_form').id,
-            view_type='form')
+        view_info = self.Discussion.get_view(self.env.ref('test_new_api.discussion_form').id, 'form')
         field_onchange = self.Discussion._onchange_spec(view_info=view_info)
         self.assertEqual(field_onchange.get('messages'), '1')
 
@@ -443,7 +442,7 @@ class TestOnChange(SavepointCaseWithUserDemo):
         })
 
         # check if server-side cache is working correctly
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         self.assertIn(email, discussion.emails)
         self.assertNotIn(email, discussion.important_emails)
         email.important = True
@@ -452,23 +451,23 @@ class TestOnChange(SavepointCaseWithUserDemo):
         # check that when trigger an onchange, we don't reset important emails
         # (force `invalidate` as but appear in onchange only when we get a cache
         # miss)
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         self.assertEqual(len(discussion.messages), 4)
         values = {
             'name': "Foo Bar",
             'moderator': demo.id,
-            'categories': [(4, cat.id) for cat in discussion.categories],
-            'messages': [(4, msg.id) for msg in discussion.messages],
-            'participants': [(4, usr.id) for usr in discussion.participants],
-            'important_messages': [(4, msg.id) for msg in discussion.important_messages],
-            'important_emails': [(4, eml.id) for eml in discussion.important_emails],
+            'categories': [Command.link(cat.id) for cat in discussion.categories],
+            'messages': [Command.link(msg.id) for msg in discussion.messages],
+            'participants': [Command.link(usr.id) for usr in discussion.participants],
+            'important_messages': [Command.link(msg.id) for msg in discussion.important_messages],
+            'important_emails': [Command.link(eml.id) for eml in discussion.important_emails],
         }
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         result = discussion.onchange(values, 'name', field_onchange)
 
         self.assertEqual(
             result['value']['important_emails'],
-            [(5,), (1, email.id, {
+            [Command.clear(), Command.update(email.id, {
                 'name': u'[Foo Bar] %s' % USER.name,
                 'body': BODY,
                 'author': USER.name_get()[0],
@@ -495,15 +494,15 @@ class TestOnChange(SavepointCaseWithUserDemo):
             'message_currency': self.env.user.name_get()[0],
         }
 
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         Message = self.env['test_new_api.related']
-        result = Message.onchange(value, ['message', 'message_name', 'message_currency'], field_onchange)
+        result = Message.onchange(value, 'message', field_onchange)
 
         self.assertEqual(result['value'], onchange_result)
 
-        self.env.cache.invalidate()
+        self.env.invalidate_all()
         Message = self.env(user=self.user_demo.id)['test_new_api.related']
-        result = Message.onchange(value, ['message', 'message_name', 'message_currency'], field_onchange)
+        result = Message.onchange(value, 'message', field_onchange)
 
         self.assertEqual(result['value'], onchange_result)
 
@@ -531,35 +530,123 @@ class TestOnChange(SavepointCaseWithUserDemo):
 
         # changing 'discussion' on message should not read 'messages' on discussion
         with patch.object(type(discussion), 'read', mock_read, create=True):
-            self.env.cache.invalidate()
+            self.env.invalidate_all()
             self.Message.onchange(values, 'discussion', field_onchange)
 
         self.assertFalse(called[0], "discussion.messages has been read")
 
+    def test_onchange_one2many_many2one_in_form(self):
+        order = self.env['test_new_api.monetary_order'].create({
+            'currency_id': self.env.ref('base.USD').id,
+        })
+
+        # this call to onchange() is made when creating a new line in field
+        # order.line_ids; check what happens when the line's form view contains
+        # the inverse many2one field
+        values = {'order_id': {'id': order.id, 'currency_id': order.currency_id.id}}
+        field_onchange = dict.fromkeys(['order_id', 'subtotal'], '')
+        result = self.env['test_new_api.monetary_order_line'].onchange(values, [], field_onchange)
+
+        self.assertEqual(result['value']['order_id'], (order.id, order.display_name))
+
+    def test_onchange_inherited(self):
+        """ Setting an inherited field should assign the field on the parent record. """
+        foo, bar = self.env['test_new_api.multi.tag'].create([{'name': 'Foo'}, {'name': 'Bar'}])
+        view = self.env['ir.ui.view'].create({
+            'name': 'Payment form view',
+            'model': 'test_new_api.payment',
+            'arch': """
+                <form>
+                    <field name="move_id" readonly="1" required="0"/>
+                    <field name="tag_id"/>
+                    <field name="tag_name"/>
+                    <field name="tag_repeat"/>
+                    <field name="tag_string"/>
+                </form>
+            """,
+        })
+
+        # both fields 'tag_id' and 'tag_name' are inherited through 'move_id';
+        # assigning 'tag_id' should modify 'move_id.tag_id' accordingly, which
+        # should in turn recompute `move.tag_name` and `tag_name`
+        form = Form(self.env['test_new_api.payment'], view)
+        self.assertEqual(form.tag_name, False)
+        form.tag_id = foo
+        self.assertEqual(form.tag_name, 'Foo')
+        self.assertEqual(form.tag_string, '')
+        form.tag_repeat = 2
+        self.assertEqual(form.tag_name, 'Foo')
+        self.assertEqual(form.tag_string, 'FooFoo')
+
+        payment = form.save()
+        self.assertEqual(payment.tag_id, foo)
+        self.assertEqual(payment.tag_name, 'Foo')
+        self.assertEqual(payment.tag_repeat, 2)
+        self.assertEqual(payment.tag_string, 'FooFoo')
+
+        with Form(payment, view) as form:
+            form.tag_id = bar
+            self.assertEqual(form.tag_name, 'Bar')
+            self.assertEqual(form.tag_string, 'BarBar')
+            form.tag_repeat = 3
+            self.assertEqual(form.tag_name, 'Bar')
+            self.assertEqual(form.tag_string, 'BarBarBar')
+
+        self.assertEqual(payment.tag_id, bar)
+        self.assertEqual(payment.tag_name, 'Bar')
+        self.assertEqual(payment.tag_repeat, 3)
+        self.assertEqual(payment.tag_string, 'BarBarBar')
+
+    def test_display_name(self):
+        self.env['ir.ui.view'].create({
+            'name': 'test_new_api.multi.tag form view',
+            'model': 'test_new_api.multi.tag',
+            'arch': """
+                <form>
+                    <field name="name"/>
+                    <field name="display_name"/>
+                </form>
+            """,
+        })
+
+        form = common.Form(self.env['test_new_api.multi.tag'])
+        self.assertEqual(form.name, False)
+        self.assertEqual(form.display_name, "")
+
+        record = form.save()
+        self.assertEqual(record.name, False)
+        self.assertEqual(record.display_name, "")
+        self.assertEqual(record.name_get(), [(record.id, "")])
+
 
 class TestComputeOnchange(common.TransactionCase):
 
     def test_create(self):
         model = self.env['test_new_api.compute.onchange']
 
+        # compute 'bar' (readonly) and 'baz' (editable)
+        record = model.create({'active': True})
+        self.assertEqual(record.bar, "r")
+        self.assertEqual(record.baz, "z")
+
         # compute 'bar' and 'baz'
         record = model.create({'active': True, 'foo': "foo"})
-        self.assertEqual(record.bar, "foo")
-        self.assertEqual(record.baz, "foo")
+        self.assertEqual(record.bar, "foor")
+        self.assertEqual(record.baz, "fooz")
 
         # compute 'bar' but not 'baz'
         record = model.create({'active': True, 'foo': "foo", 'bar': "bar", 'baz': "baz"})
-        self.assertEqual(record.bar, "foo")
+        self.assertEqual(record.bar, "foor")
         self.assertEqual(record.baz, "baz")
 
         # compute 'bar' and 'baz', but do not change its value
         record = model.create({'active': False, 'foo': "foo"})
-        self.assertEqual(record.bar, "foo")
+        self.assertEqual(record.bar, "foor")
         self.assertEqual(record.baz, False)
 
         # compute 'bar' but not 'baz'
         record = model.create({'active': False, 'foo': "foo", 'bar': "bar", 'baz': "baz"})
-        self.assertEqual(record.bar, "foo")
+        self.assertEqual(record.bar, "foor")
         self.assertEqual(record.baz, "baz")
 
     def test_copy(self):
@@ -571,20 +658,20 @@ class TestComputeOnchange(common.TransactionCase):
             {'name': 'bar1'},
         ])
 
-        # compute 'bar', 'baz', 'line_ids' and 'tag_ids'
+        # compute 'bar' (readonly), 'baz', 'line_ids' and 'tag_ids' (editable)
         record = Model.create({'active': True, 'foo': "foo1"})
-        self.assertEqual(record.bar, "foo1")
-        self.assertEqual(record.baz, "foo1")
+        self.assertEqual(record.bar, "foo1r")
+        self.assertEqual(record.baz, "foo1z")
         self.assertEqual(record.line_ids.mapped('foo'), ['foo1'])
         self.assertEqual(record.tag_ids, tag_foo)
 
         # manually update 'baz' and 'lines' to test copy attribute
         record.write({
             'baz': "baz1",
-            'line_ids': [(0, 0, {'foo': 'bar'})],
-            'tag_ids': [(4, tag_bar.id)],
+            'line_ids': [Command.create({'foo': 'bar'})],
+            'tag_ids': [Command.link(tag_bar.id)],
         })
-        self.assertEqual(record.bar, "foo1")
+        self.assertEqual(record.bar, "foo1r")
         self.assertEqual(record.baz, "baz1")
         self.assertEqual(record.line_ids.mapped('foo'), ['foo1', 'bar'])
         self.assertEqual(record.tag_ids, tag_foo + tag_bar)
@@ -592,7 +679,7 @@ class TestComputeOnchange(common.TransactionCase):
         # copy the record, and check results
         copied = record.copy()
         self.assertEqual(copied.foo, "foo1 (copy)")   # copied and modified
-        self.assertEqual(copied.bar, "foo1 (copy)")   # computed
+        self.assertEqual(copied.bar, "foo1 (copy)r")  # computed
         self.assertEqual(copied.baz, "baz1")          # copied
         self.assertEqual(record.line_ids.mapped('foo'), ['foo1', 'bar'])  # copied
         self.assertEqual(record.tag_ids, tag_foo + tag_bar)  # copied
@@ -600,141 +687,172 @@ class TestComputeOnchange(common.TransactionCase):
     def test_write(self):
         model = self.env['test_new_api.compute.onchange']
         record = model.create({'active': True, 'foo': "foo"})
-        self.assertEqual(record.bar, "foo")
-        self.assertEqual(record.baz, "foo")
+        self.assertEqual(record.bar, "foor")
+        self.assertEqual(record.baz, "fooz")
 
-        # recompute 'bar' and 'baz'
+        # recompute 'bar' (readonly) and 'baz' (editable)
         record.write({'foo': "foo1"})
-        self.assertEqual(record.bar, "foo1")
-        self.assertEqual(record.baz, "foo1")
+        self.assertEqual(record.bar, "foo1r")
+        self.assertEqual(record.baz, "foo1z")
 
         # recompute 'bar' but not 'baz'
         record.write({'foo': "foo2", 'bar': "bar2", 'baz': "baz2"})
-        self.assertEqual(record.bar, "foo2")
+        self.assertEqual(record.bar, "foo2r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'bar' and 'baz', but do not change its value
         record.write({'active': False, 'foo': "foo3"})
-        self.assertEqual(record.bar, "foo3")
+        self.assertEqual(record.bar, "foo3r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'bar' but not 'baz'
         record.write({'active': False, 'foo': "foo4", 'bar': "bar4", 'baz': "baz4"})
-        self.assertEqual(record.bar, "foo4")
+        self.assertEqual(record.bar, "foo4r")
         self.assertEqual(record.baz, "baz4")
 
     def test_set(self):
         model = self.env['test_new_api.compute.onchange']
         record = model.create({'active': True, 'foo': "foo"})
-        self.assertEqual(record.bar, "foo")
-        self.assertEqual(record.baz, "foo")
+        self.assertEqual(record.bar, "foor")
+        self.assertEqual(record.baz, "fooz")
 
-        # recompute 'bar' and 'baz'
+        # recompute 'bar' (readonly) and 'baz' (editable)
         record.foo = "foo1"
-        self.assertEqual(record.bar, "foo1")
-        self.assertEqual(record.baz, "foo1")
+        self.assertEqual(record.bar, "foo1r")
+        self.assertEqual(record.baz, "foo1z")
 
         # do not recompute 'baz'
         record.baz = "baz2"
-        self.assertEqual(record.bar, "foo1")
+        self.assertEqual(record.bar, "foo1r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'baz', but do not change its value
         record.active = False
-        self.assertEqual(record.bar, "foo1")
+        self.assertEqual(record.bar, "foo1r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'baz', but do not change its value
         record.foo = "foo3"
-        self.assertEqual(record.bar, "foo3")
+        self.assertEqual(record.bar, "foo3r")
         self.assertEqual(record.baz, "baz2")
 
         # do not recompute 'baz'
         record.baz = "baz4"
-        self.assertEqual(record.bar, "foo3")
+        self.assertEqual(record.bar, "foo3r")
         self.assertEqual(record.baz, "baz4")
 
     def test_set_new(self):
         model = self.env['test_new_api.compute.onchange']
-        record = model.new({'active': True, 'foo': "foo"})
-        self.assertEqual(record.bar, "foo")
-        self.assertEqual(record.baz, "foo")
+        record = model.new({'active': True})
+        self.assertEqual(record.bar, "r")
+        self.assertEqual(record.baz, "z")
 
-        # recompute 'bar' and 'baz'
+        # recompute 'bar' (readonly) and 'baz' (editable)
         record.foo = "foo1"
-        self.assertEqual(record.bar, "foo1")
-        self.assertEqual(record.baz, "foo1")
+        self.assertEqual(record.bar, "foo1r")
+        self.assertEqual(record.baz, "foo1z")
 
         # do not recompute 'baz'
         record.baz = "baz2"
-        self.assertEqual(record.bar, "foo1")
+        self.assertEqual(record.bar, "foo1r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'baz', but do not change its value
         record.active = False
-        self.assertEqual(record.bar, "foo1")
+        self.assertEqual(record.bar, "foo1r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'baz', but do not change its value
         record.foo = "foo3"
-        self.assertEqual(record.bar, "foo3")
+        self.assertEqual(record.bar, "foo3r")
         self.assertEqual(record.baz, "baz2")
 
         # do not recompute 'baz'
         record.baz = "baz4"
-        self.assertEqual(record.bar, "foo3")
+        self.assertEqual(record.bar, "foo3r")
         self.assertEqual(record.baz, "baz4")
 
     def test_onchange(self):
+        # check computations of 'bar' (readonly) and 'baz' (editable)
         form = common.Form(self.env['test_new_api.compute.onchange'])
+        self.assertEqual(form.bar, "r")
+        self.assertEqual(form.baz, False)
         form.active = True
+        self.assertEqual(form.bar, "r")
+        self.assertEqual(form.baz, "z")
         form.foo = "foo1"
-        self.assertEqual(form.bar, "foo1")
-        self.assertEqual(form.baz, "foo1")
+        self.assertEqual(form.bar, "foo1r")
+        self.assertEqual(form.baz, "foo1z")
         form.baz = "baz2"
-        self.assertEqual(form.bar, "foo1")
+        self.assertEqual(form.bar, "foo1r")
         self.assertEqual(form.baz, "baz2")
         form.active = False
-        self.assertEqual(form.bar, "foo1")
+        self.assertEqual(form.bar, "foo1r")
         self.assertEqual(form.baz, "baz2")
         form.foo = "foo3"
-        self.assertEqual(form.bar, "foo3")
+        self.assertEqual(form.bar, "foo3r")
         self.assertEqual(form.baz, "baz2")
         form.active = True
-        self.assertEqual(form.bar, "foo3")
-        self.assertEqual(form.baz, "foo3")
+        self.assertEqual(form.bar, "foo3r")
+        self.assertEqual(form.baz, "foo3z")
+
+        with form.line_ids.new() as line:
+            # check computation of 'bar' (readonly)
+            self.assertEqual(line.foo, False)
+            self.assertEqual(line.bar, "r")
+            line.foo = "foo"
+            self.assertEqual(line.foo, "foo")
+            self.assertEqual(line.bar, "foor")
 
         record = form.save()
-        self.assertEqual(record.bar, "foo3")
-        self.assertEqual(record.baz, "foo3")
+        self.assertEqual(record.bar, "foo3r")
+        self.assertEqual(record.baz, "foo3z")
 
         form = common.Form(record)
-        self.assertEqual(form.bar, "foo3")
-        self.assertEqual(form.baz, "foo3")
+        self.assertEqual(form.bar, "foo3r")
+        self.assertEqual(form.baz, "foo3z")
         form.foo = "foo4"
-        self.assertEqual(form.bar, "foo4")
-        self.assertEqual(form.baz, "foo4")
+        self.assertEqual(form.bar, "foo4r")
+        self.assertEqual(form.baz, "foo4z")
         form.baz = "baz5"
-        self.assertEqual(form.bar, "foo4")
+        self.assertEqual(form.bar, "foo4r")
         self.assertEqual(form.baz, "baz5")
         form.active = False
-        self.assertEqual(form.bar, "foo4")
+        self.assertEqual(form.bar, "foo4r")
         self.assertEqual(form.baz, "baz5")
         form.foo = "foo6"
-        self.assertEqual(form.bar, "foo6")
+        self.assertEqual(form.bar, "foo6r")
         self.assertEqual(form.baz, "baz5")
 
+    def test_onchange_default(self):
+        form = common.Form(self.env['test_new_api.compute.onchange'].with_context(
+            default_active=True, default_foo="foo", default_baz="baz",
+        ))
+        # 'baz' is computed editable, so when given a default value it should
+        # 'not be recomputed, even if a dependency also has a default value
+        self.assertEqual(form.foo, "foo")
+        self.assertEqual(form.bar, "foor")
+        self.assertEqual(form.baz, "baz")
+
+    def test_onchange_once(self):
+        """ Modifies `foo` field which will trigger an onchange method and
+        checks it was triggered only one time. """
+        form = Form(self.env['test_new_api.compute.onchange'].with_context(default_foo="oof"))
+        record = form.save()
+        self.assertEqual(record.foo, "oof")
+        self.assertEqual(record.count, 1, "value onchange must be called only one time")
+
     def test_onchange_one2many(self):
         record = self.env['test_new_api.model_parent_m2o'].create({
             'name': 'Family',
             'child_ids': [
-                (0, 0, {'name': 'W', 'cost': 10}),
-                (0, 0, {'name': 'X', 'cost': 10}),
-                (0, 0, {'name': 'Y'}),
-                (0, 0, {'name': 'Z'}),
+                Command.create({'name': 'W', 'cost': 10}),
+                Command.create({'name': 'X', 'cost': 10}),
+                Command.create({'name': 'Y'}),
+                Command.create({'name': 'Z'}),
             ],
         })
-        record.flush()
+        self.env.flush_all()
         self.assertEqual(record.child_ids.mapped('name'), list('WXYZ'))
         self.assertEqual(record.cost, 22)
 
@@ -755,7 +873,7 @@ class TestComputeOnchange(common.TransactionCase):
     def test_onchange_editable_compute_one2many(self):
         # create a record with a computed editable field ('edit') on lines
         record = self.env['test_new_api.compute_editable'].create({'line_ids': [(0, 0, {'value': 7})]})
-        record.flush()
+        self.env.flush_all()
         line = record.line_ids
         self.assertRecordValues(line, [{'value': 7, 'edit': 7, 'count': 0}])
 
@@ -767,15 +885,15 @@ class TestComputeOnchange(common.TransactionCase):
         # the fields in the dictionary.  This ensures that the value set by the
         # user on a computed editable field on a line is not lost.
         line_ids = [
-            (1, line.id, {'value': 8, 'edit': 9, 'count': 0}),
-            (0, 0, {'value': 8, 'edit': 9, 'count': 0}),
+            Command.update(line.id, {'value': 8, 'edit': 9, 'count': 0}),
+            Command.create({'value': 8, 'edit': 9, 'count': 0}),
         ]
         result = record.onchange({'line_ids': line_ids}, 'line_ids', spec)
         expected = {'value': {
             'line_ids': [
-                (5,),
-                (1, line.id, {'value': 8, 'edit': 9, 'count': 8}),
-                (0, 0, {'value': 8, 'edit': 9, 'count': 8}),
+                Command.clear(),
+                Command.update(line.id, {'value': 8, 'edit': 9, 'count': 8}),
+                Command.create({'value': 8, 'edit': 9, 'count': 8}),
             ],
         }}
         self.assertEqual(result, expected)
@@ -787,3 +905,47 @@ class TestComputeOnchange(common.TransactionCase):
         ]
         result = record.onchange({'line_ids': line_ids}, 'line_ids', spec)
         self.assertEqual(result, expected)
+
+    def test_computed_editable_one2many_domain(self):
+        """ Test a computed, editable one2many field with a domain. """
+        record = self.env['test_new_api.one2many'].create({'name': 'foo'})
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+        ])
+
+        # trigger recomputation by changing name
+        record.name = 'bar'
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+            {'name': 'bar', 'count': 1},
+        ])
+
+        # manually adding a line should not trigger recomputation
+        record.line_ids.create({'name': 'baz', 'container_id': record.id})
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+            {'name': 'bar', 'count': 1},
+            {'name': 'baz', 'count': 1},
+        ])
+
+        # changing the field in the domain should not trigger recomputation...
+        record.line_ids[-1].count = 2
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+            {'name': 'bar', 'count': 1},
+            {'name': 'baz', 'count': 2},
+        ])
+
+        # ...and may show cache inconsistencies
+        record.line_ids[-1].count = 0
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+            {'name': 'bar', 'count': 1},
+            {'name': 'baz', 'count': 0},
+        ])
+        self.env.flush_all()
+        self.env.invalidate_all()
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+            {'name': 'bar', 'count': 1},
+        ])
