@@ -4,6 +4,7 @@ import json
 import logging
 import os
 import shutil
+import subprocess
 import tempfile
 import threading
 import traceback
@@ -26,6 +27,7 @@ import odoo.sql_db
 import odoo.tools
 from odoo.sql_db import db_connect
 from odoo.release import version_info
+from odoo.tools import find_pg_tool, exec_pg_environ
 
 _logger = logging.getLogger(__name__)
 
@@ -62,7 +64,7 @@ def _initialize_db(id, db_name, demo, lang, user_password, login='admin', countr
 
         registry = odoo.modules.registry.Registry.new(db_name, demo, None, update_module=True)
 
-        with closing(db.cursor()) as cr:
+        with closing(registry.cursor()) as cr:
             env = odoo.api.Environment(cr, SUPERUSER_ID, {})
 
             if lang:
@@ -103,7 +105,9 @@ def _create_empty_database(name):
         if cr.fetchall():
             raise DatabaseExists("database %r already exists!" % (name,))
         else:
-            cr.autocommit(True)     # avoid transaction block
+            # database-altering operations cannot be executed inside a transaction
+            cr.rollback()
+            cr._cnx.autocommit = True
 
             # 'C' collate is only safe with template0, but provides more useful indexes
             collate = sql.SQL("LC_COLLATE 'C'" if chosen_template == 'template0' else "")
@@ -112,14 +116,22 @@ def _create_empty_database(name):
                 sql.Identifier(name), collate, sql.Identifier(chosen_template)
             ))
 
-    if odoo.tools.config['unaccent']:
-        try:
-            db = odoo.sql_db.db_connect(name)
-            with closing(db.cursor()) as cr:
+    # TODO: add --extension=trigram,unaccent
+    try:
+        db = odoo.sql_db.db_connect(name)
+        with db.cursor() as cr:
+            cr.execute("CREATE EXTENSION IF NOT EXISTS pg_trgm")
+            if odoo.tools.config['unaccent']:
                 cr.execute("CREATE EXTENSION IF NOT EXISTS unaccent")
-                cr.commit()
-        except psycopg2.Error:
-            pass
+                # From PostgreSQL's point of view, making 'unaccent' immutable is incorrect
+                # because it depends on external data - see
+                # https://www.postgresql.org/message-id/flat/201012021544.oB2FiTn1041521@wwwmaster.postgresql.org#201012021544.oB2FiTn1041521@wwwmaster.postgresql.org
+                # But in the case of Odoo, we consider that those data don't
+                # change in the lifetime of a database. If they do change, all
+                # indexes created with this function become corrupted!
+                cr.execute("ALTER FUNCTION unaccent(text) IMMUTABLE")
+    except psycopg2.Error as e:
+        _logger.warning("Unable to create PostgreSQL extensions : %s", e)
 
 @check_db_management_enabled
 def exp_create_database(db_name, demo, lang, user_password='admin', login='admin', country_code=None, phone=None):
@@ -130,12 +142,13 @@ def exp_create_database(db_name, demo, lang, user_password='admin', login='admin
     return True
 
 @check_db_management_enabled
-def exp_duplicate_database(db_original_name, db_name):
+def exp_duplicate_database(db_original_name, db_name, neutralize_database=False):
     _logger.info('Duplicate database `%s` to `%s`.', db_original_name, db_name)
     odoo.sql_db.close_db(db_original_name)
     db = odoo.sql_db.db_connect('postgres')
     with closing(db.cursor()) as cr:
-        cr.autocommit(True)     # avoid transaction block
+        # database-altering operations cannot be executed inside a transaction
+        cr._cnx.autocommit = True
         _drop_conn(cr, db_original_name)
         cr.execute(sql.SQL("CREATE DATABASE {} ENCODING 'unicode' TEMPLATE {}").format(
             sql.Identifier(db_name),
@@ -147,6 +160,8 @@ def exp_duplicate_database(db_original_name, db_name):
         # if it's a copy of a database, force generation of a new dbuuid
         env = odoo.api.Environment(cr, SUPERUSER_ID, {})
         env['ir.config_parameter'].init(force=True)
+        if neutralize_database:
+            odoo.modules.neutralize.neutralize_database(cr)
 
     from_fs = odoo.tools.config.filestore(db_original_name)
     to_fs = odoo.tools.config.filestore(db_name)
@@ -179,7 +194,8 @@ def exp_drop(db_name):
 
     db = odoo.sql_db.db_connect('postgres')
     with closing(db.cursor()) as cr:
-        cr.autocommit(True) # avoid transaction block
+        # database-altering operations cannot be executed inside a transaction
+        cr._cnx.autocommit = True
         _drop_conn(cr, db_name)
 
         try:
@@ -225,8 +241,8 @@ def dump_db(db_name, stream, backup_format='zip'):
 
     _logger.info('DUMP DB: %s format %s', db_name, backup_format)
 
-    cmd = ['pg_dump', '--no-owner']
-    cmd.append(db_name)
+    cmd = [find_pg_tool('pg_dump'), '--no-owner', db_name]
+    env = exec_pg_environ()
 
     if backup_format == 'zip':
         with tempfile.TemporaryDirectory() as dump_dir:
@@ -238,7 +254,7 @@ def dump_db(db_name, stream, backup_format='zip'):
                 with db.cursor() as cr:
                     json.dump(dump_db_manifest(cr), fh, indent=4)
             cmd.insert(-1, '--file=' + os.path.join(dump_dir, 'dump.sql'))
-            odoo.tools.exec_pg_command(*cmd)
+            subprocess.run(cmd, env=env, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT, check=True)
             if stream:
                 odoo.tools.osutil.zip_dir(dump_dir, stream, include_dir=False, fnct_sort=lambda file_name: file_name != 'dump.sql')
             else:
@@ -248,7 +264,7 @@ def dump_db(db_name, stream, backup_format='zip'):
                 return t
     else:
         cmd.insert(-1, '--format=c')
-        stdin, stdout = odoo.tools.exec_pg_command_pipe(*cmd)
+        stdout = subprocess.Popen(cmd, env=env, stdin=subprocess.DEVNULL, stdout=subprocess.PIPE).stdout
         if stream:
             shutil.copyfileobj(stdout, stream)
         else:
@@ -270,12 +286,13 @@ def exp_restore(db_name, data, copy=False):
     return True
 
 @check_db_management_enabled
-def restore_db(db, dump_file, copy=False):
+def restore_db(db, dump_file, copy=False, neutralize_database=False):
     assert isinstance(db, str)
     if exp_db_exist(db):
-        _logger.info('RESTORE DB: %s already exists', db)
+        _logger.warning('RESTORE DB: %s already exists', db)
         raise Exception("Database already exists")
 
+    _logger.info('RESTORING DB: %s', db)
     _create_empty_database(db)
 
     filestore_path = None
@@ -298,11 +315,13 @@ def restore_db(db, dump_file, copy=False):
             pg_cmd = 'pg_restore'
             pg_args = ['--no-owner', dump_file]
 
-        args = []
-        args.append('--dbname=' + db)
-        pg_args = args + pg_args
-
-        if odoo.tools.exec_pg_command(pg_cmd, *pg_args):
+        r = subprocess.run(
+            [find_pg_tool(pg_cmd), '--dbname=' + db, *pg_args],
+            env=exec_pg_environ(),
+            stdout=subprocess.DEVNULL,
+            stderr=subprocess.STDOUT,
+        )
+        if r.returncode != 0:
             raise Exception("Couldn't restore database")
 
         registry = odoo.modules.registry.Registry.new(db)
@@ -311,6 +330,9 @@ def restore_db(db, dump_file, copy=False):
             if copy:
                 # if it's a copy of a database, force generation of a new dbuuid
                 env['ir.config_parameter'].init(force=True)
+            if neutralize_database:
+                odoo.modules.neutralize.neutralize_database(cr)
+
             if filestore_path:
                 filestore_dest = env['ir.attachment']._filestore()
                 shutil.move(filestore_path, filestore_dest)
@@ -324,7 +346,8 @@ def exp_rename(old_name, new_name):
 
     db = odoo.sql_db.db_connect('postgres')
     with closing(db.cursor()) as cr:
-        cr.autocommit(True)     # avoid transaction block
+        # database-altering operations cannot be executed inside a transaction
+        cr._cnx.autocommit = True
         _drop_conn(cr, old_name)
         try:
             cr.execute(sql.SQL('ALTER DATABASE {} RENAME TO {}').format(sql.Identifier(old_name), sql.Identifier(new_name)))
@@ -342,7 +365,7 @@ def exp_rename(old_name, new_name):
 @check_db_management_enabled
 def exp_change_admin_password(new_password):
     odoo.tools.config.set_admin_password(new_password)
-    odoo.tools.config.save()
+    odoo.tools.config.save(['admin_passwd'])
     return True
 
 @check_db_management_enabled
