@@ -8,97 +8,111 @@ the database, *not* a database abstraction toolkit. Database abstraction is what
 the ORM does, in fact.
 """
 
-from contextlib import contextmanager
-from functools import wraps
-import itertools
 import logging
+import os
+import re
+import threading
 import time
 import uuid
 import warnings
+from contextlib import contextmanager
+from datetime import datetime, timedelta
+from inspect import currentframe
 
-from decorator import decorator
 import psycopg2
-import psycopg2.extras
 import psycopg2.extensions
+import psycopg2.extras
 from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT, ISOLATION_LEVEL_READ_COMMITTED, ISOLATION_LEVEL_REPEATABLE_READ
 from psycopg2.pool import PoolError
+from psycopg2.sql import SQL, Identifier
 from werkzeug import urls
 
-from odoo.api import Environment
+from . import tools
+from .tools.func import frame_codeinfo, locked
 
 psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
 
+def undecimalize(value, cr):
+    if value is None:
+        return None
+    return float(value)
+
+psycopg2.extensions.register_type(psycopg2.extensions.new_type((700, 701, 1700), 'float', undecimalize))
+
 _logger = logging.getLogger(__name__)
+_logger_conn = _logger.getChild("connection")
 
-def unbuffer(symb, cr):
-    if symb is None:
-        return None
-    return str(symb)
+real_time = time.time.__call__  # ensure we have a non patched time for query times when using freezegun
 
-def undecimalize(symb, cr):
-    if symb is None:
-        return None
-    return float(symb)
-
-psycopg2.extensions.register_type(psycopg2.extensions.new_type((700, 701, 1700,), 'float', undecimalize))
-
-
-from . import tools
-from .tools.func import frame_codeinfo
-
-from .tools import parse_version as pv
-if pv(psycopg2.__version__) < pv('2.7'):
-    from psycopg2._psycopg import QuotedString
-    def adapt_string(adapted):
-        """Python implementation of psycopg/psycopg2#459 from v2.7"""
-        if '\x00' in adapted:
-            raise ValueError("A string literal cannot contain NUL (0x00) characters.")
-        return QuotedString(adapted)
-
-    psycopg2.extensions.register_adapter(str, adapt_string)
-
-from datetime import timedelta
-import threading
-from inspect import currentframe
-
-
-def flush_env(cr, *, clear=True):
-    """ Retrieve and flush an environment corresponding to the given cursor.
-        Also clear the environment if ``clear`` is true.
-    """
-    env_to_flush = None
-    for env in list(Environment.envs):
-        # don't flush() on another cursor or with a RequestUID
-        if env.cr is cr and (isinstance(env.uid, int) or env.uid is None):
-            env_to_flush = env
-            if env.uid is not None:
-                break               # prefer an environment with a real uid
-
-    if env_to_flush is not None:
-        env_to_flush['base'].flush()
-        if clear:
-            env_to_flush.clear()    # clear remaining new records to compute
-
-def clear_env(cr):
-    """ Retrieve and clear an environment corresponding to the given cursor """
-    for env in list(Environment.envs):
-        if env.cr is cr:
-            env.clear()
-            break
-
-import re
-re_from = re.compile('.* from "?([a-zA-Z_0-9]+)"? .*$')
-re_into = re.compile('.* into "?([a-zA-Z_0-9]+)"? .*$')
+re_from = re.compile('.* from "?([a-zA-Z_0-9]+)"? .*$', re.MULTILINE | re.IGNORECASE)
+re_into = re.compile('.* into "?([a-zA-Z_0-9]+)"? .*$', re.MULTILINE | re.IGNORECASE)
 
 sql_counter = 0
 
 
-@decorator
-def check(f, self, *args, **kwargs):
-    """ Wrap a cursor method that cannot be called when the cursor is closed. """
-    if self._closed:
-        raise psycopg2.OperationalError('Unable to use a closed cursor.')
-    return f(self, *args, **kwargs)
+class Savepoint:
+    """ Reifies an active breakpoint, allows :meth:`BaseCursor.savepoint` users
+    to internally rollback the savepoint (as many times as they want) without
+    having to implement their own savepointing, or triggering exceptions.
+
+    Should normally be created using :meth:`BaseCursor.savepoint` rather than
+    directly.
+
+    The savepoint will be rolled back on unsuccessful context exits
+    (exceptions). It will be released ("committed") on successful context exit.
+    The savepoint object can be wrapped in ``contextlib.closing`` to
+    unconditionally roll it back.
+
+    The savepoint can also safely be explicitly closed during context body. This
+    will rollback by default.
+
+    :param BaseCursor cr: the cursor to execute the `SAVEPOINT` queries on
+    """
+    def __init__(self, cr):
+        self.name = str(uuid.uuid1())
+        self._name = Identifier(self.name)
+        self._cr = cr
+        self.closed = False
+        cr.execute(SQL('SAVEPOINT {}').format(self._name))
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close(rollback=exc_type is not None)
+
+    def close(self, *, rollback=True):
+        if not self.closed:
+            self._close(rollback)
+
+    def rollback(self):
+        self._cr.execute(SQL('ROLLBACK TO SAVEPOINT {}').format(self._name))
+
+    def _close(self, rollback):
+        if rollback:
+            self.rollback()
+        self._cr.execute(SQL('RELEASE SAVEPOINT {}').format(self._name))
+        self.closed = True
+
+
+class _FlushingSavepoint(Savepoint):
+    def __init__(self, cr):
+        cr.flush()
+        super().__init__(cr)
+
+    def rollback(self):
+        self._cr.clear()
+        super().rollback()
+
+    def _close(self, rollback):
+        try:
+            if not rollback:
+                self._cr.flush()
+        except Exception:
+            rollback = True
+            raise
+        finally:
+            super()._close(rollback)
 
 
 class BaseCursor:
@@ -109,29 +123,40 @@ class BaseCursor:
         self.postcommit = tools.Callbacks()
         self.prerollback = tools.Callbacks()
         self.postrollback = tools.Callbacks()
+        # By default a cursor has no transaction object.  A transaction object
+        # for managing environments is instantiated by registry.cursor().  It
+        # is not done here in order to avoid cyclic module dependencies.
+        self.transaction = None
 
-    @contextmanager
-    @check
-    def savepoint(self, flush=True):
-        """context manager entering in a new savepoint"""
-        name = uuid.uuid1().hex
+    def flush(self):
+        """ Flush the current transaction, and run precommit hooks. """
+        if self.transaction is not None:
+            self.transaction.flush()
+        self.precommit.run()
+
+    def clear(self):
+        """ Clear the current transaction, and clear precommit hooks. """
+        if self.transaction is not None:
+            self.transaction.clear()
+        self.precommit.clear()
+
+    def reset(self):
+        """ Reset the current transaction (this invalidates more that clear()).
+            This method should be called only right after commit() or rollback().
+        """
+        if self.transaction is not None:
+            self.transaction.reset()
+
+    def savepoint(self, flush=True) -> Savepoint:
+        """context manager entering in a new savepoint
+
+        With ``flush`` (the default), will automatically run (or clear) the
+        relevant hooks.
+        """
         if flush:
-            flush_env(self, clear=False)
-            self.precommit.run()
-        self.execute('SAVEPOINT "%s"' % name)
-        try:
-            yield
-            if flush:
-                flush_env(self, clear=False)
-                self.precommit.run()
-        except Exception:
-            if flush:
-                clear_env(self)
-                self.precommit.clear()
-            self.execute('ROLLBACK TO SAVEPOINT "%s"' % name)
-            raise
+            return _FlushingSavepoint(self)
         else:
-            self.execute('RELEASE SAVEPOINT "%s"' % name)
+            return Savepoint(self)
 
     def __enter__(self):
         """ Using the cursor as a contextmanager automatically commits and
@@ -146,9 +171,11 @@ class BaseCursor:
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
-        if exc_type is None:
-            self.commit()
-        self.close()
+        try:
+            if exc_type is None:
+                self.commit()
+        finally:
+            self.close()
 
 
 class Cursor(BaseCursor):
@@ -202,13 +229,7 @@ class Cursor(BaseCursor):
         As a result of the above, we have selected ``REPEATABLE READ`` as
         the default transaction isolation level for OpenERP cursors, as
         it will be mapped to the desired ``snapshot isolation`` level for
-        all supported PostgreSQL version (8.3 - 9.x).
-
-        Note: up to psycopg2 v.2.4.2, psycopg2 itself remapped the repeatable
-        read level to serializable before sending it to the database, so it would
-        actually select the new serializable mode on PostgreSQL 9.1. Make
-        sure you use psycopg2 v2.4.2 or newer if you use PostgreSQL 9.1 and
-        the performance hit is a concern for you.
+        all supported PostgreSQL version (>10).
 
         .. attribute:: cache
 
@@ -223,48 +244,49 @@ class Cursor(BaseCursor):
     """
     IN_MAX = 1000   # decent limit on size of IN queries - guideline = Oracle limit
 
-    def __init__(self, pool, dbname, dsn, serialized=True):
+    def __init__(self, pool, dbname, dsn, **kwargs):
         super().__init__()
-
+        if 'serialized' in kwargs:
+            warnings.warn("Since 16.0, 'serialized' parameter is not used anymore.", DeprecationWarning, 2)
+        assert kwargs.keys() <= {'serialized'}
         self.sql_from_log = {}
         self.sql_into_log = {}
 
         # default log level determined at cursor creation, could be
         # overridden later for debugging purposes
-        self.sql_log = _logger.isEnabledFor(logging.DEBUG)
-
         self.sql_log_count = 0
+        self._sql_table_tracking = False
 
         # avoid the call of close() (by __del__) if an exception
-        # is raised by any of the following initialisations
+        # is raised by any of the following initializations
         self._closed = True
 
         self.__pool = pool
         self.dbname = dbname
-        # Whether to enable snapshot isolation level for this cursor.
-        # see also the docstring of Cursor.
-        self._serialized = serialized
 
         self._cnx = pool.borrow(dsn)
         self._obj = self._cnx.cursor()
-        if self.sql_log:
+        if _logger.isEnabledFor(logging.DEBUG):
             self.__caller = frame_codeinfo(currentframe(), 2)
         else:
             self.__caller = False
-        self._closed = False   # real initialisation value
-        self.autocommit(False)
-
-        self._default_log_exceptions = True
+        self._closed = False   # real initialization value
+        # See the docstring of this class.
+        self.connection.set_isolation_level(ISOLATION_LEVEL_REPEATABLE_READ)
 
         self.cache = {}
+        self._now = None
 
     def __build_dict(self, row):
         return {d.name: row[i] for i, d in enumerate(self._obj.description)}
+
     def dictfetchone(self):
         row = self._obj.fetchone()
         return row and self.__build_dict(row)
+
     def dictfetchmany(self, size):
         return [self.__build_dict(row) for row in self._obj.fetchmany(size)]
+
     def dictfetchall(self):
         return [self.__build_dict(row) for row in self._obj.fetchall()]
 
@@ -283,46 +305,60 @@ class Cursor(BaseCursor):
             _logger.warning(msg)
             self._close(True)
 
-    @check
-    def execute(self, query, params=None, log_exceptions=None):
+    def _format(self, query, params=None):
+        encoding = psycopg2.extensions.encodings[self.connection.encoding]
+        return self._obj.mogrify(query, params).decode(encoding, 'replace')
+
+    def execute(self, query, params=None, log_exceptions=True):
+        global sql_counter
         if params and not isinstance(params, (tuple, list, dict)):
             # psycopg2's TypeError is not clear if you mess up the params
             raise ValueError("SQL query parameters should be a tuple, list or dict; got %r" % (params,))
 
-        if self.sql_log:
-            encoding = psycopg2.extensions.encodings[self.connection.encoding]
-            _logger.debug("query: %s", self._obj.mogrify(query, params).decode(encoding, 'replace'))
-        now = time.time()
+        start = real_time()
         try:
             params = params or None
             res = self._obj.execute(query, params)
         except Exception as e:
-            if self._default_log_exceptions if log_exceptions is None else log_exceptions:
+            if log_exceptions:
                 _logger.error("bad query: %s\nERROR: %s", tools.ustr(self._obj.query or query), e)
             raise
+        finally:
+            delay = real_time() - start
+            if _logger.isEnabledFor(logging.DEBUG):
+                _logger.debug("[%.3f ms] query: %s", 1000 * delay, self._format(query, params))
 
         # simple query count is always computed
         self.sql_log_count += 1
-        delay = (time.time() - now)
-        if hasattr(threading.current_thread(), 'query_count'):
-            threading.current_thread().query_count += 1
-            threading.current_thread().query_time += delay
+        sql_counter += 1
 
-        # advanced stats only if sql_log is enabled
-        if self.sql_log:
+        current_thread = threading.current_thread()
+        if hasattr(current_thread, 'query_count'):
+            current_thread.query_count += 1
+            current_thread.query_time += delay
+
+        # optional hooks for performance and tracing analysis
+        for hook in getattr(current_thread, 'query_hooks', ()):
+            hook(self, query, params, start, delay)
+
+        # advanced stats
+        if _logger.isEnabledFor(logging.DEBUG) or self._sql_table_tracking:
             delay *= 1E6
 
-            query_lower = self._obj.query.decode().lower()
-            res_from = re_from.match(query_lower)
-            if res_from:
-                self.sql_from_log.setdefault(res_from.group(1), [0, 0])
-                self.sql_from_log[res_from.group(1)][0] += 1
-                self.sql_from_log[res_from.group(1)][1] += delay
-            res_into = re_into.match(query_lower)
+            decoded_query = self._obj.query.decode()
+            res_into = re_into.search(decoded_query)
+            # prioritize `insert` over `select` so `select` subqueries are not
+            # considered when inside a `insert`
             if res_into:
                 self.sql_into_log.setdefault(res_into.group(1), [0, 0])
                 self.sql_into_log[res_into.group(1)][0] += 1
                 self.sql_into_log[res_into.group(1)][1] += delay
+            else:
+                res_from = re_from.search(decoded_query)
+                if res_from:
+                    self.sql_from_log.setdefault(res_from.group(1), [0, 0])
+                    self.sql_from_log[res_from.group(1)][0] += 1
+                    self.sql_from_log[res_from.group(1)][1] += delay
         return res
 
     def split_for_in_conditions(self, ids, size=None):
@@ -333,7 +369,7 @@ class Cursor(BaseCursor):
     def print_log(self):
         global sql_counter
 
-        if not self.sql_log:
+        if not _logger.isEnabledFor(logging.DEBUG):
             return
         def process(type):
             sqllogs = {'from': self.sql_from_log, 'into': self.sql_into_log}
@@ -352,24 +388,40 @@ class Cursor(BaseCursor):
         process('from')
         process('into')
         self.sql_log_count = 0
-        self.sql_log = False
 
-    @check
+    @contextmanager
+    def _enable_logging(self):
+        """ Forcefully enables logging for this cursor, restores it afterwards.
+
+        Updates the logger in-place, so not thread-safe.
+        """
+        level = _logger.level
+        _logger.setLevel(logging.DEBUG)
+        try:
+            yield
+        finally:
+            _logger.setLevel(level)
+
+    @contextmanager
+    def _enable_table_tracking(self):
+        try:
+            old = self._sql_table_tracking
+            self._sql_table_tracking = True
+            yield
+        finally:
+            self._sql_table_tracking = old
+
     def close(self):
-        return self._close(False)
+        if not self.closed:
+            return self._close(False)
 
     def _close(self, leak=False):
-        global sql_counter
-
         if not self._obj:
             return
 
         del self.cache
 
-        # simple query count is always computed
-        sql_counter += self.sql_log_count
-
-        # advanced stats only if sql_log is enabled
+        # advanced stats only at logging.DEBUG level
         self.print_log()
 
         self._obj.close()
@@ -390,83 +442,56 @@ class Cursor(BaseCursor):
             self._cnx.leaked = True
         else:
             chosen_template = tools.config['db_template']
-            templates_list = tuple(set(['template0', 'template1', 'postgres', chosen_template]))
-            keep_in_pool = self.dbname not in templates_list
+            keep_in_pool = self.dbname not in ('template0', 'template1', 'postgres', chosen_template)
             self.__pool.give_back(self._cnx, keep_in_pool=keep_in_pool)
 
-    @check
     def autocommit(self, on):
+        warnings.warn(
+            f"Deprecated Methods since 16.0, use {'`_cnx.autocommit = True`' if on else '`_cnx.set_isolation_level`'} instead.",
+            DeprecationWarning, stacklevel=2
+        )
         if on:
             isolation_level = ISOLATION_LEVEL_AUTOCOMMIT
         else:
-            # If a serializable cursor was requested, we
-            # use the appropriate PotsgreSQL isolation level
-            # that maps to snaphsot isolation.
-            # For all supported PostgreSQL versions (8.3-9.x),
-            # this is currently the ISOLATION_REPEATABLE_READ.
-            # See also the docstring of this class.
-            # NOTE: up to psycopg 2.4.2, repeatable read
-            #       is remapped to serializable before being
-            #       sent to the database, so it is in fact
-            #       unavailable for use with pg 9.1.
-            isolation_level = \
-                ISOLATION_LEVEL_REPEATABLE_READ \
-                if self._serialized \
-                else ISOLATION_LEVEL_READ_COMMITTED
+            isolation_level = ISOLATION_LEVEL_REPEATABLE_READ if self._serialized else ISOLATION_LEVEL_READ_COMMITTED
         self._cnx.set_isolation_level(isolation_level)
 
-    @check
-    def after(self, event, func):
-        """ Register an event handler.
-
-            :param event: the event, either `'commit'` or `'rollback'`
-            :param func: a callable object, called with no argument after the
-                event occurs
-
-            Be careful when coding an event handler, since any operation on the
-            cursor that was just committed/rolled back will take place in the
-            next transaction that has already begun, and may still be rolled
-            back or committed independently. You may consider the use of a
-            dedicated temporary cursor to do some database operation.
-        """
-        warnings.warn(
-            "Cursor.after() is deprecated, use Cursor.postcommit.add() instead.",
-            DeprecationWarning,
-        )
-        if event == 'commit':
-            self.postcommit.add(func)
-        elif event == 'rollback':
-            self.postrollback.add(func)
-
-    @check
     def commit(self):
         """ Perform an SQL `COMMIT` """
-        flush_env(self)
-        self.precommit.run()
+        self.flush()
         result = self._cnx.commit()
+        self.clear()
+        self._now = None
         self.prerollback.clear()
         self.postrollback.clear()
         self.postcommit.run()
         return result
 
-    @check
     def rollback(self):
         """ Perform an SQL `ROLLBACK` """
-        clear_env(self)
-        self.precommit.clear()
+        self.clear()
         self.postcommit.clear()
         self.prerollback.run()
         result = self._cnx.rollback()
+        self._now = None
         self.postrollback.run()
         return result
 
-    @check
     def __getattr__(self, name):
+        if self._closed and name == '_obj':
+            raise psycopg2.InterfaceError("Cursor already closed")
         return getattr(self._obj, name)
 
     @property
     def closed(self):
-        return self._closed
+        return self._closed or self._cnx.closed
+
+    def now(self):
+        """ Return the transaction's timestamp ``NOW() AT TIME ZONE 'UTC'``. """
+        if self._now is None:
+            self.execute("SELECT (now() AT TIME ZONE 'UTC')")
+            self._now = self.fetchone()[0]
+        return self._now
 
 
 class TestCursor(BaseCursor):
@@ -474,71 +499,101 @@ class TestCursor(BaseCursor):
         the transaction open across requests, and simulates committing, rolling
         back, and closing:
 
-              test cursor           | queries on actual cursor
-            ------------------------+---------------------------------------
-              cr = TestCursor(...)  | SAVEPOINT test_cursor_N
-                                    |
-              cr.execute(query)     | query
-                                    |
-              cr.commit()           | SAVEPOINT test_cursor_N
-                                    |
-              cr.rollback()         | ROLLBACK TO SAVEPOINT test_cursor_N
-                                    |
-              cr.close()            | ROLLBACK TO SAVEPOINT test_cursor_N
-                                    |
-
+        +------------------------+---------------------------------------------------+
+        |  test cursor           | queries on actual cursor                          |
+        +========================+===================================================+
+        |``cr = TestCursor(...)``| SAVEPOINT test_cursor_N                           |
+        +------------------------+---------------------------------------------------+
+        | ``cr.execute(query)``  | query                                             |
+        +------------------------+---------------------------------------------------+
+        |  ``cr.commit()``       | RELEASE SAVEPOINT test_cursor_N                   |
+        |                        | SAVEPOINT test_cursor_N (lazy)                    |
+        +------------------------+---------------------------------------------------+
+        |  ``cr.rollback()``     | ROLLBACK TO SAVEPOINT test_cursor_N (if savepoint)|
+        +------------------------+---------------------------------------------------+
+        |  ``cr.close()``        | ROLLBACK TO SAVEPOINT test_cursor_N (if savepoint)|
+        |                        | RELEASE SAVEPOINT test_cursor_N (if savepoint)    |
+        +------------------------+---------------------------------------------------+
     """
-    _savepoint_seq = itertools.count()
-
+    _cursors_stack = []
     def __init__(self, cursor, lock):
+        super().__init__()
+        self._now = None
         self._closed = False
         self._cursor = cursor
         # we use a lock to serialize concurrent requests
         self._lock = lock
         self._lock.acquire()
+        self._cursors_stack.append(self)
         # in order to simulate commit and rollback, the cursor maintains a
-        # savepoint at its last commit
-        self._savepoint = "test_cursor_%s" % next(self._savepoint_seq)
-        self._cursor.execute('SAVEPOINT "%s"' % self._savepoint)
+        # savepoint at its last commit, the savepoint is created lazily
+        self._savepoint = self._cursor.savepoint(flush=False)
+
+    def execute(self, *args, **kwargs):
+        if not self._savepoint:
+            self._savepoint = self._cursor.savepoint(flush=False)
+
+        return self._cursor.execute(*args, **kwargs)
 
     def close(self):
         if not self._closed:
             self.rollback()
             self._closed = True
+            if self._savepoint:
+                self._savepoint.close(rollback=False)
+
+            tos = self._cursors_stack.pop()
+            if tos is not self:
+                _logger.warning("Found different un-closed cursor when trying to close %s: %s", self, tos)
+
             self._lock.release()
 
     def autocommit(self, on):
-        _logger.debug("TestCursor.autocommit(%r) does nothing", on)
+        warnings.warn("Deprecated method and does nothing since 16.0", DeprecationWarning, 2)
 
-    @check
     def commit(self):
         """ Perform an SQL `COMMIT` """
-        flush_env(self)
-        self.precommit.run()
-        self._cursor.execute('SAVEPOINT "%s"' % self._savepoint)
+        self.flush()
+        if self._savepoint:
+            self._savepoint.close(rollback=False)
+            self._savepoint = None
+        self.clear()
         self.prerollback.clear()
         self.postrollback.clear()
         self.postcommit.clear()         # TestCursor ignores post-commit hooks
 
-    @check
     def rollback(self):
         """ Perform an SQL `ROLLBACK` """
-        clear_env(self)
-        self.precommit.clear()
+        self.clear()
         self.postcommit.clear()
         self.prerollback.run()
-        self._cursor.execute('ROLLBACK TO SAVEPOINT "%s"' % self._savepoint)
+        if self._savepoint:
+            self._savepoint.rollback()
         self.postrollback.run()
 
     def __getattr__(self, name):
-        value = getattr(self._cursor, name)
-        if callable(value) and self._closed:
-            raise psycopg2.OperationalError('Unable to use a closed cursor.')
-        return value
+        return getattr(self._cursor, name)
+
+    def now(self):
+        """ Return the transaction's timestamp ``datetime.now()``. """
+        if self._now is None:
+            self._now = datetime.now()
+        return self._now
 
 
 class PsycoConnection(psycopg2.extensions.connection):
-    pass
+    def lobject(*args, **kwargs):
+        pass
+
+    if hasattr(psycopg2.extensions, 'ConnectionInfo'):
+        @property
+        def info(self):
+            class PsycoConnectionInfo(psycopg2.extensions.ConnectionInfo):
+                @property
+                def password(self):
+                    pass
+            return PsycoConnectionInfo(self)
+
 
 class ConnectionPool(object):
     """ The pool of connections to database(s)
@@ -549,17 +604,6 @@ class ConnectionPool(object):
         The connections are *not* automatically closed. Only a close_db()
         can trigger that.
     """
-
-    def locked(fun):
-        @wraps(fun)
-        def _locked(self, *args, **kwargs):
-            self._lock.acquire()
-            try:
-                return fun(self, *args, **kwargs)
-            finally:
-                self._lock.release()
-        return _locked
-
     def __init__(self, maxconn=64):
         self._connections = []
         self._maxconn = max(maxconn, 1)
@@ -571,7 +615,7 @@ class ConnectionPool(object):
         return "ConnectionPool(used=%d/count=%d/max=%d)" % (used, count, self._maxconn)
 
     def _debug(self, msg, *args):
-        _logger.debug(('%r ' + msg), self, *args)
+        _logger_conn.debug(('%r ' + msg), self, *args)
 
     @locked
     def borrow(self, connection_info):
@@ -592,7 +636,7 @@ class ConnectionPool(object):
                 _logger.info('%r: Free leaked connection to %r', self, cnx.dsn)
 
         for i, (cnx, used) in enumerate(self._connections):
-            if not used and cnx._original_dsn == connection_info:
+            if not used and self._dsn_equals(cnx.dsn, connection_info):
                 try:
                     cnx.reset()
                 except psycopg2.OperationalError:
@@ -627,9 +671,8 @@ class ConnectionPool(object):
         except psycopg2.Error:
             _logger.info('Connection to the database failed')
             raise
-        result._original_dsn = connection_info
         self._connections.append((result, True))
-        self._debug('Create new connection')
+        self._debug('Create new connection backend PID %d', result.get_backend_pid())
         return result
 
     @locked
@@ -653,29 +696,52 @@ class ConnectionPool(object):
         count = 0
         last = None
         for i, (cnx, used) in tools.reverse_enumerate(self._connections):
-            if dsn is None or cnx._original_dsn == dsn:
+            if dsn is None or self._dsn_equals(cnx.dsn, dsn):
                 cnx.close()
                 last = self._connections.pop(i)[0]
                 count += 1
         _logger.info('%r: Closed %d connections %s', self, count,
                     (dsn and last and 'to %r' % last.dsn) or '')
 
+    def _dsn_equals(self, dsn1, dsn2):
+        alias_keys = {'dbname': 'database'}
+        ignore_keys = ['password']
+        dsn1, dsn2 = ({
+            alias_keys.get(key, key): str(value)
+            for key, value in (psycopg2.extensions.parse_dsn(dsn) if isinstance(dsn, str) else dsn).items()
+            if key not in ignore_keys
+        } for dsn in (dsn1, dsn2))
+        return dsn1 == dsn2
+
 
 class Connection(object):
     """ A lightweight instance of a connection to postgres
     """
     def __init__(self, pool, dbname, dsn):
-        self.dbname = dbname
-        self.dsn = dsn
+        self.__dbname = dbname
+        self.__dsn = dsn
         self.__pool = pool
 
-    def cursor(self, serialized=True):
-        cursor_type = serialized and 'serialized ' or ''
-        _logger.debug('create %scursor to %r', cursor_type, self.dsn)
-        return Cursor(self.__pool, self.dbname, self.dsn, serialized=serialized)
+    @property
+    def dsn(self):
+        dsn = dict(self.__dsn)
+        dsn.pop('password', None)
+        return dsn
 
-    # serialized_cursor is deprecated - cursors are serialized by default
-    serialized_cursor = cursor
+    @property
+    def dbname(self):
+        return self.__dbname
+
+    def cursor(self, **kwargs):
+        if 'serialized' in kwargs:
+            warnings.warn("Since 16.0, 'serialized' parameter is deprecated", DeprecationWarning, 2)
+        cursor_type = kwargs.pop('serialized', True) and 'serialized ' or ''
+        _logger.debug('create %scursor to %r', cursor_type, self.dsn)
+        return Cursor(self.__pool, self.__dbname, self.__dsn)
+
+    def serialized_cursor(self, **kwargs):
+        warnings.warn("Since 16.0, 'serialized_cursor' is deprecated, use `cursor` instead", DeprecationWarning, 2)
+        return self.cursor(**kwargs)
 
     def __bool__(self):
         raise NotImplementedError()
@@ -692,6 +758,11 @@ def connection_info_for(db_or_uri):
     :param str db_or_uri: database name or postgres dsn
     :rtype: (str, dict)
     """
+    if 'ODOO_PGAPPNAME' in os.environ:
+        # Using manual string interpolation for security reason and trimming at default NAMEDATALEN=63
+        app_name = os.environ['ODOO_PGAPPNAME'].replace('{pid}', str(os.getpid()))[0:63]
+    else:
+        app_name = "odoo-%d" % os.getpid()
     if db_or_uri.startswith(('postgresql://', 'postgres://')):
         # extract db from uri
         us = urls.url_parse(db_or_uri)
@@ -701,9 +772,9 @@ def connection_info_for(db_or_uri):
             db_name = us.username
         else:
             db_name = us.hostname
-        return db_name, {'dsn': db_or_uri}
+        return db_name, {'dsn': db_or_uri, 'application_name': app_name}
 
-    connection_info = {'database': db_or_uri}
+    connection_info = {'database': db_or_uri, 'application_name': app_name}
     for p in ('host', 'port', 'user', 'password', 'sslmode'):
         cfg = tools.config['db_' + p]
         if cfg:
