@@ -10,6 +10,7 @@ import base64
 import io
 import logging
 import os.path
+import pprint
 import re
 import subprocess
 import warnings
@@ -30,6 +31,7 @@ from .config import config
 from .misc import file_open, unquote, ustr, SKIPPED_ELEMENT_TYPES
 from .translate import _
 from odoo import SUPERUSER_ID, api
+from odoo.exceptions import ValidationError
 
 _logger = logging.getLogger(__name__)
 
@@ -53,6 +55,7 @@ class RecordDictWrapper(dict):
 
 def _get_idref(self, env, model_str, idref):
     idref2 = dict(idref,
+                  Command=odoo.fields.Command,
                   time=time,
                   DateTime=datetime,
                   datetime=datetime,
@@ -301,10 +304,10 @@ form: module.record_id""" % (xml_id,)
             for group in g_names:
                 if group.startswith('-'):
                     group_id = self.id_get(group[1:])
-                    groups_value.append((3, group_id))
+                    groups_value.append(odoo.Command.unlink(group_id))
                 else:
                     group_id = self.id_get(group)
-                    groups_value.append((4, group_id))
+                    groups_value.append(odoo.Command.link(group_id))
             res['groups_id'] = groups_value
         if rec.get('paperformat'):
             pf_name = rec.get('paperformat')
@@ -405,10 +408,10 @@ form: module.record_id""" % (xml_id,)
             for group in g_names:
                 if group.startswith('-'):
                     group_id = self.id_get(group[1:])
-                    groups_value.append((3, group_id))
+                    groups_value.append(odoo.Command.unlink(group_id))
                 else:
                     group_id = self.id_get(group)
-                    groups_value.append((4, group_id))
+                    groups_value.append(odoo.Command.link(group_id))
             res['groups_id'] = groups_value
 
         if rec.get('target'):
@@ -467,10 +470,10 @@ form: module.record_id""" % (xml_id,)
         for group in rec.get('groups', '').split(','):
             if group.startswith('-'):
                 group_id = self.id_get(group[1:])
-                groups.append((3, group_id))
+                groups.append(odoo.Command.unlink(group_id))
             elif group:
                 group_id = self.id_get(group)
-                groups.append((4, group_id))
+                groups.append(odoo.Command.link(group_id))
         if groups:
             values['groups_id'] = groups
 
@@ -484,7 +487,7 @@ form: module.record_id""" % (xml_id,)
         for child in rec.iterchildren('menuitem'):
             self._tag_menuitem(child, parent=menu.id)
 
-    def _tag_record(self, rec):
+    def _tag_record(self, rec, extra_vals=None):
         rec_model = rec.get("model")
         env = self.get_env(rec)
         rec_id = rec.get("id", '')
@@ -530,6 +533,7 @@ form: module.record_id""" % (xml_id,)
                 raise Exception("Cannot update missing record %r" % xid)
 
         res = {}
+        sub_records = []
         for field in rec.findall('./field'):
             #TODO: most of this code is duplicated above (in _eval_xml)...
             f_name = field.get("name")
@@ -551,7 +555,7 @@ form: module.record_id""" % (xml_id,)
                 _fields = env[rec_model]._fields
                 # if the current field is many2many
                 if (f_name in _fields) and _fields[f_name].type == 'many2many':
-                    f_val = [(6, 0, [x[f_use] for x in s])]
+                    f_val = [odoo.Command.set([x[f_use] for x in s])]
                 elif len(s):
                     # otherwise (we are probably in a many2one field),
                     # take the first element of the search
@@ -561,7 +565,10 @@ form: module.record_id""" % (xml_id,)
                     val = self.model_id_get(f_ref)
                     f_val = val[0] + ',' + str(val[1])
                 else:
-                    f_val = self.id_get(f_ref)
+                    f_val = self.id_get(f_ref, raise_if_not_found=nodeattr2bool(rec, 'forcecreate', True))
+                    if not f_val:
+                        _logger.warning("Skipping creation of %r because %s=%r could not be resolved", xid, f_name, f_ref)
+                        return None
             else:
                 f_val = _eval_xml(self, field, env)
                 if f_name in model._fields:
@@ -574,7 +581,19 @@ form: module.record_id""" % (xml_id,)
                         f_val = float(f_val)
                     elif field_type == 'boolean' and isinstance(f_val, str):
                         f_val = str2bool(f_val)
+                    elif field_type == 'one2many':
+                        for child in field.findall('./record'):
+                            sub_records.append((child, model._fields[f_name].inverse_name))
+                        if isinstance(f_val, str):
+                            # We do not want to write on the field since we will write
+                            # on the childrens' parents later
+                            continue
+                    elif field_type == 'html':
+                        if field.get('type') == 'xml':
+                            _logger.warning('HTML field %r is declared as `type="xml"`', f_name)
             res[f_name] = f_val
+        if extra_vals:
+            res.update(extra_vals)
 
         data = dict(xml_id=xid, values=res, noupdate=self.noupdate)
         record = model._load_records([data], self.mode == 'update')
@@ -582,6 +601,8 @@ form: module.record_id""" % (xml_id,)
             self.idref[rec_id] = record.id
         if config.get('import_partial'):
             env.cr.commit()
+        for child_rec, inverse_name in sub_records:
+            self._tag_record(child_rec, extra_vals={inverse_name: record.id})
         return rec_model, record.id
 
     def _tag_template(self, el):
@@ -637,7 +658,7 @@ form: module.record_id""" % (xml_id,)
         groups = el.attrib.pop('groups', None)
         if groups:
             grp_lst = [("ref('%s')" % x) for x in groups.split(',')]
-            record.append(Field(name="groups_id", eval="[(6, 0, ["+', '.join(grp_lst)+"])]"))
+            record.append(Field(name="groups_id", eval="[Command.set(["+', '.join(grp_lst)+"])]"))
         if el.get('primary') == 'True':
             # Pseudo clone mode, we'll set the t-name to the full canonical xmlid
             el.append(
@@ -663,7 +684,7 @@ form: module.record_id""" % (xml_id,)
     def model_id_get(self, id_str, raise_if_not_found=True):
         if '.' not in id_str:
             id_str = '%s.%s' % (self.module, id_str)
-        return self.env['ir.model.data'].xmlid_to_res_model_res_id(id_str, raise_if_not_found=raise_if_not_found)
+        return self.env['ir.model.data']._xmlid_to_res_model_res_id(id_str, raise_if_not_found=raise_if_not_found)
 
     def _tag_root(self, el):
         for rec in el:
@@ -677,8 +698,17 @@ form: module.record_id""" % (xml_id,)
                 f(rec)
             except ParseError:
                 raise
+            except ValidationError as err:
+                msg = "while parsing {file}:{viewline}\n{err}\n\nView error context:\n{context}\n".format(
+                    file=rec.getroottree().docinfo.URL,
+                    viewline=rec.sourceline,
+                    context=pprint.pformat(getattr(err, 'context', None) or '-no context-'),
+                    err=err.args[0],
+                )
+                _logger.debug(msg, exc_info=True)
+                raise ParseError(msg) from None  # Restart with "--log-handler odoo.tools.convert:DEBUG" for complete traceback
             except Exception as e:
-                raise ParseError('while parsing %s:%s, near\n%s' % (
+                raise ParseError('while parsing %s:%s, somewhere inside\n%s' % (
                     rec.getroottree().docinfo.URL,
                     rec.sourceline,
                     etree.tostring(rec, encoding='unicode').rstrip()
@@ -737,7 +767,7 @@ def convert_file(cr, module, filename, idref, mode='update', noupdate=False, kin
             raise ValueError("Can't load unknown file type %s.", filename)
 
 def convert_sql_import(cr, fp):
-    cr.execute(fp.read())
+    cr.execute(fp.read()) # pylint: disable=sql-injection
 
 def convert_csv_import(cr, module, fname, csvcontent, idref=None, mode='init',
         noupdate=False):
