@@ -35,4 +35,7 @@ def to_text(source):
     if isinstance(source, bytes):
         return source.decode('utf-8')
 
+    if isinstance(source, str):
+        return source
+
     return str(source)
